/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AppParamReader                                                *
 * Package : Zeus                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:22:03 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AppParamReader.cpp,v $
 *   Revision 1.2  2005/11/25 09:22:03  bhadorn
 *   Reading and parsing the application parameters before loading the framework
 *
 *   Revision 1.1  2005/10/28 12:09:54  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <examples/framework/AppParamReader.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/File.h>
#include <zeusbase/System/StringList.h>
#include <zeusbase/Security/SecurityManager.h>
#include <stdlib.h>
#include <stdio.h>
#include "string.h"

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Erstellen des Application Parameter Readers
*/
/***************************************************************************/
TAppParamReader::TAppParamReader() : TZObject()
{
  m_bStartFrame = true;
  m_bHasConfigFile = false;
  m_bHasPropertyFile = false;
  m_bRegister = false;
  m_bUnregister = false;
  m_bDoList = false;
}

/***************************************************************************/
/*! Zerstoeren des Application Parameter Readers
*/
/***************************************************************************/
TAppParamReader::~TAppParamReader()
{
}

/***************************************************************************/
/*! Parameter parsen
    \param argc : Anzahl Parameter
    \param argv : Parameterliste
*/
/***************************************************************************/
void TAppParamReader::parse(int argc, char *argv[])
{
  TString strAppName = argv[0];
  TString strBaseName = TFile(strAppName).extractFileNameBase();

  m_strConfigFile = strBaseName + L".xml";
  m_strPropertyFile = strBaseName + L".properties";
    
  for(int i = 1; i < argc; i++)
  {
    
    //Parameter enthaelt kein '-' -> Aktion abbrechen
    if (argv[i][0] != '-')
    {
      break;
    }
    else
    {
      if (!strcmp(argv[i], "-?") || !strcmp(argv[i], "--help"))
      {
        printUsage();
        m_bStartFrame = false;
        break;
      }
      else if (!strncmp(argv[i], "-conf=",6))
      {
        m_strConfigFile = &argv[i][6];
        m_bHasConfigFile = true;
      }
      else if (!strncmp(argv[i], "-prop=",6))
      {
        m_strPropertyFile = &argv[i][6];
        m_bHasPropertyFile = true;
      }
      else if (!strncmp(argv[i], "-file=",6))
      {
        m_strRegFileName = &argv[i][6];
      }
      else if (!strncmp(argv[i], "-module=",8))
      {
        m_strModuleName = &argv[i][8];
      }
      else if (!strncmp(argv[i], "-reg",4))
      {
        m_bRegister = true;
        m_bStartFrame = false;
      }
      else if (!strncmp(argv[i], "-unreg",6))
      {
        m_bUnregister = true;
        m_bStartFrame = false;
      }
      else if (!strncmp(argv[i], "-listmodules", 12))
      {
        m_bDoList = true;
        m_bStartFrame = false;
      }
      else if (!strncmp(argv[i], "--version", 9))
      {
        m_bStartFrame = false;
        printf("\nVersion is %s\n", ZEUS_VERSION);
      }
    }
  }
  
  if (m_bHasPropertyFile)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Use config file = %s", m_strConfigFile.c_str());
  }
  
  if (m_bHasPropertyFile)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Use property file = %s", m_strPropertyFile.c_str());
  }
}


/***************************************************************************/
/*! Executes some registry commands
*/
/***************************************************************************/
void TAppParamReader::executeCommands()
{
  if (m_bUnregister)
  {
    unregisterModule(m_strModuleName);
  }
  else if (m_bRegister)
  {
    registerModule(m_strModuleName, m_strRegFileName);
  }
  else if (m_bDoList)
  {
    listModules();
  }
}

/***************************************************************************/
/*! Register a code module
    \param name : Name of the code module
*/
/***************************************************************************/
void TAppParamReader::registerModule(const TString& name, const TString& path)
{
  ZLOGGER.printfln(LOGMODE_INFO, "Register module %s...", name.c_str());
  
  TSecurityManager::getInstance().unregisterModule(name);
  unsigned long ulError = TSecurityManager::getInstance().registerModule(name, path);
  if (ulError == RET_NOERROR)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "..done");
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_INFO, "..failed.");
    switch(ulError)
    {
      case RET_REQUEST_FAILED : 
        ZLOGGER.printfln(LOGMODE_ERROR, "Code module %s already registered.", name.c_str());
      break;
      
      case RET_INVALID_DATA :
        ZLOGGER.printfln(LOGMODE_ERROR, "Code module %s not found. Must be a valid filename.", name.c_str());
      break;
      
      default:
        ZLOGGER.printfln(LOGMODE_ERROR, "Internal error occured. Security data base not loaded.");      
      break;
    }
  }
}

/***************************************************************************/
/*! Register a code module
    \param name : Name of the code module
*/
/***************************************************************************/
void TAppParamReader::unregisterModule(const TString& name)
{
  ZLOGGER.printfln(LOGMODE_INFO, "Unregister module %s...", name.c_str());
  
  unsigned long ulError = TSecurityManager::getInstance().unregisterModule(name);
  if (ulError == RET_NOERROR)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "..done");
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_INFO, "..failed.");
    switch(ulError)
    {
      case RET_REQUEST_FAILED : 
        ZLOGGER.printfln(LOGMODE_ERROR, "Code module %s not registered.", name.c_str());
      break;
      
      case RET_INVALID_DATA :
        ZLOGGER.printfln(LOGMODE_ERROR, "Code module %s not found. Must be a valid filename.", name.c_str());
      break;
      
      default:
        ZLOGGER.printfln(LOGMODE_ERROR, "Internal error occured. Security data base not loaded.");      
      break;
    }
  }
} 


/***************************************************************************/
/*! Lists all registrated code modules
*/
/***************************************************************************/
void TAppParamReader::listModules()
{
  TStringList lstModules;
  TSecurityManager::getInstance().getModules(&lstModules);
  IListIterator<IString*>* pIt = lstModules.getIterator();
  printf("List of registered modules:\n");
  while(pIt->hasNextItem())
  {
    printf("  %s\n", TString(pIt->getNextItem()).c_str());
  }
  printf("end of list.\n");
  lstModules.releaseIterator(pIt);
}

/***************************************************************************/
/*! checks if the framework should be started up
    \return true : start it up
    \retval false : do not start it up
*/
/***************************************************************************/
bool TAppParamReader::shouldStartup()
{
  return m_bStartFrame;
}

/***************************************************************************/
/*! checks if the property file name is set
    \return true : file name is set
    \retval false : file name is not set
*/
/***************************************************************************/
bool TAppParamReader::hasPropertyFile()
{
  return m_bHasPropertyFile;
}

/***************************************************************************/
/*! checks if the config file name is set
    \return true : file name is set
    \retval false : file name is not set
*/
/***************************************************************************/
bool TAppParamReader::hasConfigFile()
{
  return m_bHasConfigFile;
}

/***************************************************************************/
/*! Ausgeben der Hilfe zu den Startparametern
*/
/***************************************************************************/
void TAppParamReader::printUsage()
{
  printf("\n");
  printf("\
Zeus-Framework Version %s\n\
\n\
Welcome to Cell Computing Model. This application \n\
implements biological behaviours and is used for \n\
clustering and software decentralisation.\n\
To report problems please contact <bhadorn@swissinfo.org>\n\
\n\
Usage of zeus:\n\
  -conf=[XML_FILE]              Load the MOM Configuration (default=./zeus.xml)\n\
  -prop=[PROPERTY FILE]         Load a property file (default=./zeus.properties)\n\
\n\
Security related commands:\n\
  -file=[FILENAME]              File name of the module\n\
  -module=[NAME]                Module name\n\
  -reg                          Registeres a new code module for security.\n\
                                The -file and -modules parameters have to be specified\n\
  -unreg                        Unregisteres a code module from security manager\n\
                                The -modules parameters has to be specified\n\
  -listmodules                  This will list all registrated code modules\n\
\n\
Other flags:\n\
  -?, --help                    Get help how to use zeus command tool.\n\
  --version                     Show the version of zeus-framework.\n\
\n", ZEUS_VERSION);
}

/***************************************************************************/
/*! returns the configuration file of zeus
*/
/***************************************************************************/
TString TAppParamReader::getConfigFile()
{
  return m_strConfigFile;
}

/***************************************************************************/
/*! returns the property file of zeus
*/
/***************************************************************************/
TString TAppParamReader::getPropertyFile()
{
  return m_strPropertyFile;
}
