/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AppParamReader                                                *
 * Package : Zeus                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:22:03 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AppParamReader.h,v $
 *   Revision 1.2  2005/11/25 09:22:03  bhadorn
 *   Reading and parsing the application parameters before loading the framework
 *
 *   Revision 1.1  2005/10/28 12:09:54  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef APPPARAMREADER_H
#define APPPARAMREADER_H

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>
#include <zeusbase/System/Interfaces/IZUnknown.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Die Klasse AppParamReader liest alle Startparameter der Applikation 
    ein und Speichert diese in lokalen Speichervariablen ab.    
*/
/***************************************************************************/
class TAppParamReader : public TZObject
{
  public:
    TAppParamReader();
    virtual ~TAppParamReader();

    
    void parse(int argc, char *argv[]);
    bool shouldStartup();
    bool hasPropertyFile();
    bool hasConfigFile();
    TString getConfigFile();
    TString getPropertyFile();
    
    void executeCommands();
  
  private:
    ///Flag must be set, to start up the framework
    bool m_bStartFrame;
    ///Flag if the configfile is set
    bool m_bHasConfigFile;
    ///Flag if the property file is set
    bool m_bHasPropertyFile;
    ///XML Configuration file
    TString m_strConfigFile;
    ///Propertz File name
    TString m_strPropertyFile;
    ///Flag to register a module
    bool m_bRegister;
    ///Flag to unregister a module
    bool m_bUnregister;
    ///List all modules
    bool m_bDoList;
    ///File name for registering
    TString m_strRegFileName;
    ///Module name of registering and unregistering
    TString m_strModuleName;
    
    void printUsage();
    void registerModule(const TString& name, const TString& path);
    void unregisterModule(const TString& name);
    void listModules();
};

END_NAMESPACE_Zeus


#endif
