/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FrameLoader                                                   *
 * Package : zeus                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:22:03 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CCMFrameLoader.cpp,v $
 *   Revision 1.2  2005/11/25 09:22:03  bhadorn
 *   Reading and parsing the application parameters before loading the framework
 *
 *   Revision 1.1  2005/10/28 12:09:54  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <examples/framework/CCMFrameLoader.h> 
#include <examples/framework/AppParamReader.h>
#include <zeusbase/System/Application.h>
#include <zeusbase/System/LoggerManager.h>


USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates the FrameLoader
*/
/***************************************************************************/
TCCMFrameLoader::TCCMFrameLoader(int argc, char *argv[]) : TAbstractFrameLoader(L"zeus", argc, argv)
{
  m_pApplication = new TApplication();
  //Parse all application parameters
  m_pAppParameter = new TAppParamReader();
  m_pAppParameter->parse(argc, argv);
  
  if (m_pAppParameter->hasPropertyFile())
  {
    m_strPropertyFileName = m_pAppParameter->getPropertyFile();  
  }
  
  if (m_pAppParameter->hasConfigFile())
  {
    m_strConfigFileName = m_pAppParameter->getConfigFile();
  }
}


/***************************************************************************/
/*! Destroys the FrameLoader
*/
/***************************************************************************/
TCCMFrameLoader::~TCCMFrameLoader()
{
  m_pApplication->release();
  m_pApplication = NULL;
  
  m_pAppParameter->release();
  m_pAppParameter = NULL;
}

/***************************************************************************/
/*! See TAbstractFrameLoader
*/
/***************************************************************************/
void TCCMFrameLoader::initialize()
{
  TAbstractFrameLoader::initialize();
  m_pAppParameter->executeCommands();
}

/***************************************************************************/
/*! See TAbstractFrameLoader
*/
/***************************************************************************/
bool TCCMFrameLoader::canStartup()
{
  return m_pAppParameter->shouldStartup();
}
    
/***************************************************************************/
/*! Starts the application and returns the error code
*/
/***************************************************************************/
long TCCMFrameLoader::run()
{
  m_pApplication->start();
  return 0;
}

/***************************************************************************/
/*! This method starts the framework
*/
/***************************************************************************/
unsigned long TCCMFrameLoader::startup()
{
  ZLOGGER.printfln(LOGMODE_INFO, "Start up Cell Computing Model...");
  ZLOGGER.indent();
  
  unsigned long ulRetval = TAbstractFrameLoader::startup();
  
  if (ulRetval == RET_NOERROR)
  {
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "..done");
    
    ZLOGGER.printfln(LOGMODE_INFO, "Started up. Status is ready.");
  }
  else
  {
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "..failed");
  }
  return ulRetval;
}

/***************************************************************************/
/*! This method terminates the framework
*/
/***************************************************************************/
unsigned long TCCMFrameLoader::shutdown()
{
  ZLOGGER.printfln(LOGMODE_INFO, "Shutdown Cell Computing Model...");
  ZLOGGER.indent();
  
  unsigned long ulRetval = TAbstractFrameLoader::shutdown();
  
  ZLOGGER.unindent();
  ZLOGGER.printfln(LOGMODE_INFO, "..done");
  return ulRetval;
}
