/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : genetic_test                                                  *
 * Package : examples                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:00 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ExIndividual.cpp,v $
 *   Revision 1.1  2005/10/28 12:11:00  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
     
#include <zeusbase/Math/GALongGene.hpp>
#include <zeusbase/Math/GAChromosone.h>
#include <zeusbase/System/Time.h>
#include <ExIndividual.h>
#include <stdio.h>
    
USING_NAMESPACE_Zeus


/***************************************************************************/
/*! Creates the example individual
*/
/***************************************************************************/
TExIndividual::TExIndividual(TTravellingSalesMan& salesman) : TGAIndividual(), m_SalesMan(salesman)
{}
    
/***************************************************************************/
/*! Destroys the example individual
*/
/***************************************************************************/
TExIndividual::~TExIndividual()
{}


/***************************************************************************/
/*! See IGAIndividual
*/
/***************************************************************************/
void MQUALIFIER TExIndividual::createChromosones()
{
  TGAChromosone* pChrom = new TGAChromosone(0, TGAChromosone::ctCustom, customCrossFunction);
  TArrayList<long> lstCities;
  m_SalesMan.getRandomWay(&lstCities);
  long lCount = lstCities.getCount();
  for(int i = 0; i < lCount; i++)
  {
    TGALongGene* pGene = new TGALongGene(lstCities[i]);
    pChrom->addGene(pGene);
    pGene->release();
  } 

  this->m_lstChromosones.add(pChrom);
}


/***************************************************************************/
/*! See IGAIndividual
*/
/***************************************************************************/
double MQUALIFIER TExIndividual::getFitness()
{
  TArrayList<long> lstCities;
  IGAChromosone* pChromosone = NULL;
  if (this->getChromosone(0, &pChromosone) == RET_NOERROR)
  {
    long lCount = pChromosone->getGeneCount();
    for(int i = 0; i < lCount; i++)
    { 
      IGALongGene* pGene = NULL;
      if (pChromosone->getGene(i, (IGAGene**)&pGene) == RET_NOERROR)
      {
        lstCities.add(pGene->getLongValue());
        pGene->release();
      }
    }
    pChromosone->release();
  }
  return m_SalesMan.calcWay(&lstCities);
}

/***************************************************************************/
/*! Returns the index of a specific city
    \param lst : List of all cities
    \param value : city
    \return Index
*/
/***************************************************************************/
long TExIndividual::findCity(TArrayList<long>& lst, long value)
{
  long lIndex = 0;
  long lCount = lst.getCount();
  while(lst[lIndex] != value && lIndex < lCount)
  {
    lIndex++;
  }
  return lIndex;
}

/***************************************************************************/
/*! Finds an unvisited city
    \param mark : List of visited/unvisited cities
    \param index : start index
    \return Index of unvisited city
*/
/***************************************************************************/
long TExIndividual::findUnvisitedCity(TArrayList<bool> mark, long index)
{
  long lCount = mark.getCount();

  long lI = 1;
  long lNextPos = (index + lI) % lCount;
  while(mark[lNextPos] == true && lI <= lCount)
  {
    lI++;
    lNextPos = (index + lI) % lCount;
  }
  return lNextPos;
}

                   
/***************************************************************************/
/*! Special algorithme to cross two chromosones of the salesman problem.
    It has been invented by M. Abbhl
    \param lstGenesA : List of genes from chromosone A
    \param lstGenesA : List of genes from chromosone B
    \param lstNewGene : List of genes of the new chromosone
*/
/***************************************************************************/
void TExIndividual::crossAlgo(TArrayList<long>& lstGenesA, TArrayList<long>& lstGenesB, TArrayList<long>& lstNewGene)
{
  /*--------------------------------------------------
     Algorithme to cross the chromosones (By M.Abbhl)
    --------------------------------------------------

     1. Initialise the startvalues
       1.1. StartIndexA = 0
            StartIndexB = 0
       1.2. parent = random({A,B})
       1.3. Decide witch parent is selected
         1.3.1. If parent == A then lNextValue = A(StartIndexA)
         1.3.2. If parent == B then lNextValue = B(StartIndexA)
       1.4 Child.add(lNextValue)

     2. Repeate n-times do
       2.1. If parent == A then
         2.1.1. StartIndexB = findCity(B, A(StartIndexA))
         2.1.2. visitCity(B, StartIndexB)

       else
         2.1.3 StartIndexA = findCity(A, B(StartIndexB))
         2.1.3. visitCity(A, StartIndexA)

         //Illustrates if random parent is A
                    StartIndexA
                    |
         Parent_A:  1 - 3 - 4 - 5 - 2
         Parent_B:  5 - 2 - 3 - 1 - 4
                                |
                                StartIndexB


       2.2. Decide witch parent we take to evaluate the next value
         2.2.1. parent = random({A,B})
         2.2.2. If parent == A then
                      StartIndexA
                      |   lNextValue
                      |   |
           Parent_A:  1 - 3 - 4 - 5 - 2
           Parent_B:  5 - 2 - 3 - 1 - 4

           2.2.2.1. lNextValue = findUnvisitedCity(A, StartIndexA)
           2.2.2.2. StartIndexA = lNextValue
           2.2.2.3. visitCity(A, StartIndexA)

         2.2.3. If parent == B then
           Parent_A:  1 - 3 - 4 - 5 - 2
           Parent_B:  5 - 2 - 3 - 1 - 4
                                  |   |
                                  |   lNextValue
                                  StartIndexB
           2.2.3.1 lNextValue = findUnvisitedCity(B, StartIndexB)
           2.2.3.2. StartIndexB = lNextValue
           2.2.3.3. visitCity(B, StartIndexB)

       2.3. Child.add(lNextValue)
            Child:     1 - 4     //If we have taken B for example.

  */


  /*1.1*/
  long lCount = lstGenesA.getCount();
  long lStartIndexA = 0;
  long lStartIndexB = 0;
  TArrayList<bool> lstVisitedA;
  TArrayList<bool> lstVisitedB;
  for(int i = 0; i < lCount; i++)
  {
    lstVisitedA.add(false);
    lstVisitedB.add(false);
  }

  /*1.2*/
  long lParent = TTime::random(0, 100)%2;

  /*1.3*/
  long lNextValue;
  if (lParent == 0)
  {
    lNextValue = lstGenesA[0];
    lstVisitedA[0] = true;
  }
  else
  {
    lNextValue = lstGenesB[0];
    lstVisitedB[0] = true;
  }

  /*1.4*/
  lstNewGene.add(lNextValue);

  /*2.*/
  for(int i = 0; i < lCount-1; i++)
  {
    /*2.1*/
    if (lParent == 0)
    {
      lStartIndexB = findCity(lstGenesB, lstGenesA[lStartIndexA]);
      lstVisitedB[lStartIndexB] = true;
    }
    else
    {
      lStartIndexA = findCity(lstGenesA, lstGenesB[lStartIndexB]);
      lstVisitedA[lStartIndexA] = true;
    }/*2.1*/

    /*2.2.1*/
    lParent = TTime::random(0, 100)%2;
    /*2.2.2*/
    if (lParent == 0)
    {
      /*2.2.2.1*/
      lStartIndexA = findUnvisitedCity(lstVisitedA, lStartIndexA);
      /*2.2.2.2*/
      lNextValue = lstGenesA[lStartIndexA];
      /*2.2.2.3*/
      lstVisitedA[lStartIndexA] = true;
    }
    /*2.2.3*/
    else
    {
      /*2.2.3.1*/
      lStartIndexB = findUnvisitedCity(lstVisitedB, lStartIndexB);
      /*2.2.3.2*/
      lNextValue = lstGenesB[lStartIndexB];
      /*2.2.3.3*/
      lstVisitedB[lStartIndexB] = true;
    }

    /*2.3*/
    lstNewGene.add(lNextValue);
  }/*2.*/
}

/***************************************************************************/
/*! Customized crossing function.
*/
/***************************************************************************/
unsigned long TExIndividual::customCrossFunction(IGAChromosone* parent1,
                                                 IGAChromosone* parent2, 
                                                 TGAChromosone* new_chrom1, 
                                                 TGAChromosone* new_chrom2)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (parent1 == NULL || parent2 == NULL || new_chrom1 == NULL || new_chrom2 == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    long lCount = parent1->getGeneCount();
    TArrayList<long> lstGenesA(lCount+2);
    TArrayList<long> lstGenesB(lCount+2);
    TArrayList<long> lstNewGenes1(lCount+2);
    TArrayList<long> lstNewGenes2(lCount+2);

    //Convert genes into a array list of simple values
    for(int i = 0; i < lCount; i++)
    {
      IGALongGene* pGene = NULL;
      if (parent1->getGene(i, (IGAGene**)&pGene) == RET_NOERROR)
      {
        lstGenesA.add(pGene->getLongValue());
        pGene->release();
      }
      if (parent2->getGene(i, (IGAGene**)&pGene) == RET_NOERROR)
      {
        lstGenesB.add(pGene->getLongValue());
        pGene->release();
      }
    }

    //Generate 2 new chromosone for child 1 and child 2
    crossAlgo(lstGenesA, lstGenesB, lstNewGenes1);
    crossAlgo(lstGenesA, lstGenesB, lstNewGenes2);

    //Convert the arrays back into genes
    for(int i = 0; i < lCount; i++)
    {
      IGAGene* pGeneA = new TGALongGene(lstNewGenes1[i]);
      IGAGene* pGeneB = new TGALongGene(lstNewGenes2[i]);
      new_chrom1->addGene(pGeneA);
      new_chrom2->addGene(pGeneB);
      pGeneA->release();
      pGeneB->release();
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


