/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : TravellingSalesMan                                            *
 * Package : examples                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:00 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: TravellingSalesMan.cpp,v $
 *   Revision 1.1  2005/10/28 12:11:00  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/    
    
#include <zeusbase/System/Time.h>
#include <TravellingSalesMan.h>    
#include <math.h>
#include <stdio.h>

USING_NAMESPACE_Zeus
    
/***************************************************************************/
/*! Create a travelling salesman problem.
*/
/***************************************************************************/
TTravellingSalesMan::TTravellingSalesMan() : TZObject(), m_lstCities(RCity(L"",0,0), 20)

{
  //Create 10 Cities to travel to
  m_lstCities.add(RCity("A", 4, 4));
  m_lstCities.add(RCity("B", 6,15));
  m_lstCities.add(RCity("C",11, 2));
  m_lstCities.add(RCity("D", 8, 9));
  m_lstCities.add(RCity("E",15,10));
  m_lstCities.add(RCity("F", 3,10));
  m_lstCities.add(RCity("G",12,14));
  m_lstCities.add(RCity("H", 8, 5));
  m_lstCities.add(RCity("I",18, 6));
  m_lstCities.add(RCity("J",20,15));

  //More cities to add
 // m_lstCities.add(RCity("K",0,1));
}


/***************************************************************************/
/*! Destructor
*/
/***************************************************************************/
TTravellingSalesMan::~TTravellingSalesMan()
{}
 
/***************************************************************************/
/*! Calculates the way of a possible solution
    \param way: List of indexes representing cities.
*/
/***************************************************************************/
double TTravellingSalesMan::calcWay(TArrayList<long>* way)
{
  double dRetval = 0.0;
  long lCount = way->getCount();;
  RCity Point1 = m_lstCities[way->getItem(0)];
    
  //Get one more than the list contains -> get first as last
  for(int i = 1; i <= lCount; i++)
  {
    RCity& Point2 = m_lstCities[way->getItem(i%lCount)];
    dRetval += sqrt(pow(Point2.lPosX - Point1.lPosX,2.0) + pow(Point2.lPosY - Point1.lPosY,2.0));
    Point1 = Point2;
  }

  return dRetval;
}


/***************************************************************************/
/*! Returns the shortest way using a brutforce algorithme
*/
/***************************************************************************/
double TTravellingSalesMan::getWayFromBruteForce()
{
  //Start city 
  long lCount = m_lstCities.getCount();
  TArrayList<long> lstIndexes;
  for(int i = 0; i < lCount; i++)
  {
    lstIndexes.add(i);
  }
  return getShortestWay(lstIndexes, 0);
}


double TTravellingSalesMan::getShortestWay(TArrayList<long>& cities, long order_index)
{
  //Debug out to check if each city has been visited
/*  printf("Permutation way generated: ");
  for(int i = 0; i < cities.getCount(); i++)
  {
    printf(";%d", cities[i]);
  }
  printf(";\n");
*/
  
  double dRetval = calcWay(&cities);
  long lCount = cities.getCount();
  if (order_index < lCount-1)
  {
    long lSwap = cities[order_index];
    for(int i = order_index+1; i < lCount; i++)
    {
      cities[order_index] = cities[i];
      cities[i] = lSwap;
      double dWay = getShortestWay(cities, order_index + 1);
      if (dWay < dRetval)
      {
        dRetval = dWay;
      }
      cities[i] = cities[order_index];
      cities[order_index] = lSwap;
    }
  }
  return dRetval;
}

/***************************************************************************/
/*! Creates a random way 
*/
/***************************************************************************/
void TTravellingSalesMan::getRandomWay(TArrayList<long>* way)
{
  long lCount = m_lstCities.getCount();
  for (int i = 0; i < lCount; i++)
  {
    way->add(i);
  }
  
  //Shuffle function
  for (int i = 0; i < 10; i++)
  {
    unsigned long ulIndex1 = TTime::random(0, lCount-1);
    unsigned long ulIndex2 = TTime::random(0, lCount-1);
    long lSwapValue = way->getItem(ulIndex1);
    way->getItem(ulIndex1) = way->getItem(ulIndex2);
    way->getItem(ulIndex2) = lSwapValue;
  }  

  //Debug out to check if each city has been visited
  /*
  printf("Random way generated: ");
  for(int i = 0; i < way->getCount(); i++)
  {
    printf(";%d", way->getItem(i));
  }
  printf(";\n");
  */
}
