/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : TravellingSalesMan                                            *
 * Package : examples                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:00 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: TravellingSalesMan.h,v $
 *   Revision 1.1  2005/10/28 12:11:00  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef TravellingSalesManH 
#define TravellingSalesManH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ArrayList.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This structure represents a city with name and coordinates
*/
/***************************************************************************/
struct RCity
{
  ///Name of the city
  TString strName;
  ///Coordinate in X
  long lPosX;
  ///Coordinate in Y
  long lPosY;
  ///Flag is used for brute force algorithme
  bool bVisited;
  
  RCity()
  {
    lPosX = 0;
    lPosY = 0;
    bVisited = false;
  }
  
  RCity(TString name, long x ,long y)
  {
    strName = name;
    lPosX = x;
    lPosY = y;
    bVisited = false;
  }
  
  bool operator==(const RCity& inpar) const
  {
    return (strName == inpar.strName &&
            lPosX == inpar.lPosX &&
            lPosY == inpar.lPosY &&
            bVisited == inpar.bVisited);
  }
};


/***************************************************************************/
/*! Travelling sales man problem
*/
/***************************************************************************/
class TTravellingSalesMan : public TZObject
{
  public:
    TTravellingSalesMan();
    virtual ~TTravellingSalesMan();
        
    double getWayFromBruteForce();
    double calcWay(TArrayList<long>* way);
    void getRandomWay(TArrayList<long>* way);
    
  protected:
    
  private:
    //List of all cities
    TArrayList<RCity> m_lstCities;

    double getShortestWay(TArrayList<long>& cities, long order_index);
};


END_NAMESPACE_Zeus

#endif
 
