/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : genetic_test                                                  *
 * Package : examples                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:00 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: genetic_test.cpp,v $
 *   Revision 1.1  2005/10/28 12:11:00  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
  
#include <zeusbase/Math/GAPopulation.h> 
#include <zeusbase/System/Time.h> 
#include <ExIndividual.h> 
#include <TravellingSalesMan.h> 
#include <stdio.h>

//Tests the crossing method of the salesman problem
//#define TEST_CROSSING

//Calculates the bruteforce time for a given city count
//#define GET_BRUTEFORCE_TIME 

USING_NAMESPACE_Zeus

TTravellingSalesMan g_SalesManProblem;


/***************************************************************************/
/*! Factory function to create individuals
    \param ind : Return value of the individual
    \param create_chromosones : Flag if chromosones should be created.
*/
/***************************************************************************/
static unsigned long createExampleIndividuals(IGAIndividual** ind, bool create_chromosones)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ind == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *ind = new TExIndividual(g_SalesManProblem);
    
    if (create_chromosones)
    {
      (*ind)->createChromosones();
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

#ifdef TEST_CROSSING

/***************************************************************************/
/*! Test if the cross algorithme is stable
*/
/***************************************************************************/
void testCrossAlgorithme()
{
  //Test chromosone A
  TArrayList<long> lstA;
  lstA.add(3);
  lstA.add(0);
  lstA.add(6);
  lstA.add(4);
  lstA.add(2);
  lstA.add(9);
  lstA.add(7);
  lstA.add(8);
  lstA.add(1);
  lstA.add(5);
  printf("Chromosone A: ");
  for(int i = 0; i < lstA.getCount(); i++)
  {
    printf("%d:", (int)lstA[i]);
  }
  printf("\n");

  //Test chromosone B
  TArrayList<long> lstB;
  lstB.add(6);
  lstB.add(9);
  lstB.add(4);
  lstB.add(3);
  lstB.add(5);
  lstB.add(8);
  lstB.add(1);
  lstB.add(7);
  lstB.add(2);
  lstB.add(0);
  printf("Chromosone B: ");
  for(int i = 0; i < lstB.getCount(); i++)
  {
    printf("%d:", (int)lstB[i]);
  }
  printf("\n");

  //Cross the chromosone and get chromosone C
  TArrayList<long> lstNewA;
  TExIndividual::crossAlgo(lstA, lstB, lstNewA);
  printf("Chromosone C: ");
  for(int i = 0; i < lstNewA.getCount(); i++)
  {
    printf("%d:", (int)lstNewA[i]);

    //Remove in B every city found in C.
    lstB.remove(lstNewA[i]);
  }
  printf("\n");

  //Throw exception if not all cities has been removed
  // -> Not every city has been visited
  if (lstB.getCount() > 0)
  {
    printf("Error B: ");
    for(int i = 0; i < lstB.getCount(); i++)
    {
      printf("%d:", (int)lstB[i]);
    }
    printf("\n");
    throw lstNewA;
  }
}

#endif

/***************************************************************************/
/* This is a example how to use the genetic algorithmes of the zeus
   framework
*/ 
/***************************************************************************/
int main(int /*argc*/, char** /*argv[]*/)
{  
  TTime::initRandom();
  
  //--------------------------------------------------------------------------
  //Test the crossing algorithme
  #ifdef TEST_CROSSING
    for(int i = 0; i < 1000; i++)
    {
      testCrossAlgorithme();
    }

  //--------------------------------------------------------------------------
  //get the brute force time
  #else
  #ifdef GET_BRUTEFORCE_TIME
    
    //Get time stamp to measure performance
    double dTimeStamp1 = TTime::getTimeStamp();
    double dWay = g_SalesManProblem.getWayFromBruteForce();
    double dTimeStamp2 = TTime::getTimeStamp();
    printf("Shortest way is %f\n",dWay);
    printf("Time spend for brute force: %f seconds.\n", dTimeStamp2-dTimeStamp1);
    
  //--------------------------------------------------------------------------
  //Test the genetic algorithme
  #else
    // Creates a population.
    TGAPopulation* pPopulation = new TGAPopulation(createExampleIndividuals,         /*Factory function*/
                                                   TGAPopulation::cmKeepBestParents, /*Keep the best parent individuals*/
                                                   TGAPopulation::cmCrossBestPrior,  /*The best will have more chance to be crossed*/
                                                   TGAPopulation::fmSmallerBetter,   /*Priorize smaller fitness (shorter way to visit cities)*/
                                                   80.0                              /*Replace 80% of the population with new children*/
                                                   );

    //Generate 100 Individuals
    pPopulation->createPopulation(100);
    printf("Population created\n\n");

    //Get time stamp to measure performance
    double dTimeStamp1 = TTime::getTimeStamp();

    //Repeate 40 times having a possible best individual before aborting
    printf("%d Generation waited\n", pPopulation->makeGenerations(40));

    //Get time stamp 2 to measure performance
    double dTimeStamp2 = TTime::getTimeStamp();
    printf("Time spend for genetic algorithme: %f seconds.\n", dTimeStamp2-dTimeStamp1);

    //Get the best individual out of the population and
    // print the result
    IGAIndividual* pIndividual = NULL;
    if (pPopulation->getBestIndividual(&pIndividual) == RET_NOERROR)
    {
      printf("\nFitness of the best is: %f\n", pIndividual->getFitness());
      pIndividual->release();
    }

    pPopulation->release();

  #endif
  #endif
  //--------------------------------------------------------------------------

  return 0;
}
