/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : CommandDispatcher                                             *
 * Package : mandelbrot                                                    *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:19:27 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommandDispatcher.cpp,v $
 *   Revision 1.1  2005/11/25 09:19:27  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <examples/mandelbrot/CommandDispatcher.h>
#include <examples/mandelbrot/MandelbrotFrame.h>
#include <examples/mandelbrot/GUI/GraphicView.h>
#include <zeusbase/CellModel/Interfaces/ILocalPipeRegistry.hpp>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/Messaging/Interfaces/IXMLMessage.hpp>
#include <zeusbase/Messaging/Interfaces/IObjectMessage.hpp>

#include <qmutex.h>
#include <qapplication.h>
#include <qprogressbar.h>

//tmp
#include <zeusbase/Messaging/ObjectMessage.h>



USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a command dispatcher
*/
/***************************************************************************/
TCommandDispatcher::TCommandDispatcher(QApplication& app) : TThread(), m_Application(app)
{
  m_pGraphic = NULL;
  m_pGUIPipe = NULL;
  m_pOutputPipe = NULL;
  m_pRegistry = NULL;
}


/***************************************************************************/
/*! Destroys the command dispatcher
*/
/***************************************************************************/
TCommandDispatcher::~TCommandDispatcher()
{
  releaseCommunication();
}
 
/***************************************************************************/
/*! sends a request out to the 
*/
/***************************************************************************/
void TCommandDispatcher::sendRequest(IMandelbrotFrame* frame)
{
  if (m_pOutputPipe != NULL && frame != NULL)
  {
    TObjectMessage* pMessage = NULL;
    if (TObjectMessage::createMessage(L"Request", L"RelayCell", L"OutgoingPipe", &pMessage) == RET_NOERROR)
    {
      TString strCellName(L"GUIGATEWAY");
      TString strPipeName(PIPENAME_STDIN);
      pMessage->setSenderCellName(&strCellName);
      pMessage->setSenderPipeName(&strPipeName);
      pMessage->setObject(frame);
      pMessage->setDataType(MTYPE_MANDELBROT);
      if (m_pOutputPipe->sendMessage(pMessage) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Dispatcher: Message sent");
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_ERROR, "Dispatcher: Could not send message");
      }
      pMessage->release();
    }
  }
}

/***************************************************************************/
/*! Run method of the dispatcher thread
*/
/***************************************************************************/
void TCommandDispatcher::execute()
{
  while(!this->m_bInterrupted)
  {
    if (!m_pGUIPipe->isEmpty())
    {
      IMessage* pMessage = NULL;
      unsigned long long ullTime = 0;
      if (m_pGUIPipe->getMessage(&pMessage, &ullTime) == RET_NOERROR)
      {        
        switch(pMessage->getInterfaceID())
        {                  
          case INTERFACE_IObjectMessage:
          {
            IObjectMessage* pObjectMessage = NULL;
            if (pMessage->askForInterface(INTERFACE_IObjectMessage, (IZUnknown**)&pObjectMessage) == RET_NOERROR)
            {
              ZLOGGER.println(LOGMODE_INFO, "ObjectMessage received.");
              this->dispatchObjectMessage(pObjectMessage);
              pObjectMessage->release();
            }
          }
          break;
          
          default:
            ZLOGGER.println(LOGMODE_INFO, "Unknown message type received");
          break;
        }
        pMessage->release();
      }
      //ZLOGGER.println(LOGMODE_INFO, "pipe is not empty");
    }
    else
    {
      //ZLOGGER.println(LOGMODE_INFO, "pipe is empty");
    }
    TTime::usleep(1000);
  }
}

/***************************************************************************/
/*! This method dispatches all incomming messages to the screen.
*/
/***************************************************************************/
void TCommandDispatcher::dispatchObjectMessage(IObjectMessage* msg)
{
  unsigned long ulType = msg->getDataType();
  
  m_Application.lock();
  switch(ulType)
  {
    case MTYPE_MANDELBROT :
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Dispatcher: Mandelbrot message recevied");
      IMandelbrotFrame* pFrame = NULL;
      if (m_pGraphic != NULL && msg->getObject((ISerializable**)&pFrame) == RET_NOERROR)
      {
        m_pGraphic->showData(pFrame);
        pFrame->release();
      }
    }
    break;
    
    default:    
      ZLOGGER.printfln(LOGMODE_INFO, "Dispatcher: Invalid message received");
    break;
  }  
  m_Application.unlock();
}


/***************************************************************************/
/*! Releases the communication
*/
/***************************************************************************/
void TCommandDispatcher::releaseCommunication()
{
  if (m_pOutputPipe != NULL)
  {
    m_pOutputPipe->release();
    m_pOutputPipe = NULL;
  }
  
  if (m_pRegistry != NULL)
  {
    m_pRegistry->unregisterPipe(m_pGUIPipe);
    m_pRegistry->release();
    m_pRegistry = NULL;
  }
  if (m_pGUIPipe != NULL)
  {
    m_pGUIPipe->lock();
    m_pGUIPipe->release();
    m_pGUIPipe = NULL;
  }
  kill();
}

/***************************************************************************/
/*! Sets up the communication for communicating with the cells
*/
/***************************************************************************/
void TCommandDispatcher::setupCommunication(ILocalPipeRegistry* reg)
{
  releaseCommunication();
  m_pRegistry = reg;
  if (m_pRegistry != NULL)
  {
    m_pRegistry->addRef();
    //Pipe for GUI gateway
    TString strPipeName = PIPENAME_STDIN;
    TString strModulName = L"GUIGATEWAY";
    if (m_pRegistry->createPipe(&strModulName, &strPipeName, PIPEPRIORITY_HIGH, true, &m_pGUIPipe) == RET_NOERROR)
    {
      m_pGUIPipe->unlock();
      ZLOGGER.printfln(LOGMODE_INFO, "GUI Gateway pipe created");  
      m_pRegistry->registerPipe(m_pGUIPipe);
    }
    
    TString strOutModuleName = L"RelayCell";
    TString strOutPipeName = L"OutgoingPipe";
    if (m_pRegistry->getPipe(&strOutModuleName, &strOutPipeName, &m_pOutputPipe) == RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "GUI Gateway outputpipe created");  
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "No output pipe for GUI Gateway created.");
    }
  }
}

/***************************************************************************/
/*! Sets the reference to the original display instance
*/
/***************************************************************************/
void TCommandDispatcher::setGraphicDisplay(TGraphicView* disp)
{
  m_pGraphic = disp;
}

