/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : CommandDispatcher                                             *
 * Package : mandelbrot                                                    *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:19:27 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommandDispatcher.h,v $
 *   Revision 1.1  2005/11/25 09:19:27  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CommandDispatcherH
#define CommandDispatcherH 

#include <zeusbase/System/Thread.h>

class QApplication;
class QProgressBar;
class TGraphicView;

BEGIN_NAMESPACE_Zeus

class ILocalPipeRegistry;
class IOwnCommPipe;
class ICommPipe;
class IObjectMessage;
class IMandelbrotFrame;

/***************************************************************************/
/*! Dispatcher Interface to dispatch the commands from the gui to the 
    ccm
*/
/***************************************************************************/
class TCommandDispatcher : public TThread
{
  public:
    TCommandDispatcher(QApplication& app);
    
    void setupCommunication(ILocalPipeRegistry* reg);
    void releaseCommunication();
    
    void setGraphicDisplay(TGraphicView* disp);
    void sendRequest(IMandelbrotFrame* frame);
    
  protected:
    virtual ~TCommandDispatcher();
    virtual void execute();
    
  private:
    ///Display to output the original movie
    TGraphicView* m_pGraphic;
    ///Input pipe
    IOwnCommPipe* m_pGUIPipe;
    ///Command output pipe
    ICommPipe* m_pOutputPipe;
    ///Pipe registry
    ILocalPipeRegistry* m_pRegistry;
    ///Application object of QT
    QApplication& m_Application;
    
    void dispatchObjectMessage(IObjectMessage* msg);
    
};

END_NAMESPACE_Zeus

#endif
