/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : GraphicView                                                   *
 * Package : mandelbrot                                                    *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:19:57 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GraphicView.cpp,v $
 *   Revision 1.1  2005/11/25 09:19:57  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <qpainter.h> 
#include <math.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <examples/mandelbrot/GUI/GraphicView.h>    
#include <examples/mandelbrot/MandelbrotFrame.h>    
    
USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new movie display
*/
/***************************************************************************/
TGraphicView::TGraphicView(QWidget* parent, const char* name) : QWidget(parent, name)
                                                                
{
  m_dPictureTimeStamp = 0;
  m_dFrameTimeAv = 0;
  m_lIndex = 0;
  m_pDispatcher = NULL;
  m_pImage = new QImage(0,0,32);
  m_lbFrame = NULL;
  m_lbPicture = NULL;
  m_lbPosX = NULL;
  m_lbPosY = NULL;
  
  setUpdatesEnabled(true);
  setMouseTracking(true);
  m_lWidth = 0;
  m_lHeight = 0;
  m_dScale = 100;
  m_dZoomFactor = 1.5;
  m_lMaxRange = 10000;
  m_lIterations = 1000;
  
  m_dCenterX = 0;
  m_dCenterY = 0;
  m_lScreenCenterX = 0;
  m_lScreenCenterY = 0;
  
  TFuzzySet R1(2550);
  TFuzzySet G1(2550);
  TFuzzySet B1(2550);
  R1.initTrapezodial(50, 420,850,1275);
  G1.initTrapezodial(71,420,640, 1275);
  B1.initTrapezodial(0, 900, 1225, 1275);
  
  TFuzzySet R2(2550);
  TFuzzySet G2(2550);
  TFuzzySet B2(2550);
  R2.initTrapezodial(1275+50, 1275+420,1275+850, 1275+1275);
  G2.initTrapezodial(1275+71,1275+420, 1275+640, 1275+1275);
  B2.initTrapezodial(1275+0, 1275+900, 1275+1225, 1275+1275);
  
  TFuzzySet R3 = R1.unionS(R2);
  TFuzzySet G3 = G1.unionS(G2);
  TFuzzySet B3 = B1.unionS(B2);
  
  
  m_pRedSet = new TFuzzySet(R3);
  m_pGreenSet = new TFuzzySet(G3);
  m_pBlueSet = new TFuzzySet(B3);
  
/*  m_pRedSet->initTrapezodial(420, 840,1700,2550);
  m_pGreenSet->initTrapezodial(0,420,1280, 2550);
  m_pBlueSet->initTrapezodial(840, 1280, 2450, 2550);*/
}

/***************************************************************************/
/*! Destrys the movie display
*/
/***************************************************************************/
TGraphicView::~TGraphicView()
{
  if (m_pDispatcher != NULL)
  {
    m_pDispatcher->release();
    m_pDispatcher = NULL;
  }
  delete m_pImage;
  delete m_pRedSet;
  delete m_pBlueSet;
  delete m_pGreenSet;
}


/***************************************************************************/
/*! Sets the dispatcher
*/
/***************************************************************************/
void TGraphicView::setDispatcher(TCommandDispatcher* disp)
{
  if (m_pDispatcher != NULL)
  {
    m_pDispatcher->release();
    m_pDispatcher = NULL;
  }
  m_pDispatcher = disp;
  if (m_pDispatcher != NULL)
  {
    m_pDispatcher->addRef();
  }  
  else
  {
    m_lbFrame = NULL;
    m_lbPicture = NULL;
    m_lbPosX = NULL;
    m_lbPosY = NULL;
  }
}

/***************************************************************************/
/*! Sets the picture time information label
*/
/***************************************************************************/
void TGraphicView::setPictureTimeInfo(QLabel* lb)
{
  m_lbPicture = lb;
}

/***************************************************************************/
/*! Sets the frame time information label
*/
/***************************************************************************/
void TGraphicView::setFrameTimeInfo(QLabel* lb)
{
  m_lbFrame = lb;
}

/***************************************************************************/
/*! Sets the position information label
*/
/***************************************************************************/
void TGraphicView::setPositionInfo(QLabel* lbX, QLabel* lbY)
{
  m_lbPosX = lbX;
  m_lbPosY = lbY;
}


/***************************************************************************/
/*! Changes the zoom factor
*/
/***************************************************************************/
void TGraphicView::setZoomFactor(double value)
{
  m_dZoomFactor = value;
  ZLOGGER.printfln(LOGMODE_INFO, "Zoom Factor changed [%f]", m_dZoomFactor);
}

/***************************************************************************/
/*! Changes the number of iterations
*/
/***************************************************************************/
void TGraphicView::setIterations(long value)
{
  m_lIterations = value;
  ZLOGGER.printfln(LOGMODE_INFO, "Iterations changed [%d]", m_lIterations);
}
    
/***************************************************************************/
/*! Changes the maximum range
*/
/***************************************************************************/
void TGraphicView::setMaxRange(long value)
{
  m_lMaxRange = value;
  ZLOGGER.printfln(LOGMODE_INFO, "Max range changed [%d]", m_lMaxRange);
}

/***************************************************************************/
/*! Resets the graphic view
*/
/***************************************************************************/
void TGraphicView::reset()
{
  m_dScale = 100;
  m_dCenterX = 0;
  m_dCenterY = 0;
  refresh();
}

/***************************************************************************/
/*! Test print of the color table
*/
/***************************************************************************/
void TGraphicView::printColorTable()
{
  for(int i = 0; i < 255; i++)
  {
    unsigned long ulRed = (unsigned long)(m_pRedSet->getValue(i*10)*255.0);
    unsigned long ulGreen = (unsigned long)(m_pGreenSet->getValue(i*10)*255.0);
    unsigned long ulBlue = (unsigned long)(m_pBlueSet->getValue(i*10)*255.0);
    m_pImage->setPixel(5 + i, m_lHeight - 5, QColor(ulRed,
                                       ulGreen,
                                       ulBlue).rgb());
    m_pImage->setPixel(5 + i, m_lHeight - 4, QColor(ulRed,
                                       ulGreen,
                                       ulBlue).rgb());
    
  }
  repaint(); 
}

/***************************************************************************/
/*! Shows the actual data
*/
/***************************************************************************/
void TGraphicView::showData(IMandelbrotFrame* frame)
{
  if (frame != NULL && frame->getTimeStamp() == m_dPictureTimeStamp)
  {
    double dTimeNow = TTime::getTimeStamp();
    long lX1 = frame->getX1();
    long lY1 = frame->getY1();
    long lX2 = frame->getX2();
    long lY2 = frame->getY2();
    IList<double>* pList = NULL;
    frame->getData(&pList);
    long lIndex = 0;
    for(int i = lY1; i < lY2; i++)
    {
      for(int j = lX1; j < lX2; j++)
      {
        setDataPixel(j, i, pList->getItem(lIndex));
        lIndex++;
      }
    }
    
    m_lIndex++;
    double dSpentTime = dTimeNow - frame->getTimeStamp();
    
    //Time spent printing whole image
    if (m_lbPicture != NULL)
    {
      TString strData = TString(L"Picture: ") + TString(dSpentTime) + L" sec";
      m_lbPicture->setText(_QTWSTR(strData));
    }
    
    //Average time spent for frames
    if (m_lbFrame != NULL)
    {
      m_dFrameTimeAv = (m_dFrameTimeAv*(m_lIndex-1) + dSpentTime)/m_lIndex;
      TString strData = TString(L"Frame: ") + TString(m_dFrameTimeAv) + L" sec";
      m_lbFrame->setText(_QTWSTR(strData));
    }
      
    repaint(m_lScreenCenterX+lX1, 
            m_lScreenCenterY+lY1, 
            lX2-lX1, 
            lY2-lY1, false); 
  }
  else if(frame != NULL)
  {
    ZLOGGER.printfln(LOGMODE_WARN, "Picture frame is wasted.");  
  }
}

/***************************************************************************/
/*! Shows the actual data
*/
/***************************************************************************/
void TGraphicView::setDataPixel(long x, long y, double data)
{
  double dRangeVal = ((data*2550.0)/(double)m_lIterations);
  long lRange = (long)dRangeVal;// % 2550;
  
  unsigned long ulRed = (unsigned long)(m_pRedSet->getValue(lRange)*255.0);
  unsigned long ulGreen = (unsigned long)(m_pGreenSet->getValue(lRange)*255.0);
  unsigned long ulBlue = (unsigned long)(m_pBlueSet->getValue(lRange)*255.0);

  m_pImage->setPixel(m_lScreenCenterX + x, m_lScreenCenterY + y, QColor(ulRed,
                                       ulGreen,
                                       ulBlue).rgb());
}


/***************************************************************************/
/*! Refresh the graphic view
*/
/***************************************************************************/
void TGraphicView::refresh()
{
  m_dPictureTimeStamp = TTime::getTimeStamp();
  m_dFrameTimeAv = 0;
  m_lIndex = 0;
  
  //ZLOGGER.printfln(LOGMODE_INFO, "Refresh picture");
  delete m_pImage;
  m_pImage = new QImage(m_lWidth, m_lHeight, 32);
  if (m_pDispatcher != NULL && m_lWidth > 0 && m_lHeight > 0)
  {
    long lFrameWidth = 64;
    for(int i = -m_lScreenCenterY; i < m_lScreenCenterY-lFrameWidth; i += lFrameWidth)
    {
      for(int j = -m_lScreenCenterX; j < m_lScreenCenterX-lFrameWidth; j += lFrameWidth)
      {
        TMandelbrotFrame* pFrame = new TMandelbrotFrame(j, 
                                                        i, 
                                                        j+lFrameWidth, 
                                                        i+lFrameWidth, 
                                                        m_lIterations, 
                                                        m_lMaxRange, 
                                                        m_dScale, 
                                                        m_dCenterX, 
                                                        m_dCenterY, 
                                                        m_dPictureTimeStamp);
        m_pDispatcher->sendRequest(pFrame);
        pFrame->release();
      }
    }
  }
  else
  {
  }
  adjustSize();
  repaint(false); 
}

/***************************************************************************/
/*! Mouse event received
*/
/***************************************************************************/
void TGraphicView::mousePressEvent( QMouseEvent* event)
{
  m_dCenterX += (m_lScreenCenterX - event->x())/m_dScale;
  m_dCenterY += (m_lScreenCenterY - event->y())/m_dScale;
  if (event->button() == Qt::LeftButton)
  {
    m_dScale *= m_dZoomFactor;
  }
  else if (event->button() == Qt::RightButton)
  {
    m_dScale /= m_dZoomFactor;
  }
  refresh();
}

/***************************************************************************/
/*! Mouse move event
*/
/***************************************************************************/
void TGraphicView::mouseMoveEvent ( QMouseEvent * event )
{ 
  if (m_lbPosX != NULL && m_lbPosY != NULL)
  {
    double dPosX = m_dCenterX + (m_lScreenCenterX - event->x())/m_dScale;
    double dPosY = m_dCenterY + (m_lScreenCenterY - event->y())/m_dScale;
    TString strDataX = TString(L"X:") + dPosX;
    TString strDataY = TString(L"Y:") + dPosY;
    
    m_lbPosX->setText(_QTWSTR(strDataX));
    m_lbPosY->setText(_QTWSTR(strDataY));
    
  }
}


/***************************************************************************/
/*! Paint event
*/
/***************************************************************************/
void TGraphicView::paintEvent(QPaintEvent* event)
{
  QPainter painter(this);
  painter.save();
  QPoint Point(event->rect().left(),event->rect().top());
  painter.drawImage(Point, *m_pImage, event->rect());  
  painter.restore();
}

/*****************************************************************************/
/*! Resize event
*/
/*****************************************************************************/
void TGraphicView::resizeEvent(QResizeEvent* event)
{
  m_lWidth = event->size().width();
  m_lHeight = event->size().height();
  m_lScreenCenterX = m_lWidth/2;
  m_lScreenCenterY = m_lHeight/2;
}


/*****************************************************************************/
/*! Return the size of the image
*/
/*****************************************************************************/
QSize TGraphicView::sizeHint() const 
{
  QSize size(m_lWidth, m_lHeight);
  return size;
}

