/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : GraphicView                                                   *
 * Package : mandelbrot                                                    *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:19:57 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GraphicView.h,v $
 *   Revision 1.1  2005/11/25 09:19:57  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef GraphicViewH
#define GraphicViewH

#include <qwidget.h>
#include <qlabel.h>
#include <qimage.h>
#include <qpen.h>
#include <qpointarray.h>
#include <zeusbase/Math/FuzzySet.h>
#include <examples/mandelbrot/CommandDispatcher.h>
#include <examples/mandelbrot/Interfaces/IMandelbrotFrame.hpp>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! 
*/
/***************************************************************************/
class TGraphicView : public QWidget
{  
  public:
    
    TGraphicView(QWidget* parent, const char* name);
    virtual ~TGraphicView();
    
    void setDispatcher(TCommandDispatcher* disp);
    void setPictureTimeInfo(QLabel* lb);
    void setFrameTimeInfo(QLabel* lb);
    void setPositionInfo(QLabel* lbX, QLabel* lbY);
    void showData(IMandelbrotFrame* frame);
    
    void setZoomFactor(double value);
    void setIterations(long value);
    void setMaxRange(long value);
    void reset();
    void refresh();
    void printColorTable();
      
  protected:
    virtual void paintEvent(QPaintEvent* event);
    virtual void resizeEvent(QResizeEvent * event);
    virtual void mousePressEvent( QMouseEvent* event);
    virtual void mouseMoveEvent ( QMouseEvent* event );
    
    ///ID of the actual frames
    double m_dPictureTimeStamp;
    ///Image witch contains the visual data 
    QImage* m_pImage;
    ///Image width
    long m_lWidth;
    ///Image height
    long m_lHeight;
    ///Center of the graphic in pixels
    long m_lScreenCenterX;
    ///Center of the graphic in pixels
    long m_lScreenCenterY;
    ///Max value range
    long m_lMaxRange;
    ///MAx iterations
    long m_lIterations;
    
    long double m_dScale;
    double m_dZoomFactor;
    long double m_dCenterX;
    long double m_dCenterY;
    double m_dFrameTimeAv;
    double m_dPictureTime;
    long m_lIndex;
    
    
  private:
    void setDataPixel(long x, long y, double data);
  
    NAMESPACE_Zeus::TFuzzySet* m_pRedSet;
    NAMESPACE_Zeus::TFuzzySet* m_pGreenSet;
    NAMESPACE_Zeus::TFuzzySet* m_pBlueSet;
    TCommandDispatcher* m_pDispatcher;
    QLabel* m_lbFrame;
    QLabel* m_lbPicture;
    QLabel* m_lbPosX;
    QLabel* m_lbPosY;
    
  public slots:
    QSize sizeHint() const;
};

#endif 
