/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

/****************************************************************************/
/*! Unfreezes the form and adds the widget for displaying the mandelbrot set.
*/
/****************************************************************************/
void TMainForm::unfreeze()
{
  m_pGraphicLayout = new QVBoxLayout( m_pnView, 1, 1, "m_pGraphicLayout");  
  m_pGraphicDisplay = new TGraphicView(m_pnView, "m_pGraphicDisplay");
  m_pGraphicLayout->addWidget(m_pGraphicDisplay);
  m_pGraphicDisplay->setIterations(m_pIterSlide->value());
  m_pGraphicDisplay->setMaxRange(m_pRangeSlide->value());
  m_pGraphicDisplay->setZoomFactor(m_edtZoom->value()/100.0);
  m_pGraphicDisplay->setPictureTimeInfo(this->m_lbPictureTime);
  m_pGraphicDisplay->setFrameTimeInfo(this->m_lbFrameTime);
  m_pGraphicDisplay->setPositionInfo(this->m_lbPosX, this->m_lbPosY);
  m_pDispatcher = NULL;
}


/****************************************************************************/
/*! Freezes the form
*/
/****************************************************************************/
void TMainForm::freeze()
{
  if (m_pDispatcher != NULL)
  {
    m_pDispatcher->setGraphicDisplay(NULL);
    m_pDispatcher->release();
    m_pDispatcher = NULL;
  }
}

/****************************************************************************/
/*! The refresh button is pushed. It will redraw the actual view using the new settings
*/
/****************************************************************************/
void TMainForm::onRefresh()
{
  if (m_pGraphicDisplay != NULL)
  {
    m_pGraphicDisplay->refresh() ;
  }
}


/****************************************************************************/
/*! The Zoom has been changed. This will set the zoom factor
*/
/****************************************************************************/
void TMainForm::onZoomChanged( int value)
{
  if (m_pGraphicDisplay != NULL)
  {
    m_pGraphicDisplay->setZoomFactor(value/100.0);
  }
}


/****************************************************************************/
/*! This changed the number of iterations for calculating the manderbrot set
*/
/****************************************************************************/
void TMainForm::onIterationChanged( int value )
{
  if (m_pGraphicDisplay != NULL)
  {
    m_pGraphicDisplay->setIterations(value);
  }
}


/****************************************************************************/
/*! This changed the maximum range for calculating the manderbrot set
*/
/****************************************************************************/
void TMainForm::onRangeChanged( int value )
{
  if (m_pGraphicDisplay != NULL)
  {
    m_pGraphicDisplay->setMaxRange(value);
  }
}


void TMainForm::onReset()
{
  if (m_pGraphicDisplay != NULL)
  {
    m_pGraphicDisplay->reset();
  }
}


void TMainForm::setCommandDispatcher( NAMESPACE_Zeus::TCommandDispatcher * value )
{
  if (m_pDispatcher != NULL)
  {
    m_pDispatcher->release();
    m_pDispatcher = NULL;
  }
  this->m_pDispatcher = value;
  if (m_pDispatcher != NULL)
  {
    m_pDispatcher->addRef();
  }
  if (m_pGraphicDisplay != NULL && m_pDispatcher != NULL)
  {
    m_pGraphicDisplay->setDispatcher(m_pDispatcher);
    m_pDispatcher->setGraphicDisplay(m_pGraphicDisplay);
  }
}


void TMainForm::onShowColorTable()
{
  if (m_pGraphicDisplay != NULL)
  {
    m_pGraphicDisplay->printColorTable();
  }
}
