/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : MandelbrotCell                                                *
 * Package : MandelbrotCellModule                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: MandelbrotCell.h,v $
 *   Revision 1.1  2005/11/25 09:20:44  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef MandelbrotCellH
#define MandelbrotCellH

#include <zeusdev/AbstractCell.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/System/XObjectFactory.h>

BEGIN_NAMESPACE_Zeus

class IMandelbrotFrame;

/****************************************************************************/
/*! This cell calculates a range of mandelbrot values and returns them
    to the caller
*/
/****************************************************************************/
class TMandelbrotCell : public TAbstractCell
{
  public :
    TMandelbrotCell(IXMLNode* node);
    
    REG_SUB_BEGIN(TMandelbrotCell)
    REG_SUB_ADD(TMandelbrotCell, L"TMandelbrotCell");
    REG_SUB_END
    
  protected:
    virtual ~TMandelbrotCell();
    
    virtual void performObjectMessage(IObjectMessage* msg, unsigned long long timer);
    virtual void onInitialLife();
    virtual void onEndLife();
    
  private:
    //Nummer of the Message
    unsigned long m_ulMessageID;
 
    unsigned long calcData(IMandelbrotFrame* frame);
    double getValue(double x, double y, long it, long range);
    double getValueTest(double x, double y, long it, long range);
  
};

END_NAMESPACE_Zeus

#endif
