/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : MandelbrotFrame                                               *
 * Package : mandelbrot                                                    *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:19:27 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: MandelbrotFrame.cpp,v $
 *   Revision 1.1  2005/11/25 09:19:27  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <examples/mandelbrot/MandelbrotFrame.h>
#include <zeusbase/System/CriticalSection.h>    

USING_NAMESPACE_Zeus


/***************************************************************************/
/*! Creates a frame out of a stream
*/
/***************************************************************************/
SERIAL_CONSTRUCTOR_START(TMandelbrotFrame);
  m_pLock = new TCriticalSection();
SERIAL_CONSTRUCTOR_END

/***************************************************************************/
/*! Creates an empty frame
*/
/***************************************************************************/
TMandelbrotFrame::TMandelbrotFrame() : TZObject()
{
  m_pLock = new TCriticalSection();
  m_lX1 = 0;
  m_lY1 = 0;
  m_lX2 = 0;
  m_lY2 = 0;
  m_lRange = 0;
  m_lIterations = 0;
  m_dScale = 1;
  m_dCenterX = 0;
  m_dCenterY = 0;
  m_dStamp = 0;
}

/***************************************************************************/
/*! Creates a valid mandelbrot frame
    \param x1 : Start point in pixels
    \param y1 : Start point in pixels
    \param x2 : End point in pixels
    \param y2 : End point in pixels
    \param it : Max iterations
    \param range : Max value range
    \param scale : Scale of the view
    \param center_x : Center of the view (value)
    \param center_y : Center of the view (value)
    \param stamp : Time stamp and id of the picture
*/
/***************************************************************************/
TMandelbrotFrame::TMandelbrotFrame(long x1, 
                                   long y1, 
                                   long x2, 
                                   long y2, 
                                   long it, 
                                   long range, 
                                   double scale, 
                                   double center_x, 
                                   double center_y,
                                   double stamp)
{
  m_pLock = new TCriticalSection();
  m_lX1 = x1;
  m_lY1 = y1;
  m_lX2 = x2;
  m_lY2 = y2;
  m_lRange = range;
  m_lIterations = it;
  m_dScale = scale;
  m_dCenterX = center_x;
  m_dCenterY = center_y;
  m_dStamp = stamp;
}

/***************************************************************************/
/*! Destroys the frame
*/
/***************************************************************************/
TMandelbrotFrame::~TMandelbrotFrame()
{ 
  m_pLock->release(); 
  m_pLock = NULL;
}
    
long MQUALIFIER TMandelbrotFrame::getX1() const
{ return m_lX1; }
long MQUALIFIER TMandelbrotFrame::getY1() const
{ return m_lY1; }

long MQUALIFIER TMandelbrotFrame::getX2() const
{ return m_lX2; }

long MQUALIFIER TMandelbrotFrame::getY2() const
{ return m_lY2; }

long MQUALIFIER TMandelbrotFrame::getMaxRange() const
{ return m_lRange; }

long MQUALIFIER TMandelbrotFrame::getIterations() const
{ return m_lIterations; }

double MQUALIFIER TMandelbrotFrame::getCenterX() const
{ return m_dCenterX; }

double MQUALIFIER TMandelbrotFrame::getCenterY() const
{ return m_dCenterY; }

double MQUALIFIER TMandelbrotFrame::getScale() const
{ return m_dScale; }

double MQUALIFIER TMandelbrotFrame::getTimeStamp() const
{ return m_dStamp; }


/***************************************************************************/
/*! Returns the data pointer to read or add data to the mandelbrot set
    \param data : Return parameter of the data set
    \retval RET_NOERROR : Pointer returned
    \retval RET_INVALID_PARAMETER : Invalid parameter received
*/
/***************************************************************************/
unsigned long MQUALIFIER TMandelbrotFrame::getData(IList<double>** data)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (data != NULL)
  {
    *data = &m_lstData;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}
    
/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TMandelbrotFrame, m_pLock);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;
 
