/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : MandelbrotFrame                                               *
 * Package : mandelbrot                                                    *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:19:27 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: MandelbrotFrame.h,v $
 *   Revision 1.1  2005/11/25 09:19:27  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MandelbrotFrameH 
#define MandelbrotFrameH

#include <examples/mandelbrot/Interfaces/IMandelbrotFrame.hpp>
#include <zeusbase/System/ArrayList.hpp>
#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus

class TCriticalSection;

/***************************************************************************/
/*! This is the data package witch will be sent to cells for calculating 
    the mandelbrot set and to return the data for visualisation
*/
/***************************************************************************/
class TMandelbrotFrame : public TZObject, public IMandelbrotFrame
{
  public:
    TMandelbrotFrame();
    TMandelbrotFrame(long x1, long y1, long x2, long y2, long it, long range, double scale, double center_x, double center_y, double stamp);
    
    virtual long MQUALIFIER getX1() const;
    virtual long MQUALIFIER getY1() const;
    virtual long MQUALIFIER getX2() const;
    virtual long MQUALIFIER getY2() const;
    virtual long MQUALIFIER getMaxRange() const;
    virtual long MQUALIFIER getIterations() const;
    virtual double MQUALIFIER getCenterX() const;
    virtual double MQUALIFIER getCenterY() const;
    virtual double MQUALIFIER getScale() const;
    virtual double MQUALIFIER getTimeStamp() const;
    virtual unsigned long MQUALIFIER getData(IList<double>** data);
    
    SERIAL_START(TMandelbrotFrame, L"TMandelbrotFrame");
      SERIAL_LONG(m_lIterations)
      SERIAL_LONG(m_lRange)
      SERIAL_LONG(m_lX1)
      SERIAL_LONG(m_lY1)
      SERIAL_LONG(m_lX2)
      SERIAL_LONG(m_lY2)
      SERIAL_DOUBLE(m_dCenterX)
      SERIAL_DOUBLE(m_dCenterY)
      SERIAL_DOUBLE(m_dScale)
      SERIAL_DOUBLE(m_dStamp)
      SERIAL_DOUBLELIST(m_lstData)
    SERIAL_END
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TMandelbrotFrame();
  
  private:
    ///Startpos in X
    long m_lX1;
    ///Startpos in Y
    long m_lY1;
    ///Endpos in X
    long m_lX2;
    ///Endpos in Y
    long m_lY2;
    ///Range
    long m_lRange;
    ///Iterations
    long m_lIterations;
    ///Center in X
    double m_dCenterX;
    ///Center in Y
    double m_dCenterY;
    ///Scale
    double m_dScale;
    ///Timestamp and ID of the picture
    double m_dStamp;
    ///Data 
    TArrayList<double> m_lstData;
    ///This lock protects the object (release and addRef)
    TCriticalSection* m_pLock;
};

END_NAMESPACE_Zeus


#endif
 
