/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : CommandDispatcher                                             *
 * Package : motiondetect                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:20:58 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommandDispatcher.cpp,v $
 *   Revision 1.2  2005/12/14 09:20:58  bhadorn
 *   Possibility for displaying the time to analyse a film.
 *
 *   Revision 1.1  2005/10/28 12:11:50  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <examples/motiondetect/CommandDispatcher.h>
#include <examples/motiondetect/Interfaces/IDIBFragment.hpp>
#include <examples/motiondetect/ModecDefines.hpp> 
#include <zeusbase/CellModel/Interfaces/ILocalPipeRegistry.hpp>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/Messaging/Interfaces/IXMLMessage.hpp>
#include <zeusbase/Messaging/Interfaces/IObjectMessage.hpp>

#include <qmutex.h>
#include <qapplication.h>
#include <qprogressbar.h>
#include <qlabel.h>

//tmp
#include <zeusbase/Messaging/XMLMessage.h>



USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a command dispatcher
*/
/***************************************************************************/
TCommandDispatcher::TCommandDispatcher(QApplication& app) : TThread(), m_Application(app)
{
  m_pOriginalDisplay = NULL;
  m_pGUIPipe = NULL;
  m_pOutputPipe = NULL;
  m_pRegistry = NULL;
  m_pMotionDisplay = NULL;
  m_pProgressBar = NULL;
  m_pTimeLabel = NULL;
  m_dTimeStamp = 0;
}


/***************************************************************************/
/*! Destroys the command dispatcher
*/
/***************************************************************************/
TCommandDispatcher::~TCommandDispatcher()
{
  releaseCommunication();
}
 

/***************************************************************************/
/*! Run method of the dispatcher thread
*/
/***************************************************************************/
void TCommandDispatcher::execute()
{
  while(!this->m_bInterrupted)
  {
    if (!m_pGUIPipe->isEmpty())
    {
      IMessage* pMessage = NULL;
      unsigned long long ullTime = 0;
      if (m_pGUIPipe->getMessage(&pMessage, &ullTime) == RET_NOERROR)
      {        
        switch(pMessage->getInterfaceID())
        {        
          //Number of samples
          case INTERFACE_IXMLMessage :
          {
            IXObject* pObj = NULL;
            if (pMessage->askForInterface(INTERFACE_IXObject, (IZUnknown**)&pObj) == RET_NOERROR)
            {
              TString strName;
              pObj->getName(&strName);
              ZLOGGER.println(LOGMODE_INFO, "Message name is %s", strName.c_str());
              if (strName == L"SampleCount")
              {
                ZLOGGER.println(LOGMODE_INFO, "Maximum of frames received %d", pMessage->getNumber());
                if (m_pProgressBar != NULL)
                {
                  m_pProgressBar->setTotalSteps(pMessage->getNumber());
                }
              }
              pObj->release();
            }
          }
          break;
          
          //Object message -> Image to show
          case INTERFACE_IObjectMessage:
          {
            IObjectMessage* pObjectMessage = NULL;
            if (pMessage->askForInterface(INTERFACE_IObjectMessage, (IZUnknown**)&pObjectMessage) == RET_NOERROR)
            {
              //ZLOGGER.println(LOGMODE_INFO, "ObjectMessage received.");
              this->dispatchObjectMessage(pObjectMessage);
              pObjectMessage->release();
            }
          }
          break;
          
          default:
            ZLOGGER.println(LOGMODE_INFO, "Unknown message type received");
          break;
        }
        pMessage->release();
      }
      //ZLOGGER.println(LOGMODE_INFO, "pipe is not empty");
    }
    else
    {
      //ZLOGGER.println(LOGMODE_INFO, "pipe is empty");
    }
    TTime::usleep(1000);
  }
}

/***************************************************************************/
/*! This method dispatches all incomming messages to the screen.
*/
/***************************************************************************/
void TCommandDispatcher::dispatchObjectMessage(IObjectMessage* msg)
{
  unsigned long ulType = msg->getDataType();
  
  m_Application.lock();
  switch(ulType)
  {
    case PICTURE_VECTOR :
    {
      IDIBFragment* pFragment = NULL;
      if (msg->getObject((ISerializable**)&pFragment) == RET_NOERROR)
      {
        m_pMotionDisplay->setVectorFragment(pFragment);
        pFragment->release();
      }
    }
    break;
  
    case PICTURE_ORIGINAL :
    {
      IDIBFrame* pFrame = NULL;
      if (msg->getObject((ISerializable**)&pFrame) == RET_NOERROR)
      {
        m_pOriginalDisplay->setNewFrame(pFrame);
        pFrame->release();
      }
      if (m_pProgressBar != NULL)
      {
        m_pProgressBar->setProgress(msg->getNumber());
      }
      if (m_pTimeLabel != NULL)
      {
        
        TString strText = TString(L"Time: ") + TString(TTime::getTimeStamp() - m_dTimeStamp) + TString(L" sec");
        m_pTimeLabel->setText(strText.c_str());
      }
    }
    break;
    
    case PICTURE_MOTION :
    {
      IDIBFrame* pFrame = NULL;
      if (msg->getObject((ISerializable**)&pFrame) == RET_NOERROR)
      {
        m_pMotionDisplay->setNewFrame(pFrame);
        pFrame->release();
      }
    }
    default:
    
    break;
  }  
  m_Application.unlock();
}


/***************************************************************************/
/*! Releases the communication
*/
/***************************************************************************/
void TCommandDispatcher::releaseCommunication()
{
  if (m_pOutputPipe != NULL)
  {
    m_pOutputPipe->release();
    m_pOutputPipe = NULL;
  }
  
  if (m_pRegistry != NULL)
  {
    m_pRegistry->unregisterPipe(m_pGUIPipe);
    m_pRegistry->release();
    m_pRegistry = NULL;
  }
  if (m_pGUIPipe != NULL)
  {
    m_pGUIPipe->lock();
    m_pGUIPipe->release();
    m_pGUIPipe = NULL;
  }
  kill();
}

/***************************************************************************/
/*! Sets up the communication for communicating with the cells
*/
/***************************************************************************/
void TCommandDispatcher::setupCommunication(ILocalPipeRegistry* reg)
{
  releaseCommunication();
  m_pRegistry = reg;
  if (m_pRegistry != NULL)
  {
    m_pRegistry->addRef();
    //Pipe for GUI gateway
    TString strPipeName = PIPENAME_STDIN;
    TString strModulName = L"GUIGATEWAY";
    if (m_pRegistry->createPipe(&strModulName, &strPipeName, PIPEPRIORITY_HIGH, true, &m_pGUIPipe) == RET_NOERROR)
    {
      m_pGUIPipe->unlock();
      ZLOGGER.printfln(LOGMODE_INFO, "GUI Gateway pipe created");  
      m_pRegistry->registerPipe(m_pGUIPipe);
    }
    
    TString strOutModuleName = L"ModecInput";
    TString strOutPipeName = PIPENAME_STDIN;
    if (m_pRegistry->getPipe(&strOutModuleName, &strOutPipeName, &m_pOutputPipe) == RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "GUI Gateway outputpipe created");  
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "No output pipe for GUI Gateway created.");
    }
  }
}

/***************************************************************************/
/*! Sets the reference to the original display instance
*/
/***************************************************************************/
void TCommandDispatcher::setOriginalDisplay(TMovieDisplay* disp)
{
  m_pOriginalDisplay = disp;
}

/***************************************************************************/
/*! Sets the reference to the motion display instance
*/
/***************************************************************************/
void TCommandDispatcher::setMotionDisplay(TMovieDisplay* disp)
{
  m_pMotionDisplay = disp;
}

/***************************************************************************/
/*! Sets the reference of the progress bar
*/
/***************************************************************************/
void TCommandDispatcher::setProgressBar(QProgressBar* prog)
{
  m_pProgressBar = prog;
}

/***************************************************************************/
/*! Sets the reference of the time label
*/
/***************************************************************************/
void TCommandDispatcher::setTimeLabel(QLabel* label)
{
  m_pTimeLabel = label;
}



/***************************************************************************/
/*! Loads a movie file for motion detection
    \param filename : Name of the file
    \retval RET_NOERROR : File loaded
    \retval RET_REQUEST_FAILED : Could not load file
*/
/***************************************************************************/
unsigned long TCommandDispatcher::loadMovieFile(TString filename, long& lFrameCount)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pOutputPipe != NULL)
  {
    ZLOGGER.println(LOGMODE_INFO, "Load movie: %s", filename.c_str()); 
    IXMLMessage* pMessage = NULL;
    if (TXMLMessage::createMessage(L"LoadMovie", L"ModecInput", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
    {
      IXMLNode* pNode = NULL;
      if(pMessage->getDataNode(&pNode) == RET_NOERROR)
      {
        TString strAttName=L"FileName";
        pNode->setAttributeValue(&strAttName, &filename);
        pNode->release();
        /////////////
/*        TByteArray aTemp;
        pMessage->streamObject(&aTemp);
        ZLOGGER.println(LOGMODE_INFO, "Teststream: %s", aTemp.getArray()); */
        if (m_pOutputPipe->sendMessage(pMessage,0) == RET_NOERROR)
        {
          ZLOGGER.println(LOGMODE_INFO, "Load command message sent."); 
        }
      }
      else
      {
        ZLOGGER.println(LOGMODE_INFO, "Could not send load command message."); 
      }
      pMessage->release();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ZLOGGER.println(LOGMODE_INFO, "Could not load movie. No command output pipe."); 
  }
  return ulRetval;
}  

/***************************************************************************/
/*! Changes to the vector mode if the value is true
*/
/***************************************************************************/
void TCommandDispatcher::changeToVector(bool value)
{
  if (m_pOutputPipe != NULL)
  {
    ZLOGGER.println(LOGMODE_INFO, "Change mode"); 
    IXMLMessage* pMessage = NULL;
    if (TXMLMessage::createMessage(L"ChangeMode", L"ModecInput", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
    {
      IXMLNode* pNode = NULL;
      if(pMessage->getDataNode(&pNode) == RET_NOERROR)
      {
        TString strAttName=L"Mode";
        TString strValue = (long)value;
        pNode->setAttributeValue(&strAttName, &strValue);
        pNode->release();
      }
      m_pOutputPipe->sendMessage(pMessage,0);
      pMessage->release();
    }
  }
  else
  {
    ZLOGGER.println(LOGMODE_INFO, "Could not start movie. No command output pipe."); 
  }
}


/***************************************************************************/
/*! Starts the movie
*/
/***************************************************************************/
unsigned long TCommandDispatcher::onStart()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  m_dTimeStamp = TTime::getTimeStamp();
  if (m_pOutputPipe != NULL)
  {
    ZLOGGER.println(LOGMODE_INFO, "Start movie"); 
    IXMLMessage* pMessage = NULL;
    if (TXMLMessage::createMessage(L"StartMovie", L"ModecInput", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
    {
      m_pOutputPipe->sendMessage(pMessage,0);
      pMessage->release();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ZLOGGER.println(LOGMODE_INFO, "Could not start movie. No command output pipe."); 
  }
  return ulRetval;
}

/***************************************************************************/
/*! Stops the movie
*/
/***************************************************************************/
unsigned long TCommandDispatcher::onStop()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pOutputPipe != NULL)
  {
    ZLOGGER.println(LOGMODE_INFO, "Stop movie"); 
    IXMLMessage* pMessage = NULL;
    if (TXMLMessage::createMessage(L"StopMovie", L"ModecInput", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
    {
      m_pOutputPipe->sendMessage(pMessage,0);
      pMessage->release();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ZLOGGER.println(LOGMODE_INFO, "Could not stop movie. No command output pipe."); 
  }
  return ulRetval;
}
