/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : CommandDispatcher                                             *
 * Package : motiondetect                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:20:58 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommandDispatcher.h,v $
 *   Revision 1.2  2005/12/14 09:20:58  bhadorn
 *   Possibility for displaying the time to analyse a film.
 *
 *   Revision 1.1  2005/10/28 12:11:50  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CommandDispatcherH
#define CommandDispatcherH 

#include <zeusbase/System/Thread.h>
#include <examples/motiondetect/MovieDisplay.h>

class QApplication;
class QProgressBar;
class QLabel;

BEGIN_NAMESPACE_Zeus

class ILocalPipeRegistry;
class IOwnCommPipe;
class ICommPipe;
class IObjectMessage;

/***************************************************************************/
/*! Dispatcher Interface to dispatch the commands from the gui to the 
    ccm
*/
/***************************************************************************/
class TCommandDispatcher : public TThread
{
  public:
    TCommandDispatcher(QApplication& app);
    
    void setupCommunication(ILocalPipeRegistry* reg);
    void releaseCommunication();
    
    unsigned long loadMovieFile(TString filename, long& lFrameCount);
    unsigned long onStart();
    unsigned long onStop();
    void setOriginalDisplay(TMovieDisplay* disp);
    void setMotionDisplay(TMovieDisplay* disp);
    void setProgressBar(QProgressBar* prog);
    void setTimeLabel(QLabel* label);
    void changeToVector(bool value);
    
  protected:
    virtual ~TCommandDispatcher();
    virtual void execute();
    
  private:
    ///Display to output the original movie
    TMovieDisplay* m_pOriginalDisplay;
    ///Display to out put the motion
    TMovieDisplay* m_pMotionDisplay;
    ///Input pipe
    IOwnCommPipe* m_pGUIPipe;
    ///Command output pipe
    ICommPipe* m_pOutputPipe;
    ///Pipe registry
    ILocalPipeRegistry* m_pRegistry;
    ///Application object of QT
    QApplication& m_Application;
    ///Progress bar
    QProgressBar* m_pProgressBar;
    ///Label to show time
    QLabel* m_pTimeLabel;
    ///Timestamp
    double m_dTimeStamp;
    
    void dispatchObjectMessage(IObjectMessage* msg);
    
};

END_NAMESPACE_Zeus

#endif
