/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : IDIBBlock                                                  *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:49 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IDIBBlock.hpp,v $
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IDIBBlockHPP
#define IDIBBlockHPP

#include <zeusbase/System/Interfaces/ISerializable.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! The DIBBlock interface defines a block for block matching algorithme.
    A block is defined by a base position in the DIB Image, the size and 
    search aera.
*/
/***************************************************************************/
class IDIBBlock : public ISerializable
{
  public:
    
    /***********************************************************************/
    /*! gets the base pos in X (pixels)
    */
    /***********************************************************************/
    virtual long MQUALIFIER getBasePosX() const=0;
    
    /***********************************************************************/
    /*! gets the base pos in Y (pixels)
    */
    /***********************************************************************/
    virtual long MQUALIFIER getBasePosY() const=0;
    
    /***********************************************************************/
    /*! Gets the block size in pixels
    */
    /***********************************************************************/
    virtual long MQUALIFIER getSize() const=0;
    
    /***********************************************************************/
    /*! Gets the maximum movments in pixels. This defines the border of the
        block, witch has to be evaluated.
    */
    /***********************************************************************/
    virtual long MQUALIFIER getMaxMove() const=0;
    
    /***********************************************************************/
    /*! gets the move in X 
    */
    /***********************************************************************/
    virtual long MQUALIFIER getMoveX() const=0;
    
    /***********************************************************************/
    /*! gets the move in Y
    */
    /***********************************************************************/
    virtual long MQUALIFIER getMoveY() const=0;
    
    /***********************************************************************/
    /*! sets the move in X
    */
    /***********************************************************************/
    virtual void MQUALIFIER setMoveX(long value)=0;
    
    /***********************************************************************/
    /*! sets the move in Y
    */
    /***********************************************************************/
    virtual void MQUALIFIER setMoveY(long value)=0;
};


END_NAMESPACE_Zeus


#endif  
 
 
