/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : IDIBFragment                                                  *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:49 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IDIBFragment.hpp,v $
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IDIBFragmentHPP
#define IDIBFragmentHPP

#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <examples/motiondetect/Interfaces/IDIBFrame.hpp>
#include <examples/motiondetect/Interfaces/IDIBBlock.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! DIB Frames are containing the data and size of movie frames in DIB 
    format
*/
/***************************************************************************/
class IDIBFragment : public ISerializable
{
  public:
    
    /***********************************************************************/
    /*! Gets the ID of the current frame.
        \return Frame ID
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getFrameID() const=0;
    
    /***********************************************************************/
    /*! Gets the index of the fragment.
        \return Fragement index
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getFragmentIndex() const=0;
    
    /***********************************************************************/
    /*! Gets the count of blocks inside the fragment
        \return Fragement count
    */
    /***********************************************************************/
    virtual long MQUALIFIER getBlockCount() const=0;
    
    /***********************************************************************/
    /*! Gets a block out of the fragments
        \param index : Index of the block
        \param block : Return parameter of the block
        \retval RET_NOERROR : Block returned
        \retval RET_REQUEST_FAILED : Invalid index
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getBlock(long index, IDIBBlock** block) const=0;
    
    /***********************************************************************/
    /*! gets the current frame of the image
        \param frame : return parameter
        \retval RET_NOERROR : Frame returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No frame available
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getCurrentFrame(IDIBFrame** frame) const=0;
    
    /***********************************************************************/
    /*! gets the last frame of the image
        \param frame : return parameter
        \retval RET_NOERROR : Frame returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No frame available
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getLastFrame(IDIBFrame** frame) const=0;
    
    /***********************************************************************/
    /*! gets the total of fragments
    */
    /***********************************************************************/
    virtual long MQUALIFIER getTotalFragment() const=0;
    
    /***********************************************************************/
    /*! gets the total of fragments
    */
    /***********************************************************************/
    virtual void MQUALIFIER setTotalFragment(long count)=0;

    /***********************************************************************/
    /*! Adds a block to the fragment
        \param block : dib block
    */
    /***********************************************************************/
    virtual void MQUALIFIER addBlock(IDIBBlock* block)=0;
};


END_NAMESPACE_Zeus


#endif  
 
