/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : IDIBFrame                                                     *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:49 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IDIBFrame.hpp,v $
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IDIBFrameHPP
#define IDIBFrameHPP

#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/Interfaces/IByteArray.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! DIB Frames are containing the data and size of movie frames in DIB 
    format
*/
/***************************************************************************/
class IDIBFrame : public ISerializable
{
  public:
    
    /***********************************************************************/
    /*! Gets the id of the frame
        \return ID
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getID() const=0;
    
    /***********************************************************************/
    /*! Gets the width of a device independent image
        \return width
    */
    /***********************************************************************/
    virtual long MQUALIFIER getWidth() const=0;
    
    /***********************************************************************/
    /*! Gets the height of a device independent image
        \return height
    */
    /***********************************************************************/
    virtual long MQUALIFIER getHeight() const=0;
    
    /***********************************************************************/
    /*! Gets the count of image planes. Colored dib frames normaly has 3
        planes for red-green-blue
        \return plane count
    */
    /***********************************************************************/
    virtual long MQUALIFIER getPlaneCount() const=0;
    
    /***********************************************************************/
    /*! Returns the data of a dib
        \return data
    */
    /***********************************************************************/
    virtual const IByteArray& MQUALIFIER getData() const=0;
};


END_NAMESPACE_Zeus


#endif  
