/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : BlockMatchCell                                                *
 * Package : MotionDetectModule                                            *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/15 07:10:29 $
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: BlockMatchCell.cpp,v $
 *   Revision 1.3  2005/12/15 07:10:29  bhadorn
 *   Error in ssd algorithme corrected
 *
 *   Revision 1.2  2005/12/14 09:22:55  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <examples/motiondetect/MotionDetectModule/BlockMatchCell.h> 
#include <examples/motiondetect/Interfaces/IDIBFragment.hpp>
#include <examples/motiondetect/ModecDefines.hpp>
#include <zeusbase/Messaging/Interfaces/IXMLMessage.hpp>
#include <zeusbase/Messaging/ObjectMessage.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <math.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a blockmatching cell
*/
/***************************************************************************/
TBlockMatchCell::TBlockMatchCell(IXMLNode* node) : TAbstractCell(node)
{
  m_lSleepTime = 1000;
  m_dBias = 0.3;
  
  if (node != NULL)
  {
    TString strBiasName = L"Bias";
    TString strValue;
    node->getAttributeValue(&strBiasName, &strValue);
 
    m_dBias = strValue.toDouble(m_dBias);
  }
}

/***************************************************************************/
/*! destroys the cell
*/
/***************************************************************************/
TBlockMatchCell::~TBlockMatchCell()
{}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TBlockMatchCell::onInitialLife()
{}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TBlockMatchCell::onEndLife()
{
}

/***************************************************************************/
/*! Processing of binary messages
*/
/***************************************************************************/
void TBlockMatchCell::performObjectMessage(IObjectMessage* msg, unsigned long long timer)
{
  if (m_pCommunication != NULL && msg != NULL)
  {
    if (msg->getDataType() == PICTURE_FRAGMENT)
    {
      IDIBFragment* pFragment = NULL;
      if (msg->getObject((ISerializable**)&pFragment) == RET_NOERROR)
      {
        //Call the block matching algorithme for all blocks inside this fragment
        long lCount = pFragment->getBlockCount();
        for(int i = 0; i < lCount; i++)
        {
          doMatchBlock(i, pFragment);
        }
        pFragment->release();
      }
      
      //Send fragment to output cell
      TString strCellName = L"ModecOutput";
      TString strPipeName = PIPENAME_STDIN;
      msg->setTargetCellName(&strCellName);
      msg->setTargetPipeName(&strPipeName);
      m_pCommunication->sendMessage(msg, 0);
    }
  }
}

/***************************************************************************/
/*! Match the blocks. In this version of the modec we are using the SSD 
    algorithme to determinize the difference
    \param index : Index of the block
    \param fragment : Fragment of a movie frame
*/
/***************************************************************************/
void TBlockMatchCell::doMatchBlock(long index, IDIBFragment* fragment)
{
  IDIBBlock* pBlock = NULL;
  if (fragment->getBlock(index, &pBlock) == RET_NOERROR)
  {
    IDIBFrame* pFrame1 = NULL;  
    IDIBFrame* pFrame2 = NULL;
    
    fragment->getLastFrame(&pFrame1);
    fragment->getCurrentFrame(&pFrame2);
    
    if(pFrame1 != NULL && pFrame2 != NULL)
    {
      long lX = 0;
      long lY =0;
      
      ssd(pFrame1, pFrame2, pBlock->getBasePosX(), pBlock->getBasePosY(), pBlock->getSize(), pBlock->getMaxMove(), lX, lY);
      pBlock->setMoveX(lX);
      pBlock->setMoveY(lY);
    }

    if(pFrame1 != NULL)
    {
      pFrame1->release();
    }
    if(pFrame2 != NULL)
    {
      pFrame2->release();
    }
    pBlock->release();
  }
}

/***************************************************************************/
/*! Function to detect the offset of the move
    \param frame1 : previous frame
    \param frame2 : The frame now
    \param baseX : Center of the block in X
    \param baseY : Center of the block in Y
    \param block_size : Size of the block in pixels
    \param max_move : Maximum of the movement
    \param x : return value of the x offset
    \param y : return value of the y offset
*/
/***************************************************************************/
void TBlockMatchCell::ssd(IDIBFrame* frame1, IDIBFrame* frame2, long baseX, long baseY, long block_size, long max_move, long& x, long& y)
{
  //Minimum sum of differences
  double dMin = 0.0;
  //Min. sum is initialized = true
  bool bMinInit = false;
  //Search range in one direction (total is -lRange to lRange)
  long lRange = block_size/2;
  //Number of planes (colored images has 3 planes)
  long lPlaneCount = frame1->getPlaneCount();
  //Width of the picture
  long lWidth = frame1->getWidth();
  
  //Data of previous frame
  unsigned char* pBuffer1 = (unsigned char*)frame1->getData().getArray();
  //Data of this frame
  unsigned char* pBuffer2 = (unsigned char*)frame2->getData().getArray();
    
  //We are using the 3-step search strategy to minimize the 
  // work. O(8*log(n))
  long lPow = 2;
  //Step size
  long lStepSize = max_move;
  //temp. loaction of minimum differenc in X
  long lCenterX = 0;
  //temp. loaction of minimum differenc in Y
  long lCenterY = 0;
  
  long lMinPosX = baseX-lRange;
  long lMaxPosX = baseX+lRange;
  long lMinPosY = baseY-lRange;
  long lMaxPosY = baseY+lRange;
  
  //-----------------------------------------------------------------
  //Repeat until the stepsize became 1
  while (lStepSize > 1)
  {
    lStepSize = (long)ceil((double)max_move/lPow);
    lPow *= 2;
    long lMinX = -lStepSize + lCenterX;
    long lMaxX = lStepSize + lCenterX;
    long lMinY = -lStepSize + lCenterY;
    long lMaxY = lStepSize + lCenterY;
    for(int px = lMinX; px <= lMaxX; px += lStepSize)
    {
      for(int py = lMinY; py <= lMaxY; py += lStepSize)
      {
//        if (py == lMinY+lStepSize || px == lMinX+lStepSize)
        {
          //Get sum
          double dSum = 0.0;
          for(int ix = lMinPosX; ix < lMaxPosX-1; ix++)
          {
            for(int iy = lMinPosY; iy < lMaxPosY-1; iy++)
            {
              int k=0;
              //for(int k = 0; k < lPlaneCount; k++)
              {
                long lPosArray1 = ix*lPlaneCount+k + iy*lWidth*lPlaneCount;
                long lPosArray2 = (ix+px)*lPlaneCount+k + (iy+py)*lWidth*lPlaneCount;
                if (lPosArray1 >= 0 && lPosArray2 >= 0)
                {
                  double dVal1 = pBuffer1[lPosArray1]/255.0;
                  double dVal2 = pBuffer2[lPosArray2]/255.0;
                  double dDiff =  dVal1 - dVal2;
                              ;
                  dSum += dDiff*dDiff;
                }
              }
            }
          }
          
          if (!bMinInit || dMin > dSum)
          {
            dMin = dSum;
            x = px;
            y = py;
            bMinInit = true;
          }
        }
      }
    }
    
    //Set temporary location of minimum.
    lCenterX = x;
    lCenterY = y;
  }
  
  //We ignore differences smaller than the bias of 0.3
  if (dMin < m_dBias)
  {
    x = 0;
    y = 0;
  }
}
 
