/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : DIBBlock                                                   *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:49 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: DIBBlock.cpp,v $
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <examples/motiondetect/MotionDetectModule/DIBBlock.h>
#include <zeusbase/System/CriticalSection.h>

USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR_START(TDIBBlock)
{
  m_pLock = new TCriticalSection();
}
SERIAL_CONSTRUCTOR_END

/***************************************************************************/
/* Create a fragment of a DIB frame
*/
/***************************************************************************/
TDIBBlock::TDIBBlock(long blocksize, long maxmove, long posx, long posy) : TZObject()
{
  m_lBlockSize = blocksize;
  m_lMaxMove = maxmove;
  m_lBasePosX = posx;
  m_lBasePosY = posy;
  m_lMoveX = 0;
  m_lMoveY = 0;
  m_pLock = new TCriticalSection();
}

TDIBBlock::TDIBBlock() : TZObject()
{
  m_lBlockSize = 0;
  m_lMaxMove = 0;
  m_lBasePosX = 0;
  m_lBasePosY = 0;
  m_lMoveX = 0;
  m_lMoveY = 0;
  m_pLock = new TCriticalSection();
}

/***************************************************************************/
/*! Destroys the DIB fragment
*/
/***************************************************************************/
TDIBBlock::~TDIBBlock()
{
  m_pLock->release();
}

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
long MQUALIFIER TDIBBlock::getSize() const
{ 
  long lRetval = 0;
  m_pLock->enter();
  lRetval = m_lBlockSize;
  m_pLock->leave();
  return lRetval;
}

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
long MQUALIFIER TDIBBlock::getMaxMove() const
{ 
  long lRetval = 0;
  m_pLock->enter();
  lRetval = m_lMaxMove;
  m_pLock->leave();
  return lRetval;
}

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
long MQUALIFIER TDIBBlock::getBasePosX() const
{ 
  long lRetval = 0;
  m_pLock->enter();
  lRetval = m_lBasePosX;
  m_pLock->leave();
  return lRetval;
}

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
long MQUALIFIER TDIBBlock::getBasePosY() const
{ 
  long lRetval = 0;
  m_pLock->enter();
  lRetval = m_lBasePosY;
  m_pLock->leave();
  return lRetval;
}

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
long MQUALIFIER TDIBBlock::getMoveX() const
{ return m_lMoveX; }

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
long MQUALIFIER TDIBBlock::getMoveY() const
{ return m_lMoveY; }

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
void MQUALIFIER TDIBBlock::setMoveX(long value)
{ m_lMoveX = value; }

/***************************************************************************/
/*! See IDIBBlock
*/
/***************************************************************************/
void MQUALIFIER TDIBBlock::setMoveY(long value)
{ m_lMoveY = value; }

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TDIBBlock, m_pLock);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;
 
 
