/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : DIBBlock                                                  *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:49 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: DIBBlock.h,v $
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef DIBBlockH
#define DIBBlockH

#include <examples/motiondetect/Interfaces/IDIBBlock.hpp>
#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus

class TCriticalSection;
/***************************************************************************/
/*! The implementation of DIB Block for block matching
*/
/***************************************************************************/
class TDIBBlock : public TZObject, public IDIBBlock
{
  public:
    TDIBBlock(long blocksize, long maxmove, long posx, long posy);
    
    virtual long MQUALIFIER getSize() const;
    virtual long MQUALIFIER getMaxMove() const;
    virtual long MQUALIFIER getBasePosX() const;
    virtual long MQUALIFIER getBasePosY() const;
    virtual long MQUALIFIER getMoveX() const;
    virtual long MQUALIFIER getMoveY() const;
    virtual void MQUALIFIER setMoveX(long value);
    virtual void MQUALIFIER setMoveY(long value);
    
    //Methods of ISerializable
    SERIAL_START(TDIBBlock, L"TDIBBlock")
      SERIAL_LONG(m_lBlockSize);
      SERIAL_LONG(m_lMaxMove);
      SERIAL_LONG(m_lBasePosX);
      SERIAL_LONG(m_lBasePosY);
      SERIAL_LONG(m_lMoveX);
      SERIAL_LONG(m_lMoveY);
    SERIAL_END

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    TDIBBlock();
    virtual ~TDIBBlock();
    
    long m_lMoveX;
    long m_lMoveY;
    long m_lBlockSize;
    long m_lMaxMove;
    long m_lBasePosX;
    long m_lBasePosY;
  
  private:
    TCriticalSection* m_pLock;
};


END_NAMESPACE_Zeus


#endif  
  
