/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : DIBFragment                                                   *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:49 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: DIBFragment.cpp,v $
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <examples/motiondetect/MotionDetectModule/DIBFragment.h>
#include <zeusbase/System/CriticalSection.h>

USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR_START(TDIBFragment)
{
  m_pLock = new TCriticalSection();
}
SERIAL_CONSTRUCTOR_END

/***************************************************************************/
/* Create a fragment of a DIB frame
*/
/***************************************************************************/
TDIBFragment::TDIBFragment(IDIBFrame& current_frame, IDIBFrame& last_frame, unsigned long index)
{
  m_pCurrentFrame = &current_frame;
  m_pCurrentFrame->addRef();
  m_pLastFrame = &last_frame;
  m_pLastFrame->addRef();
  m_ulFragmentIndex = index;
  m_lCount = 0;
  m_pLock = new TCriticalSection();
}

/***************************************************************************/
/*! Create an empty fragment
*/
/***************************************************************************/
TDIBFragment::TDIBFragment() : TZObject()
{
  m_pCurrentFrame = NULL;
  m_pLastFrame = NULL;
  m_ulFragmentIndex = 0;
  m_lCount = 0;
  m_pLock = new TCriticalSection();
}    
  
/***************************************************************************/
/*! Destroys the fragment
*/
/***************************************************************************/
TDIBFragment::~TDIBFragment()
{
  if (m_pCurrentFrame != NULL)
  {
    m_pCurrentFrame->release();
    m_pCurrentFrame = NULL;
  }
  if (m_pLastFrame != NULL)
  {
    m_pLastFrame->release();
    m_pLastFrame = NULL;
  }
  m_pLock->release();
}
  
/***************************************************************************/
/*! Adds a new block to the fragment
*/
/***************************************************************************/
void MQUALIFIER TDIBFragment::addBlock(IDIBBlock* block)
{
  
  if (block != NULL)
  {
    m_pLock->enter();
    m_lstBlocks.add(block);
    block->addRef();
    m_pLock->leave();
  }
}
    
/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
unsigned long MQUALIFIER TDIBFragment::getFrameID() const
{
  unsigned long ulRetval = 0;
  if (m_pCurrentFrame != NULL)
  {
    ulRetval = m_pCurrentFrame->getID();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
unsigned long MQUALIFIER TDIBFragment::getFragmentIndex() const
{
  return m_ulFragmentIndex;
}

/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
long MQUALIFIER TDIBFragment::getBlockCount() const
{
  long lRetval = 0;
  m_pLock->enter();
  lRetval = m_lstBlocks.getCount();
  m_pLock->leave();
  return lRetval;
}

/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
unsigned long MQUALIFIER TDIBFragment::getBlock(long index, IDIBBlock** block) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (block == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    m_pLock->enter();
    *block = m_lstBlocks.getItemCopy(index);
    if (*block != NULL)
    {
      (*block)->addRef();
      m_pLock->leave();
      ulRetval = RET_NOERROR;
    }
    else
    {
      m_pLock->leave();
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
unsigned long MQUALIFIER TDIBFragment::getCurrentFrame(IDIBFrame** frame)const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (frame == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pCurrentFrame != NULL)
  {
    m_pLock->enter();
    *frame = m_pCurrentFrame;
    m_pCurrentFrame->addRef();
    m_pLock->leave();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
unsigned long MQUALIFIER TDIBFragment::getLastFrame(IDIBFrame** frame) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (frame == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pLastFrame != NULL)
  {
    m_pLock->enter();
    *frame = m_pLastFrame;
    m_pLastFrame->addRef();
    m_pLock->leave();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
long MQUALIFIER TDIBFragment::getTotalFragment() const
{
  return m_lCount;
}

/***************************************************************************/
/*! See IDIBFragment
*/
/***************************************************************************/
void MQUALIFIER TDIBFragment::setTotalFragment(long count)
{ m_lCount = count; }

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TDIBFragment, m_pLock);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;
 
 
