/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : DIBFragment                                                   *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:22:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: DIBFragment.h,v $
 *   Revision 1.2  2005/12/14 09:22:56  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef DIBFragmentH
#define DIBFragmentH

#include <examples/motiondetect/Interfaces/IDIBFragment.hpp>
#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus

class TCriticalSection;

/***************************************************************************/
/*! The implementation of DIB Fragments of a specific DIB Frame
*/
/***************************************************************************/
class TDIBFragment : public TZObject, public IDIBFragment
{
  public:
    TDIBFragment(IDIBFrame& current_frame, IDIBFrame& last_frame, unsigned long index);
    
    virtual void MQUALIFIER addBlock(IDIBBlock* block);
    virtual unsigned long MQUALIFIER getFrameID() const;
    virtual unsigned long MQUALIFIER getFragmentIndex() const;
    virtual long MQUALIFIER getBlockCount() const;
    virtual unsigned long MQUALIFIER getBlock(long index, IDIBBlock** block) const;
    virtual unsigned long MQUALIFIER getCurrentFrame(IDIBFrame** frame) const;
    virtual unsigned long MQUALIFIER getLastFrame(IDIBFrame** frame) const;
    virtual long MQUALIFIER getTotalFragment() const;
    virtual void MQUALIFIER setTotalFragment(long count);
    
    //Methods of ISerializable
    SERIAL_START(TDIBFragment, L"TDIBFragment")
      SERIAL_LONG(m_ulFragmentIndex)
      SERIAL_LONG(m_lCount)
      SERIAL_OBJECT(IDIBFrame, m_pCurrentFrame)
      SERIAL_OBJECT(IDIBFrame, m_pLastFrame)
      SERIAL_OBJECTLIST(IDIBBlock, m_lstBlocks)
    SERIAL_END
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    TDIBFragment();
    virtual ~TDIBFragment();
    
    ///Fragment index (relating to the number of fragments
    unsigned long m_ulFragmentIndex;
    ///number of fragments still pending (is used for assembling the fragments)
    long m_lCount;
    ///Current movie frame
    IDIBFrame* m_pCurrentFrame;
    ///Last movie frame
    IDIBFrame* m_pLastFrame;
    ///Blocks to match
    TArrayList<IDIBBlock*> m_lstBlocks;
  
  private:
    ///Lock for critical sections
    TCriticalSection* m_pLock;
};


END_NAMESPACE_Zeus


#endif  
  
