/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : DIBFrame                                                      *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/15 07:10:59 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: DIBFrame.cpp,v $
 *   Revision 1.2  2005/12/15 07:10:59  bhadorn
 *   Do copy array instead of using shared arrays
 *
 *   Revision 1.1  2005/12/14 09:22:55  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:50  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
     
#include <examples/motiondetect/MotionDetectModule/DIBFrame.h>
#include <examples/motiondetect/MotionDetectModule/DIBBlock.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/LoggerManager.h>
 
 
USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR_START(TDIBFrame)
{
  m_pLock = new TCriticalSection();
  m_pData = new TByteArray();
}
SERIAL_CONSTRUCTOR_END

/**************************************************************************/
/*! Creates a DIB frame 
    \param bitmap : Data stream containing the picture
    \param width : Picture width
    \param height : Picture height
    \param planes : Plane count for colored images
*/
/**************************************************************************/
TDIBFrame::TDIBFrame(unsigned long id, IByteArray& bitmap, long width, long height, long planes) : TZObject()
{
  m_ulID = id;
  m_lWidth = width;
  m_lHeight = height;
  m_lPlaneCount = planes;
  m_pData = new TByteArray(&bitmap);
  m_pLock = new TCriticalSection();
}

/**************************************************************************/
/*! Creates an empty DIB frame
*/
/**************************************************************************/
TDIBFrame::TDIBFrame() :TZObject()
{
  m_pLock = new TCriticalSection();
  m_ulID = 0;
  m_lWidth = 0;
  m_lHeight = 0;
  m_lPlaneCount = 0;
  m_pData = new TByteArray();
}
    
/**************************************************************************/
/*! Destroys the DIB frames
*/
/**************************************************************************/
TDIBFrame::~TDIBFrame()
{
  ZLOGGER.println(LOGMODE_INFO, "DIBFrame destroyed.");
  m_pLock->release();
  m_pData->release();
}

/**************************************************************************/
/*! see IDIBFrame
*/
/**************************************************************************/
unsigned long MQUALIFIER TDIBFrame::getID() const
{
  return m_ulID;
}

/**************************************************************************/
/*! Returns the width of the picture
*/
/**************************************************************************/
long MQUALIFIER TDIBFrame::getWidth() const
{ return m_lWidth; }

/**************************************************************************/
/*! Returns the height of the picture
*/
/**************************************************************************/
long TDIBFrame::getHeight() const
{ return m_lHeight; }

/**************************************************************************/
/*! Returns the plane count of the picture
*/
/**************************************************************************/
long MQUALIFIER TDIBFrame::getPlaneCount() const
{ return m_lPlaneCount; }

/**************************************************************************/
/*! Returns the data of the picture
*/
/**************************************************************************/
const IByteArray& MQUALIFIER TDIBFrame::getData() const
{ return *m_pData; }

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TDIBFrame, m_pLock);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;
