/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : DIBFrame                                                      *
 * Package : motiondetect                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:22:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: DIBFrame.h,v $
 *   Revision 1.1  2005/12/14 09:22:56  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:50  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef DIBFrameH
#define DIBFrameH

#include <examples/motiondetect/Interfaces/IDIBFrame.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ArrayList.hpp>
#include <zeusbase/System/ByteArray.hpp>

BEGIN_NAMESPACE_Zeus

class TCriticalSection;
class TDIBBlock;

/***************************************************************************/
/*! DIB Frames are containing the data and size of movie frames in DIB 
    format
*/
/***************************************************************************/
class TDIBFrame : public TZObject, public IDIBFrame
{
  public:
    TDIBFrame(unsigned long id, IByteArray& bitmap, long width, long height, long planes);
    TDIBFrame();
    
    //Methods of IDIBFrame
    virtual unsigned long MQUALIFIER getID() const;
    virtual long MQUALIFIER getWidth() const;
    virtual long MQUALIFIER getHeight() const;
    virtual long MQUALIFIER getPlaneCount() const;
    virtual const IByteArray& MQUALIFIER getData() const;
        
    //Methods of ISerializable
    SERIAL_START(TDIBFrame, L"TDIBFrame")
      SERIAL_LONG(m_ulID)
      SERIAL_LONG(m_lWidth)
      SERIAL_LONG(m_lHeight)
      SERIAL_LONG(m_lPlaneCount)
      SERIAL_IBYTEARRAY_PTR(m_pData)
    SERIAL_END

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  protected:
    virtual ~TDIBFrame();
    
  private:
    ///ID of the frame
    unsigned long m_ulID;
    ///Width of the Picture
    long m_lWidth;
    ///Height of the Picture
    long m_lHeight;
    ///Caount of Frames
    long m_lPlaneCount;
    ///Array of picture data
    IByteArray* m_pData;
    ///Lock
    TCriticalSection* m_pLock;

};


END_NAMESPACE_Zeus


#endif  
