/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : DifferenceCell                                                *
 * Package : MotionDetectModule                                            *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:22:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: DifferenceCell.cpp,v $
 *   Revision 1.2  2005/12/14 09:22:56  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <examples/motiondetect/MotionDetectModule/DifferenceCell.h> 
#include <examples/motiondetect/MotionDetectModule/DIBFrame.h>
#include <examples/motiondetect/ModecDefines.hpp>
#include <zeusbase/Messaging/Interfaces/IXMLMessage.hpp>
#include <zeusbase/Messaging/ObjectMessage.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <math.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! creates a simple work cell
*/
/***************************************************************************/
TDifferenceCell::TDifferenceCell(IXMLNode* node) : TAbstractCell(node)
{
  m_pLastFrame = NULL;
  m_lSleepTime = 1000;
}

/***************************************************************************/
/*! destroys the simple work cell
*/
/***************************************************************************/
TDifferenceCell::~TDifferenceCell()
{}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TDifferenceCell::onInitialLife()
{}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TDifferenceCell::onEndLife()
{
  if (m_pLastFrame != NULL)
  {
    m_pLastFrame->release();
    m_pLastFrame = NULL;
  }
}

/***************************************************************************/
/*! Processing of xml messages
*/
/***************************************************************************/
void TDifferenceCell::performXMLMessage(IXMLMessage* msg, unsigned long long timer)
{
  if (m_pCommunication != NULL && msg != NULL)
  {
    TString strName;
    msg->getMessageName(&strName);
    if (strName == L"Reset")
    {
      ZLOGGER.println(LOGMODE_INFO, "Difference cell reset.");
      if (m_pLastFrame != NULL)
      {
        m_pLastFrame->release();
        m_pLastFrame = NULL;
      }
/*      TString strCellName = L"ModecOutput";
      TString strPipeName = PIPENAME_STDIN;
      msg->setTargetCellName(&strCellName);
      msg->setTargetPipeName(&strPipeName);
      m_pCommunication->sendMessage(msg, 0);*/
    }
  }
}

/***************************************************************************/
/*! Processing of binary messages
*/
/***************************************************************************/
void TDifferenceCell::performObjectMessage(IObjectMessage* msg, unsigned long long timer)
{
  if (m_pCommunication != NULL && msg != NULL)
  {
    IDIBFrame* pCurrentFrame = NULL;
    if (m_pLastFrame == NULL)
    {
      msg->getObject((ISerializable**)&m_pLastFrame);
    }
    else if (msg->getObject((ISerializable**)&pCurrentFrame) == RET_NOERROR)
    {
      long lSize = pCurrentFrame->getWidth()*pCurrentFrame->getHeight()*pCurrentFrame->getPlaneCount();
      unsigned char* pBuffer = new unsigned char[lSize];
      memset(pBuffer, 0, lSize);
      unsigned char* pCurrentData = (unsigned char*)pCurrentFrame->getData().getArray();
      unsigned char* pLastData = (unsigned char*)m_pLastFrame->getData().getArray();
      for(int i = 0; i < lSize; i++)
      {
        //pLastData[i] = (unsigned char)(0.1*pCurrentData[i] + 0.9*pLastData[i]);
        pBuffer[i] = ::abs(pCurrentData[i] - pLastData[i]);
        if (pBuffer[i] < 50)
        {
          pBuffer[i] = 0;
        }
        else
        {
          pBuffer[i] = (unsigned char)(1.0/(double)pBuffer[i] + 255.0);
        }
      }
      
      TByteArray* pDiffData = new TByteArray((char*)pBuffer, lSize);
      TDIBFrame* pDiffFrame = new TDIBFrame(pCurrentFrame->getID(), 
                                            *pDiffData, 
                                            pCurrentFrame->getWidth(), 
                                            pCurrentFrame->getHeight(), 
                                            pCurrentFrame->getPlaneCount());
      pDiffData->release();
      TObjectMessage* pMessage = NULL;
      if (TObjectMessage::createMessage(L"DiffImage", L"ModecOutput", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
      {
        pMessage->setDataType(PICTURE_MOTION);
        pMessage->setObject(pDiffFrame);
        m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
        pMessage->release();
      }
      pDiffFrame->release();
      pCurrentFrame->release();
    }
  }
}
 
