/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : ModecInputCell                                                *
 * Package : MotionDetectModule                                            *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/15 07:11:15 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ModecInputCell.cpp,v $
 *   Revision 1.3  2005/12/15 07:11:15  bhadorn
 *   Do copy array instead of using shared arrays
 *
 *   Revision 1.2  2005/12/14 09:22:56  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <examples/motiondetect/MotionDetectModule/ModecInputCell.h> 
#include <examples/motiondetect/MotionDetectModule/DIBBlock.h> 
#include <examples/motiondetect/MotionDetectModule/DIBFragment.h> 
#include <examples/motiondetect/MotionDetectModule/DIBFrame.h>
#include <examples/motiondetect/ModecDefines.hpp> 
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/ServerSocket.h>
#include <zeusbase/System/File.h>
#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/Messaging/ObjectMessage.h>
#include <stdio.h>
USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates an input cell
*/
/***************************************************************************/
TModecInputCell::TModecInputCell(IXMLNode* node) : TAbstractCell(node)
{
  m_lSleepTime = 2000;
  m_lFrameCount = 0;
  m_pAVIFile = NULL;
  m_pVideoStream = NULL;
  m_bStarted = false;
  m_bEndReached = false;
  m_bVectorMode = false;
  m_pLastFrame = NULL;
  m_lBlockSize = 15;
  m_lFragmentBlockCount = 4;
  m_lMaxMovment = 16;  
  
  this->readAttribute(L"BlockSize", m_lBlockSize);
  this->readAttribute(L"FragmentBlockCount", m_lFragmentBlockCount);
  this->readAttribute(L"MaxMovment", m_lMaxMovment);
  ZLOGGER.printfln(LOGMODE_INFO,"BlockSize = %d", m_lBlockSize);
  ZLOGGER.printfln(LOGMODE_INFO,"FragmentBlockCount = %d", m_lFragmentBlockCount);
  ZLOGGER.printfln(LOGMODE_INFO,"MaxMovment = %d", m_lMaxMovment);
}

/***************************************************************************/
/*! Destroys the input cell
*/
/***************************************************************************/
TModecInputCell::~TModecInputCell()
{
}

/***************************************************************************/
/*! Flush all frames
*/
/***************************************************************************/
void TModecInputCell::flushFrames()
{
  if (m_pLastFrame != NULL)
  {
    m_pLastFrame->release();
    m_pLastFrame = NULL;
  }
  m_bStarted = false;
  m_lFrameCount = 0;
  if (m_pAVIFile != NULL)
  {
    m_pVideoStream = NULL;
    delete m_pAVIFile;
    m_pAVIFile = NULL;
  }
}


/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TModecInputCell::onInitialLife()
{
}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TModecInputCell::onEndLife()
{
  flushFrames();
}


/***************************************************************************/
/*! Stream the movie
    See ICellKernel
*/
/***************************************************************************/
void TModecInputCell::sendLifeSignal()
{
  TAbstractCell::sendLifeSignal();
  if (m_bStarted && m_pVideoStream != NULL)
  {
    if (!m_pVideoStream->Eof())
    {
      m_lFrameCount++;
      avm::CImage* pImage = m_pVideoStream->GetFrame(true);
      if (pImage != NULL)
      {
        long lBPP = pImage->Bpp();
        long lHeight = pImage->Height();
        long lWidth = pImage->Width();
        TByteArray* pData = new TByteArray();
        pData->setArray((char*)pImage->Data(), lWidth*lHeight*lBPP);
        //Stream data
        TDIBFrame* pFrame = new TDIBFrame(m_lFrameCount, *pData, lWidth, lHeight, lBPP);
        pData->release();
        
        //Send to get vectorized image
        if (m_bVectorMode)
        {
          sendFragments(pFrame);
        }
        //Send to get difference
        else
        {
          sendDifferenceEval(pFrame);
        }
        
        //Send message for GUI Gateway        
        TObjectMessage* pMessage = NULL;
        if (TObjectMessage::createMessage(L"DIBFrame", L"ModecOutput", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
        {
          pMessage->setObject(pFrame);
          pMessage->setDataType(PICTURE_ORIGINAL);
          pMessage->setNumber(m_lFrameCount);
          m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
          pMessage->release();
        }
        pFrame->release();
        pImage->Release();
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Movie stream has ended.");
        m_bStarted = false;
        m_bEndReached = true;
      }
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Movie stream is empty");
      m_bStarted = false;
      m_bEndReached = true;
    }
  }
  else
  {
    m_bStarted = false;
  }
}


/***************************************************************************/
/*! Processing of xml messages
*/
/***************************************************************************/
void TModecInputCell::performXMLMessage(IXMLMessage* msg, unsigned long long timer)
{
  TString strName;
  msg->getMessageName(&strName);
  
  //Load a new movie
  if (strName == L"LoadMovie")
  {
    ZLOGGER.printfln(LOGMODE_INFO,"LoadMovie message received.");
    IXMLNode* pNode = NULL;
    if (msg->getDataNode(&pNode) == RET_NOERROR)
    {
      TString strFileName;
      TString strAttName = L"FileName";
      pNode->getAttributeValue(&strAttName, &strFileName);
      loadMovieFile(strFileName);
      pNode->release();
    }
  }
  
  //change motion detection mode (0=difference; 1 = vectorizing)
  else if (strName == L"ChangeMode")
  {
    ZLOGGER.printfln(LOGMODE_INFO,"ChangeMode message received.");
    IXMLNode* pNode = NULL;
    if (msg->getDataNode(&pNode) == RET_NOERROR)
    {
      TString strMode;
      TString strAttName = L"Mode";
      pNode->getAttributeValue(&strAttName, &strMode);
      m_bVectorMode = strMode.toLong();
      pNode->release();
    }  
  }
  
  // start sampling the movie
  else if (strName == L"StartMovie")
  {
    ZLOGGER.printfln(LOGMODE_INFO,"StartMovie message received.");
    //start streaming
    if (m_pVideoStream != NULL)
    {
      if (m_bEndReached)
      {
        loadMovieFile(m_strFileName);
      }
      if (m_pVideoStream != NULL)
      {
        m_bStarted = true;
        m_pVideoStream->StartStreaming();
      }
    }
  }
  
  //stop sampling the movie
  else if (strName == L"StopMovie")
  {
    ZLOGGER.printfln(LOGMODE_INFO,"StopMovie message received.");
    m_bStarted = false;
    if (m_pVideoStream != NULL)
    {
      m_pVideoStream->StopStreaming();
    }
  }
  
  //Unknown request
  else
  {
    ZLOGGER.printfln(LOGMODE_INFO,"Unknown message received.");
  }
}

/***************************************************************************/
/*! Loading the movie file
    \param filename : Name of the file to be loaded. 
    \retval RET_NOERROR : Movie loaded
    \retval RET_REQUEST_FAILED : Could not load movie
*/
/***************************************************************************/
unsigned long TModecInputCell::loadMovieFile(TString filename)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  flushFrames();
  
  ZLOGGER.println(LOGMODE_INFO, "Try to load file %s...",filename.c_str()); 
  ZLOGGER.indent();
  TFile ImageFile(filename);
  
  if (ImageFile.exists())
  {
    m_strFileName = filename;
    m_pAVIFile = avm::CreateReadFile(filename.c_str());
    
    if (!m_pAVIFile->IsValid())
    {
      ZLOGGER.println(LOGMODE_INFO, "Movie is not valid."); 
    }
    else
    {  
      ZLOGGER.println(LOGMODE_INFO, "Movie is valid."); 
      
      long lCount = m_pAVIFile->StreamCount();
      ZLOGGER.println(LOGMODE_INFO, "%d Streams found.", (int)lCount);
      bool bVideoFound = false; 
      for(int i = 0; i < lCount && !bVideoFound; i++)
      {
        m_pVideoStream = m_pAVIFile->GetStream(i, avm::IStream::Video);
        if (m_pVideoStream != NULL)
        {
          ZLOGGER.println(LOGMODE_INFO, "Stream received."); 
          
          resetCellSystem(m_pVideoStream->GetLength());
          
          bVideoFound = true;
        }
      }
      
      //Sampling the movie
      if (!bVideoFound)
      {
        m_pVideoStream = NULL;
      }
      else
      {
        m_bEndReached = false;
        ulRetval = RET_NOERROR;
      }
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! This method is used to reset the whole cell system
*/
/***************************************************************************/
void TModecInputCell::resetCellSystem(long frame_count)
{
  //Send message to reset the difference cell
  IXMLMessage* pMessage = NULL;
  if (TXMLMessage::createMessage(L"Reset", L"Difference", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
  {
    pMessage->setNumber(frame_count);
    m_pCommunication->sendMessage(pMessage, 0);
    pMessage->release();
  }
  
  //Send message to reset the output cell
  pMessage = NULL;
  if (TXMLMessage::createMessage(L"Reset", L"ModecOutput", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
  {
    pMessage->setNumber(frame_count);
    m_pCommunication->sendMessage(pMessage, 0);
    pMessage->release();
  }
  
  //Send message for GUI Gateway about sample count
  pMessage = NULL;
  if (TXMLMessage::createMessage(L"SampleCount", L"GUIGATEWAY", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
  {
    pMessage->setNumber(frame_count);
    m_pCommunication->sendMessage(pMessage, 0);
    pMessage->release();
  }
}


/***************************************************************************/
/*! Send fragments for block matching
    \param mainfarme : Main frame of sampling
*/
/***************************************************************************/
void TModecInputCell::sendFragments(TDIBFrame* mainframe)
{
  TArrayList<TDIBBlock*> lstBlocks;      
  TArrayList<TDIBFragment*> lstFragments;      
  long lBlockSize = m_lBlockSize;    
  long lBlockCount = m_lFragmentBlockCount;
  long lMaxMovment = m_lMaxMovment;
  
  if (m_pLastFrame != NULL && mainframe != NULL && 
      this->createBlocks(mainframe, lBlockSize,lBlockCount,lMaxMovment, lstBlocks) == RET_NOERROR)
  {
    long lCount = lstBlocks.getCount();
    long lFragmentIndex = 0;
    for(int i = 0;i < lCount;)
    {
      TDIBFragment* pFragment = new TDIBFragment(*mainframe, *m_pLastFrame, lFragmentIndex);
      long lBlockIndex = 0;
      while(i < lCount && lBlockIndex < lBlockCount)
      {
        pFragment->addBlock(lstBlocks[i]);
        lstBlocks[i]->release();
        i++;
        lBlockIndex++;
      }
      
      lFragmentIndex++;
      lstFragments.add(pFragment);
    }
    
    lCount = lstFragments.getCount();  
    for(int i = 0; i < lCount; i++ )
    {
      lstFragments[i]->setTotalFragment(lCount);
      
      //Send message for Blockmatching Cell
      TObjectMessage* pMessage = NULL;
      if (TObjectMessage::createMessage(L"DIBFrame", L"BlockMatch", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
      {
        pMessage->setObject(lstFragments[i]);
        pMessage->setDataType(PICTURE_FRAGMENT);
        pMessage->setNumber(i);
        m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
        pMessage->release();
      }
      lstFragments[i]->release();
    }
  }
  
  //Keep last frame
  if (m_pLastFrame != NULL)
  {
    m_pLastFrame->release();
  }
  m_pLastFrame = mainframe;
  if (m_pLastFrame != NULL)
  {
    m_pLastFrame->addRef();
  }  
}

/**************************************************************************/
/*! Creates fragments of the existing bitmap.
    \param block_size : Size of the block in pixels
    \param fragment_size : Block count of the fragment
    \param max_movment : maximum movement to be registered
    \param fragments : Return parameter of created fragements
    \retval RET_NOERROR : Fragment created
    \retval RET_INVALID_DATA : Invalid data received. Blocksize and maximum
             movment parameter to big
    \retval RET_REQUEST_FAILED : No valid dib available
*/
/**************************************************************************/
unsigned long TModecInputCell::createBlocks(TDIBFrame* mainframe, 
                                            long block_size, 
                                            long fragment_size, 
                                            long max_movment, 
                                            TArrayList<TDIBBlock*>& blocks)
{
  long lWidth = mainframe->getWidth();
  long lHeight = mainframe->getHeight();
  long lPlaneCount = mainframe->getPlaneCount();
  
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (lWidth > 0 && lHeight > 0 && lPlaneCount > 0)
  {
    long lBasePosY = max_movment + block_size/2;
    while(lBasePosY + max_movment + block_size/2 <= lHeight)
    {
      long lBasePosX = max_movment + block_size/2;
      while(lBasePosX + max_movment + block_size/2 <= lWidth)
      {
        TDIBBlock* pBlock = new TDIBBlock(block_size, max_movment, lBasePosX, lBasePosY);
        
        blocks.add(pBlock);
        lBasePosX += block_size;
      }
      lBasePosY += block_size;
    }
    
    //Check if block count is more than 0
    if (blocks.getCount() > 0)
    {
      ZLOGGER.println(LOGMODE_INFO, "%d blocks created with max move %d", blocks.getCount(), max_movment); 
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_INVALID_DATA;
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! Send message to get the moving difference.
    \param mainframe : Frame for difference
*/
/***************************************************************************/
void TModecInputCell::sendDifferenceEval(TDIBFrame* mainframe)
{
  TObjectMessage* pMessage = NULL;
  if (TObjectMessage::createMessage(L"DIBFrame", L"Difference", PIPENAME_STDIN, &pMessage) == RET_NOERROR)
  {
    pMessage->setObject(mainframe);
    pMessage->setDataType(PICTURE_MOTION);
    pMessage->setNumber(m_lFrameCount);
    m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
    pMessage->release();
  }
}

