/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : ModecInputCell                                                *
 * Package : MotionDetectModule                                            *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:22:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ModecInputCell.h,v $
 *   Revision 1.2  2005/12/14 09:22:56  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ModecInputCellH
#define ModecInputCellH

#include <zeusdev/AbstractCell.h>
#include <zeusbase/System/ArrayList.hpp>
#include <zeusbase/System/XObjectFactory.h>
#include <avifile-0.7/avifile.h>

BEGIN_NAMESPACE_Zeus

class TDIBFrame;
class TDIBBlock;

/***************************************************************************/
/*! This cells fragments a incomming stream of data. There are two modes
    for processing the data. First mode is the difference mode. The movie
    frames will be sent to the difference cell. The second mode is the 
    vectorizing mode. All frames are fragmented into blocks and sent to 
    the block matching cell. 
*/
/***************************************************************************/
class TModecInputCell : public TAbstractCell
{
  public:
    TModecInputCell(IXMLNode* node);
    
    REG_SUB_BEGIN(TModecInputCell)
    REG_SUB_ADD(TModecInputCell, L"TModecInputCell");
    REG_SUB_END
    
    virtual void sendLifeSignal();

  protected:    
    virtual ~TModecInputCell();
    virtual void performXMLMessage(IXMLMessage* msg, unsigned long long timer);
    
  private:
    //Size of the pixel blocks
    long m_lBlockSize;    //32
    ///Count of blocks inside a fragment
    long m_lFragmentBlockCount;
    ///Maximum movments
    long m_lMaxMovment;
    ///Last frame sent 
    TDIBFrame* m_pLastFrame; 
    ///File for AVI
    avm::IReadFile* m_pAVIFile;
    ///Video stream of avi file
    avm::IReadStream* m_pVideoStream;
    ///total frames to send
    long m_lFrameCount;
    ///started sending frames
    bool m_bStarted;
    ///End of movie reached
    bool m_bEndReached;
    ///Filename of the avi
    TString m_strFileName;
    ///send for vectorizing
    bool m_bVectorMode;
  
    virtual void onInitialLife();
    virtual void onEndLife();
    
    void resetCellSystem(long frame_count);
    void flushFrames();
    unsigned long loadMovieFile(TString filename);
    void sendFragments(TDIBFrame* mainframe);
    void sendDifferenceEval(TDIBFrame* mainframe);
    unsigned long createBlocks(TDIBFrame* mainframe, long block_size, long fragment_size, long max_movment, TArrayList<TDIBBlock*>& blocks);

};

END_NAMESPACE_Zeus

#endif 
 
