/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : ModecOutputCell                                               *
 * Package : MotionDetectModule                                            *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:22:56 $                                  *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ModecOutputCell.cpp,v $
 *   Revision 1.2  2005/12/14 09:22:56  bhadorn
 *   Redesing of the motion detection module
 *
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <examples/motiondetect/MotionDetectModule/ModecOutputCell.h> 
#include <examples/motiondetect/ModecDefines.hpp>
#include <examples/motiondetect/Interfaces/IDIBFrame.hpp>
#include <examples/motiondetect/Interfaces/IDIBFragment.hpp>
#include <zeusbase/Messaging/ObjectMessage.h>
#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! creates an output cell
*/
/***************************************************************************/
TModecOutputCell::TModecOutputCell(IXMLNode* node) : TAbstractCell(node), m_lstVectorBuffer(200)
{
  m_lSleepTime = 1000;
  m_ulNextFrameID = 2;
}

/***************************************************************************/
/*! destroys the output cell
*/
/***************************************************************************/
TModecOutputCell::~TModecOutputCell()
{
  flushBuffer();
}

/***************************************************************************/
/*! flush the buffer lists
*/
/***************************************************************************/
void TModecOutputCell::flushBuffer()
{
  while(!m_lstMotionBuffer.isEmpty())
  {
    m_lstMotionBuffer[0]->release();
    m_lstMotionBuffer.deleteItem(0);
  }
  while(!m_lstOrigBuffer.isEmpty())
  {
    m_lstOrigBuffer[0]->release();
    m_lstOrigBuffer.deleteItem(0);
  }
  while(!m_lstVectorBuffer.isEmpty())
  {
    m_lstVectorBuffer[0]->release();
    m_lstVectorBuffer.deleteItem(0);
  }
  
}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TModecOutputCell::onInitialLife()
{
}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TModecOutputCell::onEndLife()
{
  flushBuffer();
}

/***************************************************************************/
/*! Processing of xml messages
*/
/***************************************************************************/
void TModecOutputCell::performXMLMessage(IXMLMessage* msg, unsigned long long timer)
{
  if (m_pCommunication != NULL && msg != NULL)
  {
    TString strName;
    msg->getMessageName(&strName);
    if (strName == L"Reset")
    {
      ZLOGGER.println(LOGMODE_INFO, "Output cell reset.");
      flushBuffer();
      m_ulNextFrameID = 2;
    }
  }
}

/***************************************************************************/
/*! Processing of stream messages
*/
/***************************************************************************/
void TModecOutputCell::performObjectMessage(IObjectMessage* msg, unsigned long long timer)
{
  if (m_pCommunication != NULL && msg != NULL)
  {
    switch(msg->getDataType())
    {
      case PICTURE_FRAGMENT :
      {
        //ZLOGGER.println(LOGMODE_INFO, "Receive Fragment");
        IDIBFragment* pFragment = NULL;
        if (msg->getObject((ISerializable**)&pFragment) == RET_NOERROR)
        {
          unsigned long ulThisFrameID = pFragment->getFrameID();
          long lIndex = findFragment(ulThisFrameID);
          long lFramesToReceive = 0;
          if (lIndex == -1)
          {
            lFramesToReceive = pFragment->getTotalFragment()-1;
            pFragment->setTotalFragment(lFramesToReceive); //One frame already received;
            m_lstVectorBuffer.add(pFragment);
            lIndex = 0;
            //ZLOGGER.println(LOGMODE_INFO, "Fragment added. Still %d fragments to receive", lFramesToReceive);
          }
          else
          {
            lFramesToReceive = addBlocksToFragment(pFragment, m_lstVectorBuffer[lIndex]);
            //ZLOGGER.println(LOGMODE_INFO, "Blocks added. Still %d fragments to receive", lFramesToReceive);
            pFragment->release();
          }
          
          //All fragments received -> send fragment to gui gateway 
          // and remove it from the list
          if(lFramesToReceive == 0)
          {
            sendVectorFragment();
          }
        }
      }
      break;
      
      case PICTURE_ORIGINAL:
      {
        IDIBFrame* pFrame = NULL;
        if (msg->getObject((ISerializable**)&pFrame) == RET_NOERROR)
        {
          long lIndex = findFrame(pFrame->getID(), m_lstMotionBuffer);
          if (lIndex == -1)
          {
            m_lstOrigBuffer.add(pFrame);
          }
          else
          {
            IDIBFrame* pMotionFrame = m_lstMotionBuffer[lIndex];
            sendFrames(pFrame, pMotionFrame);
            m_lstMotionBuffer.deleteItem(lIndex);
            pFrame->release();
            pMotionFrame->release();
            ZLOGGER.println(LOGMODE_INFO, "Original sent to GUI");
          }
        }
      }  
      break;
      
      case PICTURE_MOTION : //fall through
      default:  
      {
        IDIBFrame* pFrame = NULL;
        if (msg->getObject((ISerializable**)&pFrame) == RET_NOERROR)
        {
          long lIndex = findFrame(pFrame->getID(), m_lstOrigBuffer);
          if (lIndex == -1)
          {
            m_lstMotionBuffer.add(pFrame);
          }
          else
          {
            IDIBFrame* pOrigFrame = m_lstOrigBuffer[lIndex];
            sendFrames(pOrigFrame, pFrame);
            m_lstOrigBuffer.deleteItem(lIndex);
            pFrame->release();
            pOrigFrame->release();
            ZLOGGER.println(LOGMODE_INFO, "Motion sent to GUI");
          }
          /*sendFrames(NULL, pFrame);
          pFrame->release();*/
        }
      }
      break;  
    }
  }
}

/***************************************************************************/
/*! Returns the index of a fragment already buffered. The id represents the 
    FrameID
    \retval >=0 : Index
    \retval -1 : Error
*/
/***************************************************************************/
long TModecOutputCell::findFragment(unsigned long id)
{
  long lRetval = 0;
  bool bFound = false;
  IListIterator<IDIBFragment*>* pIt = m_lstVectorBuffer.getIterator();
  while(!bFound && pIt->hasNextItem())
  {
    if (pIt->getNextItem()->getFrameID() == id)
    {
      bFound = true;
    }
    else
    {
      lRetval++;
    }
  } 
  m_lstVectorBuffer.releaseIterator(pIt);
  
  if (!bFound)
  {
    lRetval = -1;
  }
  return lRetval;  
}

/***************************************************************************/
/*! Adds the blocks of fragment src to the fragment dest. It will return 
    the total fragments to receive 
*/
/***************************************************************************/
long TModecOutputCell::addBlocksToFragment(IDIBFragment* src, IDIBFragment* dest)
{ 
  long lCount = src->getBlockCount();
  for(int i = 0; i < lCount; i++)
  {
    IDIBBlock* pBlock = NULL;
    if (src->getBlock(i, &pBlock) == RET_NOERROR)
    {
      dest->addBlock(pBlock);
      pBlock->release();
    }
  }
  
  long lFragToReceive = dest->getTotalFragment()-1;
  dest->setTotalFragment(lFragToReceive);
  return lFragToReceive;
}

long TModecOutputCell::findFrame(unsigned long id, TSingleLinkedList<IDIBFrame*>& list)
{
  long lRetval = 0;
  bool bFound = false;
  IListIterator<IDIBFrame*>* pIt = list.getIterator();
  while(!bFound && pIt->hasNextItem())
  {
    if (pIt->getNextItem()->getID() == id)
    {
      bFound = true;
    }
    else
    {
      lRetval++;
    }
  } 
  list.releaseIterator(pIt);
  
  if (!bFound)
  {
    lRetval = -1;
  }
  return lRetval;
}
    

   
void TModecOutputCell::sendFrames(IDIBFrame* orig, IDIBFrame* motion)
{
  if (motion != NULL)
  {
    TObjectMessage* pMessage = NULL;
    if (TObjectMessage::createMessage(L"DIBFrame", L"GUIGATEWAY", PIPENAME_STDIN, &pMessage)== RET_NOERROR)
    {
      pMessage->setDataType(PICTURE_MOTION);
      pMessage->setObject(motion);
      pMessage->setNumber(motion->getID());
      m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
      pMessage->release();
    }
  }
  if (orig != NULL)
  {
    TObjectMessage* pMessage = NULL;
    if (TObjectMessage::createMessage(L"DIBFrame", L"GUIGATEWAY", PIPENAME_STDIN, &pMessage)== RET_NOERROR)
    {
      pMessage->setDataType(PICTURE_ORIGINAL);
      pMessage->setObject(orig);
      pMessage->setNumber(orig->getID());
      m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
      pMessage->release();
    }
  }
}

/***************************************************************************/
/*! Sends a vectorfragment to the GUI gateway and removes it from the buffer
    list
*/
/***************************************************************************/
void TModecOutputCell::sendVectorFragment()
{
  bool bSent = true;
  while(bSent)
  {
    bSent = false;
    for(int i = 0; i < m_lstVectorBuffer.getCount(); i++)
    {
      IDIBFragment* pFragment = m_lstVectorBuffer[i];
      
      unsigned long ulID = pFragment->getFrameID();
      if (pFragment->getTotalFragment() == 0 && m_ulNextFrameID == ulID)
      {
        long lIndex = findFrame(ulID, m_lstOrigBuffer);
        if (lIndex > -1)
        {
          IDIBFrame* pFrame = m_lstOrigBuffer[lIndex];
          TObjectMessage* pMessage = NULL;
          if (TObjectMessage::createMessage(L"DIBFrame", L"GUIGATEWAY", PIPENAME_STDIN, &pMessage)== RET_NOERROR)
          {
            pMessage->setDataType(PICTURE_ORIGINAL);
            pMessage->setObject(pFrame);
            pMessage->setNumber(ulID);
            m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
            pMessage->release();
          }
          m_lstOrigBuffer.deleteItem(lIndex);
          pFrame->release();
        }
        
        TObjectMessage* pMessage = NULL;
        if (TObjectMessage::createMessage(L"DIBFrame", L"GUIGATEWAY", PIPENAME_STDIN, &pMessage)== RET_NOERROR)
        {
          pMessage->setDataType(PICTURE_VECTOR);
          pMessage->setObject(m_lstVectorBuffer[i]);
          pMessage->setNumber(ulID);
          m_pCommunication->sendMessage((IObjectMessage*)pMessage, 0);
          pMessage->release();
          bSent = true;
        }  
        m_lstVectorBuffer[i]->release();
        m_lstVectorBuffer.deleteItem(i);
        i--;
        m_ulNextFrameID++;
      }
    }
  }  
}
 
