/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : ModecOutputCell                                               *
 * Package : MotionDetectModule                                            *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:49 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ModecOutputCell.h,v $
 *   Revision 1.1  2005/10/28 12:11:49  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef ModecOutputCellH
#define ModecOutputCellH

#include <zeusdev/AbstractCell.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/ArrayList.hpp>

BEGIN_NAMESPACE_Zeus

class IDIBFrame;
class IDIBFragment;

/***************************************************************************/
/*! This class defines an output cell. Its responsible for assembling and
    synchronize the frames of block matching and original movie. Also
    it must garantie that the order of the frames are correct. Therefore
    it uses buffers to store all incomming products.
*/
/***************************************************************************/
class TModecOutputCell : public TAbstractCell
{
  public:
    TModecOutputCell(IXMLNode* node);
    
    REG_SUB_BEGIN(TModecOutputCell)
    REG_SUB_ADD(TModecOutputCell, L"TModecOutputCell");
    REG_SUB_END

  protected:
    virtual ~TModecOutputCell();
    virtual void performObjectMessage(IObjectMessage* msg, unsigned long long timer);
    virtual void performXMLMessage(IXMLMessage* msg, unsigned long long timer);
    
    virtual void onInitialLife();
    virtual void onEndLife();
    
  private:
    ///Buffer list of original movie stream
    TSingleLinkedList<IDIBFrame*> m_lstOrigBuffer;
    ///Buffer list of motion movie stream
    TSingleLinkedList<IDIBFrame*> m_lstMotionBuffer;
    /// Buffer of Vector blocks
    TArrayList<IDIBFragment*> m_lstVectorBuffer;
    
    ///next frame index sent
    unsigned long m_ulNextFrameID;
    
    void flushBuffer();
    long findFragment(unsigned long id);
    long addBlocksToFragment(IDIBFragment* src, IDIBFragment* dest);

    long findFrame(unsigned long id, TSingleLinkedList<IDIBFrame*>& list);
    long findOrigFrame(unsigned long id);
    void sendFrames(IDIBFrame* orig, IDIBFrame* motion);
    void sendVectorFragment();
};

END_NAMESPACE_Zeus

#endif 
 
