/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : MovieDisplay                                                  *
 * Package : motiondetect                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: MovieDisplay.cpp,v $
 *   Revision 1.1  2005/10/28 12:11:50  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <qpainter.h> 
#include <zeusbase/System/LoggerManager.h>
#include <examples/motiondetect/MovieDisplay.h>    
    
USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new movie display
*/
/***************************************************************************/
TMovieDisplay::TMovieDisplay(QWidget* parent, const char* name) : QWidget(parent, name)
{
  m_pImage = new QImage(0,0,32);
  m_Points.putPoints(0,1,0,0);
  m_FramePoints.putPoints(0,1,0,0);
  

  m_FramePen.setWidth(1);
  m_FramePen.setColor(Qt::darkGreen);
  m_Pen.setWidth(1);
  m_Pen.setColor(Qt::red);
  
  setUpdatesEnabled(true);
  m_lWidth = 0;
  m_lHeight = 0;
}

/***************************************************************************/
/*! Destrys the movie display
*/
/***************************************************************************/
TMovieDisplay::~TMovieDisplay()
{
  delete m_pImage;
}

/***************************************************************************/
/*! sets a new frame of binary data (image)
    \param frame : Image data container
*/
/***************************************************************************/
void TMovieDisplay::setNewFrame(NAMESPACE_Zeus::IDIBFrame* frame)
{
  delete m_pImage;
  long lWidth = frame->getWidth();
  long lHeight = frame->getHeight();
  long lPlanes = frame->getPlaneCount();
  char* aData = frame->getData().getArray();
  
  
  m_pImage = new QImage(lWidth, lHeight, 32);
  
  //long lCount = width*height;
  m_lWidth = lWidth;
  m_lHeight = lHeight;

//  m_lstPoints.setArray(data, lCount);
  if (m_lWidth > 0 && m_lHeight > 0)
  {
    for(int i = 0; i < m_lHeight; i++)
    {
      for(int j = 0; j < m_lWidth; j++)
      {
        long lBaseIndex = j*lPlanes + i*m_lWidth*lPlanes;
        if (lPlanes == 3)
        {
           lBaseIndex += 2;
           m_pImage->setPixel(j,i, QColor((unsigned char)aData[lBaseIndex--],
                                          (unsigned char)aData[lBaseIndex--],
                                          (unsigned char)aData[lBaseIndex]).rgb());
        }
        else if (lPlanes == 1)
        {
          unsigned int uiData = (unsigned char)aData[lBaseIndex];
          m_pImage->setPixel(j,i, QColor(uiData,uiData,uiData).rgb());
        }
      }
    }
  }
  adjustSize();
  repaint(false);  
}

/***************************************************************************/
/*! Show the vector fragment using its block movments.
    \param fragment : Vector and image data container
*/
/***************************************************************************/
void TMovieDisplay::setVectorFragment(NAMESPACE_Zeus::IDIBFragment* fragment)
{
  m_Points.resize(1);
  m_FramePoints.resize(1);
  
  
  if (fragment != NULL)
  {
    IDIBFrame* pFrame = NULL;
    if (fragment->getCurrentFrame(&pFrame) == RET_NOERROR)
    {
      setNewFrame(pFrame);
      pFrame->release();
    }
    
    
    long lCount = fragment->getBlockCount();
    for(int i = 0; i < lCount; i++)
    {
      IDIBBlock* pBlock = NULL;
      if (fragment->getBlock(i,&pBlock) == RET_NOERROR)
      {
//  ZLOGGER.println(LOGMODE_INFO, "---------------------Vector draw");
        m_Points.putPoints(i*2, 1, pBlock->getBasePosX(), pBlock->getBasePosY());
        m_Points.putPoints(i*2+1, 1, pBlock->getBasePosX()+pBlock->getMoveX(), pBlock->getBasePosY()+pBlock->getMoveY());
        
        m_FramePoints.putPoints(i*2, 2, pBlock->getBasePosX()-pBlock->getSize()/2, pBlock->getBasePosY()-pBlock->getSize()/2,
                                   pBlock->getBasePosX()+pBlock->getSize()/2, pBlock->getBasePosY()+pBlock->getSize()/2);

        pBlock->release();
      }
    }

  }
  adjustSize();
  repaint(false);  
}


/***************************************************************************/
/*! Paint event
*/
/***************************************************************************/
void TMovieDisplay::paintEvent(QPaintEvent* /*event*/)
{
  QPainter painter(this);
  painter.save();
  QPoint Point(0,0);
  
  painter.drawImage(Point, *m_pImage);
    
  long lCount = m_Points.size();
  if (lCount != 0)
  {
    painter.setPen(m_FramePen);
    long lCount = m_FramePoints.size();
    for(int i = 0; i < lCount/2; i++)
    {
      painter.drawRect(QRect(m_FramePoints.point(i*2), m_FramePoints.point(i*2+1))); 
    }
    
    painter.setPen(m_Pen);
    painter.drawLineSegments(m_Points, 0, -1);
  }
  
  painter.restore();
}

/*****************************************************************************/
/*! Return the size of the image
*/
/*****************************************************************************/
QSize TMovieDisplay::sizeHint() const 
{
  QSize size(m_lWidth, m_lHeight);
  return size;
}

