/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : CCM                                                           *
 * Module  : MovieDisplay                                                  *
 * Package : motiondetect                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:11:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: MovieDisplay.h,v $
 *   Revision 1.1  2005/10/28 12:11:50  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MovieDisplayH
#define MovieDisplayH

#include <qwidget.h>
#include <qimage.h>
#include <qpen.h>
#include <qpointarray.h>
#include <zeusbase/System/ByteArray.hpp>
#include <examples/motiondetect/Interfaces/IDIBFrame.hpp>
#include <examples/motiondetect/Interfaces/IDIBFragment.hpp>


/***************************************************************************/
/*! This class draws the output image (stream of movie). There are two 
    formats supported. The DIB Frame witch contains the image and the more
    complex format DIBFragment witch contains also vectorized information
    abaout the block movement.
*/
/***************************************************************************/
class TMovieDisplay : public QWidget
{  
  public:
    
    TMovieDisplay(QWidget* parent, const char* name);
    virtual ~TMovieDisplay();
    
    void setNewFrame(NAMESPACE_Zeus::IDIBFrame* frame);
    void setVectorFragment(NAMESPACE_Zeus::IDIBFragment* fragment);
  
  protected:
    void paintEvent(QPaintEvent* event);
    
    ///Image witch contains the visual data 
    QImage* m_pImage;
    ///Pen to draw the vectors
    QPen m_Pen;
    ///Pen to draw the frame (blocks)
    QPen m_FramePen;
    ///Points of the vector (line end points)
    QPointArray m_Points;
    ///edges of the blocks
    QPointArray m_FramePoints;
    ///Image width
    long m_lWidth;
    ///Image height
    long m_lHeight;
  
  public slots:
    QSize sizeHint() const;
};

#endif 
