/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FrameLoader                                                   *
 * Package : zeus                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:21:25 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: QTFrameLoader.cpp,v $
 *   Revision 1.2  2005/12/14 09:21:25  bhadorn
 *   no message
 *
 *   Revision 1.1  2005/10/28 12:11:50  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <examples/motiondetect/QTFrameLoader.h> 
#include <examples/motiondetect/CommandDispatcher.h> 
#include <examples/motiondetect/GUI/MainForm.h> 
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/CellModel/Interfaces/ILocalPipeRegistry.hpp>
#include <qapplication.h>
#include <qplatinumstyle.h>


USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Erstellen des FrameLoaders
*/
/***************************************************************************/
TQTFrameLoader::TQTFrameLoader(int& argc, char** argv) : TAbstractFrameLoader(L"modec", argc, argv), m_Application(argc, argv)
{
  m_Application.setStyle(new QPlatinumStyle());  
  
  m_pMainForm = new TMainForm();
  QString strTitle = "Diplomarbeit: Motion Detection";
  m_pMainForm->setCaption(strTitle);
  m_pMainForm->unfreeze();
  
  m_Application.setMainWidget(m_pMainForm);
  m_pDispatcher = new TCommandDispatcher(m_Application);  
}


/***************************************************************************/
/*! Zerstoeren des FrameLoaders
*/
/***************************************************************************/
TQTFrameLoader::~TQTFrameLoader()
{
  m_pDispatcher->release();
}


/***************************************************************************/
/*! See TFrameLoader
*/
/***************************************************************************/
bool TQTFrameLoader::canStartup()
{
  return true;
}

/***************************************************************************/
/*! Starts the application and returns the error code
*/
/***************************************************************************/
long TQTFrameLoader::run()
{
  m_Application.connect(&m_Application, SIGNAL(lastWindowClosed()), &m_Application, SLOT(quit()));
  return m_Application.exec();
}

/***************************************************************************/
/*! This method starts the framework
*/
/***************************************************************************/
unsigned long TQTFrameLoader::startup()
{
  ZLOGGER.printfln(LOGMODE_INFO, "Start up Motion detection application...");
  ZLOGGER.indent();
  
  unsigned long ulRetval = TAbstractFrameLoader::startup();
  
  if (ulRetval == RET_NOERROR)
  {
    IXObject* pRoot = NULL;
    if (this->getRootObject(&pRoot) == RET_NOERROR)
    {
      //-----------------------------------------------------------------------
      // Try to get the local pipe registry of this process
      ZLOGGER.printfln(LOGMODE_INFO, "Get local pipe registry...");
      ZLOGGER.indent();
      IXObject* pRegObj = NULL;
      TString strPath = L"/LocalPipeRegistry";
      if (pRoot->getObject(&strPath, &pRegObj) == RET_NOERROR)
      {
        ILocalPipeRegistry* pRegistry = NULL;
        if (pRegObj->askForInterface(INTERFACE_ILocalPipeRegistry, (IZUnknown**)&pRegistry) == RET_NOERROR)
        {
          m_pDispatcher->setupCommunication(pRegistry);
          m_pDispatcher->start();
          
          ZLOGGER.unindent();
          ZLOGGER.printfln(LOGMODE_INFO, "...done.");
          pRegistry->release();
        }
        else
        {
          ZLOGGER.unindent();
          ZLOGGER.printfln(LOGMODE_INFO, "...failed.");
        }
        pRegObj->release();
      }
      else
      {
        ZLOGGER.unindent();
        ZLOGGER.printfln(LOGMODE_INFO, "...failed.");
      } 
      pRoot->release();
    }
    
    m_pMainForm->setCommandDispatcher(m_pDispatcher);
    m_pMainForm->show();
    
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "..done");
    
    ZLOGGER.printfln(LOGMODE_INFO, "Started up. Status is ready.");
  }
  else
  {
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "..failed");
  }
  return ulRetval;
}

/***************************************************************************/
/*! This method terminates the framework
*/
/***************************************************************************/
unsigned long TQTFrameLoader::shutdown()
{
  ZLOGGER.printfln(LOGMODE_INFO, "Shutdown Motion detection application...");
  ZLOGGER.indent();
  m_pMainForm->freeze();
  m_pDispatcher->releaseCommunication();
  
  unsigned long ulRetval = TAbstractFrameLoader::shutdown();
  
  ZLOGGER.unindent();
  ZLOGGER.printfln(LOGMODE_INFO, "..done");
  return ulRetval;
}
