/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ZeusLogger                                                    *
 * Package : LOG_Service                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:23:41 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZeusLogger.cpp,v $
 *   Revision 1.2  2005/11/25 09:23:41  bhadorn
 *   Eliminate warnings
 *
 *   Revision 1.1  2005/10/28 12:13:11  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <ZeusLogger.h> 
#include <log4cxx/basicconfigurator.h>
#include <log4cxx/propertyconfigurator.h>
#include <log4cxx/helpers/exception.h>


USING_NAMESPACE_Zeus 

using namespace log4cxx;
using namespace log4cxx::helpers;


/**************************************************************************/
/*! Creates a logger class
*/
/**************************************************************************/
TZeusLogger::TZeusLogger(TString name) : TZObject(name)
{
  if (name == L"")
  {
    m_Logger = Logger::getRootLogger();
  }
  else
  {
    m_Logger = Logger::getLogger(name.c_str());
  }
}

/**************************************************************************/
/*! Destroys the logger
*/
/**************************************************************************/
TZeusLogger::~TZeusLogger()
{}

/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
unsigned long MQUALIFIER TZeusLogger::getLogger(IString* name, ILogger** logger)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (logger == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *logger = new TZeusLogger(TString(name));
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
unsigned long MQUALIFIER TZeusLogger::printDebug(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_Logger == NULL)
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  else if (m_Logger->isDebugEnabled())
  {
    TString strData(message);
    m_Logger->debug(strData.c_str());
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
unsigned long MQUALIFIER TZeusLogger::printError(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_Logger == NULL)
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  else if (m_Logger->isErrorEnabled())
  {
    TString strData(message);
    m_Logger->error(strData.c_str());
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
unsigned long MQUALIFIER TZeusLogger::printFatal(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_Logger == NULL)
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  else if (m_Logger->isFatalEnabled())
  {
    TString strData(message);
    m_Logger->fatal(strData.c_str());
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
unsigned long MQUALIFIER TZeusLogger::printInfo(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_Logger == NULL)
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  else if (m_Logger->isInfoEnabled())
  {
    TString strData(message);
    m_Logger->info(strData.c_str());
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
unsigned long MQUALIFIER TZeusLogger::printWarn(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_Logger == NULL)
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  else if (m_Logger->isWarnEnabled())
  {
    TString strData(message);
    m_Logger->warn(strData.c_str());
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
bool MQUALIFIER TZeusLogger::isDebugEnabled()
{
  bool bRetval = false;
  if (m_Logger != NULL)
  {
    bRetval = m_Logger->isDebugEnabled();
  }
  return bRetval;
}

/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
bool MQUALIFIER TZeusLogger::isErrorEnabled()
{
  bool bRetval = false;
  if (m_Logger != NULL)
  {
    bRetval = m_Logger->isErrorEnabled();
  }
  return bRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
bool MQUALIFIER TZeusLogger::isFatalEnabled()
{
  bool bRetval = false;
  if (m_Logger != NULL)
  {
    bRetval = m_Logger->isFatalEnabled();
  }
  return bRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
bool MQUALIFIER TZeusLogger::isInfoEnabled()
{
  bool bRetval = false;
  if (m_Logger != NULL)
  {
    bRetval = m_Logger->isInfoEnabled();
  }
  return bRetval;
}


/**************************************************************************/
/*! See ILogger
*/
/**************************************************************************/
bool MQUALIFIER TZeusLogger::isWarnEnabled()
{
  bool bRetval = false;
  if (m_Logger != NULL)
  {
    bRetval = m_Logger->isWarnEnabled();
  }
  return bRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TZeusLogger::loadConfiguration(IString* file)
{
  unsigned long ulRetval = RET_NOERROR;
  try
  {
    if (file == NULL)
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    else if (file->equals(L"BasicConfiguration"))
    {
      BasicConfigurator::configure();
    }
    else
    {
      USES_CONVERSION;
      TString strPropertyFileName(file);
      
      std::string strFileName = strPropertyFileName.c_str();
      PropertyConfigurator::configure(strFileName);  
    }
  }
  catch(Exception& e)
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TZeusLogger::loadProperties(IPropertyAccess* /*prop*/)
{
  return RET_METHOD_NOT_IMPL;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TZeusLogger);
  INTERFACE_CAST(ILogger, INTERFACE_ILogger);
MEMORY_MANAGER_IMPL_END;


