/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : TestModule                                                    *
 * Package : Test Module for Cell Computing                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:24:30 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: NameServer.cpp,v $
 *   Revision 1.3  2005/11/25 09:24:30  bhadorn
 *   Renamed start method to startSkeleton
 *
 *   Revision 1.2  2005/11/16 11:56:46  bhadorn
 *   Connecting the skeleton through the method start() and checking the return value.
 *
 *   Revision 1.1  2005/11/10 11:05:49  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h> 
#include <NameServer.h>
#include <Naming_Skel.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/ServerSocket.h>
#include <zeusbase/System/Thread.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/Remote/Naming.h>
#include <zeusdev/AbstractModuleSession.h>

USING_NAMESPACE_Zeus

///////////////////////////////////////////////
// Exported Factories
MExportXObjectFactory(TNameServer);
///////////////////////////////////////////////

MregisterLibrary_Start(L"NameServer");
{
  TString strNamingName = L"INaming"; \
  INaming* pNaming = NULL;
  if (manager->getSingleton(&strNamingName, (IZUnknown**)&pNaming) == RET_NOERROR)
  {
    //If the service is already remote -> dont delegate (otherwise it will loop)
    if (!pNaming->isRemote())
    {
      Naming.setDelegationInterface(pNaming);
    }
    pNaming->release();
  }
}
MregisterLibrary_End;

MunregisterLibrary_Start()
  Naming.setDelegationInterface(NULL);
MunregisterLibrary_End;



/***************************************************************************/   
/*! Creates the name server
*/
/***************************************************************************/   
TNameServer::TNameServer(IXMLNode* node) : TXObject(node)
{
  m_pSkeleton = new TNaming_Skel(&Naming);
}
    
/***************************************************************************/   
/*! Destroys the server
*/
/***************************************************************************/   
TNameServer::~TNameServer()
{
  m_pSkeleton->release();
  m_pSkeleton = NULL;
}
    
/***************************************************************************/   
/*! See TXObject
*/
/***************************************************************************/   
bool MQUALIFIER TNameServer::freeze()
{
  bool bRetval = TXObject::freeze();
  if (bRetval)
  {
    m_pSkeleton->kill();
    Naming.reset();
  }  
  return bRetval;
}

/***************************************************************************/   
/*! See TXObject
*/
/***************************************************************************/   
bool MQUALIFIER TNameServer::unfreeze()
{
  bool bRetval = TXObject::unfreeze();
  if (bRetval)
  {
    TString strAddress;
    unsigned long ulPort;
    this->readAttribute(L"Address", strAddress);
    this->readAttribute(L"Port", ulPort);
    if (m_pSkeleton->startSkeleton(strAddress, ulPort, false) != RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "Could not start name server [%s:%d]", strAddress.c_str(), ulPort);
    }
  }  
  return bRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TNameServer);
  INTERFACE_CAST(IRunnable, INTERFACE_IRunnable);
MEMORY_MANAGER_IMPL_PARENT_END(TXObject);


//---------------------------------------------------------------------------
#ifdef LINUX

#else

#include <windows.h>
//---------------------------------------------------------------------------
//   Wichtiger Hinweis zur DLL-Speicherverwaltung, falls die DLL die statische
//   Version der Laufzeitbibliothek (RTL) verwendet:
//
//   Wenn die DLL Funktionen exportiert, die String-Objekte (oder Strukturen/
//   Klassen, die verschachtelte Strings enthalten) als Parameter oder Funktionsergebnisse bergibt,
//   mu�die Bibliothek MEMMGR.LIB im DLL-Projekt und anderen Projekten,
//   die die DLL verwenden, vorhanden sein. Sie ben�igen MEMMGR.LIB auch dann,
//   wenn andere Projekte, die die DLL verwenden, new- oder delete-Operationen
//   auf Klassen anwenden, die nicht von TZObject abgeleitet sind und die aus der DLL exportiert
//   werden. Durch das Hinzufgen von MEMMGR.LIB wird die DLL und deren aufrufende EXEs
//   angewiesen, BORLNDMM.DLL als Speicherverwaltung zu benutzen. In diesem Fall
//   sollte die Datei BORLNDMM.DLL zusammen mit der DLL weitergegeben werden.
//
//   Um die Verwendung von BORLNDMM.DLL, zu vermeiden, sollten String-Informationen als "char *" oder
//   ShortString-Parameter weitergegeben werden.
//
//   Falls die DLL die dynamische Version der RTL verwendet, mssen Sie
//   MEMMGR.LIB nicht explizit angeben.
//---------------------------------------------------------------------------
int WINAPI DllEntryPoint(HINSTANCE /*hinst*/, unsigned long /*reason*/, void* /*lpReserved*/)
{
        return 1;
}
//---------------------------------------------------------------------------
#endif
