/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Naming_Skel                                                   *
 * Package : NameServer                                                    *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:55:00 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Naming_Skel.cpp,v $
 *   Revision 1.2  2005/11/16 11:55:00  bhadorn
 *   New method for dispatching implemented: getObjectNames()
 *
 *   Revision 1.1  2005/11/10 11:05:49  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
  
#include <Naming_Skel.h>
#include <zeusbase/Remote/Naming.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/StringList.h>
#include <zeusbase/System/ZVariant.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/* Dispatching of the incomming method calls
*/
/***************************************************************************/
SKEL_DISPATCH_START(TNaming_Skel)
  //------------------------------------------------------------------------
  // Dispatching of bind2
  SKEL_DISP_SWITCH(L"bind2")
    IZVariant* pVarName = NULL;
    IZVariant* pVarCat = NULL;
    IZVariant* pVarObject = NULL;
    SKEL_READ_PARAM("name", pVarName);
    SKEL_READ_PARAM("cat", pVarCat);
    SKEL_READ_PARAM_REMOTEREF("object", pVarObject);
    
    if (pVarName != NULL && pVarCat != NULL && SKEL_REMOTEREF(pVarObject) != NULL)
    {
      TString strName;
      TString strCat;
      pVarName->getString(&strName);
      pVarCat->getString(&strCat);
      ulRetval = Naming.bind2(&strName, &strCat, SKEL_REMOTEREF(pVarObject));
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of bind2
  SKEL_DISP_SWITCH(L"unbind2")
    IZVariant* pVarName = NULL;
    IZVariant* pVarCat = NULL;
    SKEL_READ_PARAM("name", pVarName);
    SKEL_READ_PARAM("cat", pVarCat);
    
    if (pVarName != NULL && pVarCat != NULL)
    {
      TString strName;
      TString strCat;
      pVarName->getString(&strName);
      pVarCat->getString(&strCat);
      ulRetval = Naming.unbind2(&strName, &strCat);
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of rebind2
  SKEL_DISP_SWITCH(L"rebind2")
    IZVariant* pVarName = NULL;
    IZVariant* pVarCat = NULL;
    IZVariant* pVarObject = NULL;
    SKEL_READ_PARAM("name", pVarName);
    SKEL_READ_PARAM("cat", pVarCat);
    SKEL_READ_PARAM_REMOTEREF("object", pVarObject);
    
    if (pVarName != NULL && pVarCat != NULL && SKEL_REMOTEREF(pVarObject) != NULL)
    {
      TString strName;
      TString strCat;
      pVarName->getString(&strName);
      pVarCat->getString(&strCat);
      ulRetval = Naming.rebind2(&strName, &strCat, SKEL_REMOTEREF(pVarObject));
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of lookup2
  SKEL_DISP_SWITCH(L"lookup2")
    IZVariant* pVarName = NULL;
    IZVariant* pVarCat = NULL;
    SKEL_READ_PARAM("name", pVarName);
    SKEL_READ_PARAM("cat", pVarCat);
    
    if (pVarName != NULL && pVarCat != NULL)
    {
      TString strName;
      TString strCat;
      pVarName->getString(&strName);
      pVarCat->getString(&strCat);
      IRemoteObject* pObject = NULL;    
      ulRetval = Naming.lookup2(&strName, &strCat, &pObject);
      if (ulRetval == RET_NOERROR)
      {
        TRemoteReference* pRemoteRef = new TRemoteReference(pObject);
        TZVariant* pVarObject = new TZVariant(L"object", (ISerializable*)pRemoteRef);
        pReturnCall->addParameter(pVarObject);
        pObject->release();
        pVarObject->release();
        pRemoteRef->release();
      }
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END

  //------------------------------------------------------------------------
  // Dispatching of lookupAny
  SKEL_DISP_SWITCH(L"lookupAny")
    IZVariant* pVarCat = NULL;
    SKEL_READ_PARAM("cat", pVarCat);
    
    if (pVarCat != NULL)
    {
      TString strCat;
      pVarCat->getString(&strCat);
      IRemoteObject* pObject = NULL;    
      ulRetval = Naming.lookupAny(&strCat, &pObject);
      if (ulRetval == RET_NOERROR)
      {
        TRemoteReference* pRemoteRef = new TRemoteReference(pObject);
        TZVariant* pVarObject = new TZVariant(L"object", (ISerializable*)pRemoteRef);
        pReturnCall->addParameter(pVarObject);
        pObject->release();
        pVarObject->release();
        pRemoteRef->release();
      }
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of lookupMultibleObjects
  SKEL_DISP_SWITCH(L"lookupMultibleObjects")
    IZVariant* pVarName = NULL;
    SKEL_READ_PARAM("name", pVarName);
    
    if (pVarName != NULL)
    {
      TString strName;
      pVarName->getString(&strName);
      TSingleLinkedList<IRemoteObject*> lstObjects;
      TSingleLinkedList<ISerializable*> lstSerialObjects;
      ulRetval = Naming.lookupMultibleObjects(&strName, &lstObjects);
      if (ulRetval == RET_NOERROR)
      {
        IListIterator<IRemoteObject*>* pIt = lstObjects.getIterator();
        while(pIt->hasNextItem())
        {
          IRemoteObject* pObject = pIt->getNextItem();
          lstSerialObjects.add((ISerializable*)new TRemoteReference(pObject));          
          pObject->release();
        }
        lstObjects.releaseIterator(pIt);
        lstObjects.clear();
        
        TZVariant* pVarObjects = new TZVariant(L"objects", &lstSerialObjects);
        pReturnCall->addParameter(pVarObjects);
        pVarObjects->release();
        
        while(lstSerialObjects.getCount() > 0)
        {
          lstSerialObjects[0]->release();
          lstSerialObjects.deleteItem(0);
        }
      }
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END

  //------------------------------------------------------------------------
  // Dispatching of lookupCategory
  SKEL_DISP_SWITCH(L"lookupCategory")
    IZVariant* pVarCat = NULL;
    SKEL_READ_PARAM("cat", pVarCat);
    
    if (pVarCat != NULL)
    {
      TString strCat;
      pVarCat->getString(&strCat);
      TSingleLinkedList<IRemoteObject*> lstObjects;
      TSingleLinkedList<ISerializable*> lstSerialObjects;
      ulRetval = Naming.lookupCategory(&strCat, &lstObjects);
      if (ulRetval == RET_NOERROR)
      {
        IListIterator<IRemoteObject*>* pIt = lstObjects.getIterator();
        while(pIt->hasNextItem())
        {
          IRemoteObject* pObject = pIt->getNextItem();
          lstSerialObjects.add((ISerializable*)new TRemoteReference(pObject));          
          pObject->release();
        }
        lstObjects.releaseIterator(pIt);
        lstObjects.clear();
        
        TZVariant* pVarObjects = new TZVariant(L"objects", &lstSerialObjects);
        pReturnCall->addParameter(pVarObjects);
        pVarObjects->release();
        
        while(lstSerialObjects.getCount() > 0)
        {
          lstSerialObjects[0]->release();
          lstSerialObjects.deleteItem(0);
        }
      }
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END

  //------------------------------------------------------------------------
  // Dispatching of getObjectNames
  SKEL_DISP_SWITCH(L"getObjectNames")
    IZVariant* pVarCat = NULL;
    SKEL_READ_PARAM("cat", pVarCat);
    if (pVarCat != NULL)
    {
      TString strCat;
      pVarCat->getString(&strCat);
      TStringList lstNames;
      ulRetval = Naming.getObjectNames(&strCat, &lstNames);
      if (ulRetval == RET_NOERROR)
      {
        TZVariant* pCountVar = new TZVariant(RPARAM_LIST_COUNT("names"), (long)lstNames.getCount());
        pReturnCall->addParameter(pCountVar);
        
        IListIterator<IString*>* pIt = lstNames.getIterator();
        long lIndex = 0;
        while(pIt->hasNextItem())
        {
          TZVariant* pVarString = new TZVariant(RPARAM_LIST_ITEM("names", lIndex), pIt->getNextItem());
          pReturnCall->addParameter(pVarString);
          pVarString->release();
          lIndex++;
        }
        lstNames.releaseIterator(pIt);
        lstNames.clear();
        
        pCountVar->release();
      }
    }
    else
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END
SKEL_DISPATCH_END
