
#include <XMLDocument.h>
#include <zeusbase/System/String.h>
#include <XMLNode.h>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMWriter.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>
#include <xercesc/util/XMLUni.hpp>

#include <stdio.h>

#ifdef LINUX

#else
  #pragma warn -8080
#endif


XERCES_CPP_NAMESPACE_USE

USING_NAMESPACE_Zeus


/***************************************************************************/
/*! Creates the XML Document
    \param docu : Document Instance
    \param filename : Name of the file
*/
/***************************************************************************/
TXMLDocument::TXMLDocument(XERCES_CPP_NAMESPACE::DOMDocument* docu, TString filename) //: TZObject()
{
  m_strFileName = filename;
  m_pDocument = docu;
}

/***************************************************************************/
/*! Destroys the XML Document
*/
/***************************************************************************/
TXMLDocument::~TXMLDocument()
{
  if (m_pDocument != NULL)
  {
    m_pDocument->release();
    m_pDocument = NULL;
  }
}

/***************************************************************************/
/*! see IXMLDocument
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLDocument::getMainNode(IXMLNode** main_node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;;
  if (main_node == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }  
  else if (m_pDocument != NULL)
  {
    DOMNodeList* pChildList = m_pDocument->getChildNodes();
    DOMNode* pNode = NULL;
    bool bAboard = false;
    long lIndex = 0;
    while(!bAboard)
    {
      pNode = pChildList->item(lIndex);
      if (pNode == NULL)
      {
        bAboard = true;
      }
      else if (pNode->getNodeType() == DOMNode::ELEMENT_NODE)
      {
        bAboard = true;
      }
      else
      {
        pNode = NULL;
        lIndex++;
      }
    }

    if (pNode != NULL)
    {
      *main_node = new TXMLNode(pNode);
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLDocument
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLDocument::getStream(IString* stream)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (stream == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pDocument != NULL)
  {
    try
    {
      // get a serializer, an instance of DOMWriter
      XMLCh strTemp[100];
      XMLString::transcode("LS", strTemp, 99);
      DOMImplementation* pImpl = DOMImplementationRegistry::getDOMImplementation(strTemp);
      DOMWriter* pSerializer = ((DOMImplementationLS*)pImpl)->createDOMWriter();

      XMLCh* cBuffer = pSerializer->writeToString(*m_pDocument);
      TString strRet = TXMLNode::transcode(cBuffer);
      stream->assignStr(&strRet);

      delete cBuffer;
      delete pSerializer;
      ulRetval = RET_NOERROR;

    }
    catch (XMLException& e)
    {
      printf("An error occurred during creation of output transcoder. Msg is: %s\n",
              XMLString::transcode(e.getMessage()));
      ulRetval = RET_REQUEST_FAILED;
    }
    catch(...)
    {
      printf("An error occurred during creation of output transcoder.\n");
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLDocument
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLDocument::save()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDocument != NULL)
  {
    try
    {
      // get a serializer, an instance of DOMWriter
      XMLCh strTemp[100];
      XMLString::transcode("LS", strTemp, 99);
      DOMImplementation* pImpl = DOMImplementationRegistry::getDOMImplementation(strTemp);
      DOMWriter* pSerializer = ((DOMImplementationLS*)pImpl)->createDOMWriter();

      // set user specified output encoding
     // theSerializer->setEncoding(gOutputEncoding);

      // set feature if the serializer supports the feature/mode
      /*if (theSerializer->canSetFeature(XMLUni::fgDOMWRTSplitCdataSections, gSplitCdataSections))
          theSerializer->setFeature(XMLUni::fgDOMWRTSplitCdataSections, gSplitCdataSections);

      if (theSerializer->canSetFeature(XMLUni::fgDOMWRTDiscardDefaultContent, gDiscardDefaultContent))
          theSerializer->setFeature(XMLUni::fgDOMWRTDiscardDefaultContent, gDiscardDefaultContent);

      if (theSerializer->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint, gFormatPrettyPrint))
          theSerializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, gFormatPrettyPrint);

      if (theSerializer->canSetFeature(XMLUni::fgDOMWRTBOM, gWriteBOM))
          theSerializer->setFeature(XMLUni::fgDOMWRTBOM, gWriteBOM);
      */

      XMLFormatTarget* pFormTarget = new LocalFileFormatTarget(m_strFileName.c_str());
      pSerializer->writeNode(pFormTarget, *m_pDocument);

      delete pSerializer;
      delete pFormTarget;
      ulRetval = RET_NOERROR;

    }
    catch (XMLException& e)
    {
      printf("An error occurred during creation of output transcoder. Msg is: %s\n",
              XMLString::transcode(e.getMessage()));
      ulRetval = RET_REQUEST_FAILED;
    }
    catch(...)
    {
      printf("An error occurred during creation of output transcoder.\n");
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! Siehe IXMLDocument
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLDocument::saveAs(IString* filename)
{
  unsigned long ulRetval = RET_NOERROR;
  if (filename == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    m_strFileName = filename->c_bstr();
    ulRetval = this->save();
  }
  return ulRetval;
}

/***************************************************************************/
/*! Siehe IXMLDocument
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLDocument::createNewNode(unsigned long node_type,
                                        IString* name,
                                        IString* uri,
                                        IXMLNode** node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || uri == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pDocument != NULL)
  {
    try
    {
      DOMNode* pNode = NULL;
      XMLCh* cName = XMLString::transcode(TString(name).c_str());
      XMLCh* cURI = XMLString::transcode(TString(uri).c_str());
      switch(node_type)
      {
        
        case TYPE_NODE_DOCUMENT_FRAGMENT :
          pNode = m_pDocument->createDocumentFragment();
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;
        break;
        
        case TYPE_NODE_TEXT :
          m_pDocument->createTextNode(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;
        break;
        
        case TYPE_NODE_DOCUMENT_TYPE :
          pNode = m_pDocument->createDocumentType(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;
        break;
        
        case TYPE_NODE_ELEMENT :
          pNode = m_pDocument->createElement(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;
        break;
        
        case TYPE_NODE_ENTITY :
          pNode = m_pDocument->createEntity(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;        
        break;
        
        case TYPE_NODE_CDATA_SECTION :
          pNode = m_pDocument->createCDATASection(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;        
        break;
        
        case TYPE_NODE_COMMENT :
          pNode = m_pDocument->createComment(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;        
        break;
        
        case TYPE_NODE_DOCUMENT :
        break;
        
        case TYPE_NODE_ENTITY_REFERENCE :
          pNode = m_pDocument->createEntityReference(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;        
        break;
        
        case TYPE_NODE_NOTATION :
          pNode = m_pDocument->createNotation(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;        
        break;
        
        case TYPE_NODE_PROCESSING_INSTRUCTION :
          pNode = m_pDocument->createProcessingInstruction(cName,cURI); 
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;        
        break;
        
        case TYPE_NODE_ATTRIBUTE :
          pNode = m_pDocument->createAttribute(cName);
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;
        break;
        
        case TYPE_NODE_DECL :
        break;
        
        default:
          ulRetval = RET_REQUEST_FAILED;
      }
      XMLString::release(&cName);
      XMLString::release(&cURI);
    }
    catch(...)
    {
      ulRetval = RET_REQUEST_FAILED;
    }    
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TXMLDocument);
  INTERFACE_CAST(IXMLDocument, INTERFACE_IXMLDocument);
MEMORY_MANAGER_IMPL_END;

