/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XMLNode                                                       *
 * Package : XML_Service                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:19:28 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XMLNode.cpp,v $
 *   Revision 1.3  2005/12/14 09:19:28  bhadorn
 *   corrected resetNodeIterator() to accept NULL as argument.
 *
 *   Revision 1.2  2005/11/25 10:16:21  bhadorn
 *   pragma only supported by borland compiler
 *
 *   Revision 1.1  2005/10/28 12:13:11  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <XMLNode.h>
#include <XMLNodeList.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/String.h>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMWriter.hpp>
#include <stdio.h>

#ifdef LINUX

#else
  #pragma warn -8080
#endif

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Erstellen des XML Nodes 
*/
/***************************************************************************/
TXMLNode::TXMLNode(DOMNode* node) : TZObject()
{
  m_pNode = node;
  m_lAttributeIndex = 0;
  m_lNodeIndex = 0;
//  m_pLock = new TCriticalSection();
}

/***************************************************************************/
/*! Zerstoeren des XML Nodes 
*/
/***************************************************************************/
TXMLNode::~TXMLNode()
{
  m_pNode = NULL;
//  m_pLock->release();
}

/***************************************************************************/
/*! Einen XMLString in einen TString umwandeln
*/
/***************************************************************************/
TString TXMLNode::transcode(const XMLCh* xml)
{
  TString strRetval;
  char* cBuffer = XMLString::transcode(xml);
  strRetval = TString(cBuffer);
  XMLString::release(&cBuffer);
  
  return strRetval;
}

/***************************************************************************/
/*! see IXMLNode 
*/
/***************************************************************************/
bool MQUALIFIER TXMLNode::hasChildren()
{
  bool bRetval = false;
  if (m_pNode != NULL)
  {
    bRetval = m_pNode->hasChildNodes();
  }
  return bRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::appendNode(IXMLNode* new_node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (new_node == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    //Check if class name is IMPCLASSNAME_XERCESC
    if (new_node->getImplClass() == IMPCLASSNAME_XERCESC)
    {
      //Try to add node
      DOMNode* pNode2Insert = ((TXMLNode*)new_node)->m_pNode;
      try
      {
        ((TXMLNode*)new_node)->m_pNode = m_pNode->appendChild(pNode2Insert);
        ulRetval = RET_NOERROR;
      }
      catch(...)
      {
        ulRetval = RET_INTERNAL_ERROR;
      }
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::clone(bool recurs, IXMLNode** node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (node == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    //Clone node
    DOMNode* pClonedNode = m_pNode->cloneNode(recurs);
    if (pClonedNode != NULL)
    {
      *node = new TXMLNode(pClonedNode);
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::createNewChildNode(unsigned long type, IString* name, IString* uri, IXMLNode** node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || uri == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    try
    {
      XERCES_CPP_NAMESPACE::DOMDocument* pDocument = m_pNode->getOwnerDocument();
      if (pDocument != NULL)
      {
        DOMNode* pNode = NULL;
        XMLCh* cName = XMLString::transcode(TString(name).c_str());
        XMLCh* cURI = XMLString::transcode(TString(uri).c_str());
        switch(type)
        {

          case TYPE_NODE_DOCUMENT_FRAGMENT :
            pNode = pDocument->createDocumentFragment();
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_TEXT :
            pDocument->createTextNode(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_DOCUMENT_TYPE :
            pNode = pDocument->createDocumentType(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_ELEMENT :
            pNode = pDocument->createElement(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_ENTITY :
            pNode = pDocument->createEntity(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_CDATA_SECTION :
            pNode = pDocument->createCDATASection(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_COMMENT :
            pNode = pDocument->createComment(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_DOCUMENT :
          break;

          case TYPE_NODE_ENTITY_REFERENCE :
            pNode = pDocument->createEntityReference(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_NOTATION :
            pNode = pDocument->createNotation(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_PROCESSING_INSTRUCTION :
            pNode = pDocument->createProcessingInstruction(cName, cURI);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_ATTRIBUTE :
            pNode = pDocument->createAttribute(cName);
            *node = new TXMLNode(pNode);
            ulRetval = RET_NOERROR;
          break;

          case TYPE_NODE_DECL :
          break;

          default:
            ulRetval = RET_REQUEST_FAILED;
        }
        
        XMLString::release(&cName);
        XMLString::release(&cURI);
        ulRetval = appendNode(*node);
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    }
    catch(...)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getAttributeValue(IString* name, IString* value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || value == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    DOMNamedNodeMap* pAttribs = m_pNode->getAttributes();
    if(pAttribs != NULL)
    {
      for(unsigned int i = 0; i < pAttribs->getLength(); i++)
      {
        DOMNode* pAttNode = pAttribs->item(i);
        TString strName = transcode(pAttNode->getNodeName());
        if (name->equalsStr(&strName))
        {
          TString strValue = transcode(pAttNode->getNodeValue());
          value->assignStr(&strValue);
          ulRetval = RET_NOERROR;
          break;
        }
      }
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
void MQUALIFIER TXMLNode::resetAttributeIterator()
{
  m_lAttributeIndex = -1;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
void MQUALIFIER TXMLNode::resetNodeIterator(IString* name)
{
  if (name == NULL)
  {
    m_strNodeName = L"";
  }
  else
  {
    m_strNodeName = name->c_bstr();
  }
  m_lNodeIndex = -1;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getImplClass()
{
  return IMPCLASSNAME_XERCESC;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getNextAttribute(IString* name, IString* value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || value == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    DOMNamedNodeMap* pAttribs = m_pNode->getAttributes();
    if(pAttribs != NULL)
    {
      m_lAttributeIndex++;
      DOMNode* pAttNode = pAttribs->item(m_lAttributeIndex);
      TString strName = transcode(pAttNode->getNodeName());
      TString strValue = transcode(pAttNode->getNodeValue());
      name->assignStr(&strName);
      value->assignStr(&strValue);
    }
    ulRetval = RET_NOERROR;
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getNextNode(IXMLNode** node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (node == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    *node = NULL;
    DOMNodeList* pNodeList = m_pNode->getChildNodes();
    if (pNodeList != NULL)
    {
      bool bFound = false;
      while(!bFound && pNodeList->getLength() > (unsigned long)m_lNodeIndex + 1)
      {
        m_lNodeIndex++;
        DOMNode* pNode = pNodeList->item(m_lNodeIndex);
        TString strName = transcode(pNode->getNodeName());
        if (m_strNodeName == L"" || m_strNodeName.equalsStr(&strName))
        {
          bFound = true;
          *node = new TXMLNode(pNode);
          ulRetval = RET_NOERROR;
        }
      }
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getNodeList(IXMLNodeList** list)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (list == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    DOMNodeList* pList = m_pNode->getChildNodes();
    *list = new TXMLNodeList(pList);
    ulRetval = RET_NOERROR;
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getNodeName(IString* name)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    TString strName = transcode(m_pNode->getNodeName());
    name->assignStr(&strName);
    ulRetval = RET_NOERROR;
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getNodeType()
{
  unsigned long ulRetval = TYPE_NODE_ELEMENT;

  if (m_pNode != NULL)
  {
    switch(m_pNode->getNodeType())
    {
      case DOMNode::ELEMENT_NODE :
        ulRetval = TYPE_NODE_ELEMENT;
      break;

      case DOMNode::ATTRIBUTE_NODE :
        ulRetval = TYPE_NODE_ATTRIBUTE;
      break;

      case DOMNode::TEXT_NODE :
        ulRetval = TYPE_NODE_TEXT;
      break;

      case DOMNode::CDATA_SECTION_NODE :
        ulRetval = TYPE_NODE_CDATA_SECTION;
      break;

      case DOMNode::ENTITY_REFERENCE_NODE :
        ulRetval = TYPE_NODE_ENTITY_REFERENCE;
      break;

      case DOMNode::ENTITY_NODE :
        ulRetval = TYPE_NODE_ENTITY;
      break;

      case DOMNode::PROCESSING_INSTRUCTION_NODE :
        ulRetval = TYPE_NODE_PROCESSING_INSTRUCTION;
      break;

      case DOMNode::COMMENT_NODE :
        ulRetval = TYPE_NODE_COMMENT;
      break;

      case DOMNode::DOCUMENT_NODE :
        ulRetval = TYPE_NODE_DOCUMENT;
      break;

      case DOMNode::DOCUMENT_TYPE_NODE :
        ulRetval = TYPE_NODE_DOCUMENT_TYPE;
      break;

      case DOMNode::DOCUMENT_FRAGMENT_NODE :
        ulRetval = TYPE_NODE_DOCUMENT_FRAGMENT;
      break;

      case DOMNode::NOTATION_NODE :
        ulRetval = TYPE_NODE_NOTATION;
      break;

      default:
        ulRetval = TYPE_NODE_ELEMENT;
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getNodeValue(IString* value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (value == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    TString strValue = transcode(m_pNode->getNodeValue());
    value->assignStr(&strValue);
    ulRetval = RET_NOERROR;
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::getParentNode(IXMLNode** parent)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (parent == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    *parent = new TXMLNode(m_pNode->getParentNode());
    ulRetval = RET_NOERROR;
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::insertNode(IXMLNode* node, IXMLNode* before_node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (node == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    ulRetval = RET_NOERROR;

    //Get reference node
    DOMNode* pRefNode = NULL;
    if (before_node != NULL && before_node->getImplClass() == IMPCLASSNAME_XERCESC)
    {
      pRefNode = ((TXMLNode*)before_node)->m_pNode;
    }
    else if (before_node != NULL)
    {
      ulRetval = RET_REQUEST_FAILED;
    }

    //Get node to insert
    DOMNode* pNode2Insert = NULL;
    if (node->getImplClass() == IMPCLASSNAME_XERCESC)
    {
      pNode2Insert = ((TXMLNode*)node)->m_pNode;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }


    //Insert node
    if (ulRetval == RET_NOERROR)
    {
      try
      {
        m_pNode->insertBefore(pNode2Insert,pRefNode);
      }
      catch(...)
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::removeMe()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    DOMNode* pParent = m_pNode->getParentNode();
    if (pParent != NULL)
    {
      try
      {
        pParent->removeChild(m_pNode);
        ulRetval = RET_NOERROR;
      }
      catch(...)
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::selectNode(IString* /*query*/, IXMLNode** /*node*/)
{
  return RET_METHOD_NOT_IMPL;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::selectNodes(IString* /*query*/, IXMLNodeList** /*list*/)
{
  return RET_METHOD_NOT_IMPL;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::setAttributeValue(IString* name, IString* value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || value == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL && m_pNode->getNodeType() == DOMNode::ELEMENT_NODE)
  {
    DOMElement* pElement = (DOMElement*)m_pNode;
    try
    {
      XMLCh* cNameBuffer = XMLString::transcode(TString(name).c_str());
      XMLCh* cValueBuffer = XMLString::transcode(TString(value).c_str());

      pElement->setAttribute(cNameBuffer, cValueBuffer);

      XMLString::release(&cNameBuffer);
      XMLString::release(&cValueBuffer);

      ulRetval = RET_NOERROR;
    }
    catch(...)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::setNodeValue(IString* data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    try
    {
      TString strName(data);
      XMLCh* cBuffer = XMLString::transcode(strName.c_str());
      m_pNode->setNodeValue(cBuffer);

      XMLString::release(&cBuffer);
      ulRetval = RET_NOERROR;
    }
    catch(...)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! Siehe IXMLNode
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLNode::serializeNode(IString* stream)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (stream == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    try
    {
      XMLCh strTemp[100];
      XMLString::transcode("LS", strTemp, 99);
      DOMImplementation* pImpl = DOMImplementationRegistry::getDOMImplementation(strTemp);
      DOMWriter* pSerializer = ((DOMImplementationLS*)pImpl)->createDOMWriter();
      XMLCh* cBuffer = pSerializer->writeToString(*m_pNode);
      TString strStream = this->transcode(cBuffer);

      stream->assignStr(&strStream);

      delete cBuffer;
      delete pSerializer;
      ulRetval = RET_NOERROR;
    }
    catch(...)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;

}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
//MEMORY_MANAGER_IMPL_LOCK(TXMLNode, m_pLock);
MEMORY_MANAGER_IMPL(TXMLNode);
  INTERFACE_CAST(IXMLNode, INTERFACE_IXMLNode);
MEMORY_MANAGER_IMPL_END;


