/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XMLNode                                                       *
 * Package : XML_Service                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:13:11 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XMLNode.h,v $
 *   Revision 1.1  2005/10/28 12:13:11  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef XMLNODE_H
#define XMLNODE_H

#ifndef LINUX
  #include <mem.h>
#endif

#include <zeusbase/System/Interfaces/IXMLNode.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>
#include <xercesc/dom/DOM.hpp>


XERCES_CPP_NAMESPACE_USE

BEGIN_NAMESPACE_Zeus

class TCriticalSection;
/***************************************************************************/
/*! Implementierung der Wrapperklasse fuer einen XML Node
*/
/***************************************************************************/
class TXMLNode : public TZObject, public IXMLNode
{
  public:
    TXMLNode(DOMNode* node);

    static TString transcode(const XMLCh* xml);
    
    //Interface IXMLNode
    virtual bool MQUALIFIER hasChildren();
    virtual unsigned long MQUALIFIER appendNode(IXMLNode* new_node);
    virtual unsigned long MQUALIFIER clone(bool recurs, IXMLNode** node);
    virtual unsigned long MQUALIFIER createNewChildNode( unsigned long type,
                                              IString* name,
                                              IString* uri,
                                              IXMLNode** node);
    virtual unsigned long MQUALIFIER getAttributeValue(IString* name, IString* value);
    virtual void MQUALIFIER resetAttributeIterator();
    virtual void MQUALIFIER resetNodeIterator(IString* name);
    virtual unsigned long MQUALIFIER getImplClass();
    virtual unsigned long MQUALIFIER getNextAttribute(IString* name, IString* value);
    virtual unsigned long MQUALIFIER getNextNode(IXMLNode** node);
    virtual unsigned long MQUALIFIER getNodeList(IXMLNodeList** list);
    virtual unsigned long MQUALIFIER getNodeName(IString* name);
    virtual unsigned long MQUALIFIER getNodeType();
    virtual unsigned long MQUALIFIER getNodeValue(IString* value);
    virtual unsigned long MQUALIFIER getParentNode(IXMLNode** parent);
    virtual unsigned long MQUALIFIER insertNode(IXMLNode* node, IXMLNode* before_node);
    virtual unsigned long MQUALIFIER removeMe();
    virtual unsigned long MQUALIFIER selectNode(IString* query, IXMLNode** node);
    virtual unsigned long MQUALIFIER selectNodes(IString* query, IXMLNodeList** list);
    virtual unsigned long MQUALIFIER setAttributeValue(IString* name, IString* value);
    virtual unsigned long MQUALIFIER setNodeValue(IString* data);
    virtual unsigned long MQUALIFIER serializeNode(IString* stream);


    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    virtual ~TXMLNode();

  private:
    ///Parser Instanz
    DOMNode* m_pNode;
    ///Aktueller Attributeindex fuer Iterator
    long m_lAttributeIndex;
    ///Aktueller Nodeindex fuer Iterator
    long m_lNodeIndex;
    ///Name der gesuchten Nodes
    TString m_strNodeName;
    ///Lock
   // TCriticalSection* m_pLock;

};
END_NAMESPACE_Zeus

#endif
