/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Code Module                                                   *
 * Package : Code Package                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:16:21 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XMLParser.cpp,v $
 *   Revision 1.2  2005/11/25 10:16:21  bhadorn
 *   pragma only supported by borland compiler
 *
 *   Revision 1.1  2005/10/28 12:13:11  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <XMLParser.h>
#include <zeusbase/System/String.h>

#include <XMLDocument.h>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMWriter.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/framework/StdOutFormatTarget.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>
#include <xercesc/util/XMLUni.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>
#include <stdio.h>

#ifdef LINUX

#else
  #pragma warn -8080
#endif

XERCES_CPP_NAMESPACE_USE
USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Erstellen des XML Parsers
*/
/***************************************************************************/
TXMLParser::TXMLParser() : TZObject()
{
  m_pParser = NULL;
}


/***************************************************************************/
/*! Zerstoeren des XML Parsers
*/
/***************************************************************************/
TXMLParser::~TXMLParser()
{
  if (m_pParser != NULL)
  {
    delete m_pParser;
  }
}

/***************************************************************************/
/*! Siehe IXMLParser
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLParser::loadFromFile(const IString* filename, NAMESPACE_Zeus::IXMLDocument** docu)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (filename == NULL || docu == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    bool bErrorOccured = false;
    if (m_pParser != NULL)
    {
      delete m_pParser;
    }
  
    m_pParser = new XercesDOMParser;
    m_pParser->setValidationScheme(XercesDOMParser::Val_Auto);
  //  m_pParser->setDoNamespaces(false);
  //  m_pParser->setDoSchema(false);
  //  m_pParser->setValidationSchemaFullChecking(false);
  //  m_pParser->setCreateEntityReferenceNodes(false);
    TString strFileName = TString(filename);
  
    try
    {
      m_pParser->parse(strFileName.c_str());
    }
    catch (const OutOfMemoryException&)
    {
      printf("Error occured parsing file %s: OutOfMemoryException\n",
          strFileName.c_str());
      bErrorOccured = true;
    }
    catch (const XMLException& e)
    {
      printf("Error occured parsing file %s: Message: %s\n",
          strFileName.c_str(),
          XMLString::transcode(e.getMessage()));
      bErrorOccured = true;
    }
  
    catch (const DOMException& e)
    {
      const unsigned int maxChars = 2047;
      XMLCh errText[maxChars + 1];
      printf("Error occured parsing file %s: DOM Error [Code = %d]\n",
          strFileName.c_str(), e.code);
  
      if (DOMImplementation::loadDOMExceptionMsg(e.code, errText, maxChars))
      {
        printf("\tMessage is :%s", XMLString::transcode(errText));
      }
  
      bErrorOccured = true;
    }
    catch(...)
    {
      printf("Error occured parsing file %s\n",
          strFileName.c_str());
      bErrorOccured = true;
    }
  
    //creates the xml document
    if (bErrorOccured != true)
    {
      *docu = new TXMLDocument(m_pParser->adoptDocument(),strFileName);
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! Siehe IXMLParser
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLParser::parseStream(const IString* stream, NAMESPACE_Zeus::IXMLDocument** docu)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (stream == NULL || docu == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    bool bErrorOccured = false;
    if (m_pParser != NULL)
    {
      delete m_pParser;
    }
  
    m_pParser = new XercesDOMParser;
    m_pParser->setValidationScheme(XercesDOMParser::Val_Auto);
    m_pParser->setDoNamespaces(false);
    m_pParser->setDoSchema(false);
    m_pParser->setValidationSchemaFullChecking(false);
    m_pParser->setCreateEntityReferenceNodes(false);
    TString strBuffer(stream);
    MemBufInputSource* pMemBuffer = new MemBufInputSource((const XMLByte*)strBuffer.c_str(),
                                                        strBuffer.getSize(),
                                                        "Buffer",
                                                        false);
  
    try
    {
      m_pParser->parse(*pMemBuffer);
    }
    catch (const OutOfMemoryException&)
    {
      printf("Error occured parsing buffer %s: OutOfMemoryException\n",
          strBuffer.c_str());
      bErrorOccured = true;
    }
    catch (const XMLException& e)
    {
      printf("Error occured parsing buffer %s: Message: %s\n",
          strBuffer.c_str(),
          XMLString::transcode(e.getMessage()));
      bErrorOccured = true;
    }
  
    catch (const DOMException& e)
    {
      const unsigned int maxChars = 2047;
      XMLCh errText[maxChars + 1];
      printf("Error occured parsing buffer %s: DOM Error [Code = %d]\n",
          strBuffer.c_str(), e.code);
  
      if (DOMImplementation::loadDOMExceptionMsg(e.code, errText, maxChars))
      {
        printf("\tMessage is :%s", XMLString::transcode(errText));
      }
  
      bErrorOccured = true;
    }
    catch(...)
    {
      printf("Error occured parsing buffer %s\n",
          strBuffer.c_str());
      bErrorOccured = true;
    }
    
    delete pMemBuffer;
    pMemBuffer = NULL;
  
    //creates the xml document
    if (bErrorOccured != true)
    {
      *docu = new TXMLDocument(m_pParser->adoptDocument(),L"");
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  return ulRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TXMLParser);
  INTERFACE_CAST(IXMLParser, INTERFACE_IXMLParser);
MEMORY_MANAGER_IMPL_END;


