/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FrameLoader                                                   *
 * Package : zeusbase                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:17:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractFrameLoader.h,v $
 *   Revision 1.2  2005/11/25 09:17:53  bhadorn
 *   Register RelayCell class
 *
 *   Revision 1.1  2005/10/28 12:19:16  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef AbstractFrameLoaderH
#define AbstractFrameLoaderH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/PropertyFile.h>
#include <zeusbase/System/Directory.h>
#include <zeusbase/System/Interfaces/IXObject.hpp>

/***************************************************************************/
/*! Starts the framework and initializes the logging. This macro can be used
    in the main function to start up every thing.
    \param loggerclass : Class of the logging 
    \param frameclass : Class of the specialized framework
    \param retval : int value for return values of the application
*/
/***************************************************************************/
#define MStartZeusFramework(loggerclass, frameclass, retval) \
  ILogger* pStartLogger = new loggerclass();\
  TLoggerManager::getInstance().setLogger(pStartLogger);\
  TString strSingName = L"ILogger";\
  TSingletonManager::getInstance().addSingleton(&strSingName, pStartLogger);\
  pStartLogger->release();\
  \
  frameclass* pLoader = NULL;\
  try\
  {\
    pLoader = new frameclass(argc, argv);\
    pLoader->initialize();\
    if (pLoader->canStartup())\
    {\
      ZLOGGER.printfln(LOGMODE_INFO, "Start up zeus framework...");\
      ZLOGGER.indent();\
      pLoader->startup();\
      ZLOGGER.unindent();\
      ZLOGGER.printfln(LOGMODE_INFO, "done");\
      \
      retval = pLoader->run();\
      \
      ZLOGGER.printfln(LOGMODE_INFO, "Shutdown framework...");\
      ZLOGGER.indent();\
      pLoader->shutdown();\
      ZLOGGER.unindent();\
      ZLOGGER.printfln(LOGMODE_INFO, "...done");\
    }\
    pLoader->release();\
    pLoader = NULL;\
  }\
  catch(...)\
  {\
    ZLOGGER.unindent();\
    ZLOGGER.printfln(LOGMODE_INFO, "Exception received. Killing all life :-(");\
  }\



BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! The directory class implements methods to access the system directory
*/
/***************************************************************************/
class TAbstractFrameLoader : public TZObject
{
  public:
    TAbstractFrameLoader(TString base_name, int argc, char *argv[]);
    
    virtual void initialize();
    virtual bool canStartup();
    virtual unsigned long startup();
    virtual unsigned long shutdown();
    virtual long run()=0;
    
    unsigned long getRootObject(IXObject** root);
    
  protected:
    virtual ~TAbstractFrameLoader();
    virtual unsigned long buildXObjectTree(TString filename = L"");
    virtual void registerClasses();
    
    //Initialisation methods
    virtual unsigned long initProperties();
    virtual unsigned long initSecurity();
    virtual unsigned long initSystemLibraries();
  
    ///Working directory
    TDirectory* m_pMainDir;
    ///Root object of the Framework
    IXObject* m_pRootObject;
    ///Loaded property file
    TPropertyFile* m_pSystemSettings;
    ///Property file name
    TString m_strPropertyFileName;
    ///Configuration file of the XML MOM structure
    TString m_strConfigFileName;
  
  private:
    ///Initialized flag
    bool m_bInitialized;
    ///Startup Flag
    bool m_bStartedUp;
    ///Name of the base
    TString m_strBaseName;
    
    void releaseXRootObject();
};

END_NAMESPACE_Zeus

#endif
