/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Cell                                                          *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Cell.h,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CellH
#define CellH

//#include <zeusbase/CellModel/Interfaces/ICell.hpp>
#include <zeusbase/CellModel/Interfaces/ICellComm.hpp>
#include <zeusbase/CellModel/Interfaces/ICell.hpp>
#include <zeusbase/System/XObject.h>
#include <zeusbase/System/XObjectFactory.h>

BEGIN_NAMESPACE_Zeus
 
class ICellKernel;

/**************************************************************************/
/*! The cell class implements the behaviour of a basic cell
*/
/**************************************************************************/
class TCell : public TXObject, public ICellComm, public ICell
{
  public:
    TCell(IXMLNode* node);
        
    //Methods of ICell
    virtual void MQUALIFIER resetPing();
    virtual bool MQUALIFIER getPingResponse();
    virtual double MQUALIFIER getLifePower();
    virtual void MQUALIFIER recalculateLife();
    virtual unsigned long MQUALIFIER clone(ICell** cell);
    virtual unsigned long MQUALIFIER kill();
    
    //Methods of ICellComm
    virtual unsigned long MQUALIFIER getInputMessage(IZUnknown* receiver, 
                                                     IMessage** msg, 
                                                     unsigned long long* proctime);
    virtual unsigned long MQUALIFIER sendMessage(IMessage* message, 
                                                 unsigned long long inittime = 0);
    virtual unsigned long MQUALIFIER sendControlMessage(IMessage* message, 
                                                        unsigned long long inittime = 0);
    virtual long MQUALIFIER getJobCount();
    virtual void MQUALIFIER flushAllPipes();
    virtual void MQUALIFIER updatePipes();
  
    //Methods of IModule
    virtual bool MQUALIFIER freeze();
    virtual bool MQUALIFIER unfreeze();
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
    REG_SUB_BEGIN(TCell)
    REG_SUB_ADD(TCell, L"TCell");
    REG_SUB_END
  
  protected:
    virtual ~TCell();
    
  private:
    ///Communication Interface for the Messaging
    ICellComm* m_pCommunication;
    ///Cell Kernel 
    ICellKernel* m_pCellKernel;
    ///Live Power
    double m_dLifePower;
    ///Removing Power of the cell
    double m_dRemovingPower;
    ///Adding Power of the cell
    double m_dAddingPower;
    ///Flag signalizes that a message has been performed
    bool m_bMessageEaten;
    ///Ping flag
    bool m_bPing;
    
    void releaseInstances();
};

END_NAMESPACE_Zeus

#endif
