/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CellEnvironment                                               *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CellEnvironment.h,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CellEnvironmentH
#define CellEnvironmentH

#include <zeusbase/CellModel/Interfaces/ICellComm.hpp>
#include <zeusbase/CellModel/Interfaces/ICellControl.hpp>
#include <zeusbase/CellModel/Interfaces/IOwnCommPipe.hpp>
#include <zeusbase/MOM/Module.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/SingleLinkedList.hpp>

BEGIN_NAMESPACE_Zeus

class TOutputPipeManager;
class ILocalPipeRegistry;
class ICellSurvey;

/***************************************************************************/
/*! The Cell Environment manages all the pipe connections to other cell
    environments.
*/
/***************************************************************************/
class TCellEnvironment : public TModule, public ICellComm//, public ICellControl
{
  public:
    TCellEnvironment(IXMLNode* node);
    
    //Methods of ICellComm
    virtual unsigned long MQUALIFIER getInputMessage(IZUnknown* receiver, 
                                                     IMessage** msg, 
                                                     unsigned long long* proctime);
    virtual unsigned long MQUALIFIER sendMessage(    IMessage* message, 
                                                     unsigned long long inittime = 0);
    virtual unsigned long MQUALIFIER sendControlMessage(IMessage* message, 
                                                        unsigned long long inittime = 0);
    virtual long MQUALIFIER getJobCount();
    virtual void MQUALIFIER flushAllPipes();
    virtual void MQUALIFIER updatePipes();
    
    //Methods of ICellControl
/*    virtual unsigned long MQUALIFIER clone();
    virtual unsigned long MQUALIFIER controlAndKill();
    virtual unsigned long MQUALIFIER getCellCount();
    virtual unsigned long MQUALIFIER sendControlMessage(IMessageObject* message, unsigned long long inittime = 0);
    virtual unsigned long MQUALIFIER updatePipes();*/
    
    //Methods of IModule
    virtual bool MQUALIFIER freeze();
    virtual bool MQUALIFIER unfreeze();
    
    //Methods of IXObject
    virtual unsigned long MQUALIFIER addChild(IXObject* child);
    virtual unsigned long MQUALIFIER deleteChild(long index);
    virtual unsigned long MQUALIFIER removeChild(IXObject* child);
  
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
        
    REG_SUB_BEGIN(TCellEnvironment)
    REG_SUB_ADD(TCellEnvironment, L"TCellEnvironment");
    REG_SUB_END
    
  protected:
    virtual ~TCellEnvironment();
    ///Lookup registry for cell pipes
    ILocalPipeRegistry* m_pRegistry;
    ///Cell Survey 
    ICellSurvey* m_pCellSurvey;
    ///Output pipe manager
    TOutputPipeManager* m_pOutputPipes;
    ///Lock for input message pipe access
    TCriticalSection* m_pInputPipeLock;
    ///Incomming Message Pipes are owned by this module
    TSingleLinkedList<IOwnCommPipe*> m_lstInputPipes;
    
  private:
    void releaseAllInstances();
    void createInputPipes();
    void tryToRegisterCell(IXObject* obj);
    void tryToUnregisterCell(IXObject* obj);

};

END_NAMESPACE_Zeus

#endif 
