/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CellSurvey                                                    *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CellSurvey.h,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef CellSurveyH
#define CellSurveyH

#include <zeusbase/System/String.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/Interfaces/IXMLNode.hpp>
#include <zeusbase/CellModel/Interfaces/ICellSurvey.hpp>
#include <zeusdev/AbstractCell.h>

#define PIPENAME_SURVEY                                        L"SURVEY"

BEGIN_NAMESPACE_Zeus

class TCommPipe;
class ICommPipe;
class TGrowthCalc;
class TCriticalSection;

/***************************************************************************/
/*! This class defines some kind of cell supervisor to survey the cell
    structure. It implements the cloning and removing algorithm for 
    dynamical program grows.
*/
/***************************************************************************/
class TCellSurvey : public TAbstractCell, public ICellSurvey
{
  public:
    TCellSurvey(IXMLNode* node);

    //Methods of ICellSurvey
    virtual void MQUALIFIER registerCell(ICell* cell);
    virtual void MQUALIFIER unregisterCell(ICell* cell);
    virtual void MQUALIFIER unregisterAllCells();
    virtual unsigned long MQUALIFIER getInputPipe(ICommPipe** pipe);
    
    //Methods of ICellKernel
    virtual void MQUALIFIER initialize(ICellComm* cell_interface);
    virtual void MQUALIFIER uninitialize();
    virtual bool MQUALIFIER isInitialized();
    
    //Methods of IXObject
    virtual bool MQUALIFIER freeze();
    virtual bool MQUALIFIER unfreeze();
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
 
    REG_SUB_BEGIN(TCellSurvey)
    REG_SUB_ADD(TCellSurvey, L"TCellSurvey");
    REG_SUB_END
  
  protected:
    virtual ~TCellSurvey();
    virtual void sendLifeSignal();
    virtual void performSystemMessage(ISystemMessage* msg, unsigned long long timer);
     
  private:
    ///Cell List lock
    TCriticalSection* m_pCellLock;
    ///List of all registered cells
    TSingleLinkedList<ICell*> m_lstCells;
    ///Communication pipe to receive status information about the cell structure
    TCommPipe* m_pInputPipe;
    ///Name of the cell
    TString m_strCellName;
    ///Flag is set if a ping has been sent to the cell structure
    bool m_bPingSent;
    ///Timestamp of a sented ping
    double m_dTimeStamp;
    ///Sleep time of the survey
    double m_dSurveySleepTime;
    ///Growth calculator
    TGrowthCalc* m_pCalc;
    ///Max sleeping time
    double m_dMaxSleepTime;
    ///Min sleeping time;
    double m_dMinSleepTime;
    
    ///Update rate of pipes
    double m_dUpdatePipeRate;
    ///Timestamp of last pipe update
    double m_dLastPipeUpdate;
    ///
    long m_lActCellListIndex;

    void checkMaxSleepTime();
    void checkMinSleepTime();
    void releaseInstances();
    double getCellRemoveProb(double x);
    unsigned long controlAndKill();
};

END_NAMESPACE_Zeus

#endif
