/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus                                                          *
 * Module  : CommPipe                                                      *
 * Package : CellModul                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:22:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommPipe.h,v $
 *   Revision 1.2  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//---------------------------------------------------------------------------

#ifndef CommPipeH
#define CommPipeH

#include <zeusbase/CellModel/Interfaces/IOwnCommPipe.hpp>
#include <zeusbase/Remote/AbstractRemoteObject.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/ServerSocket.h>
#include <zeusbase/System/Thread.h>
#include <zeusbase/System/SingleLinkedList.hpp>

BEGIN_NAMESPACE_Zeus

class TCommPipe_Skel;

/*****************************************************************************/
/*! Die Communication Pipe implementiert eine FIFO Pipe mit zusaetzlichen
    Features wie das Voranstellen einer Urgend-Message.
*/
/*****************************************************************************/
class TCommPipe : public TAbstractRemoteObject, public IOwnCommPipe
{
  public :
    TCommPipe(TString name);
    TCommPipe(unsigned long priority, TString name, TString ownername);

    //Interface ICommPipe
    virtual unsigned long MQUALIFIER sendMessage(IMessage* msg, unsigned long long inittime = 0);
    virtual unsigned long MQUALIFIER getPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getCellName(IString* name) const;
    virtual unsigned long MQUALIFIER getPriority() const;
    virtual bool MQUALIFIER isLocked() const;
    virtual bool MQUALIFIER hasRemoteAccess() const;

    //Methods of IOwnCommPipe
    virtual void MQUALIFIER lock();
    virtual void MQUALIFIER unlock();
    virtual void MQUALIFIER flush();
    virtual unsigned long MQUALIFIER getMessage(IMessage** msg, unsigned long long* proctime = NULL);
    virtual unsigned long MQUALIFIER peekMessage(IMessage** msg, unsigned long long* proctime= NULL);
    virtual bool MQUALIFIER isEmpty() const;
    virtual long MQUALIFIER getCount() const;

    //Methods of IRemoteObject
    REMOTE_OBJECT_DECL
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    virtual ~TCommPipe();
  
  private:
    ///Lock der FIFO
    TCriticalSection* m_pLock;
    ///FIFO Pioritate
    unsigned long m_ulPriority;
    ///Sperrflag. Ist das Flag gesetzt, kann keine Message gelesen oder geschrieben werden.
    bool m_bLockFlag;
    ///Groesse der FIFO
    long m_lSize;
    ///Name of the pipe
    TString m_strName;
    ///Name of the cell
    TString m_strCellName;

    //------------------------------------------------------------------------
    /*************************************************************************/
    /*! Der PipeContainer beinhaltet eine PipeMessage vom Typ ISTUnknown.
        Die PipeContainer bilden untereinander eine Single Linked List
    */
    /*************************************************************************/
    class TPipeContainer
    {
      public :
        /*********************************************************************/
        /*! Erstellen des Containers.
            \param msg : Datenobjekt
        */
        /*********************************************************************/
        TPipeContainer(IMessage* msg, unsigned long long inittime)
        {
          m_pData = msg;
          m_pPrevContainer = NULL;
          if (m_pData != NULL)
          {
            m_pData->addRef();
          } 
          m_ulTime = TTime::getTickCount() - inittime;
        }
        //---------------------------------------------------------------------

        /*********************************************************************/
        /*! Zerstoeren des Containers. Das Datenobjekt wird freigegeben
        */
        /*********************************************************************/
        virtual ~TPipeContainer()
        {
          if (m_pData != NULL)
          {
            m_pData->release();
          }
        }
        //---------------------------------------------------------------------

        /*********************************************************************/
        /*! Ermitteln des Datenobjekts. Die Referenz wird dadurch automatisch
            alloziert
            \param data : Rueckgabeparameter des Datenobjektsr
            \retval INT_ERR_INVALID_PARAMETER : Fehlerhafter Rueckgabeparameter
            \retval INT_ERR_REQUESTFAILED : Das Datenobjekt ist NULL
            \retval INT_NOERROR : Fehlerfreie Rueckgabe des Datenobjekts
        */
        /*********************************************************************/
        unsigned long getData(IMessage** data)
        {
          unsigned long ulRetval = RET_REQUEST_FAILED;
          
          if (data == NULL)
          {
            ulRetval = RET_INVALID_PARAMETER;
          }
          else
          {
            *data = m_pData;
            if (m_pData != NULL)
            {
              m_pData->addRef();
              ulRetval =  RET_NOERROR;
            }
            else
            {
              ulRetval = RET_REQUEST_FAILED;
            }
          }
          return ulRetval;
        }
        //---------------------------------------------------------------------

        /*********************************************************************/
        /*! Ermitteln des Vorgaenger-Containers
            \return m_pPrevContainer
        */
        /*********************************************************************/
        TPipeContainer* getPrev() { return m_pPrevContainer; }
        //---------------------------------------------------------------------

        /*********************************************************************/
        /*! Setzen des Vorgaenger-Containers
            \param  prev : Vorgaenger-Container
        */
        /*********************************************************************/
        void setPrev(TPipeContainer* prev) { m_pPrevContainer = prev; }
        //---------------------------------------------------------------------

        unsigned long long getTime()
        {
          return TTime::getTickCount() - m_ulTime;
        }

      private:
        ///Referenz auf das Datenobjekt
        IMessage* m_pData;
        ///Referenz auf den Vorgaenger- Container
        TPipeContainer* m_pPrevContainer;
        ///Zeitreferenz
        unsigned long long m_ulTime;

    };

    ///Das Kopfelement der FIFO
    TPipeContainer* m_pHeadElement;
    ///Das Ende der FIFO
    TPipeContainer* m_pTailElement;
};

END_NAMESPACE_Zeus

//---------------------------------------------------------------------------
#endif
