/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CommPipe_Skel                                                 *
 * Package : CellModul                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:16:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommPipe_Skel.cpp,v $
 *   Revision 1.4  2005/11/25 09:16:56  bhadorn
 *   Using readArray to get messages from the socket. Safe reading.
 *
 *   Revision 1.3  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.2  2005/11/10 11:00:46  bhadorn
 *   Apply CCTP changes of response data type. New is TByteArray
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 
#include <zeusbase/CellModel/CommPipe_Skel.h>
#include <zeusbase/CellModel/CommPipe.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/ZObjectFactory.h>
#include <zeusbase/System/Protocols/CCTPRequest.h>
#include <zeusbase/System/Protocols/CCTPResponse.h>
     
/***************************************************************************/
/*! Creats a skeleton for the communication pipe
*/
/***************************************************************************/
TCommPipe_Skel::TCommPipe_Skel(TCommPipe* instance) : TAbstractSkel()
{
  m_pInstance = instance; /*No addRef needed*/
}

/***************************************************************************/
/*! Destroys the communication pipe skeleton
*/
/***************************************************************************/
TCommPipe_Skel::~TCommPipe_Skel()
{
  m_pInstance = NULL;
}

/***************************************************************************/
/*! Not used for this skeleton
*/
/***************************************************************************/
// unsigned long TCommPipe_Skel::dispatchCommand(IRemoteMethodCall* method, IRemoteMethodCall** methodreturn)
// { 
//   return RET_METHOD_NOT_IMPL; 
// }
SKEL_DISPATCH_START(TCommPipe_Skel)
  //------------------------------------------------------------------------
  // Dispatching of isLocked
  SKEL_DISP_SWITCH(L"isLocked")
    bool bLocked = m_pInstance->isLocked();
    pVarRetval->setLong(bLocked);
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of getPiority
  SKEL_DISP_SWITCH(L"getPriority")
    unsigned long ulValue = m_pInstance->getPriority();
    pVarRetval->setLong(ulValue);
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of getCellName
  SKEL_DISP_SWITCH(L"getCellName")
    TString strName;
    ulRetval = m_pInstance->getCellName(&strName);
    if (ulRetval == RET_NOERROR)
    {
      TZVariant* pVarName = new TZVariant(L"name", strName);
      pReturnCall->addParameter(pVarName);
      pVarName->release();
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END

  //------------------------------------------------------------------------
  // Dispatching of getPipeName
  SKEL_DISP_SWITCH(L"getPipeName")
    TString strName;
    ulRetval = m_pInstance->getPipeName(&strName);
    if (ulRetval == RET_NOERROR)
    {
      TZVariant* pVarName = new TZVariant(L"name", strName);
      pReturnCall->addParameter(pVarName);
      pVarName->release();
    }
    pVarRetval->setLong(ulRetval);
  SKEL_DISP_SWITCH_END
  
SKEL_DISPATCH_END

/***************************************************************************/
/*! The dispatch2Method does the requests and responses of
    commands sent by clients. This method takes directly the protocol 
    objects. This is a faster way as the common remote method invocation.
    \param request: Request of the client
    \param response : Respone to the client
    \retval RET_NOERROR : dispatching performed (may return error to the client)
    \retval RET_REQUEST_FAILED : No dispatching was possible
*/
/***************************************************************************/
void TCommPipe_Skel::dispatch2(const TCCTPRequest& request, TCCTPResponse& response)
{
  TString strResponseData;
  unsigned long ulResponseStatus;
  if (m_pInstance != NULL)
  {    
    //------------------------------------------------------------------
    //Create a message object an send it to the servant
    IMessage* pMessage = NULL;
    unsigned long ulRetval = TZObjectFactory::getInstance().createObjectFromStream(&request.getData(), (ISerializable**)&pMessage);
    
    switch(ulRetval)
    {
      case RET_NOERROR :
      {
        if (m_pInstance->sendMessage(pMessage,0) == RET_NOERROR)
        {
          ulResponseStatus = CCTP_STATUS_OK;
        }
        else
        {
          ulResponseStatus = CCTP_STATUS_BADREQUEST;
          strResponseData = L"Pipe is locked. Sending failed.";
        }
        pMessage->release();
      }
      break;
      
      case RET_UNREGISTERED_CLASS : 
      {
        ulResponseStatus = CCTP_STATUS_BADREQUEST;
        strResponseData = L"Unknown class";
      }
      break;
      
      case RET_INVALID_DATA : 
      {
        ulResponseStatus = CCTP_STATUS_BADREQUEST;
        strResponseData = L"Could not parse Stream";
      }
      break;
      
      case RET_REQUEST_FAILED :
      {
        ulResponseStatus = CCTP_STATUS_BADREQUEST;
        strResponseData = L"Error occured deserializing the object";
      }
      break;   
      
      default:
      {
        ulResponseStatus = CCTP_STATUS_INTERNALSERVERERROR;
        strResponseData = L"Error creating object";
      }
      break;   
    }  
  }
  else
  {
    ulResponseStatus = CCTP_STATUS_INTERNALSERVERERROR;
    strResponseData = L"Internal error inside skeleton.";
  }
  
  response.setState(ulResponseStatus);
  if (ulResponseStatus != CCTP_STATUS_OK)
  {
    TByteArray aResponseData = TString::transcode(strResponseData);
    response.setData(aResponseData);
  }
  ZLOGGER.println(LOGMODE_INFO, "Sending response to client: [%d] %s", ulResponseStatus, strResponseData.c_str());
}

  
/***************************************************************************/
/*! See TAbstractSkel
*/
/***************************************************************************/
void TCommPipe_Skel::startNewHandler(TSocket* client)
{
  TPipeHandlerThread* pClientThread = new TPipeHandlerThread(this, client);
  pClientThread->start();
  m_lstClients.add(pClientThread);
}

/***************************************************************************/
/* Creates a client handler thread
*/
/***************************************************************************/
TCommPipe_Skel::TPipeHandlerThread::TPipeHandlerThread(TCommPipe_Skel* parent, TSocket* socket) : TThread()
{
  m_pParent = parent;
  m_pClientSocket = socket;
  if (m_pClientSocket != NULL)
  {
    m_pClientSocket->addRef();
  }
}

/***************************************************************************/
/* Destroys the client handler thread
*/
/***************************************************************************/
TCommPipe_Skel::TPipeHandlerThread::~TPipeHandlerThread()
{
  if (m_pClientSocket != NULL)
  {
    m_pClientSocket->disconnect();
    m_pClientSocket->release();
    m_pClientSocket = NULL;
  }
}
    
/***************************************************************************/
/* Kills the thread
*/
/***************************************************************************/
void TCommPipe_Skel::TPipeHandlerThread::kill()
{
  if (!this->isDead())
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Try to stop client handler thread...");
    ZLOGGER.indent();
    m_bExit = true;
    TThread::kill();
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "...done");
  }
}

/***************************************************************************/
/* Execute code of the thread
*/
/***************************************************************************/
void TCommPipe_Skel::TPipeHandlerThread::execute()
{
  if (m_pClientSocket != NULL && m_pParent != NULL)
  {
    m_bExit = false;
    long lQuitLen = TString(REMOTE_QUIT).getSize() + 1;
    m_pClientSocket->setBlockable(false);
    while(!m_bExit)
    {
      TCCTPRequest Request;
      TCCTPResponse Response;
      
      TByteArray aData;
      m_pClientSocket->readArray(&aData);
      
      if (aData.getCount() == lQuitLen)
      {
        TString strQuit = TString::transcode(aData);
        if (strQuit == REMOTE_QUIT)
        {
          ZLOGGER.printfln(LOGMODE_INFO, "Skel: Received remote quit signal.");
          m_bExit = true;
        }
      }
      
      if (aData.getCount() > 0)
      {  
        ZLOGGER.println(LOGMODE_DEBUG, "Data received [Skel: %d Bytes]", aData.getCount());
        aData.add('\0');
        if (Request.parse(aData) == RET_NOERROR)
        {
          switch(Request.getObjectType())
          {
            //Direct dispatching
            case INTERFACE_ICommPipe :
              m_pParent->dispatch2(Request, Response);
            break;
            
            //Remote method call dispatching
            default:
              m_pParent->dispatch1(Request, Response);
            break;
          }
        }
        else
        {
          Response.setState(CCTP_STATUS_BADREQUEST);
          TByteArray aData = TString::transcode(L"Could not parse request.");
          Response.setData(aData);
          ZLOGGER.println(LOGMODE_INFO, "CommPipe_Skel: Could not parse request");
        }
          
        //Write response
        TByteArray aOut;
        Response.toStream(&aOut);
        m_pClientSocket->writeArray(&aOut); 
      }
      TTime::usleep(1000);
    }
    m_pClientSocket->disconnect();
    m_pParent->setCleanFlag();
  }  
}
