/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CommPipe_Skel                                                 *
 * Package : CellModul                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:16:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommPipe_Skel.h,v $
 *   Revision 1.3  2005/11/25 09:16:56  bhadorn
 *   Using readArray to get messages from the socket. Safe reading.
 *
 *   Revision 1.2  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//---------------------------------------------------------------------------

#ifndef CommPipe_SkelH
#define CommPipe_SkelH 

#include <zeusbase/Remote/AbstractSkel.h>


BEGIN_NAMESPACE_Zeus

class TCommPipe;
class TCCTPRequest;
class TCCTPResponse;

/***************************************************************************/
/*! Skeleton of a remote pipe. Implements multiple access structure.
*/
/***************************************************************************/
class TCommPipe_Skel : public TAbstractSkel
{
  public :
    TCommPipe_Skel(TCommPipe* instance);
    void dispatch2(const TCCTPRequest& request, TCCTPResponse& response);
  
  protected:
    virtual ~TCommPipe_Skel();
    
    virtual unsigned long dispatchCommand(IRemoteMethodCall* method, IRemoteMethodCall** methodreturn);
    virtual void startNewHandler(TSocket* client);
  
  private: 
    ///Instance to delegate method calls
    TCommPipe* m_pInstance;
   
    /**********************************************************************/
    /* This inner class manages the incomming requests of clients and
       calls the parenr dispatching method
    */
    /**********************************************************************/
    class TPipeHandlerThread : public TThread
    {
      public :
        TPipeHandlerThread(TCommPipe_Skel* parent, TSocket* socket);
    
        virtual void kill();
        virtual void execute();
      
      protected:  
        virtual ~TPipeHandlerThread();
        
      private:
        TCommPipe_Skel* m_pParent;
        TSocket* m_pClientSocket;
        bool m_bExit;
    };
};

END_NAMESPACE_Zeus


#endif
