/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CommPipe_Stub                                                 *
 * Package : CellModul                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:22:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommPipe_Stub.h,v $
 *   Revision 1.2  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef CommPipe_StubH 
#define CommPipe_StubH

#include <zeusbase/CellModel/Interfaces/ICommPipe.hpp>
#include <zeusbase/Remote/AbstractStub.h>
#include <zeusbase/System/Time.h>


BEGIN_NAMESPACE_Zeus

class TSocket;
class TCriticalSection;

/*****************************************************************************/
/*! Implements the Communication Pipe Interface. All calls are delegated
    over a network to a remote communication pipe.
*/
/*****************************************************************************/
class TCommPipe_Stub : public TAbstractStub, public ICommPipe
{
  public:
    TCommPipe_Stub();
    
    //Methods of TAbstractStub
    virtual unsigned long MQUALIFIER connect(const IString* address, unsigned long port);
        
    //Interface ICommPipe
    virtual unsigned long MQUALIFIER sendMessage(IMessage* msg, unsigned long long inittime = 0);
    virtual unsigned long MQUALIFIER getPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getCellName(IString* name) const;
    virtual unsigned long MQUALIFIER getPriority() const;
    virtual bool MQUALIFIER isLocked() const;
    virtual bool MQUALIFIER hasRemoteAccess() const;

    //Methods of Abstract Stub
    STUB_CLASS_DECL(TCommPipe_Stub, L"TCommPipe_Stub");
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    virtual ~TCommPipe_Stub();
    void lock();
    void unlock();
    
  private:
    ///Lock to protect object
    TCriticalSection* m_pLock;
    ///Lock flag 
    bool m_bLocked;
    
    
    unsigned long sendMessageAsBinary(IMessage* msg, unsigned long long inittime);
};

END_NAMESPACE_Zeus

#endif
