/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : GrowthCalc                                                    *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GrowthCalc.cpp,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/CellModel/GrowthCalc.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a growth calculator
*/
/***************************************************************************/
TGrowthCalc::TGrowthCalc() :TZObject()
{
  m_dBoundary = 20;
  m_dReactionLimit = 10;
  m_ulJobCount = 0;
  m_lJobDiff = 0;
  m_bExceeded = false;
}

/***************************************************************************/
/*! destroys the growth calculator
*/
/***************************************************************************/
TGrowthCalc::~TGrowthCalc()
{

}
    
/***************************************************************************/
/*! sets the max load
*/
/***************************************************************************/
void TGrowthCalc::setBoundary(double value)
{
  m_dBoundary = value;
  TString strOut = TString(L"Set Boundary =") + m_dBoundary;
  ZLOGGER.printfln(LOGMODE_INFO, strOut.c_str());
}

/***************************************************************************/
/*! sets the reaction limit
*/
/***************************************************************************/
void TGrowthCalc::setReactionLimit(double value)
{
  m_dReactionLimit = value;
  TString strOut = TString(L"Set Reaction Limit =") + m_dReactionLimit;
  ZLOGGER.printfln(LOGMODE_INFO, strOut.c_str());
}

/***************************************************************************/
/*! Adds a new set of data.
*/
/***************************************************************************/
void TGrowthCalc::addData(double /*processtime*/, unsigned long jobs)
{  
  long lJobDiff_N = jobs - m_ulJobCount;
  long lCDiff = lJobDiff_N - m_lJobDiff;
  
  if ((lCDiff >=0 && lJobDiff_N >= 0 && jobs > m_dReactionLimit) || (jobs > m_dBoundary))
  {
    m_bExceeded = true;
  }
  else
  {
    m_bExceeded = false;
  }
  m_ulJobCount = jobs;
  m_lJobDiff = lJobDiff_N;
}

/***************************************************************************/
/*! resets the calculator
*/
/***************************************************************************/
void TGrowthCalc::reset()
{
  m_ulJobCount = 0;
  m_lJobDiff = 0;
}

/***************************************************************************/
/*! Checks the load is exceeded
    \retval true : limit reached.
    \retval false : limit not reached
*/
/***************************************************************************/
bool TGrowthCalc::isExceeded()
{
  return m_bExceeded;
}
