/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ICellComm                                                     *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ICellComm.hpp,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ICellCommHPP
#define ICellCommHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/Messaging/Interfaces/IMessage.hpp>

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! Das ICellComm interface ist die interne Schnittstelle des Zellenmantel. Das
    Interface wird ausschliesslich vom Zellkern verwendet, um Informationen und
    Jobs zu ermitteln.
*/
/******************************************************************************/
class ICellComm : public IZUnknown
{
  public:
    /*************************************************************************/
    /*! Ermitteln einer eingehenden Meldung.
        \param receiver : Receiver Cell Reference is used as a handle 
                          (ICellKernel Reference must be given)
        \param msg : Rueckgabeparameter der Meldung
        \param proctime : Rueckgabewert der Verarbeitungszeit
        \retval INT_ERR_INVALID_PARAMETER : Fehlerhafter Rueckgabeparameter
        \retval INT_ERR_REQESTFAILED : Keine Meldung vorhanden
        \retval INT_NOERROR : Fehlerfreie Rueckgabe einer Meldung
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER getInputMessage(IZUnknown* receiver, IMessage** msg, unsigned long long* proctime) = 0;

    /*************************************************************************/
    /*! Senden einer Meldung an eine andere Zelle
        \param message : Referenz auf eine Meldung
        \param inittime : Initial time
        \retval INT_NOERROR : Fehlerfreies absetzen der Meldung
        \retval INT_ERR_INVALID_PARAMETER : Ungueltige Message Referenz
        \retval INT_ERR_REQUESTFAILED : Die Zielzelle wurde nicht gefunden
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER sendMessage(IMessage* message, unsigned long long inittime = 0)=0;

    /*************************************************************************/
    /*! Senden einer Kontroll-Meldung an das eigene System
        \param message : Referenz auf eine Meldung
        \param inittime : Initial time
        \retval INT_NOERROR : Fehlerfreies absetzen der Meldung
        \retval INT_ERR_INVALID_PARAMETER : Ungueltige Message Referenz
        \retval INT_ERR_REQUESTFAILED : Die Zielzelle wurde nicht gefunden
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER sendControlMessage(IMessage* message, 
                                                        unsigned long long inittime = 0)=0;
    
    /*************************************************************************/
    /*! Ermitteln der Anzahl Meldungen dieser Zelle. Es wird die Summe ueber alle
        InputPipes gebildet.
        \return Anzahl zuverarbeitende Meldungen
    */
    /*************************************************************************/
    virtual long MQUALIFIER getJobCount() = 0;
    
    /*************************************************************************/
    /*! Flushes all pipes. The jobs are lost after this call.
    */
    /*************************************************************************/
    virtual void MQUALIFIER flushAllPipes() = 0;
    
    /*************************************************************************/
    /*! Update the list of possible output pipes. The pipe list will be 
        renewed using the registry informations
    */
    /*************************************************************************/
    virtual void MQUALIFIER updatePipes() = 0;

};

END_NAMESPACE_Zeus

#endif
