/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ICellControl                                                  *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ICellControl.hpp,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef ICellControlHPP
#define ICellControlHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/Messaging/Interfaces/IMessage.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines the cell control interface. Its used to manage
    the cell structure behavoirs such as cloning, killing or checking the
    finger prints
*/
/***************************************************************************/
class ICellControl : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! This method clones a cell
        \retval RET_NOERROR : Cell has been cloned
        \retval RET_REQUEST_FAILED : Could not clone the cell
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER clone()=0;


    /***********************************************************************/
    /*! This method controls the live power of a cell system and
        kills cells if the life power is low
        \param adding : Amount of life power to add
        \param removing : Amout of life power to remove
        \retval RET_NOERROR : System has been controled
        \retval RET_REQUEST_FAILED : Could not control system (only one cell left)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER controlAndKill(double adding, double removing)=0;

    /***********************************************************************/
    /*! This method returns the number of cells
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getCellCount()=0;
    
    /*************************************************************************/
    /*! Senden einer Meldung an die zu ueberwachende Zelle (Zellensystem)
        \param message : Referenz auf eine Meldung
        \param inittime : Time
        \retval INT_NOERROR : Fehlerfreies absetzen der Meldung
        \retval INT_ERR_INVALID_PARAMETER : Ungueltige Message Referenz
        \retval INT_ERR_REQUESTFAILED : Die Zielzelle wurde nicht gefunden
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER sendControlMessage(IMessage* message, unsigned long long inittime = 0)=0;
    
    virtual unsigned long MQUALIFIER updatePipes()=0;
    
    
};

END_NAMESPACE_Zeus

#endif
