/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ICellKernel                                                     *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ICellKernel.hpp,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef ICellKernelHPP
#define ICellKernelHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/CellModel/Interfaces/ICellComm.hpp>

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! The Cell Kernel Interface defines a possibility to access a cell kernel.
    The implementation of this Interface can be out sourced into shared libs
*/
/******************************************************************************/
class ICellKernel : public IZUnknown
{
  public:
    /**************************************************************************/
    /*! Initializes the cell kernel. The ICellComm Interface is needed to 
        communicate with the sub layer. This layer is implemented in the
        same thread
        \param cell_interface : Cell interface
    */
    /**************************************************************************/
    virtual void MQUALIFIER initialize(ICellComm* cell_interface)=0;

    /**************************************************************************/
    /*! Uninitializes the cell kernel.
    */
    /**************************************************************************/
    virtual void MQUALIFIER uninitialize()=0;

    /**************************************************************************/
    /*! Checks if the cell kernel is initialized.
        \retval true : cell kernel is initialized
        \retval false: cell kernel is not initialized
    */
    /**************************************************************************/
    virtual bool MQUALIFIER isInitialized()=0;
    
    /**************************************************************************/
    /*! After Initialization, the cell will send a life signal to the kernel.        
    */
    /**************************************************************************/
    //virtual void MQUALIFIER sendLifeSignal()=0;

};

END_NAMESPACE_Zeus

#endif
