/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ICommPipe                                                     *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:25:23 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ICommPipe.hpp,v $
 *   Revision 1.2  2005/11/16 11:25:23  bhadorn
 *   New communication interface for pipes created for naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ICommPipeHPP
#define ICommPipeHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/Messaging/Interfaces/IMessage.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>

#define PIPENAME_CONTROL                        L"CONTROL"

#define PIPEPRIORITY_SYNCHRON                   0
#define PIPEPRIORITY_HIGH                       10
#define PIPEPRIORITY_NORMAL                     50
#define PIPEPRIORITY_LOW                        100

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! Das ICommPipe Interface wird als Zugriffsinterface zu einer FIFO Pipe
    definiert. Hier koennen asynchron Meldungen abgesetzt werden.
*/
/******************************************************************************/
class ICommPipe : public IZUnknown
{
  public:
    /*************************************************************************/
    /*! Eine Messages wird gesendet.
        \param msg : Referenz auf eine Message
        \param inittime : Initialzeit der Message
        \retval RET_NOERROR : Message has been sent
        \retval RET_REQUEST_FAILED : Could not send message of any reason
        \retval RET_REMOTECALL_ERROR : Could not send message to the remote system
        \retval RET_INTERNAL_ERROR : Could not send message because of internal
                   error. In this case the pipe should be removed.
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER sendMessage(IMessage* msg, unsigned long long inittime = 0)=0;

    /*************************************************************************/
    /*! Ermitteln der Prioritaet
        \return Prioritaet
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER getPriority() const=0;

    /*************************************************************************/
    /*! Gets the Name of the pipe
        \param name : Return parameter of the name
        \retval RET_NOERROR : Name has been returned
        \retval RET_INVALID_PARAMETER : Invalid parameter name received.
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER getPipeName(IString* name) const=0;

    /*************************************************************************/
    /*! Gets the Name of the owner cell
        \param name : Return parameter of the name
        \retval RET_NOERROR : Name has been returned
        \retval RET_INVALID_PARAMETER : Invalid parameter name received.
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER getCellName(IString* name) const=0;
    
    /*************************************************************************/
    /*! Checks if the pipe is open or locked
        \retval false : open and ready for communication
        \retval true : locked
    */
    /*************************************************************************/
    virtual bool MQUALIFIER isLocked() const=0;

    /*************************************************************************/
    /*! Checks if the pipe has a remote access.
        \retval true : Remote access available
        \retval false: No remote access
    */
    /*************************************************************************/
    virtual bool MQUALIFIER hasRemoteAccess() const=0;
};

END_NAMESPACE_Zeus

#endif
