/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IOwnCommPipe                                                  *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:25:23 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IOwnCommPipe.hpp,v $
 *   Revision 1.2  2005/11/16 11:25:23  bhadorn
 *   New communication interface for pipes created for naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IOwnCommPipeHPP
#define IOwnCommPipeHPP

#include <zeusbase/CellModel/Interfaces/ICommPipe.hpp>

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! The IOwnCommPipe interface defines a pipe, witch is owned by some one.
    Through this interface a message can be read. The sender interface is
    defined by ICommPipe, witch defines methods to send a message. This interface
    inherits the ICommInterface, so the owner can do both, sending and receiving
    through this pipe.
*/
/******************************************************************************/
class IOwnCommPipe : public ICommPipe
{
  public:
    /*************************************************************************/
    /*! This method locks the pipe. Throu a locked pipe no message can be 
        sent or received
    */
    /*************************************************************************/
    virtual void MQUALIFIER lock()=0;
    
    /*************************************************************************/
    /*! This method unlocks the pipe. This method must be called before a 
        communication starts using this pipe.
    */
    /*************************************************************************/
    virtual void MQUALIFIER unlock()=0;
    
    /*************************************************************************/
    /*! This flushes the pipe. All messages and their objects will be 
        released. 
    */
    /*************************************************************************/
    virtual void MQUALIFIER flush()=0;
    
    /*************************************************************************/
    /*! Returns the top message object from the pipe. The message will be 
        removed from the pipe.
        \param msg : Return value of the message
        \param proctime : Time since the message has been sent or posted to
               this pipe.
        \retval RET_NOERROR : Message has been successfully returned
        \retval RET_INVALID_PARAMETER : The msg parameter is invalid
        \retval RET_REQUEST_FAILED : No message available or pipe is locked
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER getMessage(IMessage** msg, unsigned long long* proctime = NULL)=0;
    
    /*************************************************************************/
    /*! Returns the top message object from the pipe. The message wont be 
        removed from the pipe. To remove the message call getMessage().
        \param msg : Return value of the message
        \param proctime : Time since the message has been sent or posted to
               this pipe.
        \retval RET_NOERROR : Message has been successfully returned
        \retval RET_INVALID_PARAMETER : The msg parameter is invalid
        \retval RET_REQUEST_FAILED : No message available or pipe is locked
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER peekMessage(IMessage** msg, unsigned long long* proctime= NULL)=0;
    
    /*************************************************************************/
    /*! Checkes if the pipe is empty.
        \retval true : Pipe is empty
        \retval false: pipe is not empty
    */
    /*************************************************************************/
    virtual bool MQUALIFIER isEmpty() const=0;
    
    /*************************************************************************/
    /*! Returns the number of posted messages, witch a still remaining in the 
        pipe.
        \return count
    */
    /*************************************************************************/
    virtual long MQUALIFIER getCount() const=0;
};

END_NAMESPACE_Zeus

#endif
 
