/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : LocalPipeRegistry                                             *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:22:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LocalPipeRegistry.h,v $
 *   Revision 1.2  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#ifndef LocalPipeRegistryH
#define LocalPipeRegistryH

#include <zeusbase/CellModel/Interfaces/ILocalPipeRegistry.hpp>
#include <zeusbase/System/XObject.h>
#include <zeusbase/System/StringMap.hpp>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/XObjectFactory.h>

BEGIN_NAMESPACE_Zeus

class TCriticalSection;
class TCommPipe;

/***************************************************************************/
/*! The local pipe registry manages all pipes of a process and eventually
    creates pipe connections to remote systems
*/
/***************************************************************************/
class TLocalPipeRegistry : public TXObject, public ILocalPipeRegistry
{
  public:
    TLocalPipeRegistry(IXMLNode* node);
    
    //Method of ILocalPipeRegistry
    virtual unsigned long MQUALIFIER isConnected() const;
    virtual unsigned long MQUALIFIER getPort() const;
    virtual unsigned long MQUALIFIER getAddress(IString* addr) const;
    virtual unsigned long MQUALIFIER createPipe(const IString* cellname, 
                                                const IString* pipename, 
                                                unsigned long priority, 
                                                bool create_remote_access, 
                                                IOwnCommPipe** pipe);
    virtual unsigned long MQUALIFIER registerPipe(ICommPipe* pipe);
    virtual unsigned long MQUALIFIER unregisterPipe(ICommPipe* pipe);
    virtual unsigned long MQUALIFIER getPipe(const IString* cellname, const IString* pipename, ICommPipe** pipe);
    virtual unsigned long MQUALIFIER getPipes(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list);
    virtual unsigned long MQUALIFIER updatePipes(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list);

    //Methods of IXObject
    virtual bool MQUALIFIER freeze();
    virtual bool MQUALIFIER unfreeze();
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
    REG_SUB_BEGIN(TLocalPipeRegistry)
    REG_SUB_ADD(TLocalPipeRegistry, L"TLocalPipeRegistry");
    REG_SUB_END

  protected:    
    virtual ~TLocalPipeRegistry();
  
  private:
    ///Flag is the connection to the naming service is ok
    bool m_bConnected;
    ///IP address of the naming service
    TString m_strAddress;
    ///TCP port of the naming service
    unsigned long m_ulPort;
    ///Map of pipe references
    TStringMap<ICommPipe*> m_mapPipes;
    ///Data lock 
    TCriticalSection* m_pDataLock;

    void removeFromNaming(ICommPipe* pipe, const TString& objname);
    unsigned long getPipesFromRemote(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list, long max_count);
    unsigned long getPipesFromLocal(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list);    
    static TString getPipeObjectName(ICommPipe* pipe);
    unsigned long addPipe(const TString& key, ICommPipe* pipe);
    unsigned long removePipe(const TString& key, ICommPipe* pipe);
};

END_NAMESPACE_Zeus

#endif
