/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : OutputPipeList                                                   *
 * Package : CellModel                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: OutputPipeList.cpp,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/CellModel/OutputPipeList.h>
#include <zeusbase/System/Time.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new Output Pipe List
*/
/***************************************************************************/
TOutputPipeList::TOutputPipeList(TString cellname, TString pipename, bool added_from_reg)
{
  m_strCellName = cellname;
  m_strPipeName = pipename;
  m_bAddedFromReg = added_from_reg;
}


/***************************************************************************/
/*! Destroys the Output Pipe List
*/
/***************************************************************************/
TOutputPipeList::~TOutputPipeList()
{
  releaseInstances();
}
    

/***************************************************************************/
/*! Returns any pipe out of this list
    \param pipe : return parameter of a pipe
    \retval RET_NOERROR : Pipe returned
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_REQUEST_FAILED : No pipe returned
*/
/***************************************************************************/
unsigned long TOutputPipeList::getAnyPipe(ICommPipe** pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (pipe == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    long lIndex = TTime::random(0, this->getCount()-1);
    *pipe = this->getItem(lIndex);
    if (*pipe != NULL)
    {
      (*pipe)->addRef();
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! Checks if the pipes has been added from the registry
    \retval true : Pipes were added from the registry
    \retval false: Other pipes
*/
/***************************************************************************/
bool TOutputPipeList::isFromRegistry()
{
  return m_bAddedFromReg;
}

/***************************************************************************/
/*! Return all locked pipes of a system
    \param list : Return parameter of all locked pipes
    \retval RET_NOERROR : Lockup was successful 
    \retval RET_REQUEST_FAILED : No locked pipes found
    \retval RET_INVALID_PARAMETER : Invalid parameter received
*/
/***************************************************************************/
unsigned long TOutputPipeList::getAllLockedPipes(IList<ICommPipe*>* list)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (list == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    IListIterator<ICommPipe*>* pIterator = this->getIterator();
    ICommPipe* pPipe = pIterator->getNextItem();
    while (pPipe != NULL)
    {
      if (pPipe->isLocked())
      {
        list->add(pPipe);
        pPipe->addRef();
        ulRetval = RET_NOERROR;
      }
      pPipe = pIterator->getNextItem();
    }
    this->releaseIterator(pIterator);
  }
  return ulRetval;
}

/***************************************************************************/
/*! Releases all instances of the list
*/
/***************************************************************************/
void TOutputPipeList::releaseInstances()
{
  while(this->getCount() > 0)
  {
    this->getItem(0)->release();
    this->deleteItem(0);
  }
}


/***************************************************************************/
/*! Returns the cell name
*/
/***************************************************************************/
TString TOutputPipeList::getCellName()
{
  return m_strCellName;
}

/***************************************************************************/
/*! returns the pipe name
*/
/***************************************************************************/
TString TOutputPipeList::getPipeName()
{
  return m_strPipeName;
}


