/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : OutputPipeManager                                             *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: OutputPipeManager.h,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef OutputPipeManagerH
#define OutputPipeManagerH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/CellModel/Interfaces/ICommPipe.hpp>
#include <zeusbase/CellModel/Interfaces/ILocalPipeRegistry.hpp>

BEGIN_NAMESPACE_Zeus

class TOutputPipeList;

/***************************************************************************/
/*! This class handles all out put pipes for a cell module. It takes care 
    of its management, locking, removing and refreshing of this pipe 
    references
*/
/***************************************************************************/
class TOutputPipeManager : public TZObject
{
  public:
    TOutputPipeManager();
    
    void addPipe(ICommPipe* pipe);
    unsigned long findPipe(IString* cellname, IString* pipename, ICommPipe** pipe);
    unsigned long refreshPipes();
    void setRegistry(ILocalPipeRegistry* registry);
    void removeAllOutputPipes();
    void removeLockedOutputPipes();
  
  protected:  
    virtual ~TOutputPipeManager();
  
  private:
    ///Outgoing Message Pipes (References to other pipes in the CCM)
    TSingleLinkedList<TOutputPipeList*> m_lstOutputPipes;
    ///Lock for output message pipe access
    TCriticalSection* m_pOutputPipeLock;
    ///Lookup registry for cell pipes
    ILocalPipeRegistry* m_pRegistry;
    
    void unregisterPipes(TOutputPipeList* pipes);
    
};

END_NAMESPACE_Zeus

#endif
