/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : RelayCell                                                     *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:14:41 $                                  *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RelayCell.cpp,v $
 *   Revision 1.1  2005/11/25 09:14:41  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <zeusbase/CellModel/RelayCell.h>
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus

/****************************************************************************/
/*! Creates a cell relay
*/
/****************************************************************************/
TRelayCell::TRelayCell(IXMLNode* node) : TAbstractCell(node), m_mapNames(RRelayEntry(L"", L""))
{
  if (node != NULL)
  {
    IXMLNode* pJobNode = NULL;
    TString strName(L"Job");
    node->resetNodeIterator(&strName);
    while(node->getNextNode(&pJobNode) == RET_NOERROR)
    {
      TString strSender;
      TString strAttName = L"Sender";
      pJobNode->getAttributeValue(&strAttName, &strSender);
      if (!m_mapNames.hasItem(strSender))
      {
        TString strForward;
        TString strForwardPipe;
        strAttName = L"ForwardTo";
        pJobNode->getAttributeValue(&strAttName, &strForward);
        strAttName = L"ForwardPipe";
        pJobNode->getAttributeValue(&strAttName, &strForwardPipe);
        if (strForwardPipe == L"")
        {
          strForwardPipe = PIPENAME_STDIN;
        }
        
        m_mapNames.setItem(strSender, RRelayEntry(strForward, strForwardPipe));
        ZLOGGER.printfln(LOGMODE_DEBUG, "Relay for sender added: [%s to %s]", strSender.c_str(), strForward.c_str());
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "Sender already exists: [%s]", strSender.c_str());
      }
      pJobNode->release();
    }
  }
}

/****************************************************************************/
/*! Destroys the cell relay
*/
/****************************************************************************/
TRelayCell::~TRelayCell()
{}
 
/***************************************************************************/
/*! see ICell
*/
/***************************************************************************/
void TRelayCell::sendLifeSignal()
{
  if (m_pCommunication != NULL)
  {
    IMessage* pMessage = NULL;
    unsigned long long ulTimer = 0;
    if (m_pCommunication->getInputMessage((ICellKernel*)this, &pMessage, &ulTimer) == RET_NOERROR)
    {
      TString strSenderCellName;
      TString strSenderPipeName = PIPENAME_STDIN;
      pMessage->getSenderCellName(&strSenderCellName);
      if (m_mapNames.hasItem(strSenderCellName))
      {
        RRelayEntry& Entry =  m_mapNames.getItem(strSenderCellName);
        pMessage->setTargetCellName(&Entry.strCellName);
        pMessage->setTargetPipeName(&Entry.strPipeName);
        pMessage->setSenderPipeName(&strSenderPipeName);
        m_pCommunication->sendMessage(pMessage, ulTimer);
      }
      else
      {
        dispatchMessage(pMessage, ulTimer);
      }
      pMessage->release();
    }
  }
}
