/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IModule                                                       *
 * Package : system                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:58:11 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IModule.hpp,v $
 *   Revision 1.2  2005/12/14 08:58:11  bhadorn
 *   Declared some methods as const
 *
 *   Revision 1.1  2005/11/10 10:29:32  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IModuleHPP
#define IModuleHPP



#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/MOM/Interfaces/IZeusAPI.hpp>
#include <zeusbase/MOM/Interfaces/IModuleSession.hpp>

#define MODULEMODE_NORMAL                                        0
#define MODULEMODE_LOAD                                          1

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! The IModule Interface defines an abstract node of the MOM tree
*/
/***************************************************************************/
class IModule : public IZUnknown
{
  public:

    /**********************************************************************/
    /*! Gets the mode of a module
        \retval MODULEMODE_NORMAL : Normal mode. Module will be loaded
                                    when needed
        \retval MODULEMODE_LOAD : Loads the module imediately
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getMode()const=0;

    /**********************************************************************/
    /*! This method activates a module. 
    */
    /**********************************************************************/
    virtual void MQUALIFIER activate()=0;
    
    /**********************************************************************/
    /*! This method deactivates a module. 
    */
    /**********************************************************************/
    virtual void MQUALIFIER deactivate()=0;
    
    /**********************************************************************/
    /*! Checks if a module is activated
        \retval true : Module is activated
        \retval false : The module is not activated
    */
    /**********************************************************************/
    virtual bool MQUALIFIER isActive()const=0;

    /**********************************************************************/
    /*! Returns the API of zeus for this specific module.
        \param api : Return value of the api
        \retval RET_NOERROR : API returned
        \retval RET_INVALID_PARAMETER : Invalid paramter received
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getZeusAPI(IZeusAPI** api)=0;
    
    /**********************************************************************/
    /*! This method is used to get a module by name. Its functionality is the
        same as IXObject::getObject() but it returns only object of Type 
        IModule. For all others use getObject directly.
        \param name : Name or path of the module
        \param module : return parameter of the module
        \retval RET_NOERROR : Module found
        \retval RET_REQUEST_FAILED ; No module with this name found
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getModule(IString* name, IModule** module)=0;

    /**********************************************************************/
    /*! Returns the name of a module
        \param name : Return parameter of the name
        \retval RET_NOERROR : Name returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getName(IString* name)const=0;
    
    /**********************************************************************/
    /*! This method registeres a new active module. The active module
        must be a child of the parent, otherwise RET_REQUEST_FAILED will be
        returned.
        \param module : new active module
        \retval RET_NOERROR : No Error occured registring the active module
        \retval RET_REQUEST_FAILED : unable to register module as active 
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER registerActiveModule(IModule* module)=0; 

    /**********************************************************************/
    /*! This method unregisteres an active module. The active module
        must be a active child of the parent, otherwise RET_REQUEST_FAILED 
        will be returned.
        \param module : module to deactivate
        \retval RET_NOERROR : No Error occured registring the deactive module
        \retval RET_REQUEST_FAILED : unable to unregister module as deactive 
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER unregisterActiveModule(IModule* module)=0; 
    
    /**********************************************************************/
    /*! This method retuns the active module of a manager.
        \param module : new active module
        \retval RET_NOERROR : active module returned
        \retval RET_REQUEST_FAILED : No active module found
        \retval RET_INVALID_PARAMETER : Invalid parameter found.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getActiveModule(IModule** module)=0; 

    /**********************************************************************/
    /*! This method retuns the module session object of this module. The
        session is normally created within a library and is used as a non
        dynamic factory interface (gateway) to the library objects
        \param session : return parameter to receive the loaded session
        \retval RET_NOERROR : session returned
        \retval RET_REQUEST_FAILED : No session available
        \retval RET_INVALID_PARAMETER : Invalid parameter found.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getModuleSession(IModuleSession** session)=0; 
};

END_NAMESPACE_Zeus

#endif
