/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ServiceModulSession                                           *
 * Package : Help Implementations                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:29:32 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IModuleSession.hpp,v $
 *   Revision 1.1  2005/11/10 10:29:32  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IModuleSessionHPP 
#define IModuleSessionHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/MOM/Interfaces/IZeusAPI.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! The module session interface defines a instance of the module ressource.
    Through this instance various non xobjects can be reached, and created.
*/
/***************************************************************************/
class IModuleSession : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! This method creates various objects (inheriting IZUnknown Interface)
        of a module.
        \param interface_id : Interface ID of the Object
        \param iface : Return value of the interface
        \retval RET_NOERROR : Object created and successfully returned
        \retval RET_INVALID_PARAMETER : Invalid parameter
        \retval RET_UNKNOWN_INTERFACE : Unknown ID
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createObjectOfModule(unsigned long interface_id, IZUnknown** iface)=0;
    
    /***********************************************************************/
    /*! Initializes the Module session
        \param api : Framework communication API
    */
    /***********************************************************************/
    virtual void MQUALIFIER initialize(IZeusAPI* api)=0;
    
};

END_NAMESPACE_Zeus

#endif
