/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IZuesAPI                                                      *
 * Package : Main Interface of the framework                               *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:29:32 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IZeusAPI.hpp,v $
 *   Revision 1.1  2005/11/10 10:29:32  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef IZeusAPIHPP
#define IZeusAPIHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IXObjectFactory.hpp>
#include <zeusbase/System/Interfaces/ILibraryManager.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! The Zeus Application Programming Interface is used to access the
    framework functionality and to receive other framework interfaces
*/
/***************************************************************************/
class IZeusAPI : public IZUnknown
{
  public:
    /**********************************************************************/
    /*! Try to find an object in the MOM System
        \param path : Name of the object to be returned
        \param obj : Return parameter of the object
        \retval RET_NOERROR : Object found and returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No object or module found
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getXObject(IString* path, IXObject** obj)=0;
    
    /***********************************************************************/
    /*! This method is used to load a new Interface out of a specific module
        of the MOM system
        \param module_name : Name of the module
        \param interface_id : ID of the required interface
        \param iface : Return value of the interface found
        \retval RET_NOERROR : Interface returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the interface, either
                 the module name or the interface_id is invalid
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createObjectOfModule(IString* module_name, unsigned long interface_id, IZUnknown** iface)=0;
    
};

END_NAMESPACE_Zeus

#endif
