/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Module                                                        *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:57:02 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Module.h,v $
 *   Revision 1.2  2005/12/14 08:57:02  bhadorn
 *   Changed to new IModule Interface
 *
 *   Revision 1.1  2005/11/10 10:28:57  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ModuleH
#define ModuleH

#include <zeusbase/MOM/Interfaces/IModule.hpp>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/XLoaderObject.h>

BEGIN_NAMESPACE_Zeus

class TZeusAPI;

/***************************************************************************/
/*! The TModule class spezifies a node of the MOM Tree
*/
/***************************************************************************/
class TModule : public TXLoaderObject, public IModule
{
  public:
    TModule(IXMLNode* node);
    
    //Methods of IModule
    virtual unsigned long MQUALIFIER getMode() const;
    virtual void MQUALIFIER activate();
    virtual void MQUALIFIER deactivate();
    virtual bool MQUALIFIER isActive() const;
    virtual unsigned long MQUALIFIER getZeusAPI(IZeusAPI** api);
    virtual unsigned long MQUALIFIER getModule(IString* name, IModule** module);
    virtual unsigned long MQUALIFIER getName(IString* name) const;
    virtual unsigned long MQUALIFIER registerActiveModule(IModule* module);
    virtual unsigned long MQUALIFIER unregisterActiveModule(IModule* module);
    virtual unsigned long MQUALIFIER getActiveModule(IModule** module);
    virtual unsigned long MQUALIFIER getModuleSession(IModuleSession** session);

    //Methods of IXLoaderObject
    virtual unsigned long MQUALIFIER load();
    virtual unsigned long MQUALIFIER unload();
    
    //Methods of IXObject
    virtual bool MQUALIFIER freeze();
    virtual bool MQUALIFIER unfreeze();
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    REG_SUB_BEGIN(TModule)
    REG_SUB_ADD(TModule, L"TModule");
    REG_SUB_END
    
    ///Zeus API Object
    TZeusAPI* m_pZeusAPI;
    ///Module Session Object
    IModuleSession* m_pSession;
    
    virtual ~TModule();
          
  private:
    ///Flag if the module is active
    bool m_bActive;
    ///Active module reference
    IModule* m_pActiveModule;
    
    void activateFirstChild();

};

END_NAMESPACE_Zeus
#endif
