/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ModuleFunctions                                               *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:57:46 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ModuleFunctions.hpp,v $
 *   Revision 1.2  2005/12/14 08:57:46  bhadorn
 *   Corrected an error activating a module
 *
 *   Revision 1.1  2005/11/10 10:28:57  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ModuleFunctionsHPP
#define ModuleFunctionsHPP

#include <zeusbase/MOM/Module.h>
#include <zeusbase/MOM/Interfaces/IModule.hpp>


BEGIN_NAMESPACE_Zeus


/*************************************************************************/
/*! This template class implements some basic functionality of a module
     Its needed in the two classes TModule and TSystemManager
*/
/*************************************************************************/
template <class TModuleType> class TModuleFunctions
{
  public:
      
    /*************************************************************************/
    /*! This method helps to get the first child object, witch is a module
        and marks this module as activated. This module will register themself
        automatically.
    */
    /*************************************************************************/
    static void activateFirstChild(TModuleType* mod, bool& active_flag)
    {
      long lCount = mod->getChildCount();
      bool bFound = false;
      IXObject* pObject = NULL;
      IModule* pModule = NULL;
      for(int i = 0; i < lCount && !bFound; i++)
      {
        if (mod->getChild(i, &pObject) == RET_NOERROR)
        {
          if (pObject->askForInterface(INTERFACE_IModule, (IZUnknown**)&pModule) == RET_NOERROR)
          {
            /////////////////////
            //Secure code: The flag must be set befor calling the activate() method 
            active_flag = true;
            /////////////////////
            
            pModule->activate();
            pModule->release();
            bFound = true;
          }
          pObject->release();
        }
      }
    }
    
    /***************************************************************************/
    /*! See IModule
    */
    /***************************************************************************/
    static unsigned long getModule(TModuleType* mod, IString* name, IModule** module)
    {
      unsigned long ulRetval = RET_REQUEST_FAILED;
      IXObject* pObject = NULL;
      ulRetval = mod->getObject(name, &pObject);
      if (ulRetval == RET_NOERROR)
      {
        if (pObject->askForInterface(INTERFACE_IModule, (IZUnknown**)module) == RET_NOERROR)
        {
          ulRetval = RET_NOERROR;
        }
        else
        {
          ulRetval = RET_REQUEST_FAILED;
        }
        pObject->release();
      }
      return ulRetval;
    }
  
};


END_NAMESPACE_Zeus

#endif
