/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SystemManager                                                 *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:57:02 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SystemManager.cpp,v $
 *   Revision 1.2  2005/12/14 08:57:02  bhadorn
 *   Changed to new IModule Interface
 *
 *   Revision 1.1  2005/11/10 10:28:57  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <zeusbase/MOM/SystemManager.h>
#include <zeusbase/MOM/ModuleFunctions.hpp>
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates the module
*/
/***************************************************************************/
TSystemManager::TSystemManager(IXMLNode* node, NAMESPACE_Zeus::IXMLDocument* docu) : TXRootObject(node, docu)
{
  m_pActiveModule = NULL;
}


/***************************************************************************/
/*! Destroys the module
*/
/***************************************************************************/
TSystemManager::~TSystemManager()
{
  if (m_pActiveModule != NULL)
  {
    m_pActiveModule->release();
    m_pActiveModule = NULL;
  }
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TSystemManager::freeze()
{
  bool bRetval = TXRootObject::freeze();
  if (bRetval)
  {
    TString strName;
    this->getName(&strName);
    ZLOGGER.printfln(LOGMODE_INFO, "Freeze TSystemManager %s.", strName.c_str());
    ZLOGGER.indent();
    ZLOGGER.unindent();
  }
  return bRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TSystemManager::unfreeze()
{
  bool bRetval = TXRootObject::unfreeze();
  if (bRetval)
  {
    TString strName;
    this->getName(&strName);
    ZLOGGER.printfln(LOGMODE_INFO, "Unfreeze TSystemManager %s.", strName.c_str());
    ZLOGGER.indent();
    ZLOGGER.unindent();
  }
  return bRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::getMode() const
{
  return MODULEMODE_NORMAL;
}

/***************************************************************************/
/*! See IModule. 
    If no active module is registered and this is not a leaf module, it will
    activate the first module found in the list. This will go on until the 
    leaf is reached or a activemodule is found.
*/
/***************************************************************************/
void MQUALIFIER TSystemManager::activate()
{
  TString strName;
  this->getName(&strName);
  ZLOGGER.printfln(LOGMODE_INFO, "TSystemManager %s activated.", strName.c_str());
  //Try to activate a child module
  if (m_pActiveModule == NULL)
  {
    activateFirstChild();
  }
}

/***************************************************************************/
/*! This method helps to get the first child object, witch is a module
    and marks this module as activated. This module will register themself
    automatically.
*/
/***************************************************************************/
void TSystemManager::activateFirstChild()
{
  bool bActive = false;
  TModuleFunctions<TSystemManager>::activateFirstChild(this, bActive);
}

/***************************************************************************/
/*! NOT USED.
*/
/***************************************************************************/
void MQUALIFIER TSystemManager::deactivate()
{}


/***************************************************************************/
/*! NOT USED.
*/
/***************************************************************************/
bool MQUALIFIER TSystemManager::isActive() const
{
  return true;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::getZeusAPI(IZeusAPI** /*api*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::getModule(IString* name, IModule** module)
{
  return TModuleFunctions<TSystemManager>::getModule(this, name, module);
}
    
/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::getName(IString* name) const
{
  return TXObject::getName(name);
}

/***************************************************************************/
/*! See IModule.
    A module is registring themself as active. If this module (parent) is 
    not marked as active itself, iy will to it at this moment. This 
    will go on until the root module or a already active parent is reached.
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::registerActiveModule(IModule* module)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (module != NULL)
  {
    if (m_pActiveModule != NULL)
    {
      //The module will unregister 
      m_pActiveModule->deactivate();
      
      //////////////////////////////
      //Secure code only. In regular use this will never apply
      if (m_pActiveModule != NULL)
      {
        m_pActiveModule->release();
        m_pActiveModule = NULL;
      }
      //////////////////////////////
    }
    m_pActiveModule = module;
    m_pActiveModule->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::unregisterActiveModule(IModule* module)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (module != NULL)
  {
    if (m_pActiveModule != NULL)
    {
      m_pActiveModule->release();
      m_pActiveModule = NULL;
      
      //This module stays activated even a child is unregistring themself.
      /////////////////
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::getActiveModule(IModule** module)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (module != NULL)
  {
    if (m_pActiveModule != NULL)
    {
      ulRetval = m_pActiveModule->getActiveModule(module);
    }
    else
    {
      *module = this;
      this->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemManager::getModuleSession(IModuleSession** /*session*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  return ulRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TSystemManager);
  INTERFACE_CAST(IModule, INTERFACE_IModule);
MEMORY_MANAGER_IMPL_PARENT_END(TXRootObject);

 
