/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SystemManager                                                 *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:57:02 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SystemManager.h,v $
 *   Revision 1.2  2005/12/14 08:57:02  bhadorn
 *   Changed to new IModule Interface
 *
 *   Revision 1.1  2005/11/10 10:28:57  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SystemManagerH
#define SystemManagerH

#include <zeusbase/MOM/Interfaces/IModule.hpp>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/XRootObject.h>

BEGIN_NAMESPACE_Zeus

class TZeusAPI;

/***************************************************************************/
/*! The TSystemManager class spezifies the root node of the MOM Tree.
    This node is able to save the XML Tree and inherites therefore the
    class TXRootObject
*/
/***************************************************************************/
class TSystemManager : public TXRootObject, public IModule
{
  public:
    TSystemManager(IXMLNode* node, NAMESPACE_Zeus::IXMLDocument* docu);
    
    //Methods of IModule
    virtual unsigned long MQUALIFIER getMode() const;
    virtual void MQUALIFIER activate();
    virtual void MQUALIFIER deactivate();
    virtual bool MQUALIFIER isActive() const;
    virtual unsigned long MQUALIFIER getZeusAPI(IZeusAPI** api);
    virtual unsigned long MQUALIFIER getModule(IString* name, IModule** module);
    virtual unsigned long MQUALIFIER getName(IString* name) const;
    virtual unsigned long MQUALIFIER registerActiveModule(IModule* module);
    virtual unsigned long MQUALIFIER unregisterActiveModule(IModule* module);
    virtual unsigned long MQUALIFIER getActiveModule(IModule** module);
    virtual unsigned long MQUALIFIER getModuleSession(IModuleSession** session);

    //Methods of IXObject
    virtual bool MQUALIFIER freeze();
    virtual bool MQUALIFIER unfreeze();
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    REG_ROOT_BEGIN(TSystemManager)
    REG_ROOT_ADD(TSystemManager, L"TSystemManager");
    REG_ROOT_END
        
    virtual ~TSystemManager();
          
  private:
    ///Active module reference
    IModule* m_pActiveModule;
    
    void activateFirstChild();
};

END_NAMESPACE_Zeus
#endif
 
