/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ZeusAPI                                                       *
 * Package : Main Interface of the framework                               *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:28:57 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZeusAPI.cpp,v $
 *   Revision 1.1  2005/11/10 10:28:57  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeusbase/MOM/ZeusAPI.h>
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new Zeus API Object
*/
/***************************************************************************/
TZeusAPI::TZeusAPI(TModule* module) : TZObject()
{
  m_pModule = module;
}

/***************************************************************************/
/*! Destroys the Zeus API Object
*/
/***************************************************************************/
TZeusAPI::~TZeusAPI()
{}


/***************************************************************************/
/*! See IZeusAPI
*/
/***************************************************************************/
unsigned long MQUALIFIER TZeusAPI::getXObject(IString* path, IXObject** obj)
{
  return m_pModule->getObject(path, obj);
}

/***************************************************************************/
/*! See IZeusAPI
*/
/***************************************************************************/
unsigned long MQUALIFIER TZeusAPI::createObjectOfModule(IString* module_name, unsigned long interface_id, IZUnknown** iface)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if(module_name == NULL || iface == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strOut = TString(L"Try to get interface of module ") + TString(module_name) + L" ...";
    ZLOGGER.printf(LOGMODE_INFO, strOut.c_str());
    
    IXObject* pTragetObject = NULL;
    if (m_pModule->getObject(module_name, &pTragetObject) == RET_NOERROR)
    {
      IModule* pTragetModule = NULL;
      if (pTragetObject->askForInterface(INTERFACE_IModule, (IZUnknown**)&pTragetModule) == RET_NOERROR)
      {
        ZLOGGER.print(LOGMODE_INFO, "Module found...");
        IModuleSession* pSession = NULL;
        if (pTragetModule->getModuleSession(&pSession) == RET_NOERROR)
        {
          ulRetval = pSession->createObjectOfModule(interface_id, iface);
          pSession->release();
        }
        else
        {
          ZLOGGER.print(LOGMODE_INFO, "ModuleSession not found...");
        }
        pTragetModule->release();
      }
      else
      {
        ZLOGGER.print(LOGMODE_INFO, "Object is not a module...");
      }
      pTragetObject->release();
      
      if (ulRetval == RET_NOERROR)
      {
        ZLOGGER.println(LOGMODE_INFO, "...done");
      }
      else
      {
        ZLOGGER.println(LOGMODE_INFO, "...failed");
      }
      
      pTragetModule->release();
    }
    else
    {
      ZLOGGER.println(LOGMODE_INFO, "Module not found...Failed.");
    }
  }
  return ulRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TZeusAPI);
  INTERFACE_CAST(IZeusAPI, INTERFACE_IZeusAPI);
MEMORY_MANAGER_IMPL_END;

