/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : GAChromosone                                                  *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GAChromosone.h,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GAChromosoneH
#define GAChromosoneH

#include <zeusbase/Math/Interfaces/IGAChromosone.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ArrayList.hpp>

BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! Implementation of a chromosone 
*/
/**************************************************************************/
class TGAChromosone : public TZObject, public IGAChromosone
{
  public:
    /***********************************************************************/
    /*! This enumeration represents the kind of the crossover.
    */
    /***********************************************************************/
    enum TCrossOverKind 
    { 
      ctMultiPoint,         /*!< Multi point crossover (randomly). Takes the parameter how many crossovers has to be made*/
      ctUniform,            /*!< Takes every chance for a crossover*/
      ctInterRecombination, /*!< O1 = P1 + (P2 - P1)*a with a from -0.25 to 1.25 randomly for each gene value*/
      ctLineRecombination,  /*!< Same as inter recombination but the a is the same for all gene values*/
      ctCustom              /*!< Customized crossing*/
    };
    
    typedef unsigned long (*FCrossFunction)(IGAChromosone* parent1, IGAChromosone* parent2, TGAChromosone* new_chrom1, TGAChromosone* new_chrom2);
    
    TGAChromosone(unsigned long type, TCrossOverKind kind, FCrossFunction funct = NULL);
    
    void setMultiPointParameter(long value);
    void addGene(IGAGene* gene);
    
    //Methods of IGAChromosone
    virtual unsigned long MQUALIFIER getType();
    virtual long MQUALIFIER getGeneCount();
    virtual unsigned long MQUALIFIER getGene(long index, IGAGene** gene);
    virtual unsigned long MQUALIFIER crossOver(IGAChromosone* chrom, IGAChromosone** new_chrom1, IGAChromosone** new_chrom2);

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
  protected:
    virtual ~TGAChromosone();
    
    ///Type of the chromosone
    unsigned long m_ulType;
    ///Kind of crossover
    TCrossOverKind m_eCrossKind;
    ///List of Chromosones
    TArrayList<IGAGene*> m_lstGenes;
    ///Multipoint parameter
    long m_lMultiPointParam;
    ///Cross function for customized crossing
    FCrossFunction m_Funct;
    
  private:
    void doMultiCrossOver(IGAChromosone* parent2, TGAChromosone* chrom1, TGAChromosone* chrom2);
};

END_NAMESPACE_Zeus
 
#endif
 
