/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : GAIndividual                                                  *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GAIndividual.cpp,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Math/GAIndividual.h>

USING_NAMESPACE_Zeus

/**************************************************************************/
/*! Creates an individual
*/
/**************************************************************************/
TGAIndividual::TGAIndividual() : TZObject()
{}


/**************************************************************************/
/*! Destroys the individual
*/
/**************************************************************************/
TGAIndividual::~TGAIndividual()
{
  long lCount = m_lstChromosones.getCount();
  for(int i = 0; i < lCount; i++)
  {
    m_lstChromosones[i]->release();
  }
  m_lstChromosones.clear();
}


/**************************************************************************/
/*! See IGAIndividual
*/
/**************************************************************************/
long MQUALIFIER TGAIndividual::getChromosoneCount()
{
  return m_lstChromosones.getCount();
}

/**************************************************************************/
/*! See IGAIndividual
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAIndividual::getChromosone(long index, IGAChromosone** chromosone)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (chromosone == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *chromosone = m_lstChromosones[index];
    if (*chromosone != NULL)
    {
      (*chromosone)->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}


/**************************************************************************/
/*! See IGAIndividual
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAIndividual::addChromosone(IGAChromosone* chromosone)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (chromosone == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    m_lstChromosones.add(chromosone);
    chromosone->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}



/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TGAIndividual);
  INTERFACE_CAST(IGAIndividual, INTERFACE_IGAIndividual);
MEMORY_MANAGER_IMPL_END;
 
