/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : GALongGene                                                    *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GALongGene.hpp,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GALongGeneH
#define GALongGeneH

#include <zeusbase/Math/Interfaces/IGAGene.hpp>
#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! Implements a gene witch contains a long value
*/
/**************************************************************************/
class TGALongGene : public TZObject, public IGALongGene
{
  public:
    /**********************************************************************/
    /*! Creates a gene 
        \param value : LOng value of the gene
    */
    /**********************************************************************/
    TGALongGene(long value) : TZObject()
    {
      m_lValue = value;
    }
    
    /**********************************************************************/
    /*! See IGAGene
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getInterfacID()
    {
      return INTERFACE_IGALongGene;
    }
    
    /**********************************************************************/
    /*! See IGAGene
    */
    /**********************************************************************/
    virtual long MQUALIFIER getLongValue()
    {
      return m_lValue;
    }
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
  protected:
    /**********************************************************************/
    /*! Destroys the gene 
    */
    /**********************************************************************/
    virtual ~TGALongGene()
    {}
    
  private:
    ///Long Value
    long m_lValue;
  
};

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TGALongGene);
  INTERFACE_CAST(IGALongGene, INTERFACE_IGALongGene);
MEMORY_MANAGER_IMPL_END;


END_NAMESPACE_Zeus

#endif 
