/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IGAChromosone                                                 *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:27:22 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IGAChromosone.hpp,v $
 *   Revision 1.1  2005/11/10 10:27:22  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IGAChromosoneHPP
#define IGAChromosoneHPP 

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IArray.hpp>
#include <zeusbase/Math/Interfaces/IGAGene.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Individual Interface of an individum of a genetic population
*/
/***************************************************************************/
class IGAChromosone : public IZUnknown
{
  public:
  
    /***********************************************************************/
    /*! Gets the type of an chromosone
        \return type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getType()=0;
    
    /***********************************************************************/
    /*! Returns the number of genes
        \return number of genes
    */
    /***********************************************************************/
    virtual long MQUALIFIER getGeneCount()=0;
    
    /***********************************************************************/
    /*! Returns a gene. Each gene represents a set of data and 
        a border of the crossover algorithme
        \param index : Index of the gene
        \param gene : Return parameter
        \retval RET_NOERROR : Gene returned.
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getGene(long index, IGAGene** gene)=0;    

    /***********************************************************************/
    /*! This method performs the crossover of two chromosones of 
        the same kind. It is important that the type of the two chromosones 
        are the same.
        \param chrom : 2nd chromosone. 
        \param new_chrom1 : generated chromosone. Must be added to the new 
                           individual
        \param new_chrom2 : generated chromosone. Is the inverse Chromosone to 
                           new_chrom1
        \retval RET_NOERROR : Crossing completed
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER crossOver(IGAChromosone* chrom, IGAChromosone** new_chrom1, IGAChromosone** new_chrom2)=0;
};

END_NAMESPACE_Zeus

#endif 
