/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IGAIndividual                                                 *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:27:22 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IGAIndividual.hpp,v $
 *   Revision 1.1  2005/11/10 10:27:22  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IGAIndividualHPP 
#define IGAIndividualHPP 

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/Math/Interfaces/IGAChromosone.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Individual Interface of an individum of a genetic population
*/
/***************************************************************************/
class IGAIndividual : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! Gets the fitness of the individual
    */
    /***********************************************************************/
    virtual double MQUALIFIER getFitness()=0;
    
    /***********************************************************************/
    /*! Returns the count of the available chromosones
        \return count
    */
    /***********************************************************************/
    virtual long MQUALIFIER getChromosoneCount()=0;
    
    /***********************************************************************/
    /*! Returns a chromosones of this individual
        \param index : Index of the chromosone
        \param chromosone : Return value of the chromosone
        \retval RET_NOERROR : Chromosone returned
        \retval RET_REQUEST_FAILED : Could not get the chromosone
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getChromosone(long index, IGAChromosone** chromosone)=0;
    
    /***********************************************************************/
    /*! Adds a new chromosone to the individual
        \param chromosone : New instance of a chromosone
        \retval RET_NOERROR : Chromosone added
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER addChromosone(IGAChromosone* chromosone)=0;
    
    /***********************************************************************/
    /*! Creates chromosones randomly. This is used to create a initial 
        set of individuals
    */
    /***********************************************************************/
    virtual void MQUALIFIER createChromosones()=0;
};

END_NAMESPACE_Zeus

#endif
