/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IGAPopulation                                                 *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:27:22 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IGAPopulation.hpp,v $
 *   Revision 1.1  2005/11/10 10:27:22  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IGAPopulationHPP 
#define IGAPopulationHPP 

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/Math/Interfaces/IGAIndividual.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! A genetic population of individuals. Through this population the cycle 
    of regeneration is controlled.
*/
/***************************************************************************/
class IGAPopulation : public IZUnknown
{
  public:
  
    /***********************************************************************/
    /*! Gets the count of the individuals
        \return count
    */
    /***********************************************************************/
    virtual long MQUALIFIER getCount()=0;
    
    /***********************************************************************/
    /*! Gets a specific individual
        \param index : Index of Individual
        \param individual : Return parameter of the individual
        \retval RET_NOERROR : Individual returned
        \retval RET_REQUEST_FAILED : Could not return the individual
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getIndividual(long index, IGAIndividual** individual)=0;

    /***********************************************************************/
    /*! Returns the individual with the best fitness
        \param individual : Return parameter of the individual
        \retval RET_NOERROR : Individual returned
        \retval RET_REQUEST_FAILED : Could not return the individual
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getBestIndividual(IGAIndividual** individual)=0;
    
    /***********************************************************************/
    /*! Generates a next generation of individuals
        \retval RET_NOERROR : Generation created
        \retval RET_REQUEST_FAILED : Could not generate a next generation
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER makeNextGeneration()=0;
    
    /***********************************************************************/
    /*! Generates new populations until no better individual has been found
        \param abort : How many populations having the best individual 
                       should be created before it terminates
        \return count of generations
    */
    /***********************************************************************/
    virtual long MQUALIFIER makeGenerations(long abort)=0;
};

END_NAMESPACE_Zeus

#endif
