/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Math                                                          *
 * Package : Math                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Math.hpp,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MathHPP 
#define MathHPP

#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! This class contains generous mathematical functions
*/
/**************************************************************************/
class TMath : public TZObject
{
  public:
    /**********************************************************************/
    /*! Gets the maximum of two double values
        \param d1 : value 1
        \param d2 : value 2
        \return max(value 1, value 2)
    */
    /**********************************************************************/
    static double maxDouble(double d1, double d2)
    {
      double dRetval = d1;
      if (d1 < d2)
      {
        dRetval = d2;
      }
      return dRetval;
    }
    
    /**********************************************************************/
    /*! Gets the minimum of two double values
        \param d1 : value 1
        \param d2 : value 2
        \return min(value 1, value 2)
    */
    /**********************************************************************/
    static double minDouble(double d1, double d2)
    {
      double dRetval = d1;
      if (d1 > d2)
      {
        dRetval = d2;
      }
      return dRetval;
    }
    
    /**********************************************************************/
    /*! Gets the maximum of two int values
        \param d1 : value 1
        \param d2 : value 2
        \return max(value 1, value 2)
    */
    /**********************************************************************/
    static int maxInt(int d1, int d2)
    {
      int iRetval = d1;
      if (d1 < d2)
      {
        iRetval = d2;
      }
      return iRetval;
    }
    
    /**********************************************************************/
    /*! Gets the minimum of two int values
        \param d1 : value 1
        \param d2 : value 2
        \return min(value 1, value 2)
    */
    /**********************************************************************/
    static int minInt(int d1, int d2)
    {
      int iRetval = d1;
      if (d1 > d2)
      {
        iRetval = d2;
      }
      return iRetval;
    }
};

END_NAMESPACE_Zeus

#endif

