/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : BinaryMessage                                                 *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:57:11 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: BinaryMessage.cpp,v $
 *   Revision 1.1  2005/11/10 10:57:11  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Messaging/BinaryMessage.h>

#include <zeusbase/System/CriticalSection.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR_START(TBinaryMessage)
{
  m_pLock = new TCriticalSection();
}
SERIAL_CONSTRUCTOR_END

/***************************************************************************/
/*! Creates a new simple system message object
*/
/***************************************************************************/
unsigned long TBinaryMessage::createMessage(TString name, TString targetcell, TString targetpipe, TBinaryMessage** message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *message = new TBinaryMessage(name);
    (*message)->setTargetCellName(&targetcell);
    (*message)->setTargetPipeName(&targetpipe);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! Creates a new xml message object
*/
/***************************************************************************/
TBinaryMessage::TBinaryMessage(TString name) : TZObject(name)
{
  m_pLock = new TCriticalSection();
  m_ulDataType = 0;
  m_lNumber = 0;
}


/***************************************************************************/
/*! Destroys the message object
*/
/***************************************************************************/
TBinaryMessage::~TBinaryMessage()
{
  m_pLock->release();
}



/***************************************************************************/
/*! see IBinaryMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getData(IByteArray* data) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    data->setArray(m_aData.getArray(), m_aData.getCount());
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IBinaryMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getDataType() const
{
  return m_ulDataType;
}
/***************************************************************************/
/*! see IBinaryMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::setData(IByteArray* data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    m_aData = data;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IBinaryMessage
*/
/***************************************************************************/
void MQUALIFIER TBinaryMessage::setDataType(unsigned long type)
{
  m_ulDataType = type;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getInterfaceID() const
{
  return INTERFACE_IBinaryMessage;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getMessageName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assign(this->getObjName().c_bstr());
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
long MQUALIFIER TBinaryMessage::getNumber() const
{ return m_lNumber; }



/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getSenderPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strSenderPipeName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getSenderCellName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strSenderCellName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getTargetPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strTargetPipeName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}  

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::getTargetCellName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strTargetCellName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TBinaryMessage::setMessageName(const IString* name)
{
  TString strName(name);
  this->setObjName(strName);
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TBinaryMessage::setNumber(long value)
{
  m_lNumber = value;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TBinaryMessage::setSenderCellName(IString* name)
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  if (name != NULL)
  {
    m_strSenderCellName = name;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TBinaryMessage::setSenderPipeName(const IString* name)
{
  m_strSenderPipeName = name;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TBinaryMessage::setTargetPipeName(const IString* name)
{
  m_strTargetPipeName = name;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TBinaryMessage::setTargetCellName(const IString* name)
{
  m_strTargetCellName = name;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TBinaryMessage, m_pLock);
  INTERFACE_CAST(IMessage, INTERFACE_IMessage);
  INTERFACE_CAST(IBinaryMessage, INTERFACE_IBinaryMessage);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;
