/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : BinaryMessage                                                 *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:57:11 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: BinaryMessage.h,v $
 *   Revision 1.1  2005/11/10 10:57:11  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BinaryMessageH
#define BinaryMessageH

#include <zeusbase/Messaging/Interfaces/IBinaryMessage.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>

BEGIN_NAMESPACE_Zeus

class TCriticalSection;

/***************************************************************************/
/*! This message class implements a format to transfer binary data.
*/
/***************************************************************************/
class TBinaryMessage : public TZObject, public IBinaryMessage
{
  public:
    static unsigned long createMessage(TString name, TString targetcell, TString targetpipe, TBinaryMessage** message);
        
    //Methods of IBinaryMessage;
    virtual unsigned long MQUALIFIER getData(IByteArray* data) const;
    virtual unsigned long MQUALIFIER getDataType() const;
    virtual unsigned long MQUALIFIER setData(IByteArray* data);
    virtual void MQUALIFIER setDataType(unsigned long type);
        
    //Methods of IMessage;
    virtual unsigned long MQUALIFIER getInterfaceID() const;
    virtual unsigned long MQUALIFIER getMessageName(IString* name) const;
    virtual long MQUALIFIER getNumber() const;
    virtual unsigned long MQUALIFIER getSenderPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getSenderCellName(IString* name) const;
    virtual unsigned long MQUALIFIER getTargetPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getTargetCellName(IString* name) const;
    virtual void MQUALIFIER setMessageName(const IString* name);
    virtual void MQUALIFIER setNumber(long value);
    virtual unsigned long MQUALIFIER setSenderCellName(IString* name);
    virtual void MQUALIFIER setSenderPipeName(const IString* name);
    virtual void MQUALIFIER setTargetPipeName(const IString* name);
    virtual void MQUALIFIER setTargetCellName(const IString* name);
    
    //Methods of ISerializable
    SERIAL_START(TBinaryMessage, L"TBinaryMessage")
      SERIAL_STRING_SET_GET(setObjName, getObjName)
      SERIAL_LONG(m_lNumber)
      SERIAL_STRING(m_strSenderCellName)
      SERIAL_STRING(m_strSenderPipeName)
      SERIAL_STRING(m_strTargetCellName)
      SERIAL_STRING(m_strTargetPipeName)
      SERIAL_LONG(m_ulDataType)
      SERIAL_BYTEARRAY(m_aData)
    SERIAL_END

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    TBinaryMessage(TString name=L"");
    virtual ~TBinaryMessage();

    TString m_strSenderCellName;
    TString m_strSenderPipeName;
    TString m_strTargetCellName;
    TString m_strTargetPipeName;
    TByteArray m_aData;
    unsigned long m_ulDataType;
    long m_lNumber;
        
  private:    
    TCriticalSection* m_pLock;
};

END_NAMESPACE_Zeus

#endif 
 
