/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IMessage                                                      *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:57:46 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IMessage.hpp,v $
 *   Revision 1.1  2005/11/10 10:57:46  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef IMessageHPP
#define IMessageHPP
 
#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/Interfaces/IByteArray.hpp>
#include <zeusbase/Messaging/Interfaces/MessageDefines.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines a general message object. This kind of message
    can be exchanged between cells. There are different sub types of this
    message interface, like IBinaryMessageObject or ISXMessageObject etc.
    These messages may use different transport protocols like CCTP (Complex
    Communication transfer protocol) witch goes with IBinaryMessageObject
    or the TCTP (Simple Communication Transport Protocol) witch goes with
    ISXMessageObject.
*/
/***************************************************************************/
class IMessage : public ISerializable
{
  public:
    /***********************************************************************/
    /*! This method returns the interface id of the message. The difference
        between the getPrimaryType() from IStreamable and this method is,
        that this id represents the Message Object Type ID 
        (format of the message), the primary type is the base object type.
        \return Message Interface Type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getInterfaceID() const=0;
    
    /***********************************************************************/
    /*! gets the message name
        \param name: Name of the message
        \retval RET_NOERROR : value returned
        \retval RET_INVALID_PARAMETER : invalid parameter received
        \retval RET_INTERNAL_ERROR : No data node
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getMessageName(IString* name) const=0;
    
    /***********************************************************************/
    /*! gets the message number.
        \return number of the message
    */
    /***********************************************************************/
    virtual long MQUALIFIER getNumber() const=0;
    
    /***********************************************************************/
    /*! gets the name of the sender pipe
        \param name : return value
        \retval RET_NOERROR : value returned
        \retval RET_INVALID_PARAMETER : invalid parameter received
        \retval RET_INTERNAL_ERROR : No data node
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getSenderPipeName(IString* name) const=0;

    /***********************************************************************/
    /*! gets the name of the sender cell
        \param name : return value
        \retval RET_NOERROR : value returned
        \retval RET_INVALID_PARAMETER : invalid parameter received
        \retval RET_INTERNAL_ERROR : No data node
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getSenderCellName(IString* name) const=0;

    /***********************************************************************/
    /*! gets the name of the target pipe
        \param name : return value
        \retval RET_NOERROR : value returned
        \retval RET_INVALID_PARAMETER : invalid parameter received
        \retval RET_INTERNAL_ERROR : No data node
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getTargetPipeName(IString* name) const=0;

    /***********************************************************************/
    /*! gets the name of the target cell
        \param name : return value
        \retval RET_NOERROR : value returned
        \retval RET_INVALID_PARAMETER : invalid parameter received
        \retval RET_INTERNAL_ERROR : No data node
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getTargetCellName(IString* name) const=0;

    /***********************************************************************/
    /*! sets the message name
        \param name: Name of the message
    */
    /***********************************************************************/
    virtual void MQUALIFIER setMessageName(const IString* name)=0;
    
    /***********************************************************************/
    /*! sets the message number.
        \param value : any number
    */
    /***********************************************************************/
    virtual void MQUALIFIER setNumber(long value)=0;
    
    /***********************************************************************/
    /*! sets the name of the sender cell
        \param name : name of the sender
        \retval RET_NOERROR : value set
        \retval RET_INVALID_PARAMETER : invalid parameter received
        \retval RET_INTERNAL_ERROR : No data node
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setSenderCellName(IString* name)=0;
    
    /***********************************************************************/
    /*! sets the name of the sender pipe
        \param name : name of the sender pipe
    */
    /***********************************************************************/
    virtual void MQUALIFIER setSenderPipeName(const IString* name)=0;
    
    /***********************************************************************/
    /*! sets the name of the target pipe
        \param name : name of the target pipe
    */
    /***********************************************************************/
    virtual void MQUALIFIER setTargetPipeName(const IString* name)=0;
    
    /***********************************************************************/
    /*! sets the name of the target cell
        \param name : name of the target cell
    */
    /***********************************************************************/
    virtual void MQUALIFIER setTargetCellName(const IString* name)=0;
};


END_NAMESPACE_Zeus

#endif
