/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IObjectMessage                                                *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:57:46 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IObjectMessage.hpp,v $
 *   Revision 1.1  2005/11/10 10:57:46  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef IObjectMessageHPP
#define IObjectMessageHPP


#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/Messaging/Interfaces/IMessage.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This message contains a streamable object
*/
/***************************************************************************/
class IObjectMessage : public IMessage
{
  public:
    /***********************************************************************/
    /*! Returns the reference of the object
        \param obj : Return parameter
        \retval RET_NOERROR : object successfully returned
        \retval RET_REQUEST_FAILED : Could not return object
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getObject(ISerializable** obj)=0;
    
    /***********************************************************************/
    /*! Sets a new object
        \param obj : Stream object reference
        \retval RET_NOERROR : object successfully set
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setObject(ISerializable* obj)=0;
    
    /***********************************************************************/
    /*! Returns the data type of the streaming object
        \return data type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getDataType() const=0;
    
    /***********************************************************************/
    /*! Sets the data type of the streaming object
        \param value : data type
    */
    /***********************************************************************/
    virtual void MQUALIFIER setDataType(unsigned long value)=0;
};

END_NAMESPACE_Zeus

#endif 
 
