/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StreamMessage                                                 *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/15 07:09:52 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ObjectMessage.cpp,v $
 *   Revision 1.2  2005/12/15 07:09:52  bhadorn
 *   Added locks
 *
 *   Revision 1.1  2005/11/10 10:57:11  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Messaging/ObjectMessage.h>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/String.h>

USING_NAMESPACE_Zeus

/// Constructor of the serializing 
SERIAL_CONSTRUCTOR_START(TObjectMessage)
{
  m_pLock = new TCriticalSection();
}
SERIAL_CONSTRUCTOR_END

///static method to creat a message
unsigned long TObjectMessage::createMessage(TString name, TString targetcell, TString targetpipe, TObjectMessage** message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *message = new TObjectMessage(name);
    (*message)->setTargetCellName(&targetcell);
    (*message)->setTargetPipeName(&targetpipe);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! Creates a stream message
    \param name : Name of the message
*/
/***************************************************************************/
TObjectMessage::TObjectMessage(TString name) : TZObject(name)
{
  m_pLock = new TCriticalSection();
  m_pStreamObject = NULL;
  m_ulDataType = 0;
  m_lNumber = 0;
}

/***************************************************************************/
/*! Destroys a message
*/
/***************************************************************************/
TObjectMessage::~TObjectMessage()
{

  if (m_pStreamObject != NULL)
  {
    m_pStreamObject->release();
    m_pStreamObject = NULL;
  }
  m_pLock->release();
}
    
/***************************************************************************/
/*! See IStreamMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getObject(ISerializable** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  m_pLock->enter();
  if(obj == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pStreamObject != NULL)
  {
    *obj = m_pStreamObject;
    m_pStreamObject->addRef();
    ulRetval = RET_NOERROR;
  }
  m_pLock->leave();
  return ulRetval;
}

/***************************************************************************/
/*! See IStreamMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::setObject(ISerializable* obj)
{
  m_pLock->enter();
  if (m_pStreamObject != NULL)
  {
    m_pStreamObject->release();
  }
  m_pStreamObject = obj;
  if (m_pStreamObject != NULL)
  {
    m_pStreamObject->addRef();
  }
  m_pLock->leave();
  return RET_NOERROR;
}

/***************************************************************************/
/*! See IStreamMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getDataType() const
{   
  return m_ulDataType;
}

/***************************************************************************/
/*! See IStreamMessage
*/
/***************************************************************************/
void MQUALIFIER TObjectMessage::setDataType(unsigned long value)
{ m_ulDataType = value;}


/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getInterfaceID() const
{ return INTERFACE_IObjectMessage; }

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getMessageName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_pLock->enter();
    name->assign(this->getObjName().c_bstr());
    ulRetval = RET_NOERROR;
    m_pLock->leave();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
long MQUALIFIER TObjectMessage::getNumber() const
{ return m_lNumber; }

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getSenderPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_pLock->enter();
    name->assignStr(&m_strSenderPipeName);
    ulRetval = RET_NOERROR;
    m_pLock->leave();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getSenderCellName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_pLock->enter();
    name->assignStr(&m_strSenderCellName);
    ulRetval = RET_NOERROR;
    m_pLock->leave();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getTargetPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_pLock->enter();
    name->assignStr(&m_strTargetPipeName);
    ulRetval = RET_NOERROR;
    m_pLock->leave();
  }
  return ulRetval;
}  

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::getTargetCellName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_pLock->enter();
    name->assignStr(&m_strTargetCellName);
    ulRetval = RET_NOERROR;
    m_pLock->leave();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
void MQUALIFIER TObjectMessage::setMessageName(const IString* name)
{
  m_pLock->enter();
  TString strName(name);
  this->setObjName(strName);
  m_pLock->leave();
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
void MQUALIFIER TObjectMessage::setNumber(long value)
{
  m_pLock->enter();
  m_lNumber = value;
  m_pLock->leave();
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TObjectMessage::setSenderCellName(IString* name)
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  if (name != NULL)
  {
    m_pLock->enter();
    m_strSenderCellName = name;
    ulRetval = RET_NOERROR;
    m_pLock->leave();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
void MQUALIFIER TObjectMessage::setSenderPipeName(const IString* name)
{
  m_pLock->enter();
  m_strSenderPipeName = name;
  m_pLock->leave();
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
void MQUALIFIER TObjectMessage::setTargetPipeName(const IString* name)
{
  m_pLock->enter();
  m_strTargetPipeName = name;
  m_pLock->leave();
}

/***************************************************************************/
/*! See IMessage
*/
/***************************************************************************/
void MQUALIFIER TObjectMessage::setTargetCellName(const IString* name)
{
  m_pLock->enter();
  m_strTargetCellName = name;
  m_pLock->leave();
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TObjectMessage, m_pLock);
  INTERFACE_CAST(IObjectMessage, INTERFACE_IMessage);
  INTERFACE_CAST(IObjectMessage, INTERFACE_IObjectMessage);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;
 
