/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SystemMessage                                                 *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:00:48 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SystemMessage.cpp,v $
 *   Revision 1.2  2005/12/14 09:00:48  bhadorn
 *   createMessage was not declared with MQUALIFIER
 *
 *   Revision 1.1  2005/11/10 10:57:11  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Messaging/SystemMessage.h>
#include <zeusbase/System/CriticalSection.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR_START(TSystemMessage)
{
  m_pLock = new TCriticalSection();
}
SERIAL_CONSTRUCTOR_END

/***************************************************************************/
/*! Creates a new simple system message object
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::createMessage(TString name, TString targetcell, TString targetpipe, TSystemMessage** message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *message = new TSystemMessage(name);
    (*message)->setTargetCellName(&targetcell);
    (*message)->setTargetPipeName(&targetpipe);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! Creates a new xml message object
*/
/***************************************************************************/
TSystemMessage::TSystemMessage(TString name) : TZObject(name)
{
  m_ulMessageType = MSG_NOP;
  m_pLock = new TCriticalSection();
  m_lNumber = 0;
}


/***************************************************************************/
/*! Destroys the message object
*/
/***************************************************************************/
TSystemMessage::~TSystemMessage()
{
  m_pLock->release();
}

/***************************************************************************/
/*! see ISystemMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::getMessageType() const
{
  return m_ulMessageType;
}

/***************************************************************************/
/*! see ISystemMessage
*/
/***************************************************************************/
void MQUALIFIER TSystemMessage::setMessageType(unsigned long type)
{
  m_ulMessageType = type;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::getInterfaceID() const
{
  return INTERFACE_ISystemMessage;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::getMessageName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assign(this->getObjName().c_bstr());
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
long MQUALIFIER TSystemMessage::getNumber() const
{ return m_lNumber; }


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::setSenderCellName(IString* name)
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  if (name != NULL)
  {
    m_strSenderCellName = name;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::getSenderPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strSenderPipeName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::getSenderCellName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strSenderCellName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::getTargetPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strTargetPipeName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}  

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TSystemMessage::getTargetCellName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strTargetCellName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TSystemMessage::setSenderPipeName(const IString* name)
{
  m_strSenderPipeName = name;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TSystemMessage::setTargetPipeName(const IString* name)
{
  m_strTargetPipeName = name;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TSystemMessage::setTargetCellName(const IString* name)
{
  m_strTargetCellName = name;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TSystemMessage::setMessageName(const IString* name)
{
  TString strName(name);
  this->setObjName(strName);
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TSystemMessage::setNumber(long value)
{
  m_lNumber = value;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TSystemMessage, m_pLock);
  INTERFACE_CAST(IMessage, INTERFACE_IMessage);
  INTERFACE_CAST(ISystemMessage, INTERFACE_ISystemMessage);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;

 
