/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SystemMessage                                                 *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:00:48 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SystemMessage.h,v $
 *   Revision 1.2  2005/12/14 09:00:48  bhadorn
 *   createMessage was not declared with MQUALIFIER
 *
 *   Revision 1.1  2005/11/10 10:57:11  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SystemMessageH
#define SystemMessageH

#include <zeusbase/Messaging/Interfaces/ISystemMessage.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>

BEGIN_NAMESPACE_Zeus

class TCriticalSection;

/***************************************************************************/
/*! This class implements a system message. Those messages are used 
    for controlling and status reports. The format is binary.
*/
/***************************************************************************/
class TSystemMessage : public TZObject, public ISystemMessage
{
  public:
    static unsigned long MQUALIFIER createMessage(TString name, TString targetcell, TString targetpipe, TSystemMessage** message);
    
    //Methods of ISystemMessage;
    virtual unsigned long MQUALIFIER getMessageType() const;
    virtual void MQUALIFIER setMessageType(unsigned long type);
        
    //Methods of IMessage;
    virtual unsigned long MQUALIFIER getInterfaceID() const;
    virtual unsigned long MQUALIFIER getMessageName(IString* name) const;
    virtual long MQUALIFIER getNumber() const;
    virtual unsigned long MQUALIFIER getSenderPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getSenderCellName(IString* name) const;
    virtual unsigned long MQUALIFIER getTargetPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getTargetCellName(IString* name) const;
    virtual void MQUALIFIER setMessageName(const IString* name);
    virtual void MQUALIFIER setNumber(long value);
    virtual unsigned long MQUALIFIER setSenderCellName(IString* name);
    virtual void MQUALIFIER setSenderPipeName(const IString* name);
    virtual void MQUALIFIER setTargetPipeName(const IString* name);
    virtual void MQUALIFIER setTargetCellName(const IString* name);

    //Methods of ISterializable
    SERIAL_START(TSystemMessage, L"TSystemMessage")
      SERIAL_STRING_SET_GET(setObjName, getObjName)
      SERIAL_LONG(m_lNumber)
      SERIAL_STRING(m_strSenderCellName)
      SERIAL_STRING(m_strSenderPipeName)
      SERIAL_STRING(m_strTargetCellName)
      SERIAL_STRING(m_strTargetPipeName)
      SERIAL_LONG(m_ulMessageType)
    SERIAL_END

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    TSystemMessage(TString name=L"");
    virtual ~TSystemMessage();
    
  private:    
    TString m_strSenderCellName;
    TString m_strSenderPipeName;
    TString m_strTargetCellName;
    TString m_strTargetPipeName;
    unsigned long m_ulMessageType;
    long m_lNumber;
    TCriticalSection* m_pLock;
};

END_NAMESPACE_Zeus

#endif 
