/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XMLMessage                                                    *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:59:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XMLMessage.h,v $
 *   Revision 1.2  2005/12/14 08:59:56  bhadorn
 *   createObject was not declared with MQUALIFIER
 *
 *   Revision 1.1  2005/11/10 10:57:11  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef XMLMessageH
#define XMLMessageH

#include <zeusbase/Messaging/Interfaces/IXMLMessage.hpp>
#include <zeusbase/System/XRootObject.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/String.h>


BEGIN_NAMESPACE_Zeus

class TCriticalSection;
class IXMLParser;
class IXMLNode;

/***************************************************************************/
/*! This class implements a message using the message object
    protocol MOP
*/
/***************************************************************************/
class TXMLMessage : public TXRootObject, public IXMLMessage
{
  public:
    static unsigned long createMessage(TString name, TString targetcell, TString targetpipe, IXMLMessage** message);
    static unsigned long MQUALIFIER createObject(unsigned long objectID, IInputStream* stream, IZUnknown** obj);

    /**********************************************************************/
    /*! Method for auto registration
    */
    /**********************************************************************/
    static void autoreg(bool reg)
    { 
      static unsigned long ulHandle = 0;
      TString strClassName(L"TXMLMessage"); 
      if (reg) 
      { 
        ulHandle = TZObjectFactory::getInstance().registerClass(&strClassName, createObject); 
      } 
      else 
      { 
        TZObjectFactory::getInstance().unregisterClass(&strClassName, ulHandle);
      } 
    } 
        
    //Methods of IXMLMessage;
    virtual unsigned long MQUALIFIER getDataNode(IXMLNode** node);
        
    //Methods of IMessage;
    virtual unsigned long MQUALIFIER getInterfaceID() const;
    virtual unsigned long MQUALIFIER getMessageName(IString* name) const;
    virtual long MQUALIFIER getNumber() const;
    virtual unsigned long MQUALIFIER getSenderPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getSenderCellName(IString* name) const;
    virtual unsigned long MQUALIFIER getTargetPipeName(IString* name) const;
    virtual unsigned long MQUALIFIER getTargetCellName(IString* name) const;
    virtual void MQUALIFIER setMessageName(const IString* name);
    virtual void MQUALIFIER setNumber(long value);
    virtual unsigned long MQUALIFIER setSenderCellName(IString* name);
    virtual void MQUALIFIER setSenderPipeName(const IString* name);
    virtual void MQUALIFIER setTargetPipeName(const IString* name);
    virtual void MQUALIFIER setTargetCellName(const IString* name);

    //Methods of ISerializable
    virtual unsigned long MQUALIFIER serialize(IByteArray* stream) const;
    virtual unsigned long MQUALIFIER getObjectID() const;

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    TXMLMessage(IXMLNode* main_node, NAMESPACE_Zeus::IXMLDocument* docu);
    virtual ~TXMLMessage();
    
    ///////////////////////////////////////////////////
    // Auto registration of XRootObject
    REG_ROOT_BEGIN(TXMLMessage)
    REG_ROOT_ADD(TXMLMessage, L"TXMLMessage");
    REG_ROOT_END
    
  private:    
    ///Lock for multithreading access
    TCriticalSection* m_pLock;
};

END_NAMESPACE_Zeus

#endif 
