/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractRemoteObject                                          *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:39:16 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractRemoteObject.cpp,v $
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/Remote/AbstractRemoteObject.h>
#include <zeusbase/Remote/AbstractSkel.h>

USING_NAMESPACE_Zeus



/***************************************************************************/
/*! Creates the abstract remote object
*/
/***************************************************************************/
TAbstractRemoteObject::TAbstractRemoteObject() : TZObject()
{
  m_pSkeleton = NULL;
  m_ulPort = 0;
}

  
/***************************************************************************/
/*! Destroys the abstract remote object
*/
/***************************************************************************/
TAbstractRemoteObject::~TAbstractRemoteObject()
{
  disconnect();
}

/***************************************************************************/
/*! Disconnects or kills the remote object server (skeleton). After this
    call, all remote access connections will be released.
    \retval RET_NOERROR : Connection closed
    \retval RET_REQUEST_FAILED : Object was not connected
*/
/***************************************************************************/
unsigned long TAbstractRemoteObject::disconnect()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pSkeleton != NULL)
  {
    m_pSkeleton->kill();
    m_pSkeleton->release();
    m_pSkeleton = NULL;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IRemoteObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TAbstractRemoteObject::getConnectionParam(IString* address, unsigned long* port)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (address != NULL && port != NULL)
  {
    address->assignStr(&m_strAddress);
    *port = m_ulPort;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}
  
MEMORY_MANAGER_IMPL(TAbstractRemoteObject);
  INTERFACE_CAST(IRemoteObject, INTERFACE_IRemoteObject);
MEMORY_MANAGER_IMPL_END;
