/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractRemoteObject                                          *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:25:51 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractRemoteObject.h,v $
 *   Revision 1.3  2005/11/25 09:25:51  bhadorn
 *   Renamed start method to startSkeleton
 *
 *   Revision 1.2  2005/11/16 11:52:22  bhadorn
 *   Dynamic binding of skeleton server
 *
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef AbstractRemoteObjectH
#define AbstractRemoteObjectH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/Remote/Interfaces/IRemoteObject.hpp>


/***************************************************************************/
/*! This macro defines concrete methods of the abstract remote object. Use
    it inside the concrete remote object declaration instead of writing 
    lots of code.
*/
/***************************************************************************/
#define REMOTE_OBJECT_DECL \
    virtual unsigned long MQUALIFIER getClassName(IString* name); \
    virtual unsigned long MQUALIFIER getCodeModuleName(IString* name); \
    virtual unsigned long connect(TString address, unsigned long port); \
    virtual unsigned long connect();



/***************************************************************************/
/*! This macro defines concrete methods of the abstract remote object. Use
    it inside the concrete remote object implementation instead of writing 
    lots of code.
    \param classid : ID of the concrete class
    \param __classname : Name of the concrete class
    \param __codemodule : Name of the code module
    \param skel_class : ClassID of the skeleton class
*/
/***************************************************************************/
#define REMOTE_OBJECT_IMPL(classid, __classname, __codemodule, skel_class) \
unsigned long MQUALIFIER classid::getClassName(IString* name) \
{ \
  unsigned long ulRetval = RET_INVALID_PARAMETER; \
  if (name != NULL) \
  { \
    name->assign(__classname); \
    ulRetval = RET_NOERROR; \
  } \
  return ulRetval; \
} \
unsigned long MQUALIFIER classid::getCodeModuleName(IString* name) \
{ \
  unsigned long ulRetval = RET_INVALID_PARAMETER; \
  if (name != NULL) \
  { \
    name->assign(__codemodule); \
    ulRetval = RET_NOERROR; \
  } \
  return ulRetval; \
} \
unsigned long classid::connect(TString address, unsigned long port)\
{ \
  m_strAddress = address; \
  m_ulPort = port; \
  m_pSkeleton = new skel_class(this); \
  return m_pSkeleton->startSkeleton(address, port, false); \
} \
\
unsigned long classid::connect() \
{ \
  m_strAddress = L"127.0.0.1"; \
  m_ulPort = 0; \
  m_pSkeleton = new skel_class(this); \
  return m_pSkeleton->startSkeleton(m_strAddress, m_ulPort, true); \
} \
  

BEGIN_NAMESPACE_Zeus

class TAbstractSkel;


/***************************************************************************/
/*! The abstract remote object defines how to create the remote
    access for an object. Each remote object should inherit this class.
*/
/***************************************************************************/
class TAbstractRemoteObject : public TZObject, public IRemoteObject
{
  public:
    TAbstractRemoteObject();
  
    virtual unsigned long connect()=0;
    virtual unsigned long connect(TString address, unsigned long port)=0;
    virtual unsigned long disconnect();
      
    //Methods of IRemoteObject
    virtual unsigned long MQUALIFIER getClassName(IString* name)=0;
    virtual unsigned long MQUALIFIER getCodeModuleName(IString* name)=0;
    virtual unsigned long MQUALIFIER getConnectionParam(IString* address, unsigned long* port);
  
      //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
  protected:
    ///Skeleton in case of binding the object for remote access
    TAbstractSkel* m_pSkeleton;
    ///Address of skeleton
    TString m_strAddress;
    ///Port of the skeleton
    unsigned long m_ulPort;

    virtual ~TAbstractRemoteObject();
  
  private:
};

END_NAMESPACE_Zeus

#endif
