/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IRemoteMethodCall                                             *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:39:54 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IRemoteMethodCall.hpp,v $
 *   Revision 1.1  2005/11/10 10:39:54  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef IRemoteMethodCallHPP
#define IRemoteMethodCallHPP
 

#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/Interfaces/IZVariant.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Interface declaration of the remote method call. A remote method call is
    a serializable object witch contains arguments and
    the return value of method call. This method call object will be sent 
    over network to invoke a real method call on a remote object.
*/
/***************************************************************************/
class IRemoteMethodCall : public ISerializable
{
  public:

    /***********************************************************************/
    /*! Adds a new parameter to the list. 
        \note : The param variant must be created on the heap (with new)
        \param param : new parameter (name and value)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER addParameter(IZVariant* param)=0;
    
    /***********************************************************************/
    /*! Gets the name of the method
        \param name : method name
        \retval RET_NOERROR : Name returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getMethodName(IString* name) const=0;
    
    /***********************************************************************/
    /*! returns a named parameter of the method call (name and value pair)
        \param name : Name of the parameter
        \param param : return parameter
        \retval RET_NOERROR : Parameter returned
        \retval RET_REQUEST_FAILED : No parameter found on index
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getParameterByName(IString* name, IZVariant** param) const=0;
    
    /***********************************************************************/
    /*! returns a indexed parameter of the method call (name and value pair)
        \param index : Index of the parameter
        \param param : return parameter
        \retval RET_NOERROR : Parameter returned
        \retval RET_REQUEST_FAILED : No parameter found on index
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getParameter(long index, IZVariant** param) const=0;
    
    /***********************************************************************/
    /*! Returns the number of parameters
    */
    /***********************************************************************/
    virtual long MQUALIFIER getParameterCount() const=0;
    
    /***********************************************************************/
    /*! Gets the return value of the method
        \param param : Return value to be get
        \retval RET_NOERROR : Return value returned
        \retval RET_REQUEST_FAILED : No return value available
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getReturnValue(IZVariant** param) const=0;
    
    /***********************************************************************/
    /*! sets a new method name 
        \param name : New name
        \retval RET_NOERROR : Name set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setMethodName(IString* name)=0;
    
    /***********************************************************************/
    /*! Sets the return value of the method
        \param param : Return value to be set
        \retval RET_NOERROR : Return value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setReturnValue(IZVariant* param)=0;
};

END_NAMESPACE_Zeus

#endif
