/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IRemoteObject                                                 *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:39:54 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IRemoteObject.hpp,v $
 *   Revision 1.1  2005/11/10 10:39:54  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef IRemoteObjectHPP
#define IRemoteObjectHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! The remote object interface defines methods for invoking methods 
    remotely and to access an object from a remote location.
*/
/***************************************************************************/
class IRemoteObject : public IZUnknown
{
  public:
    
    /***********************************************************************/
    /*! Returns the classname of the wrapped object
        \param name : Return parameter
        \retval RET_NOERROR : Name returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received;
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getClassName(IString* name)=0;
    
    /***********************************************************************/
    /*! Returns the name of the codemodule for the wrapped object
        \param name : Return parameter
        \retval RET_NOERROR : Name returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received;
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getCodeModuleName(IString* name)=0;
    
    /***********************************************************************/
    /*! Gets all information about the running server such as address and
        port of the connection.
        \param address : Return parameter of the address
        \param port : Return parameter of the port
        \retval RET_NOERROR : Connection parameters returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not get the connection
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getConnectionParam(IString* address, unsigned long* port)=0;
};

END_NAMESPACE_Zeus

#endif
 
