/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Naming                                                        *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:45:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Naming.h,v $
 *   Revision 1.2  2005/11/16 11:45:24  bhadorn
 *   Input parameter are const. New method added for getting only the object names instead of the stubs.
 *
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef NamingH
#define NamingH
 

#include <zeusbase/Remote/Interfaces/INaming.hpp>
#include <zeusbase/Remote/AbstractStub.h>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/StringMap.hpp>

BEGIN_NAMESPACE_Zeus

///Defines the static access for Naming service
#define Naming          TNaming::getInstance()

class TSocket;
class TCriticalSection;

/***************************************************************************/
/*! The Naming class is used as a singleton to access a naming service 
    through a stub or if no stub is allocated, this class is directly the
    database of the naming server
*/
/***************************************************************************/
class TNaming : public TZObject, public INaming
{
  public:
    static TNaming& getInstance();
    
    unsigned long connect(const TString& address, unsigned long port);
    void setDelegationInterface(INaming* iface);
    void reset();
    void useRemote(bool mode =true);
    
    //Methods of INaming
    virtual unsigned long MQUALIFIER bind(const IString* name, IRemoteObject* object);
    virtual unsigned long MQUALIFIER bind2(const IString* name, const IString* cat, IRemoteObject* object);
    virtual unsigned long MQUALIFIER rebind(const IString* name, IRemoteObject* object);
    virtual unsigned long MQUALIFIER rebind2(const IString* name, const IString* cat, IRemoteObject* object);
    virtual unsigned long MQUALIFIER unbind(const IString* name);
    virtual unsigned long MQUALIFIER unbind2(const IString* name, const IString* cat);
    virtual unsigned long MQUALIFIER lookup(const IString* name, IRemoteObject** object);
    virtual unsigned long MQUALIFIER lookup2(const IString* name, const IString* cat, IRemoteObject** object);
    virtual unsigned long MQUALIFIER lookupAny(const IString* cat, IRemoteObject** object);
    virtual unsigned long MQUALIFIER lookupMultibleObjects(const IString* name, IList<IRemoteObject*>* objects);
    virtual unsigned long MQUALIFIER lookupCategory(const IString* cat, IList<IRemoteObject*>* objects);
    virtual unsigned long MQUALIFIER getObjectNames(const IString* cat, IList<IString*>* names);
    virtual bool MQUALIFIER isRemote() const;
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    TNaming();
    virtual ~TNaming();
    
  private:
    typedef TStringMap<IRemoteObject*> TRemoteObjectMap;
    
    ///Static reference
    static TNaming m_Naming;
    ///Map of all registered named objects
    TStringMap<TRemoteObjectMap*> m_mapCategory;
    ///Lock of the data base
    TCriticalSection* m_pLock;
    ///Stub or delegation interface
    INaming* m_pDelegationInterface;
    ///Using naming service for remote
    bool m_bRemote;
    
    void removeAllObjects();
};

/***************************************************************************/
/* Stub declaration. All methods of the Interface INaming (and sub 
   interfaces) must be declared here.
*/
/***************************************************************************/
class TNaming_Stub : public TAbstractStub, public INaming
{
  public:
    TNaming_Stub();
    
    //Methods of INaming
    virtual unsigned long MQUALIFIER bind(const IString* name, IRemoteObject* object);
    virtual unsigned long MQUALIFIER bind2(const IString* name, const IString* cat, IRemoteObject* object);
    virtual unsigned long MQUALIFIER rebind(const IString* name, IRemoteObject* object);
    virtual unsigned long MQUALIFIER rebind2(const IString* name, const IString* cat, IRemoteObject* object);
    virtual unsigned long MQUALIFIER unbind(const IString* name);
    virtual unsigned long MQUALIFIER unbind2(const IString* name, const IString* cat);
    virtual unsigned long MQUALIFIER lookup(const IString* name, IRemoteObject** object);
    virtual unsigned long MQUALIFIER lookup2(const IString* name, const IString* cat, IRemoteObject** object);
    virtual unsigned long MQUALIFIER lookupAny(const IString* cat, IRemoteObject** object);
    virtual unsigned long MQUALIFIER lookupMultibleObjects(const IString* name, IList<IRemoteObject*>* objects);
    virtual unsigned long MQUALIFIER lookupCategory(const IString* cat, IList<IRemoteObject*>* objects);
    virtual unsigned long MQUALIFIER getObjectNames(const IString* cat, IList<IString*>* names);
    virtual bool MQUALIFIER isRemote() const;
    
    STUB_CLASS_DECL(TNaming_Stub, "TNaming_Stub");
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TNaming_Stub();
  
  private:
  
};

END_NAMESPACE_Zeus

#endif
