/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : RemoteMethodCall                                              *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:39:16 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RemoteMethodCall.cpp,v $
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Remote/RemoteMethodCall.h>
#include <zeusbase/System/LoggerManager.h>


USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR(TRemoteMethodCall);

/**************************************************************************/
/*! Creates a remote method call object
    \param name : Name of the method
*/
/**************************************************************************/
TRemoteMethodCall::TRemoteMethodCall(TString name /*=L""*/) : TZObject(name)
{
  m_pReturnValue = NULL;
  m_strMethdoName = name;
}
    
/**************************************************************************/
/*! Destroys the remote method call object
*/
/**************************************************************************/
TRemoteMethodCall::~TRemoteMethodCall()
{
  while(m_lstArguments.getCount() > 0)
  {
    m_lstArguments[0]->release();
    m_lstArguments.deleteItem(0);
  }
  
  if (m_pReturnValue != NULL)
  {
    m_pReturnValue->release();
    m_pReturnValue = NULL;
  }
}

/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
unsigned long MQUALIFIER TRemoteMethodCall::addParameter(IZVariant* param)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (param != NULL)
  {
    TString strName;
    param->getVariantName(&strName);
    ZLOGGER.printfln(LOGMODE_DEBUG, "Parameter added [%s]", strName.c_str());
    param->addRef();
    m_lstArguments.add(param);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
unsigned long MQUALIFIER TRemoteMethodCall::getMethodName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strMethdoName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
unsigned long MQUALIFIER TRemoteMethodCall::getParameterByName(IString* name, IZVariant** param) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (param != NULL)
  {
    long lCount = m_lstArguments.getCount();
    bool bFound = false;
    for(int i = 0; !bFound && i < lCount; i++)
    {
      TString strName;
      IZVariant* pVar = m_lstArguments.getItemCopy(i);
      pVar->getVariantName(&strName);
      if (strName.equalsStr(name))
      {
        *param = pVar;
        pVar->addRef();
        ulRetval = RET_NOERROR;
        bFound = true;
      }    
    }
  }
  return ulRetval;
}

    
/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
unsigned long MQUALIFIER TRemoteMethodCall::getParameter(long index, IZVariant** param) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (param != NULL)
  {
    IZVariant* pVar = m_lstArguments.getItemCopy(index);
    if (pVar != NULL)
    {
      pVar->addRef();
      *param = pVar;
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  return ulRetval;

}
    
/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
long MQUALIFIER TRemoteMethodCall::getParameterCount() const
{
  return m_lstArguments.getCount();
}

/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
unsigned long MQUALIFIER TRemoteMethodCall::getReturnValue(IZVariant** param) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (param != NULL)
  {
    if (m_pReturnValue != NULL)
    {
      *param = m_pReturnValue;
      m_pReturnValue->addRef();
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
unsigned long MQUALIFIER TRemoteMethodCall::setMethodName(IString* name)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_strMethdoName.assignStr(name);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteMethodCall
*/
/**************************************************************************/
unsigned long MQUALIFIER TRemoteMethodCall::setReturnValue(IZVariant* param)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (param != NULL)
  {
    if (m_pReturnValue != NULL)
    {
      m_pReturnValue->release();
    }
    m_pReturnValue = param;
    if (m_pReturnValue != NULL)
    {
      m_pReturnValue->addRef();
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TRemoteMethodCall);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
  INTERFACE_CAST(IRemoteMethodCall, INTERFACE_IRemoteMethodCall);
MEMORY_MANAGER_IMPL_END;
