/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : RemoteMethodCall                                              *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:39:16 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RemoteMethodCall.h,v $
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef RemoteMethodCallH
#define RemoteMethodCallH
 

#include <zeusbase/Remote/Interfaces/IRemoteMethodCall.hpp>
#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/Interfaces/IZVariant.hpp>
#include <zeusbase/System/ArrayList.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/StringMap.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! The remote method call class implements the way how to wrapp a method
    call and how to serialize it. This object will be serialized and sent 
    over a network to a remote destination, where it will be deserialized
    again and interpreted as a command object.
*/
/***************************************************************************/
class TRemoteMethodCall : public TZObject, public IRemoteMethodCall
{
  public:
    TRemoteMethodCall(TString name=L"");
    
    //Methods of IRemoteMethodCall
    virtual unsigned long MQUALIFIER addParameter(IZVariant* param);
    virtual unsigned long MQUALIFIER getMethodName(IString* name) const;
    virtual unsigned long MQUALIFIER getParameterByName(IString* name, IZVariant** param) const;
    virtual unsigned long MQUALIFIER getParameter(long index, IZVariant** param) const;
    virtual long MQUALIFIER getParameterCount() const;
    virtual unsigned long MQUALIFIER getReturnValue(IZVariant** param) const;
    virtual unsigned long MQUALIFIER setMethodName(IString* name);
    virtual unsigned long MQUALIFIER setReturnValue(IZVariant* param);
    
    //Methods of ISterializable
    SERIAL_START(TRemoteMethodCall, L"TRemoteMethodCall")
      SERIAL_STRING_SET_GET(setObjName, getObjName)
      SERIAL_STRING(m_strMethdoName)
      SERIAL_OBJECTLIST(IZVariant, m_lstArguments)
      SERIAL_OBJECT(IZVariant, m_pReturnValue)
    SERIAL_END

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TRemoteMethodCall();
    
  private:
    ///Name of the method
    TString m_strMethdoName;
    ///List of arguments
    TArrayList<IZVariant*> m_lstArguments;
    ///Return value
    IZVariant* m_pReturnValue;
};


 
END_NAMESPACE_Zeus

#endif
