/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : RemoteReference                                               *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:39:16 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RemoteReference.h,v $
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef RemoteReferenceH
#define RemoteReferenceH
 

#include <zeusbase/Remote/Interfaces/IRemoteObject.hpp>
#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! This class defines a remote object reference. This is used
    to access a remote object through networking. The parameters like 
    classname or code module name are used to create a stub on the
    client side
*/
/**************************************************************************/
class TRemoteReference : public TZObject, public ISerializable, private IRemoteObject
{
  public :
    TRemoteReference();
    TRemoteReference(IRemoteObject* object);
    TRemoteReference(TString address, long port, TString codemodule, TString classname);
    
    unsigned long getPort();
    TString getAddress();
    TString getCodeModule();
    TString getClassName();
    
    unsigned long createStub(IRemoteObject** obj);
        
    //Methods of ISterializable
    SERIAL_START(TRemoteReference, L"TRemoteReference")
      SERIAL_STRING(m_strAddress)
      SERIAL_LONG(m_ulPort)
      SERIAL_STRING(m_strCodeModuleName)
      SERIAL_STRING(m_strClassName)
    SERIAL_END
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TRemoteReference();
    
    //Methods of IRemoteObject
    virtual unsigned long MQUALIFIER getClassName(IString* name);
    virtual unsigned long MQUALIFIER getCodeModuleName(IString* name);
    virtual unsigned long MQUALIFIER getConnectionParam(IString* address, unsigned long* port);
    
    ///Class name of the remote object
    TString m_strClassName;
    ///name of the code module
    TString m_strCodeModuleName;
    ///Address of the remote object
    TString m_strAddress;
    ///Port of the remote object
    unsigned long m_ulPort;
    
  private:
    
};
 
END_NAMESPACE_Zeus

#endif
