/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FingerPrint                                                   *
 * Package : Security                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:43:51 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FingerPrint.h,v $
 *   Revision 1.2  2005/11/16 11:43:51  bhadorn
 *   Method added to limit the file size for generate the hash.
 *
 *   Revision 1.1  2005/11/10 10:41:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
//--------------------------------------------------------------------------

#ifndef FingerPrintH
#define FingerPrintH


#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/System/Interfaces/ISerializable.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This generates a finger print using the one way hash function 
    Whirlpool. 
*/
/***************************************************************************/
class TFingerPrint : public TZObject, public ISerializable
{
  public:
    TFingerPrint();
    virtual ~TFingerPrint();

    unsigned long generatePrint(const TString& keyname, const TString& data);
    unsigned long generatePrint(const TString& keyname, IByteArray* arr);
    unsigned long generatePrintFromFile(const TString& keyname, const TString& filename, unsigned long max_size =2000000);
    unsigned long getFingerPrint(IByteArray* value) const;
    TString getKeyName() const;
    
    bool operator==(const TFingerPrint& inpar) const;
    bool operator!=(const TFingerPrint& inpar) const;
    
    //Methods of ISerializable
    SERIAL_START(TFingerPrint, L"TFingerPrint")
      SERIAL_STRING(m_strKeyName)
      SERIAL_BOOL(m_bGenerated)
      SERIAL_BYTEARRAY(m_lstData)
    SERIAL_END
        
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
  protected:

  private:
    ///Data of Fingerprint
    TByteArray m_lstData;
    ///Flag is set if a finger print has been generated
    bool m_bGenerated;
    ///name of the finger print
    TString m_strKeyName;
    
    
    unsigned long generate(const TString& keyname, const unsigned char* buffer, long size);

};

END_NAMESPACE_Zeus
//---------------------------------------------------------------------------
#endif
