/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SecurityManager                                               *
 * Package : Security                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:43:51 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SecurityManager.h,v $
 *   Revision 1.2  2005/11/16 11:43:51  bhadorn
 *   Method added to limit the file size for generate the hash.
 *
 *   Revision 1.1  2005/11/10 10:41:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#ifndef SecurityManagerH
#define SecurityManagerH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/System/StringList.h>
#include <zeusbase/System/StringMap.hpp>
#include <zeusbase/Security/FingerPrint.h>

BEGIN_NAMESPACE_Zeus

/****************************************************************************/
/*! The security manager is responsible for all security maners of the 
    system
*/
/****************************************************************************/
class TSecurityManager : public TZObject
{
  public:
    static TSecurityManager& getInstance();
    
    void setMaxFileSize(unsigned long max_size);
    unsigned long getModules(TStringList* list);
    unsigned long loadRegistry(const TString& filename);
    unsigned long registerModule(const TString& name, const TString& path);
    unsigned long unregisterModule(const TString& name);
    
    bool eval(const TFingerPrint* print) const;
    bool evalCodeModule(const TString& name, const TString& path) const;
    
  protected:
    TSecurityManager();
    virtual ~TSecurityManager();
  
  private:
    ///Singleton instance
    static TSecurityManager m_Instance;
    ///Map of all registered fingerprints
    TStringMap<TFingerPrint*> m_mapReg;
    ///Flag will be set if the registry files has been loaded
    bool m_bLoaded;
    ///Flag will be set if the entries changed
    bool m_bChanged;
    ///Name of the data base
    TString m_strDataBaseName;
    ///MAx file size
    unsigned long m_ulMaxFileSize; 
    
    void closeRegistry();
    void checkAndCreateNewDB(const TString& filename);
};

END_NAMESPACE_Zeus
#endif
