/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Application                                                   *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:17:27 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Application.cpp,v $
 *   Revision 1.1  2005/11/10 10:17:27  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
#include <zeusbase/System/Application.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/LoggerManager.h>
 

#ifdef LINUX

  #include <iostream>
  #include <cstdlib>
  #include <signal.h>
  #include <sys/types.h>
  #include <fcntl.h>
  #include <unistd.h>
  #include <stdlib.h>
#else
  #include <windows.h>

#endif

USING_NAMESPACE_Zeus

//Initialize static variables
bool TApplication::m_bTerminate = false;



//##########################################################################
// LINUX SPECIFIC FUNCTIONS

#ifdef LINUX
/****************************************************************************/
/*! handler function
*/
/****************************************************************************/
static void sigProcessHandler(int signr)
{
  ZLOGGER.printfln(LOGMODE_DEBUG, "Application Signal received: %d", signr);
  TApplication::terminate();
}

/****************************************************************************/
/*! Installs the sighandler to receive the keyboard interrupt CTRL-C
*/
/****************************************************************************/
void initProcessSigHandler()
{
  sigset_t Mask;
  struct sigaction SigAction;

  sigemptyset(&Mask);
  SigAction.sa_handler = sigProcessHandler;
  SigAction.sa_mask = Mask;
  SigAction.sa_flags = 0;

  if (sigaction(SIGINT, &SigAction, NULL) < 0)
  {
    ZLOGGER.printfln(LOGMODE_WARN, "Unable to init sig handler for SIGINT");
  }
}

//##########################################################################


//##########################################################################
// WINDOWS SPECIFIC FUNCTIONS
#else


/****************************************************************************/
/*! Handles CTRL-C and Break- Events of a console application
*/
/****************************************************************************/
BOOL __stdcall breakRoutine(DWORD dwCtrlType)
{
  bool bRetval = false;
  if (dwCtrlType == CTRL_C_EVENT || dwCtrlType == CTRL_BREAK_EVENT)
  {
    bRetval = true;
    TApplication::terminate();
  }
  return bRetval;
}

#endif


/***************************************************************************/
/*! Creates an new application object
*/
/***************************************************************************/
TApplication::TApplication() : TZObject()
{
  #ifdef LINUX
    initProcessSigHandler();
  #else
    SetConsoleCtrlHandler(breakRoutine, true);
  #endif
}

/***************************************************************************/   
/*! Destroys the application object
*/
/***************************************************************************/   
TApplication::~TApplication()
{

}
 
/***************************************************************************/
/*! Starts the application and keeps it running, until Control + C is
    pressed
*/
/***************************************************************************/
void TApplication::start()
{
  while(!TApplication::m_bTerminate)
  {
    TTime::sleep(1);
  }
}


/***************************************************************************/
/*! Terminates the application
*/
/***************************************************************************/
void TApplication::terminate()
{
  TApplication::m_bTerminate = true;
}

/***************************************************************************/
/*! Checks if the application has been terminated
    \retval true : terminated
    \retval false: not terminated
*/
/***************************************************************************/
bool TApplication::isTerminated()
{
  return TApplication::m_bTerminate;
}

