/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ArrayList                                                     *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:33:11 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ArrayList.hpp,v $
 *   Revision 1.2  2005/11/25 09:33:11  bhadorn
 *   Corrected error using iterator and implemented const_iterator
 *
 *   Revision 1.1  2005/11/10 10:18:25  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ArrayListH
#define ArrayListH

#include <zeusbase/System/Interfaces/IArray.hpp>

#ifdef LINUX
  #include <string.h>
#else
  #include <mem.h>
#endif

BEGIN_NAMESPACE_Zeus

/*****************************************************************************/
/*! Template Klasse fuer Listen. Sie verwendet den abstrakten Datentypen
    IList. Die Listen sind als Single Linked List implementiert und
    verwalten einen Datentyp T.
*/
/*****************************************************************************/
template <class T> class TArrayList : public IArray<T>
{
  public:
    TArrayList(long size = 20);
    TArrayList(const T* array, long size);
    TArrayList(const T& emptyitem, long size);

    virtual ~TArrayList();

    ///Kopierkonstruktor
    TArrayList(const TArrayList<T>& list);
    TArrayList(const IArray<T>* list);

    //Methods of IArray
    virtual T* MQUALIFIER getArray() const;
    virtual long MQUALIFIER getArraySize() const;
    virtual unsigned long MQUALIFIER setArray(const T* array, long size);
    virtual unsigned long MQUALIFIER appendArray(const T* array, long size);
    virtual unsigned long MQUALIFIER copyToArray(IArray<T>* dest, long start, long end) const;

    //Methods of IList
    virtual long MQUALIFIER add(const T& Item);
    virtual void MQUALIFIER copyToList(IList<T>* list) const;
    virtual void MQUALIFIER clear();
    virtual unsigned long MQUALIFIER deleteItem(long Index);
    virtual unsigned long MQUALIFIER remove(const T& Item);
    virtual long MQUALIFIER getCount() const;
    virtual T& MQUALIFIER getItem(long index);
    virtual T MQUALIFIER getItemCopy(long Index) const;
    virtual bool MQUALIFIER equalsItem(long Index, const T& Item) const;
    virtual bool MQUALIFIER equals(const IList<T>* list) const;
    virtual long MQUALIFIER indexOf(const T& Item) const;
    virtual long MQUALIFIER insert(long Index, const T& Item);
    virtual IListIterator<T>* MQUALIFIER getIterator() const;
    virtual const IListIterator<T>* MQUALIFIER getConstIterator() const;
    virtual void MQUALIFIER releaseIterator(const IListIterator<T>* iterator) const;
    virtual bool MQUALIFIER isEmpty() const;

    //Interface IZUnknown
    virtual unsigned long MQUALIFIER askForInterface(long interface_id, IZUnknown** iface);
    virtual unsigned long MQUALIFIER addRef();
    virtual unsigned long MQUALIFIER release();

    //Operators
    T& operator[](long Index);
    TArrayList<T>& operator=(const TArrayList<T>& list);
    TArrayList<T>& operator=(const IArray<T>* list);
    bool operator==(const TArrayList<T>& list) const;
    T& getEmptyItem();

  private:

    /*********************************************************************/
    /*! Iterator der Liste
    */
    /*********************************************************************/
    class TArrayListIterator :public IListIterator<T>
    {
      public:
        /*****************************************************************/
        /*! Creates the ArrayList iterator
            \param parent : Parameter list
        */
        /*****************************************************************/
        TArrayListIterator(TArrayList* parent)
        {
          m_pParent = parent;
          m_pNextIterator = NULL;
          m_lActElement = 0;
          m_tEmpty = m_pParent->getEmptyItem();
        }

        /*****************************************************************/
        /*! Creates the ArrayList const iterator
            \param parent : Parameter list
        */
        /*****************************************************************/
        TArrayListIterator(const TArrayList* parent)
        {
          m_pParent = (TArrayList<T>*)parent;
          m_pNextIterator = NULL;
          m_lActElement = 0;
          m_tEmpty = m_pParent->getEmptyItem();
        }

        /*****************************************************************/
        /*! Destroys the iterator
        */
        /*****************************************************************/
        virtual ~TArrayListIterator()
        {}
        
        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual void MQUALIFIER reset() const
        {
          m_lActElement = 0;
        }

        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual T& MQUALIFIER getNextItem() const
        {
          if (this->hasNextItem())
          {
            m_lActElement++;
            return m_pParent->getItem(m_lActElement-1);
          }
          else
          {
            return m_tEmpty;
          }
        }

        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual const T& MQUALIFIER getNextItemConst() const
        {
          return (const T&)this->getNextItem();
        }

        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual bool MQUALIFIER hasNextItem() const
        {
          return (m_pParent != NULL && m_lActElement < m_pParent->getCount());
        }
        
        /*****************************************************************/
        /*! Ermitteln des n�hsten Iterator Pointers
        */
        /*****************************************************************/
        TArrayListIterator* getNextIterator() const
        {
          return m_pNextIterator;
        }

        /*****************************************************************/
        /*! Setzen des n�hsten Iterator Pointers
        */
        /*****************************************************************/
        void setNextIterator(TArrayListIterator* pIt)
        {
          m_pNextIterator = pIt;
        }

        /*****************************************************************/
        /*! Lesen der aktueller Elementposition
        */
        /*****************************************************************/
        long getActElement()
        {
          return m_lActElement;
        }

        /*****************************************************************/
        /*! Setzen der aktueller Elementposition
        */
        /*****************************************************************/
        void setActElement(long lElement)
        {
          m_lActElement = lElement;
        }
        
        /*****************************************************************/
        /*! Sets the parent to NULL
        */
        /*****************************************************************/
        void clearParent()
        {
          m_pParent = NULL;
        }

      private:
        mutable TArrayList* m_pParent;
        mutable TArrayListIterator* m_pNextIterator;
        mutable long m_lActElement;
        mutable T m_tEmpty;
    };

    ///Reference Counter
    unsigned long m_ulRefCounter;
    ///Array welches alle Daten beinhaltet
    T* m_pDataArray;
    ///Iterator Zeiger
    mutable TArrayListIterator* m_pIterator;
    ///Zaehler der Listenelemente
    long m_lCounter;
    ///Groesse der Liste
    long m_lSize;
    ///Originalgroesse
    long m_lOrigSize;
    ///Leeres Objekt bei Fehler
    T m_tEmpty;
    
    void resize(long newsize);
};


/******************************************************************************/
/*! Konstruktor der SingleLinked List
    \param size : Groesse des Speicherbereichs welcher alloziert werden soll
*/
/******************************************************************************/
template <class T> TArrayList<T>::TArrayList(long size)
{
  m_ulRefCounter = 1;
  m_lSize = size;
  m_lOrigSize = size;
  m_lCounter = 0;
  m_pIterator = NULL;
  m_pDataArray = new T[m_lSize];
  memset(&m_tEmpty, 0, sizeof(T));
}


/******************************************************************************/
/*! Creates an array list 
    \param array : Array to be wrapped
    \param size : Size of the array
*/
/******************************************************************************/
template <class T> TArrayList<T>::TArrayList(const T* array, long size)
{
  m_ulRefCounter = 1;
  m_lSize = size;
  m_lOrigSize = size;
  m_lCounter = 0;
  m_pIterator = NULL;
  m_pDataArray = new T[m_lSize];
  memset(&m_tEmpty, 0, sizeof(T));
  
  setArray(array, size);
}

/******************************************************************************/
/*! Bei diesem Konstruktor kann der Rckgabetyp gesetzt werden,
    wenn ein Indexfehler auftritt.
    \param emptyitem : Leeres Element welches beim Fehlerfall zurueckgegeben
                       wird.
    \param size : Groesse des Speicherbereichs welcher alloziert werden soll
*/
/******************************************************************************/
template <class T> TArrayList<T>::TArrayList(const T& emptyitem, long size)
{
  m_ulRefCounter = 1;
  m_lSize = size;
  m_lOrigSize = size;
  m_lCounter = 0;
  m_pIterator = NULL;
  m_pDataArray =  new T[m_lSize];
  m_tEmpty = emptyitem;
}

/******************************************************************************/
/*! Kopierkonstruktor
    \param list : ArrayList to cony
*/
/******************************************************************************/
template <class T> TArrayList<T>::TArrayList(const TArrayList<T>& list)
{
  m_ulRefCounter = 1;
  m_lSize = 20;
  m_lOrigSize = 20;
  m_lCounter = 0;
  m_pIterator = NULL;
  m_pDataArray = new T[m_lSize];
  memset(&m_tEmpty, 0, sizeof(T));
  this->setArray(list.getArray(), list.getCount());
}

/******************************************************************************/
/*! Kopierkonstruktor
    \param list : ArrayList to cony
*/
/******************************************************************************/
template <class T> TArrayList<T>::TArrayList(const IArray<T>* list)
{
  m_ulRefCounter = 1;
  m_lSize = 20;
  m_lOrigSize = 20;
  m_lCounter = 0;
  m_pIterator = NULL;
  m_pDataArray = new T[m_lSize];
  memset(&m_tEmpty, 0, sizeof(T));
  this->setArray(list->getArray(), list->getCount());
}


/******************************************************************************/
/*! Destruktor der Liste. Die Liste wird geloescht
*/
/******************************************************************************/
template <class T> TArrayList<T>::~TArrayList()
{
  //Reseten aller Iteratoren
  TArrayListIterator* it = m_pIterator;
  while (it != NULL)
  {
    it->clearParent();
    it = it->getNextIterator();
  }
  delete [] m_pDataArray;
}


/******************************************************************************/
/*! This returns the allocated size of the array. The difference between the 
    getCount() method and getArraySize() is, that getCount() returns the
    count of items in an array, and getArraySize() returns the size of the 
    array, witch is normaly bigger or equal to the count.
    \return array size:
*/
/******************************************************************************/
template <class T>long MQUALIFIER TArrayList<T>::getArraySize() const
{
  return m_lSize;
}


/******************************************************************************/
/*!  See IArray
*/
/******************************************************************************/
template <class T> T* MQUALIFIER TArrayList<T>::getArray() const
{
  return m_pDataArray;
}

/******************************************************************************/
/*!  See IArray
*/
/******************************************************************************/
template <class T> unsigned long MQUALIFIER TArrayList<T>::setArray(const T* array, long size)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  //Speciality: Sets the original size to the new size required and call clear, witch will
  // free the actual array and allocate the array with original size
  m_lOrigSize = size+2; 
  this->clear();
  ///////////////
  
  if (array == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    ::memcpy(m_pDataArray, array, sizeof(T)*size);
    m_lCounter = size;
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}

/******************************************************************************/
/*!  See IArray
*/
/******************************************************************************/
template <class T>unsigned long MQUALIFIER TArrayList<T>::appendArray(const T* array, long size)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (array == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    if (m_lCounter + size >= m_lSize)
    {
      resize(m_lSize + size);
    }

    ::memcpy(&m_pDataArray[m_lCounter], array, sizeof(T)*size);
    m_lCounter += size;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/******************************************************************************/
/*!  See IArray
*/
/******************************************************************************/
template <class T>unsigned long MQUALIFIER TArrayList<T>::copyToArray(IArray<T>* dest, long start, long end) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (dest == NULL || start > end)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (start < 0 || end >= m_lCounter)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else
  {
    long lArraySize = end - start + 1;
    T* pArr = new T[lArraySize];
    ::memcpy(pArr, &(m_pDataArray[start]), sizeof(T)*lArraySize);
    dest->setArray(pArr, lArraySize);
    ulRetval = RET_NOERROR;
    delete [] pArr;
  }
  return ulRetval;
}


/******************************************************************************/
/*!  Resizes the array 
     \param newsize : New size of the array list
*/
/******************************************************************************/
template <class T>void TArrayList<T>::resize(long newsize)
{
  m_lSize = newsize;
  T* pNewArray = new T[m_lSize];
  for(int i = 0; i < m_lCounter; i++)
  {
    pNewArray[i] = m_pDataArray[i];
  }
  delete [] m_pDataArray;
  m_pDataArray = pNewArray;
}


/******************************************************************************/
/*!  See IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TArrayList<T>::add(const T& Item)
{

  //Vergroessern des Arrays
  if (m_lCounter >= m_lSize)
  {
    resize((long)(m_lSize*1.5));
  }

  m_pDataArray[m_lCounter] = Item;
  m_lCounter++;

  return m_lCounter-1;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> void MQUALIFIER TArrayList<T>::copyToList(IList<T>* list) const
{
  if (list != NULL)
  {
    list->clear();
  
    long lCounter = 0;
    while (lCounter < m_lCounter)
    {
      list->add(m_pDataArray[lCounter]);
      lCounter++;
    }
  }
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> void MQUALIFIER TArrayList<T>::clear()
{
  delete [] m_pDataArray;
  m_pDataArray = new T[m_lOrigSize];
  m_lSize = m_lOrigSize;

  //Zuruecksetzen der Liste
  m_lCounter = 0;

  //Reseten aller Iteratoren
  TArrayListIterator* it = m_pIterator;
  while (it != NULL)
  {
    it->reset();
    it = it->getNextIterator();
  }
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> unsigned long MQUALIFIER TArrayList<T>::deleteItem(long Index)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;

  if (Index >= 0 && Index < m_lCounter)
  {
    for(int i = Index; i < m_lCounter-1; i++)
    {
      m_pDataArray[i] = m_pDataArray[i+1];
    }
    m_lCounter--;

    //Anpassen aller Iteratoren die den aktuellen Index bereits ueberstiegen
    //haben
    TArrayListIterator* it = m_pIterator;
    while (it != NULL)
    {
      //Alle Iteratoren welche das aktuelle Element groesser als Index haben
      if (it->getActElement() > Index)
      {
        it->setActElement(it->getActElement()-1);
      }
      it = it->getNextIterator();
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TArrayList<T>::indexOf(const T& Item) const
{
  long lCounter = 0;
  while (lCounter < m_lCounter)
  {
    if (m_pDataArray[lCounter] == Item)
    {
      return lCounter;
    }
    lCounter++;
  }
  return -1;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TArrayList<T>::insert(long Index, const T& Item)
{
  if (Index >= 0)
  {
    long lRetval = -1;

    //Vergroessern des Arrays
    if (m_lCounter >= m_lSize)
    {
      resize((long)(m_lSize*1.5));
    }

    //Anfuegen eines Items
    if (Index == m_lCounter)
    {
      m_pDataArray[m_lCounter] = Item;
      m_lCounter++;
      lRetval = m_lCounter-1;
    }

    //Einfuegen
    else if (Index < m_lCounter)
    {
      for(int i = m_lCounter-1; i >= Index; i--)
      {
        m_pDataArray[i+1] = m_pDataArray[i];
      }
      m_pDataArray[Index] = Item;
      m_lCounter++;
      lRetval = Index;
    }

    //Anpassen aller Iteratoren die den aktuellen Index bereits ueberstiegen
    //haben
    TArrayListIterator* it = m_pIterator;
    while (it != NULL)
    {
      //Alle Iteratoren welche das aktuelle Element groesser als Index haben
      if (it->getActElement() >= Index)
      {
        it->setActElement(it->getActElement()+1);
      }
      it = it->getNextIterator();
    }

    return lRetval;
  }
  else
  {
    return -1;
  }
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> unsigned long MQUALIFIER TArrayList<T>::remove(const T& Item)
{
  return deleteItem(indexOf(Item));
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TArrayList<T>::getCount() const
{
  return  m_lCounter;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> T& MQUALIFIER TArrayList<T>::getItem(long Index)
{
  if (Index >= 0 && Index < m_lCounter)
  {
    return m_pDataArray[Index];
  }
  else
  {
    return m_tEmpty;
  }
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
template <class T> T MQUALIFIER TArrayList<T>::getItemCopy(long Index) const
{
  if (Index >= 0 && Index < m_lCounter)
  {
    return m_pDataArray[Index];
  }
  else
  {
    return m_tEmpty;
  }
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> bool MQUALIFIER TArrayList<T>::equalsItem(long Index, const T& Item) const
{
  return (Index >= 0 && Index < m_lCounter && getItemCopy(Index) == Item);
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> bool MQUALIFIER TArrayList<T>::equals(const IList<T>* list) const
{
  bool bRetval = false;
  if (list != NULL)
  {
    bRetval = (list->getCount() == m_lCounter);
    
    if (bRetval)
    {
      for(int i = 0; i < m_lCounter && bRetval; i++)
      {
        bRetval &= list->equalsItem(i,m_pDataArray[i]);
      }
    }
  }
  return bRetval;
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> IListIterator<T>* MQUALIFIER TArrayList<T>::getIterator() const
{
  TArrayListIterator* iterator = new TArrayListIterator(this);
  iterator->setNextIterator(m_pIterator);
  m_pIterator = iterator;
  return m_pIterator;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> const IListIterator<T>* MQUALIFIER TArrayList<T>::getConstIterator() const
{
  TArrayListIterator* iterator = new TArrayListIterator(this);
  iterator->setNextIterator(m_pIterator);
  m_pIterator = iterator;
  return m_pIterator;
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> void MQUALIFIER TArrayList<T>::releaseIterator(const IListIterator<T>* iterator) const
{
  TArrayListIterator* it = m_pIterator;
  TArrayListIterator* it_prev = NULL;
  while (it != NULL)
  {
    if (it == iterator)
    {
      //Erster Iterator loeschen
      if (it_prev == NULL)
      {
        m_pIterator = it->getNextIterator();
        delete it;
      }
      else
      {
        it_prev->setNextIterator(it->getNextIterator());
        delete it;
      }
      break;
    }
    it_prev = it;
    it = it->getNextIterator();
  }
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> bool MQUALIFIER TArrayList<T>::isEmpty() const
{
  return (m_lCounter == 0);
}

/******************************************************************************/
/*!
    Operator zur fr das einfache Zugreifen auf den Elementinhalt
    \param Index : Index auf ein Element der Liste
    \return gibt den Dateninhalt eines Elements der Liste zurck
*/
/******************************************************************************/
template <class T> T& TArrayList<T>::operator[](long Index)
{
  return getItem(Index);
}

/******************************************************************************/
/*! Operator zum kopieren einer Liste
    \param list : Ausgangsliste
    \return neu kopierte Liste
*/
/******************************************************************************/
template <class T> TArrayList<T>& TArrayList<T>::operator=(const TArrayList<T>& list)
{
  list.copyToList(this);
  return *this;
}

/******************************************************************************/
/*! Operator zum kopieren einer Liste
    \param list : Ausgangsliste
    \return neu kopierte Liste
*/
/******************************************************************************/
template <class T> TArrayList<T>& TArrayList<T>::operator=(const IArray<T>* list)
{
  if (list != NULL)
  {
    list->copyToArray(this, 0, list->getCount()-1);
  }
  return *this;
}


/******************************************************************************/
/*! Operator to check equality of two lists
    \param list : list to check
    \retval true : Equal
    \retval false : not equal
*/
/******************************************************************************/
template <class T> bool  TArrayList<T>::operator==(const TArrayList<T>& list) const
{
  return this->equals(&list);
}

/***************************************************************************/
/*! Returns the empty data element
*/
/***************************************************************************/
template <class T> T& TArrayList<T>::getEmptyItem()
{
  return m_tEmpty;
}


/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
template <class T> unsigned long MQUALIFIER TArrayList<T>::askForInterface(long interface_id, IZUnknown** iface)
{
  unsigned long ulRetval = RET_NOERROR;
  if (iface == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    switch(interface_id)
    {
      default:
        ulRetval = RET_UNKNOWN_INTERFACE;
    }
  }

  return ulRetval;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
template <class T> unsigned long MQUALIFIER TArrayList<T>::addRef()
{
  ++m_ulRefCounter;
  return RET_NOERROR;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
template <class T> unsigned long MQUALIFIER TArrayList<T>::release()
{
  --m_ulRefCounter;
  if (m_ulRefCounter <= 0)
  {
    delete this;
  }
  return RET_NOERROR;
}





END_NAMESPACE_Zeus

#endif

