/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ByteArrayInputStream                                          *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:54:37 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ByteArrayInputStream.cpp,v $
 *   Revision 1.3  2005/11/25 09:54:37  bhadorn
 *   Eliminate compiler warnings
 *
 *   Revision 1.2  2005/11/16 11:27:05  bhadorn
 *   Changed parameter of constructor to [const].
 *
 *   Revision 1.1  2005/11/10 10:18:25  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/ByteArrayInputStream.h>
#include <zeusbase/System/Platforms/STLBindings.hpp>
#include <stdio.h>
 
USING_NAMESPACE_Zeus 
 
/***************************************************************************/
/*! Creates an object streamer
*/
/***************************************************************************/
TByteArrayInputStream::TByteArrayInputStream() : TZObject()
{
  m_pBuffer = new std::stringbuf();
  m_pStream = new std::iostream(m_pBuffer);
}

/***************************************************************************/
/*! Creates an object streamer and sets the stream
    \param array : binary stream ob object data
*/
/***************************************************************************/
TByteArrayInputStream::TByteArrayInputStream(const IByteArray* array): TZObject()
{
  m_pBuffer = new std::stringbuf();
  m_pStream = new std::iostream(m_pBuffer);
  if (array->getCount() > 0)
  {
    m_pStream->write(array->getArray(), array->getCount());
  }
}

/***************************************************************************/
/*!Destroys the object streamer
*/
/***************************************************************************/
TByteArrayInputStream::~TByteArrayInputStream()
{
  delete m_pStream;
  delete m_pBuffer;
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
bool MQUALIFIER TByteArrayInputStream::available() const
{ return true; }


/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
bool MQUALIFIER TByteArrayInputStream::isEmpty() const
{
  return m_pStream->eof();
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayInputStream::read(char* buffer, long buffer_size, long* valid_size) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (buffer == NULL || valid_size == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (!this->isEmpty())
  {
    char* pBuffer = new char[buffer_size];
    ::memset(pBuffer, 0, sizeof(char)*buffer_size);
      
    m_pStream->read(pBuffer, buffer_size);
    *valid_size = m_pStream->gcount();
    ::memcpy(buffer, pBuffer, sizeof(char)*buffer_size);
    delete [] pBuffer;    
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
double MQUALIFIER TByteArrayInputStream::readDouble() const
{
  return TSTLBindings::readDouble(m_pStream);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
long MQUALIFIER TByteArrayInputStream::readLong() const
{
  return TSTLBindings::readLong(m_pStream);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
char MQUALIFIER TByteArrayInputStream::readByte() const
{
  return (char)m_pStream->get();
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
bool MQUALIFIER TByteArrayInputStream::readBool() const
{
  bool bRetval = m_pStream->get();
  return bRetval;
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayInputStream::readArray(IByteArray* data) const
{
  return TSTLBindings::readArray(m_pStream, data);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayInputStream::readString(IString* data) const
{
  return TSTLBindings::readString(m_pStream, data);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
void MQUALIFIER TByteArrayInputStream::close()
{}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TByteArrayInputStream);
  INTERFACE_CAST(IInputStream, INTERFACE_IInputStream);
MEMORY_MANAGER_IMPL_END;
