/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ByteArrayOutputStream                                         *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:18:25 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ByteArrayOutputStream.cpp,v $
 *   Revision 1.1  2005/11/10 10:18:25  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/ByteArrayOutputStream.h>
 
USING_NAMESPACE_Zeus 
 
/***************************************************************************/
/*! Creates an object streamer
*/
/***************************************************************************/
TByteArrayOutputStream::TByteArrayOutputStream() : TZObject()
{
  m_pBuffer = new std::stringbuf();
  m_pStream = new std::iostream(m_pBuffer);
}

/***************************************************************************/
/*!Destroys the object streamer
*/
/***************************************************************************/
TByteArrayOutputStream::~TByteArrayOutputStream()
{
  delete m_pStream;
  delete m_pBuffer;
}
 
/***************************************************************************/
/*! Returns the stream of an object. The data will be removed from the 
    stream
    \param array : Return parameter of stream data
    \retval RET_NOERROR : Stream returned
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_REQUEST_FAILED : No stream available
*/
/***************************************************************************/
unsigned long TByteArrayOutputStream::popData(IByteArray* array)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (array == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  } 
  else
  {
    long lSize = m_pBuffer->in_avail();
    if (lSize > 0)
    {
      char* pBuffer = new char[lSize];
      m_pStream->read(pBuffer, lSize);
      
      array->setArray(pBuffer, lSize);
      delete [] pBuffer;
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayOutputStream::write(char* buffer, long buffer_size)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (buffer == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (buffer_size > 0)
  {
    m_pStream->write(buffer, sizeof(char)*buffer_size);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayOutputStream::writeDouble(double data)
{
  m_pStream->write((char*)&data, sizeof(double));
  return RET_NOERROR;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayOutputStream::writeLong(long data)
{
  m_pStream->write((char*)&data, sizeof(long));
  return RET_NOERROR;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayOutputStream::writeByte(char data)
{
  m_pStream->put(data);
  return RET_NOERROR;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayOutputStream::writeBool(bool data)
{
  m_pStream->put(data);
  return RET_NOERROR;
}


/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayOutputStream::writeArray(const IByteArray* data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    this->writeLong(data->getCount()); //Size of the array
    m_pStream->write(data->getArray(), data->getCount());
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TByteArrayOutputStream::writeString(const IString* data)
{
  TString strData(data);
  this->writeLong(strData.getSize()+1); //Size of the string
  m_pStream->write(strData.c_str(), strData.getSize()+1);
  return RET_NOERROR;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
void MQUALIFIER TByteArrayOutputStream::close()
{}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
void MQUALIFIER TByteArrayOutputStream::flush()
{}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TByteArrayOutputStream);
  INTERFACE_CAST(IOutputStream, INTERFACE_IOutputStream);
MEMORY_MANAGER_IMPL_END;
