/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CriticalSection                                               *
 * Package : System                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:18:25 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CriticalSection.h,v $
 *   Revision 1.1  2005/11/10 10:18:25  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CriticalSectionH
#define CriticalSectionH

#include <zeusbase/System/Interfaces/ICriticalSection.hpp>
#include <zeusbase/System/ZObject.h>

#ifdef LINUX
  #include <pthread.h>
#else
  #include <Windows.h>
#endif

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This class implements a critical section (mutex) for multithreading 
*/
/***************************************************************************/
class TCriticalSection : public TZObject, public ICriticalSection
{
  public:
    enum TLockType {ltFast, ltRecursive, ltError};
    TCriticalSection(TLockType type = ltFast);
    
    bool isLocked();
    
    //Methods of ICriticalSection
    virtual void MQUALIFIER enter();
    virtual void MQUALIFIER leave();
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    virtual ~TCriticalSection();
  
  private:
    ///Counter
    long m_lCount;
    
    #ifdef LINUX
      ///Mutex
      pthread_mutex_t m_Mutex;
      ///Mutex attribute
      pthread_mutexattr_t m_Attribute;
    #else
      CRITICAL_SECTION m_CriticalSection;
    #endif
};

END_NAMESPACE_Zeus

#endif
