/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Directory                                                     *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:18:25 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Directory.cpp,v $
 *   Revision 1.1  2005/11/10 10:18:25  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/System/Directory.h> 

#ifdef LINUX
  #include <unistd.h>
  #include <stdlib.h>
  #include <errno.h>
  #include <sys/types.h>
#else
  #include <dir.h>
  #include <stdlib.h>
  #include <errno.h>
  #include <sys/types.h>
#endif


USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates an new Directory wrapper object
*/
/***************************************************************************/
TDirectory::TDirectory(TString abs_name) : TZObject()
{
  m_strAbsName = abs_name;
}


/***************************************************************************/
/*! Destroys the Directory wrapper object
*/
/***************************************************************************/
TDirectory::~TDirectory()
{

}


/***************************************************************************/
/*! Returns the absolute name of the directory
*/
/***************************************************************************/
TString TDirectory::getAbsoluteName()
{
  return m_strAbsName;
}


/***************************************************************************/
/*! Static method to receive the actual directory
*/
/***************************************************************************/
unsigned long TDirectory::getCurrentDirectory(TDirectory** dir)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (dir == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    
    //Try to get directory name
    char* cBuffer = NULL;
    size_t lSize = 200;
    bool bBreak = false;
    while(!bBreak)
    {
      cBuffer = new char[lSize];
      if (::getcwd(cBuffer, lSize) == cBuffer)
      {
        bBreak = true;
      }
      else
      {
        delete [] cBuffer;
        cBuffer = NULL;
        if (errno == ERANGE)
        {
          bBreak = true;
        }
        else
        {
          lSize *= 3;
        }
      }
    }
    
    //Check if Name is valid
    if (cBuffer != NULL)
    {    
      TString strDirName = TDirectory::completePath(cBuffer);
      
      *dir = new TDirectory(strDirName);
    
      delete [] cBuffer;
      cBuffer = NULL;
      ulRetval = RET_NOERROR;    
    }    
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
    }
  }
  
  return ulRetval;
}


/***************************************************************************/
/*! This method completes a file path. The file path must end with the 
    PATH_DELIMITER. 
*/
/***************************************************************************/
TString TDirectory::completePath(TString path)
{
  if (path.getChar(path.getSize()-1) != PATH_DELIMITER)
  {
    wchar_t cBuffer[2] = {PATH_DELIMITER, L'\0'};
    path.concat(cBuffer);
  }
  
  return path;
}
