/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus                                                          *
 * Module  : Event                                                         *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:04:28 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Event.cpp,v $
 *   Revision 1.3  2005/12/14 09:04:28  bhadorn
 *   errors for windows compilation corrected
 *
 *   Revision 1.2  2005/11/25 09:51:22  bhadorn
 *   Eliminate compiler warnings
 *
 *   Revision 1.1  2005/11/10 10:18:25  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/Event.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new event
*/
/***************************************************************************/
TEvent::TEvent() : TZObject()
{
  #ifdef LINUX
  #else
    m_hHandle = ::CreateEvent(NULL, true, false, NULL);
  #endif
}

/***************************************************************************/
/*! Destroys the event
*/
/***************************************************************************/
TEvent::~TEvent()
{
  #ifdef LINUX
  #else
    ::CloseHandle(m_hHandle);
  #endif
}

/***************************************************************************/
/*! set signal
*/
/***************************************************************************/
void TEvent::set()
{
  #ifdef LINUX
  #else
    ::SetEvent(m_hHandle);
  #endif
}

/***************************************************************************/
/*! reset the event
*/
/***************************************************************************/
void TEvent::reset()
{
  #ifdef LINUX
  #else
    ::ResetEvent(m_hHandle);
  #endif
}

/***************************************************************************/
/*! Wait for a signalification
    \param sec : time to wait.
*/
/***************************************************************************/
bool TEvent::wait(double sec)
{
  bool bRetval = false;

  #ifdef LINUX
    sec;
  #else
    bRetval = (::WaitForSingleObject(m_hHandle, (unsigned long)(sec*1000)) != WAIT_TIMEOUT);
  #endif

  return bRetval;
}

