/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FileInputStream                                               *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:49:21 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FileInputStream.cpp,v $
 *   Revision 1.2  2005/11/25 09:49:21  bhadorn
 *   Changed open mode to be win32 compatible
 *
 *   Revision 1.1  2005/11/10 10:18:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/FileInputStream.h>
#include <zeusbase/System/Platforms/STLBindings.hpp>


USING_NAMESPACE_Zeus


/***************************************************************************/
/*! Creates an input stream opening a file 
    \param filename : Name of the file
*/
/***************************************************************************/
TFileInputStream::TFileInputStream(TString filename)
{
  m_pFile = new TFile(filename);
  m_pStream = new std::ifstream(m_pFile->getAbsoluteName().c_str());
  //m_pStream->setmode(std::filebuf::binary);
  m_bAvailable = m_pFile->exists();
  m_bOpen = m_pStream->is_open();
}


/***************************************************************************/
/*! Creates an input stream opening a file 
    \param file : File object
*/
/***************************************************************************/
TFileInputStream::TFileInputStream(TFile* file)
{
  m_pFile = file;
  if (m_pFile != NULL)
  {
    m_pFile->addRef();
    m_pStream = new std::ifstream(m_pFile->getAbsoluteName().c_str());
    //m_pStream->setmode(std::filebuf::binary);
    m_bAvailable = m_pFile->exists();
    m_bOpen = m_pStream->is_open();
  }
  else
  {
    m_pStream = new std::ifstream();
    m_bAvailable = false;
    m_bOpen = false;
  }
}

/***************************************************************************/
/*! Destroys the file stream
*/
/***************************************************************************/
TFileInputStream::~TFileInputStream()
{
  close();
  if (m_pFile != NULL)
  {
    m_pFile->release();
  }
  delete m_pStream;
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
bool MQUALIFIER TFileInputStream::available() const
{ return m_bAvailable; }

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
bool MQUALIFIER TFileInputStream::isEmpty() const
{
  return m_pStream->eof();
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileInputStream::read(char* buffer, long buffer_size, long* valid_size) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if(buffer == NULL || valid_size == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (buffer_size == 0)
  {
    ulRetval = RET_INVALID_DATA;
  }
  else if (m_bAvailable && m_bOpen && !isEmpty())
  {
    m_pStream->read(buffer, buffer_size);
    *valid_size = m_pStream->gcount();
    if (*valid_size == -1)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
    else
    {
      ulRetval = RET_NOERROR;
    }
  }  
  return ulRetval;
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
double MQUALIFIER TFileInputStream::readDouble() const
{
  return TSTLBindings::readDouble(m_pStream);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
long MQUALIFIER TFileInputStream::readLong() const
{
  return TSTLBindings::readLong(m_pStream);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
char MQUALIFIER TFileInputStream::readByte() const
{
  return (char)m_pStream->get();
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
bool MQUALIFIER TFileInputStream::readBool() const
{
  return m_pStream->get();
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileInputStream::readArray(IByteArray* data) const
{
  return TSTLBindings::readArray(m_pStream, data);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileInputStream::readString(IString* data) const
{  
  return TSTLBindings::readString(m_pStream, data);
}

/***************************************************************************/
/*! See IInputStream
*/
/***************************************************************************/
void MQUALIFIER TFileInputStream::close()
{
  if (m_bOpen)
  {
    m_pStream->close();
  }
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TFileInputStream);
  INTERFACE_CAST(IInputStream, INTERFACE_IInputStream);
MEMORY_MANAGER_IMPL_END;

