/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FileInputStream                                               *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:18:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FileInputStream.h,v $
 *   Revision 1.1  2005/11/10 10:18:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef FileInputStreamH
#define FileInputStreamH

#include <zeusbase/System/Interfaces/IInputStream.hpp>
#include <zeusbase/System/String.h>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/File.h>
#include <fstream>
#include <stdio.h>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Implements a input stream to read out of a file
*/
/***************************************************************************/
class TFileInputStream : public TZObject, public IInputStream
{
  public:
    TFileInputStream(TString filename);
    TFileInputStream(TFile* file);
    
    //Methods of IInputStream
    virtual bool MQUALIFIER available() const;
    virtual bool MQUALIFIER isEmpty() const;
    virtual unsigned long MQUALIFIER read(char* buffer, long buffer_size, long* valid_size) const;
    virtual double MQUALIFIER readDouble() const;
    virtual long MQUALIFIER readLong() const;
    virtual char MQUALIFIER readByte() const;
    virtual bool MQUALIFIER readBool() const;
    virtual unsigned long MQUALIFIER readArray(IByteArray* data) const;
    virtual unsigned long MQUALIFIER readString(IString* data) const;
    virtual void MQUALIFIER close();
  
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TFileInputStream();
  
  private:
    ///File instance
    TFile* m_pFile;
    ///Input File stream object
    std::ifstream* m_pStream;
    ///flag to check if stream is open
    bool m_bOpen;
    ///flag to check if stream is valid
    bool m_bAvailable;
};


END_NAMESPACE_Zeus

#endif
