/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FileOutputStream                                         *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:49:21 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FileOutputStream.cpp,v $
 *   Revision 1.2  2005/11/25 09:49:21  bhadorn
 *   Changed open mode to be win32 compatible
 *
 *   Revision 1.1  2005/11/10 10:18:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/FileOutputStream.h>
 
USING_NAMESPACE_Zeus 
 
/***************************************************************************/
/*! Creates an output file stream
    \param filename : Name of the file to open
    \param append : if set the data will be appended to the existing 
                     content of the file
*/
/***************************************************************************/
TFileOutputStream::TFileOutputStream(TString filename, bool append /*=true*/)
{
  m_bAppend = append;
  std::ios_base::openmode iMode = std::ios::out;
  //std::_Ios_Openmode iMode = std::ios::out;
  if (m_bAppend)
  {
    iMode = std::ios::app;
  }
  m_pFile = new TFile(filename);
  m_pStream = new std::ofstream(m_pFile->getAbsoluteName().c_str(), iMode);
  //m_pStream->setmode(std::filebuf::binary);
  m_bAvailable = !m_pStream->fail();
  m_bOpen = m_pStream->is_open();
}

/***************************************************************************/
/*! Creates an output file stream
    \param file : File object
    \param append : if set the data will be appended to the existing 
                     content of the file
*/
/***************************************************************************/
TFileOutputStream::TFileOutputStream(TFile* file, bool append /*=true*/)
{
  m_bAppend = append;
  std::ios_base::openmode iMode = std::ios::out;
//  std::_Ios_Openmode iMode = std::ios::out;
  if (m_bAppend)
  {
    iMode = std::ios::app;
  }
  m_pFile = file;
  if (m_pFile != NULL)
  {
    m_pFile->addRef();
    m_pStream = new std::ofstream(m_pFile->getAbsoluteName().c_str(), iMode);
    //m_pStream->setmode(std::filebuf::binary);
    m_bAvailable = !m_pStream->fail();
    m_bOpen = m_pStream->is_open();
  }
  else
  {
    m_pStream = new std::ofstream();
    m_bAvailable = false;
    m_bOpen = false;
  }
}

/***************************************************************************/
/*! Destroys the output file stream
*/
/***************************************************************************/
TFileOutputStream::~TFileOutputStream()
{
  close();
  if (m_pFile != NULL)
  {
    m_pFile->release();
  }
  delete m_pStream;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileOutputStream::write(char* buffer, long buffer_size)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if(buffer == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (buffer_size == 0)
  {
    ulRetval = RET_INVALID_DATA;
  }
  else if (m_bAvailable && m_bOpen)
  {
    m_pStream->write(buffer, buffer_size);
    ulRetval = RET_NOERROR;
  }  
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileOutputStream::writeDouble(double data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bAvailable && m_bOpen)
  {
    m_pStream->write((char*)&data, sizeof(double));
    ulRetval =  RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileOutputStream::writeLong(long data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bAvailable && m_bOpen)
  {
    m_pStream->write((char*)&data, sizeof(long));
    ulRetval =  RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileOutputStream::writeByte(char data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bAvailable && m_bOpen)
  {
    m_pStream->put(data);
    ulRetval =  RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileOutputStream::writeBool(bool data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bAvailable && m_bOpen)
  {
    m_pStream->put(data);
    ulRetval =  RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileOutputStream::writeArray(const IByteArray* data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bAvailable && m_bOpen)
  {
    this->writeLong(data->getCount()); //Size of the array
    m_pStream->write(data->getArray(), data->getCount());
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
unsigned long MQUALIFIER TFileOutputStream::writeString(const IString* data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bAvailable && m_bOpen)
  {
    TString strData(data);
    this->writeLong(strData.getSize()+1); //Size of the string
    m_pStream->write(strData.c_str(), strData.getSize()+1);
    ulRetval =  RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
void MQUALIFIER TFileOutputStream::close()
{
  if (m_bOpen)
  {
    m_pStream->close();
  }
}

/***************************************************************************/
/*! See IOutputStream
*/
/***************************************************************************/
void MQUALIFIER TFileOutputStream::flush()
{
  m_pStream->flush();
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TFileOutputStream);
  INTERFACE_CAST(IOutputStream, INTERFACE_IOutputStream);
MEMORY_MANAGER_IMPL_END;
 
