/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FileOutputStream                                              *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:18:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FileOutputStream.h,v $
 *   Revision 1.1  2005/11/10 10:18:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef FileOutputStreamH
#define FileOutputStreamH

#include <zeusbase/System/Interfaces/IOutputStream.hpp>
#include <zeusbase/System/String.h>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/File.h>
#include <fstream>
#include <stdio.h>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Implements an output stream to write into a file
*/
/***************************************************************************/
class TFileOutputStream : public TZObject, public IOutputStream
{
  public:
    TFileOutputStream(TString filename, bool append=true);
    TFileOutputStream(TFile* file, bool append=true);
    
    //Methods of IOutputStream
    virtual unsigned long MQUALIFIER write(char* buffer, long buffer_size);
    virtual unsigned long MQUALIFIER writeDouble(double data);
    virtual unsigned long MQUALIFIER writeLong(long data);
    virtual unsigned long MQUALIFIER writeByte(char data);
    virtual unsigned long MQUALIFIER writeBool(bool data);
    virtual unsigned long MQUALIFIER writeArray(const IByteArray* data);
    virtual unsigned long MQUALIFIER writeString(const IString* data);
    virtual void MQUALIFIER close();
    virtual void MQUALIFIER flush();
  
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TFileOutputStream();
  
  private:
    ///File instance
    TFile* m_pFile;
    ///Input File stream object
    std::ofstream* m_pStream;
    ///flag to check if stream is open
    bool m_bOpen;
    ///flag to check if stream is valid
    bool m_bAvailable;
    ///flag for appending data
    bool m_bAppend;
};


END_NAMESPACE_Zeus

#endif
 
