/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IList                                                         *
 * Package : IList                                                         *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IArray.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IArrayHPP
#define IArrayHPP

#include <zeusbase/System/Interfaces/IList.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines an array, witch also inherites the behavoir of
    a list.
*/
/***************************************************************************/
template <class T> class IArray : public IList<T>
{
  public:
    /***********************************************************************/
    /*! Returns the array of the data
        \return array
    */
    /***********************************************************************/
    virtual T* MQUALIFIER getArray() const =0;
    
    /***********************************************************************/
    /*! Returns the size of the valid array data
        \return size
    */
    /***********************************************************************/
    virtual long MQUALIFIER getArraySize() const =0;
    
    /***********************************************************************/
    /*! Sets a new array of data. The array will be copied into the 
        member of the arraylist 
        \param array : pointer to the array of data
        \param size : Size of the array
        \retval RET_NOERROR : Array copied
        \retval RET_REQUEST_FAILED : Could not copy this array
        \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setArray(const T* array, long size)=0;
    
    /***********************************************************************/
    /*! This appends an array to the current array.
        \param array : Array of Data
        \param size : Size of the array
        \retval RET_NOERROR : Data appended
        \retval RET_REQUEST_FAILED: Could not append data.
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER appendArray(const T* array, long size)=0;
    
    /***********************************************************************/
    /*! Copies a subsequence of this array into the destination
        \param dest : Destination array
        \param start : Start index of the first element
        \param end : End index of the last element
        \retval RET_NOERROR : Array copied
        \retval RET_REQUEST_FAILED : Could not copy array
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER copyToArray(IArray<T>* dest, long start, long end) const =0;
};

END_NAMESPACE_Zeus

#endif
