/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IInputStream                                                  *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IInputStream.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IInputStreamHPP
#define IInputStreamHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IByteArray.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>

BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! Interface is defining a input stream
*/
/**************************************************************************/
class IInputStream : public IZUnknown
{
  public:
    /**********************************************************************/
    /*! Returns if the input stream is available (valid). 
        \retval true : stream is ok
        \retval false : invalid stream
    */
    /**********************************************************************/
    virtual bool MQUALIFIER available() const=0;
    
    /**********************************************************************/
    /*! This method checks if the stream is empty. If a stream is empty
        the methods readDouble(), readLong(), readByte() and readBool()
        should not called anymore. They will return zero. 
        You must call this method before getting data from the stream.
        \retval true : Stream is empty
        \retval false : Stream is not empty
    */
    /**********************************************************************/
    virtual bool MQUALIFIER isEmpty() const=0;
    
    /**********************************************************************/
    /*! Reads from a stream into a buffer
        \param buffer : Char Array
        \param buffer_size : Size of the Array
        \param valid_size : Length of the read buffer (return parameter)
        \retval RET_NOERROR : Reading was successful
        \retval RET_REQUEST_FAILED : Could not read from Socket
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER read(char* buffer, long buffer_size, long* valid_size) const=0;
    
    /**********************************************************************/
    /*! Reads a double value from a stream
        \return double value 
    */
    /**********************************************************************/
    virtual double MQUALIFIER readDouble() const=0;
    
    /**********************************************************************/
    /*! Reads a long value from a stream
        \return long value 
    */
    /**********************************************************************/
    virtual long MQUALIFIER readLong() const=0;
    
    /**********************************************************************/
    /*! Reads a byte value from a stream
        \return char value 
    */
    /**********************************************************************/
    virtual char MQUALIFIER readByte() const=0;
    
    /**********************************************************************/
    /*! Reads a bool value from a stream
        \return bool value 
    */
    /**********************************************************************/
    virtual bool MQUALIFIER readBool() const=0;
    
    /**********************************************************************/
    /*! Reads an array of bytes from a stream
        \param data : Return parameter 
        \retval RET_NOERROR : Bytes readed
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER readArray(IByteArray* data) const=0;
        
    /**********************************************************************/
    /*! Reads a string (character based) from the stream
        \param data : Return parameter 
        \retval RET_NOERROR : String readed
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER readString(IString* data) const=0;
    
    /**********************************************************************/
    /*! Closes the input stream
    */
    /**********************************************************************/
    virtual void MQUALIFIER close()=0;
};

END_NAMESPACE_Zeus

#endif 
