/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ILibraryManager                                               *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ILibraryManager.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef ILibraryManagerHPP
#define ILibraryManagerHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! System Library manager
*/
/***************************************************************************/
class ILibraryManager : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! This method creates an object out of the codemodule
        \param codemodule : Module name
        \param classname : Name of the class
        \param iface : Reference to return
        \retval RET_NOERROR : Object created
        \retval RET_REQUEST_FAILED : Class not exported
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_INTERNAL_ERROR : Internal error occured
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createObject(const IString* codemodule, const IString* classname, IZUnknown** iface)=0;

    /***********************************************************************/
    /*! Registers a x Factory function and returns the handle of this function
        \param codemodule : Module name
        \param classname : Name of the class
        \param index : Return parameter of the function handle
        \retval RET_NOERROR : Function found
        \retval RET_REQUEST_FAILED : Class not exported
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_INTERNAL_ERROR : Internal error occured
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER registerXClass(const IString* codemodule, const IString* classname, long* index)=0;
};

END_NAMESPACE_Zeus

#endif 
