/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IList                                                         *
 * Package : IList                                                         *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:57:06 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IList.hpp,v $
 *   Revision 1.2  2005/11/25 09:57:06  bhadorn
 *   Changes for const_iterator
 *
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IListHPP
#define IListHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <stddef.h>

BEGIN_NAMESPACE_Zeus

/*****************************************************************************/
/*! Abstakter Datentyp Iterator. Der Iterator kann eine Liste iterieren
*/
/*****************************************************************************/
template <class T> class IListIterator
{
  public:
    /*************************************************************************/
    /*! Resets the iterator
    */
    /*************************************************************************/
    virtual void MQUALIFIER reset() const=0;

    /*************************************************************************/
    /*! Gets the next Item
    */
    /*************************************************************************/
    virtual T& MQUALIFIER getNextItem() const=0;

    /*************************************************************************/
    /*! Gets the next Item
    */
    /*************************************************************************/
    virtual const T& MQUALIFIER getNextItemConst() const=0;

    /*************************************************************************/
    /*! Checks if there is a next item
        \retval true : Has next item
        \retval false : No next item
    */
    /*************************************************************************/
    virtual bool MQUALIFIER hasNextItem() const=0;
};

/*****************************************************************************/
/*! Abstakter Datentyp Liste. Das Interface IStList beschreibt das
    Verhalten der Liste.
*/
/*****************************************************************************/
template <class T> class IList : public IZUnknown
{
  public:

    /**************************************************************************/
    /*! Ein Element in die Liste hinzufuegen                                <BR>
        Precondition:  keine                                                <BR>
        Postcondition: newSize = oldSize + 1 und Element an der Length-1.
                       Stelle ist Item.                                     <BR>
                       Vorher:  e0, e1, e2 ... en                           <BR>
                       Nachher: e0, e1, e2 ... en, Item

        \param Item : Einzufuegendes Element
        \return Position des eingefuegten Elements
    */
    /**************************************************************************/
    virtual long MQUALIFIER add(const T& Item)=0;

    /**************************************************************************/
    /*! This method assigns a new list of items to this list. The old list will
        be replaced
        \param list : list to assign
    */
    /**************************************************************************/
    virtual void MQUALIFIER copyToList(IList<T>* list) const =0;
    
    /**************************************************************************/
    /*! Liste loeschen. Alle Elemente werden aus der Liste entfernt         <BR>
        Precondition:  keine                                                <BR>
        Postcondition: Liste ist leer, Size = 0
    */
    /**************************************************************************/
    virtual void MQUALIFIER clear()=0;

    /**************************************************************************/
    /*! Ein Element aus der Liste geloeschen                                <BR>
        Precondition:  Index >= 0 und Index < Size                          <BR>
        Postcondition: newSize = oldSize - 1 und Element an der Stelle Index
                       ist der ehemalige Nachfolger des geloschten Elements <BR>
                       Vorher:  e0, e1, ... eIndex, eIndex+1 ... en         <BR>
                       Nachher: e0, e1, ... eIndex+1, eIndex+2 ... en

        \param Index : Position des zu loeschenden Elements
        \retval RET_NOERROR : Element removed
        \retval RET_REQUEST_FAILED: Could not remove element
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER deleteItem(long Index)=0;

    /**************************************************************************/
    /*! Ein Element aus der Liste entfernt.                                 <BR>
        Precondition:  Item muss vorhanden sein                             <BR>
        Postcondition: newSize = oldSize - 1 und Element an der Stelle Index
                       ist der ehemalige Nachfolger des geloschten Elements <BR>
                       Vorher:  e0, e1, ... Item, Item+1 ... en             <BR>
                       Nachher: e0, e1, ... Item+1, Item+2 ... en

        \param Item : Einzufuegendes Element
        \retval RET_NOERROR : Element removed
        \retval RET_REQUEST_FAILED: Could not remove element
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER remove(const T& Item)=0;

    /**************************************************************************/
    /*! Die Groesse der Liste wird zurueckgegeben                           <BR>
        Precondition:  keine                                                <BR>
        Postcondition: Liste bleibt unveraendert

        \return Groesse der Liste
    */
    /**************************************************************************/
    virtual long MQUALIFIER getCount() const =0;

    /**************************************************************************/
    /*! Ein indexiertes Element der Liste wird zurueckgegeben               <BR>
        Precondition:  Index >= 0 und Index < Size                          <BR>
        Postcondition: Liste bleibt unveraendert

        \param index : Listenindex des gesuchten Elements
        \retval Leeres Element : Element nicht gefunden
        \retval Element[index] : Element an der Stelle [index] der Liste
    */
    /**************************************************************************/
    virtual T& MQUALIFIER getItem(long index)=0;

    /**************************************************************************/
    /*! Ein indexiertes Element der Liste wird zurueckgegeben               <BR>
        Precondition:  Index >= 0 und Index < Size                          <BR>
        Postcondition: Liste bleibt unveraendert

        \param index : Listenindex des gesuchten Elements
        \retval Leeres Element : Element nicht gefunden
        \retval Element[index] : Element an der Stelle [index] der Liste
    */
    /**************************************************************************/
    virtual T MQUALIFIER getItemCopy(long index) const=0;
    
    /**************************************************************************/
    /*! Eines Elements kann die Listenposition (Index) abgefragt werden     <BR>
        Precondition:  Item muss in der Liste sein, sonst wird -1
                       zurueckgegeben                                       <BR>
        Postcondition: Liste bleibt unveraendert

        \param Item : Element
        \retval -1 : Element nicht gefunden
        \retval >= 0: Element an der Stelle [retval] in der Liste gefunden
    */
    /**************************************************************************/
    virtual long MQUALIFIER indexOf(const T& Item) const=0;

    /**************************************************************************/
    /*! Ein Element wird in die Liste eingefuegt                            <BR>
        Precondition:  falls Index eine gueltige Position ist und Index = 0,
                       wird ein prepend ausgefuehrt,
                       ist die Position = Size wird ein append ausgefuehrt,
                       sonst ein insert an Position Index. Ist
                       index < 0 oder > Size wird -1 zurueckgegeben         <BR>
        Postcondition: newSize = oldSize + 1 und Element an der Stelle Index
                       ist der neue Nachfolger des Elements Item<BR>
                       Vorher:  e0, e1, ... e[index], e[Item+1] ... en      <BR>
                       Nachher: e0, e1, ... Item, e[index+1] ... en

        \param Item : einzufuegendes Element
        \param Index : Listenposition des einzufuegenden Elements
        \retval -1 : Element nicht gefunden
        \retval >= 0: Element an der Stelle [retval] in die Liste eingefuegt
    */
    /**************************************************************************/
    virtual long MQUALIFIER insert(long Index, const T& Item)=0;

    /**************************************************************************/
    /*! This method checks if an item is equal to an item inside this list
        \param Index : Index of the item to be checked
        \param Item : Item to check
        \retval true : Item is equal
        \retval false: Item is not equal
    */
    /**************************************************************************/
    virtual bool MQUALIFIER equalsItem(long Index, const T& Item) const =0;

    /**************************************************************************/
    /*! This method checks if two lists has the same content
        \param list : list to check
        \retval true : list is equal
        \retval false: list is not equal
    */
    /**************************************************************************/
    virtual bool MQUALIFIER equals(const IList<T>* list) const =0;
        
    /**************************************************************************/
    /*! Returns the iterator of this list object. This iterator is used to 
        traverse the list. The iterator will be allocated inside this method and
        must be released through the method releaseIterator(). Its possible 
        to use the method as nested calls. The order of the releasing doesn't has
        to mach the getIterator() calls.
        \return pointer to a allocated iterator.
    */
    /**************************************************************************/
    virtual IListIterator<T>* MQUALIFIER getIterator() const=0;
    
    /**************************************************************************/
    /*! Returns the const iterator of this list object. This iterator is used to
        traverse the list. The iterator will be allocated inside this method and
        must be released through the method releaseIterator(). Its possible 
        to use the method as nested calls. The order of the releasing doesn't has
        to mach the getIterator() calls.
        \return pointer to a allocated iterator.
    */
    /**************************************************************************/
    virtual const IListIterator<T>* MQUALIFIER getConstIterator() const=0;

    /**************************************************************************/
    /*! This method releases a iterator object. This must be called after a
        call of getIterator(), otherwise it will cause a memory leak.
        \param iterator : Iterator to release.
    */
    /**************************************************************************/
    virtual void MQUALIFIER releaseIterator(const IListIterator<T>* iterator) const=0;
    
    /**************************************************************************/
    /*! Checks if the list is empty
        \retval true : List is empty
        \retval false : List is not empty
    */
    /**************************************************************************/
    virtual bool MQUALIFIER isEmpty() const =0;
};


END_NAMESPACE_Zeus

#endif
