/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ILogger                                                       *
 * Package : LOG_Service                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ILogger.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

 
#ifndef ILoggerHPP
#define ILoggerHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IPropertyAccess.hpp>

BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! This interface defines the logging mechanisme.
*/
/**************************************************************************/
class ILogger : public IZUnknown
{
  public:
  
    /**********************************************************************/
    /*! Returns a new logger interface. If the name is NULL or empty, the
        root logger will be returned.
        \param name : Name of the logger
        \param logger : Return parameter of the logger
        \retval RET_NOERROR : Logger returned
        \retval RET_ERR_INVALID_PARAMETER : Invalid parameter received
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getLogger(IString* name, ILogger** logger)=0;
    
    virtual unsigned long MQUALIFIER printDebug(IString* message)=0;
    virtual unsigned long MQUALIFIER printError(IString* message)=0;
    virtual unsigned long MQUALIFIER printFatal(IString* message)=0;
    virtual unsigned long MQUALIFIER printInfo(IString* message)=0;
    virtual unsigned long MQUALIFIER printWarn(IString* message)=0;
    virtual bool MQUALIFIER isDebugEnabled()=0;
    virtual bool MQUALIFIER isErrorEnabled()=0;
    virtual bool MQUALIFIER isFatalEnabled()=0;
    virtual bool MQUALIFIER isInfoEnabled()=0;
    virtual bool MQUALIFIER isWarnEnabled()=0;
    
    virtual unsigned long MQUALIFIER loadConfiguration(IString* file)=0;
    virtual unsigned long MQUALIFIER loadProperties(IPropertyAccess* prop)=0;
    
  
};

END_NAMESPACE_Zeus

#endif

