/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IOutputStream                                                 *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IOutputStream.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IOutputStreamHPP
#define IOutputStreamHPP


#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IByteArray.hpp>

BEGIN_NAMESPACE_Zeus


/**************************************************************************/
/*! Interface is defining a output stream
*/
/**************************************************************************/
class IOutputStream : public IZUnknown
{
  public:    
    /**********************************************************************/
    /*! Writes to a stream out of a buffer
        \param buffer : Char Array
        \param buffer_size : Size of the Array
        \retval RET_NOERROR : Writing was successful
        \retval RET_REQUEST_FAILED : Could not write to stream
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER write(char* buffer, long buffer_size)=0;

    /**********************************************************************/
    /*! Writes a double value to a stream
        \param data : Double value
        \retval RET_NOERROR : Writing was successful
        \retval RET_REQUEST_FAILED : Could not write to stream
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER writeDouble(double data)=0;
    
        /**********************************************************************/
    /*! Writes a double value to a stream
        \param data : Double value
        \retval RET_NOERROR : Writing was successful
        \retval RET_REQUEST_FAILED : Could not write to stream
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER writeLong(long data)=0;
    
    /**********************************************************************/
    /*! Writes a byte value to a stream
        \param data : Bytevalue
        \retval RET_NOERROR : Writing was successful
        \retval RET_REQUEST_FAILED : Could not write to stream
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER writeByte(char data)=0;
    
    /**********************************************************************/
    /*! Writes a long value to a stream
        \param data : Long value
        \retval RET_NOERROR : Writing was successful
        \retval RET_REQUEST_FAILED : Could not write to stream
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER writeBool(bool data)=0;
    
    /**********************************************************************/
    /*! Writes a byte array to a stream
        \param data : Byte Array
        \retval RET_NOERROR : Writing was successful
        \retval RET_REQUEST_FAILED : Could not write to stream
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER writeArray(const IByteArray* data)=0;
    
    /**********************************************************************/
    /*! Writes a string to a stream (8bit only)
        \param data : String value
        \retval RET_NOERROR : Writing was successful
        \retval RET_REQUEST_FAILED : Could not write to stream
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER writeString(const IString* data)=0;
    
    /**********************************************************************/
    /*! Closes the output stream
    */
    /**********************************************************************/
    virtual void MQUALIFIER close()=0;
    
    /**********************************************************************/
    /*! Flush the output stream. This will send the buffer to the 
        destination.
    */
    /**********************************************************************/
    virtual void MQUALIFIER flush()=0;
};

END_NAMESPACE_Zeus

#endif
