/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IProperty                                                *
 * Package : system                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IProperty.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
  
#ifndef IPropertyHPP
#define IPropertyHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>

/***************************************************************************/
/*! This interface defines the access to a property object.
*/
/***************************************************************************/
class IProperty : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! This method returns a string value 
        \param name : Name of the property
        \param value : return parameter
        \retval RET_NOERROR : Returned a valid value
        \retval RET_REQUEST_FAILED: Non valid property
        \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getString(IString* name, IString* value)=0;

    /***********************************************************************/
    /*! This method returns a long value 
        \param name : Name of the property
        \param value : return parameter
        \retval RET_NOERROR : Returned a valid value
        \retval RET_REQUEST_FAILED: Non valid property
        \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getLong(IString* name, long* value)=0;

    /***********************************************************************/
    /*! This method returns a double value 
        \param name : Name of the property
        \param value : return parameter
        \retval RET_NOERROR : Returned a valid value
        \retval RET_REQUEST_FAILED: Non valid property
        \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getDouble(IString* name, double* value)=0;

    /***********************************************************************/
    /*! This method sets a string value 
        \param name : Name of the property
        \param value : parameter value
        \retval RET_NOERROR : Value set
        \retval RET_REQUEST_FAILED: Non valid property
        \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setString(IString* name, IString* value)=0;

    /***********************************************************************/
    /*! This method sets a long value 
        \param name : Name of the property
        \param value : parameter value
        \retval RET_NOERROR : Value set
        \retval RET_REQUEST_FAILED: Non valid property
        \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setLong(IString* name, long value)=0;

    /***********************************************************************/
    /*! This method sets a double value 
        \param name : Name of the property
        \param value : parameter value
        \retval RET_NOERROR : Value set
        \retval RET_REQUEST_FAILED: Non valid property
        \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setDouble(IString* name, double value)=0;
};


#endif
