/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IPropertyAccess                                               *
 * Package : system                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IPropertyAccess.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef IPropertyAccessHPP
#define IPropertyAccessHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IStringList.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines the access to a property collection (file)
*/
/***************************************************************************/
class IPropertyAccess : public IZUnknown
{
  public:
    /**********************************************************************/
    /*! adds a property value
        \param name : Name of the property
        \param value : Property value
        \retval RET_NOERROR : value set
        \retval RET_REQUEST_FAILED : Invalid name
        \retval RET_INVALID_PARAMETER : name or value is not valid
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER addProperty(IString* name, IString* value)=0; 
    
    /**********************************************************************/
    /*! Checks if a property exists
        \param name : Name of the property
        \retval false : Property does not exist
        \retval true : Property exists
    */
    /**********************************************************************/
    virtual bool MQUALIFIER existsProperty(IString* name)=0;
    
    /**********************************************************************/
    /*! Returns all property names of this set
        \param list : Return parameter
        \retval RET_NOERROR : List returned
        \retval RET_INVALID_PARAMETER : list is not valid
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getPropertyNames(IStringList* list)=0;
    
    /**********************************************************************/
    /*! Returns a property value
        \param name : Name of the property
        \param value : Property value
        \retval RET_NOERROR : value returned
        \retval RET_REQUEST_FAILED : Invalid name
        \retval RET_INVALID_PARAMETER : name or value is not valid
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER getPropertyValue(IString* name, IString* value)=0; 
    
    /**********************************************************************/
    /*! Sets a property value
        \param name : Name of the property
        \param value : Property value
        \retval RET_NOERROR : value set
        \retval RET_REQUEST_FAILED : Invalid name
        \retval RET_INVALID_PARAMETER : name or value is not valid
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER setPropertyValue(IString* name, IString* value)=0; 
};

END_NAMESPACE_Zeus

#endif
