/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ISerializable                                                   *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ISerializable.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ISerializableHPP
#define ISerializableHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IInputStream.hpp>
#include <zeusbase/System/Interfaces/IOutputStream.hpp>


BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines a serializable Object interface
*/
/***************************************************************************/
class ISerializable : public IZUnknown
{
  public:  
    /*************************************************************************/
    /*! Wandelt ein Objekt in einen Bytestream um
        \param stream : Rckgabeparameter des Streams
        \retval RET_NOERROR : Fehlerfreies Streamen
        \retval RET_ERR_INVALID_PARAMETER: Fehlerhafter Rckgabeparameter
        \retval RET_ERR_REQUEST_FAILED : Fehler beim Streamen aufgetreten
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER serialize(IByteArray* stream) const =0;

    /*************************************************************************/
    /*! This methods returns the ID of the object. Normaly this ID is equal to
        the address of the object. However it may be usefull to change this 
        to implement other behavours.
        \return Object ID
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER getObjectID() const = 0;

};

END_NAMESPACE_Zeus

#include <zeusbase/System/Interfaces/ISerializableImplHelper.hpp>

#endif
