/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ISerializableImplHelper                                       *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/15 07:09:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ISerializableImplHelper.hpp,v $
 *   Revision 1.4  2005/12/15 07:09:17  bhadorn
 *   Serial constructor had problems with user coding.
 *
 *   Revision 1.3  2005/12/14 09:10:51  bhadorn
 *   no message
 *
 *   Revision 1.2  2005/11/25 09:59:10  bhadorn
 *   Serializing macro for double lists defined
 *
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#ifndef ISerializableImplHelperHPP
#define ISerializableImplHelperHPP


#include <zeusbase/System/ZObjectFactory.h>
#include <zeusbase/System/ByteArrayOutputStream.h>
#include <zeusbase/System/ByteArray.hpp>

///Header bytes (4 bytes) to specify the stream as a object stream.
#define SERIALOBJECT_HEADER   0xF5E4D3C2
 
#define SERIAL_LONG_ID        0x10
#define SERIAL_DOUBLE_ID      0x11
#define SERIAL_BOOL_ID        0x12
#define SERIAL_BYTE_ID        0x14
#define SERIAL_STRING_ID      0x20
#define SERIAL_ARRAY_ID       0x21
#define SERIAL_OBJECT_ID      0x22
#define SERIAL_OBJECTLIST_ID  0x24
#define SERIAL_DOUBLELIST_ID  0x30

#define CHECK_ATTRIBUTE(stream, value) (stream->readByte() == value)
#define CHECK_LONG(stream)       CHECK_ATTRIBUTE(stream, SERIAL_LONG_ID)
#define CHECK_DOUBLE(stream)     CHECK_ATTRIBUTE(stream, SERIAL_DOUBLE_ID)
#define CHECK_BOOL(stream)       CHECK_ATTRIBUTE(stream, SERIAL_BOOL_ID)
#define CHECK_BYTE(stream)       CHECK_ATTRIBUTE(stream, SERIAL_BYTE_ID)
#define CHECK_STRING(stream)     CHECK_ATTRIBUTE(stream, SERIAL_STRING_ID)
#define CHECK_ARRAY(stream)      CHECK_ATTRIBUTE(stream, SERIAL_ARRAY_ID)
#define CHECK_OBJECT(stream)     CHECK_ATTRIBUTE(stream, SERIAL_OBJECT_ID)
#define CHECK_OBJECTLIST(stream) CHECK_ATTRIBUTE(stream, SERIAL_OBJECTLIST_ID)
#define CHECK_DOUBLELIST(stream) CHECK_ATTRIBUTE(stream, SERIAL_DOUBLELIST_ID)


/***************************************************************************/
/*! Macro to register the serializable class at the object factory.
    \param classid : ID of the serializable class
*/
/***************************************************************************/
#define OBJECTFACTORY_REGISTER_CLASS(classid) \
  classid::autoreg(true);

/***************************************************************************/
/*! Implementation of some methods and the serializing constructor
    \param classid : ID of the serializable class
*/
/***************************************************************************/
#define SERIAL_CONSTRUCTOR_START(classid) \
unsigned long MQUALIFIER classid::createObject(unsigned long objectID, IInputStream* stream, IZUnknown** obj) \
{ \
  unsigned long ulRetval = RET_INVALID_PARAMETER;\
  if (stream != NULL && obj != NULL)\
  { \
    classid* pNewObject = NULL; \
    try \
    { \
        pNewObject = new classid(objectID, stream);\
    } \
    catch(...) \
    { \
      if (pNewObject != NULL) \
      { \
        pNewObject->release(); \
        pNewObject = NULL; \
      } \
    } \
    *obj = (ISerializable*)pNewObject; \
    if (*obj != NULL) \
    { \
      ulRetval = RET_NOERROR;\
    } \
    else \
    { \
      ulRetval = RET_REQUEST_FAILED; \
    } \
  } \
  else if (stream == NULL) \
  { \
    *obj = (ISerializable*)new classid();\
    ulRetval = RET_NOERROR;\
  } \
  return ulRetval;\
} \
\
classid::classid(unsigned long objectID, IInputStream* stream) : TZObject()\
{ \

/***************************************************************************/
/*! Termination of the serializing constructor. Between the beginning and 
    this end, you may place your own code to initialize the object.
*/
/***************************************************************************/
#define SERIAL_CONSTRUCTOR_END \
  if (doSerialize(objectID, stream, this, NULL, NULL) != RET_NOERROR) \
  { \
    throw 1;\
  } \
}

/***************************************************************************/
/*! Standard constructor for serializing 
    \param classid : ID of the serializable class
*/
/***************************************************************************/
#define SERIAL_CONSTRUCTOR(classid) \
  SERIAL_CONSTRUCTOR_START(classid) \
  SERIAL_CONSTRUCTOR_END
  
/***************************************************************************/
/*! Starts the serializing part of your class.
    \param classid : ID of the serializable class
    \param classname : Name of the serializable class
*/
/***************************************************************************/
#define SERIAL_START(classid, classname) \
  public : \
    classid(unsigned long objectID, IInputStream* stream);\
    static unsigned long MQUALIFIER createObject(unsigned long objectID, IInputStream* stream, IZUnknown** obj); \
    \
    static void autoreg(bool reg)\
    { \
      static unsigned long classid##Handle = 0;\
      TString strClassName(classname); \
      if (reg) \
      { \
        classid##Handle = TZObjectFactory::getInstance().registerClass(&strClassName, createObject); \
      } \
      else \
      { \
        TZObjectFactory::getInstance().unregisterClass(&strClassName, classid##Handle); \
      } \
    } \
    \
    virtual unsigned long MQUALIFIER serialize(IByteArray* stream) const \
    { \
      TByteArrayOutputStream* pStream = new TByteArrayOutputStream();\
      unsigned long ulRetval = doSerialize(0, NULL, NULL, pStream, this); \
      pStream->popData(stream);\
      pStream->release();\
      return ulRetval; \
    } \
    \
    virtual unsigned long MQUALIFIER getObjectID() const \
    { return m_ulID; } \
    \
    \
    static unsigned long doSerialize(unsigned long obj_id_in, \
                                     IInputStream* stream_in, \
                                     classid* obj_new, \
                                     IOutputStream* stream_out, \
                                     const classid* obj_old) \
    { \
      unsigned long ulRetval = RET_REQUEST_FAILED; \
      long lMode = 0; \
      if (stream_in != NULL && obj_new != NULL) \
      { \
        lMode = 1; \
        obj_new->m_ulID = obj_id_in; \
      } \
      else if (stream_out != NULL && obj_old != NULL) \
      { \
        lMode = 2; \
        stream_out->writeLong(SERIALOBJECT_HEADER);\
        stream_out->writeLong(obj_old->m_ulID);\
        TString strClassName(classname);\
        stream_out->writeString(&strClassName);\
      } \
      \
      if (lMode > 0) \
      { \
        ulRetval = RET_NOERROR; 


/***************************************************************************/
/*! Serializing a string attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_STRING(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_STRING_ID); \
            stream_out->writeString(&obj_old->attrib); \
          } \
          else if (CHECK_STRING(stream_in))\
          { \
            stream_in->readString(&obj_new->attrib); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a string attribute using getter and setter method to receive
    the string value. 
    \param setmethod : Method to set the string value
    \param getmethod : Method to get the string value
*/
/***************************************************************************/
#define SERIAL_STRING_SET_GET(setmethod, getmethod) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            TString strValue = obj_old->getmethod(); \
            stream_out->writeByte(SERIAL_STRING_ID); \
            stream_out->writeString(&strValue); \
          } \
          else if (CHECK_STRING(stream_in)) \
          { \
            TString strValue; \
            stream_in->readString(&strValue); \
            obj_new->setmethod(strValue);\
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a byte array attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_BYTEARRAY(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_ARRAY_ID); \
            stream_out->writeArray(&obj_old->attrib); \
          } \
          else if (CHECK_ARRAY(stream_in)) \
          { \
            stream_in->readArray(&obj_new->attrib); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a byte array interface attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_IBYTEARRAY_PTR(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_ARRAY_ID); \
            stream_out->writeArray(obj_old->attrib); \
          } \
          else if (CHECK_ARRAY(stream_in)) \
          { \
            stream_in->readArray(obj_new->attrib); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }        
        
/***************************************************************************/
/*! Serializing a double list attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_DOUBLELIST(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_DOUBLELIST_ID); \
            long lAttCount = obj_old->attrib.getCount(); \
            stream_out->writeLong(lAttCount); \
            for(int i = 0; i < lAttCount; i++) \
            { \
              stream_out->writeDouble(obj_old->attrib.getItemCopy(i)); \
            } \
          } \
          else if (CHECK_DOUBLELIST(stream_in)) \
          { \
            long lAttCount = stream_in->readLong(); \
            for(int i = 0; i < lAttCount; i++) \
            { \
              obj_new->attrib.add(stream_in->readDouble()); \
            } \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a long attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_LONG(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_LONG_ID); \
            stream_out->writeLong(obj_old->attrib); \
          } \
          else if (CHECK_LONG(stream_in)) \
          { \
            obj_new->attrib = stream_in->readLong(); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a long attribute. 
    \param attrib : Signature of the attribute
    \param enumtype : Type of the enumeration (used for casting)
*/
/***************************************************************************/
#define SERIAL_LONG_ENUM(attrib, enumtype) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_LONG_ID); \
            stream_out->writeLong((long)obj_old->attrib); \
          } \
          else if (CHECK_LONG(stream_in)) \
          { \
            obj_new->attrib = (enumtype)stream_in->readLong(); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a double attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_DOUBLE(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_DOUBLE_ID); \
            stream_out->writeDouble(obj_old->attrib); \
          } \
          else if (CHECK_DOUBLE(stream_in))\
          { \
            obj_new->attrib = stream_in->readDouble(); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a bool attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_BOOL(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_BOOL_ID); \
            stream_out->writeBool(obj_old->attrib); \
          } \
          else if (CHECK_BOOL(stream_in)) \
          { \
            obj_new->attrib = stream_in->readBool(); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }
        
/***************************************************************************/
/*! Serializing a byte attribute. 
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_BYTE(attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_BYTE_ID); \
            stream_out->writeByte(obj_old->attrib); \
          } \
          else if (CHECK_BYTE(stream_in)) \
          { \
            obj_new->attrib = stream_in->readByte(); \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing a sub object of this object. The sub object must inhire also
    the ISerializable Interface. 
    \param subclass : Concrete class id of this object. Needed for up casts
    \param attrib : Signature of the attribute
*/
/***************************************************************************/
#define SERIAL_OBJECT(subclass, attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            stream_out->writeByte(SERIAL_OBJECT_ID); \
            TByteArray aArray; \
            if (obj_old->attrib != NULL)\
            {\
              obj_old->attrib->serialize(&aArray);\
            }\
            stream_out->writeArray(&aArray); \
          } \
          else if (CHECK_OBJECT(stream_in)) \
          { \
            TByteArray aArray; \
            stream_in->readArray(&aArray); \
            ISerializable* pObject = NULL;\
            if (TZObjectFactory::getInstance().createObjectFromStream(&aArray, &pObject) == RET_NOERROR)\
            { \
              obj_new->attrib = (subclass*)pObject;\
            } \
            else \
            { \
              obj_new->attrib = NULL; \
            } \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }

/***************************************************************************/
/*! Serializing an object list 
    \param subclass : Concrete class id of this object. Needed for up casts
    \param attrib : Signature of the attribute list
*/
/***************************************************************************/
#define SERIAL_OBJECTLIST(subclass, attrib) \
        if (ulRetval == RET_NOERROR) \
        { \
          if (lMode == 2) \
          { \
            long lCount = obj_old->attrib.getCount();\
            stream_out->writeByte(SERIAL_OBJECTLIST_ID); \
            stream_out->writeLong(lCount); \
            for(int i = 0; i < lCount; i++) \
            { \
              TByteArray aArray; \
              subclass* pObject = obj_old->attrib.getItemCopy(i);\
              if (pObject != NULL)\
              {\
                pObject->serialize(&aArray);\
              }\
              stream_out->writeArray(&aArray); \
            } \
          } \
          else if (CHECK_OBJECTLIST(stream_in)) \
          { \
            long lCount = stream_in->readLong(); \
            for(int i = 0; i < lCount; i++) \
            { \
              TByteArray aArray; \
              stream_in->readArray(&aArray); \
              ISerializable* pObject = NULL;\
              if (aArray.getCount() > 0 && TZObjectFactory::getInstance().createObjectFromStream(&aArray, &pObject) == RET_NOERROR)\
              { \
                obj_new->attrib.add((subclass*)pObject);\
              } \
              else if (aArray.getCount() == 0) \
              { \
                obj_new->attrib.add(NULL); \
              } \
            } \
          } \
          else \
          { \
            ulRetval = RET_REQUEST_FAILED;\
          } \
        }
        
/***************************************************************************/
/*! This starts the customized serializing of data.
*/
/***************************************************************************/
#define SERIAL_CUSTOM_START \
        if (ulRetval == RET_NOERROR) \
        { \

/***************************************************************************/
/*! This ends the customized serializing of data.
*/
/***************************************************************************/
#define SERIAL_CUSTOM_END \
        }
        
/***************************************************************************/
/*! End of the serializing part.
*/
/***************************************************************************/
#define SERIAL_END \
      } \
      return ulRetval; \
    }

#endif
