/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ISimpleCommand                                                *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:06:10 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ISimpleCommand.hpp,v $
 *   Revision 1.1  2005/12/14 09:06:10  bhadorn
 *   created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef ISimpleCommandHPP
#define ISimpleCommandHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>


BEGIN_NAMESPACE_Zeus

class IXObject;

/******************************************************************************/
/*! The simple command interface is used  to
    define and execute commands for the framework. This commands are the base
    architecture for actions of the framework
*/
/******************************************************************************/
class ISimpleCommand : public IZUnknown
{
  public:
    /**************************************************************************/
    /*! Returns the name of a target object. This name tells for witch xobject
        this command must be used
        \param name : return parameter of the target object name
        \retval RET_NOERROR: Name returned
        \retval RET_INVALID_PARAMETER : Inavlid return parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getTargetObjectName(IString* name)const =0;

    /**************************************************************************/
    /*! Returns the main data item of the command
        \param data : return parameter for main data
        \retval RET_NOERROR: Data returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getMainData(IString* data)const =0;

    /**************************************************************************/
    /*! returns the additional data item of the command
        \param data : return parameter for additional data
        \retval RET_NOERROR: Data returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getAddData(IString* data)const =0;

    /**************************************************************************/
    /*! Returns the special data stream of a command. The stream may represent a
        unparsed xml document or just a text document such as ini, txt etc.
        \param data : return parameter for the document
        \retval RET_NOERROR: Data returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getStream(IString* data) const=0;

    /**************************************************************************/
    /*! Returns the mode of the command to execute. This tells the meaning of
        the data items.
        \return mode of the command
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getMode() const=0;

    /**************************************************************************/
    /*! Returns the security level for this command
        \return security level
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getSecurity() const=0;

    /**************************************************************************/
    /*! Sets the name of the target object
        \param name : name of the target object
    */
    /**************************************************************************/
    virtual void MQUALIFIER setTargetObjectName(const IString* name)=0;

    /**************************************************************************/
    /*! Sets the main data stream
        \param data : main data object
    */
    /**************************************************************************/
    virtual void MQUALIFIER setMainData(const IString* data)=0;

    /**************************************************************************/
    /*! Sets the additional data item
        \param data: additional data
    */
    /**************************************************************************/
    virtual void MQUALIFIER setAddData(const IString* data)=0;

    /**************************************************************************/
    /*! Sets the special data stream object, (document)
        \param data : data stream
    */
    /**************************************************************************/
    virtual void MQUALIFIER setStream(const IString* data)=0;

    /**************************************************************************/
    /*! Sets the mode of the simple command
        \param mode : new mode
    */
    /**************************************************************************/
    virtual void MQUALIFIER setMode(unsigned long mode)=0;

    /**************************************************************************/
    /*! Sets the security level of the command
        \param value : security value
    */
    /**************************************************************************/
    virtual void MQUALIFIER setSecurity(unsigned long value)=0;

    /**************************************************************************/
    /*! Executes the command. The command it self can have a command listener
        interface. The listener will receive the command if this method is
        called.
        \param mode : mode of execution
    */
    /**************************************************************************/
    virtual void MQUALIFIER execute(unsigned long mode)=0;

    /**************************************************************************/
    /*! This method can enable or disable the execution with a specific mode
        \param enable : true = enabled, false = block
        \param mode : execution mode (see execute())
    */
    /**************************************************************************/
    virtual void MQUALIFIER enableExecuteMode(bool enable, unsigned long mode)=0;

    /**************************************************************************/
    /*! Sets the listener interface. Only one listener can be registered.
        \param pListener : Listener interface
        \retval RET_NOERROR : Listener registered
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER setListener(IXObject* pListener)=0;
};

END_NAMESPACE_Zeus

#include <zeusbase/System/Interfaces/IXObject.hpp>

#endif
