/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ISingletonManager                                             *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ISingletonManager.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ISingletonManagerHPP
#define ISingletonManagerHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface is used to define the singleton manager. Each loaded
    module will receive such an interface, to access the singletons of the
    system
*/
/***************************************************************************/
class ISingletonManager : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! adds a singleton to the registry. A singleton can be added only 
        once using the name. 
        \param name : Name of the singleton object
        \param iface : Reference to the singleton object
        \retval RET_NOERROR : Singleton added
        \retval RET_REQUEST_FAILED : Could not add singleton
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER addSingleton(IString* name, IZUnknown* iface)=0;
    
    
    /***********************************************************************/
    /*! Gets a singleton object referenced by the name of the object.
        \param name : Name of the singleton object
        \param iface : Return parameter. Reference to the singleton object
        \retval RET_NOERROR : Singleton returned
        \retval RET_REQUEST_FAILED : Could not find singleton with this name
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getSingleton(IString* name, IZUnknown** iface)=0;
    
    /***********************************************************************/
    /*! Removes a singleton. If an added singleton is not removed manually, 
        the manager will remove it when the application terminates. For 
        singletons provided in code modules (shared libraries) its 
        important, that the user (programmer) removes it manually before 
        the code module has been unloaded.
        \param name : Name of the object to be removed
        \retval RET_NOERROR : Singleton removed
        \retval RET_REQUEST_FAILED : Could not find singleton with this name
        \retval RET_INVALID_PARAMETER : Invalid parameter received    
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER removeSingleton(IString* name)=0;
};

END_NAMESPACE_Zeus

#endif

