/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IString                                                       *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IString.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IStringHPP
#define IStringHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/Config/PlatformDefines.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Das Interface IString definiert ein WideString
*/
/***************************************************************************/
class IString : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! Ermitteln des WideArray eines Strings
        \return Wide Array
    */
    /***********************************************************************/
    virtual wchar_t* MQUALIFIER c_bstr() const=0;

    /***********************************************************************/
    /*! Rueckgabe des char-Buffers.
        \return Character array;
    */
    /***********************************************************************/
    virtual char* MQUALIFIER c_str() const=0;
    
    /**************************************************************************/
    /*! Diese Funktion konvertiert einen String zum double
        \param predef : Default value
        \return double value
    */
    /**************************************************************************/
    virtual double MQUALIFIER toDouble(double predef = 0.0) const=0;
    
    /**************************************************************************/
    /*! Diese Funktion konvertiert einen String zum long
        \param predef : Default value
        \return long value
    */
    /**************************************************************************/
    virtual long MQUALIFIER toLong(long predef = 0) const=0;
    
    /**************************************************************************/
    /*! Ermitteln, ob der String dem WCHAR Array entspricht. Zeichenvergleich.
        \param inbuffer : Zuvergleichendes Array
        \retval true : Inhalte sind gleich
        \retval false: Inhalte sind verschieden
    */
    /**************************************************************************/
    virtual bool MQUALIFIER equals(const wchar_t* inbuffer) const=0;

    /**************************************************************************/
    /*! Ermitteln, ob der String gleich des String instr ist. Zeichenvergleich.
        \param instr : Zuvergleichender String
        \retval true : Inhalte sind gleich
        \retval false: Inhalte sind verschieden
    */
    /**************************************************************************/
    virtual bool MQUALIFIER equalsStr(const IString* instr) const=0;

    /**************************************************************************/
    /*! Dem String ein WideArray anfuegen.
        \param inbuffer : Wide Array
    */
    /**************************************************************************/
    virtual void MQUALIFIER concat(const wchar_t* inbuffer)=0;

    /**************************************************************************/
    /*! Zwei Strings zusammenfuegen.
        \param instr : String zum anfuegen
    */
    /**************************************************************************/
    virtual void MQUALIFIER concatStr(const IString* instr)=0;

    /**************************************************************************/
    /*! Ein WideArray in den Stringbuffer abspeichern.
        \param inbuffer : Neues WideArray
    */
    /**************************************************************************/
    virtual void MQUALIFIER assign(const wchar_t* inbuffer)=0;

    /**************************************************************************/
    /*! Ein String in den Stringbuffer kopieren.
        \param instr : Neuer String
    */
    /**************************************************************************/
    virtual void MQUALIFIER assignStr(const IString* instr)=0;

    /**************************************************************************/
    /*! Ermitteln der Stringgroesse
        \return Groesse in Zeichen (ohne NULL-Char)
    */
    /**************************************************************************/
    virtual long MQUALIFIER getSize()const=0;

    /**************************************************************************/
    /*! Ermitteln eines bestimmten Wide-Characters
        \param index : Index des Characters
        \return Character
    */
    /**************************************************************************/
    virtual wchar_t MQUALIFIER getChar(long index)const=0;


    /**************************************************************************/
    /*! Die Suche nach dem SubString wurde nach dem Knuth-Morris-Pratt
        Verfahren implementiert. In einem Preprocessing-Verfahren wird
        aus dem gesuchten SubString eine Sprungtabelle erstellt. Je nach dem,
        wieviele Zeichen bei der Suche uebereinstimmten, kann direkt eine
        bestimmte Distanz gesprungen werden, falls das darauffolgende Zeichen
        nicht uebereinstimmt.                                               <BR>
                                                                            <BR>
        Preprocessing Algorithmus:                                          <BR>
        Initialisiere die Tabelle mit Sprungweite 1                         <BR>
        Initialisiere die Vergleichsvariable k mit 0                        <BR>
        Wiederhole solange bis Ende des SubStrings erreicht
          - Falls das Zeichen[i] ungleich Zeichen[k] ist dann
            Setze die Sprungweite[i+1] = i+1
          - Sonst setze Sprungweite[i+1] = Sprungweite[i] und wenn
            i groesser als 0 ist dann inkrementiere k um eins               <BR>
                                                                            <BR>
        Siehe IString

    */
    /**************************************************************************/
    virtual long MQUALIFIER getPos(const IString* substring)const=0;
    
    /**************************************************************************/
    /*! Checks if a string is empty
        \retval true : String is empty (same as strData->equals(L"") but
                       faster.
        \retval false : String contains data 
    */
    /**************************************************************************/
    virtual bool MQUALIFIER isEmpty()const=0;
};

END_NAMESPACE_Zeus


#endif

