/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IXLoaderObject                                                *
 * Package : system                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IXLoaderObject.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IXLoaderObjectHPP
#define IXLoaderObjectHPP
 
#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IXMLDocument.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines a loadable object node, witch can load and unload the
    children objects (load and unload a ressource library)
*/
/***************************************************************************/
class IXLoaderObject : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! Load the ressource DLL
        \retval RET_NOERROR : Resource loaded
        \retval RET_INTERNAL_ERROR : An internal unknown error occured 
                                     loading the resource
        \retval RET_REQUEST_FAILED : Could not load the resource (file 
                                     not available)
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER load()=0;
    
    /***********************************************************************/
    /*! Unload the resource
        \retval RET_NOERROR : Resource unloaded
        \retval RET_INTERNAL_ERROR : An internal unknown error occured 
                                     unloading the resource
        \retval RET_REQUEST_FAILED : Could not unload the resource 
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER unload()=0;
    
    /***********************************************************************/
    /*! Check if the resource is loaded
        \retval true : Resource is loaded
        \retval false : Resource is not loaded
    */
    /***********************************************************************/
    virtual bool MQUALIFIER isLoaded()=0;
    
    /***********************************************************************/
    /*! This sets a external XML Reference as a XMLDocument. Only one xml 
        document can be loaded externaly.
        \param doc : Loaded document
        \param main_ref : Main node of the document
        \retval RET_NOERROR : Ext reference added
        \retval RET_REQUEST_FAILED : Could not add document, because one 
                                     is loaded alreay
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setExtXMLRef(IXMLDocument* doc, IXMLNode* main_ref)=0;
};

END_NAMESPACE_Zeus

#endif
