/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IXMLDocument                                                  *
 * Package : XML_Service                                                   *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IXMLDocument.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IXMLDocumentHPP 
#define IXMLDocumentHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IXMLNode.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Das IXMLParser Interface definiert einen XML Parser
*/
/***************************************************************************/
class IXMLDocument : public IZUnknown
{
  public:
    
    /**************************************************************************/
    /*! Returns the main node of the xml dom tree
        \param main : return parameter of the main node
        \retval RET_NOERROR : main node returned.
        \retval RET_REQUEST_FAILED : Could not return the main node
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getMainNode(IXMLNode** main)=0;
    
    /**************************************************************************/
    /*! Returns the DOM buffer as a stream.
        \param stream : return parameter of a stream of data
        \retval RET_NOERROR : Stream returned.
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getStream(IString* stream)=0;
    
    /**************************************************************************/
    /*! Saves the document. This method will fail, if a stream has been loaded
        previously. 
        \retval RET_NOERROR : File saved
        \retval RET_REQUEST_FAILED : Could not save file
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER save()=0;
    
    /**************************************************************************/
    /*! Saves the document using a new file name. This method will fail, 
        if a stream has been loaded previously. 
        \param filename : Name of the new file
        \retval RET_NOERROR : File saved
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not save file
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER saveAs(IString* filename)=0;
    
    /**************************************************************************/
    /*! Creates a new xml DOM node. The node will be returned to the caller and
        can be inserted or added using the IXMLNode method insert() or append().
        \param node_type : Type of the node (see defines)
        \param name : Name of the node
        \param uri : Ressource identifier (not used for all types 
                      -> see XML spec. of xercesc
        \param node : return parameter of the created node.
        \retval RET_NOERROR : Node created.
        \retval RET_REQUEST_FAILED : Could not create the node
        \retval RET_INVALID_PARAMETER : Could not create the node
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER createNewNode(unsigned long node_type,
                                        IString* name,
                                        IString* uri,
                                        IXMLNode** node)=0;
};

END_NAMESPACE_Zeus

#endif 
