#ifndef IXMLNodeHPP
#define IXMLNodeHPP

#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IXMLNodeList.hpp>

#define TYPE_NODE_CDATA_SECTION                   1
#define TYPE_NODE_COMMENT                         2
#define TYPE_NODE_DOCUMENT                        3
#define TYPE_NODE_DOCUMENT_FRAGMENT               4
#define TYPE_NODE_TEXT                            5
#define TYPE_NODE_DOCUMENT_TYPE                   6
#define TYPE_NODE_ELEMENT                         7
#define TYPE_NODE_ENTITY                          8
#define TYPE_NODE_ENTITY_REFERENCE                9
#define TYPE_NODE_NOTATION                        10
#define TYPE_NODE_PROCESSING_INSTRUCTION          11
#define TYPE_NODE_ATTRIBUTE                       12
#define TYPE_NODE_DECL                            13


#define IMPCLASSNAME_XERCESC                      0

BEGIN_NAMESPACE_Zeus

class IXMLNodeList;

/******************************************************************************/
/*! Das Interface IXMLNode definiert die Schnittstelle zu einem Node des
    DOM Trees
*/
/******************************************************************************/
class IXMLNode : public IZUnknown
{
  public :

    /**************************************************************************/
    /*! Den Namen des Nodes ermitteln
        \param name : Rckgabeparameter des Nodenamens
        \retval RET_NOERROR : Fehlerfreie Rckgabe
        \retval RET_REQUEST_FAILED : Fehler beim Zurckgeben aufgetreten
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getNodeName(IString* name)=0;

    /**************************************************************************/
    /*! Den Typ des Nodes ermitteln. Der Typ wird Implementationsunabh�gig
        zurckgegeben (Define).
        \retval TYPE_NODE_CDATA_SECTION                   1
        \retval TYPE_NODE_COMMENT                         2
        \retval TYPE_NODE_DOCUMENT                        3
        \retval TYPE_NODE_DOCUMENT_FRAGMENT               4
        \retval TYPE_NODE_TEXT                            5
        \retval TYPE_NODE_DOCUMENT_TYPE                   6
        \retval TYPE_NODE_ELEMENT                         7
        \retval TYPE_NODE_ENTITY                          8
        \retval TYPE_NODE_ENTITY_REFERENCE                9
        \retval TYPE_NODE_NOTATION                        10
        \retval TYPE_NODE_PROCESSING_INSTRUCTION          11
        \retval TYPE_NODE_ATTRIBUTE                       12
        \retval TYPE_NODE_DECL                            13
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getNodeType()=0;

    /**************************************************************************/
    /*! Den Nodewert ermitteln.
        \param value : Rckgabeparameter des Nodewerts
        \retval RET_NOERROR : Fehlerfreie Rckgabe
        \retval RET_REQUEST_FAILED : Fehler beim Zurckgeben aufgetreten
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getNodeValue(IString* value)=0;

    /**************************************************************************/
    /*! Mit dieser Methode kann einfach der Wert eines Attributs des Nodes
        abgefragt werden.
        \param name : Name des Attributs
        \param value : Rueckgabewert des Attributs
        \retval RET_NOERROR : Fehlerfreie Rckgabe
        \retval RET_REQUEST_FAILED : Fehler beim Zurckgeben aufgetreten
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getAttributeValue(IString* name, IString* value) = 0;

    /**************************************************************************/
    /*! Diese Methode gibt alle SubNodes eines Nodes in einer IXMLNodeList
        zurck.
        \param list : Rckgabeparameter der Nodeliste
        \retval RET_NOERROR : Fehlerfreie Rckgabe
        \retval RET_REQUEST_FAILED : Fehler beim Zurckgeben aufgetreten
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getNodeList(IXMLNodeList** list)=0;

    /**************************************************************************/
    /*! Diese Funktion erlaubt es nach einem gestimmten NodeNamen in der
        NodeListe eines Nodes zu suchen.<BR>
        Wird im Parameter NodeNamen ein leerer String bergeben wird einfach der
        erste Node der Liste zurckgegeben.
        \param name : Name des Nodes nach dem gesucht werden soll
    */
    /**************************************************************************/
    virtual void MQUALIFIER resetNodeIterator(IString* name)=0;

    /**************************************************************************/
    /*! Diese Funktion sollte erst aufgerufen werden, wenn zuvor einmal die <BR>
        Funktion getFirstNode aufgerufen wurde. Denn diese Funktion gibt den <BR>
        n�hsten Node der dem Namen des Parameters NodeName entspricht. <BR>
        Diese Funktion kann nun mehrmals hintereinander aufgerufen werden, bis
        kein entsprechender Node mehr gefunden wird.
        \param node : Rckgabeparameter des XMLNodes
        \retval RET_NOERROR : Fehlerfreie Rckgabe
        \retval RET_REQUEST_FAILED : Fehler beim Zurckgeben aufgetreten
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getNextNode(IXMLNode** node)=0;

    /**************************************************************************/
    /*! Mit dieser Methode kann einfach ermittelt werden, ob ein Node noch
        SubNodes besitzt.
        \retval true : ja der Node besitzt noch SubNodes
        \retval false: nein der Node besitzt keine SubNodes
    */
    /**************************************************************************/
    virtual bool MQUALIFIER hasChildren()=0;

    /**************************************************************************/
    /*! Mit dieser Funtkion kann ein neuer SubNode hinzugefgt werden.
        \param new_node : Zeiger auf ein IXMLNode Interface
        \retval RET_NOERROR : Node erfolgreich hinzugefgt
        \retval RET_REQUEST_FAILED : Node kann nicht hinzugefgt werden
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER appendNode(IXMLNode* new_node)=0;

    /**************************************************************************/
    /*! Mit dieser Funktion wird ein neuer SubNode erzeugt. Dieser Node ist auch
        automatisch in der NodeList aufgenommen.
        \param type  : bestimmt die Art des Nodes
        \param name : Namen des neuen Nodes
        \param uri  : namespaceURI parameter der als prefix fr den <BR>
                          Nodenamen dient. Wird ein leerer String bergeben,<BR>
                          wird der Default namespaceURI verwendet.
        \param node : Rckgabeparameter des XMLNodes
        \retval RET_NOERROR : Fehlerfreie Rckgabe
        \retval RET_REQUEST_FAILED : Fehler beim Zurckgeben aufgetreten
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER createNewChildNode(unsigned long type,
                                                     IString* name,
                                                     IString* uri,
                                                     IXMLNode** node)=0;

    /**************************************************************************/
    /*! Setzen eines neuen Nodewerts
        \param data : neuer Wert des Nodes
        \retval RET_NOERROR : Wert wurde ge�dert
        \retval RET_REQUEST_FAILED : Wert konnte nicht ge�dert werden
        \retval RET_INVALID_PARAMETER : Fehlerhafter �ergabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER setNodeValue(IString* data)=0;

    /**************************************************************************/
    /*! Setzen eines Attributwerts
        \param name  : Name des Attributes
        \param value : neuer Wert fr das Attribute
        \retval RET_NOERROR : Wert wurde ge�dert
        \retval RET_REQUEST_FAILED : Wert kann nicht ver�dert werden
        \retval RET_INVALID_PARAMETER : Fehlerhafter �ergabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER setAttributeValue(IString* name, IString* value)=0;

    /**************************************************************************/
    /*! Entfernt den Node aus der XMLNodeList des ParentNodes
        \retval RET_NOERROR : Knoten wurde entfernt
        \retval RET_REQUEST_FAILED : Knoten konnte nicht entfernt werden
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER removeMe()=0;

    /**************************************************************************/
    /*! Mit dieser Methode kann ermittelt werden, welche Parserimplementation
        fr das Interface verwendet wurde.
        \retval IMPCLASSNAME_STDOMNODE_XERCESC
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getImplClass()=0;


    /**************************************************************************/
    /*! This methods resets the attribute iterator. The next call of 
        getNextAttribute() will return the first attribute
    */
    /**************************************************************************/
    virtual void MQUALIFIER resetAttributeIterator() = 0;


    /**************************************************************************/
    /*! Mit dieser Methode wird das naechste Attribut zurueckgegben
        \param name : Referenz auf den Attributnamen
        \param value : Referenz auf den Wert des Attributs
        \retval RET_NOERROR : Attribut gefunden
        \retval RET_REQUEST_FAILED : kein Attribut gefunden
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getNextAttribute(IString* name, IString* value) = 0;

    /**************************************************************************/
    /*! Ermittelt den Parent des DOMNodes
        \param parent : Parentknoten
        \retval RET_NOERROR : Parent zurckgegeben
        \retval RET_REQUEST_FAILED : kein Parent gefunden
        \retval RET_INVALID_PARAMETER : Fehlerhafter Rckgabeparameter
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER getParentNode(IXMLNode** parent)=0;

    /**************************************************************************/
    /*! Suchen eines Knoten nach dem XPfad-Query
        \param query : XPfad des gesuchten Knoten
        \param node : Rckgabeparameter des Knoten
        \retval INT_ERR_REQUESTFAILED : Knoten nicht gefunden
        \retval INT_ERR_INVALID_PARAMETER: Ungltiger �ergabeparameter
        \retval INT_NOERROR : Fehlerfreie �ergabe
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER selectNode(IString* query, IXMLNode** node) = 0;

    /**************************************************************************/
    /*! Suchen einer Menge von Knoten nach dem XPfad-Query
        \param query : XPfad des gesuchten Knoten
        \param list : Rckgabeparameter des Knotenmenge
        \retval INT_ERR_REQUESTFAILED : Keine Knoten gefunden
        \retval INT_ERR_INVALID_PARAMETER: Ungltiger �ergabeparameter
        \retval INT_NOERROR : Fehlerfreie �ergabe
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER selectNodes(IString* query, IXMLNodeList** list) = 0;

    /**************************************************************************/
    /*! Ein Knoten klonen. Es entsteht ein extaktes Abbild des Knotens
        \param recurs : Rekursives Klonen
        \param node : Rckgabeparameter des Knotens
        \retval INT_ERR_REQUESTFAILED : Kann Knoten nicht klonen
        \retval INT_ERR_INVALID_PARAMETER: Ungltiger �ergabeparameter
        \retval INT_NOERROR : Fehlerfreie �ergabe
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER clone(bool recurs, IXMLNode** node) = 0;

    /**************************************************************************/
    /*! Ein Child-Knoten einfgen.
        \param node : Knoten welcher eingefgt werden soll
        \param before_node : Vor diesem Knoten einfgen
        \retval INT_ERR_REQUESTFAILED : Kann Knoten nicht eingefgt werden
        \retval INT_ERR_INVALID_PARAMETER: Ungltiger �ergabeparameter
        \retval INT_NOERROR : Fehlerfreies Einfgen
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER insertNode(IXMLNode* node, IXMLNode* before_node) = 0;
    
    /**************************************************************************/
    /*! serializes the node
        \param stream : Return parameter of the stream
        \retval INT_ERR_REQUESTFAILED : Could not serialize the node
        \retval INT_ERR_INVALID_PARAMETER: Invalid parameter received
        \retval INT_NOERROR : Serializing was successful
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER serializeNode(IString* stream)=0;

};


END_NAMESPACE_Zeus


#endif
