/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IXMLParser                                                  *
 * Package : XML_Service                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IXMLParser.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IXMLParserHPP 
#define IXMLParserHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IXMLDocument.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Das IXMLParser Interface definiert einen XML Parser
*/
/***************************************************************************/
class IXMLParser : public IZUnknown
{
  public:
    /**************************************************************************/
    /*! This method loads a xml file into the buffer using a dom parser.
        \param filename : name of the file (path)
        \param docu : Return parameter of the parsed document
        \retval RET_NOERROR : File loaded.
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER loadFromFile(const IString* filename, IXMLDocument** docu)=0;
    
    /**************************************************************************/
    /*! This method loads a xml from a stream (string) into the buffer 
        using a dom parser.
        \param stream : stream of data
        \param docu : Return parameter of the parsed document
        \retval RET_NOERROR : Stream loaded.
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**************************************************************************/
    virtual unsigned long MQUALIFIER parseStream(const IString* stream, IXMLDocument** docu)=0;
};

END_NAMESPACE_Zeus

#endif
