/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IXObject                                                      *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:11:32 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IXObject.hpp,v $
 *   Revision 1.3  2005/12/14 09:11:32  bhadorn
 *   Declared some methods as const
 *
 *   Revision 1.2  2005/11/25 09:57:41  bhadorn
 *   Commented
 *
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IXObjectHPP
#define IXObjectHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IByteArray.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/ICriticalSection.hpp>
#include <zeusbase/System/Interfaces/IXMLNode.hpp>
#include <zeusbase/System/Interfaces/IXMLDocument.hpp>
#include <zeusbase/System/Interfaces/IXObjectImplHelper.hpp>


BEGIN_NAMESPACE_Zeus

class ISimpleCommand;

/***************************************************************************/
/*! This interface defines a xml streamable Object interface
*/
/***************************************************************************/
class IXObject : public IZUnknown
{
  public:  
    //function definition for factory methods
    typedef unsigned long MQUALIFIER (*FCreateXObject)(IXMLNode* main_node, IXMLDocument* docu, IXObject** obj);

    /***********************************************************************/
    /*! This methods returns the name of the object
        \param name : return parameter
        \retval RET_NOERROR : name returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getName(IString* name)const=0;
    
    /***********************************************************************/
    /*! This methods returns the id of an object
        \return id of object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getID()const=0;
    
    /***********************************************************************/
    /*! This methods adds an XObject as a child to this object
        \param child : child object to add
        \retval RET_NOERROR : Child object added
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not add object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER addChild(IXObject* child)=0;
    
    /***********************************************************************/
    /*! Deletes a child out of the list.
        \param index : Index of child to Remove
        \retval RET_NOERROR : Child deleted
        \retval RET_REQUEST_FAILED : Could not delete child.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER deleteChild(long index) = 0;

    /***********************************************************************/
    /*! This methods removes an XObject
        \param child : child object to remove
        \retval RET_NOERROR : Child object added
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not remove object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER removeChild(IXObject* child)=0;
    
    /***********************************************************************/
    /*! Get count of the children
        \return count
    */
    /***********************************************************************/
    virtual long MQUALIFIER getChildCount()const=0;
    
    /***********************************************************************/
    /*! Get a child object by index
        \param index : Index of the object
        \param child : Return parameter of the found object
        \retval RET_NOERROR : Child object found
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No child found
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getChild(long index, IXObject** child)=0;

    /***********************************************************************/
    /*! Get a child object by name
        \param name : Name of the object
        \param child : Return parameter of the found object
        \retval RET_NOERROR : Child object found
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No child found
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getChildByName(const IString* name, IXObject** child)=0;
    
    /***********************************************************************/
    /*! Get the parent object
        \param parent : Return parameter of the found object
        \retval RET_NOERROR : parent found
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No parent found -> this is the root object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getParent(IXObject** parent)=0;
    
    /***********************************************************************/
    /*! Get the xml node of the xobject
        \param node : return parameter of the xml node
        \retval RET_NOERROR : node returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not get the xml node
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getXMLNode(IXMLNode** node)const=0;
  
    /***********************************************************************/
    /*! Set the parent object. Only for internal use
        \param parent : Parameter to set. If NULL is set, the object resets its parent
        \retval RET_NOERROR : parent set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the parent
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setParent(IXObject* parent)=0;
    
    /***********************************************************************/
    /*! Returns the Attribue Schema of this object
        \return Schema
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getAttributeSchema()const=0;
    
    /***********************************************************************/
    /*! This methods sets an XObject as a child of this object. Do not use
        this method. Its only used by the factory creating the tree. Therefor
        the xml structure wont be touched. For adding children, use the method
        addChild() instead. It will update the xml structure as well.
        \param child : child object to add
        \retval RET_NOERROR : Child object added
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not add object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setChild(IXObject* child)=0;

    /***********************************************************************/
    /*! Returns the root object of the object tree.
        \param obj : Return parameter
        \retval RET_NOERROR : Root object returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received;
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getRootObject(IXObject** obj)=0;
    
    /***********************************************************************/
    /*! This method is used to get an object from the tree. Therefor a 
        special Syntax is used. <BR>
        \param path : Objectpath
        \param obj : Return parameter of the object
        \retval RET_NOERROR : object found and returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not find object
        \retval RET_INVALID_DATA : Invalid path
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getObject(IString* path, IXObject** obj)=0;
    
    /***********************************************************************/
    /*! This methods checks if the children components can be created 
        \retval true : Able to create children
        \retval false : Do not create children
    */
    /***********************************************************************/
    virtual bool MQUALIFIER canCreateChildren()const=0;
    
    /**********************************************************************/
    /*! This method freezes a xobject. All external references has to be
        released at that point.
        \retval true : Freezing has been performed
        \retval false : Already frozen
    */
    /**********************************************************************/
    virtual bool MQUALIFIER freeze()=0;

    /**********************************************************************/
    /*! This method unfreezes a xobject. The external References begin to
        live and are accecible for others
        \retval true : Unfreezing has been performed
        \retval false : Already unfrozen
    */
    /**********************************************************************/
    virtual bool MQUALIFIER unfreeze()=0;

    /**********************************************************************/
    /*! Checks if a xobject is frozen
        \retval true : xobject is frozen
        \retval false : xobject is alive
    */
    /**********************************************************************/
    virtual bool MQUALIFIER isFrozen()const=0;
    
    /**********************************************************************/
    /*! This method sets a lock for the whole XObject tree. This enables
        a secure multithreading access to the xobject tree
        \param section : Lock object
    */
    /**********************************************************************/
    virtual void MQUALIFIER setTreeLock(ICriticalSection* section)=0;
    
    /**********************************************************************/
    /*! This locks the xobject tree. Before using this method, set a
        lock object using setTreeLock()
    */
    /**********************************************************************/
    virtual void MQUALIFIER lockTree()=0;
    
    /**********************************************************************/
    /*! Unlocks the xobject tree
    */
    /**********************************************************************/
    virtual void MQUALIFIER unlockTree()=0;

    /**********************************************************************/
    /*! Executes a simple command. This can be used as a action method
        \param pCommand : Command witch has been executed
        \param ulMode : Mode of execution
        \retval RET_NOERROR : Command executed
        \retval RET_REQUEST_FAILED : Command has not been understand
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /**********************************************************************/
    virtual unsigned long MQUALIFIER onExecute(const ISimpleCommand* pCommand, unsigned long ulMode)=0;
};


END_NAMESPACE_Zeus

#include <zeusbase/System/Interfaces/ISimpleCommand.hpp>

#endif
 
