/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IXObjectImplHelper                                            *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:17:37 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IXObjectImplHelper.hpp,v $
 *   Revision 1.3  2005/12/14 09:17:37  bhadorn
 *   the factory method with MQUALIFIER
 *
 *   Revision 1.2  2005/11/25 09:58:16  bhadorn
 *   Eliminated compiler warnings
 *
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IXObjectImplHelperHPP
#define IXObjectImplHelperHPP 

/**************************************************************************/
/*! Declaration of autoregistration and factory method to create a 
    XRootObject using the XObjectFactory.
    \param classid : ID of the class to be registered
*/
/**************************************************************************/
#define REG_ROOT_BEGIN(classid) \
  protected: \
    static unsigned long MQUALIFIER createXRootObject(IXMLNode* main_node, IXMLDocument* docu, IXObject** obj) \
    { \
      unsigned long ulRetval = RET_REQUEST_FAILED; \
      if (main_node != NULL && docu != NULL && obj != NULL) \
      { \
        *obj = new classid(main_node, docu); \
        ulRetval = RET_NOERROR; \
      } \
      else \
      { \
        ulRetval = RET_INTERNAL_ERROR; \
      } \
      return ulRetval; \
    } \
  public: \
    static bool autoregRoot(bool reg) \
    { \
      TString strClassName;\
      
/**************************************************************************/
/*! Entry of autoregistration. This will be added to the database of the
    XObjectFactory. 
    \param classid : ID of the class
    \param classname: String value representing the class name
*/
/**************************************************************************/
#define REG_ROOT_ADD(classid, classname) \
      strClassName = classname; \
      static unsigned long classid##Handle = 0;\
      if (reg)\
      {\
        classid##Handle = TXObjectFactory::getInstance().registerXClass(&strClassName, createXRootObject); \
      }\
      else\
      {\
        TXObjectFactory::getInstance().unregisterXClass(classid##Handle, &strClassName); \
      }

/**************************************************************************/
/*! End of registration.
*/
/**************************************************************************/
#define REG_ROOT_END \
      return true;\
    }
      
/**************************************************************************/
/*! Declaration of autoregistration and factory method to create a 
    XObject using the XObjectFactory. Those Objects are sub objects of a 
    XRootObject.
    \param classid : ID of the class to be registered
*/
/**************************************************************************/
#define REG_SUB_BEGIN(classid) \
  protected: \
    static unsigned long MQUALIFIER createXObject(IXMLNode* main_node, IXMLDocument* /*docu*/, IXObject** obj) \
    { \
      unsigned long ulRetval = RET_REQUEST_FAILED; \
      if (main_node != NULL && obj != NULL) \
      { \
        *obj = new classid(main_node); \
        ulRetval = RET_NOERROR; \
      } \
      else \
      { \
        ulRetval = RET_INTERNAL_ERROR; \
      } \
      return ulRetval; \
    } \
  public: \
    static bool autoregSub(bool reg) \
    { \
      TString strClassName;\

/**************************************************************************/
/*! Entry of autoregistration. This will be added to the database of the
    XObjectFactory. 
    \param classid : ID of the class
    \param classname: String value representing the class name
*/
/**************************************************************************/
#define REG_SUB_ADD(classid, classname) \
      strClassName = classname; \
      static unsigned long classid##Handle = 0;\
      if (reg)\
      {\
        classid##Handle = TXObjectFactory::getInstance().registerXClass(&strClassName, createXObject); \
      }\
      else\
      {\
        TXObjectFactory::getInstance().unregisterXClass(classid##Handle, &strClassName); \
      }

/**************************************************************************/
/*! End of registration, same as REG_ROOT_END
*/
/**************************************************************************/
#define REG_SUB_END \
      REG_ROOT_END
              

/**************************************************************************/
/*! Implementation of the registration. This will registrate the class 
    explicitly.
*/
/**************************************************************************/
#define XOBJECTFACTORY_REGISTER_ROOT(classid) \
classid::autoregRoot(true);

/**************************************************************************/
/*! Implementation of the unregistration. This will unregistrate the class 
    explicitly.
*/
/**************************************************************************/
#define XOBJECTFACTORY_UNREGISTER_ROOT(classid) \
classid::autoregRoot(false);
    
/**************************************************************************/
/*! Implementation of the registration. This will registrate the class 
    explicitly.
*/
/**************************************************************************/
#define XOBJECTFACTORY_REGISTER_SUB(classid) \
classid::autoregSub(true);

/**************************************************************************/
/*! Implementation of the unregistration. This will unregistrate the class 
    explicitly.
*/
/**************************************************************************/
#define XOBJECTFACTORY_UNREGISTER_SUB(classid) \
classid::autoregSub(false);



#endif
