/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IZObjectFactory                                               *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:14:12 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IZObjectFactory.hpp,v $
 *   Revision 1.3  2005/12/14 09:14:12  bhadorn
 *   the factory method with MQUALIFIER
 *
 *   Revision 1.2  2005/11/16 11:32:50  bhadorn
 *   In parameter are [const]
 *
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

 
#ifndef IZObjectFactoryHPP
#define IZObjectFactoryHPP

#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IByteArray.hpp>

 
BEGIN_NAMESPACE_Zeus


/***************************************************************************/
/*! The IZObjectFactory Interface defines the factory to create serializable
    objects by class name and streaming information 
*/
/***************************************************************************/
class IZObjectFactory : public IZUnknown
{
  public:
    //function definition for factory methods
    typedef unsigned long MQUALIFIER (*FCreateZObject)(unsigned long objectID, IInputStream* stream, IZUnknown** obj);
    
    /***********************************************************************/
    /*! Factory method to create a registered serializable object out of a 
        byte stream.
        \param stream : Byte stream
        \param obj : Return value of created object
        \retval RET_NOERROR : Object created
        \retval RET_REQUEST_FAILED : Invalid stream or Object could not be 
                   created. Eventually not a registered class
        \retval RET_INVALID_PARAMETER : Invalid parameter received      
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createObjectFromStream(const IByteArray* stream, ISerializable** obj)=0;
    
    /***********************************************************************/
    /*! Factory method to create a registered serializable object 
        according to the classname. This will return a empty object created
        with the standard constructor
        \param classname : Name of the class
        \param codemodule : Code module
        \param obj : Return value of created object
        \retval RET_NOERROR : Object created
        \retval RET_REQUEST_FAILED : not a registered class
        \retval RET_INVALID_PARAMETER : Invalid parameter received      
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createObjectByClassName(const IString* classname, const IString* codemodule, IZUnknown** obj)=0;
    
    /***********************************************************************/
    /*! Register a class
        \param name : Name of the class
        \param function : Factory function to call for creating an object 
                          of this class
        \return Handle of the creation. If handle is NULL (0) the 
                          registration failed
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER registerClass(const IString* name, IZObjectFactory::FCreateZObject function)=0;
    
    /***********************************************************************/
    /*! Unregsiter a registered class
        \param name : Name of the class
        \param handle : Handle received by the registration
        \retval RET_NOERROR : Class unregistered
        \retval RET_REQUEST_FAILED : Invalid handle or class not registered
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER unregisterClass(const IString* name, unsigned long handle)=0;
    
  
};


END_NAMESPACE_Zeus

#endif
