/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus                                                          *
 * Module  : IZUnknownImplHelper                                           *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IZUnknownImplHelper.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IZUnknownImplHelperHPP
#define IZUnknownImplHelperHPP

/**************************************************************************/
/*! \def MEMORY_MANAGER_DECL 
    This macro defines the declaration part of the memory management.
*/
/**************************************************************************/
#define MEMORY_MANAGER_DECL \
  public: \
    virtual unsigned long MQUALIFIER askForInterface(long interface_id, IZUnknown** iface);\
    virtual unsigned long MQUALIFIER addRef();\
    virtual unsigned long MQUALIFIER release();    

//Defines the start of the askForInterface Method. Do not use directly
#define INTERFACE_CAST_START(classid) \
  unsigned long MQUALIFIER classid::askForInterface(long interface_id, IZUnknown** iface)\
  { \
    unsigned long ulRetval = RET_INVALID_PARAMETER; \
    if (iface != NULL) \
    { \
      switch(interface_id) \
      {
    
/**************************************************************************/
/*! \def MEMORY_MANAGER_IMPL_PARENT
    Implementation of the memory management. This part of the memory 
    management delegates it to a parent class.
    \param classid : ID of the class
    \param parent : ID of the parent class
*/
/**************************************************************************/
#define MEMORY_MANAGER_IMPL_PARENT(classid, parent) \
  unsigned long MQUALIFIER classid::addRef()\
  { \
    return parent::addRef(); \
  } \
  unsigned long MQUALIFIER classid::release() \
  { \
    return parent::release(); \
  } \
  INTERFACE_CAST_START(classid)

/**************************************************************************/
/*! \def MEMORY_MANAGER_IMPL
    This macro is similar to the MEMORY_MANAGER_IMPL_PARENT macro. It uses
    the TZObject as parent class.
    \param classid : ID of the class
*/
/**************************************************************************/
#define MEMORY_MANAGER_IMPL(classid) \
  MEMORY_MANAGER_IMPL_PARENT(classid, TZObject)

/**************************************************************************/
/*! Implementation of memorymanagement using a lock object. It uses the 
    TZObject as a parent class. if the lock object is NULL, it works like 
    the macro MEMORY_MANAGER_IMPL 
    \param classid : ID of the class
    \param lock : Lock object
*/
/**************************************************************************/
#define MEMORY_MANAGER_IMPL_LOCK(classid, lock) \
  unsigned long MQUALIFIER classid::addRef()\
  { \
    unsigned long ulRetval = RET_NOERROR;\
    if (lock != NULL)\
    { \
      lock->enter();\
    } \
    ulRetval = TZObject::addRef(); \
    if (lock != NULL)\
    { \
      lock->leave();\
    }\
    return ulRetval;\
  } \
  unsigned long MQUALIFIER classid::release() \
  { \
    unsigned long ulRetval = RET_NOERROR;\
    if (lock != NULL)\
    { \
      lock->enter();\
    }\
    if (getRefCounter() == 1)\
    {\
      if (lock != NULL)\
      { \
        lock->leave();\
      }\
      ulRetval = TZObject::release(); \
    }\
    else\
    {\
      ulRetval = TZObject::release(); \
      if (lock != NULL)\
      { \
        lock->leave();\
      }\
    }\
    return ulRetval;\
  } \
  INTERFACE_CAST_START(classid)
  
/**************************************************************************/
/*! \def INTERFACE_CAST
    This macro is used to cast the object according the interface_id. The
    casting class must be specified and inherited.
    \param interface_type : Class id of the interface
    \param interface_id : Nummeric value representing an interface
*/
/**************************************************************************/
#define INTERFACE_CAST(interface_type, interface_id) \
        case interface_id : \
          *iface = (interface_type*)this; \
          this->addRef(); \
          ulRetval = RET_NOERROR; \
        break;\

/**************************************************************************/
/* \def MEMORY_MANAGER_IMPL_PARENT_END
   End of memory management implementation.
   \param parent : Class id of the parent class
*/
/**************************************************************************/
#define MEMORY_MANAGER_IMPL_PARENT_END(parent) \
        default: \
          ulRetval = parent::askForInterface(interface_id, iface); \
      } \
    } \
    return ulRetval; \
  }\

/**************************************************************************/
/* \def MEMORY_MANAGER_IMPL_END
   End of memory management implementation. Same as macro 
   MEMORY_MANAGER_IMPL_PARENT_END. The parent will be TZObject.
*/
/**************************************************************************/
#define MEMORY_MANAGER_IMPL_END \
  MEMORY_MANAGER_IMPL_PARENT_END(TZObject)
  
#endif 
