/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Code Module                                                   *
 * Package : Code Package                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LibLoader.h,v $
 *   Revision 1.1  2005/11/10 10:20:17  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIBLOADER_H
#define LIBLOADER_H

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>
#include <zeusbase/System/ArrayList.hpp>
#include <zeusbase/System/Interfaces/IZUnknown.hpp>


BEGIN_NAMESPACE_Zeus

#define LIB_FUNCTION_REG        L"registerLibrary"
#define LIB_FUNCTION_UNREG      L"unregisterLibrary"


/***************************************************************************/
/*! Der Library Loader ist der API Wrapper des Betriebssystem-spezifischen
    Laden einer Library
*/
/***************************************************************************/
class TLibLoader : public TZObject
{
  public:
    TLibLoader(TString name, const TString& filename);
    
    TString getFileName();
    unsigned long registerLibrary();
    unsigned long unregisterLibrary();
    unsigned long createObject(const TString& classname, IZUnknown** iface);
    
    unsigned long load();
    unsigned long unload();
    bool isLoaded();
    void* getFunction(TString& functname);
    unsigned long registerXClass(const IString* classname, long* index);

  protected:
    virtual ~TLibLoader();

  private:
    ///Dateinamen der Lirbrary
    TString m_strFileName;
    ///Library Handle
    void* m_pHandle;
    ///Flag if library is registered
    bool m_bRegistered;

    struct RRegFunction
    {
      TString strName;
      long lHandle;
      RRegFunction()
      {
        lHandle = 0;
      }
      bool operator==(const RRegFunction& inpar)
      {
        return ( strName == inpar.strName &&
                 lHandle == inpar.lHandle);
      }
    };
    ///
    TArrayList<RRegFunction> m_lstRegFunction;

};

END_NAMESPACE_Zeus

#endif
