/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : LibraryManager                                                *
 * Package : System                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LibraryManager.h,v $
 *   Revision 1.1  2005/11/10 10:20:17  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
#ifndef LibraryManagerH
#define LibraryManagerH

#include <zeusbase/System/Interfaces/ILibraryManager.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/StringMap.hpp>

#define XML_LIBRARY    L"XML_Service"
#define LOG_LIBRARY    L"LOG_Service"

BEGIN_NAMESPACE_Zeus

class TLibLoader;

/***************************************************************************/
/*! The library manager contains a list of loaded libraries and their 
    module factories
*/
/***************************************************************************/
class TLibraryManager : public TZObject, public ILibraryManager
{
  public:
    enum TSecurityLevel {slUseSecurity, slNoSecurity};
    
    static TLibraryManager& getInstance();
    void setDelegationInterface(ILibraryManager* manager);
    void releaseDelegationInterface();
    
    unsigned long addCodeModule(const TString& name);
    unsigned long addCodeModule(const TString& name, const TString& path);
    bool hasCodeModule(const TString& name);
    void setCodeModulePath(const TString& path);
    void setSecurity(TSecurityLevel mode);
    unsigned long remove(const TString& name);
    void removeAll();
    
    //Methods of ILibraryManager
    virtual unsigned long MQUALIFIER createObject(const IString* codemodule, const IString* classname, IZUnknown** iface);
    virtual unsigned long MQUALIFIER registerXClass(const IString* codemodule, const IString* classname, long* index);

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  private:
    ///Security level
    TSecurityLevel m_Security;  
    ///List of all loaded libraries
    TStringMap<TLibLoader*> m_mapLibraries;
    ///Name of the code module path
    TString m_strCodeModulePath;
    ///Type of the extentions of Code modules
    TString m_strExtentions;
    ///Delegation Interface
    ILibraryManager* m_pDelegation;
    ///Static object
    static TLibraryManager m_LibManager;
  
    TLibraryManager(TString name);
    virtual ~TLibraryManager();
};

END_NAMESPACE_Zeus

#endif
