/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Code Module                                                   *
 * Package : Code Package                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Logger.cpp,v $
 *   Revision 1.1  2005/11/10 10:20:17  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/System/Logger.h>
#include <zeusbase/System/LibraryManager.h>

#ifdef LINUX
  #include <stdio.h>
  #include <stdarg.h>
#else
  #include <mem.h>
  #include <stdio.h>
  #include <stdarg.h>
#endif

#define DEBUG_MAXLEN        1024

#define CHECK_AND_PRINT(pInterface, mode, strOut)                          \
          switch(mode)                                                     \
          {                                                                \
            case LOGMODE_DEBUG :                                           \
              if (pInterface->isDebugEnabled())                            \
              {                                                            \
                 pInterface->printDebug(&strOut);                           \
              }                                                            \
            break;                                                         \
            case LOGMODE_INFO :                                            \
              if (pInterface->isInfoEnabled())                             \
              {                                                            \
                pInterface->printInfo(&strOut);                            \
              }                                                            \
            break;                                                         \
            case LOGMODE_WARN :                                            \
              if (pInterface->isWarnEnabled())                             \
              {                                                            \
                pInterface->printWarn(&strOut);                            \
              }                                                            \
            break;                                                         \
            case LOGMODE_ERROR :                                           \
              if (pInterface->isErrorEnabled())                            \
              {                                                            \
                pInterface->printError(&strOut);                           \
              }                                                            \
            break;                                                         \
            case LOGMODE_FATAL:                                            \
              if (pInterface->isFatalEnabled())                            \
              {                                                            \
                pInterface->printFatal(&strOut);                           \
              }                                                            \
            break;                                                         \
            default:                                                       \
             ;                                                             \
          }                                                                \



USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates the Loggers
*/
/***************************************************************************/
TLogger::TLogger(ILogger* obj) : TZObjectPtr<ILogger>(obj)
{
  m_lOffset = 0;
}


/***************************************************************************/
/*! Destroyes des Loggers
*/
/***************************************************************************/
TLogger::~TLogger()
{}



/***************************************************************************/
/*! Print out a text without using the indention and no new line at the 
    end
    \param mode : logging mode (see define)
    \param txt : Text
*/
/***************************************************************************/
void TLogger::print(unsigned long mode, char* txt,...)
{
  va_list vaParameter;
  
  va_start(vaParameter,txt);
  char cBuffer[DEBUG_MAXLEN];
  
  if (vsnprintf(cBuffer, DEBUG_MAXLEN, txt, vaParameter) != -1)
  {
    TString strText = cBuffer;
    if (m_pInterface != NULL)
    {
      CHECK_AND_PRINT(m_pInterface, mode, strText);
    }
  }
  
  va_end(vaParameter);
}

/***************************************************************************/
/*! Print out a text without using the indention with a new line at the end
    \param mode : logging mode (see define)
    \param txt : Text
*/
/***************************************************************************/
void TLogger::println(unsigned long mode, char* txt,...)
{
  va_list vaParameter;
  
  va_start(vaParameter,txt);
  char cBuffer[DEBUG_MAXLEN];
  
  if (vsnprintf(cBuffer, DEBUG_MAXLEN, txt, vaParameter) != -1)
  {
    TString strText = cBuffer;
    if (m_pInterface != NULL)
    {
      CHECK_AND_PRINT(m_pInterface, mode, strText);
    }
  }
  
  va_end(vaParameter);
}

/***************************************************************************/
/*! Print out a text using the indention with a no new line at the end
    \param mode : logging mode (see define)
    \param txt : Text
*/
/***************************************************************************/
void TLogger::printf(unsigned long mode, char* txt,...)
{
  va_list vaParameter;
  
  va_start(vaParameter,txt);
  char cBuffer[DEBUG_MAXLEN];
  
  if (vsnprintf(cBuffer, DEBUG_MAXLEN, txt, vaParameter) != -1)
  {
    TString strText;
    if (m_lOffset > 0)
    {
      char* pBuffer = new char[m_lOffset+1];
    
      ::memset(pBuffer, 0x20, m_lOffset);
      pBuffer[m_lOffset] = 0x00;
      strText = pBuffer;
      strText = strText + cBuffer; 
      delete [] pBuffer;
    }
    else
    {
      strText = cBuffer;
    }
    
    if (m_pInterface != NULL)
    {
      CHECK_AND_PRINT(m_pInterface, mode, strText);
    }
  }
  
  va_end(vaParameter);
}

/***************************************************************************/
/*! Print out a text using the indention with a new line at the end
    \param mode : logging mode (see define)
    \param txt : Text
*/
/***************************************************************************/
void TLogger::printfln(unsigned long mode, char* txt,...)
{
  va_list vaParameter;
  
  va_start(vaParameter,txt);
  char cBuffer[DEBUG_MAXLEN];
  
  if (vsnprintf(cBuffer, DEBUG_MAXLEN, txt, vaParameter) != -1)
  {
    TString strText;
    if (m_lOffset > 0)
    {
      char* pBuffer = new char[m_lOffset+1];
    
      ::memset(pBuffer, 0x20, m_lOffset);
      pBuffer[m_lOffset] = 0x00;
      
      strText = pBuffer;
      strText = strText + cBuffer; 
      delete [] pBuffer;
    }
    else
    {
      strText = cBuffer;
    }
    
    if (m_pInterface != NULL)
    {
      CHECK_AND_PRINT(m_pInterface, mode, strText);
    }
  }
  
  va_end(vaParameter);
}

/***************************************************************************/
/*! Resets the offset of the indention back to 0
*/
/***************************************************************************/
void TLogger::out(char* txt,...)
{
  va_list vaParameter;
  
  va_start(vaParameter,txt);
  char cBuffer[DEBUG_MAXLEN];
  if (vsnprintf(cBuffer, DEBUG_MAXLEN, txt, vaParameter) != -1)
  {
    ::printf(cBuffer);
  }
  else
  {
    ::printf(txt);
  }
  va_end(vaParameter);
}


/***************************************************************************/
/*! Resets the offset of the indention back to 0
*/
/***************************************************************************/
void TLogger::resetOffset()
{
  m_lOffset = 0;
}


/***************************************************************************/
/*! Sets the offset of the indention to value
*/
/***************************************************************************/
void TLogger::setOffset(long value)
{
  m_lOffset = value;
}

/***************************************************************************/
/*! Increases the Offset
*/
/***************************************************************************/
void TLogger::indent()
{
  m_lOffset++;
}


/***************************************************************************/
/*! Decreases the Offset
*/
/***************************************************************************/
void TLogger::unindent()
{
  m_lOffset--;
  if (m_lOffset < 0)
  {
    m_lOffset = 0;
  }
}
