/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : LoggerManager                                                 *
 * Package : help_impl                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LoggerManager.cpp,v $
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
    
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/LibraryManager.h>

USING_NAMESPACE_Zeus

//Initialisation of the Loggermanager
TLoggerManager TLoggerManager::m_LoggerManager;

/***************************************************************************/
/*! Creates a logger manager
*/
/***************************************************************************/
TLoggerManager::TLoggerManager() : TZObject(), m_RootLogger(NULL)
{}

/***************************************************************************/
/*! Destroys the logger manager
*/
/***************************************************************************/
TLoggerManager::~TLoggerManager()
{}

/***************************************************************************/
/*! Returns the LoggerManager 
*/
/***************************************************************************/
TLoggerManager& TLoggerManager::getInstance()
{
  return m_LoggerManager;
}

/***************************************************************************/
/*! Returns the root logger object.
*/
/***************************************************************************/
TLogger& TLoggerManager::getRoot()
{
  return getInstance().m_RootLogger;
}


/***************************************************************************/
/*! Sets a new logger as a root logger
*/
/***************************************************************************/
void TLoggerManager::setLogger(ILogger* logger)
{
  m_RootLogger = logger;
}


/***************************************************************************/
/*! initializes the root logger. The logger will be allocated from a system 
    module, witch must be previously loaded using the library manager.
    \param modulename : Name of the module
    \retval RET_NOERROR : Root logger initialized
    \retval RET_REQUEST_FAILED : Could not initialize the logger
*/
/***************************************************************************/
unsigned long TLoggerManager::initLogger(TString modulename)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  TString strClassName = L"ILogger";
  m_RootLogger.printf(LOGMODE_INFO,"Init logger loading module %s...", modulename.c_str());
  ILogger* pLogger = NULL;
  if (TLibraryManager::getInstance().createObject(&modulename, &strClassName, (IZUnknown**)&pLogger) == RET_NOERROR)
  {
    m_RootLogger.println(LOGMODE_INFO,"done.");
  }
  else
  {
    m_RootLogger.println(LOGMODE_INFO,"failed.[Could not create object]");
  }

  if (pLogger != NULL)
  {
    this->setLogger(pLogger);
    pLogger->release();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! This method must be called at the end of the program, before all libraries
    are unloaded.
*/
/***************************************************************************/
void TLoggerManager::uninit()
{
  m_RootLogger = NULL;
}

