/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : NetworkInterface                                              *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:53:28 $
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: NetworkInterface.cpp,v $
 *   Revision 1.1  2005/11/16 11:53:28  bhadorn
 *   Created network interface class
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/System/NetworkInterface.h>
#include <zeusbase/System/LoggerManager.h>

#ifdef LINUX
  #include <sys/types.h> 
  #include <sys/socket.h> 
  #include <arpa/inet.h> 
  #include <net/if.h> 
  #include <netinet/in.h> 
  #include <stdio.h> 
  #include <stdlib.h> 
  #include <string.h>
  #include <sys/ioctl.h>
  
  /* get iface list */
  #define SIOCGIFCONF 0x8912 

#else
  /* get iface list */
  #define SIOCGIFCONF 0x8912

#endif
USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a network interface object
    \param name : Name of the interface 
    \param address : address of the interface
*/
/***************************************************************************/
TNetworkInterface::TNetworkInterface(TString name, TString address) : TZObject(name)
{
  m_strAddress = address;
}


/***************************************************************************/
/*! Destroys the network interface object
*/
/***************************************************************************/
TNetworkInterface::~TNetworkInterface()
{

}

/***************************************************************************/
/*! Returns the address of the network interface.
*/
/***************************************************************************/
TString TNetworkInterface::getAddress()
{
  return m_strAddress;
}


/***************************************************************************/
/*! Static method to return a interface by name.
    \param name : Name of the interface
    \param iface : network interface
    \retval RET_NOERROR : Networkinterface returned
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_REQUEST_FAILED : No such interface found
*/
/***************************************************************************/
unsigned long TNetworkInterface::getInterfaceByName(TString name, TNetworkInterface** iface)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (iface != NULL)
  {
  
  //Linux implementation 
  #ifdef LINUX
    if (name.getSize() > 64)
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "Could not find networkinterface. Name too big [%s]", name.c_str());
    }
    else
    {
      int iSocket = socket(AF_INET, SOCK_STREAM, 0); 
      int iNumRegs = 30;
      
      struct ifconf ifc; 
      ifc.ifc_buf = NULL; 
      ifc.ifc_len = sizeof(struct ifreq) * iNumRegs; 
      ifc.ifc_buf = (char*)realloc(ifc.ifc_buf, ifc.ifc_len); 
      if (::ioctl(iSocket, SIOCGIFCONF, &ifc) < 0) 
      { 
        ZLOGGER.printfln(LOGMODE_ERROR, "Could not get list of network interfaces");
      } 
      else
      {
        bool bFound = false;
        struct ifreq* pIfr = NULL; 
        struct in_addr* pAddress = NULL;
        pIfr = ifc.ifc_req; 
        for (int i = 0; !bFound && i < ifc.ifc_len; i += sizeof(struct ifreq)) 
        { 
          pAddress = (struct in_addr *) ((pIfr->ifr_ifru.ifru_addr.sa_data)+2); 
          if (name == pIfr->ifr_ifrn.ifrn_name)
          {
            *iface = new TNetworkInterface(pIfr->ifr_ifrn.ifrn_name, inet_ntoa(*pAddress));
            bFound = true;
            ulRetval = RET_NOERROR;
          }
          else
          {
            pIfr++;
          }
        }
      }
      free(ifc.ifc_buf); 
    }
  #else
    //TODO: Windows implementation
  #endif
  }
  return ulRetval;
}

 
