/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : STLBindings                                                   *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:23:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: STLBindings.hpp,v $
 *   Revision 1.1  2005/11/10 10:23:50  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef STLBindingsHPP
#define STLBindingsHPP 

#include <istream>
#include <zeusbase/System/String.h>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This class does the bindings to classes of the standard library and
    implements some usefull functions
*/
/***************************************************************************/
class TSTLBindings
{
  public:
    /***********************************************************************/
    /*! Read a double out of a input stream
        \param stream : Stream class
        \return double value
    */
    /***********************************************************************/
    static double readDouble(std::istream* stream)
    {
      double dRetval = 0.0;
      long lSize = sizeof(double);
      char* pBuffer = new char[lSize];
      ::memset(pBuffer, 0, sizeof(char)*lSize);
      
      stream->read(pBuffer, lSize);
      dRetval = *((double*)pBuffer);
      
      delete [] pBuffer;
      return dRetval;
    }

    /***********************************************************************/
    /*! Read a long out of a input stream
        \param stream : Stream class
        \return long value
    */
    /***********************************************************************/
    static long readLong(std::istream* stream)
    {
      long lRetval = 0;
      long lSize = sizeof(long);
      char* pBuffer = new char[lSize];
      ::memset(pBuffer, 0, sizeof(char)*lSize);
    
      stream->read(pBuffer, lSize);
      lRetval = *((long*)pBuffer);
    
      delete [] pBuffer;
      return lRetval;
    }
    
    /***********************************************************************/
    /*! Read an array of bytes out of a input stream
        \param stream : Stream class
        \param data : Return parameter of the array
        \retval RET_NOERROR : Array read
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    static unsigned long readArray(std::istream* stream, IByteArray* data)
    {
      unsigned long ulRetval = RET_INVALID_PARAMETER;
      if (data != NULL)
      {
        long lSize = TSTLBindings::readLong(stream);
        if (lSize > 0)
        {
          char* pBuffer = new char[lSize];
          ::memset(pBuffer, 0, sizeof(char)*lSize);
          
          stream->read(pBuffer, lSize);
          
          data->setArray(pBuffer, lSize);
          delete [] pBuffer;    
        }
        ulRetval = RET_NOERROR;
      }
      return ulRetval;
    }
    
    /***********************************************************************/
    /*! Read a string of bytes out of a input stream
        \param stream : Stream class
        \param data : Return parameter of the string
        \retval RET_NOERROR : string read
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    static unsigned long readString(std::istream* stream, IString* data)
    {
      unsigned long ulRetval = RET_INVALID_PARAMETER;
      if (data != NULL)
      {
        TString strRetval;
        long lStringSize = TSTLBindings::readLong(stream);
        if (lStringSize > 0)
        {
          char* pBuffer = new char[lStringSize];
          ::memset(pBuffer, 0, sizeof(char)*lStringSize);
          
          stream->read(pBuffer, lStringSize);
          
          strRetval = pBuffer;
          delete [] pBuffer;
        }
        data->assignStr(&strRetval);
        ulRetval = RET_NOERROR;
      }
      return ulRetval;
    }
};

END_NAMESPACE_Zeus

#endif
