/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Transcoder                                                    *
 * Package : System/Platforms                                              *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:23:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Transcoder.hpp,v $
 *   Revision 1.1  2005/11/10 10:23:50  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
 
#ifndef TranscoderHPP
#define TranscoderHPP

#include <zeusbase/System/String.h>
#include <stdlib.h>


//#############################################################################
//STL Transforms
#ifdef USE_STL_BINDINGS
  
#include <string>

#define _STD_STR(arg)   NAMESPACE_Zeus::TSTLTranscoder::transcodeStr(arg)
#define _STD_WSTR(arg)  NAMESPACE_Zeus::TSTLTranscoder::transcodeWStr(arg)

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! This transcoder is used to transform stl datatypes to the zeus-datatypes
*/
/******************************************************************************/
class TSTLTranscoder
{
  public:  
    /**************************************************************************/  
    /*! Transform a string to a zeus-string
        \param value : Input string
        \retval zeus-string
    */
    /**************************************************************************/  
    static TString transcodeStr(const std::string& value)
    {
      return TString(value.c_str());
    }
    
    /**************************************************************************/  
    /*! Transform a zeus-string to a stl-string
        \param value : Input string
        \retval stl-string
    */
    /**************************************************************************/  
    static std::string transcodeStr(const TString& value)
    {
      return std::string(value.c_str());
    }

    /**************************************************************************/  
    /*! Transform a wide-string to a zeus-string
        \param value : Input string
        \retval zeus-string
    */
    /**************************************************************************/  
    static TString transcodeWStr(const std::wstring& value)
    {
      return TString(value.c_str());
    }
    
    /**************************************************************************/  
     /*! Transform a zeus-string to a wide-string
        \param value : Input string
        \retval wide-string
    */
    /**************************************************************************/  
    static std::wstring transcodeWStr(const TString& value)
    {
      return std::wstring(value.c_bstr());
    }
};
END_NAMESPACE_Zeus
#endif

//#############################################################################
// QT Transforms
#ifdef USE_QT_BINDINGS

#define _QTWSTR(arg)  NAMESPACE_Zeus::TQTTranscoder::transcodeStr(arg)


#include <qstring.h>
BEGIN_NAMESPACE_Zeus
/******************************************************************************/
/*! This transcoder is used to transform QT datatypes to the zeus-datatypes
*/
/******************************************************************************/
class TQTTranscoder 
{
  public:

    /**************************************************************************/  
    /*! Transform a qstring to a zeus-string
        \param value : Input qstring
        \retval zeus-string
    */
    /**************************************************************************/  
    static TString transcodeStr(const QString& value)
    {
      return TString(value.ascii());
    }
    
    /**************************************************************************/  
     /*! Transform a zeus-string to a wide-string
        \param value : Input string
        \retval wide-string
    */
    /**************************************************************************/  
    static QString transcodeStr(const TString& value)
    {
      return QString::fromAscii(value.c_str());
    }
};
END_NAMESPACE_Zeus
 
#endif
     
//#############################################################################
// Borland Transforms
#ifdef USE_BCB_BINDINGS
#include <Classes.hpp>

#define _BCBSTR(arg)   NAMESPACE_Zeus::TBCBTranscoder::transcodeStr(arg)
#define _BCBWSTR(arg)  NAMESPACE_Zeus::TBCBTranscoder::transcodeWStr(arg)

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! This transcoder is used to transform borland datatypes to the zeus-datatypes
*/
/******************************************************************************/
class TBCBTranscoder 
{
  public:
    /**************************************************************************/  
    /*! 
    */
    /**************************************************************************/  
    static TString transcodeStr(const AnsiString& value)
    {
      return TString(value.c_str());
    }
    
    /**************************************************************************/  
    /*! 
    */
    /**************************************************************************/  
    static TString transcodeWStr(const WideString& value)
    {
      return TString(value.c_bstr());
    }
    
    /**************************************************************************/  
    /*! 
    */
    /**************************************************************************/  
    static AnsiString transcodeStr(const TString& value)
    {
      return AnsiString(value.c_str());
    }
    
    /**************************************************************************/  
    /*! 
    */
    /**************************************************************************/  
    static WideString transcodeWStr(const TString& value)
    {
      return WideString(value.c_bstr());
    }
};
END_NAMESPACE_Zeus

#endif


#endif
