/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : PropertyFile                                                  *
 * Package : system                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: PropertyFile.h,v $
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef PropertyFileH
#define PropertyFileH

#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/String.h>
#include <zeusbase/System/File.h>
#include <zeusbase/System/Interfaces/IPropertyAccess.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines the access to a property collection (file)
*/
/***************************************************************************/
class TPropertyFile : public TFile, public IPropertyAccess
{
  public:    
    TPropertyFile(TString filename);

    unsigned long load();
    
    //Methods of IPropertyAccess
    virtual unsigned long MQUALIFIER addProperty(IString* name, IString* value);
    virtual bool MQUALIFIER existsProperty(IString* name);
    virtual unsigned long MQUALIFIER getPropertyNames(IStringList* list);
    virtual unsigned long MQUALIFIER getPropertyValue(IString* name, IString* value); 
    virtual unsigned long MQUALIFIER setPropertyValue(IString* name, IString* value); 
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    virtual ~TPropertyFile();

  private:
    /*********************************************************************/
    /*! Record of a property
    */
    /*********************************************************************/
    struct RProperty
    {
      ///Name of the property
      TString strName;
      ///Value of the property
      TString strValue;
    };
    
    ///List of properties
    TSingleLinkedList<RProperty*> m_lstProperties;
    
    
    void removeAll();
    
};
 
END_NAMESPACE_Zeus


#endif
