/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Cell Registry                                                 *
 * Module  : CCTPRequest                                                    *
 * Package : Protocols                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:35:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CCTPReguest.cpp,v $
 *   Revision 1.2  2005/11/16 11:35:17  bhadorn
 *   In parameter are [const]
 *
 *   Revision 1.1  2005/11/10 10:24:34  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeusbase/System/Protocols/CCTPRequest.h>
#include <stdio.h>


USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a CCTP Request Object
*/
/***************************************************************************/
TCCTPRequest::TCCTPRequest() : TZObject()
{
  m_ulVersion = CCTP_VERSION_1; //Version 1
  m_ulMethod = CCTP_METHOD_POST;
  m_ulObjectType = INTERFACE_IZUnknown;
}

/***************************************************************************/
/*! Destroys the CCTP Request Object
*/
/***************************************************************************/
TCCTPRequest::~TCCTPRequest()
{} 


/***************************************************************************/
/*! Returns the data body
*/
/***************************************************************************/
const TByteArray& TCCTPRequest::getData() const
{ return m_aData; }

/***************************************************************************/
/*! Returns the method of the CCTP
*/
/***************************************************************************/
unsigned long TCCTPRequest::getMethod() const
{ return m_ulMethod; }

/***************************************************************************/
/*! returns the object type
    \return Type of object 
*/
/***************************************************************************/
unsigned long TCCTPRequest::getObjectType() const
{
  return m_ulObjectType;
}


/***************************************************************************/
/*! Sets the data body
*/
/***************************************************************************/
void TCCTPRequest::setData(const TByteArray& data)
{
  m_aData.setArray(data.getArray(), data.getCount());
}

/***************************************************************************/
/*! Sets a method of the CCTP
*/
/***************************************************************************/
void TCCTPRequest::setMethod(unsigned long method)
{
  m_ulMethod = method;
}

/***************************************************************************/
/*! Sets the object type
    \param obj : Type of object 
*/
/***************************************************************************/
void TCCTPRequest::setObjectType(unsigned long obj)
{
  m_ulObjectType = obj;
}


/***************************************************************************/
/*! Returns a stream witch contains the CCTP Request
*/
/***************************************************************************/
unsigned long TCCTPRequest::toStream(TByteArray* array) const
{  
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (array == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    //--------------------------------------------
    // Protocol for Requests
    //--------------------------------------------
    // Byte1    | Byte2    | Byte3    | Byte4    |
    //--------------------------------------------
    // Version  | Method   | res      | res      |
    //--------------------------------------------
    // Object Type (Interface or Class ID)
    //--------------------------------------------
    // Body Size
    //--------------------------------------------
    // Body
    // ...
    //--------------------------------------------
    
   
    long lBodySize = m_aData.getCount();
       
    //Erstellen der Bytereihenfolge
    array->clear();
    array->add((char)m_ulVersion);
    array->add((char)m_ulMethod);
    array->add(0x00);
    array->add(0x00);
    
    char* cDW2Byte;
    cDW2Byte = (char*)&m_ulObjectType;
    array->add(cDW2Byte[0]);
    array->add(cDW2Byte[1]);
    array->add(cDW2Byte[2]);
    array->add(cDW2Byte[3]);

    cDW2Byte = (char*)&lBodySize;
    //printf("stream body size = %d", lBodySize);
    array->add(cDW2Byte[0]);
    array->add(cDW2Byte[1]);
    array->add(cDW2Byte[2]);
    array->add(cDW2Byte[3]);
    
    array->appendArray(m_aData.getArray(), lBodySize);
    
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! This Method parses a incomming request
    \param in : Input data
    \retval RET_NOERROR : Parsing ok
    \retval RET_REQUEST_FAILED : wrong format
*/
/***************************************************************************/
unsigned long TCCTPRequest::parse(const TByteArray& in)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (in.getCount() >= 12)
  {
    m_ulVersion = in.getItemCopy(0);
    m_ulMethod = in.getItemCopy(1);
        
    char cByte2DWObj[4] = {in.getItemCopy(4), in.getItemCopy(5), in.getItemCopy(6), in.getItemCopy(7)};
    unsigned long* pObjectType = (unsigned long*)cByte2DWObj;
    
    char cByte2DWBody[4] = {in.getItemCopy(8), in.getItemCopy(9), in.getItemCopy(10), in.getItemCopy(11)};
    unsigned long* pBodySize = (unsigned long*)cByte2DWBody;
    //printf("parse body size = %d", *pBodySize);
    
    m_ulObjectType = *pObjectType;
    
    m_aData.clear();
    
    if (in.getCount() > 12 && *pBodySize > 0)
    {
      in.copyToArray(&m_aData, 12, *pBodySize + 12 -1);
    }
    ulRetval = RET_NOERROR;
  } 
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}
