/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Cell Registry                                                 *
 * Module  : CCTPResponse                                                   *
 * Package : Protocols                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:35:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CCTPResponse.cpp,v $
 *   Revision 1.2  2005/11/16 11:35:17  bhadorn
 *   In parameter are [const]
 *
 *   Revision 1.1  2005/11/10 10:24:34  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeusbase/System/Protocols/CCTPResponse.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a CCTP Request Object
*/
/***************************************************************************/
TCCTPResponse::TCCTPResponse() : TZObject()
{
  m_ulVersion = CCTP_VERSION_1;
  m_ulState = CCTP_STATUS_OK;
  
}

/***************************************************************************/
/*! Destroys the CCTP Request Object
*/
/***************************************************************************/
TCCTPResponse::~TCCTPResponse()
{} 
    
/***************************************************************************/
/*! Gets the data body of the response
*/
/***************************************************************************/
const TByteArray& TCCTPResponse::getData() const
{
  return m_aData;
}

/***************************************************************************/
/*! Gets the state of the response
*/
/***************************************************************************/
unsigned long TCCTPResponse::getState() const
{
  return m_ulState;
}

/***************************************************************************/
/*! Sets the data body of the response
*/
/***************************************************************************/
void TCCTPResponse::setData(const TByteArray& data)
{
  m_aData.setArray(data.getArray(), data.getCount());
}


/***************************************************************************/
/*! Sets the status of the response
*/
/***************************************************************************/
void TCCTPResponse::setState(unsigned long state)
{
  m_ulState = state;
}

/***************************************************************************/
/*! Returns a string witch contains the CCTP Request
*/
/***************************************************************************/
unsigned long TCCTPResponse::toStream(IByteArray* array) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (array == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    //--------------------------------------------
    // Protocol for Requests
    //--------------------------------------------
    // Byte1    | Byte2    | Byte3    | Byte4    |
    //--------------------------------------------
    // Version  | res   | res      | res      |
    //--------------------------------------------
    // State
    //--------------------------------------------
    // Data Size
    //--------------------------------------------
    // Data 
    // ...
    //--------------------------------------------
    
   
    //Erstellen der Bytereihenfolge
    array->clear();
    array->add((char)m_ulVersion);
    array->add(0x00);
    array->add(0x00);
    array->add(0x00);
    
    //State
    char* cDW2Byte;
    cDW2Byte = (char*)&m_ulState;
    array->add(cDW2Byte[0]);
    array->add(cDW2Byte[1]);
    array->add(cDW2Byte[2]);
    array->add(cDW2Byte[3]);
    
    //Size of data
    long lDataSize = m_aData.getCount();
    cDW2Byte = (char*)&lDataSize;
    array->add(cDW2Byte[0]);
    array->add(cDW2Byte[1]);
    array->add(cDW2Byte[2]);
    array->add(cDW2Byte[3]);

    array->appendArray(m_aData.getArray(), lDataSize);
    
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! This Method parses a incomming response
    \param in : Input data
    \retval RET_NOERROR : Parsing ok
    \retval RET_REQUEST_FAILED : wrong format
*/
/***************************************************************************/
unsigned long TCCTPResponse::parse(const TByteArray& in)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (in.getCount() >= 12)
  {
    m_ulVersion = in.getItemCopy(0);
        
    char cByte2DWState[4] = {in.getItemCopy(4), in.getItemCopy(5), in.getItemCopy(6), in.getItemCopy(7)};
    unsigned long* pState = (unsigned long*)cByte2DWState;
    
    char cByte2DWData[4] = {in.getItemCopy(8), in.getItemCopy(9), in.getItemCopy(10), in.getItemCopy(11)};
    unsigned long* pDataSize = (unsigned long*)cByte2DWData;
    
    m_ulState = *pState;
    
    if (in.getCount() > 12 && *pDataSize > 0)
    {
      
      in.copyToArray(&m_aData, 12, *pDataSize+12-1);
      
      //m_strData = TString::transcode(aData);
    }
    ulRetval = RET_NOERROR;
  } 
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}





