/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Cell Registry                                                 *
 * Module  : HTTPRequest                                                    *
 * Package : Protocols                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:24:34 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: HTTPReguest.cpp,v $
 *   Revision 1.1  2005/11/10 10:24:34  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeusbase/System/Protocols/HTTPRequest.h>


USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a HTTP Request Object
*/
/***************************************************************************/
THTTPRequest::THTTPRequest() : TZObject(), m_lstProperties(RProperty())
{
  m_strProtocol = L"HTTP/1.0";
  m_ulMethod = HTTP_METHOD_POST;
}

/***************************************************************************/
/*! Destroys the HTTP Request Object
*/
/***************************************************************************/
THTTPRequest::~THTTPRequest()
{} 


/***************************************************************************/
/*! Returns the data body
*/
/***************************************************************************/
TString THTTPRequest::getData()
{ return m_strData; }

/***************************************************************************/
/*! Returns the method of the HTTP
*/
/***************************************************************************/
unsigned long THTTPRequest::getMethod()
{ return m_ulMethod; }

/***************************************************************************/
/*! Returns the uri of a ressource
*/
/***************************************************************************/
TString THTTPRequest::getRessource()
{ return m_strRessourceURI; }

/***************************************************************************/
/*! Sets the data body
*/
/***************************************************************************/
void THTTPRequest::setData(TString data)
{
  m_strData = data;
}

/***************************************************************************/
/*! Sets a method of the HTTP
*/
/***************************************************************************/
void THTTPRequest::setMethod(unsigned long method)
{
  m_ulMethod = method;
}

/***************************************************************************/
/*! Sets an uri of a ressource
*/
/***************************************************************************/
void THTTPRequest::setRessource(TString ressource)
{
  m_strRessourceURI = ressource;
}
/***************************************************************************/
/*! Returns the value of a named property
*/
/***************************************************************************/
TString THTTPRequest::getProperty(TString name)
{
  TString strRetval;
  if (name != L"")
  {
    long lIndex = this->findProperty(name);
    strRetval = m_lstProperties[lIndex].strValue;
  }
  return strRetval;
}

/***************************************************************************/
/*! Tries to find a property by name.
    \param name : Name of the property
    \retval -1 : No property with this name found
    \retval >=0 : Index of property
*/
/***************************************************************************/
long THTTPRequest::findProperty(TString name)
{
  long lRetval = -1;
  bool bFound = false;
  long lIndex = 0;
  while(lIndex < m_lstProperties.getCount() && !bFound)
  {
    RProperty Prop = m_lstProperties[lIndex];
    if (Prop.strName == name)
    {
      lRetval = lIndex;
      bFound = true;
    }
    else
    {
      lIndex++;
    }
  }  
  return lRetval;
}

/***************************************************************************/
/*! Returns the value of a named property
*/
/***************************************************************************/
void THTTPRequest::setProperty(TString name, TString value)
{
  if (name != L"")
  {
    long lIndex = this->findProperty(name);
    if (lIndex >= 0)
    {
      m_lstProperties[lIndex].strValue = value;
    }
    else
    {
      m_lstProperties.add(RProperty(name, value));
    }
  }
}

/***************************************************************************/
/*! Returns a string witch contains the HTTP Request
*/
/***************************************************************************/
TString THTTPRequest::toString()
{  
  TString strRetval;
  switch(m_ulMethod)
  {
    case HTTP_METHOD_POST:  
      strRetval = L"POST";
    break;
    
    default:
      strRetval = L"UNKNOWN";
  }
  TString strRess = m_strRessourceURI;
  if (strRess == L"")
  {
    strRess = "/";
  }
  
  strRetval = strRetval + TString(L" ") + strRess + TString(L" ") + 
              m_strProtocol + TString(L"\n\n");
  
  if (m_strData != L"")
  {
    strRetval = strRetval + m_strData;
  }
  
  return strRetval;
}

/***************************************************************************/
/*! Internal Scanner to detect tokens and patterns of the incomming 
    request.
    \param buffer : Char Buffer
    \param size: Size of the buffer
    \param queue : Queue to collect all tokens
*/
/***************************************************************************/
void THTTPRequest::scanLines(char* buffer, long size, TQueue<RToken>& queue)
{
  long lIndex = 0;
  long lStringIndex = 0;
  unsigned long ulScanState = HTTPSCAN_STATE_HEADER; /*Initial State for header: Reading version*/
  RToken Token;
  while (ulScanState == HTTPSCAN_STATE_HEADER)
  {
    switch(buffer[lIndex])
    {
      case 0x20 : /*SPACE*/
      {
        buffer[lIndex] = '\0';
        Token.strData = (char*)(buffer + lStringIndex);
        Token.ulType = HTTP_TOKEN_DATA;
        queue.apendItem(Token);
        
        Token.strData = L"";
        Token.ulType = HTTP_TOKEN_SPACE;
        queue.apendItem(Token);
        
        lStringIndex = lIndex+1;
      }
      break;
          
      case '\n' : /*NEW LINE*/
      {
        if (lStringIndex < lIndex)
        {
          buffer[lIndex] = '\0';
          Token.strData = (char*)(buffer + lStringIndex);
          Token.ulType = HTTP_TOKEN_DATA;
          queue.apendItem(Token);
        }
        else
        {
          ulScanState = HTTPSCAN_STATE_BODY;
        }
        Token.strData = L"";
        Token.ulType = HTTP_TOKEN_NEWLINE;
        queue.apendItem(Token);
        
        lStringIndex = lIndex+1;
      }
      break;
      
      case '\r' : /*Ignore CARRIGE RETURN*/
        lStringIndex++;
      break;
      
      default :
      ;
      
    }
    lIndex++;
    
    if (lIndex >= size)
    {
      ulScanState = HTTPSCAN_STATE_ERROR;
    }
  }
  
  if (ulScanState == HTTPSCAN_STATE_BODY)
  {
    Token.strData = (char*)(buffer + lStringIndex);
    Token.ulType = HTTP_TOKEN_DATA;
    queue.apendItem(Token);
  }    
}


/***************************************************************************/
/*! This Method parses a incomming request
    \param in : Input data
    \retval RET_NOERROR : Parsing ok
    \retval RET_REQUEST_FAILED : wrong format
*/
/***************************************************************************/
unsigned long THTTPRequest::parse(TString in)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  unsigned long ulParseState = HTTPPARSE_STATE_METHOD;
  
  m_ulMethod = HTTP_METHOD_POST;
  m_strData = L"";
  m_strRessourceURI = "";
  m_strProtocol = L"HTTP/1.0";

  long lLength = in.getSize();
  if (lLength > 0)
  {    
    RToken EmptyToken;
    TQueue<RToken> qTokens(EmptyToken);
    
    char* cBuffer = new char [lLength+1];
    cBuffer = strcpy(cBuffer, in.c_str());
    cBuffer[lLength] = '\0';
    scanLines(cBuffer, lLength, qTokens);
    
    while(!qTokens.isEmpty() && ulParseState != HTTPPARSE_STATE_ERROR)
    {
      RToken Token = qTokens.getItem();
      
      switch(ulParseState)
      {
        //-----------------------------------------------------------
        // Parse Method
        case HTTPPARSE_STATE_METHOD :
          switch(Token.ulType)
          {
            case HTTP_TOKEN_DATA :
              if (Token.strData == L"POST")
              {
                m_ulMethod = HTTP_METHOD_POST;
              }
              else
              {
                m_ulMethod = HTTP_METHOD_UNKNOWN;
              }
            break;
                          
            case HTTP_TOKEN_SPACE :
              ulParseState = HTTPPARSE_STATE_RESSOURCE;
            break;
            
            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
        
        //-----------------------------------------------------------
        // Parse Ressource URI
        case HTTPPARSE_STATE_RESSOURCE :
          switch(Token.ulType)
          {
            case HTTP_TOKEN_DATA :
              m_strRessourceURI = Token.strData;
              if (m_strRessourceURI == L"/")
              {
                m_strRessourceURI = L"";
              }
            break;
              
            case HTTP_TOKEN_SPACE :
              ulParseState = HTTPPARSE_STATE_PROTOCOL;
            break;
            
            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
        
        //-----------------------------------------------------------
        // Parse Method
        case HTTPPARSE_STATE_PROTOCOL :
          switch(Token.ulType)
          {
            case HTTP_TOKEN_DATA :
              m_strProtocol = Token.strData;
              if (Token.strData != L"HTTP/1.0")
              {
                ulParseState = HTTPPARSE_STATE_ERROR;
              }
              
            break;
                          
            case HTTP_TOKEN_NEWLINE :
              ulParseState = HTTPPARSE_STATE_HEADER_START;
            break;
            
            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
      
              
        //-----------------------------------------------------------
        // Start parsing Header
        case HTTPPARSE_STATE_HEADER_START :
          switch(Token.ulType)
          {            
            case HTTP_TOKEN_DATA :
              //TODO : Add Header String
              ulParseState = HTTPPARSE_STATE_HEADER;
            break;
              
            case HTTP_TOKEN_NEWLINE :
              ulParseState = HTTPPARSE_STATE_BODY;
            break;

            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
      
        //-----------------------------------------------------------
        // Parse Header
        case HTTPPARSE_STATE_HEADER :
          switch(Token.ulType)
          {            
            //TODO: Complete Header
            case HTTP_TOKEN_DATA :
            break;
            case HTTP_TOKEN_SPACE :
            break;
              
            case HTTP_TOKEN_NEWLINE :
              ulParseState = HTTPPARSE_STATE_HEADER_START;
            break;

            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
        
        //-----------------------------------------------------------
        // Parse Body
        case HTTPPARSE_STATE_BODY :
          switch(Token.ulType)
          {            
            case HTTP_TOKEN_DATA :
              m_strData = Token.strData;
            break;

            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
        
        default:
          ulParseState = HTTPPARSE_STATE_ERROR;
      }
    }
    
    if (ulParseState == HTTPPARSE_STATE_ERROR)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
    else
    {
      ulRetval = RET_NOERROR;
    }
    delete [] cBuffer;
  }
  
  return ulRetval;
}



