/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Cell Registry                                                 *
 * Module  : HTTPResponse                                                   *
 * Package : Protocols                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:24:34 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: HTTPResponse.cpp,v $
 *   Revision 1.1  2005/11/10 10:24:34  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeusbase/System/Protocols/HTTPResponse.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a HTTP Request Object
*/
/***************************************************************************/
THTTPResponse::THTTPResponse() : TZObject()
{
  m_strProtocol = L"HTTP/1.0";
  m_ulState = HTTP_STATUS_OK;
  
}

/***************************************************************************/
/*! Destroys the HTTP Request Object
*/
/***************************************************************************/
THTTPResponse::~THTTPResponse()
{} 
    
/***************************************************************************/
/*! Gets the data body of the response
*/
/***************************************************************************/
TString THTTPResponse::getData()
{
  return m_strData;
}

/***************************************************************************/
/*! Gets the state of the response
*/
/***************************************************************************/
unsigned long THTTPResponse::getState()
{
  return m_ulState;
}

/***************************************************************************/
/*! Sets the data body of the response
*/
/***************************************************************************/
void THTTPResponse::setData(TString data)
{
  m_strData = data;
}


/***************************************************************************/
/*! Sets the status of the response
*/
/***************************************************************************/
void THTTPResponse::setState(unsigned long state)
{
  m_ulState = state;
}

/***************************************************************************/
/*! Returns a string witch contains the HTTP Request
*/
/***************************************************************************/
TString THTTPResponse::toString()
{
  TString strRetval = m_strProtocol + TString(L" ") + TString((long)m_ulState) + TString(L" ");
  switch(m_ulState)
  {
    case HTTP_STATUS_OK :
      strRetval = strRetval + L"OK\n";
    break;
    
    case HTTP_STATUS_BADREQUEST :
      strRetval = strRetval + L"Bad Request\n";
    break;
    
    case HTTP_STATUS_METHODFAILED :
      strRetval = strRetval + L"Method failed\n";
    break;
    
    case HTTP_STATUS_INTERNALSERVERERROR :
      strRetval = strRetval + L"Internal Server Error\n";
    break;
    
    default: 
    ;
  } 
  
  strRetval = strRetval + TString(L"\n");
  
  if (m_strData != L"")
  {
    strRetval = strRetval + m_strData;
  } 
  return strRetval;
}

/***************************************************************************/
/*! Internal Scanner to detect tokens and patterns of the incomming 
    response.
    \param buffer : Char Buffer
    \param size: Size of the buffer
    \param queue : Queue to collect all tokens
*/
/***************************************************************************/
void THTTPResponse::scanLines(char* buffer, long size, TQueue<RToken>& queue)
{
  long lIndex = 0;
  long lStringIndex = 0;
  unsigned long ulScanState = HTTPSCAN_STATE_HEADER; /*Initial State for header: Reading version*/
  RToken Token;
  
  while (ulScanState == HTTPSCAN_STATE_HEADER)
  {
    switch(buffer[lIndex])
    {
      case 0x20 : /*SPACE*/
      {
        buffer[lIndex] = '\0';
        Token.strData = (char*)(buffer + lStringIndex);
        Token.ulType = HTTP_TOKEN_DATA;
        queue.apendItem(Token);
        
        Token.strData = L"";
        Token.ulType = HTTP_TOKEN_SPACE;
        queue.apendItem(Token);
        
        lStringIndex = lIndex+1;
      }
      break;
          
      case '\n' : /*NEW LINE*/
      {
        if (lStringIndex < lIndex)
        {
          buffer[lIndex] = '\0';
          Token.strData = (char*)(buffer + lStringIndex);
          Token.ulType = HTTP_TOKEN_DATA;
          queue.apendItem(Token);
        }
        else
        {
          ulScanState = HTTPSCAN_STATE_BODY;
        }
        Token.strData = L"";
        Token.ulType = HTTP_TOKEN_NEWLINE;
        queue.apendItem(Token);
        
        lStringIndex = lIndex+1;
      }
      break;
      
      case '\r' : /*Ignore CARRIGE RETURN*/
        lStringIndex++;
      break;
      
      default :
      ;
      
    }
    lIndex++;
    
    if (lIndex >= size)
    {
      ulScanState = HTTPSCAN_STATE_ERROR;
    }
  }
  
  if (ulScanState == HTTPSCAN_STATE_BODY)
  {
    Token.strData = (char*)(buffer + lStringIndex);
    Token.ulType = HTTP_TOKEN_DATA;
    queue.apendItem(Token);
  }    
}

/***************************************************************************/
/*! This Method parses a incomming response
    \param in : Input data
    \retval RET_NOERROR : Parsing ok
    \retval RET_REQUEST_FAILED : wrong format
*/
/***************************************************************************/
unsigned long THTTPResponse::parse(TString in)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  unsigned long ulParseState = HTTPPARSE_STATE_PROTOCOL;
  m_ulState = HTTP_STATUS_OK;
  m_strData = L"";
  m_strProtocol = L"HTTP/1.0";
  
  long lLength = in.getSize();
  if (lLength > 0)
  {    
    RToken EmptyToken;
    TQueue<RToken> qTokens(EmptyToken);
    
    char* cBuffer = new char [lLength+1];
    cBuffer = strcpy(cBuffer, in.c_str());
    cBuffer[lLength] = '\0';
    scanLines(cBuffer, lLength, qTokens);
        
    while(!qTokens.isEmpty() && ulParseState != HTTPPARSE_STATE_ERROR)
    {
      RToken Token = qTokens.getItem();
      
      switch(ulParseState)
      {
        //-----------------------------------------------------------
        // Parse Protocol
        case HTTPPARSE_STATE_PROTOCOL :
          switch(Token.ulType)
          {
            case HTTP_TOKEN_DATA :
              if (Token.strData != L"HTTP/1.0")
              {
                ulParseState = HTTPPARSE_STATE_ERROR;
              }
              else
              {
                m_strProtocol = Token.strData;
              }
              
            break;
                          
            case HTTP_TOKEN_SPACE :
              ulParseState = HTTPPARSE_STATE_STATUSCODE;
            break;
            
            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
      
        //-----------------------------------------------------------
        // Parse Status code
        case HTTPPARSE_STATE_STATUSCODE :
          switch(Token.ulType)
          {
            case HTTP_TOKEN_DATA :
              m_ulState = Token.strData.toLong(HTTP_STATUS_OK);
            break;
              
            case HTTP_TOKEN_SPACE :
              ulParseState = HTTPPARSE_STATE_STATUSTEXT;
            break;
            
            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
      
        //-----------------------------------------------------------
        // Parse Status text
        case HTTPPARSE_STATE_STATUSTEXT :
          switch(Token.ulType)
          {
            
            //Ignore Data and Spaces
            case HTTP_TOKEN_SPACE :
            break;
            
            case HTTP_TOKEN_DATA :
            break;
              
            case HTTP_TOKEN_NEWLINE :
              ulParseState = HTTPPARSE_STATE_HEADER_START;
            break;

            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
        
        //-----------------------------------------------------------
        // Start parsing Header
        case HTTPPARSE_STATE_HEADER_START :
          switch(Token.ulType)
          {            
            case HTTP_TOKEN_DATA :
              //TODO : Add Header String
              ulParseState = HTTPPARSE_STATE_HEADER;
            break;
              
            case HTTP_TOKEN_NEWLINE :
              ulParseState = HTTPPARSE_STATE_BODY;
            break;

            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
      
        //-----------------------------------------------------------
        // Parse Header
        case HTTPPARSE_STATE_HEADER :
          switch(Token.ulType)
          {            
            //TODO: Complete Header
            case HTTP_TOKEN_DATA :
            break;
            case HTTP_TOKEN_SPACE :
            break;
              
            case HTTP_TOKEN_NEWLINE :
              ulParseState = HTTPPARSE_STATE_HEADER_START;
            break;

            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
        
        //-----------------------------------------------------------
        // Parse Body
        case HTTPPARSE_STATE_BODY :
          switch(Token.ulType)
          {            
            case HTTP_TOKEN_DATA :
              m_strData = Token.strData;
            break;

            default:
              ulParseState = HTTPPARSE_STATE_ERROR;
            ;
          }  
        break;
        
        default:
          ulParseState = HTTPPARSE_STATE_ERROR;
      }
    }
    
    if (ulParseState == HTTPPARSE_STATE_ERROR)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
    else
    {
      ulRetval = RET_NOERROR;
    }
    
    delete [] cBuffer;
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  
  return ulRetval;
}





