/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Cell Registry                                                 *
 * Module  : HTTPResponse                                                   *
 * Package : Protocols                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:24:34 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: HTTPResponse.h,v $
 *   Revision 1.1  2005/11/10 10:24:34  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#ifndef HTTPResponseH 
#define HTTPResponseH
 
#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Protocols/HTTPDefines.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>
#include <zeusbase/System/Queue.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This class wrapps a HTTP Response object
*/
/***************************************************************************/
class THTTPResponse : public TZObject
{
  public:
    THTTPResponse();
    virtual ~THTTPResponse();
    
    TString getData();
    unsigned long getState();
    
    unsigned long parse(TString in);
    TString toString();
    void setData(TString data);
    void setState(unsigned long state);
    
  
  private:
    ///Protocol number
    TString m_strProtocol;
    ///Status of Response
    unsigned long m_ulState;
    ///Databody
    TString m_strData;
    
    /***********************************************************************/
    /*! Internal Scanner Token.
    */
    /***********************************************************************/
    struct RToken
    {
      ///Token Type
      unsigned long ulType;
      ///Data of Token
      TString strData;
      
      /*********************************************************************/
      /*! Creates a Scanner Token
      */
      /*********************************************************************/
      RToken()
      {
        ulType = HTTP_TOKEN_DATA;
      }

      /*********************************************************************/
      /*! Compares two Scanner Tokens
      */
      /*********************************************************************/
      bool operator==(const RToken& inpar)
      {
        return (ulType == inpar.ulType &&
                strData == inpar.strData);
      }
    };


    void scanLines(char* buffer, long size, TQueue<RToken>& queue);
};

END_NAMESPACE_Zeus

#endif
