#ifndef QueueH
#define QueueH

#include <stddef.h>
#include <string.h>


/******************************************************************************/
/*! Implementation des Abstrakten DatenTyps FIFO-Queue
*/
/******************************************************************************/
template <class T> class TQueue
{
  public:
    ///Konstruktor
    TQueue();
    TQueue(const T& EmptyObject);
    ///Destruktor
    ~TQueue();

    ///Leeren der Queue
    void flush();
    ///Ein Objekt in die Queue legen
    void apendItem(const T& data);
    ///Das oberste Objekt entfernen
    T getItem();
    ///Groesse der Queue ermitteln
    long getSize();
    ///Ermitteln ob die Queue leer ist
    bool isEmpty();


  private:
    ///Containerklasse der Daten, die verlinked werden
    struct TBaseElement
    {
      TBaseElement(T data, TBaseElement* next)
      {
        Data = data;
        Next = next;
      }

      T Data;
      TBaseElement* Next;
    };

    ///Queue Anfang
    TBaseElement* m_pElHead;
    ///Queue Ende
    TBaseElement* m_pElTail;
    ///Groesse der Queue
    long m_lSize;
    ///Das Leere Objekt
    T m_EmptyObject;

};



/******************************************************************************/
/*! Erstellen der Queue
*/
/******************************************************************************/
template <class T> TQueue<T>::TQueue()
{
  m_pElHead = NULL;
  m_pElTail = NULL;
  m_lSize = 0;
  memset(&m_EmptyObject,0, sizeof(T));
}

/******************************************************************************/
/*! Erstellen der Queue
*/
/******************************************************************************/
template <class T> TQueue<T>::TQueue(const T& EmptyObject)
{
  m_pElHead = NULL;
  m_pElTail = NULL;
  m_lSize = 0;
  m_EmptyObject = EmptyObject;
}


/******************************************************************************/
/*! Zerstoeren der Queue
*/
/******************************************************************************/
template <class T> TQueue<T>::~TQueue()
{
  flush();
  m_pElHead = NULL;
  m_pElTail = NULL;
}

/******************************************************************************/
/*! Leeren der Queue
*/
/******************************************************************************/
template <class T> void TQueue<T>::flush()
{
  while(m_pElHead != NULL)
  {
    TBaseElement* next = m_pElHead->Next;
    delete m_pElHead;
    m_pElHead = next;
  }
  m_lSize = 0;
  m_pElTail = NULL;
}


/******************************************************************************/
/*! Ein Element in die Queue legen.
    \param data : Datenobjekt
*/
/******************************************************************************/
template <class T> void TQueue<T>::apendItem(const T& data)
{
  if (m_pElTail == NULL)
  {
    m_pElTail = new TBaseElement(data, NULL);
    m_pElHead = m_pElTail;
  }
  else
  {
    m_pElTail->Next = new TBaseElement(data, NULL);
    m_pElTail = m_pElTail->Next;
  }
  m_lSize++;
}


/******************************************************************************/
/*! Das oberste Element von der Queue holen
    \return Element. Ist die Queue leer wird ein leeres Objekt zurueckgegeben
*/
/******************************************************************************/
template <class T> T TQueue<T>::getItem()
{
  if (m_pElHead != NULL)
  {
    T retval = m_pElHead->Data;
    TBaseElement* del = m_pElHead;
    m_pElHead = m_pElHead->Next;
    delete del;
    m_lSize--;
    if (m_pElHead == NULL) m_pElTail = NULL;
    return retval;
  }
  else return m_EmptyObject;
}

/******************************************************************************/
/*! Die Groesse der Queue ermitteln
    \return Size of Queue
*/
/******************************************************************************/
template <class T> long TQueue<T>::getSize()
{
  return m_lSize;
}


/******************************************************************************/
/*! Ermitteln ob die Queue leer ist.
    \retval true : Queue ist leer
    \retval false : Queue hat noch Elemente
*/
/******************************************************************************/
template <class T> bool TQueue<T>::isEmpty()
{
  return (m_lSize == 0);
}

#endif
