/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SingleLinkedList                                              *
 * Package : SingleLinkedList                                              *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:33:11 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SingleLinkedList.hpp,v $
 *   Revision 1.2  2005/11/25 09:33:11  bhadorn
 *   Corrected error using iterator and implemented const_iterator
 *
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SingleLinkedListH
#define SingleLinkedListH

#include <zeusbase/System/Interfaces/IList.hpp>
#include <string.h>

BEGIN_NAMESPACE_Zeus

/*****************************************************************************/
/*! Template Klasse fuer Listen. Sie verwendet den abstrakten Datentypen
    IList. Die Listen sind als Single Linked List implementiert und
    verwalten einen Datentyp T.
*/
/*****************************************************************************/
template <class T> class TSingleLinkedList : public IList<T>
{
  public:
    //typename TSingleLinkedListElement;
    struct TSingleLinkedListElement;
    
    TSingleLinkedList();
    TSingleLinkedList(const T& emptyitem);
    TSingleLinkedList(const TSingleLinkedList<T>& list);
    virtual ~TSingleLinkedList();

    //Interface IList
    virtual long MQUALIFIER add(const T& Item);
    virtual void MQUALIFIER copyToList(IList<T>* list) const;
    virtual void MQUALIFIER clear();
    virtual unsigned long MQUALIFIER deleteItem(long Index);
    virtual unsigned long MQUALIFIER remove(const T& Item);
    virtual long MQUALIFIER getCount() const;
    virtual T& MQUALIFIER getItem(long index);
    virtual T MQUALIFIER getItemCopy(long index) const;
    virtual bool MQUALIFIER equalsItem(long Index, const T& Item) const;
    virtual bool MQUALIFIER equals(const IList<T>* list) const;
    virtual long MQUALIFIER indexOf(const T& Item) const;
    virtual long MQUALIFIER insert(long Index, const T& Item);
    virtual IListIterator<T>* MQUALIFIER getIterator() const;
    virtual const IListIterator<T>* MQUALIFIER getConstIterator() const;
    virtual void MQUALIFIER releaseIterator(const IListIterator<T>* xiterator) const;
    virtual bool MQUALIFIER isEmpty() const;

    //Interface IZUnknown
    virtual unsigned long MQUALIFIER askForInterface(long interface_id, IZUnknown** iface);
    virtual unsigned long MQUALIFIER addRef();
    virtual unsigned long MQUALIFIER release();
    
    //Operators
    T& operator[](long Index);
    TSingleLinkedList<T>& operator=(const TSingleLinkedList<T>& list);
    bool operator==(const TSingleLinkedList<T>& list);
    TSingleLinkedListElement* getHeadElement() const;
    T& getEmptyElement();

  protected:  
    
  private:
    /*********************************************************************/
    /*! Die Struktur TBaseElement ist die Speicherstruktur der SL-List
    */
    /*********************************************************************/
    struct TSingleLinkedListElement
    {
      /*****************************************************************/
      /*! Creates a container object
          \param data: Data Element
      */
      /*****************************************************************/
      TSingleLinkedListElement(const T& data)
      {
        tData = data;
        pNext = NULL;
      }

      /*****************************************************************/
      /*! Creates a container Object, witch points to a next element
      */
      /*****************************************************************/
      TSingleLinkedListElement(const T& data, TSingleLinkedListElement* nextptr)
      {
        tData = data;
        pNext = nextptr;
      }

      /// Objekt der Datenklasse T wird hier gespeichert
      T tData;
      /// Zeiger auf das naechste Objekt (Verkettung)
      TSingleLinkedListElement* pNext;
    };

    /*********************************************************************/
    /*! Iterator der Liste
    */
    /*********************************************************************/
    class TSingleListIterator : public IListIterator<T>
    {
      public:
        /*****************************************************************/
        /*! Creates the ListIterator
            \param head : Head element
            \retval empty : Empty element
        */
        /*****************************************************************/
        TSingleListIterator(TSingleLinkedList* parent)
        {
          m_pParent = parent;
          m_pNextIterator = NULL;
          m_pActElement = m_pParent->getHeadElement();
        }

        /*****************************************************************/
        /*! Creates the ListIterator
            \param head : Head element
            \retval empty : Empty element
        */
        /*****************************************************************/
        TSingleListIterator(const TSingleLinkedList* parent)
        {
          m_pParent = (TSingleLinkedList<T>*)parent;
          m_pNextIterator = NULL;
          m_pActElement = m_pParent->getHeadElement();
        }

        /*****************************************************************/
        /*! Destroys the ListIterator
        */
        /*****************************************************************/
        virtual ~TSingleListIterator(){}
        
        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual void MQUALIFIER reset() const
        {
          m_pActElement = m_pParent->getHeadElement();
        }

        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual T& MQUALIFIER getNextItem() const
        {
          if (this->hasNextItem())
          {
            TSingleLinkedListElement* temp = m_pActElement;
            m_pActElement = m_pActElement->pNext;
            return temp->tData;
          }
          else
          {
            return m_pParent->getEmptyElement();
          }
        }


        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual const T& MQUALIFIER getNextItemConst() const
        {
          return (const T&)this->getNextItem();
        }
        
        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual bool MQUALIFIER hasNextItem() const
        {
          return (m_pActElement != NULL);
        }

        /*****************************************************************/
        /*! Ermitteln des n�hsten Iterator Pointers
        */
        /*****************************************************************/
        TSingleListIterator* getNextIterator() const
        {
          return m_pNextIterator;
        }

        /*****************************************************************/
        /*! Setzen des n�hsten Iterator Pointers
        */
        /*****************************************************************/
        void setNextIterator(TSingleListIterator* pIt)
        {
          m_pNextIterator = pIt;
        }

        /*****************************************************************/
        /*! Ermitteln des aktuellen Elements
        */
        /*****************************************************************/
        TSingleLinkedListElement* getActElement() const
        {
          return m_pActElement;
        }

        /*****************************************************************/
        /*! Setzen des aktuellen Elements
        */
        /*****************************************************************/
        void setActElement(TSingleLinkedListElement* pActElement)
        {
          m_pActElement = pActElement;
        }

      private:
        ///Parent object
        mutable TSingleLinkedList* m_pParent;
        ///Next iterator
        mutable TSingleListIterator* m_pNextIterator;
        ///Actual element of the iterator
        mutable TSingleLinkedListElement* m_pActElement;
    };

    ///Reference counter
    unsigned long m_ulRefCounter;
    ///Zeiger auf den Kopf der Liste
    TSingleLinkedListElement* m_pHead;
    ///Zeiger auf das Ende der Liste
    TSingleLinkedListElement* m_pTail;
    ///Iterator Zeiger
    mutable TSingleListIterator* m_pIterator;
    ///Zaehler der Listenelemente
    long m_lCounter;

  protected:
    ///Leeres Objekt bei Fehler
    T m_tEmpty;
};

END_NAMESPACE_Zeus

USING_NAMESPACE_Zeus

/******************************************************************************/
/*! Konstruktor der SingleLinked List
*/
/******************************************************************************/
template <class T> TSingleLinkedList<T>::TSingleLinkedList()
{
  m_ulRefCounter = 1;
  m_lCounter = 0;
  m_pIterator = NULL;
  m_pTail = NULL;
  m_pHead = NULL;
  memset(&m_tEmpty, 0, sizeof(T));
}


/******************************************************************************/
/*! Bei diesem Konstruktor kann der Rckgabetyp gesetzt werden,
    wenn ein Indexfehler auftritt.
    \param emptyitem : Leeres Element welches beim Fehlerfall zurueckgegeben
                       wird.
*/
/******************************************************************************/
template <class T> TSingleLinkedList<T>::TSingleLinkedList(const T& emptyitem) 
{
  m_ulRefCounter = 1;
  m_lCounter = 0;
  m_pIterator = NULL;
  m_pTail = NULL;
  m_pHead = NULL;
  m_tEmpty = emptyitem;
}

/******************************************************************************/
/*! Kopierkonstruktor
*/
/******************************************************************************/
template <class T> TSingleLinkedList<T>::TSingleLinkedList(const TSingleLinkedList<T>& list)
{
  m_ulRefCounter = 1;
  *this = list;
}


/******************************************************************************/
/*! Destruktor der Liste. Die Liste wird geloescht
*/
/******************************************************************************/
template <class T> TSingleLinkedList<T>::~TSingleLinkedList()
{
  clear();
}

/******************************************************************************/
/*! Gets the Headelement of the list
*/
/******************************************************************************/
template <class T> typename TSingleLinkedList<T>::TSingleLinkedListElement* TSingleLinkedList<T>::getHeadElement() const
{
  return m_pHead;
}

/******************************************************************************/
/*! Gets the Empty data element
*/
/******************************************************************************/
template <class T> T& TSingleLinkedList<T>::getEmptyElement()
{
  return m_tEmpty;
}



/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TSingleLinkedList<T>::add(const T& Item)
{
  //Noch kein Item in der Liste
  if (m_pHead == NULL)
  {
    m_pHead = new TSingleLinkedList<T>::TSingleLinkedListElement(Item);
    m_pTail = m_pHead;
  }

  //Anfuegen des Items
  else
  {
    TSingleLinkedListElement* pNewItem = new TSingleLinkedList<T>::TSingleLinkedListElement(Item);
    m_pTail->pNext = pNewItem;
    m_pTail = pNewItem;  //Neuer Tail
  }
  m_lCounter++;
  return m_lCounter-1;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> void MQUALIFIER TSingleLinkedList<T>::copyToList(IList<T>* list) const
{
  if (list != NULL)
  {
    list->clear();
  
    //Copy of the data
    TSingleLinkedListElement* pItem = m_pHead;
    while (pItem != NULL)
    {
      list->add(pItem->tData);
      pItem = pItem->pNext;
    }
  }
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> void MQUALIFIER TSingleLinkedList<T>::clear()
{

  while (m_pHead != NULL)
  {
    TSingleLinkedListElement* pNextItem = m_pHead->pNext;
    delete m_pHead;
    m_pHead = pNextItem;
    m_lCounter--;
  }

  //Zuruecksetzen der Liste
  m_pHead = NULL;
  m_pTail = NULL;
  m_lCounter = 0;

  //Loeschen des Iteratorreferenzen
  TSingleListIterator* it = m_pIterator;
  while (it != NULL)
  {
    it->reset();
    it = it->getNextIterator();
  }
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> unsigned long MQUALIFIER TSingleLinkedList<T>::deleteItem(long Index)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pHead != NULL && Index >= 0 && Index < m_lCounter)
  {
    TSingleLinkedListElement* pItem = m_pHead;
    TSingleLinkedListElement* pPrevItem = NULL;
    long lCounter =0;

    while (pItem != NULL && lCounter < Index)
    {
      pPrevItem = pItem;
      pItem = pItem->pNext;
      lCounter++;
    }

    //------------------------------------------------------------------------
    //Fall 1: Item[0] (Head) soll geloescht werden.
    if(pItem == m_pHead)
    {
      //Anpassen der neuen Headers der Iteratoren
      TSingleListIterator* it = m_pIterator;
      while (it != NULL)
      {
        if (it->getActElement() == m_pHead)
        {
          it->setActElement(m_pHead->pNext);
        }
        it = it->getNextIterator();
      }

      pItem = m_pHead->pNext;
      delete m_pHead;
      m_pHead = pItem;
      m_lCounter--;

      //Falls Nur ein Element existierte
      if (m_pHead == NULL)
      {
        m_pTail = NULL;
      }
      ulRetval = RET_NOERROR;
    }

    //------------------------------------------------------------------------
    //Fall 2: Letzes Item wird geloescht
    else if (pItem == m_pTail)
    {
      //Anpassen des aktuellen Items der Iteratoren, wenn das Item das
      //geloeschte ist.
      TSingleListIterator* it = m_pIterator;
      while (it != NULL)
      {
        if (it->getActElement() == m_pTail)
        {
          it->setActElement(NULL);
        }
        it = it->getNextIterator();
      }

      //Loeschen des Letzten Items
      delete m_pTail;
      m_pTail = pPrevItem;
      pPrevItem->pNext = NULL;
      m_lCounter--;
      ulRetval = RET_NOERROR;
    }

    //------------------------------------------------------------------------
    //Fall 3 : Loeschen eines Item in der Mitte der Liste
    else if (pItem != NULL)
    {
      TSingleLinkedListElement* pNextItem = pItem->pNext;

      //Anpassen des aktuellen Items der Iteratoren, wenn das Item das
      //geloeschte ist.
      TSingleListIterator* it = m_pIterator;
      while (it != NULL)
      {
        if (it->getActElement() == pItem)
        {
          it->setActElement(pNextItem);
        }
        it = it->getNextIterator();
      }

      delete pItem;
      pPrevItem->pNext = pNextItem;
      m_lCounter--;
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TSingleLinkedList<T>::indexOf(const T& Item) const
{
  TSingleLinkedListElement* pItem = m_pHead;
  long lCounter = -1;
  while (pItem != NULL)
  {
    lCounter++;
    if (pItem->tData == Item)
    {
      return lCounter;
    }
    pItem = pItem->pNext;
  }

  return -1;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TSingleLinkedList<T>::insert(long Index, const T& Item)
{
  if (Index >= 0)
  {
    long lCounter = 0;
    TSingleLinkedListElement* pItem = m_pHead;  //Vor diesem Item Einfuegen
    TSingleLinkedListElement* pPrevItem = NULL; //Nach diesem Item Einfuegen

    while(pItem != NULL && lCounter < Index)
    {
      pPrevItem = pItem;
      pItem = pItem->pNext;
      lCounter++;
    }

    //Fall 1: Ein Element in der Mitte der Liste einfuegen
    if (pItem != NULL)
    {
      //Als HeadElement einfuegen
      if (pPrevItem == NULL)
      {
        m_pHead = new TSingleLinkedList<T>::TSingleLinkedListElement(Item, pItem);
      }
      else
      {
        pPrevItem->pNext = new TSingleLinkedList<T>::TSingleLinkedListElement(Item, pItem);
      }
      m_lCounter++;
      return lCounter;
    }

    //Am Ende der Liste Anfuegen
    else if (pItem == NULL && Index == lCounter)
    {
      if (pPrevItem == NULL)
      {
        m_pHead = new TSingleLinkedList<T>::TSingleLinkedListElement(Item);
        m_pTail = m_pHead;
      }

      else
      {
        m_pTail = new TSingleLinkedList<T>::TSingleLinkedListElement(Item);
        pPrevItem->pNext = m_pTail;
      }
      m_lCounter++;
      return lCounter;
    }
    else
    {
      return -1;
    }
  }
  else
  {
    return -1;
  }
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> unsigned long MQUALIFIER TSingleLinkedList<T>::remove(const T& Item)
{
  return deleteItem(indexOf(Item));
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> long MQUALIFIER TSingleLinkedList<T>::getCount() const
{
  return  m_lCounter;
}


/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> T& MQUALIFIER TSingleLinkedList<T>::getItem(long Index)
{
  long lCounter = 0;
  TSingleLinkedListElement* pItem = m_pHead;  //Start

  while(pItem != NULL && lCounter < Index)
  {
    pItem = pItem->pNext;
    lCounter++;
  }

  if (pItem != NULL && Index >= 0)
  {
    return pItem->tData;
  }
  else
  {
    return m_tEmpty;
  }
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> T MQUALIFIER TSingleLinkedList<T>::getItemCopy(long Index) const
{
  long lCounter = 0;
  TSingleLinkedListElement* pItem = m_pHead;  //Start

  while(pItem != NULL && lCounter < Index)
  {
    pItem = pItem->pNext;
    lCounter++;
  }

  if (pItem != NULL && Index >= 0)
  {
    return pItem->tData;
  }
  else
  {
    return m_tEmpty;
  }
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> bool MQUALIFIER TSingleLinkedList<T>::equalsItem(long Index, const T& Item) const
{
  return (getItemCopy(Index) == Item);
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> bool MQUALIFIER TSingleLinkedList<T>::equals(const IList<T>* list) const
{
  bool bRetval = false;
  
  if (list != NULL)
  {
    bRetval = (list->getCount() == m_lCounter);
    long lCounter = 0;
    
    if (bRetval)
    {
      TSingleLinkedListElement* pItem = m_pHead;  //Start
      while(pItem != NULL && bRetval)
      {
        bRetval &= list->equalsItem(lCounter,pItem->tData); 
        pItem = pItem->pNext;
        lCounter++;
      }
    }
  }
  return bRetval; 
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> IListIterator<T>* MQUALIFIER TSingleLinkedList<T>::getIterator() const
{
  TSingleListIterator* iterator = new TSingleListIterator(this);
  iterator->setNextIterator(m_pIterator);
  m_pIterator = iterator;
  return m_pIterator;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> const IListIterator<T>* MQUALIFIER TSingleLinkedList<T>::getConstIterator() const
{
  TSingleListIterator* iterator = new TSingleListIterator(this);
  iterator->setNextIterator(m_pIterator);
  m_pIterator = iterator;
  return m_pIterator;
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> void MQUALIFIER TSingleLinkedList<T>::releaseIterator(const IListIterator<T>* xiterator) const
{
  TSingleListIterator* it = m_pIterator;
  TSingleListIterator* it_prev = NULL;
  while (it != NULL)
  {
    if (it == xiterator)
    {
      //Erster Iterator loeschen
      if (it_prev == NULL)
      {
        m_pIterator = it->getNextIterator();
        delete it;
      }
      else
      {
        it_prev->setNextIterator(it->getNextIterator());
        delete it;
      }
      break;
    }
    it_prev = it;
    it = it->getNextIterator();
  }
}

/******************************************************************************/
/*!  Siehe ADT IList
*/
/******************************************************************************/
template <class T> bool MQUALIFIER TSingleLinkedList<T>::isEmpty() const
{
  return (m_lCounter == 0);
}


/******************************************************************************/
/*!
    Operator zur fr das einfache Zugreifen auf den Elementinhalt
    \param Index : Index auf ein Element der Liste
    \return gibt den Dateninhalt eines Elements der Liste zurck
*/
/******************************************************************************/
template <class T> T& TSingleLinkedList<T>::operator[](long Index)
{
  return getItem(Index);
}


/******************************************************************************/
/*! Operator zum kopieren einer Liste
    \param list : Ausgangsliste
    \return neu kopierte Liste
*/
/******************************************************************************/
template <class T> TSingleLinkedList<T>& TSingleLinkedList<T>::operator=(const TSingleLinkedList<T>& list)
{
  list.copyToList(this);
  return *this;
}

/******************************************************************************/
/*! Operator to check equality of two lists
    \param list : list to check
    \retval true : Equal
    \retval false : not equal
*/
/******************************************************************************/
template <class T> bool  TSingleLinkedList<T>::operator==(const TSingleLinkedList<T>& list)
{
  return this->equals(&list);
}


/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
template <class T> unsigned long MQUALIFIER TSingleLinkedList<T>::askForInterface(long interface_id, IZUnknown** iface)
{
  unsigned long ulRetval = RET_NOERROR;
  if (iface == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    switch(interface_id)
    {
      default:
        ulRetval = RET_UNKNOWN_INTERFACE;
    }
  }

  return ulRetval;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
template <class T> unsigned long MQUALIFIER TSingleLinkedList<T>::addRef()
{
  ++m_ulRefCounter;
  return RET_NOERROR;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
template <class T> unsigned long MQUALIFIER TSingleLinkedList<T>::release()
{
  --m_ulRefCounter;
  if (m_ulRefCounter <= 0)
  {
    delete this;
  }
  return RET_NOERROR;
}

#endif
