/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus                                                          *
 * Module  : SingletonManager                                              *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SingletonManager.cpp,v $
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/SingletonManager.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/ZObjectFactory.h>
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/Remote/Naming.h>

USING_NAMESPACE_Zeus

TSingletonManager TSingletonManager::m_SingManager;

/**************************************************************************/
/*! Creates the SingletonManager
*/
/**************************************************************************/
TSingletonManager::TSingletonManager()
{}

/**************************************************************************/
/*! Destroys the SingletonManager
*/
/**************************************************************************/
TSingletonManager::~TSingletonManager()
{
  removeAll();
}

/**************************************************************************/
/*! Returns the Singleton Object
*/
/**************************************************************************/
TSingletonManager& TSingletonManager::getInstance()
{
  return m_SingManager;
}


void TSingletonManager::registerCommonSingletons()
{
  TString strName = L"IXObjectFactory";
  this->addSingleton(&strName, (IXObjectFactory*)&TXObjectFactory::getInstance());
  strName = L"ILibraryManager";
  this->addSingleton(&strName, (ILibraryManager*)&TLibraryManager::getInstance());
  strName = L"IZObjectFactory";
  this->addSingleton(&strName, (IZObjectFactory*)&TZObjectFactory::getInstance());
  strName = L"INaming";
  this->addSingleton(&strName, (INaming*)&Naming);
}

/**************************************************************************/
/*! Removes all Singletons
*/
/**************************************************************************/
void TSingletonManager::removeAll()
{
  IListIterator<IZUnknown*>* pIterator = m_mapSings.getIterator();
  while(pIterator->hasNextItem())
  {
    IZUnknown* pSing = pIterator->getNextItem();
    pSing->release();
  }
  m_mapSings.releaseIterator(pIterator);
  m_mapSings.clear();
}

/**************************************************************************/
/*! See ISingletonManager
*/
/**************************************************************************/
unsigned long MQUALIFIER TSingletonManager::addSingleton(IString* name, IZUnknown* iface)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;

  if (name == NULL || iface == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (!m_mapSings.hasItem(name))
  {
    iface->addRef();
    m_mapSings.setItem(name, iface);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See ISingletonManager
*/
/**************************************************************************/
unsigned long MQUALIFIER TSingletonManager::getSingleton(IString* name, IZUnknown** iface)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || iface == NULL)
  {
    ulRetval  = RET_INVALID_PARAMETER;
  }
  else if (m_mapSings.hasItem(name))
  {
    *iface = m_mapSings.getItem(name);
    (*iface)->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See ISingletonManager
*/
/**************************************************************************/
unsigned long MQUALIFIER TSingletonManager::removeSingleton(IString* name)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL)
  {
    ulRetval  = RET_INVALID_PARAMETER;
  }
  else if (m_mapSings.hasItem(name))
  {
    IZUnknown* pSing = m_mapSings.getItem(name);
    m_mapSings.deleteItem(name);
   
    pSing->release();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TSingletonManager);
MEMORY_MANAGER_IMPL_END;

