/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus                                                          *
 * Module  : SingletonManager                                              *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SingletonManager.h,v $
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SingletonManagerH
#define SingletonManagerH


#include <zeusbase/System/Interfaces/ISingletonManager.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/StringMap.hpp>


BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This class is responsible to manage all singletons of the framework
*/
/***************************************************************************/
class TSingletonManager : public TZObject, public ISingletonManager
{
  public:
    static TSingletonManager& getInstance();
    void removeAll();
    void registerCommonSingletons();

    //Methods of ISingletonManager
    virtual unsigned long MQUALIFIER addSingleton(IString* name, IZUnknown* iface);
    virtual unsigned long MQUALIFIER getSingleton(IString* name, IZUnknown** iface);
    virtual unsigned long MQUALIFIER removeSingleton(IString* name);

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:
    TSingletonManager();
    virtual ~TSingletonManager();

    ///Singleton Object
    static TSingletonManager m_SingManager;

  private:
    ///Map of all singletons
    TStringMap<IZUnknown*> m_mapSings;
};


END_NAMESPACE_Zeus

#endif
