/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Socket                                                        *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:54:01 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Socket.h,v $
 *   Revision 1.3  2005/11/25 09:54:01  bhadorn
 *   Implemented read and write methods for array and long values.
 *
 *   Revision 1.2  2005/11/16 11:32:21  bhadorn
 *   Improved error evaluation when reading fails
 *
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef SocketH
#define SocketH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>
#include <zeusbase/System/Interfaces/IInputStream.hpp>
#include <zeusbase/System/Interfaces/IOutputStream.hpp>

#ifdef LINUX
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
#else
  #include <Winsock.h>
#endif  


BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! The socket class implements all platform dependend calls to the
    network interface
*/
/**************************************************************************/
class TSocket : public TZObject, public IInputStream, public IOutputStream
{
  public:
    TSocket();
    TSocket(TString addr, unsigned long port);
    TSocket(int fd, struct sockaddr_in& addr);

    void setBlockable(bool mode);
    
    virtual unsigned long connect();
    virtual unsigned long disconnect();
    TString getAddress();
    unsigned long getPort();
    bool isConnected();

    //Methods of IInputStream
    virtual bool MQUALIFIER available() const{ return m_bConnected; }
    virtual bool MQUALIFIER isEmpty() const { return false; }
    virtual unsigned long MQUALIFIER read(char* buffer, long buffer_size, long* valid_size) const;
    virtual double MQUALIFIER readDouble() const { return RET_METHOD_NOT_IMPL; }
    virtual long MQUALIFIER readLong() const;
    virtual char MQUALIFIER readByte() const { return RET_METHOD_NOT_IMPL; }
    virtual bool MQUALIFIER readBool() const { return RET_METHOD_NOT_IMPL; }
    virtual unsigned long MQUALIFIER readArray(IByteArray* data) const;
    virtual unsigned long MQUALIFIER readString(IString* /*data*/) const { return RET_METHOD_NOT_IMPL; }
    virtual void MQUALIFIER close() { disconnect(); }

    //Methods of IOutputStream
    virtual unsigned long MQUALIFIER write(char* buffer, long buffer_size);
    virtual unsigned long MQUALIFIER writeDouble(double /*data*/){ return RET_METHOD_NOT_IMPL; }
    virtual unsigned long MQUALIFIER writeLong(long data);
    virtual unsigned long MQUALIFIER writeByte(char /*data*/) { return RET_METHOD_NOT_IMPL; }
    virtual unsigned long MQUALIFIER writeBool(bool /*data*/){ return RET_METHOD_NOT_IMPL; }
    virtual unsigned long MQUALIFIER writeArray(const IByteArray* data);
    virtual unsigned long MQUALIFIER writeString(const IString* data);
    virtual void MQUALIFIER flush() {}

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TSocket();

    ///Address object
    struct sockaddr_in m_Address;
    ///Filedescripitor for socket
    int m_iSocketFD;
    ///Connected flag
    bool m_bConnected;
    ///Flg if blockable
    bool m_bBlockable;

  private:
    ///static ref counter
    static unsigned long m_ulObjectCount;

    void initSocket();
};

END_NAMESPACE_Zeus

#endif

