#ifndef StackH
#define StackH


#include <stddef.h>
#include <string.h>

/******************************************************************************/
/*! Implementation des Abstrakten DatenTyps Pushdown-Stack
*/
/******************************************************************************/
template <class T> class TStack
{
  public:
    ///Konstruktor
    TStack();
    TStack(T& empty_object);
    ///Destruktor
    ~TStack();

    ///Leeren des Stacks
    void flush();
    ///Ein Objekt auf den Stack legen
    void push(const T& data);
    ///Das oberste Objekt entfernen
    T pop();
    ///Das oberste Element nur lesen
    T& peek();
    ///Groesse des Stacks ermitteln
    long getSize();
    ///Ermitteln ob Stack leer ist
    bool isEmpty();


  private:
    ///Containerklasse der Daten, die verlinked werden
    struct TBaseElement
    {
      TBaseElement(T data, TBaseElement* next)
      {
        Data = data;
        Next = next;
      }

      T Data;
      TBaseElement* Next;
    };

    ///Stack-Top
    TBaseElement* m_pElHead;
    ///Groesse des Stacks
    long m_lSize;
    ///Das Leere Objekt
    T m_EmptyObject;

};


/******************************************************************************/
/*! Erstellen des Stacks
*/
/******************************************************************************/
template <class T> TStack<T>::TStack()
{
  m_pElHead = NULL;
  m_lSize = 0;
  memset(&m_EmptyObject,0, sizeof(T));
}

/******************************************************************************/
/*! Erstellen des Stacks
*/
/******************************************************************************/
template <class T> TStack<T>::TStack(T& empty_object)
{
  m_pElHead = NULL;
  m_lSize = 0;
  m_EmptyObject = empty_object;
}


/******************************************************************************/
/*! Zerstoeren des Stacks
*/
/******************************************************************************/
template <class T> TStack<T>::~TStack()
{
  flush();
  m_pElHead = NULL;
}

/******************************************************************************/
/*! Leeren des Stacks
*/
/******************************************************************************/
template <class T> void TStack<T>::flush()
{
  while(m_pElHead != NULL)
  {
    TBaseElement* next = m_pElHead->Next;
    delete m_pElHead;
    m_pElHead = next;
  }
  m_lSize = 0;
}


/******************************************************************************/
/*! Ein Element auf den Stack legen.
    \param data : Datenobjekt
*/
/******************************************************************************/
template <class T> void TStack<T>::push(const T& data)
{
  m_pElHead = new TBaseElement(data, m_pElHead);
  m_lSize++;
}


/******************************************************************************/
/*! Das oberste Element vom Stack holen
    \return Element. Ist der Stack leer wird ein leeres Objekt zurueckgegeben
*/
/******************************************************************************/
template <class T> T TStack<T>::pop()
{
  if (m_pElHead != NULL)
  {
    T retval = m_pElHead->Data;
    TBaseElement* del = m_pElHead;
    m_pElHead = m_pElHead->Next;
    delete del;
    m_lSize--;
    return retval;
  }
  else return m_EmptyObject;
}


/******************************************************************************/
/*! Das oberste Element vom Stack lesen. Das Element wird nicht entfernt
    \return Element. Ist der Stack leer wird ein leeres Objekt zurueckgegeben
*/
/******************************************************************************/
template <class T>  T& TStack<T>::peek()
{
  if (m_pElHead != NULL)
  {
    return m_pElHead->Data;
  }
  else return m_EmptyObject;
}

/******************************************************************************/
/*! Die Groesse des Stacks ermitteln
    \return Size of Stack
*/
/******************************************************************************/
template <class T> long TStack<T>::getSize()
{
  return m_lSize;
}


/******************************************************************************/
/*! Ermitteln ob der Stack leer ist.
    \retval true : Stack ist leer
    \retval false : Stack hat noch Elemente
*/
/******************************************************************************/
template <class T> bool TStack<T>::isEmpty()
{
  return (m_lSize == 0);
}

#endif
