/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StdLogger                                                     *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:07 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: StdLogger.cpp,v $
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <zeusbase/System/StdLogger.h>
#include <zeusbase/System/PropertyFile.h>
#include <zeusbase/System/Time.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a standard logger
*/
/***************************************************************************/
TStdLogger::TStdLogger(const TString& name /*= L""*/) : TZObject()
{
  m_strName = name;
  if (m_strName == L"")
  {
    m_strName = L"Root";
  }
  m_bDebug = true;
  m_bInfo = true;
  m_bError = true;
  m_bFatal = true;
  m_bWarn = true;
}


/***************************************************************************/
/*! Destroys the standard logger
*/
/***************************************************************************/
TStdLogger::~TStdLogger()
{
}

/***************************************************************************/
/*! Writes the logging output
*/
/***************************************************************************/
void TStdLogger::write(TString& txt)
{
  printf("%11.4f--%s", TTime::getTimeStamp(), txt.c_str());
}


/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::getLogger(IString* name, ILogger** logger)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || logger == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *logger = new TStdLogger(name);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::printDebug(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strOut = m_strName + TString(L"----Debug:");
    strOut.concatStr(message);
    strOut.concat(L"\n");
    write(strOut);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::printError(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strOut = m_strName + TString(L"----Error:");
    strOut.concatStr(message);
    strOut.concat(L"\n");
    write(strOut);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::printFatal(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strOut = m_strName + TString(L"----Fatal:");
    strOut.concatStr(message);
    strOut.concat(L"\n");
    write(strOut);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::printInfo(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strOut = m_strName + TString(L"-----Info:");
    strOut.concatStr(message);
    strOut.concat(L"\n");
    write(strOut);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::printWarn(IString* message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strOut = m_strName + TString(L"--Warning:");
    strOut.concatStr(message);
    strOut.concat(L"\n");
    write(strOut);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
bool MQUALIFIER TStdLogger::isDebugEnabled()
{ return m_bDebug; }

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
bool MQUALIFIER TStdLogger::isErrorEnabled()
{ return m_bError; }

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
bool MQUALIFIER TStdLogger::isFatalEnabled()
{ return m_bFatal; }

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
bool MQUALIFIER TStdLogger::isInfoEnabled()
{ return m_bInfo; }

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
bool MQUALIFIER TStdLogger::isWarnEnabled()
{ return m_bWarn; }

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::loadConfiguration(IString* file)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (file == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TPropertyFile* pPropFile = new TPropertyFile(file);
    pPropFile->load();
    ulRetval = loadProperties(pPropFile);
    pPropFile->release();
  }
  return ulRetval;
}

/***************************************************************************/
/*! see ILogger
*/
/***************************************************************************/
unsigned long MQUALIFIER TStdLogger::loadProperties(IPropertyAccess* prop)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (prop == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strValue;
    TString strPropName = L"zeus.Logger.Debug";
    prop->getPropertyValue(&strPropName, &strValue);
    m_bDebug = (strValue.toLong(1) == 1);

    strPropName = L"zeus.Logger.Info";
    strValue = L"";
    prop->getPropertyValue(&strPropName, &strValue);
    m_bInfo = (strValue.toLong(1) == 1);

    strPropName = L"zeus.Logger.Warn";
    strValue = L"";
    prop->getPropertyValue(&strPropName, &strValue);
    m_bWarn = (strValue.toLong(1) == 1);

    strPropName = L"zeus.Logger.Error";
    strValue = L"";
    prop->getPropertyValue(&strPropName, &strValue);
    m_bError = (strValue.toLong(1) == 1);

    strPropName = L"zeus.Logger.Fatal";
    strValue = L"";
    prop->getPropertyValue(&strPropName, &strValue);
    m_bFatal = (strValue.toLong(1) == 1);

    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TStdLogger);
  INTERFACE_CAST(ILogger, INTERFACE_ILogger);
MEMORY_MANAGER_IMPL_END;
