/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Code Module                                                   *
 * Package : Code Package                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:56:34 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: String.cpp,v $
 *   Revision 1.2  2005/11/25 09:56:34  bhadorn
 *   Implemented standard and borland bindings for strings
 *
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <string.h>
#include <wchar.h>
#include <math.h>

#include <zeusbase/System/String.h>

USING_NAMESPACE_Zeus

/**************************************************************************/
/*! Konstruktor
*/
/**************************************************************************/
TString::TString()
{
  m_ulRefCounter = 1;
  m_pCBuffer = NULL;
  m_pWBuffer = new wchar_t[1];
  if (m_pWBuffer != NULL)
  {
    m_pWBuffer[0] = 0x0000;
  }
}


/**************************************************************************/
/*! Konstruktor.
    \param data : wchar_t Array welcher in den String aufgenommen wird. Der
                  Array wird kopiert.
*/
/**************************************************************************/
TString::TString(const wchar_t* data)
{
  m_ulRefCounter = 1;
  m_pWBuffer = new wchar_t[1];
  if (m_pWBuffer != NULL)
  {
    m_pWBuffer[0] = 0x0000;
  }
  m_pCBuffer = NULL;
  assign(data);
}


/**************************************************************************/
/*! Konstruktor.
    \param data : wchar_t Array welcher in den String aufgenommen wird. Der
                  Array wird kopiert.
*/
/**************************************************************************/
TString::TString(const char* data)
{
  m_ulRefCounter = 1;
  m_pWBuffer = NULL;
  m_pCBuffer = NULL;
  m_pWBuffer = convToWStr(data);
}


/**************************************************************************/
/*! Konstruktor.
    \param string : String Objekt. Die Daten werden in das neue Objekt
                    kopiert.
*/
/**************************************************************************/
TString::TString(const TString& string)
{
  m_ulRefCounter = 1;
  m_pCBuffer = NULL;
  m_pWBuffer = NULL;
  assign(string.m_pWBuffer);
}

/**************************************************************************/
/*! Konstruktor.
    \param string : String Objekt. Die Daten werden in das neue Objekt
                    kopiert.
*/
/**************************************************************************/
TString::TString(const IString* string)
{
  m_ulRefCounter = 1;
  m_pCBuffer = NULL;
  m_pWBuffer = NULL;
  if (string != NULL)
  {
    assign(string->c_bstr());
  }
  else
  {
    assign(L"");
  }
}

/**************************************************************************/
/*! Konstruktor.
    \param value : String Objekt. Die Daten werden in das neue Objekt
                    kopiert.
*/
/**************************************************************************/
TString::TString(long value)
{
  m_ulRefCounter = 1;
  m_pCBuffer = NULL;
  m_pWBuffer = NULL;
  TString strTemp;
  strTemp = value;
  assign(strTemp.m_pWBuffer);
}

/**************************************************************************/
/*! Konstruktor.
    \param value : String Objekt. Die Daten werden in das neue Objekt
                    kopiert.
*/
/**************************************************************************/
TString::TString(double value)
{
  m_ulRefCounter = 1;
  m_pCBuffer = NULL;
  m_pWBuffer = NULL;
  TString str = convertDouble(value);
  assign(str.m_pWBuffer);
}

/**************************************************************************/
/*! Destruktor. Der wchar_t-Array wird zerstoert.
*/
/**************************************************************************/
TString::~TString()
{
  clearBuffer();
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
wchar_t* MQUALIFIER TString::c_bstr() const
{
  return m_pWBuffer;
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
char* MQUALIFIER TString::c_str() const 
{
  return this->convToCStr(m_pWBuffer);
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
double MQUALIFIER TString::toDouble(double predef /*= 0.0*/) const
{
  double dRetval = predef;
  if ((m_pWBuffer != NULL) &&
      (m_pWBuffer[0] != 0x0000))
  {
    wchar_t* endptr = NULL;
    double result= wcstod(m_pWBuffer,&endptr);
    if (endptr != NULL && *endptr != 0x0000)
    {
      dRetval = predef;
    }
    else
    {
      dRetval = result;
    }
  }
  return dRetval;
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
long MQUALIFIER TString::toLong(long predef /*= 0*/) const
{
  long lRetval = predef;
  if ((m_pWBuffer != NULL) &&
      (m_pWBuffer[0] != 0x0000))
  {
    wchar_t* endptr = NULL;
    long result= wcstol(m_pWBuffer, &endptr, 10);
    if ((endptr != NULL && *endptr != 0x0000))
    {
      lRetval = predef;
    }
    else
    {
      lRetval = result;
    }
  }
  return lRetval;
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
bool MQUALIFIER TString::equals(const wchar_t* inbuffer) const
{
  bool bRetval = false;
  if (m_pWBuffer == inbuffer)
  {
    bRetval = true;
  }
  else if (inbuffer == NULL)
  {
    if (this->getSize()==0)
    {
      bRetval = true;
    }
    else
    {
      bRetval = false;
    }
  }
  else if (m_pWBuffer != NULL && wcscmp(m_pWBuffer, inbuffer)==0)
  {
    bRetval = true;
  }
  else
  {
    bRetval = false;
  }
  return bRetval;
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
bool MQUALIFIER TString::equalsStr(const IString* instr)const
{
  bool bRetval = false;
  if (instr != NULL)
  {
    bRetval = this->equals(instr->c_bstr());
  }
  else
  {
    bRetval = false;
  }

  return bRetval;
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
void MQUALIFIER TString::concat(const wchar_t* inbuffer)
{
  if (inbuffer != NULL)
  {
    //Erstellen des neuen Buffers
    size_t size1 = 0;
    size_t size2 = 0;
    if (m_pWBuffer != NULL)
    {
      size1 = wcslen(m_pWBuffer);
    }
    size2 = wcslen(inbuffer)+1;
    wchar_t* pNewbuffer = new wchar_t[size1+size2];
    if (pNewbuffer != NULL)
    {
      if (m_pWBuffer != NULL)
      {
        wcscpy(pNewbuffer,m_pWBuffer);
      }
      else
      {
        wcscpy(pNewbuffer,L"");
      }

      wcscat(pNewbuffer,inbuffer);
      this->assign(pNewbuffer);
      delete [] pNewbuffer;
    }
  }
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
void MQUALIFIER TString::concatStr(const IString* instr)
{
  if (instr != NULL)
  {
    concat(instr->c_bstr());
  }
  else
  {
    ///
  }
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
void MQUALIFIER TString::assign(const wchar_t* inbuffer)
{
  this->clearBuffer();
  if (inbuffer != NULL)
  {
    size_t size = wcslen(inbuffer)+1;
    if (size > 0)
    {
      m_pWBuffer = new wchar_t[size];
      if (m_pWBuffer != NULL)
      {
        wcscpy(m_pWBuffer,inbuffer);
      }
    }
    else
    {
      m_pWBuffer = new wchar_t[1];
      if (m_pWBuffer != NULL)
      {
        m_pWBuffer[0] = 0x0000;
      }
    }
  }
  else
  {
    m_pWBuffer = new wchar_t[1];
    if (m_pWBuffer != NULL)
    {
      m_pWBuffer[0] = 0x0000;
    }
  }
}


/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
void MQUALIFIER TString::assignStr(const IString* instr)
{
  if (instr != NULL)
  {
    this->assign(instr->c_bstr());
  }
  else
  {
    this->assign(NULL);
  }
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
long MQUALIFIER TString::getSize()const
{
  long lRetval = 0;
  if (m_pWBuffer != NULL)
  {
    lRetval = wcslen(m_pWBuffer);
  }
  else
  {
    lRetval = 0;
  }

  return lRetval;
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
wchar_t MQUALIFIER TString::getChar(long index)const
{
  wchar_t wcRetval = 0x0000;
  long lCounter = getSize();
  if (index < lCounter && index >= 0 && m_pWBuffer != NULL)
  {
    wcRetval = m_pWBuffer[index];
  }
  else
  {
    wcRetval = 0x0000;
  }

  return wcRetval;
}

/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
long MQUALIFIER TString::getPos(const IString* substring)const
{
  if (substring != NULL && m_pWBuffer != NULL)
  {
    int i;
    long lBaseCount = getSize();
    long lSubCount = substring->getSize();
    wchar_t* pSubBuffer = substring->c_bstr();

    if (lSubCount == 0) return -1;


    //--------------------------------------------------------------------
    //Preprozessing nach Knuth-Morris-Pratt Verfahren
    long* pJumpArray = new long[lSubCount];
    long k = 0;
    pJumpArray[0] = 1; //Initialisieren des Grundspringens

    for(i = 0; i < lSubCount-1; i++)
    {
      if (pSubBuffer[i] != pSubBuffer[k])
      {
        pJumpArray[i+1] = i+1;    //Setzen der Sprungweite
        k = 0;                    //Zuruecksetzen der Pruefposition
      }

      //Anpassen der Sprungweite, weil Pruefposition gleich dem Char. ist
      else
      {
        if (i >0) k++;
        pJumpArray[i+1] = pJumpArray[i];
      }
    }


    //--------------------------------------------------------------------
    //Nach SubString suchen
    for (i = 0; i <= lBaseCount-lSubCount;)
    {
      long lPos = i;
      while(lPos-i < lSubCount && pSubBuffer[lPos-i] == m_pWBuffer[lPos])
      {
        lPos++;
        if (pSubBuffer[lPos-1-i] == m_pWBuffer[lPos-1] && lPos-i == lSubCount)
        {
          delete [] pJumpArray;
          return i;
        }
      }
      i += pJumpArray[lPos-i];  //Optimierung: Sprung nach vorne
    }

    delete [] pJumpArray;
  }
  return -1;
}


/**************************************************************************/
/*! Siehe IString
*/
/**************************************************************************/
bool MQUALIFIER TString::isEmpty()const
{
  return (getSize() == 0);
}

/**************************************************************************/
/*! Ermitteln des Substring
    \param start : Start index
    \param end : end index
    \return substring
*/
/**************************************************************************/
TString TString::getSubString(long start, long end) const
{
  TString strRetval;
  if (start <= end && m_pWBuffer != NULL)
  {
    long lCount = end - start + 1;
    wchar_t* wBuffer = new wchar_t[lCount+1];
    if (wBuffer != NULL)
    {
      ::wmemcpy(wBuffer, m_pWBuffer+start, lCount);
      wBuffer[lCount] = L'\0';
      strRetval = wBuffer;
      delete [] wBuffer;
    }
  }
  return strRetval;
}


/**************************************************************************/
/*! Kopieroperator. Der StString wird in das Objekt kopiert.
    \param instr : zu kopierendes Stringobjekt
*/
/**************************************************************************/
TString& TString::operator =(const TString& instr)
{
  assign(instr.m_pWBuffer);
  return *this;
}


/**************************************************************************/
/*! Kopieroperator. Der char-Array wird in das Objekt kopiert.
    \param data : zu kopierendes WideChar Array
    \return Zusammengesetzter String
*/
/**************************************************************************/
TString& TString::operator =(const wchar_t* data)
{
  assign(data);
  return *this;
}

/**************************************************************************/
/*! Zuweisungsoperator: Der einen long Wert zuweist.
    \param data : Zuweisungsparameter
    \return Zusammengesetzter String
*/
/**************************************************************************/
TString& TString::operator =(const long data)
{
  int ndig = 15;
  char* pNewBuffer = new char[100];
  if (pNewBuffer != NULL)
  {
    pNewBuffer = gcvt(data, ndig, pNewBuffer);
    TString str(pNewBuffer);
    assign(str.c_bstr());
    delete [] pNewBuffer;
  }
  return *this;
}

/**************************************************************************/
/*! Zuweisungsoperator: Der einen double Wert zuweist.
    \param data : Zuweisungsparameter
    \return Zusammengesetzter String
*/
/**************************************************************************/
TString& TString::operator =(const double data)
{
  TString strValue = convertDouble(data);
  this->assign(strValue.m_pWBuffer);
  return *this;
}


/**************************************************************************/
/*! Kopieroperator. Der char-Array wird in das Objekt kopiert.
    \param data : zu kopierendes Stringobjekt
    \return Zusammengesetzter String
*/
/**************************************************************************/
TString& TString::operator =(const char* data)
{
  m_pWBuffer = convToWStr(data);
  return *this;
}

/**************************************************************************/
/*! Vergleichsoperator. Der wchar_t-Array wird mit dem des Objekts verglichen.
    \param instr : zu vergleichendes Stringobjekt
*/
/**************************************************************************/
bool TString::operator ==(const TString& instr)const
{
  return equals(instr.m_pWBuffer);
}


/**************************************************************************/
/*! Vergleichsoperator. Der wchar_t-Array wird mit dem des Objekts verglichen.
    \param inbuffer : zu vergleichendes Stringobjekt
*/
/**************************************************************************/
bool TString::operator ==(const wchar_t* inbuffer) const
{
  return equals(inbuffer);
}

/**************************************************************************/
/*! Vergleichsoperator. Der char-Array wird mit dem des Objekts verglichen.
    \param inbuffer : zu vergleichendes Stringobjekt
*/
/**************************************************************************/
bool TString::operator ==(const char* inbuffer)const
{
  TString str(inbuffer);
  return equals(str.m_pWBuffer);
}

/**************************************************************************/
/*! Vergleichsoperator: Der einen double Wert mit dem String vergleicht
    \param data : Zuweisungsparameter
*/
/**************************************************************************/
bool TString::operator ==(const double data) const
{
  bool bRetval = false;

  TString strFormat = convertDouble(data);
  bRetval = equals(strFormat.m_pWBuffer);
  return bRetval;
}

/**************************************************************************/
/*! Vergleichsoperator: Der einen long Wert mit dem String vergleicht
    \param data : Zuweisungsparameter
*/
/**************************************************************************/
bool TString::operator ==(const long data)const
{
  bool retval = false;
  int ndig = 15;
  char* pNewBuffer = new char[100];
  if (pNewBuffer != NULL)
  {
    pNewBuffer = gcvt(data, ndig, pNewBuffer);
    TString str(pNewBuffer);
    retval = equals(str.m_pWBuffer);
    delete [] pNewBuffer;
  }
  return retval;
}

/**************************************************************************/
/*! Hinzufuegen eines Strings
    \param instr : Stringobjekt
*/
/**************************************************************************/
TString TString::operator+ (const TString& instr)
{
  TString str(this->m_pWBuffer);
  str.concat(instr.m_pWBuffer);
  return str;
}


/**************************************************************************/
/*! Hinzufuegen eines wchar_t-Arrays
    \param inbuffer : wchar_t Buffer welcher hinzugefuegt wird
*/
/**************************************************************************/
TString TString::operator+ (const wchar_t* inbuffer)
{
  TString str(this->m_pWBuffer);
  str.concat(inbuffer);
  return str;
}

/**************************************************************************/
/*! Hinzufuegen eines char-Arrays
    \param inbuffer : char Buffer welcher hinzugefuegt wird
*/
/**************************************************************************/
TString TString::operator+ (const char* inbuffer)
{
  TString str(this->m_pWBuffer);
  TString strC(inbuffer);
  str.concat(strC.m_pWBuffer);
  return str;
}

/**************************************************************************/
/*! Hinzufuegen eines long-Values
    \param inval : long-Value
*/
/**************************************************************************/
TString TString::operator+ (const long inval)
{
  TString str(this->m_pWBuffer);
  int ndig = 15;
  char* pNewBuffer = new char[100];
  if (pNewBuffer != NULL)
  {
    TString strC(gcvt(inval, ndig, pNewBuffer ));
    str.concat(strC.m_pWBuffer);

    delete [] pNewBuffer;
  }
  return str;
}

/**************************************************************************/
/*! Hinzufuegen eines Double-Values
    \param inval : Double-Value
*/
/**************************************************************************/
TString TString::operator+ (const double inval)
{
  TString strTmp(this->m_pWBuffer);
  TString strConv = convertDouble(inval);
  strTmp.concatStr(&strConv);
  return strTmp;
}


/**************************************************************************/
/*! Vergleichsoperator. Der wchar_t-Array wird mit dem des Objekts verglichen.
    \param instr : zu vergleichendes Stringobjekt
*/
/**************************************************************************/
bool TString::operator !=(const TString& instr) const
{
  return !(equals(instr.m_pWBuffer));
}


/**************************************************************************/
/*! Vergleichsoperator. Der wchar_t-Array wird mit dem des Objekts verglichen.
    \param inbuffer : zu vergleichendes Stringobjekt
*/
/**************************************************************************/
bool TString::operator !=(const wchar_t* inbuffer)const
{
  return !(equals(inbuffer));
}

/**************************************************************************/
/*! Vergleichsoperator. Der char-Array wird mit dem des Objekts verglichen.
    \param inbuffer : zu vergleichendes Stringobjekt
*/
/**************************************************************************/
bool TString::operator !=(const char* inbuffer) const
{
  TString str(inbuffer);
  return !(equals(str.m_pWBuffer));
}

/**************************************************************************/
/*! Vergleichsoperator: Der einen double Wert mit dem String vergleicht
    \param data : Zuweisungsparameter
*/
/**************************************************************************/
bool TString::operator !=(const double data) const
{
  TString str(data);
  return !(equals(str.m_pWBuffer));
}

/**************************************************************************/
/*! Vergleichsoperator: Der einen long Wert mit dem String vergleicht
    \param data : Zuweisungsparameter
*/
/**************************************************************************/
bool TString::operator !=(const long data) const
{
  TString str(data);
  return !(equals(str.m_pWBuffer));
}

/**************************************************************************/
/*! Klammeroperator: Zugriff auf ein bestimmtes Character im String
    \param index : Zuweisungsparameter
    \return character an dieser Position
*/
/**************************************************************************/
wchar_t TString::operator [](long index) const
{
  return getChar(index);
}

/**************************************************************************/
/*! Converts a array of bytes to a string
*/
/**************************************************************************/
TString TString::transcode(const TByteArray& list)
{
  TString strRetval;
  long lCount = list.getCount();
  if (lCount > 0)
  {
    char* pBuffer = new char[lCount+1];
    memset(pBuffer, 0, sizeof(char)*(lCount+1));
    memcpy(pBuffer, list.getArray(), sizeof(char)*lCount);
    strRetval = pBuffer;
    delete [] pBuffer;
  }
  return strRetval;
}

/**************************************************************************/
/*! Converts a string into a arry of bytes
*/
/**************************************************************************/
TByteArray TString::transcode(const TString& value)
{
  long lCount = value.getSize() + 1;
  TByteArray array(lCount);
  array.setArray(value.c_str(), lCount);
  array[lCount-1] = 0x00;
  return array;
}


/**************************************************************************/
/*! Loeschen aller Buffer Objekte
*/
/**************************************************************************/
void TString::clearBuffer()
{
  clearCBuffer();
  clearWBuffer();
}

/**************************************************************************/
/*! Loeschen des char-Buffers
*/
/**************************************************************************/
void TString::clearCBuffer() const
{
  if (m_pCBuffer != NULL)
  {
    delete [] m_pCBuffer;
    m_pCBuffer = NULL;
  }
}

/**************************************************************************/
/*! Loeschen des wchar_t-Buffers
*/
/**************************************************************************/
void TString::clearWBuffer()
{
  if (m_pWBuffer != NULL)
  {
    delete [] m_pWBuffer;
    m_pWBuffer = NULL;
  }
}


/**************************************************************************/
/*! Konvertieren eines char_Array in ein WChar_Array
    \param data : Char_Array
*/
/**************************************************************************/
wchar_t* TString::convToWStr(const char* data)
{
  this->clearWBuffer();
  if (data != NULL)
  {
    size_t size1 = strlen(data)+1;
    m_pWBuffer = new wchar_t[size1];
    if (m_pWBuffer != NULL)
    {
      mbstowcs( m_pWBuffer, data, size1);
    }
  }
  else
  {
    m_pWBuffer = new wchar_t[1];
    if (m_pWBuffer != NULL)
    {
      m_pWBuffer[0] = 0x0000;
    }
  }
  return m_pWBuffer;
}

/**************************************************************************/
/*! Konvertieren eines wchar_Array in ein char_Array
    \param data : wchar_Array
*/
/**************************************************************************/
char* TString::convToCStr(const wchar_t* data) const
{
  this->clearCBuffer();
  if (data != NULL)
  {
    size_t size1 = wcslen(data)+1;
    m_pCBuffer = new char[size1];
    if (m_pCBuffer != NULL)
    {
      wcstombs( m_pCBuffer, data, size1);
    }
  }
  return m_pCBuffer;
}  

/*****************************************************************************/
/*! Konvertieren eines Doublewerts in einen StString
    \param value : double Wert
    \return Formatierter String
*/
/*****************************************************************************/
TString TString::convertDouble(const double value) const
{
  TString str;
  int sign = 0;
  int decimal = 0;
  int ndig = 15;
  double dTemp = ::fabs(value);

  char* pToBuffer = new char[100];
  char* pFromBuffer = new char[100];

  if (pFromBuffer != NULL && pToBuffer != NULL)
  {
    ::memset(pToBuffer,'0',100*sizeof(char));
    pFromBuffer = ::strcpy(pFromBuffer, ecvt(dTemp, ndig, &decimal, &sign));
    if (value < 0.0)
    {
      sign = 1;
    }
    else
    {
      sign = 0;
    }

    long lToIndex = 0;
    long lFromIndex = 0;
    long lFromLength = strlen(pFromBuffer);
    long lToLength = strlen(pFromBuffer);
    bool bHasDecimal = false;

    //Dezimalpunkt wird gesetzt
    if (decimal < lFromLength)
    {
      //Trimmen der L�ge
      long lTrimIndex = lFromLength-1;
      while(pFromBuffer[lTrimIndex] == '0' && lTrimIndex > 0 && lTrimIndex >= decimal)
      {
        lTrimIndex--;
      }
      if (lTrimIndex == decimal-1)
      {
        lToLength = lTrimIndex+1;
        lFromIndex = lTrimIndex;
      }
      else if (decimal <= 0)  //Dezimal punkt muss gesetzt werden
      {
        lToLength = lTrimIndex - decimal + 3; //Dezimalpunkt und eine Null
        lFromIndex = lTrimIndex;
        bHasDecimal = true;
        decimal = 1;
      }
      else
      {
        lToLength = lTrimIndex + 2; //String wird um Dezimalpunkt l�ger
        lFromIndex = lTrimIndex;
        bHasDecimal = true;
      }
    }
    //Dezimalpunkt wird nicht gesetzt
    else
    {
      lFromIndex = lFromLength-1;
      lToIndex = lToLength-1;
    }

    if (sign == 1)
    {
      lToLength++;
      decimal++;
    }

    //Initialisieren der Indexes
    lToIndex = lToLength-1;

    while (lFromIndex >= 0)
    {
      //Dezimal punkt berspringen
      if(decimal == lToIndex && bHasDecimal)
      {
        lToIndex--;
      }
      else
      {
        pToBuffer[lToIndex] = pFromBuffer[lFromIndex];
        lToIndex--;
        lFromIndex--;
      }
    }

    //Setzen des Signs
    if (sign == 1)
    {
      pToBuffer[0] = '-';
    }

    //Setzen des Dezimalpunkts
    if(bHasDecimal)
    {
      pToBuffer[decimal] = '.';
    }


    pToBuffer[lToLength] = '\0';

    str = pToBuffer;
    //free(pFromBuffer);
    //free(pToBuffer);
    delete [] pFromBuffer;
    delete [] pToBuffer;
  }
  return str;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
unsigned long MQUALIFIER TString::askForInterface(long interface_id, IZUnknown** iface)
{
  unsigned long ulRetval = RET_NOERROR;
  if (iface == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    switch(interface_id)
    {
      default:
        ulRetval = RET_UNKNOWN_INTERFACE;
    }
  }

  return ulRetval;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
unsigned long MQUALIFIER TString::addRef()
{
  ++m_ulRefCounter;
  return RET_NOERROR;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
unsigned long MQUALIFIER TString::release()
{
  --m_ulRefCounter;
  if (m_ulRefCounter <= 0)
  {
    delete this;
  }
  return RET_NOERROR;
}

#ifdef USE_STL_BINDINGS
/***************************************************************************/
/*! Assign operator for stl string class
*/
/***************************************************************************/
TString::TString(const std::string& instr)
{
  m_ulRefCounter = 1;
  m_pWBuffer = NULL;
  m_pCBuffer = NULL;
  m_pWBuffer = convToWStr(instr.c_str());
}

/***************************************************************************/
/*! Assign operator for stl string class
*/
/***************************************************************************/
TString& TString::operator =(const std::string& instr)
{
  m_pWBuffer = convToWStr(instr.c_str());
  return *this;
}

/***************************************************************************/
/*! Equals operator for stl string class
*/
/***************************************************************************/
bool TString::operator ==(const std::string& instr)const
{
  TString str(instr.c_str());
  return equals(str.m_pWBuffer);
}

/***************************************************************************/
/*! Concat operator for stl string class
*/
/***************************************************************************/
TString TString::operator+ (const std::string& instr)
{
  TString str(this->m_pWBuffer);
  TString strC(instr.c_str());
  str.concat(strC.m_pWBuffer);
  return str;
}

/***************************************************************************/
/*! Not Equal operator for stl string class
*/
/***************************************************************************/
bool TString::operator !=(const std::string& instr) const
{
  TString str(instr.c_str());
  return !equals(str.m_pWBuffer);
}
#endif  


#ifdef USE_BCB_BINDINGS
/***************************************************************************/
/*! Assign operator for Borland string class
*/
/***************************************************************************/
TString& TString::operator =(const AnsiString& instr)
{
  m_pWBuffer = convToWStr(instr.c_str());
  return *this;
}

/***************************************************************************/
/*! Equals operator for Borland string class
*/
/***************************************************************************/
bool TString::operator ==(const AnsiString& instr)const
{
  TString str(instr.c_str());
  return equals(str.m_pWBuffer);
}

/***************************************************************************/
/*! Concat operator for Borland string class
*/
/***************************************************************************/
TString TString::operator+ (const AnsiString& instr)
{
  TString str(this->m_pWBuffer);
  TString strC(instr.c_str());
  str.concat(strC.m_pWBuffer);
  return str;
}

/***************************************************************************/
/*! Not Equal operator for Borland string class
*/
/***************************************************************************/
bool TString::operator !=(const AnsiString& instr) const
{
  TString str(instr.c_str());
  return !equals(str.m_pWBuffer);
}
#endif  

