/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : String                                                        *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:56:34 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: String.h,v $
 *   Revision 1.2  2005/11/25 09:56:34  bhadorn
 *   Implemented standard and borland bindings for strings
 *
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef StringH
#define StringH

#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/ByteArray.hpp>
#include <stdlib.h>

#ifdef USE_BCB_BINDINGS
  #include <Classes.hpp>
#endif

#ifdef USE_STL_BINDINGS
  #include <string>
#endif


BEGIN_NAMESPACE_Zeus


/******************************************************************************/
/*! Die Stringklasse kapelt einen NULL-terminierenden WideString.
*/
/******************************************************************************/
class TString : public IString
{
  public:
    TString();
    TString(const wchar_t* data);
    TString(const char* data);
    TString(const TString& string);
    TString(const IString* string);
    TString(long value);
    TString(double value);
        
    virtual ~TString();

    //transcodes
    static TString transcode(const TByteArray& list);
    static TByteArray transcode(const TString& value);
        
    //Operatoren
    TString& operator =(const TString& instr);
    TString& operator =(const wchar_t* data);
    TString& operator =(const long data);
    TString& operator =(const double data);
    TString& operator =(const char* data);
    bool operator ==(const TString& instr)const;
    bool operator ==(const wchar_t* inbuffer) const;
    bool operator ==(const char* inbuffer)const;
    bool operator ==(const double data) const;
    bool operator ==(const long data)const;
    TString operator+ (const TString& instr);
    TString operator+ (const wchar_t* inbuffer);
    TString operator+ (const char* inbuffer);
    TString operator+ (const long inval);
    TString operator+ (const double inval);
    bool operator !=(const TString& instr) const;
    bool operator !=(const wchar_t* inbuffer)const;
    bool operator !=(const char* inbuffer) const;
    bool operator !=(const double data) const;
    bool operator !=(const long data) const;
    wchar_t operator [](long index) const;
    TString getSubString(long start, long end) const;
    
    //Methods of IString
    virtual wchar_t* MQUALIFIER c_bstr() const;
    virtual char* MQUALIFIER c_str() const;
    virtual double MQUALIFIER toDouble(double predef = 0.0) const;
    virtual long MQUALIFIER toLong(long predef = 0) const;
    virtual bool MQUALIFIER equals(const wchar_t* inbuffer) const;
    virtual bool MQUALIFIER equalsStr(const IString* instr) const;
    virtual void MQUALIFIER concat(const wchar_t* inbuffer);
    virtual void MQUALIFIER concatStr(const IString* instr);
    virtual void MQUALIFIER assign(const wchar_t* inbuffer);
    virtual void MQUALIFIER assignStr(const IString* instr);
    virtual long MQUALIFIER getSize()const;
    virtual wchar_t MQUALIFIER getChar(long index)const;
    virtual long MQUALIFIER getPos(const IString* substring)const;
    virtual bool MQUALIFIER isEmpty()const;
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  //STL Bindings
  #ifdef USE_STL_BINDINGS
    TString(const std::string& instr);
    TString& operator =(const std::string& instr);
    bool operator ==(const std::string& instr)const;
    TString operator+ (const std::string& instr);
    bool operator !=(const std::string& instr) const;  
  #endif  
  
  //Borland bindings for TString class
  #ifdef USE_BCB_BINDINGS
    TString& operator =(const AnsiString& instr);
    bool operator ==(const AnsiString& instr)const;
    TString operator+ (const AnsiString& instr);
    bool operator !=(const AnsiString& instr) const; 
  #endif  
  
  private:
    ///Reference Counter
    unsigned long m_ulRefCounter;
      
    ///wchar_t Array Buffer
    wchar_t* m_pWBuffer;
    ///char Array(Wird nur temp. gebraucht)
    mutable char* m_pCBuffer;

    void clearBuffer();
    void clearWBuffer();
    void clearCBuffer() const;
    wchar_t* convToWStr(const char* data);
    char* convToCStr(const wchar_t* data) const;
    TString convertDouble(const double value) const;
};

//-----------------------------------------------------------------
//Global operators 
#ifdef USE_STL_BINDINGS
  inline void operator<<(std::string& outstr, const TString& instr)
  {
    outstr = instr.c_str();
  }
  inline void operator<<(std::wstring& outstr, const TString& instr)
  {
    outstr = instr.c_bstr();
  }
#endif

#ifdef USE_BCB_BINDINGS
  inline void operator<<(AnsiString& outstr, const TString& instr)
  {
    outstr = instr.c_str();
  }
  inline void operator<<(WideString& outstr, const TString& instr)
  {
    outstr = instr.c_bstr();
  }
#endif
//-----------------------------------------------------------------

END_NAMESPACE_Zeus

#endif
