/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StringList                                                    *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:46:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: StringList.cpp,v $
 *   Revision 1.2  2005/11/25 09:46:47  bhadorn
 *   Implemented const_iterator
 *
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/StringList.h> 

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new StringList object
*/
/***************************************************************************/
TStringList::TStringList() : TZObject()
{}

/***************************************************************************/
/*! Destroys the StringList object
*/
/***************************************************************************/
TStringList::~TStringList()
{
  clear();
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
long MQUALIFIER TStringList::add(IString* const &Item)
{
  TString* pStr = NULL;
  if (Item != NULL)
  {
    pStr = new TString(Item);
  }
  return m_lstStrings.add(pStr);
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
void MQUALIFIER TStringList::copyToList(IList<IString*>* list) const
{
  if (list != NULL)
  {
    list->clear();
    long lCount = m_lstStrings.getCount();
    for(int i = 0; i < lCount; i++)
    {
      TString* pStr = new TString(m_lstStrings.getItemCopy(i));
      list->add(pStr);
      pStr->release();
    }
  }
}


/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
void MQUALIFIER TStringList::clear()
{
  while(m_lstStrings.getCount() > 0)
  {
    if (m_lstStrings[0] != NULL)
    {
      m_lstStrings[0]->release(); 
    }
    m_lstStrings.deleteItem(0);
  }  
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
unsigned long MQUALIFIER TStringList::deleteItem(long Index)
{

  IString* pStr = m_lstStrings[Index];
  unsigned long ulRetval = m_lstStrings.deleteItem(Index);

  if (pStr != NULL)
  {
    pStr->release();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
unsigned long MQUALIFIER TStringList::remove(IString* const & Item)
{
  return deleteItem(indexOf(Item));
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
long MQUALIFIER TStringList::getCount() const
{
  return m_lstStrings.getCount();
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
IString*& MQUALIFIER TStringList::getItem(long index) 
{
  return m_lstStrings.getItem(index);
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
IString* MQUALIFIER TStringList::getItemCopy(long index) const
{
  return m_lstStrings.getItemCopy(index);
}


/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
long MQUALIFIER TStringList::indexOf(IString* const & Item) const
{
  long lCount = this->getCount();
  long lIndex = -1;
  
  for(int i = 0; i < lCount; i++)
  {
    IString* pItem = m_lstStrings.getItemCopy(i);
    if (pItem->equalsStr(Item))
    {
      lIndex = i;
      break;
    }
  }
  return lIndex;
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
bool MQUALIFIER TStringList::equalsItem(long index, IString* const &Item) const
{
  bool bRetval = false;
  
  IString* pItem = getItemCopy(index);
  if (pItem == Item)
  {
    bRetval = true;
  }
  else if (pItem != NULL)
  {
    bRetval = (pItem->equalsStr(Item));
  }
  else
  {
    bRetval = false;
  }
  return bRetval;
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
bool MQUALIFIER TStringList::equals(const IStringList* list) const
{
  bool bRetval = false;
  if (list != NULL)
  {
    long lCount = this->getCount();
    bRetval = (lCount == list->getCount());
    
    if (bRetval)
    {
      for(int i = 0; i < lCount; i++)
      {
        bRetval &= (TString(getItemCopy(i)) == TString(list->getItemCopy(i)));
      }
    }
  }
  return bRetval;
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
long MQUALIFIER TStringList::insert(long Index, IString* const &Item)
{
  TString* pStr = NULL;
  if (Item != NULL)
  {
    pStr = new TString(Item);
  }
  return m_lstStrings.insert(Index, pStr);
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
IListIterator<IString*>* MQUALIFIER TStringList::getIterator() const
{
  return m_lstStrings.getIterator();
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
const IListIterator<IString*>* MQUALIFIER TStringList::getConstIterator() const
{
  return m_lstStrings.getConstIterator();
}

/***************************************************************************/
/*! See IStringList
*/
/***************************************************************************/
void MQUALIFIER TStringList::releaseIterator(const IListIterator<IString*>* iterator) const
{
  m_lstStrings.releaseIterator(iterator);
}

/******************************************************************************/
/*!  See IStringList
*/
/******************************************************************************/
bool MQUALIFIER TStringList::isEmpty() const
{
  return m_lstStrings.isEmpty();
}


/******************************************************************************/
/*!
    Operator zur fr das einfache Zugreifen auf den Elementinhalt
    \param Index : Index auf ein Element der Liste
    \return gibt den Dateninhalt eines Elements der Liste zurck
*/
/******************************************************************************/
IString*& TStringList::operator[](long Index)
{
  return getItem(Index);
}


/******************************************************************************/
/*! Operator zum kopieren einer Liste
    \param list : Ausgangsliste
    \return neu kopierte Liste
*/
/******************************************************************************/
TStringList& TStringList::operator=(const TStringList& list)
{
  list.copyToList(this);
  return *this;
}

/******************************************************************************/
/*! Operator to check equality of two lists
    \param list : list to check
    \retval true : Equal
    \retval false : not equal
*/
/******************************************************************************/
bool TStringList::operator==(const TStringList& list)
{
  return this->equals(&list);
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TStringList);
MEMORY_MANAGER_IMPL_END;

 
