/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StringList                                                    *
 * Package : system                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:46:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: StringList.h,v $
 *   Revision 1.2  2005/11/25 09:46:47  bhadorn
 *   Implemented const_iterator
 *
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  


#ifndef StringListH 
#define StringListH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ArrayList.hpp>
#include <zeusbase/System/Interfaces/IStringList.hpp>


BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This class implements a string based list.
*/
/***************************************************************************/
class TStringList : public TZObject, public IStringList
{
  public:
    TStringList();
    virtual ~TStringList();

    //Methods of IStringList
    virtual long MQUALIFIER add(IString* const &Item);
    virtual void MQUALIFIER copyToList(IList<IString*>* list) const;
    virtual void MQUALIFIER clear();
    virtual unsigned long MQUALIFIER deleteItem(long Index);
    virtual unsigned long MQUALIFIER remove(IString* const & Item);
    virtual long MQUALIFIER getCount()const;
    virtual IString*& MQUALIFIER getItem(long index);
    virtual IString* MQUALIFIER getItemCopy(long index) const;
    virtual bool MQUALIFIER equalsItem(long Index, IString* const &Item) const;
    virtual bool MQUALIFIER equals(const IStringList* list) const;
    virtual long MQUALIFIER indexOf(IString* const &Item) const;
    virtual long MQUALIFIER insert(long Index, IString* const &Item);
    virtual IListIterator<IString*>* MQUALIFIER getIterator() const;
    virtual const IListIterator<IString*>* MQUALIFIER getConstIterator() const;
    virtual void MQUALIFIER releaseIterator(const IListIterator<IString*>* iterator) const;
    virtual bool MQUALIFIER isEmpty() const;
    
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
    //Operators
    IString*& operator[](long Index);
    TStringList& operator=(const TStringList& list);
    bool operator==(const TStringList& list);
  
  private:
    ///List of strings 
    TArrayList<IString*> m_lstStrings;
};

END_NAMESPACE_Zeus

#endif
