/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CellLife                                                      *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:07 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Thread.h,v $
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ThreadH
#define ThreadH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/Event.h>
#include <zeusbase/System/Interfaces/IRunnable.hpp>

#ifdef LINUX
  #include <pthread.h>
#else
  #include <windows.h>
#endif


BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Diese Klasse kapselt einen Thread. Der Thread wird in der Methode
    execute() taetig
*/
/***************************************************************************/
class TThread : public TZObject
{
  public:
    TThread();
    TThread(IRunnable* obj);

    virtual ~TThread();

    virtual void start();
    virtual void kill();
    virtual bool isDead();
    virtual bool isInterrupted();

    virtual void execute();
    virtual void doKill();

  protected:
    ///Abbruchflag fuer Thread
    bool m_bInterrupted;

  private:
  protected:
    ///Flag gibt an ob der Thread erstellt wurde
    bool m_bCreated;
    ///Runable Interface
    IRunnable* m_pRunObj;
    ///Terminate event
    TEvent m_Event;
    #ifdef LINUX
      ///Thread
      pthread_t Thread;
    #else
      HANDLE m_hHandle;
    #endif
};

END_NAMESPACE_Zeus

#endif
