/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Time                                                          *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:07 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Time.cpp,v $
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeusbase/System/Time.h> 


#ifdef LINUX
  #include <stddef.h>
  #include <unistd.h>
  #include <sys/time.h>
  #include <stdlib.h>
#else
  #include <stdlib.h>
  #include <windows.h>
#endif

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Returns a random number, witch is between from and to
    \param from : lower limit
    \param to upper limit
    \return random number
*/
/***************************************************************************/
unsigned long TTime::random(unsigned long from, unsigned long to)
{

  unsigned long ulDiff = abs(to - from)+1;
  unsigned long long ullTimeStamp = ::rand();

  unsigned long ulRes = (unsigned long)(ullTimeStamp % ulDiff);
  return from + ulRes;
}


/***************************************************************************/
/*! Initializes the random function
*/
/***************************************************************************/
void TTime::initRandom()
{
  #ifdef LINUX
    ::srandom(getTickCount()%RAND_MAX);
  #else  
    ::randomize();
  #endif
}

/***************************************************************************/
/*! Returns the time in micro seconds
*/
/***************************************************************************/
unsigned long long TTime::getTickCount()
{
  unsigned long long ullSec = 0;
  #ifdef LINUX
    timeval rTime;
    gettimeofday(&rTime, NULL);
    ullSec = rTime.tv_sec *1000000 +  rTime.tv_usec;
  #else
    ullSec = ::GetTickCount()*1000;
  #endif

  return ullSec;
}


/***************************************************************************/
/*! Returns the time in seconds
*/
/***************************************************************************/
double TTime::getTimeStamp()
{
  unsigned long long ullSec = 0;
  #ifdef LINUX
    timeval rTime;
    gettimeofday(&rTime, NULL);
    ullSec = rTime.tv_sec *1000000+  rTime.tv_usec;
  #else
    ullSec = ::GetTickCount()*1000;
  #endif

  return ullSec/(double)1000000.0;
}

/***************************************************************************/
/*! Dwells for a time in micro seconds
*/
/***************************************************************************/
void TTime::usleep(unsigned long long dwell)
{
  #ifdef LINUX
    ::usleep(dwell);
  #else
    ::SleepEx((long)(dwell/1000), true);
  #endif
}

/***************************************************************************/
/*! Dwells for a time in seconds
*/
/***************************************************************************/
void TTime::sleep(double dwell)
{
  #ifdef LINUX
    ::usleep((unsigned int)(1000000*dwell));
  #else
    ::SleepEx((unsigned int)(1000*dwell), true);
  #endif
}

