/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XLoaderObject                                                 *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/15 07:08:28 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XLoaderObject.cpp,v $
 *   Revision 1.3  2005/12/15 07:08:28  bhadorn
 *   no message
 *
 *   Revision 1.2  2005/12/14 09:05:32  bhadorn
 *   children has been loaded twice. Error corrected.
 *
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 
#include <zeusbase/System/XLoaderObject.h>
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus

/***************************************************************************/ 
/*! Creates a loadable object node
    \param node : XML Node
*/
/***************************************************************************/ 
TXLoaderObject::TXLoaderObject(IXMLNode* node) : TXObject(node)
{
  m_bLoaded = false;
  m_pDocument = NULL;
  m_pExtRefNode = NULL;
}
    
/***************************************************************************/ 
/*! Destroys the loadable object
*/
/***************************************************************************/ 
TXLoaderObject::~TXLoaderObject()
{  
  if (m_bLoaded)
  {
    unload();
  }
  if (m_pExtRefNode != NULL)
  {
    m_pExtRefNode->release();
    m_pExtRefNode = NULL;
  }
  if (m_pDocument != NULL)
  {
    m_pDocument->release();
    m_pDocument = NULL;
  }
}
  
/***************************************************************************/ 
/*! see IXLoaderObject
*/
/***************************************************************************/ 
unsigned long MQUALIFIER TXLoaderObject::load()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bLoaded)
  {
    ulRetval = RET_NOERROR;
  }
  else
  {
    
    //Children not loaded, but module is unforzen
    // All children must be loaded and the flag must be set.
    if (!this->canCreateChildren() && !this->isFrozen())
    {
      m_bLoaded = true;
      TString strAttName = L"CreateChildren";
      long lValue = 1;
      this->storeAttribute(strAttName, lValue);
      
      TString strName;
      this->getName(&strName);
      ZLOGGER.printfln(LOGMODE_DEBUG, "Try to load object: %s...", strName.c_str());
      
      ///////////////////////
      // This will load and create all children (also external xml files)
      ulRetval = TXObjectFactory::getInstance().createChildObjects(m_pNode, this, false);
      ///////////////////////
      
      if (ulRetval == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_DEBUG,"..done.");
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_DEBUG,"..failed.");
      }
      
      //do unfreeze children
      this->unfreezeChildren();
    }
    ///////////////////

        
    //If the module can create children, bat the flag has not been set
    // all children were created and loaded already. So do nothing here !!!
    else if (this->canCreateChildren())
    {
      m_bLoaded = true;
      ulRetval = RET_NOERROR;
    }
    ///////////////////

        
    //if the module is has not created its children,but the module 
    // is not unfrozen at this point, the module can not be loaded!!!
    else
    {
    }
    ///////////////////
  }
  return ulRetval;
}


/***************************************************************************/ 
/*! see IXLoaderObject
*/
/***************************************************************************/ 
unsigned long MQUALIFIER TXLoaderObject::unload()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (!m_bLoaded)
  {
    ulRetval = RET_NOERROR;
  }
  else
  {
    TString strName;
    this->getName(&strName);
    ZLOGGER.printfln(LOGMODE_DEBUG, "Try to unload object: %s...done", strName.c_str());
    
    ulRetval = RET_NOERROR;
    m_bLoaded = false;
  }
  return ulRetval;  
}

/***************************************************************************/ 
/*! see IXLoaderObject
*/
/***************************************************************************/ 
bool MQUALIFIER TXLoaderObject::isLoaded()
{
  return m_bLoaded;
}
 
/***************************************************************************/ 
/*! see IXLoaderObject
*/
/***************************************************************************/ 
unsigned long MQUALIFIER TXLoaderObject::setExtXMLRef(NAMESPACE_Zeus::IXMLDocument* doc, IXMLNode* main_ref)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (doc != NULL && main_ref != NULL)
  {
    if (m_pDocument == NULL)
    {
      m_pDocument = doc;
      m_pDocument->addRef();
      m_pExtRefNode = main_ref;
      m_pExtRefNode->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TXLoaderObject::unfreeze()
{
  if (this->canCreateChildren())
  {
    load();
  }
  return TXObject::unfreeze();
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXLoaderObject::addChild(IXObject* child)
{
  IXMLNode* pTemp = m_pNode;
  
  //Swap node if an external reference exists
  if (m_pExtRefNode != NULL)
  {
    m_pNode = m_pExtRefNode;
  }
  unsigned long ulRetval = TXObject::addChild(child);
  m_pNode = pTemp;
  
  return ulRetval;
}


/***************************************************************************/ 
/*! see IXObject
*/
/***************************************************************************/ 
unsigned long MQUALIFIER TXLoaderObject::getChild(long index, IXObject** child)
{
  if(!m_bLoaded)
  {
    load();
  }
  return TXObject::getChild(index, child);
}

/***************************************************************************/ 
/*! see IXObject
*/
/***************************************************************************/ 
unsigned long MQUALIFIER TXLoaderObject::getChildByName(const IString* name, IXObject** child)
{
  if(!m_bLoaded)
  {
    load();
  }
  return TXObject::getChildByName(name, child);
}

 
/***************************************************************************/
/*! Check the attribute "CreateChildren". If the value is set to 1, this
    object will be loaded and all children will be created. See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TXLoaderObject::canCreateChildren()const
{
  bool bRetval = true;
  TString strAttName = L"CreateChildren";
  long lValue = 0;
  if (this->readAttribute(strAttName, lValue) == RET_NOERROR)
  {
    bRetval = (lValue == 1);
  }
  return bRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TXLoaderObject);
  INTERFACE_CAST(IXLoaderObject, INTERFACE_IXLoaderObject);
MEMORY_MANAGER_IMPL_PARENT_END(TXObject);

