/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XLoaderObject                                                 *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/15 07:08:28 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XLoaderObject.h,v $
 *   Revision 1.3  2005/12/15 07:08:28  bhadorn
 *   no message
 *
 *   Revision 1.2  2005/12/14 09:05:32  bhadorn
 *   children has been loaded twice. Error corrected.
 *
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef XLoaderObjectH
#define XLoaderObjectH 

#include <zeusbase/System/Interfaces/IXLoaderObject.hpp>
#include <zeusbase/System/XObject.h>
#include <zeusbase/System/XObjectFactory.h>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This Object is a loadable node of the xobject tree. This means, that
    a factory who creates the tree wont create its children. The children
    were defined in seperate xml document and are loaded when someone 
    need a child instance only. This mechanisme is usefull to save 
    system resources. 
*/
/***************************************************************************/
class TXLoaderObject : public TXObject, public IXLoaderObject
{
  public:
    TXLoaderObject(IXMLNode* node);
    virtual ~TXLoaderObject();
  
    //Methods of IXLoaderObject
    virtual unsigned long MQUALIFIER load();
    virtual unsigned long MQUALIFIER unload();
    virtual bool MQUALIFIER isLoaded();
    virtual unsigned long MQUALIFIER setExtXMLRef(NAMESPACE_Zeus::IXMLDocument* doc, IXMLNode* main_ref);

    //Methods of TXObject
    virtual unsigned long MQUALIFIER addChild(IXObject* child);
    virtual unsigned long MQUALIFIER getChild(long index, IXObject** child);
    virtual unsigned long MQUALIFIER getChildByName(const IString* name, IXObject** child);
    virtual bool MQUALIFIER canCreateChildren()const;
    virtual bool MQUALIFIER unfreeze();
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

    REG_SUB_BEGIN(TXLoaderObject)
    REG_SUB_ADD(TXLoaderObject, L"TXLoaderObject");
    REG_SUB_END
        
  private:
    ///Flag to be set when the resource has been loaded
    bool m_bLoaded;
    ///XML Document witch contains the external XML Reference
    IXMLDocument* m_pDocument;
    ///XML Mainnode of the external reference
    IXMLNode* m_pExtRefNode;
    
};

END_NAMESPACE_Zeus

#endif
