/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XObject                                                       *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:03:25 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObject.h,v $
 *   Revision 1.2  2005/12/14 09:03:25  bhadorn
 *   new methods for call backs added.
 *
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef XObjectH
#define XObjectH 

#include <zeusbase/System/ZObject.h> 
#include <zeusbase/System/SingleLinkedList.hpp> 
#include <zeusbase/System/Interfaces/IXMLNode.hpp>
#include <zeusbase/System/Interfaces/IXObject.hpp>

#define ATTSCHEMA_NORMAL      1
#define ATTSCHEMA_EXTENDED    2
#define ATTSCHEMA_COMPLEX     3

BEGIN_NAMESPACE_Zeus

class TCriticalSection;

/***************************************************************************/
/*! This is the base class for all XML based objects.
*/
/***************************************************************************/
class TXObject : public TZObject, public IXObject
{
  public:
    TXObject(IXMLNode* node, unsigned long attributeschema = ATTSCHEMA_NORMAL);
    
    virtual TString getXMLStream() const;
    virtual TString getCodeModule() const;
    
    //Methods of IXObject
    virtual unsigned long MQUALIFIER getName(IString* name)const;
    virtual unsigned long MQUALIFIER getID()const;
    virtual unsigned long MQUALIFIER addChild(IXObject* child);
    virtual unsigned long MQUALIFIER deleteChild(long index);
    virtual unsigned long MQUALIFIER removeChild(IXObject* child);
    virtual long MQUALIFIER getChildCount()const;
    virtual unsigned long MQUALIFIER getChild(long index, IXObject** child);
    virtual unsigned long MQUALIFIER getChildByName(const IString* name, IXObject** child);
    virtual unsigned long MQUALIFIER getParent(IXObject** parent);
    virtual unsigned long MQUALIFIER setParent(IXObject* parent);
    virtual unsigned long MQUALIFIER getXMLNode(IXMLNode** node)const;
    virtual unsigned long MQUALIFIER getAttributeSchema()const;
    virtual unsigned long MQUALIFIER setChild(IXObject* child);
    virtual unsigned long MQUALIFIER getObject(IString* path, IXObject** obj);
    virtual unsigned long MQUALIFIER getRootObject(IXObject** obj);
    virtual bool MQUALIFIER canCreateChildren()const;
    virtual bool MQUALIFIER freeze();
    virtual bool MQUALIFIER unfreeze();
    virtual bool MQUALIFIER isFrozen()const;
    virtual void MQUALIFIER setTreeLock(ICriticalSection* section);
    virtual void MQUALIFIER lockTree();
    virtual void MQUALIFIER unlockTree();
    virtual unsigned long MQUALIFIER onExecute(const ISimpleCommand* command, unsigned long ulMode);

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
    //Methods of TZObject
    virtual void setObjName(const TString& name);
    
    
  protected:   
    ///XML Node
    IXMLNode* m_pNode;
    ///Lock of the data and object tree
    ICriticalSection* m_pTreeLock;
    
    TXObject();
    virtual ~TXObject();

    virtual void onChildAdded(IXObject* /*child*/){};
    virtual void onChildDeleted(IXObject* /*child*/){};

    //Methods to read a persistent value  
    unsigned long readAttribute(TString name, TString& value) const;
    unsigned long readAttribute(TString name, long& value) const;
    unsigned long readAttribute(TString name, unsigned long& value) const;
    unsigned long readAttribute(TString name, double& value) const;
    
    //Methods to write a persistent value  
    unsigned long storeAttribute(TString name, TString value);
    unsigned long storeAttribute(TString name, long value);
    unsigned long storeAttribute(TString name, unsigned long value);
    unsigned long storeAttribute(TString name, double value);
    
    unsigned long readAttAsNode(TString name, TString& value) const;
    unsigned long writeAttAsNode(TString name, TString value);
    void unfreezeChildren();
    
  private:
    ///Liste der untergeordneten Objekte
    TSingleLinkedList<IXObject*> m_lstChildren;
    ///Parent object
    IXObject* m_pParent;
    ///Schema of attributes
    unsigned long m_ulAttributeSchema;
    ///Frozen Flag
    bool m_bFrozen;
    ///Lock of this object
    TCriticalSection* m_pLock;
}; 

END_NAMESPACE_Zeus
#endif
