/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XObjectFactory                                                *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObjectFactory.cpp,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/XRootObject.h>
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Interfaces/IXMLParser.hpp>
#include <zeusbase/System/Interfaces/IXLoaderObject.hpp>

USING_NAMESPACE_Zeus


//Initialize the static member
TArrayList<TXObjectFactory::RClassRegistry> TXObjectFactory::m_lstStaticClasses(RClassRegistry(L"", NULL), 20);
TXObjectFactory TXObjectFactory::m_Factory;

/***************************************************************************/
/*! Creates a xobject factory instance
*/
/***************************************************************************/
TXObjectFactory::TXObjectFactory()
{
  m_pDelegation = NULL;
}

    
/***************************************************************************/
/*! Destroys the xobject factory instance
*/
/***************************************************************************/
TXObjectFactory::~TXObjectFactory()
{
  releaseDelegationInterface();
}

/***************************************************************************/
/*! See IXObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObjectFactory::createObjectFromStream(const IString* stream, 
                                                                 IXObject** obj, 
                                                                 bool onlyroot /*= true*/, 
                                                                 bool dounfreeze /*= true*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->createObjectFromStream(stream, obj, onlyroot);
  }
  else
  {
    IXMLParser* pParser = NULL;
    if (obj == NULL)
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    else if (this->getXMLParser(&pParser) == RET_NOERROR)
    {
      IXMLDocument* pDocument = NULL;
      if (pParser->parseStream(stream, &pDocument) == RET_NOERROR)
      {
        IXMLNode* pMainNode = NULL;
        if (pDocument->getMainNode(&pMainNode) == RET_NOERROR)
        {
          ulRetval = createRootObject(pMainNode, pDocument, obj, onlyroot);
          pMainNode->release();
        }
        else
        {
          ulRetval = RET_REQUEST_FAILED;
        }
        
        //-------------------------------------------
        //Awake the object
        if (ulRetval == RET_NOERROR && dounfreeze)
        {
          (*obj)->unfreeze();
        }
        //-------------------------------------------
        
        pDocument->release();
      }
      else
      {
        ulRetval = RET_INVALID_DATA;
      }
      pParser->release();
    }
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObjectFactory::createObjectFromFile(const IString* filename, 
                                                               IXObject** obj, 
                                                               bool onlyroot /*= true*/, 
                                                               bool dounfreeze /*= true*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->createObjectFromFile(filename, obj, onlyroot);
  }
  else
  {
    IXMLParser* pParser = NULL;
    if (obj == NULL)
    {
      ulRetval = RET_INVALID_PARAMETER;
    }
    else if (this->getXMLParser(&pParser) == RET_NOERROR)
    {
      IXMLDocument* pDocument = NULL;
      if (pParser->loadFromFile(filename, &pDocument) == RET_NOERROR)
      {
        IXMLNode* pMainNode = NULL;
        if (pDocument->getMainNode(&pMainNode) == RET_NOERROR)
        {
          ulRetval = createRootObject(pMainNode, pDocument, obj, onlyroot);
          pMainNode->release();
        }
        else
        {
          ulRetval = RET_REQUEST_FAILED;
        }
        
        //-------------------------------------------
        //Awake the object
        if (ulRetval == RET_NOERROR && dounfreeze)
        {
          (*obj)->unfreeze();
        }
        //-------------------------------------------
        
        pDocument->release();
      }
      else
      {
        ulRetval = RET_INVALID_DATA;
      }
      pParser->release();
    }
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
    }  
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObjectFactory::createChildObjects(IXMLNode* parentnode, IXObject* parent, bool dounfreeze /*= true*/)
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->createChildObjects(parentnode, parent, dounfreeze);
  }
  else if (parentnode != NULL && parent != NULL)
  {
    ulRetval = this->createSubObjects(parentnode, parent);
    
    //-------------------------------------------
    //Awake the object
    if (ulRetval == RET_NOERROR && dounfreeze)
    {
      parent->unfreeze();
    }
    //-------------------------------------------
  }
  return ulRetval;
}


/***************************************************************************/
/*! See IXObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObjectFactory::addNewChildObjects(IXMLNode* new_node, 
                                                        IXObject* parent,
                                                        IXObject** child,  
                                                        bool dounfreeze /*= true*/)
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->addNewChildObjects(new_node, parent, child, dounfreeze);
  }
  else if (child == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (new_node != NULL && parent != NULL)
  {
    if (this->createRootObject(new_node, NULL, child, false) == RET_NOERROR)
    {
      parent->addChild(*child);
      //pObject->release();
      ulRetval = RET_NOERROR;
    }
     
    //-------------------------------------------
    //Awake the object
    if (ulRetval == RET_NOERROR && dounfreeze)
    {
      (*child)->unfreeze();
    }
    //-------------------------------------------
  }
  return ulRetval;
}


/***************************************************************************/
/*! See IXObjectFactory
*/
/***************************************************************************/
long MQUALIFIER TXObjectFactory::registerXClass(const IString* classname, IXObject::FCreateXObject funct)
{
  long lRetval = -1;
  TString strCodeModule;
  if (m_pDelegation != NULL)
  {
    lRetval = m_pDelegation->registerXClass(classname, funct);
  }
  else if (lookupFactoryFunct(classname, &strCodeModule) == NULL)
  {
    RClassRegistry pItem(classname, funct);
    lRetval = m_lstStaticClasses.add(pItem);
    ZLOGGER.println(LOGMODE_DEBUG, "Class registered: Name=%s: Handle=%d", TString(classname).c_str(), (int)lRetval);
  }
  return lRetval;
}

/***************************************************************************/
/*! See IXObjectFactory
*/
/***************************************************************************/
bool MQUALIFIER TXObjectFactory::unregisterXClass(long handle, const IString* classname)
{
  bool bRetval = false;
  if (m_pDelegation != NULL)
  {
    bRetval = m_pDelegation->unregisterXClass(handle, classname);
  }
  else 
  {
    RClassRegistry& Item = m_lstStaticClasses[handle];
    if (classname->equalsStr(&Item.strClassName))
    {
      RClassRegistry& Item = m_lstStaticClasses[handle];
      ZLOGGER.println(LOGMODE_DEBUG, "Class unregistered: Name=%s: Handle=%d", Item.strClassName.c_str(),(int)handle);
      m_lstStaticClasses.deleteItem(handle);
      bRetval = true;
    }
  }
  return bRetval;
}

/***************************************************************************/
/*! Sets a delegation interface. All method calls will be delegated.
    \param factory : Interface
*/
/***************************************************************************/
void TXObjectFactory::setDelegationInterface(IXObjectFactory* factory)
{
  releaseDelegationInterface();
  m_pDelegation = factory;
  if (m_pDelegation != NULL)
  {
    m_pDelegation->addRef();
  }
}

/***************************************************************************/
/*! Release the delegation interface
*/
/***************************************************************************/
void TXObjectFactory::releaseDelegationInterface()
{
  if (m_pDelegation != NULL)
  {
    m_pDelegation->release();
    m_pDelegation = NULL;
  }
}

/***************************************************************************/
/*! Returns the static object of the Factory (Signleton)
*/
/***************************************************************************/
TXObjectFactory& TXObjectFactory::getInstance()
{
  return m_Factory;
}


/***************************************************************************/
/*! Static method to find th class factory method for root object classes
    \param classname : Name of the class
    \param codemodule : Name of the code module
    \param index : Pointer to the Indexvalue of the static list
    \retval POINTER: Pointer to the fnction
    \retval NULL : Function not found
*/
/***************************************************************************/
IXObject::FCreateXObject TXObjectFactory::lookupFactoryFunct(const IString* classname, const IString* codemodule, long* index /*=NULL*/)
{
  IXObject::FCreateXObject pRetval = NULL;
  IListIterator<RClassRegistry>* pIt = m_lstStaticClasses.getIterator();
  bool bFound = false;
  while(!bFound && pIt->hasNextItem())
  {
    RClassRegistry& pItem = pIt->getNextItem();
    if (classname->equalsStr(&pItem.strClassName))
    {
      bFound = true;
      pRetval = pItem.Function;
    }
  }
  m_lstStaticClasses.releaseIterator(pIt);
  
  //Function not found -> try to load code module and try again
  if (!bFound && !codemodule->equals(L""))
  {
    TLibraryManager& LibManager = TLibraryManager::getInstance();
    long lIndex = 0;
    if (LibManager.registerXClass(codemodule, classname, &lIndex) == RET_NOERROR)
    {
      pRetval = m_lstStaticClasses[lIndex].Function;
    }
    if (index!= NULL)
    {
      *index = lIndex;
    }
  }

  return pRetval;
}


/***************************************************************************/
/*! this helper method searches for the classname inside the xml node and
    looks up for the correct function to call.
    \param main_node : Main node of the xml document
    \param docu : XMLDocument
    \param obj : Return parameter of the created object
    \param onlyroot : If this flag is set only the root object will
                      be created. Otherwise all objects will be created 
                      recursivly
    \pre docu and obj must be valid pointers 
    \retval RET_UNREGISTERED_CLASS : Class is not registered
    \retval RET_REQUEST_FAILED : No valid XML
    \retval RET_INTERNAL_ERROR : Internal Error occured
*/
/***************************************************************************/
unsigned long TXObjectFactory::createRootObject(NAMESPACE_Zeus::IXMLNode* main_node, NAMESPACE_Zeus::IXMLDocument* docu, IXObject** obj, bool onlyroot)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  //Read the classname
  TString strClassValue;
  TString strCodeModuleValue;
  TString strAttClassName=L"ClassName";
  TString strAttCodeModule=L"CodeModule";
  main_node->getAttributeValue(&strAttClassName, &strClassValue);
  main_node->getAttributeValue(&strAttCodeModule, &strCodeModuleValue);
  
  IXObject::FCreateXObject pFunction = TXObjectFactory::lookupFactoryFunct(&strClassValue, &strCodeModuleValue);
  
  if (pFunction != NULL)
  {
    ulRetval = pFunction(main_node, docu, obj);
    
    //Create all objects recursivly
    if (!onlyroot && ulRetval == RET_NOERROR)
    {
      this->createSubObjects(main_node, *obj);
    }
  }
  else
  {
    ulRetval = RET_UNREGISTERED_CLASS;
  }
  return ulRetval;
}

/***************************************************************************/
/*! this helper method searches for the classname inside the xml node and
    looks up for the correct function to call. It creates xobjects as sub
    objects of a given xrootobject.
    \param docu : XMLDocument
    \param parent : parent object
    \pre docu and parent must be valid pointers 
    \retval RET_UNREGISTERED_CLASS : Class is not registered
    \retval RET_REQUEST_FAILED : No valid XML
    \retval RET_INTERNAL_ERROR : Internal Error occured
*/
/***************************************************************************/
unsigned long TXObjectFactory::createSubObjects(IXMLNode* parentnode, IXObject* parent)
{
  unsigned long ulRetval = RET_NOERROR;
  
  if(parent->canCreateChildren())
  {
    //-----------------------------------------------------------------------
    //Check if a external reference exists -> load the ext. XML File
    TString strXMLRef;
    TString strAttXMLRef=L"XMLRef";
    parentnode->getAttributeValue(&strAttXMLRef, &strXMLRef);
    if (strXMLRef != L"")
    {
      //////////////////////////////////////////////////////////
      // Recursion over the method loadExtXMLRef possible
      ulRetval = this->loadExtXMLRef(parent, strXMLRef);
      //////////////////////////////////////////////////////////
    }
    
    //-----------------------------------------------------------------------
    // Same XML Tree
    else
    {
      
      //---------------------------------------------------------------------
      //Try to load all children of the node
      IXMLNode* pChildNode = NULL;
      parentnode->resetNodeIterator(NULL);
      while(parentnode->getNextNode(&pChildNode) == RET_NOERROR)
      {
        TString strClassValue;
        TString strAttClassName=L"ClassName";
        pChildNode->getAttributeValue(&strAttClassName, &strClassValue);
        
        //Try to create a sub object 
        if (strClassValue != L"")
        {
          //Reading all important data to create a sub object
          TString strCodeModuleValue;
          TString strAttCodeModule=L"CodeModule";
          pChildNode->getAttributeValue(&strAttCodeModule, &strCodeModuleValue);
          
          IXObject* pObject = NULL;
          IXObject::FCreateXObject pFunction = TXObjectFactory::lookupFactoryFunct(&strClassValue, &strCodeModuleValue);
          if (pFunction != NULL && pFunction(pChildNode, NULL, &pObject) == RET_NOERROR)
          {
            //Will add the child object, without changing the xml structure.
            parent->setChild(pObject);
            
            ////////////////////////////////////////////////////////////////
            //Recursion. It stops when the depth of the xml tree is reached
            this->createSubObjects(pChildNode, pObject);
            ////////////////////////////////////////////////////////////////
            
            pObject->release();
          }
          else
          {
            ///Not a valid or not registered classname
          }
        }
        else
        {
          //Empty classname
        }
        pChildNode->release();
      }
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! This method loads an external xml reference. This is only possible 
    using XLoaderObjects. 
    \param parent : parent object
    \param filename : Name of the xml reference file
    \retval RET_REQUEST_FAILED : Could not read xml or get a xml document
    \retval RET_INTERNAL_ERROR : No XML Parser available
    \retval RET_NOERROR : External reference set
*/
/***************************************************************************/
unsigned long TXObjectFactory::loadExtXMLRef(IXObject* parent, const TString& filename)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  IXLoaderObject* pLoaderObject = NULL;
  if (parent->askForInterface(INTERFACE_IXLoaderObject, (IZUnknown**)&pLoaderObject) == RET_NOERROR)
  {
    IXMLParser* pParser = NULL;
    if (getXMLParser(&pParser) == RET_NOERROR)
    {
      IXMLDocument* pDocument = NULL;
      if (pParser->loadFromFile(&filename, &pDocument) == RET_NOERROR)
      {
        //Create children out of the external reference node
        IXMLNode* pMainNode = NULL;
        if (pDocument->getMainNode(&pMainNode) == RET_NOERROR)
        {
          pLoaderObject->setExtXMLRef(pDocument, pMainNode);
          
          //////////////////////////////////////////////////////////
          //Recursion over the method createSubObjects possible
          ulRetval = this->createSubObjects(pMainNode, parent);
          //////////////////////////////////////////////////////////
          
          pMainNode->release();
        }
        
        pDocument->release();
      }
      pParser->release();
    }
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
    }
    pLoaderObject->release();
  }
  return ulRetval;
}


/***************************************************************************/
/*! Returns the XML parser of the current system
    \param parser : Return parameter of the xml parser
    \retval RET_REQUEST_FAILED : System has no XML Parser
    \retval RET_INVALID_PARAMETER : Invalid return parameter received
    \retval RET_NOERROR : XML Parser returned
*/
/***************************************************************************/
unsigned long TXObjectFactory::getXMLParser(IXMLParser** parser)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (parser == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strModulName = XML_LIBRARY;
    TString strClassName = L"IXMLParser";
    ulRetval = TLibraryManager::getInstance().createObject(&strModulName, &strClassName, (IZUnknown**)parser);
  }
  return ulRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TXObjectFactory);
  INTERFACE_CAST(IXObjectFactory, INTERFACE_IXObjectFactory);
MEMORY_MANAGER_IMPL_END;

