/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XObjectFactory                                                *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObjectFactory.h,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef XObjectFactoryH
#define XObjectFactoryH 

#include <zeusbase/Config/ZeusMacros.hpp> 
#include <zeusbase/System/Interfaces/IXObjectFactory.hpp> 
#include <zeusbase/System/ZObject.h> 
#include <zeusbase/System/ArrayList.hpp> 
#include <zeusbase/System/XRootObject.h> 


BEGIN_NAMESPACE_Zeus

class IXMLParser;

/***************************************************************************/
/*! This is the base class for all XML based objects.
*/
/***************************************************************************/
class TXObjectFactory : public TZObject, public IXObjectFactory
{
  public:        
    void setDelegationInterface(IXObjectFactory* factory);
    void releaseDelegationInterface();
    static TXObjectFactory& getInstance();
    
    //Methods of IXObjectFactory
    virtual unsigned long MQUALIFIER createObjectFromStream(const IString* stream, 
                                                            IXObject** obj, 
                                                            bool onlyroot = true, 
                                                            bool dounfreeze = true);
    virtual unsigned long MQUALIFIER createObjectFromFile(const IString* filename, 
                                                          IXObject** obj, 
                                                          bool onlyroot = true, 
                                                          bool dounfreeze = true);
    virtual unsigned long MQUALIFIER createChildObjects(IXMLNode* parentnode, 
                                                        IXObject* parent, 
                                                        bool dounfreeze = true);
    virtual unsigned long MQUALIFIER addNewChildObjects(IXMLNode* new_node,
                                                        IXObject* parent, 
                                                        IXObject** child, 
                                                        bool dounfreeze = true);
    virtual long MQUALIFIER registerXClass(const IString* classname, IXObject::FCreateXObject funct);
    virtual bool MQUALIFIER unregisterXClass(long handle, const IString* classname);
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
  protected:   
    ///////////////////////////////////////////////////
    // Auto registration of XRootObject
    REG_ROOT_BEGIN(TXRootObject)
    REG_ROOT_ADD(TXRootObject, L"TXRootObject");
    REG_ROOT_END
    
    REG_SUB_BEGIN(TXObject)
    REG_SUB_ADD(TXObject, L"TXObject");
    REG_SUB_END
    
  private:
    
    /**********************************************************************/
    /*! This structure contains the classname and the corresponding 
        function to create a object of this class
    */
    /**********************************************************************/
    struct RClassRegistry
    {
      ///Factory Function
      IXObject::FCreateXObject Function;
      ///Classname
      TString strClassName;
      
      /********************************************************************/
      /*! Creates an empty record
      */
      /********************************************************************/
      RClassRegistry()
      { 
        Function = NULL;
      }
      
      /********************************************************************/
      /*! Creates a record
      */
      /********************************************************************/
      RClassRegistry(TString classname, IXObject::FCreateXObject funct)
      { 
        strClassName = classname;
        Function = funct;
      }
      
      RClassRegistry& operator=(const RClassRegistry& inpar)
      {
        strClassName = inpar.strClassName;
        Function = inpar.Function;
        return *this;
      }
      
      /********************************************************************/
      /*! Compares to RClassFactoryReg objects
      */
      /********************************************************************/
      bool operator==(const RClassRegistry& inpar)
      {
        return (Function == inpar.Function && strClassName == inpar.strClassName);
      }  
    };
    
    TXObjectFactory();
    virtual ~TXObjectFactory();
    
    unsigned long createRootObject(IXMLNode* main_node, NAMESPACE_Zeus::IXMLDocument* docu, IXObject** obj, bool onlyroot);
    unsigned long createSubObjects(IXMLNode* parentnode, IXObject* parent);
    unsigned long getXMLParser(IXMLParser** parser);
    unsigned long loadExtXMLRef(IXObject* parent, const TString& filename);
  
    ///Delegation Interface
    IXObjectFactory* m_pDelegation;
    
    ///Registered class-factory functions for classes
    static TArrayList<RClassRegistry> m_lstStaticClasses;
    ///
    static TXObjectFactory m_Factory;
    
    static IXObject::FCreateXObject lookupFactoryFunct(const IString* classname, const IString* codemodule, long* index =NULL);
}; 

END_NAMESPACE_Zeus
#endif
 
