/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XObjectPath                                                   *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObjectPath.cpp,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/XObjectPath.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/XObjectPathScannException.hpp>


#include <string.h>
#include <wchar.h>

//---------------------------------------------------------------------------

USING_NAMESPACE_Zeus

/******************************************************************************/
/*! Erstellen des MOMPfad Objekts.
*/
/******************************************************************************/
TXObjectPath::TXObjectPath(IXObject* obj) : TZObject(), m_lstTokens(TString())
{
  m_pObject = obj;
  if (m_pObject != NULL)
  {
    m_pObject->addRef();
  }
  m_wcBuffer = NULL;
}


/******************************************************************************/
/*! Zerstoeren des MOMPfad Objekts.
*/
/******************************************************************************/
TXObjectPath::~TXObjectPath()
{
  if (m_wcBuffer != NULL)
  {
    delete [] m_wcBuffer;
    m_wcBuffer = NULL;
  }
  if (m_pObject != NULL)
  {
    m_pObject->release();
    m_pObject = NULL;
  }
}

/******************************************************************************/
/*! Suchen nach einem Modul
    \param path : Pfad des Moduls
    \param object : Rueckgabeparameter
    \retval NULL : Das Modul mit diesem Pfad existiert nicht
    \retval Pointer : Zeiger auf das gefundene Modul
*/
/******************************************************************************/
unsigned long TXObjectPath::getObject(IString* path, IXObject** object)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (path == NULL || object == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *object = NULL;
    
    if (m_wcBuffer != NULL)
    {
      delete [] m_wcBuffer;
      m_wcBuffer = NULL;
    }
  
    if (m_pObject != NULL)
    {
      long lSize = path->getSize()+1;
      m_wcBuffer = new wchar_t[lSize];
      
      wcscpy(m_wcBuffer,path->c_bstr());
      m_lstTokens.flush();
      m_lCounter = 0;
      
      //Startsymbol lesen
      try
      {
        ZLOGGER.indent();
        ZLOGGER.printfln(LOGMODE_DEBUG, "");
        start();
        ulRetval = findObject(object);
        ZLOGGER.unindent();
      }
      catch(TXObjectPathScannException& e)
      {
        TString strPath(path); 
        ZLOGGER.printfln(LOGMODE_DEBUG, "ScannError: The Path %s is invalid.",TString(path).c_str());
        ZLOGGER.printfln(LOGMODE_DEBUG, "  Letter [%c] at Position %d", (wchar_t)e.Letter, m_lCounter);
        ulRetval = RET_REQUEST_FAILED;
      }
      catch(...)
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "Error Occured finding an object");
        ulRetval = RET_REQUEST_FAILED;
      }
    }
  }
  return ulRetval;
}

/******************************************************************************/
/*! Diese Methode sucht nach einem Modul. Dazu werden die gescannten Tokens
    verwendet.
    \retval Pointer : Referenz auf das gefundene Modul
    \retval NULL : Das Modul ist nicht vorhanden
    \pre pModule must be valid
*/
/******************************************************************************/
unsigned long TXObjectPath::findObject(IXObject** object)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  IXObject* pParent = NULL;
  IXObject* pObject = m_pObject;
  pObject->addRef();
  pObject->getParent(&pParent);
  TString strToken = m_lstTokens.getItem();
  bool bBreak = false;
  
  while(strToken != L"" && !bBreak)
  {
    
    //Parent ermitteln
    if (strToken == L"..")
    {
      if (pObject != NULL)
      {
        pObject->release();
      }
      pObject = pParent;
      if (pObject != NULL)
      {
        pObject->addRef();
        pParent->release();
        pParent = NULL;
        pObject->getParent(&pParent);
        ZLOGGER.printfln(LOGMODE_DEBUG, "[..] Parent found.");
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "[..] Parent not found.");
        ulRetval = RET_REQUEST_FAILED;
        bBreak = true;
      }
    }
    
    //Root ermitteln
    else if (strToken == L"//")
    {
      if (pParent != NULL)
      {
        if (pObject != NULL)
        {
          pObject->release();
          pObject = NULL;
        }
        pParent->getRootObject(&pObject);
        pParent->release();
        pParent = NULL;
        if (pObject != NULL)
        {
          pObject->getParent(&pParent);
          ZLOGGER.printfln(LOGMODE_DEBUG, "[//] Root found.");
        }
        else
        {
          ZLOGGER.printfln(LOGMODE_DEBUG, "[//] Root not found.");
        }
      }
      else // pParent == NULL
      {
        if (pObject != NULL)
        {
          pParent = pObject;
          pParent->addRef();
          ZLOGGER.printfln(LOGMODE_DEBUG, "[//] Root found.");
        }
        else
        {
          pParent = NULL;
          ulRetval = RET_REQUEST_FAILED;
          bBreak = true;
          ZLOGGER.printfln(LOGMODE_DEBUG, "[//] Root not found.");
        }
      }
    }

    //Modul suchen
    else
    {
      //Suchen ob es sich um ein Child handelt
      IXObject* pTmpObject = NULL;
      IXObject* pTmpParent;
      if (pObject != NULL)
      {
        pTmpParent = pObject;
        pTmpParent->addRef();
        
        TString strOut = TString(L"[") + strToken + TString(L"]");
        if (pTmpParent->getChildByName(&strToken, &pTmpObject) == RET_NOERROR)
        {
          if (pParent != NULL)
          {
            pParent->release();
            pParent = NULL;
          }  
          pParent = pTmpParent;
          pParent->addRef();
          pObject->release();
          pObject = pTmpObject;
          pObject->addRef();
          pTmpObject->release();
          pTmpObject = NULL;
          strOut = strOut + L" Object found";
          ZLOGGER.printfln(LOGMODE_DEBUG, strOut.c_str());
        }
        else
        {
          ulRetval = RET_REQUEST_FAILED;
          bBreak = true;
          strOut = strOut + L" Object not found";
          ZLOGGER.printfln(LOGMODE_DEBUG, strOut.c_str());
        }
        
        pTmpParent->release();
        pTmpParent = NULL;
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
        bBreak = true;
     }
    }
    strToken = m_lstTokens.getItem();
  }
  
  if (pObject != NULL && !bBreak)
  {
    *object = pObject;
    ulRetval = RET_NOERROR;
    ZLOGGER.printfln(LOGMODE_DEBUG, "Successfull: Object found and returned.");
  }
  else if (pObject != NULL)
  {
    *object = NULL;
    pObject->release();
    pObject = NULL;
    ZLOGGER.printfln(LOGMODE_DEBUG, "Error: Object not found.");
  }
  
  if (pParent != NULL)
  {
    pParent->release();
    pParent = NULL;
  }
  
  return ulRetval;
}


/******************************************************************************/
/*! Start des scanners
*/
/******************************************************************************/
void TXObjectPath::start()
{
  switch(m_wcBuffer[m_lCounter])
  {
    //Punkt
    case '.' :
      m_lCounter++; punktStart();
    break;

    case '/' :
      m_lCounter++; root();
    break;

    case '\0' :
      m_lCounter++; end();
    break;

    default : m_lCounter++; label();
  }
}

/******************************************************************************/
/*! ein Punkt wurde erkannt
*/
/******************************************************************************/
void TXObjectPath::punktStart()
{
  switch(m_wcBuffer[m_lCounter])
  {
    case '.' :
      m_lCounter++; back();
    break;
    
    case '\0' : m_lCounter++; end();
    break;
    
    case '/' :
      m_lCounter++; trennLabel();
    break;
        
    default: error();
  }
}

/******************************************************************************/
/*! ein Punkt wurde erkannt
*/
/******************************************************************************/
void TXObjectPath::punkt()
{
  switch(m_wcBuffer[m_lCounter])
  {
    case '.' :
      m_lCounter++; back();
    break;
        
    default: error();
  }
}

/******************************************************************************/
/*! nach zwei Punkte gelangt der Scanner in den Back-State
*/
/******************************************************************************/
void TXObjectPath::back()
{
  m_lstTokens.apendItem("..");
  ZLOGGER.printfln(LOGMODE_DEBUG, "Token [..] found");
  switch(m_wcBuffer[m_lCounter])
  {
    case '/' : m_lCounter++; trennBack();
    break;

    case '\0' : m_lCounter++; end();
    break;

    case '.' : error();
    break;

    default : m_lCounter++; label();
  }

}


/******************************************************************************/
/*! zweites Trennzeichen repraesentiert das Root
*/
/******************************************************************************/
void TXObjectPath::root()
{
  m_lstTokens.apendItem("//");
  ZLOGGER.printfln(LOGMODE_DEBUG, "Token [//] found");
  switch(m_wcBuffer[m_lCounter])
  {
    case '\0' : m_lCounter++; end();
    break;

    case '/' : error();
    break;

    case '.' : error();
    break;

    default : m_lCounter++; label();
  }
}

/******************************************************************************/
/*! ein Bezeichner wird eingelesen
*/
/******************************************************************************/
void TXObjectPath::label()
{
  long startAt = m_lCounter-1;

  //Suchen bis ende des Bezeichners
  while(m_wcBuffer[m_lCounter] != '/' && m_wcBuffer[m_lCounter] != '\0')
  {
    m_lCounter++;
  }

  //Bezeichner in die Liste der Tokens aufnehmen
  wchar_t* newlabel= new wchar_t[m_lCounter-startAt+1];
  newlabel = wcsncpy(newlabel, &(m_wcBuffer[startAt]),(m_lCounter-startAt));
  newlabel[m_lCounter-startAt] = '\0';
  m_lstTokens.apendItem(newlabel);
  TString strOut = TString(L"Token [") + TString(newlabel) + L"] found";
  ZLOGGER.printfln(LOGMODE_DEBUG, strOut.c_str());

  delete [] newlabel;

  if(m_wcBuffer[m_lCounter] == '/')
  {
    m_lCounter++;
    trennLabel();
  }
  else
  {
    m_lCounter++;
    end();
  }
}


/******************************************************************************/
/*! Das Trennzeichen nach dem Back-State
*/
/******************************************************************************/
void TXObjectPath::trennBack()
{
  switch(m_wcBuffer[m_lCounter])
  {
    case '.' : m_lCounter++; punkt();
    break;

    case '\0' : error();
    break;

    case '/' : error();
    break;

    default : m_lCounter++; label();
  }
}

/******************************************************************************/
/*! das Trennzeichen des Bezeichners
*/
/******************************************************************************/
void TXObjectPath::trennLabel()
{
  switch(m_wcBuffer[m_lCounter])
  {
    case '.' : error();
    break;

    case '/' : error();
    break;

    case '\0' : error();
    break;

    default : m_lCounter++; label();
  }
}


/******************************************************************************/
/*! Ende der Zeichenkette
*/
/******************************************************************************/
void TXObjectPath::end()
{
  return;
}

/******************************************************************************/
/*! Fehler beim Scannen
*/
/******************************************************************************/
void TXObjectPath::error()
{
  throw TXObjectPathScannException(m_wcBuffer[m_lCounter]);
}


