/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XObjectPath                                                   *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObjectPath.h,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef XObjectPathH
#define XObjectPathH

#include <zeusbase/System/Interfaces/IXObject.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/Queue.hpp>
#include <zeusbase/System/String.h>

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! Die Klasse TMOMPath dient als Navigationsklasse zum suchen von Module
    mittels des Pfad-Syntax von StuderWIN. Es kann absoltut (vom Root aus)
    gesucht werden, oder aber relativ zu dem im Konstruktor angegebenen Modul.
    Dabei gilt:                                                             <BR>
    [/] ist die Bezeichnung fuer Root                                      <BR>
    [..] ist die Bezeichnung fuer den Parent                                <BR>
    [Bezeichner] bezeichne das Modul                                        <BR>
    [/] ist das Trennzeichen zwischen den Bezeichner und [..] bzw [//]      <BR>
                                                                            <BR>
    Root::= "/"                                                            <BR>
    Parent::= ".."                                                          <BR>
    Bezeichner:= ASCII-Zeichen ohne "." und "/"                             <BR>
    Bezeichners:= Bezeichners "/" | Bezeichner                              <BR>
    Parents::= Parents "/" | Parent                                         <BR>
    AbsString::= Root Bezeichners | Root                                    <BR>
    RelString::= Bezeichners | Parents | Parents Bezeichners                <BR>
                                                                            <BR>
    Pfad:== AbsString | RelString                                           <BR>
*/
/******************************************************************************/
class TXObjectPath : public TZObject
{
  public:
    TXObjectPath(IXObject* obj);
    virtual ~TXObjectPath();
    unsigned long getObject(IString* path, IXObject** object);

  private:
    ///Startmodul Referenz
    IXObject* m_pObject;
    ///Token Queue
    TQueue<TString> m_lstTokens;
    ///Counter
    long m_lCounter;
    ///String Buffer
    wchar_t* m_wcBuffer;

    unsigned long findObject(IXObject** object);

    void start();
    void punkt();
    void punktStart();
    void back();
    void root();
    void end();
    void error();
    void trennLabel();
    void trennBack();
    void label();
};


END_NAMESPACE_Zeus

#endif
