 
/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XMLObject                                                     *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XRootObject.cpp,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/XRootObject.h>
#include <zeusbase/System/XObjectFactory.h>


USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates an empty root object. This constructor is protected
*/
/***************************************************************************/
TXRootObject::TXRootObject() : TXObject()
{
  m_pDocument = NULL;
}


/***************************************************************************/
/*! Creates the xml root object
    \param node : XML Node witch represents the root node
    \param docu : XML Document
*/
/***************************************************************************/
TXRootObject::TXRootObject(IXMLNode* node, NAMESPACE_Zeus::IXMLDocument* docu) : TXObject(node)
{
  m_pDocument = docu;
  if (m_pDocument != NULL)
  {
    m_pDocument->addRef();
  }
}


/***************************************************************************/
/*! Creates the xml root object
    \param node : XML node
*/
/***************************************************************************/
TXRootObject::TXRootObject(IXMLNode* node) : TXObject(node)
{
  m_pDocument = NULL;
}


/***************************************************************************/
/*! Destroyes the Object
*/
/***************************************************************************/
TXRootObject::~TXRootObject()
{
  if (m_pDocument != NULL)
  { 
    m_pDocument->release();
    m_pDocument = NULL;
  } 
}

/***************************************************************************/
/*! Get infomation abaout the xmlobject
*/
/***************************************************************************/
TString TXRootObject::getXMLStream() const
{
  TString strRetval;
  if (m_pDocument != NULL)
  {
    m_pDocument->getStream(&strRetval);
  }
  return strRetval;
}

/***************************************************************************/
/*! Stores the data tree of XObjects back into a file.
    \return See IXMLDocument for error codes
*/
/***************************************************************************/
unsigned long TXRootObject::save()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDocument != NULL)
  {
    ulRetval = m_pDocument->save();
  }
  return ulRetval;
}

/***************************************************************************/
/*! Stores the data tree of XObjects back into a file.
    \param filename : Name of the file
    \return See IXMLDocument for error codes
*/
/***************************************************************************/
unsigned long TXRootObject::saveAs(TString filename)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDocument != NULL)
  {
    ulRetval = m_pDocument->saveAs(&filename);
  }
  return ulRetval;
}
