/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Object                                                   *
 * Package : System                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZObject.cpp,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/String.h>

USING_NAMESPACE_Zeus


/***************************************************************************/
/*! Erstellen des Objekts
*/
/***************************************************************************/
TZObject::TZObject()
{
  m_ulID = (unsigned long)this;
  m_ulRefCounter = 1;
}
  
/***************************************************************************/
/*! Erstellen des Objekts
*/
/***************************************************************************/
TZObject::TZObject(TString name)
{
  m_ulID = (unsigned long)this;
  m_ulRefCounter = 1;
  m_strName = name;
}

/***************************************************************************/
/*! Destroyes the Object
*/
/***************************************************************************/
TZObject::~TZObject()
{}

/***************************************************************************/
/*! Returns the name of the object
*/
/***************************************************************************/
TString TZObject::getObjName() const
{
  return m_strName;
}


/***************************************************************************/
/*! Sets the name of the object
*/
/***************************************************************************/
void TZObject::setObjName(const TString& name)
{
  m_strName = name;
}

/***************************************************************************/
/*! Get hash infomation about the object
*/
/***************************************************************************/
unsigned long TZObject::getHash() const
{
  unsigned long ulRetval = (unsigned long)this;
  return ulRetval;
}

/***************************************************************************/
/*! Returns the reference counter of this object
*/
/***************************************************************************/
unsigned long TZObject::getRefCounter() const
{
  return m_ulRefCounter;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
unsigned long MQUALIFIER TZObject::askForInterface(long interface_id, IZUnknown** iface)
{
  unsigned long ulRetval = RET_NOERROR;
  if (iface == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    switch(interface_id)
    {
      default:
        ulRetval = RET_UNKNOWN_INTERFACE;
    }
  }

  return ulRetval;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
unsigned long MQUALIFIER TZObject::addRef()
{
  ++m_ulRefCounter;
  return RET_NOERROR;
}

/***************************************************************************/
/*! Siehe IZUnknown
*/
/***************************************************************************/
unsigned long MQUALIFIER TZObject::release()
{
  --m_ulRefCounter;
  if (m_ulRefCounter <= 0)
  {
    delete this;
  }
  return RET_NOERROR;
}

