/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ZObjectFactory                                                *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:30:12 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZObjectFactory.h,v $
 *   Revision 1.2  2005/11/16 11:30:12  bhadorn
 *   All input parameter are [const]
 *
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

 
#ifndef ZObjectFactoryH
#define ZObjectFactoryH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/StringMap.hpp>
#include <zeusbase/System/Interfaces/IZObjectFactory.hpp>


BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! This Factory creates serializable objects. All object to create must be 
    registered before.
*/
/**************************************************************************/
class TZObjectFactory : public TZObject, public IZObjectFactory
{
  
  public:  
    void setDelegationInterface(IZObjectFactory* factory);
    void releaseDelegationInterface();
    static TZObjectFactory& getInstance();
    
    //Methods of IZObjectFactory
    virtual unsigned long MQUALIFIER createObjectFromStream(const IByteArray* stream, ISerializable** obj);
    virtual unsigned long MQUALIFIER createObjectByClassName(const IString* classname, const IString* codemodule, IZUnknown** obj);
    virtual unsigned long MQUALIFIER registerClass(const IString* name, IZObjectFactory::FCreateZObject function);
    virtual unsigned long MQUALIFIER unregisterClass(const IString* name, unsigned long handle);
  
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    TZObjectFactory();
    virtual ~TZObjectFactory();
    
    ///Singleton Object
    static TZObjectFactory m_Factory;
  
  private:
    ///Factory map
    TStringMap<IZObjectFactory::FCreateZObject> m_mapClasses;
    ///Delegation interface
    IZObjectFactory* m_pDelegation;
};

END_NAMESPACE_Zeus
#endif
