/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ZVariant                                                      *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZVariant.cpp,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/System/ZVariant.h>

USING_NAMESPACE_Zeus
 
SERIAL_CONSTRUCTOR(TZVariant);

/***************************************************************************/
/*! Creates an empty variant
*/
/***************************************************************************/
TZVariant::TZVariant() : TZObject()
{
  m_bAllocated = false;
  m_ArgType = varEmpty;
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a byte value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(char value) : TZObject()
{
  m_bAllocated = false;
  setByte(value);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a long value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(long value) : TZObject()
{
  m_bAllocated = false;
  setLong(value);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a long value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(int value) : TZObject()
{
  m_bAllocated = false;
  setLong(value);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a double value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(double value) : TZObject()
{
  m_bAllocated = false;
  setDouble(value);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a string value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString value) : TZObject()
{
  m_bAllocated = false;
  setString(&value);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a string value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(char* value) : TZObject()
{
  m_bAllocated = false;
  TString strValue(value);
  setString(&strValue);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a string value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(wchar_t* value) : TZObject()
{
  m_bAllocated = false;
  TString strValue(value);
  setString(&strValue);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a string value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(const IString* value)
{
  m_bAllocated = false;
  setString(value);
}


/***************************************************************************/
/*! Creates an anonymous variant wrapping an bytearray
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(const TByteArray& value) : TZObject()
{
  m_bAllocated = false;
  setByteArray(&value);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping an object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(ISerializable* value) : TZObject()
{
  m_bAllocated = false;
  setObject(value);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping an object list
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(IList<ISerializable*>* value) : TZObject()
{
  m_bAllocated = false;
  setObjectList(value);
}

/***************************************************************************/
/*! Creates a named variant wrapping a byte value
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, char value) : TZObject(name)
{
  m_bAllocated = false;
  setByte(value);
}

/***************************************************************************/
/*! Creates a named variant wrapping a long value
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, long value) : TZObject(name)
{
  m_bAllocated = false;
  setLong(value);
}

/***************************************************************************/
/*! Creates a named variant wrapping a double value
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, double value) : TZObject(name)
{
  m_bAllocated = false;
  setDouble(value);
}

/***************************************************************************/
/*! Creates a named variant wrapping a string value
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, TString value) : TZObject(name)
{
  m_bAllocated = false;
  setString(&value);
}

/***************************************************************************/
/*! Creates a named variant wrapping a string value
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, char* value) : TZObject(name)
{
  m_bAllocated = false;
  TString strValue(value);
  setString(&strValue);
}

/***************************************************************************/
/*! Creates a named variant wrapping a string value
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, wchar_t* value) : TZObject(name)
{
  m_bAllocated = false;
  TString strValue(value);
  setString(&strValue);
}

/***************************************************************************/
/*! Creates an anonymous variant wrapping a string value
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, const IString* value) : TZObject(name)
{
  m_bAllocated = false;
  setString(value);
}

/***************************************************************************/
/*! Creates a named variant wrapping a byte array
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, const TByteArray& value) : TZObject(name)
{
  m_bAllocated = false;
  setByteArray(&value);
}

/***************************************************************************/
/*! Creates a named variant wrapping an object
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, ISerializable* value) : TZObject(name)
{
  m_bAllocated = false;
  setObject(value);
}

/***************************************************************************/
/*! Creates a named variant wrapping an object list
    \param name : Name of the variant object
    \param value : Value to wrapp
*/
/***************************************************************************/
TZVariant::TZVariant(TString name, IList<ISerializable*>* value) : TZObject(name)
{
  m_bAllocated = false;
  setObjectList(value);
}
  

/***************************************************************************/
/*! Copy constructor
    \param inpar : variant to copy
*/
/***************************************************************************/
TZVariant::TZVariant(const IZVariant* inpar)
{
  m_bAllocated = false;
  this->assign(inpar);
}


/***************************************************************************/
/*! Destroys the variant
*/
/***************************************************************************/
TZVariant::~TZVariant()
{
  releaseVariant();
}

/***************************************************************************/
/*! release the variant object
*/
/***************************************************************************/
void TZVariant::releaseVariant()
{
  if (m_bAllocated)
  {
    switch(m_ArgType)
    {
      //Release object list
      case varObjectList:
      {
        while(m_Data.pObjectList->getCount() > 0)
        {
          if (m_Data.pObjectList->getItem(0) != NULL)
          {
            m_Data.pObjectList->getItem(0)->release();
          }
          m_Data.pObjectList->deleteItem(0);
        }
        m_Data.pObjectList->release();
      }
      break;
      
      //Release object
      case varObject :
      {
        m_Data.pObjectData->release();
      }
      break;
      
      //Release byte array
      case varByteArray :
      {
        m_Data.pArrayData->release();
      }
      break;
      
      //Release string
      case varString :
      {
        m_Data.pStrData->release();
      }
      break;
      
      default:
      break;
    }
    m_bAllocated = false;
    m_ArgType = varEmpty;
  }
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
TZVariantType MQUALIFIER TZVariant::getType() const
{
  return m_ArgType;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getVariantName(IString* name) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strName = getObjName();
    name->assignStr(&strName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getByte(char* ret) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ret == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_ArgType == varByte)
  {
    *ret = m_Data.cData;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getLong(long* ret) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ret == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_ArgType == varLong)
  {
    *ret = m_Data.lData;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getDouble(double* ret) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ret == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_ArgType == varDouble)
  {
    *ret = m_Data.dData;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getString(IString* ret) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ret == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_ArgType == varString)
  {
    ret->assignStr(m_Data.pStrData);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getByteArray(IByteArray* ret) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ret == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_ArgType == varByteArray)
  {
    ret->setArray(m_Data.pArrayData->getArray(), m_Data.pArrayData->getCount());
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getObject(ISerializable** ret) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ret == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_ArgType == varObject && m_bAllocated)
  {
    *ret = m_Data.pObjectData;
    (*ret)->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::getObjectList(IList<ISerializable*>* ret) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (ret == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_ArgType == varObjectList && m_bAllocated)
  {
    IListIterator<ISerializable*>* pIt = m_Data.pObjectList->getIterator();
    while(pIt->hasNextItem())
    {
      ISerializable* pObject = pIt->getNextItem();
      if (pObject != NULL)
      {
        pObject->addRef();
      }
      ret->add(pObject);
    }
    m_Data.pObjectList->releaseIterator(pIt);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}
    
/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setVariantName(IString* name)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strName(name);
    this->setObjName(strName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setByte(char val)
{
  releaseVariant();
  m_ArgType = varByte;
  m_Data.cData = val;
  m_bAllocated = false;
  return RET_NOERROR;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setLong(long val)
{
  releaseVariant();
  m_ArgType = varLong;
  m_Data.lData = val;
  m_bAllocated = false;
  return RET_NOERROR;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setDouble(double val)
{
  releaseVariant();
  m_ArgType = varDouble;
  m_Data.dData = val;
  m_bAllocated = false;
  return RET_NOERROR;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setString(const IString* val)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  releaseVariant();
  if (val != NULL)
  {
    m_bAllocated = true;
    m_Data.pStrData = new TString(val);
    m_ArgType = varString;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setByteArray(const IByteArray* val)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  releaseVariant();
  if (val != NULL)
  {
    m_bAllocated = true;
    m_Data.pArrayData = new TByteArray();
    m_Data.pArrayData->setArray(val->getArray(), val->getCount());
    m_ArgType = varByteArray;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setObject(ISerializable* val)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  releaseVariant();
  if (val != NULL)
  {
    m_bAllocated = true;
    m_Data.pObjectData = val;
    m_Data.pObjectData->addRef();
    m_ArgType = varObject;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::setObjectList(IList<ISerializable*>* val)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  releaseVariant();
  if(val != NULL)
  {
    m_bAllocated = true;
    m_Data.pObjectList = new TSingleLinkedList<ISerializable*>();
    IListIterator<ISerializable*>* pIt = val->getIterator();
    while(pIt->hasNextItem())
    {
      ISerializable* pObject = pIt->getNextItem();
      if (pObject != NULL)
      {
        pObject->addRef();
      }
      m_Data.pObjectList->add(pObject);
    }
    val->releaseIterator(pIt);
    m_ArgType = varObjectList;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
unsigned long MQUALIFIER TZVariant::assign(const IZVariant* inpar)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if(inpar != NULL)
  {
    switch(inpar->getType())
    {
      //----------------------------------------------------------
      case varByte:
      { 
        char cByte;
        if (inpar->getByte(&cByte) == RET_NOERROR)
        {
          ulRetval = this->setByte(cByte);
        }
      }  
      break;
    
      //----------------------------------------------------------
      case varLong:
      { 
        long lLong;
        if (inpar->getLong(&lLong) == RET_NOERROR)
        {
          ulRetval = this->setLong(lLong);
        }
      }  
      break;
      
      //----------------------------------------------------------
      case varDouble:
      { 
        double dDouble;
        if (inpar->getDouble(&dDouble) == RET_NOERROR)
        {
          ulRetval = this->setDouble(dDouble);
        }
      }  
      break;
      
      //----------------------------------------------------------
      case varString:
      { 
        TString strValue;
        if (inpar->getString(&strValue) == RET_NOERROR)
        {
          ulRetval = this->setString(&strValue);
        }
      }  
      break;
      
      //----------------------------------------------------------
      case varByteArray:
      { 
        TByteArray aArray;
        if (inpar->getByteArray(&aArray) == RET_NOERROR)
        {
          ulRetval = this->setByteArray(&aArray);
        }
      }  
      break;
      
      //----------------------------------------------------------
      case varObject:
      { 
        ISerializable* pObject;
        if (inpar->getObject(&pObject) == RET_NOERROR)
        {
          ulRetval = this->setObject(pObject);
          pObject->release();
        }
      }  
      break;
      
      //----------------------------------------------------------
      case varObjectList:
      { 
        TSingleLinkedList<ISerializable*> lstObjects;
        if (inpar->getObjectList(&lstObjects) == RET_NOERROR)
        {
          ulRetval = this->setObjectList(&lstObjects);
          while(lstObjects.getCount() > 0)
          {
            lstObjects[0]->release();
            lstObjects.deleteItem(0);
          }
        }
      }  
      break;

      //----------------------------------------------------------
      default:
        ulRetval = RET_REQUEST_FAILED;;
      break;
    }
    //Set the variant name
    if (ulRetval == RET_NOERROR)
    {
      TString strName;
      inpar->getVariantName(&strName);
      this->setVariantName(&strName);
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IZVariant
*/
/***************************************************************************/
bool MQUALIFIER TZVariant::equals(const IZVariant* inpar) const
{
  bool bRetval = false;
  if (inpar != NULL)
  {
    TString strName;
    inpar->getVariantName(&strName);
    if (this->m_ArgType == inpar->getType() && strName == this->getObjName())
    {
      switch(m_ArgType)
      {
        //-------------------------------------------------------------
        //Check primitive datatypes
        case varByte:
        {
          char cByte;
          if (inpar->getByte(&cByte) == RET_NOERROR)
          {
            bRetval = (this->m_Data.cData == cByte);
          }
        }
        break;
        
        //-------------------------------------------------------------
        case varLong:
        {
          long lLong1;
          if (inpar->getLong(&lLong1) == RET_NOERROR)
          {
            bRetval = (this->m_Data.lData == lLong1);
          }
        }
        break;
        
        //-------------------------------------------------------------
        case varDouble:
        {
          double dDouble1;
          if (inpar->getDouble(&dDouble1) == RET_NOERROR)
          {
            bRetval = (this->m_Data.dData == dDouble1);
          }
        }
        break;
        
        //-------------------------------------------------------------
        //Check string data type
        case varString :
        {
          TString strData;
          if (inpar->getString(&strData) == RET_NOERROR)
          {
            bRetval = this->m_Data.pStrData->equalsStr(&strData);
          }
        }
        break;
        
        //-------------------------------------------------------------
        //Check byte array
        case varByteArray :
        {  
          TByteArray aArray;
          if (inpar->getByteArray(&aArray) == RET_NOERROR)
          {
            bRetval = this->m_Data.pArrayData->equals(&aArray);
          }
        }
        break;
        
        //-------------------------------------------------------------
        //Check object
        case varObject :
        {
          ISerializable* pObject;
          if (inpar->getObject(&pObject) == RET_NOERROR)
          {
            bRetval = (this->m_Data.pObjectData->getObjectID() == pObject->getObjectID());
            pObject->release();
          }
        }
        break;
        
        //-------------------------------------------------------------
        //Check object
        case varObjectList :
        {
          TSingleLinkedList<ISerializable*> lstObjects1;
          if (inpar->getObjectList(&lstObjects1) == RET_NOERROR)
          {
            TSingleLinkedList<ISerializable*>& lstObjects2 = *this->m_Data.pObjectList;
            long lCount = lstObjects2.getCount();
            bRetval = true;
            for(int i = 0; i < lCount && bRetval; i++)
            {
              ISerializable* pObj1 = lstObjects1[i];
              ISerializable* pObj2 = lstObjects2.getItemCopy(i);
              
              if (pObj1 != NULL && pObj2 != NULL)
              {
                bRetval &= (pObj1->getObjectID() == pObj2->getObjectID());
              }
              else
              {
                bRetval = false;
              }
            }
            while(lstObjects1.getCount() > 0)
            {
              lstObjects1[0]->release();
              lstObjects1.deleteItem(0);
            }
          }
        }
        break;
        
        //-------------------------------------------------------------
        case varEmpty:
          bRetval = true;
        break;
        
        default:
        break;
      }
    }
  }
  return bRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TZVariant);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
  INTERFACE_CAST(IZVariant, INTERFACE_IZVariant);
MEMORY_MANAGER_IMPL_END;
