/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ZVariant                                                      *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:21:24 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZVariant.h,v $
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef ZVariantH
#define ZVariantH
 

#include <zeusbase/System/Interfaces/IZVariant.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/StringMap.hpp>

BEGIN_NAMESPACE_Zeus


/***************************************************************************/
/*! Variant implementation of Zeus-Framework
*/
/***************************************************************************/
class TZVariant : public TZObject , public IZVariant
{
  public :
    TZVariant();
    TZVariant(char value);
    TZVariant(int value);
    TZVariant(long value);
    TZVariant(double value);
    TZVariant(char* value);
    TZVariant(wchar_t* value);
    TZVariant(TString value);
    TZVariant(const IString* value);
    TZVariant(const TByteArray& value);
    TZVariant(ISerializable* value);
    TZVariant(IList<ISerializable*>* value);
    TZVariant(TString name, char value);
    TZVariant(TString name, long value);
    TZVariant(TString name, double value);
    TZVariant(TString name, char* value);
    TZVariant(TString name, wchar_t* value);
    TZVariant(TString name, TString value);
    TZVariant(TString name, const IString* value);
    TZVariant(TString name, const TByteArray& value);
    TZVariant(TString name, ISerializable* value);
    TZVariant(TString name, IList<ISerializable*>* value);
//     TZVariant(const TZVariant& inpar);
    TZVariant(const IZVariant* inpar);
    
    //Methods of IZVariant
    virtual TZVariantType MQUALIFIER getType() const;
    virtual unsigned long MQUALIFIER getVariantName(IString* name) const;
    virtual unsigned long MQUALIFIER getByte(char* ret) const;
    virtual unsigned long MQUALIFIER getLong(long* ret) const;
    virtual unsigned long MQUALIFIER getDouble(double* ret) const;
    virtual unsigned long MQUALIFIER getString(IString* ret) const;
    virtual unsigned long MQUALIFIER getByteArray(IByteArray* ret) const;
    virtual unsigned long MQUALIFIER getObject(ISerializable** ret) const;
    virtual unsigned long MQUALIFIER getObjectList(IList<ISerializable*>* ret) const;
    
    virtual unsigned long MQUALIFIER setVariantName(IString* name);
    virtual unsigned long MQUALIFIER setByte(char val);
    virtual unsigned long MQUALIFIER setLong(long val);
    virtual unsigned long MQUALIFIER setDouble(double val);
    virtual unsigned long MQUALIFIER setString(const IString* val);
    virtual unsigned long MQUALIFIER setByteArray(const IByteArray* val);
    virtual unsigned long MQUALIFIER setObject(ISerializable* val);
    virtual unsigned long MQUALIFIER setObjectList(IList<ISerializable*>* val);
    
    virtual unsigned long MQUALIFIER assign(const IZVariant* inpar);
    virtual bool MQUALIFIER equals(const IZVariant* inpar) const;
  
/*    bool operator==(const TZVariant& inpar) const;
    bool operator!=(const TZVariant& inpar) const;
    TZVariant& operator=(const TZVariant& inpar);*/
    
    //Methods of ISterializable
    SERIAL_START(TZVariant, L"TZVariant")
      SERIAL_STRING_SET_GET(setObjName, getObjName)
      SERIAL_LONG_ENUM(m_ArgType, TZVariantType)
      TZVariantType vtType;
      if (lMode == 2)
      {
        vtType = obj_old->m_ArgType;
      }
      else
      {
        vtType = obj_new->m_ArgType;
        obj_new->m_bAllocated = false;
      }
      switch(vtType)
      {
        case varByte :
          SERIAL_BYTE(m_Data.cData);
        break;
        
        case varLong :
          SERIAL_LONG(m_Data.lData);
        break;
      
        case varDouble :
          SERIAL_DOUBLE(m_Data.dData);
        break;
        
        case varString :
          if (ulRetval == RET_NOERROR)
          {
            if (lMode == 2)
            {
              stream_out->writeByte(SERIAL_STRING_ID);
              stream_out->writeString(obj_old->m_Data.pStrData);
            }
            else if (CHECK_STRING(stream_in))
            {
              obj_new->m_Data.pStrData = new TString();
              stream_in->readString(obj_new->m_Data.pStrData);
              obj_new->m_bAllocated = true;
            }
            else
            {
              ulRetval = RET_REQUEST_FAILED;
            }
          }
        break;
        
        case varByteArray :
          if (ulRetval == RET_NOERROR)
          {
            if (lMode == 2)
            {
              stream_out->writeByte(SERIAL_ARRAY_ID);
              stream_out->writeArray(obj_old->m_Data.pArrayData);
            }
            else if (CHECK_ARRAY(stream_in))
            {
              obj_new->m_Data.pArrayData = new TByteArray();
              stream_in->readArray(obj_new->m_Data.pArrayData);
              obj_new->m_bAllocated = true;
            }
            else
            {
              ulRetval = RET_REQUEST_FAILED;
            }
          }
        break;
        
        case varObject :
          if (ulRetval == RET_NOERROR)
          {
            if (lMode == 2)
            {
              stream_out->writeByte(SERIAL_OBJECT_ID);
              TByteArray aArray;
              if (obj_old->m_bAllocated)
              {
                obj_old->m_Data.pObjectData->serialize(&aArray);
              }
              stream_out->writeArray(&aArray);
            }
            else if (CHECK_OBJECT(stream_in))
            { 
              TByteArray aArray; 
              stream_in->readArray(&aArray);
              if (TZObjectFactory::getInstance().createObjectFromStream(&aArray, &obj_new->m_Data.pObjectData) == RET_NOERROR)
              {
                obj_new->m_bAllocated = true;
              }
              else
              {
                obj_new->m_Data.pObjectData = NULL;
                obj_new->m_ArgType = varEmpty;
                obj_new->m_bAllocated = false;
              } 
            }
            else
            {
              ulRetval = RET_REQUEST_FAILED;
            }
          }
        break;
        
        case varObjectList :
          if (ulRetval == RET_NOERROR)
          {
            if (lMode == 2)
            {
              long lCount = obj_old->m_Data.pObjectList->getCount();
              stream_out->writeByte(SERIAL_OBJECTLIST_ID);
              stream_out->writeLong(lCount);
              for(int i = 0; i < lCount; i++)
              { 
                TByteArray aArray;
                ISerializable* pObject = obj_old->m_Data.pObjectList->getItem(i);
                if (pObject != NULL)
                {
                  pObject->serialize(&aArray);
                }
                stream_out->writeArray(&aArray);
              }
            }
            else if (CHECK_OBJECTLIST(stream_in))
            {
              long lCount = stream_in->readLong();
              obj_new->m_Data.pObjectList = new TSingleLinkedList<ISerializable*>();
              for(int i = 0; i < lCount; i++)
              {
                TByteArray aArray;
                stream_in->readArray(&aArray);
                ISerializable* pObject = NULL;
                if (TZObjectFactory::getInstance().createObjectFromStream(&aArray, &pObject) == RET_NOERROR)
                {
                  obj_new->m_Data.pObjectList->add(pObject);
                }
                else
                {
                  obj_new->m_Data.pObjectList->add(NULL);
                }
              }
              obj_new->m_bAllocated = true;
            }
            else
            { 
              ulRetval = RET_REQUEST_FAILED;
            } 
          }
        break;
        
        default:
        break;
      }
    SERIAL_END
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected :
    virtual ~TZVariant();
    
    /**************************************************************/
    /* This union is private and defines the storage of the 
       variant.
    */
    /**************************************************************/
    union TZVariantData
    {
      char cData;
      long lData;
      double dData;
      IString* pStrData;
      IByteArray* pArrayData;
      ISerializable* pObjectData;
      TSingleLinkedList<ISerializable*>* pObjectList;
    };
    
    ///Type of argument
    TZVariantType m_ArgType;
    ///
    TZVariantData m_Data;
    ///Allocated memory flag
    bool m_bAllocated;
    
  private :
    void releaseVariant();
};
 
END_NAMESPACE_Zeus

#endif
