/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractCell                                                  *
 * Package : Test Module for Cell Computing                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:59:57 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractCell.h,v $
 *   Revision 1.2  2005/11/25 09:59:57  bhadorn
 *   Eliminated compiler warnings
 *
 *   Revision 1.1  2005/10/28 12:22:23  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef AbstractCellH
#define AbstractCellH

#include <zeusbase/System/XObject.h>
#include <zeusbase/System/Thread.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/CellModel/Interfaces/ICellKernel.hpp>
#include <zeusbase/Messaging/Interfaces/IBinaryMessage.hpp>
#include <zeusbase/Messaging/Interfaces/IXMLMessage.hpp>
#include <zeusbase/Messaging/Interfaces/ISystemMessage.hpp>
#include <zeusbase/Messaging/Interfaces/IObjectMessage.hpp>

BEGIN_NAMESPACE_Zeus

/**************************************************************************/
/*! This class defines an abstract cell with some basic behavoir of cells
*/
/**************************************************************************/
class TAbstractCell : public TXObject, public ICellKernel 
{
  public:
    TAbstractCell(IXMLNode* node);
    
    bool isTerminated();
    
    //Interface ICellKernel
    virtual void MQUALIFIER initialize(ICellComm* comm);
    virtual void MQUALIFIER uninitialize();
    virtual bool MQUALIFIER isInitialized();

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
    
  protected:  
    virtual ~TAbstractCell();
    
    class TCellLife;
    ///Life Object
    TCellLife* m_pLifeObj;
    ///Communication Interface
    ICellComm* m_pCommunication;
    ///Sleeping time
    long m_lSleepTime;
    
    virtual void onInitialLife() {}
    virtual void onEndLife() {}
    virtual void sendLifeSignal();
    virtual void dispatchMessage(IMessage* msg, unsigned long long timer);
  
    virtual void performSystemMessage(ISystemMessage* msg, unsigned long long timer);
    virtual void performBinaryMessage(IBinaryMessage* msg, unsigned long long timer);
    virtual void performXMLMessage(IXMLMessage* msg, unsigned long long timer);
    virtual void performObjectMessage(IObjectMessage* msg, unsigned long long timer);
  
  private:
    
    /**********************************************************************/
    /*! This inner class is the life object of a cell. It extends a
        thread.
    */
    /**********************************************************************/
    class TCellLife : public TThread
    {
      public:
        /******************************************************************/
        /*! Creates a cell life object
            \param cell : Cell Reference
            \param sleeptime : Sleep time of the thread
        */
        /******************************************************************/
        TCellLife(TAbstractCell* cell, long sleeptime) : TThread()
        {
          m_bTerminated = false;
          m_pCell = cell;
          m_lSleepTime = sleeptime;
        }
        //---------------------------------------------------------------------

        /******************************************************************/
        /*! Terminates the thread
        */
        /******************************************************************/
        void doTerminate()
        {
          m_bTerminated = true;
          this->kill();
        }
        //---------------------------------------------------------------------
        
        /******************************************************************/
        /*! Checks if the life object has been killed or the flag to terminate is
            set.
            \retval true : Thread should terminate
            \retval false: Running
        */
        /******************************************************************/
        bool isTerminated()
        {
          return m_bTerminated;
        }
        
        /******************************************************************/
        /*! Method is call from the thread. see TThread
        */
        /******************************************************************/
        void execute()
        {
          m_pCell->onInitialLife();
          try
          {
            while(!m_bTerminated)
            {
              TTime::usleep(m_lSleepTime);
              if (!m_bTerminated)
              {
                m_pCell->sendLifeSignal();
              }
            }
          }
          catch(...)
          {}
          
          m_pCell->onEndLife();
        }
        //---------------------------------------------------------------------

      private:
        ///Flag signals the threads end
        bool m_bTerminated;
        ///Reference of the cell
        TAbstractCell* m_pCell;
        ///Susbend time
        long m_lSleepTime;
    };
    friend class TCellLife;
};

END_NAMESPACE_Zeus

#endif
