/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractModuleSession                                        *
 * Package : Help Implementations                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:43:08 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractModuleSession.h,v $
 *   Revision 1.2  2005/11/10 10:43:08  bhadorn
 *   New macros designed
 *
 *   Revision 1.1  2005/10/28 12:22:23  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef AbstractModuleSessionH
#define AbstractModuleSessionH

#include <zeusbase/System/Interfaces/ISingletonManager.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/ZObjectFactory.h>
#include <zeusbase/MOM/Interfaces/IModuleSession.hpp>
#include <zeusbase/MOM/Interfaces/IZeusAPI.hpp>
#include <zeusbase/Config/ZeusMacros.hpp>


BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! \def MExportObjectFactory Defines a factory method witch will be
                             exported by the library
*/
/***************************************************************************/
#define MExportObjectFactory(InterFace, ClassName)\
MODULEXPORT_PRE unsigned long MODULEXPORT_POST create##InterFace(NAMESPACE_Zeus::IZUnknown** iface)\
{\
  unsigned long ulRetval = RET_REQUEST_FAILED;\
  if (iface != NULL)\
  {\
    *iface = (InterFace*)new ClassName();\
    ulRetval = RET_NOERROR;\
  }\
  return ulRetval;\
}\

/***************************************************************************/
/*! \def MExportXRootObjectFactory Defines a factory method to create
                XRootObjects witch will be exported by the library
*/
/***************************************************************************/
#define MExportXRootObjectFactory(classid)\
MODULEXPORT_PRE unsigned long MODULEXPORT_POST create##classid(NAMESPACE_Zeus::IXMLNode* main_node, NAMESPACE_Zeus::IXMLDocument* docu, NAMESPACE_Zeus::IXObject** obj) \
{ \
  unsigned long ulRetval = RET_REQUEST_FAILED; \
  if (main_node != NULL && docu != NULL && obj != NULL) \
  { \
    *obj = new classid(main_node, docu); \
    ulRetval = RET_NOERROR; \
  } \
  else \
  { \
    ulRetval = RET_INTERNAL_ERROR; \
  } \
  return ulRetval; \
} \


/***************************************************************************/
/*! \def MExportXObjectFactory Defines a factory method to create
                XObjects witch will be exported by the library
*/
/***************************************************************************/
#define MExportXObjectFactory(classid)\
MODULEXPORT_PRE unsigned long MODULEXPORT_POST create##classid(NAMESPACE_Zeus::IXMLNode* main_node, NAMESPACE_Zeus::IXMLDocument*, NAMESPACE_Zeus::IXObject** obj) \
{ \
  unsigned long ulRetval = RET_REQUEST_FAILED; \
  if (main_node != NULL && obj != NULL) \
  { \
    *obj = new classid(main_node); \
    ulRetval = RET_NOERROR; \
  } \
  else \
  { \
    ulRetval = RET_INTERNAL_ERROR; \
  } \
  return ulRetval; \
} \


/***************************************************************************/
/*! The MregisterLibrary_Start starts the register of object factories, 
    library loader interface and other common singletons of the framework
    \param codemodulename : Name of the codemodule (string)
*/
/***************************************************************************/
#define MregisterLibrary_Start(codemodulename)\
MODULEXPORT_PRE unsigned long MODULEXPORT_POST registerLibrary(NAMESPACE_Zeus::ISingletonManager* manager)\
{\
  if (manager != NULL)\
  {\
    TString strCodeModule(codemodulename); \
    TString strName = L"IXObjectFactory"; \
    IXObjectFactory* pFactory = NULL;\
    if (manager->getSingleton(&strName, (IZUnknown**)&pFactory) == RET_NOERROR)\
    { \
      TXObjectFactory::getInstance().setDelegationInterface(pFactory);\
      pFactory->release();\
    } \
    \
    ILibraryManager* pLibManager = NULL; \
    strName = L"ILibraryManager"; \
    if (manager->getSingleton(&strName, (IZUnknown**)&pLibManager) == RET_NOERROR)\
    { \
      TLibraryManager::getInstance().setDelegationInterface(pLibManager);\
      pLibManager->release(); \
    } \
    ILogger* pLogger = NULL;\
    strName = L"ILogger";\
    if (manager->getSingleton(&strName, (IZUnknown**)&pLogger) == RET_NOERROR)\
    { \
      ILogger* pMyLogger = NULL; \
      if (pLogger->getLogger(&strCodeModule, &pMyLogger) == RET_NOERROR) \
      { \
        NAMESPACE_Zeus::TLoggerManager::getInstance().setLogger(pMyLogger);\
        pMyLogger->release(); \
      } \
      else \
      { \
        NAMESPACE_Zeus::TLoggerManager::getInstance().setLogger(pLogger);\
      } \
      pLogger->release(); \
    } \
    \
    strName = L"IZObjectFactory"; \
    IZObjectFactory* pZFactory = NULL;\
    if (manager->getSingleton(&strName, (IZUnknown**)&pZFactory) == RET_NOERROR)\
    { \
      TZObjectFactory::getInstance().setDelegationInterface(pZFactory);\
      pZFactory->release();\
    } \
    ZLOGGER.printfln(LOGMODE_INFO, "Code module [%s] has been registered.", strCodeModule.c_str()); \
  
/***************************************************************************/
/*! End of the register function
*/
/***************************************************************************/
#define MregisterLibrary_End\
  } \
  return RET_NOERROR;\
}\

/***************************************************************************/
/*! Complete register function for simple modules
    \param codemodulename : Name of the codemodule (string)
*/
/***************************************************************************/
#define MregisterLibrary(codemodulename)\
MregisterLibrary_Start(codemodulename); \
MregisterLibrary_End;

/***************************************************************************/
/*! Start of the unregister function, witch releases all delegation 
    interfaces of the common framework singletons
*/
/***************************************************************************/
#define MunregisterLibrary_Start()\
MODULEXPORT_PRE unsigned long MODULEXPORT_POST unregisterLibrary(NAMESPACE_Zeus::ISingletonManager* )\
{\
  NAMESPACE_Zeus::TXObjectFactory::getInstance().releaseDelegationInterface();\
  NAMESPACE_Zeus::TZObjectFactory::getInstance().releaseDelegationInterface();\
  NAMESPACE_Zeus::TLibraryManager::getInstance().releaseDelegationInterface();\
  NAMESPACE_Zeus::TLoggerManager::getInstance().setLogger(NULL);\

/***************************************************************************/
/*! End of the unregister function
*/
/***************************************************************************/
#define MunregisterLibrary_End\
  return RET_NOERROR;\
}\

/***************************************************************************/
/*! Complete unregister function for simple modules, witch releases 
    all delegation interfaces of the common framework singletons
*/
/***************************************************************************/
#define MunregisterLibrary()\
  MunregisterLibrary_Start() \
  MunregisterLibrary_End; \

/***************************************************************************/
/*! This is an abstract module factory, witch is needed to implement a
    module for the zeus framework
*/
/***************************************************************************/
class TAbstractModuleSession : public TZObject, public IModuleSession
{
  public:
    TAbstractModuleSession();
        
    //Methods of IModuleSession
    virtual unsigned long MQUALIFIER createObjectOfModule(unsigned long interface_id, IZUnknown** iface)=0;
    virtual void MQUALIFIER initialize(IZeusAPI* api);

    //Methods of IZUnknown
    MEMORY_MANAGER_DECL

  protected:  
    virtual ~TAbstractModuleSession();
    ///Reference of the API
    IZeusAPI* m_pZeusAPI;
    ///Static instance counter
    static unsigned long m_ulObjectCount;
    
  private:
    
};

END_NAMESPACE_Zeus

#endif
