/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : LocalPipeRegistryTest                                         *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:01:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LocalPipeRegistryTest.cpp,v $
 *   Revision 1.2  2005/11/25 10:01:44  bhadorn
 *   nameserver XML changed
 *
 *   Revision 1.1  2005/11/16 11:37:41  bhadorn
 *   Creates test classes for communication of cells
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/CellModel/LocalPipeRegistryTest.h>
#include <zeusbase/CellModel/CommPipe.h>
#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/Remote/Naming.h>
#include <zeusbase/System/Time.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TLocalPipeRegistryTest );


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TLocalPipeRegistryTest::setUp()
{
  m_pRegistry = NULL;
  Naming.useRemote();
  TString strXMLStream = TString(L"<?xml version=\"1.0\"?>\n\
    <zeus ClassName=\"TSystemManager\">\n\
      <NameServer ClassName=\"TNameServer\" CodeModule=\"NameServer\" Address=\"127.0.0.1\" Port=\"7278\"/>\n\
    </zeus>");
  IXObject* pObj = NULL;
  if (TXObjectFactory::getInstance().createObjectFromStream(&strXMLStream, (IXObject**)&pObj, false) == RET_NOERROR)
  {
    m_lstRoots.push(pObj);
  }
  TTime::sleep(0.75);
  strXMLStream = TString(L"<?xml version=\"1.0\"?>\n\
    <zeus ClassName=\"TSystemManager\">\n\
      <LocalPipeRegistry ClassName=\"TLocalPipeRegistry\" \n\
        RegistryPort=\"7278\" \n\
        RegistryAddress=\"127.0.0.1\"/> \n\
    </zeus>");
  if (TXObjectFactory::getInstance().createObjectFromStream(&strXMLStream, (IXObject**)&pObj, false) == RET_NOERROR)
  {
    IXObject* pRegObj = NULL;
    TString strName = L"/LocalPipeRegistry";
    if (pObj->getObject(&strName, &pRegObj) == RET_NOERROR)
    {
      pRegObj->askForInterface(INTERFACE_ILocalPipeRegistry, (IZUnknown**)&m_pRegistry);
      pRegObj->release();
    }
    m_lstRoots.push(pObj);
  }
  TTime::sleep(0.75);
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TLocalPipeRegistryTest::tearDown()
{
  //Release all instances
  while(!m_lstStack.isEmpty())
  {
    m_lstStack.pop()->release();
  }
  while(!m_lstRoots.isEmpty())
  {
    IXObject* pObj = m_lstRoots.pop();
    pObj->freeze();
    pObj->release();
  }
  if (m_pRegistry != NULL)
  {
    m_pRegistry->release();
    m_pRegistry = NULL;
  }
  Naming.reset();
  Naming.useRemote(false);
}

/***************************************************************************/
/*! Creates a pipe outside of the registry
    \param cellname : Name of the cell
    \param pipename : Name of the pipe;
*/
/***************************************************************************/
void TLocalPipeRegistryTest::createPipe(TString cellname, TString pipename)
{
  TCommPipe* pPipe = new TCommPipe(PIPEPRIORITY_NORMAL, pipename, cellname);
  m_lstStack.push((ICommPipe*)pPipe);
  CPPUNIT_ASSERT(pPipe->connect() == RET_NOERROR);
  TTime::sleep(0.5);
  pPipe->unlock();
  TString strPipeAddress;
  unsigned long ulPipePort;
  pPipe->getConnectionParam(&strPipeAddress, &ulPipePort);
  
  TString strObjName = TString(L"ccm://") + 
          strPipeAddress + 
          TString(L":") + 
          TString((long)ulPipePort) + 
          TString(L"/") + 
          cellname +
          TString(L"/") + 
          pipename;
  //printf("%s\n", strObjName.c_str());
  TString strCatName = cellname + TString(L"/") + pipename;
  CPPUNIT_ASSERT(Naming.rebind2(&strObjName, &strCatName, pPipe) == RET_NOERROR);
}


/***************************************************************************/
/*! Tests the creation of the pipe registry
*/
/***************************************************************************/
void TLocalPipeRegistryTest::test_create()
{
  TString strAddress;
  CPPUNIT_ASSERT(m_pRegistry != NULL);
  CPPUNIT_ASSERT(m_pRegistry->isConnected());
  CPPUNIT_ASSERT(m_pRegistry->getPort() == 7278);
  CPPUNIT_ASSERT(m_pRegistry->getAddress(&strAddress) == RET_NOERROR);
  CPPUNIT_ASSERT(strAddress == L"127.0.0.1");
} 

/***************************************************************************/
/*! Tests the creation of a pipe
*/
/***************************************************************************/
void TLocalPipeRegistryTest::test_createPipe()
{
  CPPUNIT_ASSERT(m_pRegistry != NULL);
  
  IOwnCommPipe* pPipe;
  TString strPipeName = L"Pipe1";
  TString strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, false, &pPipe) == RET_NOERROR);
  m_lstStack.push(pPipe);
  
  TString strTestPipeName;
  TString strTestCellName;
  pPipe->getCellName(&strTestCellName);
  pPipe->getPipeName(&strTestPipeName);
  CPPUNIT_ASSERT(strTestCellName == strCellName);
  CPPUNIT_ASSERT(strTestPipeName == strPipeName);
  CPPUNIT_ASSERT(!pPipe->hasRemoteAccess());
  CPPUNIT_ASSERT(pPipe->getPriority() == PIPEPRIORITY_HIGH);

  IOwnCommPipe* pPipe2;
  strPipeName = L"Pipe2";
  strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, true, &pPipe2) == RET_NOERROR);
  m_lstStack.push(pPipe2);
  CPPUNIT_ASSERT(pPipe2->hasRemoteAccess());
  
}


/***************************************************************************/
/*! Tests the registration of a pipe
*/
/***************************************************************************/
void TLocalPipeRegistryTest::test_registerPipe()
{
  CPPUNIT_ASSERT(m_pRegistry != NULL);
  
  //Test with a local pipe
  IOwnCommPipe* pPipe;
  TString strPipeName = L"Pipe1";
  TString strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, false, &pPipe) == RET_NOERROR);
  m_lstStack.push(pPipe);
  
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe) == RET_NOERROR);
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe) != RET_NOERROR);
  
  //Test with a remote pipe
  IOwnCommPipe* pPipe2;
  strPipeName = L"Pipe2";
  strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, true, &pPipe2) == RET_NOERROR);
  m_lstStack.push(pPipe2);
  
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe2) == RET_NOERROR);
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe2) != RET_NOERROR);
}

/***************************************************************************/
/*! Tests getting pipes by cellname and pipename
*/
/***************************************************************************/
void TLocalPipeRegistryTest::test_getPipe()
{
  CPPUNIT_ASSERT(m_pRegistry != NULL);
  
  //Test with a local pipe
  IOwnCommPipe* pPipe;
  TString strPipeName = L"Pipe1";
  TString strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, false, &pPipe) == RET_NOERROR);
  m_lstStack.push(pPipe);
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe) == RET_NOERROR);

  //Get local created pipe
  ICommPipe* pTestPipe = NULL;
  CPPUNIT_ASSERT(m_pRegistry->getPipe(&strCellName, &strPipeName, &pTestPipe) == RET_NOERROR);
  m_lstStack.push(pTestPipe);
  CPPUNIT_ASSERT(pTestPipe == pPipe);
}

/***************************************************************************/
/*! Tests getting a list of pipes by cellname and pipename
*/
/***************************************************************************/
void TLocalPipeRegistryTest::test_getPipes()
{
  CPPUNIT_ASSERT(m_pRegistry != NULL);
  
  //Test with a local pipe
  IOwnCommPipe* pPipe;
  TString strPipeName = L"Pipe1";
  TString strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, false, &pPipe) == RET_NOERROR);
  m_lstStack.push(pPipe);
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe) == RET_NOERROR);

  //Test with a remote pipe
  IOwnCommPipe* pPipe2;
  strPipeName = L"Pipe1";
  strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, true, &pPipe2) == RET_NOERROR);
  m_lstStack.push(pPipe2);
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe2) == RET_NOERROR);
  
  //An other pipe outside of the registry
  createPipe(L"Cell1", L"Pipe1");
  
  //Get local created pipe
  TSingleLinkedList<ICommPipe*> lstPipes;
  CPPUNIT_ASSERT(m_pRegistry->getPipes(&strCellName, &strPipeName, &lstPipes) == RET_NOERROR);
  for(int i = 0; i < lstPipes.getCount(); i++)
  {
    m_lstStack.push(lstPipes[i]);
  }
  
  CPPUNIT_ASSERT(lstPipes.getCount() == 2);
  
  
  
  //Create 3 pipes outside of the registry  
  strPipeName = L"Pipe1";
  strCellName = L"Cell2";
  createPipe(strCellName, strPipeName);
  createPipe(strCellName, strPipeName);
  createPipe(strCellName, strPipeName);

  //Get those pipes
  TSingleLinkedList<ICommPipe*> lstPipes2;
  CPPUNIT_ASSERT(m_pRegistry->getPipes(&strCellName, &strPipeName, &lstPipes2) == RET_NOERROR);
  for(int i = 0; i < lstPipes2.getCount(); i++)
  {
    m_lstStack.push(lstPipes2[i]);
  }
  
  CPPUNIT_ASSERT(lstPipes2.getCount() == 3);
}
    
/***************************************************************************/
/*! Tests updating a list of pipes by cellname and pipename
*/
/***************************************************************************/
void TLocalPipeRegistryTest::test_updatePipes()
{
  CPPUNIT_ASSERT(m_pRegistry != NULL);
  
  //Test with a local pipe
  IOwnCommPipe* pPipe;
  TString strPipeName = L"Pipe1";
  TString strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, false, &pPipe) == RET_NOERROR);
  m_lstStack.push(pPipe);
  pPipe->unlock();
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe) == RET_NOERROR);

  //Test with a remote pipe
  IOwnCommPipe* pPipe2;
  strPipeName = L"Pipe1";
  strCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->createPipe(&strCellName, &strPipeName, PIPEPRIORITY_HIGH, true, &pPipe2) == RET_NOERROR);
  m_lstStack.push(pPipe2);
  pPipe2->unlock();
  CPPUNIT_ASSERT(m_pRegistry->registerPipe(pPipe2) == RET_NOERROR);

  TSingleLinkedList<ICommPipe*> lstPipes;
  
  TString strTestPipeName = L"Pipe1";
  TString strTestCellName = L"Cell1";
  CPPUNIT_ASSERT(m_pRegistry->updatePipes(&strTestCellName, &strTestPipeName, &lstPipes) == RET_NOERROR);
  for(int i = 0; i < lstPipes.getCount(); i++)
  {
    m_lstStack.push(lstPipes[i]);
  }
  CPPUNIT_ASSERT(lstPipes.getCount() == 2);
  
  //A pipe is added by remote
  createPipe(L"Cell1", L"Pipe1");
  lstPipes.clear();
  CPPUNIT_ASSERT(m_pRegistry->updatePipes(&strTestCellName, &strTestPipeName, &lstPipes) == RET_NOERROR);
  for(int i = 0; i < lstPipes.getCount(); i++)
  {
    m_lstStack.push(lstPipes[i]);
  }
  CPPUNIT_ASSERT(lstPipes.getCount() == 3);
  
  
  
}
