/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ModuleTest                                                    *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:18 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ModuleTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#include <zeusbase/System/LibraryManager.h> 
#include <zeustest/MOM/ModuleTest.h> 
#include <zeusbase/MOM/Module.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TModuleTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TModuleTest::setUp()
{
  m_pBaseObject = NULL;
  m_pModule = NULL;
  m_pSession = NULL;
  
  TString strModuleTree1 = L"<?xml version='1.0' encoding='ISO-8859-1'?>\
<MOM ID=\"1\" ClassName=\"TXRootObject\" AttributeSchema=\"1\">\
  <MainSystem ID=\"1\" ClassName=\"TModule\">\
    <Services ID=\"2\" ClassName=\"TModule\">\
      <SystemModule ID=\"1\" ClassName=\"TModule\" Mode=\"1\"/>\
      <SystemModule ID=\"2\" ClassName=\"TModule\" Mode=\"0\"/>\
    </Services>\
    <InputCellModule ID=\"1\" ClassName=\"TModule\" Mode=\"0\"/>\
    <WorkCellModule ID=\"2\" ClassName=\"TModule\" Mode=\"0\"/>\
    <LoadSystem ID=\"\" ClassName=\"TModule\" CreateChildren=\"0\">\
      <TestModule ID=\"1\" ClassName=\"TModule\" CreateChildren=\"1\" ModuleLibrary=\"TestModule\"/>\
    </LoadSystem>\
  </MainSystem>\
</MOM>";

  //The test of the factory: see TXObjectFactoryTest
  if (TXObjectFactory::getInstance().createObjectFromStream(&strModuleTree1, &m_pBaseObject, false) == RET_NOERROR)
  {
    IXObject* pMainSystemObject = NULL;
    TString strName = L"MainSystem";
    if (m_pBaseObject->getObject(&strName, &pMainSystemObject) == RET_NOERROR)
    {
      pMainSystemObject->askForInterface(INTERFACE_IModule, (IZUnknown**)&m_pModule);
      pMainSystemObject->release();
    }
  }
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TModuleTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
  while(!m_lstLoaderRef.isEmpty())
  {
    m_lstLoaderRef.pop()->release();
  }
  
  if (m_pModule != NULL)
  {
    m_pModule->release();
  }
  
  if (m_pBaseObject != NULL)
  {
    m_pBaseObject->release();
  }
  if (m_pSession != NULL)
  {
    m_pSession->release();
    m_pSession = NULL;  
  }
  
  TString strLibName = L"TestModule";
  TLibraryManager::getInstance().remove(strLibName);
}

/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TModuleTest::test_create()
{
  CPPUNIT_ASSERT(m_pModule != NULL);
  
  TString strName;
  m_pModule->getName(&strName);
  CPPUNIT_ASSERT(strName == L"MainSystem");
}
 
/***************************************************************************/
/*! Tests the getModule() method
*/
/***************************************************************************/
void TModuleTest::test_getModule()
{
  CPPUNIT_ASSERT(m_pModule != NULL);
  
  TString strName = L"Services";
  TString strCheckName;
  IModule* pModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  pModule->getName(&strCheckName);
  CPPUNIT_ASSERT(strCheckName == L"Services");

  strName = L"./Services/SystemModule[1]";
  strCheckName = L"";
  pModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  pModule->getName(&strCheckName);
  CPPUNIT_ASSERT(strCheckName == L"SystemModule");
  
  strName = L"/MainSystem/WorkCellModule";
  strCheckName = L"";
  pModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  pModule->getName(&strCheckName);
  CPPUNIT_ASSERT(strCheckName == L"WorkCellModule");
}
 
/***************************************************************************/
/*! Tests the getmode method
*/
/***************************************************************************/
void TModuleTest::test_getMode()
{
  CPPUNIT_ASSERT(m_pModule != NULL);
  CPPUNIT_ASSERT(m_pModule->getMode() == MODULEMODE_NORMAL);
  
  TString strName = L"./Services/SystemModule[1]";
  IModule* pModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  CPPUNIT_ASSERT(pModule->getMode() == MODULEMODE_NORMAL);
   
  strName = L"./Services/SystemModule[0]";
  pModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  CPPUNIT_ASSERT(pModule->getMode() == MODULEMODE_LOAD);
}


/***************************************************************************/
/*! Tests the activation functionality
*/
/***************************************************************************/
void TModuleTest::test_activate()
{
  //-----------------------------------------------------------------------
  //At the beginning, no module has been activated
  CPPUNIT_ASSERT(m_pModule != NULL);
  CPPUNIT_ASSERT(!m_pModule->isActive());
  IModule* pModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getActiveModule(&pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  CPPUNIT_ASSERT(m_pModule == pModule);

  //-----------------------------------------------------------------------
  //Activates the ./Services/SystemModule[1]
  TString strName = L"./Services/SystemModule[1]";
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  pModule->activate();
  CPPUNIT_ASSERT(m_pModule->isActive());
  CPPUNIT_ASSERT(pModule->isActive());
  IModule* pActiveModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getActiveModule(&pActiveModule) == RET_NOERROR);
  m_lstObjRef.push(pActiveModule);
  CPPUNIT_ASSERT(pModule == pActiveModule);
  
  
  //-----------------------------------------------------------------------
  //Activates the ./InputCellModule. The previous module 
  // ./Services/SystemModule[1] must be deactive (but is still locally active)
  IModule* pOldActivModule = pActiveModule;
  strName = L"./InputCellModule";
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  pModule->activate();
  CPPUNIT_ASSERT(m_pModule->isActive());
  CPPUNIT_ASSERT(pModule->isActive());
  pActiveModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getActiveModule(&pActiveModule) == RET_NOERROR);
  m_lstObjRef.push(pActiveModule);
  CPPUNIT_ASSERT(pModule == pActiveModule);
  CPPUNIT_ASSERT(pOldActivModule->isActive()); //locally active
  
  //Activate the parent of the module ./Services/SystemModule[1].
  //the localy activated module must be reachable as global active again.
  strName = L"./Services";
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  pModule->activate();
  pActiveModule = NULL;
  CPPUNIT_ASSERT(m_pModule->getActiveModule(&pActiveModule) == RET_NOERROR);
  m_lstObjRef.push(pActiveModule);
  CPPUNIT_ASSERT(pOldActivModule == pActiveModule);
  CPPUNIT_ASSERT(pOldActivModule->isActive()); //globaly active again
}


/***************************************************************************/
/*! Tests the isLoaded functionality
*/
/***************************************************************************/
void TModuleTest::test_isLoaded()
{
  //-----------------------------------------------------------------------
  CPPUNIT_ASSERT(m_pModule != NULL);
  IModule* pModule = NULL; 
  IXLoaderObject* pLObject = NULL;
  CPPUNIT_ASSERT(m_pModule->askForInterface(INTERFACE_IXLoaderObject, (IZUnknown**)&pLObject) == RET_NOERROR);
  m_lstLoaderRef.push(pLObject);
  CPPUNIT_ASSERT(pLObject->isLoaded());
  
  TString strName = L"./Services/SystemModule[1]";
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  CPPUNIT_ASSERT(pModule->askForInterface(INTERFACE_IXLoaderObject, (IZUnknown**)&pLObject) == RET_NOERROR);
  m_lstLoaderRef.push(pLObject);
  CPPUNIT_ASSERT(pLObject->isLoaded());
  
  // This module should not be loaded at this time. Therefore we check the
  // LibraryManager for its existance of the ResourceLibrary
  TString strLibName = L"TestModule";
  CPPUNIT_ASSERT(!TLibraryManager::getInstance().hasCodeModule(strLibName));
  strName = L"./LoadSystem/TestModule[0]";
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  CPPUNIT_ASSERT(pModule->askForInterface(INTERFACE_IXLoaderObject, (IZUnknown**)&pLObject) == RET_NOERROR);
  m_lstLoaderRef.push(pLObject);
  CPPUNIT_ASSERT(pLObject->isLoaded());
  CPPUNIT_ASSERT(TLibraryManager::getInstance().hasCodeModule(strLibName));
}

/***************************************************************************/
/*! Tests the getModuleSession functionality
*/
/***************************************************************************/
void TModuleTest::test_getModuleSession()
{
  CPPUNIT_ASSERT(m_pModule != NULL);
  IModule* pModule = NULL; 

  //Module without Session Object
  TString strName = L"./Services/SystemModule[1]";
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  CPPUNIT_ASSERT(pModule->getModuleSession(&m_pSession) == RET_REQUEST_FAILED);
  
  //Module with session object
  TString strLibName = L"TestModule";
  strName = L"./LoadSystem/TestModule[0]";
  CPPUNIT_ASSERT(m_pModule->getModule(&strName, &pModule) == RET_NOERROR);
  m_lstObjRef.push(pModule);
  CPPUNIT_ASSERT(pModule->getModuleSession(&m_pSession) == RET_NOERROR);
}

