/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SystemManagerTest                                             *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:18 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SystemManagerTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/MOM/SystemManagerTest.h> 
#include <zeusbase/MOM/Module.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TSystemManagerTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TSystemManagerTest::setUp()
{
  m_pBaseObject = NULL;
  m_pModule = NULL;
  
  TString strModuleTree1 = L"<?xml version='1.0' encoding='ISO-8859-1'?>\
<MOM ID=\"1\" ClassName=\"TSystemManager\" AttributeSchema=\"1\">\
  <MainSystem ID=\"1\" ClassName=\"TModule\">\
    <Services ID=\"2\" ClassName=\"TModule\">\
      <SystemModule ID=\"1\" ClassName=\"TModule\" Mode=\"1\"/>\
      <SystemModule ID=\"2\" ClassName=\"TModule\" Mode=\"0\"/>\
    </Services>\
    <InputCellModule ID=\"1\" ClassName=\"TModule\" Mode=\"0\"/>\
    <WorkCellModule ID=\"2\" ClassName=\"TModule\" Mode=\"0\"/>\
  </MainSystem>\
</MOM>";

  //The test of the factory: see TXObjectFactoryTest
  if (TXObjectFactory::getInstance().createObjectFromStream(&strModuleTree1, &m_pBaseObject, false) == RET_NOERROR)
  {
    IXObject* pMainSystemObject = NULL;
    TString strName = L"MainSystem";
    if (m_pBaseObject->getObject(&strName, &pMainSystemObject) == RET_NOERROR)
    {
      pMainSystemObject->askForInterface(INTERFACE_IModule, (IZUnknown**)&m_pModule);
      pMainSystemObject->release();
    }
  }
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TSystemManagerTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
  
  if (m_pModule != NULL)
  {
    m_pModule->release();
  }
  
  if (m_pBaseObject != NULL)
  {
    m_pBaseObject->release();
  }
}

/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TSystemManagerTest::test_create()
{
  CPPUNIT_ASSERT(m_pModule != NULL);
  
  TString strName;
  m_pModule->getName(&strName);
  CPPUNIT_ASSERT(strName == L"MainSystem");
}
 
