/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : BinaryMessageTest                                             *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:18 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: BinaryMessageTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  
 
#include <zeustest/Messaging/BinaryMessageTest.h> 
#include <zeusbase/System/ByteArrayInputStream.h> 
#include <zeusbase/System/ByteArrayOutputStream.h> 
#include <stdio.h>
USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TBinaryMessageTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TBinaryMessageTest::setUp()
{
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TBinaryMessageTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
}

/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TBinaryMessageTest::test_create() 
{
  TBinaryMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TBinaryMessage::createMessage(L"TestMessage","","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);
  
  //Able to create a SystemMessage out of an empty stream
  IBinaryMessage* pMessage2 = NULL;
  TByteArray Array1;
  
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&Array1, (ISerializable**)&pMessage2) != RET_NOERROR);
} 

/***************************************************************************/
/*! Tests the header methods 
*/
/***************************************************************************/
void TBinaryMessageTest::test_header() 
{
  TBinaryMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TBinaryMessage::createMessage(L"TestMessage","","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);

  TString strName = L"Sender1";
  CPPUNIT_ASSERT(pMessage1->setSenderCellName(&strName) == RET_NOERROR);
  strName = L"Pipe1";
  pMessage1->setSenderPipeName(&strName);
  strName = L"Target2";
  pMessage1->setTargetCellName(&strName);
  strName = L"Pipe2";
  pMessage1->setTargetPipeName(&strName);
  pMessage1->setNumber(99);
  
  TString strRet;
  CPPUNIT_ASSERT(pMessage1->getMessageName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"TestMessage");
  CPPUNIT_ASSERT(pMessage1->getSenderCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Sender1");
  CPPUNIT_ASSERT(pMessage1->getSenderPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe1");
  CPPUNIT_ASSERT(pMessage1->getTargetCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Target2");
  CPPUNIT_ASSERT(pMessage1->getTargetPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe2");
  CPPUNIT_ASSERT(pMessage1->getNumber() == 99);
  
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(pMessage1->serialize(&aStream) == RET_NOERROR);
  
  //Create new message out of the stream
  IBinaryMessage* pMessage2 = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pMessage2) == RET_NOERROR);
  m_lstObjRef.push(pMessage2);
  
  CPPUNIT_ASSERT(pMessage2->getMessageName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"TestMessage");
  CPPUNIT_ASSERT(pMessage2->getSenderCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Sender1");
  CPPUNIT_ASSERT(pMessage2->getSenderPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe1");
  CPPUNIT_ASSERT(pMessage2->getTargetCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Target2");
  CPPUNIT_ASSERT(pMessage2->getTargetPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe2");
  CPPUNIT_ASSERT(pMessage2->getNumber() == 99);
}

/***************************************************************************/
/*! Tests the getMessageType method
*/
/***************************************************************************/
void TBinaryMessageTest::test_getData()
{
  TBinaryMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TBinaryMessage::createMessage(L"TestMessage","","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);
  char aData[5] = {1,2,3,4,5};
  TByteArray aDataArray(aData, 5);
  TByteArray aTestArray;
  
  
  CPPUNIT_ASSERT(pMessage1->setData(&aDataArray) == RET_NOERROR);
  CPPUNIT_ASSERT(pMessage1->getData(&aTestArray) == RET_NOERROR);
  CPPUNIT_ASSERT(aTestArray == aDataArray);
    
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(pMessage1->serialize(&aStream) == RET_NOERROR);
 
  //Create new message out of the stream
  IBinaryMessage* pMessage2 = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pMessage2) == RET_NOERROR);
  m_lstObjRef.push(pMessage2);
  
  TByteArray aTestArray2;
  CPPUNIT_ASSERT(pMessage1->getData(&aTestArray2) == RET_NOERROR);
  CPPUNIT_ASSERT(aTestArray2 == aDataArray);
}
 

/***************************************************************************/
/*! Tests the interface check methods
*/
/***************************************************************************/
void TBinaryMessageTest::test_Interface()
{
  TBinaryMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TBinaryMessage::createMessage(L"TestMessage","","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);
  
  CPPUNIT_ASSERT(pMessage1->getInterfaceID() == INTERFACE_IBinaryMessage);
}

 
 
