/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XMLMessageTest                                                *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:18 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XMLMessageTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  
 
#include <zeustest/Messaging/XMLMessageTest.h> 
#include <zeusbase/System/ByteArrayInputStream.h> 
#include <zeusbase/System/ByteArrayOutputStream.h> 
#include <stdio.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TXMLMessageTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TXMLMessageTest::setUp()
{
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TXMLMessageTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
  while(!m_lstNodeRef.isEmpty())
  {
    m_lstNodeRef.pop()->release();
  }
}

/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TXMLMessageTest::test_create()
{
  IXMLMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"TestMessage","","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);
  
  TByteArray Array1;
  pMessage1->serialize(&Array1);
  
  
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&Array1, (ISerializable**)&pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);
  
  TByteArray Array2;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&Array2, (ISerializable**)&pMessage1) != RET_NOERROR);
} 

/***************************************************************************/
/*! Tests the header methods 
*/
/***************************************************************************/
void TXMLMessageTest::test_header() 
{
  IXMLMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"TestMessage","","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);

  TString strName = L"Sender1";
  CPPUNIT_ASSERT(pMessage1->setSenderCellName(&strName) == RET_NOERROR);
  strName = L"Pipe1";
  pMessage1->setSenderPipeName(&strName);
  strName = L"Target2";
  pMessage1->setTargetCellName(&strName);
  strName = L"Pipe2";
  pMessage1->setTargetPipeName(&strName);
  pMessage1->setNumber(99);
  
  TString strRet;
  CPPUNIT_ASSERT(pMessage1->getMessageName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"TestMessage");
  CPPUNIT_ASSERT(pMessage1->getSenderCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Sender1");
  CPPUNIT_ASSERT(pMessage1->getSenderPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe1");
  CPPUNIT_ASSERT(pMessage1->getTargetCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Target2");
  CPPUNIT_ASSERT(pMessage1->getTargetPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe2");
  CPPUNIT_ASSERT(pMessage1->getNumber() == 99);
  
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(pMessage1->serialize(&aStream) == RET_NOERROR);
  
  //Create new message out of the stream
  IXMLMessage* pMessage2 = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pMessage2) == RET_NOERROR);
  m_lstObjRef.push(pMessage2);
  
  CPPUNIT_ASSERT(pMessage2->getMessageName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"TestMessage");
  CPPUNIT_ASSERT(pMessage2->getSenderCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Sender1");
  CPPUNIT_ASSERT(pMessage2->getSenderPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe1");
  CPPUNIT_ASSERT(pMessage2->getTargetCellName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Target2");
  CPPUNIT_ASSERT(pMessage2->getTargetPipeName(&strRet) == RET_NOERROR);
  CPPUNIT_ASSERT(strRet == L"Pipe2");
  CPPUNIT_ASSERT(pMessage2->getNumber() == 99);
}

 
/***************************************************************************/
/*! Tests the data methods
*/
/***************************************************************************/
void TXMLMessageTest::test_data()
{
  IXMLMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"TestMessage","","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);
  
  IXMLNode* pNode = NULL;
  CPPUNIT_ASSERT(pMessage1->getDataNode(&pNode) == RET_NOERROR);
  m_lstNodeRef.push(pNode);

  TString strAttName = L"TestValue";
  TString strValue = L"This is a test";
  CPPUNIT_ASSERT(pNode->setAttributeValue(&strAttName, &strValue) == RET_NOERROR);
  
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(pMessage1->serialize(&aStream) == RET_NOERROR);
 
  //Create new message out of the stream
  IXMLMessage* pMessage2 = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pMessage2) == RET_NOERROR);
  m_lstObjRef.push(pMessage2);
  
  pNode = NULL;
  CPPUNIT_ASSERT(pMessage2->getDataNode(&pNode) == RET_NOERROR);
  m_lstNodeRef.push(pNode);
  
  TString strTestValue;
  CPPUNIT_ASSERT(pNode->getAttributeValue(&strAttName, &strTestValue) == RET_NOERROR);
  CPPUNIT_ASSERT(strTestValue == strValue);
}

/***************************************************************************/
/*! Tests the getPrimaryInterface method
*/
/***************************************************************************/
void TXMLMessageTest::test_Interface()
{
  IXMLMessage* pMessage1 = NULL;
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"TestMessage", "","", &pMessage1) == RET_NOERROR);
  m_lstObjRef.push(pMessage1);
  
  CPPUNIT_ASSERT(pMessage1->getInterfaceID() == INTERFACE_IXMLMessage);
}

