/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : NameServerTest                                                *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:03:24 $                                  *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: NameServerTest.cpp,v $
 *   Revision 1.3  2005/11/25 10:03:24  bhadorn
 *   Nameserver XML changed
 *
 *   Revision 1.2  2005/11/16 11:39:50  bhadorn
 *   New test added for getObjectNames()
 *
 *   Revision 1.1  2005/11/10 10:51:10  bhadorn
 *   created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/Remote/NameServerTest.h>
#include <zeustest/Remote/RemoteTestObject.h>
#include <zeusbase/Remote/Naming.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/StringList.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TNameServerTest ); 


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TNameServerTest::setUp()
{
  Naming.useRemote();
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TNameServerTest::tearDown()
{
  //Release all instances
  while(!m_lstStack.isEmpty())
  {
    m_lstStack.pop()->release();
  }
  
  while(!m_lstRoots.isEmpty())
  {
    IXObject* pObj = m_lstRoots.pop();
    pObj->freeze();
    pObj->release();
  }
  Naming.reset();
  Naming.useRemote(false);
}

/***************************************************************************/
/*! creates a naming server 
*/
/***************************************************************************/
void TNameServerTest::createServer()
{
  TString strXMLStream = TString(L"<?xml version=\"1.0\"?>\n\
    <zeus ClassName=\"TSystemManager\">\n\
      <NameServer ClassName=\"TNameServer\" CodeModule=\"NameServer\" Address=\"127.0.0.1\" Port=\"7278\"/>\n\
    </zeus>");
    
  IXObject* pObj = NULL;
  CPPUNIT_ASSERT(TXObjectFactory::getInstance().createObjectFromStream(&strXMLStream, (IXObject**)&pObj, false) == RET_NOERROR);
  m_lstRoots.push(pObj);
  
  //Try to create a stub and connect to the server
  TTime::sleep(0.75);
  unsigned long ulRetval = RET_NOERROR;
  CPPUNIT_ASSERT(Naming.connect(L"127.0.0.1", 7278) == ulRetval);
}


/***************************************************************************/
/*! Tests remote binding of the naming service
*/
/***************************************************************************/
void TNameServerTest::test_bind()
{
  createServer();
  
  //Create remote object
  TString strName = L"MyTestObject";
  TRemoteTestObject* pObject = new TRemoteTestObject();
  pObject->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject);
  pObject->setDoubleValue(12.25);
  pObject->setLongValue(-19);
  TString strText(L"This is a test");
  pObject->setStringValue(&strText);
  
  CPPUNIT_ASSERT(pObject->getDoubleValue() == 12.25);
  CPPUNIT_ASSERT(pObject->getLongValue() == -19);
  
  //Bind object to the remote name server
  CPPUNIT_ASSERT(Naming.bind(&strName, pObject) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.bind(&strName, pObject) != RET_NOERROR);
  
  
  //Lookup for this object
  IRemoteObject* pTestObject = NULL;
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pTestObject) == RET_NOERROR);
  m_lstStack.push(pTestObject);
  
  IRemoteTestObject* pRemoteStub = NULL;
  CPPUNIT_ASSERT(pTestObject->askForInterface(INTERFACE_IRemoteTestObject, (IZUnknown**)&pRemoteStub) == RET_NOERROR);
  m_lstStack.push(pRemoteStub);
  
//  printf("%f\n", pRemoteStub->getDoubleValue());
  CPPUNIT_ASSERT(pRemoteStub->getDoubleValue() == 12.25);
  CPPUNIT_ASSERT(pRemoteStub->getLongValue() == -19);
  TString strTestText;
  CPPUNIT_ASSERT(pRemoteStub->getStringValue(&strTestText) == RET_NOERROR);
  CPPUNIT_ASSERT(strTestText == L"This is a test");
}
 
/***************************************************************************/
/*! Tests remote unbinding of the naming service
*/
/***************************************************************************/
void TNameServerTest::test_unbind()
{
  createServer();
  
  //Create remote object
  TString strName = L"MyObject";
  TRemoteTestObject* pObject = new TRemoteTestObject();
  pObject->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject);
  pObject->setDoubleValue(11.11);
  
  //Bind object to the remote name server
  CPPUNIT_ASSERT(Naming.bind(&strName, pObject) == RET_NOERROR);
  IRemoteObject* pTestObject = NULL;
  
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pTestObject) == RET_NOERROR);
  m_lstStack.push(pTestObject);
  IRemoteTestObject* pRemoteStub = NULL;
  CPPUNIT_ASSERT(pTestObject->askForInterface(INTERFACE_IRemoteTestObject, (IZUnknown**)&pRemoteStub) == RET_NOERROR);
  m_lstStack.push(pRemoteStub);
  CPPUNIT_ASSERT(pRemoteStub->getDoubleValue() == 11.11);
    
  //Unbind object to the remote name server
  CPPUNIT_ASSERT(Naming.unbind(&strName) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pTestObject) != RET_NOERROR);
  
  CPPUNIT_ASSERT(Naming.unbind(&strName) != RET_NOERROR);
}

/***************************************************************************/
/*! Tests remote unbinding of the naming service
*/
/***************************************************************************/
void TNameServerTest::test_lookupCategory()
{
  createServer();
  
  //Create remote object1
  TString strName = L"MyObject1";
  TString strCat = L"Cat1";
  TRemoteTestObject* pObject1 = new TRemoteTestObject();
  pObject1->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject1);
  pObject1->setDoubleValue(20.1);
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObject1) == RET_NOERROR);

  //Create remote object2
  strName = L"MyObject2";
  TRemoteTestObject* pObject2 = new TRemoteTestObject();
  pObject2->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject2);
  pObject2->setDoubleValue(20.1);
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObject2) == RET_NOERROR);

  //Create remote object3
  strName = L"MyObject3";
  TRemoteTestObject* pObject3 = new TRemoteTestObject();
  pObject3->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject3);
  pObject3->setDoubleValue(20.1);
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObject3) == RET_NOERROR);

  //Create remote object4
  strName = L"MyObject4";
  strCat = L"Cat2";
  TRemoteTestObject* pObject4 = new TRemoteTestObject();
  pObject4->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject4);
  pObject4->setDoubleValue(20.4);
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObject4) == RET_NOERROR);
  
  //Test if objects are available
  TSingleLinkedList<IRemoteObject*> lstObjects;
  TString strCatTest = L"Cat1";
  CPPUNIT_ASSERT(Naming.lookupCategory(&strCatTest, &lstObjects) == RET_NOERROR);
  for(int i = 0; i < lstObjects.getCount(); i++)
  {
    m_lstStack.push(lstObjects[i]);
  }
  CPPUNIT_ASSERT(lstObjects.getCount() == 3);
  IRemoteTestObject* pRemoteStub = NULL;
  CPPUNIT_ASSERT(lstObjects[0]->askForInterface(INTERFACE_IRemoteTestObject, (IZUnknown**)&pRemoteStub) == RET_NOERROR);
  m_lstStack.push(pRemoteStub);
  CPPUNIT_ASSERT(pRemoteStub->getDoubleValue() == 20.1);
  pRemoteStub = NULL;
  CPPUNIT_ASSERT(lstObjects[1]->askForInterface(INTERFACE_IRemoteTestObject, (IZUnknown**)&pRemoteStub) == RET_NOERROR);
  m_lstStack.push(pRemoteStub);
  CPPUNIT_ASSERT(pRemoteStub->getDoubleValue() == 20.1);
  pRemoteStub = NULL;
  CPPUNIT_ASSERT(lstObjects[2]->askForInterface(INTERFACE_IRemoteTestObject, (IZUnknown**)&pRemoteStub) == RET_NOERROR);
  m_lstStack.push(pRemoteStub);
  CPPUNIT_ASSERT(pRemoteStub->getDoubleValue() == 20.1);  
}


/***************************************************************************/
/*! Test the getObjectNames method 
*/
/***************************************************************************/
void TNameServerTest::test_getObjectNames()
{
  createServer();
  
  //Create remote object1
  TString strName = L"MyObject1";
  TString strCat = L"Cat1";
  TRemoteTestObject* pObject1 = new TRemoteTestObject();
  pObject1->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject1);
  pObject1->setDoubleValue(20.1);
  CPPUNIT_ASSERT(Naming.rebind2(&strName, &strCat, pObject1) == RET_NOERROR);

  //Create remote object2
  strName = L"MyObject2";
  TRemoteTestObject* pObject2 = new TRemoteTestObject();
  pObject2->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject2);
  pObject2->setDoubleValue(20.1);
  CPPUNIT_ASSERT(Naming.rebind2(&strName, &strCat, pObject2) == RET_NOERROR);

  //Create remote object3
  strName = L"MyObject3";
  TRemoteTestObject* pObject3 = new TRemoteTestObject();
  pObject3->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject3);
  pObject3->setDoubleValue(20.1);
  CPPUNIT_ASSERT(Naming.rebind2(&strName, &strCat, pObject3) == RET_NOERROR);

  //Create remote object4
  strName = L"MyObject4";
  strCat = L"Cat2";
  TRemoteTestObject* pObject4 = new TRemoteTestObject();
  pObject4->connect();
  TTime::usleep(10000);
  m_lstStack.push((IRemoteObject*)pObject4);
  pObject4->setDoubleValue(20.4);
  CPPUNIT_ASSERT(Naming.rebind2(&strName, &strCat, pObject4) == RET_NOERROR);
  
  //Test if objects are available
  TStringList lstObjectNames;
  TString strCatTest = L"Cat1";
  CPPUNIT_ASSERT(Naming.getObjectNames(&strCatTest, &lstObjectNames) == RET_NOERROR);
  CPPUNIT_ASSERT(lstObjectNames.getCount() == 3);
  bool bFound1 = false;
  bool bFound2 = false;
  bool bFound3 = false;
  for(int i = 0; i < lstObjectNames.getCount(); i++)
  {
    TString strObjName(lstObjectNames[i]);
    if (strObjName == L"MyObject1")
    {
      bFound1 = true;
    }
    if (strObjName == L"MyObject2")
    {
      bFound2 = true;
    }
    if (strObjName == L"MyObject3")
    {
      bFound3 = true;
    }
  }
  CPPUNIT_ASSERT(bFound1);
  CPPUNIT_ASSERT(bFound2);
  CPPUNIT_ASSERT(bFound3);
}
