/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : NamingTest                                                    *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:39:11 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: NamingTest.cpp,v $
 *   Revision 1.3  2005/11/16 11:39:11  bhadorn
 *   New test added for getObjectNames()
 *
 *   Revision 1.2  2005/11/10 10:54:03  bhadorn
 *   test for lookupCategory() method
 *
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/Remote/NamingTest.h>
#include <zeusbase/System/StringList.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TNamingTest );


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TNamingTest::setUp()
{
  Naming.useRemote(false); //Test localy
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TNamingTest::tearDown()
{
  //Release all instances
  while(!m_lstStack.isEmpty())
  {
    m_lstStack.pop()->release();
  }
  Naming.reset();
}
          
/***************************************************************************/
/*! Tests local binding of the naming service
*/
/***************************************************************************/
void TNamingTest::test_bind()
{
  TRemoteTestObject* pObj = new TRemoteTestObject();
  m_lstStack.push(pObj);
  TString strName = L"Object1";
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj) != RET_NOERROR);
  
  TRemoteTestObject* pObj2 = new TRemoteTestObject();
  m_lstStack.push(pObj2);
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj2) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj2) != RET_NOERROR);
  
  //Test if objects are available
  IRemoteObject* pFoundObj1 = NULL;
  strName = L"Object1";
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);
  
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);
}

/***************************************************************************/
/*! Tests local binding of the naming service
*/
/***************************************************************************/
void TNamingTest::test_bind2()
{
  TRemoteTestObject* pObj = new TRemoteTestObject();
  m_lstStack.push(pObj);
  TString strName = L"Object1";
  TString strCat1 = L"Category1";
  TString strCat2 = L"Category2";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat1, pObj) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat2, pObj) == RET_NOERROR);  

  //Test if objects are available
/*  IRemoteObject* pFoundObj1 = NULL;
  strName = L"Object1";
  CPPUNIT_ASSERT(Naming.lookup2(&strName, &strCat1, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);
  
  strName = L"Object1";
  CPPUNIT_ASSERT(Naming.lookup2(&strName, &strCat2, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);*/
}

/***************************************************************************/
/*! Tests local rebinding of the naming service
*/
/***************************************************************************/
void TNamingTest::test_rebind()
{
  TRemoteTestObject* pObj = new TRemoteTestObject();
  m_lstStack.push(pObj);
  TString strName = L"Object1";
  CPPUNIT_ASSERT(Naming.rebind(&strName, pObj) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.rebind(&strName, pObj) == RET_NOERROR);
  
  TRemoteTestObject* pObj2 = new TRemoteTestObject();
  m_lstStack.push(pObj2);
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.rebind(&strName, pObj2) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.rebind(&strName, pObj2) == RET_NOERROR);

  //Test if objects are available
  IRemoteObject* pFoundObj1 = NULL;
  strName = L"Object1";
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);
  
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);
}

/***************************************************************************/
/*! Lookup test 
*/
/***************************************************************************/
void TNamingTest::test_lookup()
{
  TRemoteTestObject* pObj = new TRemoteTestObject();
  pObj->setLongValue(100);
  m_lstStack.push(pObj);
  TString strName = L"Object1";
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj) == RET_NOERROR);
  
  TRemoteTestObject* pObj2 = new TRemoteTestObject();
  pObj2->setLongValue(-100);
  m_lstStack.push(pObj2);
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj2) == RET_NOERROR);

  //Test if objects are available
  IRemoteObject* pFoundObj1 = NULL;
  strName = L"Object1";
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);
  CPPUNIT_ASSERT(((TRemoteTestObject*)pFoundObj1)->getLongValue() == 100);
  
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.lookup(&strName, &pFoundObj1) == RET_NOERROR);
  m_lstStack.push(pFoundObj1);
  CPPUNIT_ASSERT(((TRemoteTestObject*)pFoundObj1)->getLongValue() == -100);
}

/***************************************************************************/
/*! Test the unbinding method 
*/
/***************************************************************************/
void TNamingTest::test_unbind()
{
  TRemoteTestObject* pObj = new TRemoteTestObject();
  pObj->setLongValue(100);
  m_lstStack.push(pObj);
  TString strName = L"Object1";
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.unbind(&strName) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.bind(&strName, pObj) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.unbind(&strName) == RET_NOERROR);
  CPPUNIT_ASSERT(Naming.unbind(&strName) != RET_NOERROR);
}

/***************************************************************************/
/*! Test the lookupCategory method 
*/
/***************************************************************************/
void TNamingTest::test_lookupCategory()
{
  TRemoteTestObject* pObj = new TRemoteTestObject();
  pObj->setLongValue(100);
  m_lstStack.push(pObj);
  TString strName = L"Object1";
  TString strCat = L"Cat1";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj) == RET_NOERROR);
  
  TRemoteTestObject* pObj2 = new TRemoteTestObject();
  pObj2->setLongValue(100);
  m_lstStack.push(pObj2);
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj2) == RET_NOERROR);

  TRemoteTestObject* pObj3 = new TRemoteTestObject();
  pObj3->setLongValue(100);
  m_lstStack.push(pObj3);
  strName = L"Object3";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj3) == RET_NOERROR);
  
  TRemoteTestObject* pObj4 = new TRemoteTestObject();
  pObj4->setLongValue(-50);
  m_lstStack.push(pObj4);
  strName = L"Object4";
  strCat = L"Cat2";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj4) == RET_NOERROR);
  
  //Test if objects are available
  TSingleLinkedList<IRemoteObject*> lstObjects;
  TString strCatTest = L"Cat1";
  CPPUNIT_ASSERT(Naming.lookupCategory(&strCatTest, &lstObjects) == RET_NOERROR);
  for(int i = 0; i < lstObjects.getCount(); i++)
  {
    m_lstStack.push(lstObjects[i]);
  }
  CPPUNIT_ASSERT(lstObjects.getCount() == 3);
  CPPUNIT_ASSERT(((TRemoteTestObject*)lstObjects[0])->getLongValue() == 100);
  CPPUNIT_ASSERT(((TRemoteTestObject*)lstObjects[1])->getLongValue() == 100);
  CPPUNIT_ASSERT(((TRemoteTestObject*)lstObjects[2])->getLongValue() == 100);
}

/***************************************************************************/
/*! Test the getObjectNames method 
*/
/***************************************************************************/
void TNamingTest::test_getObjectNames()
{
  TRemoteTestObject* pObj = new TRemoteTestObject();
  pObj->setLongValue(100);
  m_lstStack.push(pObj);
  TString strName = L"Object1";
  TString strCat = L"Cat1";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj) == RET_NOERROR);
  
  TRemoteTestObject* pObj2 = new TRemoteTestObject();
  pObj2->setLongValue(100);
  m_lstStack.push(pObj2);
  strName = L"Object2";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj2) == RET_NOERROR);

  TRemoteTestObject* pObj3 = new TRemoteTestObject();
  pObj3->setLongValue(100);
  m_lstStack.push(pObj3);
  strName = L"Object3";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj3) == RET_NOERROR);
  
  TRemoteTestObject* pObj4 = new TRemoteTestObject();
  pObj4->setLongValue(-50);
  m_lstStack.push(pObj4);
  strName = L"Object4";
  strCat = L"Cat2";
  CPPUNIT_ASSERT(Naming.bind2(&strName, &strCat, pObj4) == RET_NOERROR);
  
  //Test if objects are available
  TStringList lstObjectNames;
  TString strCatTest = L"Cat1";
  CPPUNIT_ASSERT(Naming.getObjectNames(&strCatTest, &lstObjectNames) == RET_NOERROR);
  CPPUNIT_ASSERT(lstObjectNames.getCount() == 3);
  bool bFound1 = false;
  bool bFound2 = false;
  bool bFound3 = false;
  for(int i = 0; i < lstObjectNames.getCount(); i++)
  {
    TString strObjName(lstObjectNames[i]);
    if (strObjName == L"Object1")
    {
      bFound1 = true;
    }
    if (strObjName == L"Object2")
    {
      bFound2 = true;
    }
    if (strObjName == L"Object3")
    {
      bFound3 = true;
    }
  }
  CPPUNIT_ASSERT(bFound1);
  CPPUNIT_ASSERT(bFound2);
  CPPUNIT_ASSERT(bFound3);
}

