/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : RemoteMethodCallTest                                          *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:51:10 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RemoteMethodCallTest.cpp,v $
 *   Revision 1.1  2005/11/10 10:51:10  bhadorn
 *   created
 *
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/Remote/RemoteMethodCallTest.h>
#include <zeusbase/System/ZVariant.h>
#include <zeusbase/System/ZObjectFactory.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TRemoteMethodCallTest );


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TRemoteMethodCallTest::setUp()
{}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TRemoteMethodCallTest::tearDown()
{
  //Release all instances
  while(!m_lstStack.isEmpty())
  {
    m_lstStack.pop()->release();
  }
}
          
/***************************************************************************/
/*! Tests the creation of a remote method call
*/
/***************************************************************************/
void TRemoteMethodCallTest::test_create()
{
  TRemoteMethodCall* pMethod = new TRemoteMethodCall(L"getData");
  m_lstStack.push(pMethod);
  
  TString strMethodName;
  CPPUNIT_ASSERT(pMethod->getMethodName(&strMethodName) == RET_NOERROR);
  CPPUNIT_ASSERT(strMethodName == L"getData");
}
 
/***************************************************************************/
/*! Tests the add of parameters
*/
/***************************************************************************/
void TRemoteMethodCallTest::test_addParameter()
{
  TRemoteMethodCall* pMethod = new TRemoteMethodCall(L"getData");
  m_lstStack.push(pMethod);
  
  TZVariant* vParam1 = new TZVariant(L"name", L"Test");
  m_lstStack.push(vParam1);
  CPPUNIT_ASSERT(pMethod->addParameter(vParam1) == RET_NOERROR);
  CPPUNIT_ASSERT(pMethod->getParameterCount() == 1);

  TZVariant* vParam2 = new TZVariant(L"value", (long)10234);
  m_lstStack.push(vParam2);
  CPPUNIT_ASSERT(pMethod->addParameter(vParam2) == RET_NOERROR);
  CPPUNIT_ASSERT(pMethod->getParameterCount() == 2);

  IZVariant* vTestParam1 = NULL;
  TString strTest;
  CPPUNIT_ASSERT(pMethod->getParameter(0, &vTestParam1) == RET_NOERROR);
  m_lstStack.push(vTestParam1);
  CPPUNIT_ASSERT(vTestParam1->getType() == varString);
  CPPUNIT_ASSERT(vTestParam1->getString(&strTest) == RET_NOERROR);
  CPPUNIT_ASSERT(strTest == L"Test");
  
  IZVariant* vTestParam2 = NULL;
  long lTest;
  CPPUNIT_ASSERT(pMethod->getParameter(1,&vTestParam2) == RET_NOERROR);
  m_lstStack.push(vTestParam2);
  CPPUNIT_ASSERT(vTestParam2->getType() == varLong);
  CPPUNIT_ASSERT(vTestParam2->getLong(&lTest) == RET_NOERROR);
  CPPUNIT_ASSERT(lTest == 10234);

  IZVariant* vTestReturn = NULL;
  CPPUNIT_ASSERT(pMethod->getReturnValue(&vTestReturn) != RET_NOERROR);
}


/***************************************************************************/
/*! Tests the serializing of the remote method call
*/
/***************************************************************************/
void TRemoteMethodCallTest::test_setReturnValue()
{
  TRemoteMethodCall* pMethod = new TRemoteMethodCall(L"getData");
  m_lstStack.push(pMethod);
  
  TZVariant* vParam1 = new TZVariant(L"name", 12.23);
  m_lstStack.push(vParam1);
  
  CPPUNIT_ASSERT(pMethod->setReturnValue(vParam1) == RET_NOERROR);

  IZVariant* vTestReturn = NULL;
  double dValue;
  CPPUNIT_ASSERT(pMethod->getReturnValue(&vTestReturn) == RET_NOERROR);
  m_lstStack.push(vTestReturn);
  CPPUNIT_ASSERT(vTestReturn->getType() == varDouble);
  CPPUNIT_ASSERT(vTestReturn->getDouble(&dValue) == RET_NOERROR);
  CPPUNIT_ASSERT(dValue == 12.23);
}


/***************************************************************************/
/*! Tests the serializing of the remote method call
*/
/***************************************************************************/
void TRemoteMethodCallTest::test_serialize()
{
  TRemoteMethodCall* pMethod = new TRemoteMethodCall(L"getData");
  m_lstStack.push(pMethod);

  TZVariant* vParam1 = new TZVariant(L"name", L"Test");
  m_lstStack.push(vParam1);
  CPPUNIT_ASSERT(pMethod->addParameter(vParam1) == RET_NOERROR);

  TZVariant* vParam2 = new TZVariant(L"value", (long)10234);
  m_lstStack.push(vParam2);
  CPPUNIT_ASSERT(pMethod->addParameter(vParam2) == RET_NOERROR);
    
  TByteArray aData;
  CPPUNIT_ASSERT(pMethod->serialize(&aData) == RET_NOERROR);
  
  IRemoteMethodCall* pTestMethod = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aData, (ISerializable**)&pTestMethod) == RET_NOERROR); 
  m_lstStack.push(pTestMethod);
  
  IZVariant* vTestParam1 = NULL;
  TString strTest;
  CPPUNIT_ASSERT(pTestMethod->getParameter(0, &vTestParam1) == RET_NOERROR);
  m_lstStack.push(vTestParam1);
  CPPUNIT_ASSERT(vTestParam1->getType() == varString);
  CPPUNIT_ASSERT(vTestParam1->getString(&strTest) == RET_NOERROR);
  CPPUNIT_ASSERT(strTest == L"Test");
  
  IZVariant* vTestParam2 = NULL;
  long lTest;
  CPPUNIT_ASSERT(pTestMethod->getParameter(1,&vTestParam2) == RET_NOERROR);
  m_lstStack.push(vTestParam2);
  CPPUNIT_ASSERT(vTestParam2->getType() == varLong);
  CPPUNIT_ASSERT(vTestParam2->getLong(&lTest) == RET_NOERROR);
  CPPUNIT_ASSERT(lTest == 10234);

  IZVariant* vTestReturn = NULL;
  CPPUNIT_ASSERT(pTestMethod->getReturnValue(&vTestReturn) != RET_NOERROR);
}
