/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : NamingTest                                                    *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:53:18 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RemoteTestObject.cpp,v $
 *   Revision 1.2  2005/11/10 10:53:18  bhadorn
 *   Test object for new RMI concept
 *
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/Remote/RemoteTestObject.h>
#include <zeusbase/System/LoggerManager.h>
 
USING_NAMESPACE_Zeus

//////////////////////////////////////////////////////////////////////////////
// SKELETON
/***************************************************************************/
/* Dispatching of the incomming method calls
*/
/***************************************************************************/
SKEL_DISPATCH_START(TRemoteTestObject_Skel)
  //------------------------------------------------------------------------
  // Dispatching of getDoubleValue
  SKEL_DISP_SWITCH(L"getDoubleValue")
    if (m_pInstance != NULL)
    {
      pVarRetval->setDouble(m_pInstance->getDoubleValue());
    }
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
    }
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of getDoubleValue
  SKEL_DISP_SWITCH(L"getLongValue")
    if (m_pInstance != NULL)
    {
      pVarRetval->setLong(m_pInstance->getLongValue());
    }
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
    }
  SKEL_DISP_SWITCH_END
  
  //------------------------------------------------------------------------
  // Dispatching of getDoubleValue
  SKEL_DISP_SWITCH(L"getStringValue")
    if (m_pInstance != NULL)
    {
      TString strValue;
      pVarRetval->setLong((long)m_pInstance->getStringValue(&strValue));
      
      TZVariant* pRetParam = new TZVariant("str", strValue);
      pReturnCall->addParameter(pRetParam);
      pRetParam->release();
    }
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
    }
  SKEL_DISP_SWITCH_END

SKEL_DISPATCH_END
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// STUB
TRemoteTestObject_Stub::TRemoteTestObject_Stub() : TAbstractStub()
{}

TRemoteTestObject_Stub::~TRemoteTestObject_Stub() {}



/***************************************************************************/
/* Stub implementation. All methods of the Interface INaming (and sub 
   interfaces) must be declared here.
*/
/***************************************************************************/
//Methods of INaming
STUB_DELEGATION("getDoubleValue", double MQUALIFIER TRemoteTestObject_Stub::getDoubleValue())
  double dValue = 0.0;
  STUB_RETURN_START;
    STUB_RETVALUE_DOUBLE(dValue)
  STUB_RETURN_END(dValue);

STUB_DELEGATION("getLongValue", long MQUALIFIER TRemoteTestObject_Stub::getLongValue())
  long lValue = 0;
  STUB_RETURN_START;
    STUB_RETVALUE_LONG(lValue)
  STUB_RETURN_END(lValue);

STUB_DELEGATION("getStringValue", unsigned long MQUALIFIER TRemoteTestObject_Stub::getStringValue(IString* str))
  STUB_RETURN_START;
    STUB_RETPARAM_STRING("str", str);
    STUB_RETVALUE_LONG(ulRetval)
  STUB_RETURN_END(ulRetval);
  
STUB_DELEGATION("setDoubleValue", void MQUALIFIER TRemoteTestObject_Stub::setDoubleValue(double value))
  STUB_PARAM("value", value);
  STUB_RETURN_VOID;
  
STUB_DELEGATION("setLongValue", void MQUALIFIER TRemoteTestObject_Stub::setLongValue(long value))
  STUB_PARAM("value", value);
  STUB_RETURN_VOID;
  
STUB_DELEGATION("setStringValue", void MQUALIFIER TRemoteTestObject_Stub::setStringValue(const IString* value))
  STUB_PARAM("value", value);
  STUB_RETURN_VOID;    
  
MEMORY_MANAGER_IMPL(TRemoteTestObject_Stub);
  INTERFACE_CAST(IRemoteTestObject, INTERFACE_IRemoteTestObject);
MEMORY_MANAGER_IMPL_PARENT_END(TAbstractStub);
//////////////////////////////////////////////////////////////////////////////



/***************************************************************************/   
/*! Creates the remote object
*/
/***************************************************************************/   
TRemoteTestObject::TRemoteTestObject() : TAbstractRemoteObject()
{
  m_dValue = 0.0;
  m_lValue= 0;
}

/***************************************************************************/   
/*! Destroys the remote object
*/
/***************************************************************************/   
TRemoteTestObject::~TRemoteTestObject()
{}

/***************************************************************************/   
/*! Returns the double value
*/
/***************************************************************************/   
double MQUALIFIER TRemoteTestObject::getDoubleValue()
{ return m_dValue; }

/***************************************************************************/   
/*! Returns the long value
*/
/***************************************************************************/   
long MQUALIFIER TRemoteTestObject::getLongValue()
{ return m_lValue; }

/***************************************************************************/   
/*! Returns the string value
*/
/***************************************************************************/   
unsigned long MQUALIFIER TRemoteTestObject::getStringValue(IString* str)
{ 
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (str != NULL)
  {
    str->assignStr(&m_strValue);
    ulRetval = RET_NOERROR;
  }
  return ulRetval; 
}

/***************************************************************************/   
/*! sets the double value
*/
/***************************************************************************/   
void MQUALIFIER TRemoteTestObject::setDoubleValue(double value)
{ m_dValue = value; }

/***************************************************************************/   
/*! sets the long value
*/
/***************************************************************************/   
void MQUALIFIER TRemoteTestObject::setLongValue(long value)
{ m_lValue = value; }

/***************************************************************************/   
/*! sets the string value
*/
/***************************************************************************/   
void MQUALIFIER TRemoteTestObject::setStringValue(const IString* value)
{ 
  m_strValue = value; 
}

REMOTE_OBJECT_IMPL(TRemoteTestObject, L"TRemoteTestObject", L"", TRemoteTestObject_Skel);

MEMORY_MANAGER_IMPL(TRemoteTestObject);
  INTERFACE_CAST(IRemoteTestObject, INTERFACE_IRemoteTestObject);
MEMORY_MANAGER_IMPL_PARENT_END(TAbstractRemoteObject);
