/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : NamingTest                                                    *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:53:18 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RemoteTestObject.h,v $
 *   Revision 1.2  2005/11/10 10:53:18  bhadorn
 *   Test object for new RMI concept
 *
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#ifndef RemoteTestObjectH
#define RemoteTestObjectH
 
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ServerSocket.h>
#include <zeusbase/Remote/Interfaces/IRemoteObject.hpp>
#include <zeusbase/Remote/AbstractRemoteObject.h>
#include <zeusbase/Remote/AbstractSkel.h>
#include <zeusbase/Remote/AbstractStub.h>

BEGIN_NAMESPACE_Zeus

#define INTERFACE_IRemoteTestObject      0x00FFFFFF

class IRemoteTestObject : public IZUnknown
{
  public:  
    virtual double MQUALIFIER getDoubleValue()=0;
    virtual long MQUALIFIER getLongValue()=0;
    virtual unsigned long MQUALIFIER getStringValue(IString* str)=0;
    virtual void MQUALIFIER setDoubleValue(double value)=0;
    virtual void MQUALIFIER setLongValue(long value)=0;
    virtual void MQUALIFIER setStringValue(const IString* value)=0;

}; 

class TRemoteTestObject_Skel;

/***************************************************************************/
/*! Test object witch simulates a remote object
*/
/***************************************************************************/
class TRemoteTestObject : public TAbstractRemoteObject, public IRemoteTestObject
{
  public:
    TRemoteTestObject();
    
    //Methods of IRemoteTestObject
    virtual double MQUALIFIER getDoubleValue();
    virtual long MQUALIFIER getLongValue();
    virtual unsigned long MQUALIFIER getStringValue(IString* str);
    virtual void MQUALIFIER setDoubleValue(double value);
    virtual void MQUALIFIER setLongValue(long value);
    virtual void MQUALIFIER setStringValue(const IString* value);
        
    //Methods of IRemoteObject
    REMOTE_OBJECT_DECL
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual~ TRemoteTestObject();
  
  private:
    ///Double value
    double m_dValue;
    ///Long value
    long m_lValue;
    ///String value
    TString m_strValue;
 
};

///////////////////////////////////////////////////////////////////////////////////
// SKELETON
SKEL_DECL(TRemoteTestObject_Skel, TRemoteTestObject);
///////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////
// STUB
class TRemoteTestObject_Stub : public TAbstractStub, public IRemoteTestObject
{
  public:
    TRemoteTestObject_Stub();
    
    //Methods of IRemoteTestObject
    virtual double MQUALIFIER getDoubleValue();
    virtual long MQUALIFIER getLongValue();
    virtual unsigned long MQUALIFIER getStringValue(IString* str);
    virtual void MQUALIFIER setDoubleValue(double value);
    virtual void MQUALIFIER setLongValue(long value);
    virtual void MQUALIFIER setStringValue(const IString* value);
    
    STUB_CLASS_DECL(TRemoteTestObject_Stub, "TRemoteTestObject_Stub");
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TRemoteTestObject_Stub();
    
  private :
};
///////////////////////////////////////////////////////////////////////////////////

END_NAMESPACE_Zeus

#endif
