/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FingerPrintTest                                               *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:18 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FingerPrintTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#include <zeustest/Security/FingerPrintTest.h> 
#include <zeusbase/Security/FingerPrint.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TFingerPrintTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TFingerPrintTest::setUp()
{}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TFingerPrintTest::tearDown()
{}

/***************************************************************************/
/*! Tests the print generation
*/
/***************************************************************************/
void TFingerPrintTest::test_generatePrint()
{
  TFingerPrint Print;
  TByteArray aKey;
   
  //No finger print available yet
  CPPUNIT_ASSERT(Print.getFingerPrint(NULL) == RET_INVALID_PARAMETER);
  CPPUNIT_ASSERT(Print.getFingerPrint(&aKey) == RET_REQUEST_FAILED);

  TString strName = L"Test";
  TString strData = L"Hello World";
  TByteArray aData = TString::transcode(strData);
  CPPUNIT_ASSERT(Print.generatePrint(strName, &aData) == RET_NOERROR);
  
  TFingerPrint Print2;
  CPPUNIT_ASSERT(Print2.generatePrint(strName, strData) == RET_NOERROR);
  CPPUNIT_ASSERT(Print2 == Print);
  
  TFingerPrint Print3;
  strData = L"Hello world"; //Different text
  CPPUNIT_ASSERT(Print3.generatePrint(strName, strData) == RET_NOERROR);
  CPPUNIT_ASSERT(Print3 != Print);
}


/***************************************************************************/
/*! Tests the print generation
*/
/***************************************************************************/
void TFingerPrintTest::test_generatePrintFromFile()
{
  TFingerPrint Print;
  TString strFileName = L"./zeus.xml";
  TString strName = L"Test";
  
  CPPUNIT_ASSERT(Print.generatePrintFromFile(strName, strFileName) == RET_NOERROR);

  TFingerPrint Print2;
  strFileName = L"./cellreg223453.xml";
  
  CPPUNIT_ASSERT(Print2.generatePrintFromFile(strName, strFileName) != RET_NOERROR);
}
