/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SecurityManagerTest                                           *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:03:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SecurityManagerTest.cpp,v $
 *   Revision 1.3  2005/11/25 10:03:56  bhadorn
 *   Compiler problems with borland
 *
 *   Revision 1.2  2005/11/16 11:41:05  bhadorn
 *   Changed size of file checking to 4000 bytes. This will improve the speed when testing.
 *
 *   Revision 1.1  2005/10/28 12:23:18  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/Security/SecurityManagerTest.h> 
#include <zeusbase/Security/SecurityManager.h>
#include <zeusbase/System/File.h>
#include <zeusbase/System/StringList.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TSecurityManagerTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TSecurityManagerTest::setUp()
{
  TSecurityManager::getInstance().setMaxFileSize(4000);
  #ifdef LINUX
    TSecurityManager::getInstance().loadRegistry(L"./TestSecurity.db");
  #else
    TSecurityManager::getInstance().loadRegistry(L".\\TestSecurity.db");
  #endif
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TSecurityManagerTest::tearDown()
{
  TSecurityManager::getInstance().loadRegistry(L"");
  #ifdef LINUX
    TFile SecurityFile(L"./TestSecurity.db");
  #else
    TFile SecurityFile(L".\\TestSecurity.db");
  #endif
  SecurityFile.deleteFile();
  TSecurityManager::getInstance().setMaxFileSize(2000000);
}

/***************************************************************************/
/*! Tests the loading of a virgin data base
*/
/***************************************************************************/
void TSecurityManagerTest::test_loadRegistry()
{
  TSecurityManager& Security = TSecurityManager::getInstance();
  TStringList lstModules;
  
  #ifdef LINUX
    TFile SecurityFile(L"./TestSecurity.db");
  #else
    TFile SecurityFile(L".\\TestSecurity.db");
  #endif
  SecurityFile.deleteFile();
  CPPUNIT_ASSERT(!SecurityFile.exists());
  
  unsigned long ulRetval = RET_NOERROR;
  #ifdef LINUX
    CPPUNIT_ASSERT(Security.loadRegistry(L"./TestSecurity.db") == ulRetval);
  #else
    CPPUNIT_ASSERT(Security.loadRegistry(L".\\TestSecurity.db") == ulRetval);
  #endif
  CPPUNIT_ASSERT(SecurityFile.exists());
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.isEmpty());
}

/***************************************************************************/
/*! Tests the registration of a code module
*/
/***************************************************************************/
void TSecurityManagerTest::test_registerModule()
{
  TSecurityManager& Security = TSecurityManager::getInstance();
  TStringList lstModules;
  #ifdef LINUX
    TString strModuleName = L"./modules/TestModule.so";
    TString strModuleName1 = L"./modules/TestModule1.so";
  #else
    TString strModuleName = L".\\modules\\TestModule.dll";
    TString strModuleName1 = L".\\modules\\TestModule1.dll";
  #endif
  
  lstModules.clear();
  CPPUNIT_ASSERT(Security.registerModule(strModuleName,strModuleName) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 1);
  CPPUNIT_ASSERT(lstModules[0]->equalsStr(&strModuleName));

  //Can not register the same module twice
  lstModules.clear();
  CPPUNIT_ASSERT(Security.registerModule(strModuleName,strModuleName) != RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 1);

  //Module file is not valid
  lstModules.clear();
  CPPUNIT_ASSERT(Security.registerModule(strModuleName1,strModuleName1) != RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 1);

  //Second module to register
  lstModules.clear();
  CPPUNIT_ASSERT(Security.registerModule(strModuleName1,strModuleName) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 2);
}

/***************************************************************************/
/*! Tests the unregistration of a code module
*/
/***************************************************************************/
void TSecurityManagerTest::test_unregisterModule()
{
  TSecurityManager& Security = TSecurityManager::getInstance();
  TStringList lstModules;
  #ifdef LINUX
    TString strModuleName = L"./modules/TestModule.so";
    TString strModuleName1 = L"./modules/TestModule1.so";
    TString strModuleName2 = L"./modules/TestModule2.so";
  #else
    TString strModuleName = L".\\modules\\TestModule.dll";
    TString strModuleName1 = L".\\modules\\TestModule1.dll";
    TString strModuleName2 = L".\\modules\\TestModule2.dll";
  #endif
  
  //Register 2 modules
  lstModules.clear();
  CPPUNIT_ASSERT(Security.registerModule(strModuleName,strModuleName) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.registerModule(strModuleName1,strModuleName) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 2);

  //Unregister a non existing module
  lstModules.clear();
  CPPUNIT_ASSERT(Security.unregisterModule(strModuleName2) != RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 2);

  //Unregister a existing module
  lstModules.clear();
  CPPUNIT_ASSERT(Security.unregisterModule(strModuleName1) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 1);
  CPPUNIT_ASSERT(lstModules[0]->equalsStr(&strModuleName));
  
  //Can not unregister a module twice
  lstModules.clear();
  CPPUNIT_ASSERT(Security.unregisterModule(strModuleName1) != RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 1);
  
  //Unregister a existing module
  lstModules.clear();
  CPPUNIT_ASSERT(Security.unregisterModule(strModuleName) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.isEmpty());
}


/***************************************************************************/
/*! Test the evaluation of a code module
*/
/***************************************************************************/
void TSecurityManagerTest::test_evalCodeModule()
{
  TSecurityManager& Security = TSecurityManager::getInstance();
  TStringList lstModules;
  #ifdef LINUX  
    TString strModuleName = L"./modules/TestModule.so";
    TString strModuleName1 = L"./modules/TestModule1.so";
    TString strModuleName2 = L"./modules/TestModule2.so";
    TString strModuleName3 = L"./modules/XML_Service.so";
  #else
    TString strModuleName = L".\\modules\\TestModule.dll";
    TString strModuleName1 = L".\\modules\\TestModule1.dll";
    TString strModuleName2 = L".\\modules\\TestModule2.dll";
    TString strModuleName3 = L".\\modules\\XML_Service.dll";
  #endif
  
  //Register 2 modules
  lstModules.clear();
  CPPUNIT_ASSERT(Security.registerModule(strModuleName,strModuleName) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.registerModule(strModuleName1,strModuleName) == RET_NOERROR);
  CPPUNIT_ASSERT(Security.getModules(&lstModules) == RET_NOERROR);
  CPPUNIT_ASSERT(lstModules.getCount() == 2);

  //Correct evaluations
  CPPUNIT_ASSERT(Security.evalCodeModule(strModuleName, strModuleName));
  CPPUNIT_ASSERT(Security.evalCodeModule(strModuleName1, strModuleName));
  
  //Not a valid file
  CPPUNIT_ASSERT(!Security.evalCodeModule(strModuleName, strModuleName1));
  
  //Not a registered module
  CPPUNIT_ASSERT(!Security.evalCodeModule(strModuleName2, strModuleName));
  
  //Valid file but invalid finger print
/*  CPPUNIT_ASSERT(!Security.evalCodeModule(strModuleName, strModuleName3));*/
}

