/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ArrayListTest                                                 *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:55:05 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ArrayListTest.cpp,v $
 *   Revision 1.3  2005/12/14 08:55:05  bhadorn
 *   Tests for iterators
 *
 *   Revision 1.2  2005/11/10 10:45:02  bhadorn
 *   no message
 *
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#include <zeustest/System/ArrayListTest.h> 
#include <zeusbase/System/ArrayList.hpp>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TArrayListTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TArrayListTest::setUp()
{
  m_pList = NULL;
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TArrayListTest::tearDown()
{
  if (m_pList != NULL)
  {
    while(!m_lstIts.isEmpty())
    {
      m_pList->releaseIterator(m_lstIts.pop());
    }
    m_pList->release();
    m_pList = NULL;
  }
}

/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TArrayListTest::test_create()
{
  TArrayList<long> lstLong30(30);
  CPPUNIT_ASSERT(lstLong30.getArraySize() == 30); 
  CPPUNIT_ASSERT(lstLong30.getCount() == 0); 
  CPPUNIT_ASSERT(lstLong30[0] == 0); 
  
  TArrayList<long> lstLong200(99, 200);
  CPPUNIT_ASSERT(lstLong200.getArraySize() == 200); 
  CPPUNIT_ASSERT(lstLong200.getCount() == 0); 
  CPPUNIT_ASSERT(lstLong200[0] == 99); 

  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong5(aData,5);
  CPPUNIT_ASSERT(lstLong5.getArraySize() > 5); 
  CPPUNIT_ASSERT(lstLong5.getCount() == 5); 
  CPPUNIT_ASSERT(lstLong5[0] == 1); 
  CPPUNIT_ASSERT(lstLong5[1] == 2); 
  CPPUNIT_ASSERT(lstLong5[2] == 3); 
  CPPUNIT_ASSERT(lstLong5[3] == 4); 
  CPPUNIT_ASSERT(lstLong5[4] == 5); 
  
  TArrayList<long> lstLong5_1(lstLong5);
  CPPUNIT_ASSERT(lstLong5_1.getArraySize() > 5); 
  CPPUNIT_ASSERT(lstLong5_1.getCount() == 5); 
  CPPUNIT_ASSERT(lstLong5_1[0] == 1); 
  CPPUNIT_ASSERT(lstLong5_1[1] == 2); 
  CPPUNIT_ASSERT(lstLong5_1[2] == 3); 
  CPPUNIT_ASSERT(lstLong5_1[3] == 4); 
  CPPUNIT_ASSERT(lstLong5_1[4] == 5); 

  TArrayList<long> lstLong5_2(&lstLong5);
  CPPUNIT_ASSERT(lstLong5_2.getArraySize() > 5); 
  CPPUNIT_ASSERT(lstLong5_2.getCount() == 5); 
  CPPUNIT_ASSERT(lstLong5_2[0] == 1); 
  CPPUNIT_ASSERT(lstLong5_2[1] == 2); 
  CPPUNIT_ASSERT(lstLong5_2[2] == 3); 
  CPPUNIT_ASSERT(lstLong5_2[3] == 4); 
  CPPUNIT_ASSERT(lstLong5_2[4] == 5); 
}
 
/***************************************************************************/
/*! Tests the add-method
*/
/***************************************************************************/
void TArrayListTest::test_add()
{
  TArrayList<long> lstLong(3);
  CPPUNIT_ASSERT(lstLong.getArraySize() == 3); 
  CPPUNIT_ASSERT(lstLong.getCount() == 0); 
  CPPUNIT_ASSERT(lstLong.add(100) == 0); 
  CPPUNIT_ASSERT(lstLong.getCount() == 1); 
  CPPUNIT_ASSERT(lstLong[0] == 100); 
  CPPUNIT_ASSERT(lstLong[1] == 0);  
  
  CPPUNIT_ASSERT(lstLong.add(200) == 1); 
  CPPUNIT_ASSERT(lstLong.getArraySize() == 3); 
  CPPUNIT_ASSERT(lstLong.getCount() == 2); 
  CPPUNIT_ASSERT(lstLong[0] == 100); 
  CPPUNIT_ASSERT(lstLong[1] == 200);  
  CPPUNIT_ASSERT(lstLong[2] == 0);  
  
  CPPUNIT_ASSERT(lstLong.add(300) == 2); 
  CPPUNIT_ASSERT(lstLong.getArraySize() == 3); 
  CPPUNIT_ASSERT(lstLong.getCount() == 3); 
  CPPUNIT_ASSERT(lstLong[0] == 100); 
  CPPUNIT_ASSERT(lstLong[1] == 200);  
  CPPUNIT_ASSERT(lstLong[2] == 300);  
  CPPUNIT_ASSERT(lstLong[3] == 0);  
  
  //Now the array has to be resized
  CPPUNIT_ASSERT(lstLong.add(400) == 3); 
  CPPUNIT_ASSERT(lstLong.getArraySize() == 4); 
  CPPUNIT_ASSERT(lstLong.getCount() == 4); 
  CPPUNIT_ASSERT(lstLong[0] == 100); 
  CPPUNIT_ASSERT(lstLong[1] == 200);  
  CPPUNIT_ASSERT(lstLong[2] == 300);  
  CPPUNIT_ASSERT(lstLong[3] == 400);  
  CPPUNIT_ASSERT(lstLong[4] == 0);  
}

/***************************************************************************/
/*! Tests the add-method
*/
/***************************************************************************/
void TArrayListTest::test_remove()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);
  CPPUNIT_ASSERT(lstLong.getCount() == 5); 
  
  lstLong.remove(3);
  CPPUNIT_ASSERT(lstLong.getCount() == 4);
  CPPUNIT_ASSERT(lstLong[0] == 1); 
  CPPUNIT_ASSERT(lstLong[1] == 2);  
  CPPUNIT_ASSERT(lstLong[2] == 4);  
  CPPUNIT_ASSERT(lstLong[3] == 5);  
  CPPUNIT_ASSERT(lstLong[4] == 0);  
  
  //Nothing should change (3 is not here any more
  lstLong.remove(3);
  CPPUNIT_ASSERT(lstLong.getCount() == 4);
  CPPUNIT_ASSERT(lstLong[0] == 1); 
  CPPUNIT_ASSERT(lstLong[1] == 2);  
  CPPUNIT_ASSERT(lstLong[2] == 4);  
  CPPUNIT_ASSERT(lstLong[3] == 5);  
  CPPUNIT_ASSERT(lstLong[4] == 0);  

  lstLong.remove(1);
  CPPUNIT_ASSERT(lstLong.getCount() == 3);
  CPPUNIT_ASSERT(lstLong[0] == 2); 
  CPPUNIT_ASSERT(lstLong[1] == 4);  
  CPPUNIT_ASSERT(lstLong[2] == 5);  
  CPPUNIT_ASSERT(lstLong[3] == 0);  

  lstLong.remove(5);
  CPPUNIT_ASSERT(lstLong.getCount() == 2);
  CPPUNIT_ASSERT(lstLong[0] == 2);  
  CPPUNIT_ASSERT(lstLong[1] == 4);  
  CPPUNIT_ASSERT(lstLong[2] == 0);  

  lstLong.remove(2);
  CPPUNIT_ASSERT(lstLong.getCount() == 1);
  CPPUNIT_ASSERT(lstLong[0] == 4);  
  CPPUNIT_ASSERT(lstLong[1] == 0);  

  lstLong.remove(4);
  CPPUNIT_ASSERT(lstLong.getCount() == 0);
  CPPUNIT_ASSERT(lstLong[0] == 0);  

  lstLong.remove(4);
  CPPUNIT_ASSERT(lstLong.getCount() == 0);
  CPPUNIT_ASSERT(lstLong[0] == 0);  
}

/***************************************************************************/
/*! Tests the delete-method
*/
/***************************************************************************/
void TArrayListTest::test_deleteItem()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);
  CPPUNIT_ASSERT(lstLong.getCount() == 5); 
  
  lstLong.deleteItem(2);
  CPPUNIT_ASSERT(lstLong.getCount() == 4);
  CPPUNIT_ASSERT(lstLong[0] == 1); 
  CPPUNIT_ASSERT(lstLong[1] == 2);  
  CPPUNIT_ASSERT(lstLong[2] == 4);  
  CPPUNIT_ASSERT(lstLong[3] == 5);  
  CPPUNIT_ASSERT(lstLong[4] == 0);  
  
  //Nothing should change (3 is not here any more
  lstLong.deleteItem(-1);
  CPPUNIT_ASSERT(lstLong.getCount() == 4);
  CPPUNIT_ASSERT(lstLong[0] == 1); 
  CPPUNIT_ASSERT(lstLong[1] == 2);  
  CPPUNIT_ASSERT(lstLong[2] == 4);  
  CPPUNIT_ASSERT(lstLong[3] == 5);  
  CPPUNIT_ASSERT(lstLong[4] == 0);  

  lstLong.deleteItem(0);
  CPPUNIT_ASSERT(lstLong.getCount() == 3);
  CPPUNIT_ASSERT(lstLong[0] == 2); 
  CPPUNIT_ASSERT(lstLong[1] == 4);  
  CPPUNIT_ASSERT(lstLong[2] == 5);  
  CPPUNIT_ASSERT(lstLong[3] == 0);  

  lstLong.deleteItem(2);
  CPPUNIT_ASSERT(lstLong.getCount() == 2);
  CPPUNIT_ASSERT(lstLong[0] == 2);  
  CPPUNIT_ASSERT(lstLong[1] == 4);  
  CPPUNIT_ASSERT(lstLong[2] == 0);  

  lstLong.deleteItem(0);
  CPPUNIT_ASSERT(lstLong.getCount() == 1);
  CPPUNIT_ASSERT(lstLong[0] == 4);  
  CPPUNIT_ASSERT(lstLong[1] == 0);  

  lstLong.deleteItem(0);
  CPPUNIT_ASSERT(lstLong.getCount() == 0);
  CPPUNIT_ASSERT(lstLong[0] == 0);  

  lstLong.deleteItem(0);
  CPPUNIT_ASSERT(lstLong.getCount() == 0);
  CPPUNIT_ASSERT(lstLong[0] == 0);  
}

/***************************************************************************/
/*! Tests the equalsItem-method
*/
/***************************************************************************/
void TArrayListTest::test_equalsItem()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);
  
  CPPUNIT_ASSERT(lstLong.equalsItem(0, 1));
  CPPUNIT_ASSERT(lstLong.equalsItem(1, 2));
  CPPUNIT_ASSERT(lstLong.equalsItem(2, 3));
  CPPUNIT_ASSERT(lstLong.equalsItem(3, 4));
  CPPUNIT_ASSERT(lstLong.equalsItem(4, 5));
    
  CPPUNIT_ASSERT(!lstLong.equalsItem(5, 0));
  CPPUNIT_ASSERT(!lstLong.equalsItem(-1, 0));
  CPPUNIT_ASSERT(!lstLong.equalsItem(0, 2));
  CPPUNIT_ASSERT(!lstLong.equalsItem(4, 4));
}

/***************************************************************************/
/*! Tests the equals-method
*/
/***************************************************************************/
void TArrayListTest::test_equals()
{
  long aData1[5] = {1,2,3,4,5};
  long aData2[9] = {1,2,3,4,5,3,5543,21,23};
  TArrayList<long> lstLong1(aData1,5);
  TArrayList<long> lstLong2(aData2,9);
  TArrayList<long> lstLong3(89);
  lstLong3.add(1);
  lstLong3.add(2);
  lstLong3.add(3);
  lstLong3.add(4);
  lstLong3.add(5);
  
  CPPUNIT_ASSERT(lstLong1.equals(&lstLong3));
  CPPUNIT_ASSERT(!lstLong2.equals(&lstLong3));
  CPPUNIT_ASSERT(!lstLong1.equals(&lstLong2));
}

/***************************************************************************/
/*! Tests the getItem-method
*/
/***************************************************************************/
void TArrayListTest::test_getItem()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);
  CPPUNIT_ASSERT(lstLong.getItem(0) == 1);
  CPPUNIT_ASSERT(lstLong.getItem(1) == 2);
  CPPUNIT_ASSERT(lstLong.getItem(2) == 3);
  CPPUNIT_ASSERT(lstLong.getItem(3) == 4);
  CPPUNIT_ASSERT(lstLong.getItem(4) == 5);

  lstLong.getItem(4) = 8;
  CPPUNIT_ASSERT(lstLong.getItem(4) == 8);
}

/***************************************************************************/
/*! Tests the getItemCopy-method
*/
/***************************************************************************/
void TArrayListTest::test_getItemCopy()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);
  for(int i = -1; i < 6; i++)
  {
    CPPUNIT_ASSERT(lstLong.getItem(i) == lstLong.getItemCopy(i));
  }
}

/***************************************************************************/
/*! Tests the copyToList-method
*/
/***************************************************************************/
void TArrayListTest::test_copyToList()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);
  TArrayList<long> lstDest(20);
  lstLong.copyToList(&lstDest);
  
  CPPUNIT_ASSERT(lstLong.getItem(0) == lstDest.getItem(0));
  CPPUNIT_ASSERT(lstLong.getItem(1) == lstDest.getItem(1));
  CPPUNIT_ASSERT(lstLong.getItem(2) == lstDest.getItem(2));
  CPPUNIT_ASSERT(lstLong.getItem(3) == lstDest.getItem(3));
  CPPUNIT_ASSERT(lstLong.getItem(4) == lstDest.getItem(4));
}


/***************************************************************************/
/*! Tests the indexOf-method
*/
/***************************************************************************/
void TArrayListTest::test_indexOf()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);
  
  CPPUNIT_ASSERT(lstLong.indexOf(0) == -1);
  CPPUNIT_ASSERT(lstLong.indexOf(1) == 0);
  CPPUNIT_ASSERT(lstLong.indexOf(2) == 1);
  CPPUNIT_ASSERT(lstLong.indexOf(3) == 2);
  CPPUNIT_ASSERT(lstLong.indexOf(4) == 3);
  CPPUNIT_ASSERT(lstLong.indexOf(5) == 4);
  CPPUNIT_ASSERT(lstLong.indexOf(6) == -1);
}

/***************************************************************************/
/*! Tests the getArray-method
*/
/***************************************************************************/
void TArrayListTest::test_getArray()
{
  long aData[5] = {1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);

  long* pBuffer = lstLong.getArray();
  CPPUNIT_ASSERT(pBuffer[0] == 1);
  CPPUNIT_ASSERT(pBuffer[1] == 2);
  CPPUNIT_ASSERT(pBuffer[2] == 3);
  CPPUNIT_ASSERT(pBuffer[3] == 4);
  CPPUNIT_ASSERT(pBuffer[4] == 5);
}

/***************************************************************************/
/*! Tests the getArray-method
*/
/***************************************************************************/
void TArrayListTest::test_setArray()
{
  long aData[5] = {1,2,3,4,5};
  long aDataNew[8] = {8,9,10,1,2,3,4,5};
  TArrayList<long> lstLong(aData,5);

  CPPUNIT_ASSERT(lstLong.setArray(aDataNew, 8) == RET_NOERROR);
  CPPUNIT_ASSERT(lstLong[0] == 8);
  CPPUNIT_ASSERT(lstLong[1] == 9);
  CPPUNIT_ASSERT(lstLong[2] == 10);
  CPPUNIT_ASSERT(lstLong[3] == 1);
  CPPUNIT_ASSERT(lstLong[4] == 2);
  CPPUNIT_ASSERT(lstLong[5] == 3);
  CPPUNIT_ASSERT(lstLong[6] == 4);
  CPPUNIT_ASSERT(lstLong[7] == 5);
  CPPUNIT_ASSERT(lstLong[8] == 0);
}

/***************************************************************************/
/*! Tests the appendArray-method
*/
/***************************************************************************/
void TArrayListTest::test_appendArray()
{
  long aData[5] = {1,2,3,4,5};
  long aDataApp[8] = {8,9,10};
  TArrayList<long> lstLong(aData,5);
  CPPUNIT_ASSERT(lstLong.appendArray(aDataApp, 3) == RET_NOERROR);
  CPPUNIT_ASSERT(lstLong[0] == 1);
  CPPUNIT_ASSERT(lstLong[1] == 2);
  CPPUNIT_ASSERT(lstLong[2] == 3);
  CPPUNIT_ASSERT(lstLong[3] == 4);
  CPPUNIT_ASSERT(lstLong[4] == 5);
  CPPUNIT_ASSERT(lstLong[5] == 8);
  CPPUNIT_ASSERT(lstLong[6] == 9);
  CPPUNIT_ASSERT(lstLong[7] == 10);
  CPPUNIT_ASSERT(lstLong[8] == 0);
}


/***************************************************************************/
/*! Tests the copyToArray-method
*/
/***************************************************************************/
void TArrayListTest::test_copyToArray()
{
  long aData[10] = {1,2,3,4,5,6,7,8,9,10};
  TArrayList<long> lstLong(aData,10);
  TArrayList<long> lstDest(20);
  
  CPPUNIT_ASSERT(lstLong.copyToArray(&lstDest, 0,9) == RET_NOERROR);
  CPPUNIT_ASSERT(lstLong == lstDest);
  
  CPPUNIT_ASSERT(lstLong.copyToArray(&lstDest, 2,5) == RET_NOERROR);
  CPPUNIT_ASSERT(lstDest[0] == 3);
  CPPUNIT_ASSERT(lstDest[1] == 4);
  CPPUNIT_ASSERT(lstDest[2] == 5);
  CPPUNIT_ASSERT(lstDest[3] == 6);
  CPPUNIT_ASSERT(lstDest[4] == 0);

  CPPUNIT_ASSERT(lstLong.copyToArray(&lstDest, 2,2) == RET_NOERROR);
  CPPUNIT_ASSERT(lstDest[0] == 3);

  CPPUNIT_ASSERT(lstLong.copyToArray(&lstDest, -1,5) == RET_REQUEST_FAILED);
}

/***************************************************************************/
/*! Tests the copyToArray-method
*/
/***************************************************************************/
void TArrayListTest::test_assign()
{
  long aData[10] = {1,2,3,4,5,6,7,8,9,10};
  TArrayList<long> lstLong(aData,10);
  TArrayList<long> lstDest(20);
  lstDest = &lstLong;
  
  CPPUNIT_ASSERT(lstDest.getCount() == lstLong.getCount());
  CPPUNIT_ASSERT(lstDest == lstLong);
}


/***************************************************************************/
/*! Tests the iterator 
*/
/***************************************************************************/
void TArrayListTest::test_getIterator()
{
  m_pList = new TArrayList<long>();
  IListIterator<long>* pIt = m_pList->getIterator();
  m_lstIts.push(pIt);
}

/***************************************************************************/
/*! Tests the const_iterator
*/
/***************************************************************************/
void TArrayListTest::test_getConstIterator()
{

}

