/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ObjectStreamTest                                              *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ByteArrayInputStreamTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#include <zeustest/System/ByteArrayInputStreamTest.h> 
#include <stdio.h>
USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TByteArrayInputStreamTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TByteArrayInputStreamTest::setUp()
{
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TByteArrayInputStreamTest::tearDown()
{
  //Release all instances
  while(!m_lstOutObjRef.isEmpty())
  {
    m_lstOutObjRef.pop()->release();
  }
  //Release all instances
  while(!m_lstInObjRef.isEmpty())
  {
    m_lstInObjRef.pop()->release();
  }
}

/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_create()
{
  TByteArrayOutputStream* pStream = new TByteArrayOutputStream();
  m_lstOutObjRef.push(pStream);
  TByteArray Array;
  pStream->popData(&Array);
  CPPUNIT_ASSERT(Array.getCount() == 0);
}
 
/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_createWithStream()
{
  TByteArray Stream;
  Stream.add(0x00);
  Stream.add(0x00);
  Stream.add(0x00);
  Stream.add(0x20);
  TByteArrayInputStream* pStream = new TByteArrayInputStream(&Stream);
  m_lstInObjRef.push(pStream);
}
 
/***************************************************************************/
/*! Tests the popData method
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_popData()
{
  TByteArray Stream;
  Stream.add(0x00);
  Stream.add(0x00);
  Stream.add(0x00);
  Stream.add(0x20);
  TByteArrayOutputStream* pStream = new TByteArrayOutputStream();
  m_lstOutObjRef.push(pStream);
  pStream->writeArray(&Stream);
  
  TByteArray Array;
  pStream->popData(&Array); 
  CPPUNIT_ASSERT(Array.getCount() == 8);
//  CPPUNIT_ASSERT(Array == Stream);
  
  //No 2nd time to get the stream
  TByteArray Array2;
  pStream->popData(&Array2); 
  CPPUNIT_ASSERT(Array2.getCount() == 0);
}


/***************************************************************************/
/*! Tests the readLong method
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_readLong()
{
  TByteArrayOutputStream* pStream1 = new TByteArrayOutputStream();
  m_lstOutObjRef.push(pStream1);
  pStream1->writeLong(100);
  pStream1->writeLong(200);
  pStream1->writeLong(120);
  pStream1->writeLong(33);
  
  TByteArray Array;
  pStream1->popData(&Array);
  
  TByteArrayInputStream* pStream2 = new TByteArrayInputStream(&Array);
  m_lstInObjRef.push(pStream2);
  
  CPPUNIT_ASSERT(pStream2->readLong() == 100);
  CPPUNIT_ASSERT(pStream2->readLong() == 200);
  CPPUNIT_ASSERT(pStream2->readLong() == 120);
  CPPUNIT_ASSERT(pStream2->readLong() == 33);
}

/***************************************************************************/
/*! Tests the readString method
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_readString()
{
  TByteArrayOutputStream* pStream1 = new TByteArrayOutputStream();
  m_lstOutObjRef.push(pStream1);
  TString str1 = L"This is a test";
  TString str2 = L"This is A BIG test";
  TString str3 = L"This is A test";
  TString str4 = L"This was a test";
  pStream1->writeString(&str1);
  pStream1->writeString(&str2);
  pStream1->writeString(&str3);
  pStream1->writeString(&str4);
  
  TByteArray Array;
  pStream1->popData(&Array);
  
  TByteArrayInputStream* pStream2 = new TByteArrayInputStream(&Array);
  m_lstInObjRef.push(pStream2);
  
  TString strR;
  CPPUNIT_ASSERT(pStream2->readString(&strR) == RET_NOERROR);
  CPPUNIT_ASSERT(strR == str1);
  CPPUNIT_ASSERT(pStream2->readString(&strR) == RET_NOERROR);
  CPPUNIT_ASSERT(strR == str2);
  CPPUNIT_ASSERT(pStream2->readString(&strR) == RET_NOERROR);
  CPPUNIT_ASSERT(strR == str3);
  CPPUNIT_ASSERT(pStream2->readString(&strR) == RET_NOERROR);
  CPPUNIT_ASSERT(strR == str4);
}

/***************************************************************************/
/*! Tests the readString method
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_readDouble()
{
  TByteArrayOutputStream* pStream1 = new TByteArrayOutputStream();
  m_lstOutObjRef.push(pStream1);
  pStream1->writeDouble(100.12);
  pStream1->writeDouble(200.3333);
  pStream1->writeDouble(120.01);
  pStream1->writeDouble(33.33);
  
  TByteArray Array;
  pStream1->popData(&Array);
  
  TByteArrayInputStream* pStream2 = new TByteArrayInputStream(&Array);
  m_lstInObjRef.push(pStream2);
  
  CPPUNIT_ASSERT(pStream2->readDouble() == 100.12);
  CPPUNIT_ASSERT(pStream2->readDouble() == 200.3333);
  CPPUNIT_ASSERT(pStream2->readDouble() == 120.01);
  CPPUNIT_ASSERT(pStream2->readDouble() == 33.33);
}

/***************************************************************************/
/*! Tests the readString method
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_readBool()
{
  TByteArrayOutputStream* pStream1 = new TByteArrayOutputStream();
  m_lstOutObjRef.push(pStream1);
  pStream1->writeBool(true);
  pStream1->writeBool(true);
  pStream1->writeBool(false);
  pStream1->writeBool(true);
  
  TByteArray Array;
  pStream1->popData(&Array);
  
  TByteArrayInputStream* pStream2 = new TByteArrayInputStream(&Array);
  m_lstInObjRef.push(pStream2);
  
  CPPUNIT_ASSERT(pStream2->readBool() == true);
  CPPUNIT_ASSERT(pStream2->readBool() == true);
  CPPUNIT_ASSERT(pStream2->readBool() == false);
  CPPUNIT_ASSERT(pStream2->readBool() == true);
}

/***************************************************************************/
/*! Tests the readArray method
*/
/***************************************************************************/
void TByteArrayInputStreamTest::test_readArray()
{
  char aData1[5] = {1,2,3,4,5};
  char aData2[7] = {8,5,1,2,3,4,5};
  char aData3[6] = {10,1,2,3,4,5};
  char aData4[3] = {3,4,5};
  TByteArray aArray1(aData1, 5);
  TByteArray aArray2(aData2, 7);
  TByteArray aArray3(aData3, 6);
  TByteArray aArray4(aData4, 3);
  
  TByteArrayOutputStream* pStream1 = new TByteArrayOutputStream();
  m_lstOutObjRef.push(pStream1);
  
  pStream1->writeArray(&aArray1);
  pStream1->writeArray(&aArray2);
  pStream1->writeArray(&aArray3);
  pStream1->writeArray(&aArray4);
  
  TByteArray Array;
  pStream1->popData(&Array);
  
  TByteArrayInputStream* pStream2 = new TByteArrayInputStream(&Array);
  m_lstInObjRef.push(pStream2);
  
  TByteArray aTest;
  pStream2->readArray(&aTest);
  CPPUNIT_ASSERT(aTest == aArray1);
  pStream2->readArray(&aTest);
  CPPUNIT_ASSERT(aTest == aArray2);
  pStream2->readArray(&aTest);
  CPPUNIT_ASSERT(aTest == aArray3);
  pStream2->readArray(&aTest);
  CPPUNIT_ASSERT(aTest == aArray4);
}
