/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : EchoServer                                                    *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:05:21 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: EchoServer.cpp,v $
 *   Revision 1.1  2005/11/25 10:05:21  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/   

 
#include <zeustest/System/EchoServer.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/ServerSocket.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/Remote/RemoteDefines.hpp>
#include <stdio.h>
#include <signal.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a echo server
*/
/***************************************************************************/
TEchoServer::TEchoServer() : TThread()
{}
    
/***************************************************************************/
/*! Destroys the echo server
*/
/***************************************************************************/
TEchoServer::~TEchoServer()
{}
 

/***************************************************************************/
/*! Kill thread
*/
/***************************************************************************/
void TEchoServer::kill()
{
  TThread::kill();
}

/***************************************************************************/
/*! Thread execution method
*/
/***************************************************************************/
void TEchoServer::execute()
{
  TString strAddr = L"127.0.0.1";
  TSingleLinkedList<TClientHandler*> lstClients;
  TServerSocket* pServer = new TServerSocket(strAddr, 8001);
  pServer->setReUsable();
  if (pServer->bind() == RET_NOERROR)
  {
    ZLOGGER.printfln(LOGMODE_DEBUG, "Start echo server...");
    pServer->setBlockable(false);
    while(!this->isInterrupted())
    {
      TSocket* pClient = NULL;
      if (pServer->acceptTO(&pClient, 0.2) == RET_NOERROR)
      {
        TString strAddress = pClient->getAddress();
        long ulPort = pClient->getPort();
        ZLOGGER.printfln(LOGMODE_DEBUG, "Client found: %s:%d", strAddress.c_str(), ulPort);
        TClientHandler* pHandler = new TClientHandler(pClient);
        lstClients.add(pHandler);
        pHandler->start();
        pClient->release();
        pClient = NULL;
      }
      else
      {
        TTime::sleep(0.1);
      }
    }
  }
  else
  {
    printf("Could not start echo server. Binding failed.\n");
  }
  
  while(lstClients.getCount() > 0)
  {
    lstClients[0]->kill();
    lstClients[0]->release();
    lstClients.deleteItem(0);
  }
  pServer->release();
}

/***************************************************************************/
/*! Creates a client handler thread
*/
/***************************************************************************/
TEchoServer::TClientHandler::TClientHandler(TSocket* client)
{
  m_pClient = client;
  if (m_pClient != NULL)
  {
    m_pClient->addRef();
  }
  m_bExit = false;
}

/***************************************************************************/
/*! Destroys the client handler thread
*/
/***************************************************************************/
TEchoServer::TClientHandler::~TClientHandler()
{
  if (m_pClient != NULL)
  {
    m_pClient->release();
    m_pClient = NULL;
  }
}

/***************************************************************************/
/*! 
*/
/***************************************************************************/
void TEchoServer::TClientHandler::kill()
{
  if (!this->isDead())
  {
    m_bExit = true;
    TThread::kill();
  }
}

/***************************************************************************/
/*! Execution method of client handler
*/
/***************************************************************************/
void TEchoServer::TClientHandler::execute()
{
  while(!this->isInterrupted() && !m_bExit)
  {
    TByteArray aData;
    m_pClient->readArray(&aData);
    
    long lCount = aData.getCount();
    if (lCount > 0)
    {
      TByteArray aNewData(aData.getCount() + 10);
      for(int i = 0; i < lCount; i++)
      {
        aNewData.add(aData[lCount-i-1]);
      }
      m_pClient->writeArray(&aNewData);
    }
  }
  m_pClient->disconnect();
}
