/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FileTest                                                      *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FileTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeustest/System/FileTest.h>
#include <zeusbase/System/File.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TFileTest );


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TFileTest::setUp()
{}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TFileTest::tearDown()
{
}
          
/***************************************************************************/
/*! Tests the absolute Filename of a file
*/
/***************************************************************************/
void TFileTest::test_getAbsoluteName()
{
  TString strFileName = L"D:\\Hallo.txt";
  TFile fFile(strFileName);
  CPPUNIT_ASSERT(fFile.getAbsoluteName() == strFileName);
}

/***************************************************************************/
/*! Tests the Filename extraction
*/
/***************************************************************************/
void TFileTest::test_extractFileName()
{
  TString strFilePath = L"D:\\Hallo.txt";
  TString strFileName = L"Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileName() == strFileName);

  strFilePath = L"TT/Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileName() == strFileName);

  strFilePath = L".\\TestDir\\TT\\Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileName() == strFileName);

  strFilePath = L"./TestDir/TT/Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileName() == strFileName);

  strFilePath = L"Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileName() == strFileName);

  strFilePath = L"/Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileName() == strFileName);

  strFilePath = L"./Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileName() == strFileName);
}

/***************************************************************************/
/*! Tests the file extension extraction
*/
/***************************************************************************/
void TFileTest::test_extractFileExt()
{
  TString strFilePath = L"D:\\Hallo.txt";
  TString strFileExt = L"txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileExt() == strFileExt);

  strFilePath = L"D:\\Hallo.txt.dd";
  strFileExt = L"dd";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileExt() == strFileExt);

  strFilePath = L"D:\\Hallo";
  strFileExt = L"";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileExt() == strFileExt);
}


/***************************************************************************/
/*! Tests the file base name extraction
*/
/***************************************************************************/
void TFileTest::test_extractFileNameBase()
{
  TString strFilePath = L"D:\\Hallo.txt";
  TString strFileBase = L"Hallo";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileNameBase() == strFileBase);

  strFilePath = L"D:\\Hallo.txt.dd";
  strFileBase = L"Hallo.txt";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileNameBase() == strFileBase);

  strFilePath = L"D:\\Hallo";
  strFileBase = L"Hallo";
  CPPUNIT_ASSERT(TFile(strFilePath).extractFileNameBase() == strFileBase);
}



