/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SerialTestObject                                              *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SerialTestObject.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/System/SerialTestObject.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR(TSerialTestSubObject)

/***************************************************************************/
/*! Creates a sub Object to test the serializing
*/
/***************************************************************************/
TSerialTestSubObject::TSerialTestSubObject()
{
  m_dValue = 0.0;
  m_lValue = 0;
}


/***************************************************************************/
/*! Destroys the Sub Object
*/
/***************************************************************************/
TSerialTestSubObject::~TSerialTestSubObject()
{}

double TSerialTestSubObject::getDoubleValue()
{ return m_dValue; }

long TSerialTestSubObject::getLongValue()
{ return m_lValue; }

void TSerialTestSubObject::setDoubleValue(double value)
{  m_dValue = value; }

void TSerialTestSubObject::setLongValue(long value)
{ m_lValue = value; }


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TSerialTestSubObject);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;



SERIAL_CONSTRUCTOR(TSerialTestObject)

/***************************************************************************/
/*! Creates a root Object to test the serializing
*/
/***************************************************************************/
TSerialTestObject::TSerialTestObject()
{
  m_dValue = 0.0;
  m_lValue = 0;
  m_strValue = L"";
  m_pObject = new TSerialTestSubObject();
}

/***************************************************************************/
/*! Destroys the test objects
*/
/***************************************************************************/
TSerialTestObject::~TSerialTestObject()
{
  if (m_pObject != NULL)
  {
    m_pObject->release();
    m_pObject = NULL;
  }
}

double TSerialTestObject::getDoubleValue()
{ return m_dValue; }

long TSerialTestObject::getLongValue()
{ return m_lValue; }

TString TSerialTestObject::getStringValue()
{ return m_strValue; }

unsigned long TSerialTestObject::getObject(TSerialTestSubObject** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (obj == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pObject != NULL)
  {
    *obj = m_pObject;
    m_pObject->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

void TSerialTestObject::setDoubleValue(double value)
{  m_dValue = value; }

void TSerialTestObject::setLongValue(long value)
{ m_lValue = value; }

void TSerialTestObject::setStringValue(TString value)
{ m_strValue = value; }

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TSerialTestObject);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;
