/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SerialTestObject                                              *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SerialTestObject.h,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#ifndef SerialTestObjectH
#define SerialTestObjectH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/String.h>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This class describes a test object to test the serializing of sub objects.
*/
/***************************************************************************/
class TSerialTestSubObject : public TZObject, public ISerializable
{
  public:
    TSerialTestSubObject();
    
    double getDoubleValue();
    long getLongValue();
    void setDoubleValue(double value);
    void setLongValue(long value);
    
    SERIAL_START(TSerialTestSubObject, L"TSerialTestSubObject");
      SERIAL_LONG(m_lValue)
      SERIAL_DOUBLE(m_dValue)
    SERIAL_END
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TSerialTestSubObject();
    
  private:
    ///Long value
    long m_lValue;
    ///Double value
    double m_dValue;
  
};


/***************************************************************************/
/*! This class describes a test object to test the serializing of root
    objects.
*/
/***************************************************************************/
class TSerialTestObject : public TZObject, public ISerializable
{
  public:
    TSerialTestObject();
  
    double getDoubleValue();
    long getLongValue();
    TString getStringValue();
    unsigned long getObject(TSerialTestSubObject** obj);
    void setDoubleValue(double value);
    void setLongValue(long value);
    void setStringValue(TString value);
    
    SERIAL_START(TSerialTestObject, L"TSerialTestObject");
      SERIAL_LONG(m_lValue)
      SERIAL_DOUBLE(m_dValue)
      SERIAL_STRING(m_strValue)
      SERIAL_OBJECT(TSerialTestSubObject, m_pObject)
    SERIAL_END
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
      
  protected:
    virtual ~TSerialTestObject();
    
  private:
    ///Long value
    long m_lValue;
    ///Double value
    double m_dValue;
    ///String value
    TString m_strValue;
    ///An other serializable Object
    TSerialTestSubObject* m_pObject;
  
};

END_NAMESPACE_Zeus

#endif
