/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SocketTest                                                    *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:05:21 $                                  *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SocketTest.cpp,v $
 *   Revision 1.1  2005/11/25 10:05:21  bhadorn
 *   Created
 *
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/System/SocketTest.h>
#include <zeustest/System/EchoServer.h>
#include <zeusbase/System/Socket.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/Remote/RemoteDefines.hpp>
#include <stdio.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TSocketTest );



/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TSocketTest::setUp()
{
  m_strAddress = L"127.0.0.1";
  m_ulPort = 8001;
  m_pEchoServer = new TEchoServer();
  m_pEchoServer->start();
  TTime::sleep(0.5);
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TSocketTest::tearDown()
{
  //Release all instances
  while(!m_lstStack.isEmpty())
  {
    m_lstStack.pop()->release();
  }
  while(!m_lstSockets.isEmpty())
  {
    TSocket* pSocket = m_lstSockets.pop();
    pSocket->disconnect();
    pSocket->release();
  }
  if (m_pEchoServer != NULL)
  {
    m_pEchoServer->kill();
    m_pEchoServer->release();
    m_pEchoServer = NULL;
  }
}

/***************************************************************************/
/*! Send a package to the echo server
*/
/***************************************************************************/
bool TSocketTest::sendPackage(TSocket* socket, TByteArray& data, long size)
{
  bool bRetval = false;
  data.clear();
  for(int i = 0; i < size; i++)
  {
    data.add(i%256);
  }
  
  if (socket->writeArray(&data) == RET_NOERROR)
  {
    bRetval = true;
  }
  return bRetval;
}

/***************************************************************************/
/*! Receive a package from the echo server
*/
/***************************************************************************/
bool TSocketTest::receivePackage(TSocket* socket, TByteArray& data, long size)
{
  TByteArray aResponse;
  bool bRetval = true;
  
  socket->readArray(&aResponse);
  long lCount = aResponse.getCount();
  if (lCount != size)
  {
    printf("Error: not same size received. Expected=%d; Received=%d\n", size, lCount);
    bRetval = false;
  } 
  else
  {
    //printf("Data Received=%d\n", lCount);
    for(int i = 0; i < lCount; i++)
    {
      if (data[i] != aResponse[lCount-i-1])
      {
        printf("Error: Wrong character received. Expected=%d; Received=%d [%d]\n", (int)data[i], (int)aResponse[lCount-i-1], i);
        bRetval = false;
      }
    }
  }
  return bRetval;
}


/***************************************************************************/
/*! Tests connecting to the server
*/
/***************************************************************************/
void TSocketTest::test_connect()
{
  CPPUNIT_ASSERT(m_pEchoServer != NULL);
  TSocket* pSocket = new TSocket(m_strAddress, m_ulPort);
  m_lstSockets.push(pSocket);
  
  CPPUNIT_ASSERT(pSocket->connect() == RET_NOERROR);
  CPPUNIT_ASSERT(pSocket->disconnect() == RET_NOERROR);
} 

/***************************************************************************/
/*! Tests sending data packages
*/
/***************************************************************************/
void TSocketTest::test_sendData()
{
  TSocket* pSocket = new TSocket(m_strAddress, m_ulPort);
  m_lstSockets.push(pSocket);
  CPPUNIT_ASSERT(pSocket->connect() == RET_NOERROR);
  TByteArray aData(5000);
  CPPUNIT_ASSERT(sendPackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(receivePackage(pSocket,aData, 100));
  
  CPPUNIT_ASSERT(sendPackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(sendPackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(sendPackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(sendPackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(receivePackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(receivePackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(receivePackage(pSocket,aData, 100));
  CPPUNIT_ASSERT(receivePackage(pSocket,aData, 100));
  
  CPPUNIT_ASSERT(sendPackage(pSocket,aData, 5000));
  CPPUNIT_ASSERT(sendPackage(pSocket,aData, 5000));
  CPPUNIT_ASSERT(receivePackage(pSocket,aData, 5000));
  CPPUNIT_ASSERT(receivePackage(pSocket,aData, 5000));
  
  CPPUNIT_ASSERT(pSocket->disconnect() == RET_NOERROR);
}
