/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StringListTest                                                *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: StringListTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/System/StringListTest.h> 
#include <zeusbase/System/StringList.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TStringListTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TStringListTest::setUp()
{
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TStringListTest::tearDown()
{

}

/***************************************************************************/
/*! Test add
*/
/***************************************************************************/
void TStringListTest::test_add()
{
  Zeus::TStringList lstStrings;
  CPPUNIT_ASSERT(lstStrings.getCount() == 0);
  
  //Add 10 elements and check return value
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    CPPUNIT_ASSERT(lstStrings.add(&strAdd)== i);
    CPPUNIT_ASSERT(lstStrings[i]->equalsStr(&strAdd));
  }
  CPPUNIT_ASSERT(lstStrings.getCount() == 10);  
}


/***************************************************************************/
/*! Test clear
*/
/***************************************************************************/
void TStringListTest::test_clear()
{
  Zeus::TStringList lstStrings;
  CPPUNIT_ASSERT(lstStrings.getCount() == 0);
  
  //Add 10 elements and check return value
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    CPPUNIT_ASSERT(lstStrings.add(&strAdd)== i);
    CPPUNIT_ASSERT(lstStrings[i]->equalsStr(&strAdd));
  }
  CPPUNIT_ASSERT(lstStrings.getCount() == 10);  
  
  lstStrings.clear();
  CPPUNIT_ASSERT(lstStrings.getCount() == 0);
}

/***************************************************************************/
/*! Test copyToList method
*/
/***************************************************************************/
void TStringListTest::test_copyToList()
{
  Zeus::TStringList lstStringsSrc;
  Zeus::TStringList lstStringsDest;
  
  //Add 10 elements and check return value
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStringsSrc.add(&strAdd);
  }
  
  lstStringsSrc.copyToList(&lstStringsDest);
  
  CPPUNIT_ASSERT(lstStringsSrc.equals(&lstStringsDest));
}

/***************************************************************************/
/*! Test deleteItem method
*/
/***************************************************************************/
void TStringListTest::test_deleteItem()
{
  Zeus::TStringList lstStrings;

  //Add 10 elements
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  CPPUNIT_ASSERT(lstStrings.getCount() == 10);
  
  //First case: Delete top level element
  lstStrings.deleteItem(0);
  CPPUNIT_ASSERT(lstStrings.getCount() == 9);
  CPPUNIT_ASSERT(lstStrings[0]->equals(L"1"));
  
  //2nd case: delete end element
  lstStrings.deleteItem(8);
  CPPUNIT_ASSERT(lstStrings.getCount() == 8);
  CPPUNIT_ASSERT(lstStrings[7]->equals(L"8"));
  
  //3rd case: delete any element in the middle of the list
  lstStrings.deleteItem(2);
  CPPUNIT_ASSERT(lstStrings.getCount() == 7);
  CPPUNIT_ASSERT(lstStrings[1]->equals(L"2"));
  CPPUNIT_ASSERT(lstStrings[2]->equals(L"4"));

  Zeus::TStringList lstStringsFail;

  //Add 10 elements
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStringsFail.add(&strAdd);
  }
  
  lstStringsFail.deleteItem(-1);
  CPPUNIT_ASSERT(lstStringsFail.getCount() == 10);
  CPPUNIT_ASSERT(lstStringsFail[0]->equals(L"0"));
}


/***************************************************************************/
/*! Test the equals method
*/
/***************************************************************************/
void TStringListTest::test_equals()
{
  Zeus::TStringList lstStrings1;
  Zeus::TStringList lstStrings2;

  //Add 10 elements and check return value
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings1.add(&strAdd);
    lstStrings2.add(&strAdd);
  }
  
  CPPUNIT_ASSERT(lstStrings1.equals(&lstStrings2));
  CPPUNIT_ASSERT(lstStrings2.equals(&lstStrings1));
}


/***************************************************************************/
/*! Test the equalsItem method
*/
/***************************************************************************/
void TStringListTest::test_equalsItem()
{
  Zeus::TStringList lstStrings;

  //Add 10 elements 
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  
  TString strTest(L"0");
  CPPUNIT_ASSERT(lstStrings.equalsItem(0, &strTest));
  CPPUNIT_ASSERT(!lstStrings.equalsItem(1, &strTest));
  CPPUNIT_ASSERT(!lstStrings.equalsItem(-1, &strTest));
  
  TString strTest2(L"9");
  CPPUNIT_ASSERT(lstStrings.equalsItem(9, &strTest2));
  CPPUNIT_ASSERT(!lstStrings.equalsItem(0, &strTest2));
  CPPUNIT_ASSERT(!lstStrings.equalsItem(10, &strTest2));

  TString strTest3(L"5");
  CPPUNIT_ASSERT(lstStrings.equalsItem(5, &strTest3));
  CPPUNIT_ASSERT(!lstStrings.equalsItem(4, &strTest3));
  CPPUNIT_ASSERT(!lstStrings.equalsItem(6, &strTest3));
}

/***************************************************************************/
/*! Test getItem method
*/
/***************************************************************************/
void TStringListTest::test_getItem()
{
  Zeus::TStringList lstStrings;

  //Add 10 elements 
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  
  CPPUNIT_ASSERT(lstStrings.getItem(0)->equals(L"0"));
  CPPUNIT_ASSERT(lstStrings.getItem(1)->equals(L"1"));
  CPPUNIT_ASSERT(lstStrings.getItem(5)->equals(L"5"));
  CPPUNIT_ASSERT(lstStrings.getItem(9)->equals(L"9"));
  CPPUNIT_ASSERT(lstStrings.getItem(10) == NULL);
  CPPUNIT_ASSERT(lstStrings.getItem(-1) == NULL);
}

/***************************************************************************/
/*! Test getItem method
*/
/***************************************************************************/
void TStringListTest::test_getItemCopy()
{
  Zeus::TStringList lstStrings;

  //Add 10 elements 
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  
  CPPUNIT_ASSERT(lstStrings.getItemCopy(0)->equals(L"0"));
  CPPUNIT_ASSERT(lstStrings.getItemCopy(1)->equals(L"1"));
  CPPUNIT_ASSERT(lstStrings.getItemCopy(5)->equals(L"5"));
  CPPUNIT_ASSERT(lstStrings.getItemCopy(9)->equals(L"9"));
  CPPUNIT_ASSERT(lstStrings.getItemCopy(10) == NULL);
  CPPUNIT_ASSERT(lstStrings.getItemCopy(-1) == NULL);
}

/***************************************************************************/
/*! Test indexOf method
*/
/***************************************************************************/
void TStringListTest::test_indexOf()
{
  Zeus::TStringList lstStrings;

  //Add 10 elements
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  CPPUNIT_ASSERT(lstStrings.getCount() == 10);
  
  //Check IndexOf methpd of all elements
  for(long i = 0; i < 10; i ++)
  {
    TString strIndexOf(i);
    CPPUNIT_ASSERT(lstStrings.indexOf(&strIndexOf) == i);    
  }
}

/***************************************************************************/
/*! Test insert method
*/
/***************************************************************************/
void TStringListTest::test_insert()
{
  Zeus::TStringList lstStrings;
  
  //Add 10 elements
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  CPPUNIT_ASSERT(lstStrings.getCount() == 10);
  
  //Case 1: Insert at the beginning of the list
  TString strInsert1(L"-1");
  CPPUNIT_ASSERT(lstStrings.insert(0, &strInsert1) == 0);
  CPPUNIT_ASSERT(lstStrings.getCount() == 11);
  CPPUNIT_ASSERT(lstStrings.getItem(0)->equals(L"-1"));
  CPPUNIT_ASSERT(lstStrings.getItem(1)->equals(L"0"));
  CPPUNIT_ASSERT(lstStrings.getItem(2)->equals(L"1"));
  CPPUNIT_ASSERT(lstStrings.getItem(-1) == NULL);

  //Case 2: Insert at the end of the list
  TString strInsert2(L"10");
  CPPUNIT_ASSERT(lstStrings.insert(11, &strInsert2) == 11);
  CPPUNIT_ASSERT(lstStrings.getCount() == 12);
  CPPUNIT_ASSERT(lstStrings.getItem(11)->equals(L"10"));
  CPPUNIT_ASSERT(lstStrings.getItem(10)->equals(L"9"));
  CPPUNIT_ASSERT(lstStrings.getItem(9)->equals(L"8"));
  CPPUNIT_ASSERT(lstStrings.getItem(12) == NULL);

  //Case 2: Insert in the middle of the list
  TString strInsert3(L"5.5");
  CPPUNIT_ASSERT(lstStrings.insert(7, &strInsert3) == 7);
  CPPUNIT_ASSERT(lstStrings.getCount() == 13);
  CPPUNIT_ASSERT(lstStrings.getItem(7)->equals(L"5.5"));
  CPPUNIT_ASSERT(lstStrings.getItem(5)->equals(L"4"));
  CPPUNIT_ASSERT(lstStrings.getItem(6)->equals(L"5"));
  CPPUNIT_ASSERT(lstStrings.getItem(8)->equals(L"6"));
  CPPUNIT_ASSERT(lstStrings.getItem(9)->equals(L"7"));
}


/***************************************************************************/
/*! Test the iterator method
*/
/***************************************************************************/
void TStringListTest::test_iterator()
{
  Zeus::TStringList lstStrings;
  
  //Add 10 elements
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  CPPUNIT_ASSERT(lstStrings.getCount() == 10);
  
  //Test one iterator
  IListIterator<IString*>* pIterator = lstStrings.getIterator();
  CPPUNIT_ASSERT(pIterator != NULL);
  long lCount = 0;
  while(pIterator->hasNextItem())
  {
    lCount++;
    pIterator->getNextItem();
  } 
  CPPUNIT_ASSERT(lCount == 10);

  IListIterator<IString*>* pIterator1 = lstStrings.getIterator();
  CPPUNIT_ASSERT(pIterator1 != NULL);
  IListIterator<IString*>* pIterator2 = lstStrings.getIterator();
  CPPUNIT_ASSERT(pIterator2 != NULL);
  IListIterator<IString*>* pIterator3 = lstStrings.getIterator();
  CPPUNIT_ASSERT(pIterator3 != NULL);
  IListIterator<IString*>* pIterator4 = lstStrings.getIterator();
  CPPUNIT_ASSERT(pIterator4 != NULL);
    
  //Cascaded use of iterators
  long lCount1 = 0;
  while(pIterator1->hasNextItem())
  {
    lCount1++;
    
    long lCount2 = 0;
    pIterator2->reset();
    while(pIterator2->hasNextItem())
    {
      lCount2++;
      pIterator2->getNextItem();
    } 
    CPPUNIT_ASSERT(lCount2 == 10);
    
    pIterator1->getNextItem();
  } 
  CPPUNIT_ASSERT(lCount1 == 10);
  
  //Check if iterator is resistent to delete
//  CPPUNIT_ASSERT(pIterator3->getNextItem()->
//  pIterator3->getNextItem()
  
  lstStrings.releaseIterator(pIterator1);
  lstStrings.releaseIterator(pIterator3);
  lstStrings.releaseIterator(pIterator);
  lstStrings.releaseIterator(pIterator2);
  lstStrings.releaseIterator(pIterator4);
}

/***************************************************************************/
/*! Test remove method
*/
/***************************************************************************/
void TStringListTest::test_remove()
{
  Zeus::TStringList lstStrings;
  
  //Add 10 elements
  for(long i = 0; i < 10; i ++)
  {
    TString strAdd(i);
    lstStrings.add(&strAdd);
  }
  CPPUNIT_ASSERT(lstStrings.getCount() == 10);
  
  //First case: Remove top level element
  TString strRemove(L"0");
  lstStrings.remove(&strRemove);
  CPPUNIT_ASSERT(lstStrings.getCount() == 9);
  CPPUNIT_ASSERT(lstStrings[0]->equals(L"1"));
  
  //2nd case: remove end element
  TString strRemove2(L"9");
  lstStrings.remove(&strRemove2);
  CPPUNIT_ASSERT(lstStrings.getCount() == 8);
  CPPUNIT_ASSERT(lstStrings[7]->equals(L"8"));
  
  //3rd case: Remove any element in the middle of the list
  TString strRemove3(L"3");
  lstStrings.remove(&strRemove3);
  CPPUNIT_ASSERT(lstStrings.getCount() == 7);
  CPPUNIT_ASSERT(lstStrings[1]->equals(L"2"));
  CPPUNIT_ASSERT(lstStrings[2]->equals(L"4"));
}

