/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StringMapTest                                                 *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:41:52 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: StringMapTest.cpp,v $
 *   Revision 1.3  2005/11/16 11:41:52  bhadorn
 *   New tests added for iterator.
 *
 *   Revision 1.2  2005/11/10 10:45:59  bhadorn
 *   New test added for hasItem()
 *
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  


#include <zeustest/System/StringMapTest.h>
#include <zeusbase/System/StringMap.hpp>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TStringMapTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TStringMapTest::setUp()
{
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TStringMapTest::tearDown()
{

}

/***************************************************************************/
/*! Test the getItem()-Method
*/
/***************************************************************************/
void TStringMapTest::test_getItem()
{
  Zeus::TStringMap<long> mapTest;
  CPPUNIT_ASSERT(mapTest.getCount() == 0);
  
  mapTest.setItem(L"Test1", 1);
  mapTest.setItem(L"Test2", 2);
  mapTest.setItem(L"Test3", 3);
  mapTest.setItem(L"Test4", 4);
  mapTest.setItem(L"Test5", 5);

  CPPUNIT_ASSERT(mapTest.getCount() == 5);
  CPPUNIT_ASSERT(mapTest.getItem(L"Test3") == 3);
  CPPUNIT_ASSERT(mapTest.getItem(L"Test1") == 1);
  CPPUNIT_ASSERT(mapTest.getItem(L"Test5") == 5);
  CPPUNIT_ASSERT(mapTest.getItem(L"Test6") == 0);
}

/***************************************************************************/
/*! Test the getItem()-Method
*/
/***************************************************************************/
void TStringMapTest::test_hasItem()
{
  Zeus::TStringMap<long> mapTest;
  CPPUNIT_ASSERT(mapTest.getCount() == 0);
  
  mapTest.setItem(L"Test1", 1);
  mapTest.setItem(L"Test2", 2);
  mapTest.setItem(L"Test3", 3);
  mapTest.setItem(L"Test4", 4);
  mapTest.setItem(L"Test5", 5);

  CPPUNIT_ASSERT(mapTest.getCount() == 5);
  CPPUNIT_ASSERT(mapTest.hasItem(L"Test3"));
  CPPUNIT_ASSERT(mapTest.hasItem(L"Test1"));
  CPPUNIT_ASSERT(mapTest.hasItem(L"Test5"));
  CPPUNIT_ASSERT(!mapTest.hasItem(L"Test6"));
}

/***************************************************************************/
/*! Test the setItem()-Method
*/
/***************************************************************************/
void TStringMapTest::test_deleteItem()
{
  Zeus::TStringMap<long> mapTest;
  CPPUNIT_ASSERT(mapTest.getCount() == 0);
  
  mapTest.setItem(L"Test1", 1);
  mapTest.setItem(L"Test2", 2);
  mapTest.setItem(L"Test3", 3);
  mapTest.setItem(L"Test4", 4);
  mapTest.setItem(L"Test5", 5);
  CPPUNIT_ASSERT(mapTest.getCount() == 5);
  
  CPPUNIT_ASSERT(mapTest.deleteItem(L"Test2"));
  CPPUNIT_ASSERT(mapTest.getItem(L"Test2") == 0);
  CPPUNIT_ASSERT(mapTest.getCount() == 4);
  CPPUNIT_ASSERT(!mapTest.deleteItem(L"Test2"));

  CPPUNIT_ASSERT(mapTest.deleteItem(L"Test5"));
  CPPUNIT_ASSERT(mapTest.getItem(L"Test5") == 0);
  CPPUNIT_ASSERT(mapTest.getCount() == 3);
}


/***************************************************************************/
/*! Test the clear()-Method
*/
/***************************************************************************/
void TStringMapTest::test_clear()
{
  Zeus::TStringMap<long> mapTest;
  CPPUNIT_ASSERT(mapTest.getCount() == 0);
  CPPUNIT_ASSERT(mapTest.isEmpty());
  
  mapTest.setItem(L"Test1", 1);
  mapTest.setItem(L"Test2", 2);
  mapTest.setItem(L"Test3", 3);
  mapTest.setItem(L"Test4", 4);
  mapTest.setItem(L"Test5", 5);
  CPPUNIT_ASSERT(mapTest.getCount() == 5);
  
  CPPUNIT_ASSERT(!mapTest.isEmpty());
  mapTest.clear();
  CPPUNIT_ASSERT(mapTest.isEmpty());
  CPPUNIT_ASSERT(mapTest.getCount() == 0);
}

/***************************************************************************/
/*! Test the getIterator()-Method
*/
/***************************************************************************/
void TStringMapTest::test_getIterator()
{
  Zeus::TStringMap<long> mapTest;
  CPPUNIT_ASSERT(mapTest.getCount() == 0);
  
  mapTest.setItem(L"Test1", 1);
  mapTest.setItem(L"Test2", 2);
  mapTest.setItem(L"Test3", 3);
  mapTest.setItem(L"Test4", 4);
  mapTest.setItem(L"Test5", 5);
  CPPUNIT_ASSERT(mapTest.getCount() == 5);

  IListIterator<long>* pIt = mapTest.getIterator();
  long lCount = 0;
  while(pIt->hasNextItem())
  {
    CPPUNIT_ASSERT(pIt->getNextItem() == lCount+1);
    lCount++;
  }  
  mapTest.releaseIterator(pIt);
  
  IStringMapIterator<long>* pMapIt = mapTest.getIterator();
  lCount = 0;
  while(pMapIt->hasNextItem())
  {
    TString strKey;
    CPPUNIT_ASSERT(pMapIt->getNextKey(&strKey) == RET_NOERROR);
    TString strTestKey = TString(L"Test") + (long)(lCount+1);
    CPPUNIT_ASSERT(strKey == strTestKey);
    lCount++;
  }  
  mapTest.releaseIterator(pMapIt);
  CPPUNIT_ASSERT(lCount == 5);
}


 
