/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StringListTest                                                *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:05:03 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: StringTest.cpp,v $
 *   Revision 1.3  2005/11/25 10:05:03  bhadorn
 *   Removed direct output
 *
 *   Revision 1.2  2005/11/10 10:47:13  bhadorn
 *   New test added for isEmpty()
 *
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/System/StringTest.h> 
#include <zeusbase/System/String.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TStringTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TStringTest::setUp()
{
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TStringTest::tearDown()
{

}


/***************************************************************************/
/*! Tests the getSize() and isEmpty methods
*/
/***************************************************************************/
void TStringTest::test_getSize()
{
  TString strEmpty;
  
  CPPUNIT_ASSERT(strEmpty.getSize() == 0);
  CPPUNIT_ASSERT(strEmpty.isEmpty());
  CPPUNIT_ASSERT(strEmpty == L"");
  
  TString strData1(L"1234");
  CPPUNIT_ASSERT(strData1.getSize() == 4);
  CPPUNIT_ASSERT(!strData1.isEmpty());
  CPPUNIT_ASSERT(strData1 == L"1234");
}

/***************************************************************************/
/*! Test assign method
*/
/***************************************************************************/
void TStringTest::test_assign()
{
  TString strAss = L"this is a test";  
  CPPUNIT_ASSERT(strAss.equals(L"this is a test")); 
  strAss.assign(L"a good one");
  CPPUNIT_ASSERT(strAss.equals(L"a good one")); 
  
  //NULL assignment
  strAss.assign(NULL);
  CPPUNIT_ASSERT(strAss.equals(L"")); 
 
  //An assignment after the NULL assignment
  strAss.assign(L"a good one too");
  CPPUNIT_ASSERT(strAss.equals(L"a good one too")); 
}

/***************************************************************************/
/*! Test assignStr method
*/
/***************************************************************************/
void TStringTest::test_assignStr()
{
  TString strAss1 = L"this is a test";  
  TString strAss2 = L"This is a test too";
  CPPUNIT_ASSERT(strAss1.equals(L"this is a test")); 
  strAss1.assignStr(&strAss2);
  CPPUNIT_ASSERT(strAss1.equalsStr(&strAss2)); 
  
  //NULL assignment
  strAss1.assignStr(NULL);
  CPPUNIT_ASSERT(strAss1.equals(L"")); 
 
  //An assignment after the NULL assignment
  TString strAss3 = L"One more";
  strAss1.assignStr(&strAss3);
  CPPUNIT_ASSERT(strAss1.equalsStr(&strAss3)); 
}

/***************************************************************************/
/*! Test concat method
*/
/***************************************************************************/
void TStringTest::test_concat()
{
  TString strConc1 = L"This is a ";
  strConc1.concat(L"test.");
  
  CPPUNIT_ASSERT(strConc1.equals(L"This is a test."));   
  
  //Null
  strConc1.concat(NULL);
  CPPUNIT_ASSERT(strConc1.equals(L"This is a test."));   
  
  strConc1.concat(L".");
  CPPUNIT_ASSERT(strConc1.equals(L"This is a test.."));
}

/***************************************************************************/
/*! Test concatStr method
*/
/***************************************************************************/
void TStringTest::test_concatStr()
{
  TString strConc1 = L"This is a ";
  TString strConc2 = L"test.";
  TString strConc3 = L"This is a test.";
  
  strConc1.concatStr(&strConc2);
  CPPUNIT_ASSERT(strConc1.equalsStr(&strConc3));   
  
  //Null
  strConc1.concatStr(NULL);
  CPPUNIT_ASSERT(strConc1.equalsStr(&strConc3));   
  
  TString strConc4 = L".";
  TString strConc5 = L"This is a test..";
  strConc1.concatStr(&strConc4);
  CPPUNIT_ASSERT(strConc1.equalsStr(&strConc5));   
}


/***************************************************************************/
/*! Test equals method
*/
/***************************************************************************/
void TStringTest::test_equals()
{
  TString strEqual1 = L"My World";
  CPPUNIT_ASSERT(strEqual1.equals(L"My World")); 
  CPPUNIT_ASSERT(!strEqual1.equals(L"my world")); 
  
  CPPUNIT_ASSERT(strEqual1 == L"My World"); 
  CPPUNIT_ASSERT(strEqual1 != L"my world"); 

  TString strLong = L"12";
  long lTest1  = 12;
  long lTest2  = 13;
  CPPUNIT_ASSERT(strLong == lTest1); 
  CPPUNIT_ASSERT(strLong != lTest2); 
    
  TString strDouble = L"12.22";
  double dTest1  = 12.22;
  double dTest2  = 12.21;
  CPPUNIT_ASSERT(strDouble == dTest1); 
  CPPUNIT_ASSERT(strDouble != dTest2); 
}

/***************************************************************************/
/*! Test equalsStr method
*/
/***************************************************************************/
void TStringTest::test_equalsStr()
{
  TString strTest1 = L"hallo";
  TString strTest2 = L"hallo";
  TString strTest3;
  CPPUNIT_ASSERT(strTest1.equalsStr(&strTest1)); 
  CPPUNIT_ASSERT(strTest1.equalsStr(&strTest2)); 
  CPPUNIT_ASSERT(strTest2.equalsStr(&strTest1)); 
  
  CPPUNIT_ASSERT(!strTest1.equalsStr(&strTest3)); 
  CPPUNIT_ASSERT(!strTest1.equalsStr(NULL)); 
}


/***************************************************************************/
/*! Test getPos method
*/
/***************************************************************************/
void TStringTest::test_getPos()
{
  TString strTest1 = L"This is a text with some #hit and &uck*";
  TString strSub = L"This";
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==0); 
  strSub = L"it";
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==16); 
  strSub = L"uck*";
//  printf("Pos= %d\n", (int)strTest1.getPos(&strSub));
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==35); 
  strSub = L"*";
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==38); 
  strSub = L"i";
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==2); 
  strSub = L"T";
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==0); 
  strSub = L"This is a text with some #hit and &uck*";
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==0); 
  strSub = L"";
  CPPUNIT_ASSERT(strTest1.getPos(&strSub)==-1); 
}

/***************************************************************************/
/*! Test the transcode methods
*/
/***************************************************************************/
void TStringTest::test_transcode()
{
  TString strTest1 = L"This is a text with some #hit and &uck*";
  TByteArray aArray = TString::transcode(strTest1);
  CPPUNIT_ASSERT(aArray.getCount() == strTest1.getSize() + 1);
  
  TString strTest2 = TString::transcode(aArray);
  CPPUNIT_ASSERT(strTest2 == strTest1);
  
  TByteArray aArray2 = &aArray;
  TString strTest3 = TString::transcode(aArray2);
  CPPUNIT_ASSERT(strTest3 == strTest1);

  TByteArray aArray3 = aArray;
  TString strTest4 = TString::transcode(aArray3);
  CPPUNIT_ASSERT(strTest4 == strTest1);
}

