/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XLoaderObjectTest                                             *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XLoaderObjectTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeustest/System/XLoaderObjectTest.h>
#include <zeusbase/System/XObjectFactory.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TXLoaderObjectTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TXLoaderObjectTest::setUp()
{  
  m_pObject = NULL;
  
  TString strXObjTree1 = L"<?xml version='1.0' encoding='ISO-8859-1'?>\
<RootObject ID=\"1\" ClassName=\"TXRootObject\" AttributeSchema=\"1\">\
  <SubObject1 ID=\"1\" ClassName=\"TXLoaderObject\" CreateChildren=\"0\">\
    <SubSubObject1 ID=\"1\" ClassName=\"TXObject\"/>\
    <SubSubObject1 ID=\"2\" ClassName=\"TXObject\"/>\
  </SubObject1>\
  <SubObject1 ID=\"2\" ClassName=\"TXLoaderObject\" CreateChildren=\"1\">\
    <SubSubObject2 ID=\"1\" ClassName=\"TXObject\"/>\
    <SubSubObject3 ID=\"2\" ClassName=\"TXObject\"/>\
  </SubObject1>\
  <SubObject2 ID=\"1\" ClassName=\"TXObject\"/>\
</RootObject>";

  //The test of the factory: see TXObjectFactoryTest
  TXObjectFactory::getInstance().createObjectFromStream(&strXObjTree1, &m_pObject, false); 
}
 
/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TXLoaderObjectTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
  
  while(!m_lstLoadRef.isEmpty())
  {
    m_lstLoadRef.pop()->release();
  }
  
  if (m_pObject != NULL)
  {
    m_pObject->release();
  }
}


/***************************************************************************/
/*! Tests getting children
*/
/***************************************************************************/
void TXLoaderObjectTest::test_getChild()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  
  //Has 3 Children
  IXObject* pChild1 = NULL;
  IXObject* pChild1_1 = NULL;
  TString strName;
  
  //Check 1st Childobject
  CPPUNIT_ASSERT(m_pObject->getChild(0, &pChild1) == RET_NOERROR);
  m_lstObjRef.push(pChild1);
  
  pChild1->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject1");  
  CPPUNIT_ASSERT(pChild1->getID() == 1);
  
  //Get child1 of child1
  CPPUNIT_ASSERT(pChild1->getChildCount() == 0);
  CPPUNIT_ASSERT(pChild1->getChild(0, &pChild1_1) == RET_NOERROR);
  m_lstObjRef.push(pChild1_1);
  CPPUNIT_ASSERT(pChild1->getChildCount() == 2);
  
  pChild1_1->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubSubObject1");
  CPPUNIT_ASSERT(pChild1_1->getID() == 1);
}


/***************************************************************************/
/*! Tests getting children by name
*/
/***************************************************************************/
void TXLoaderObjectTest::test_getChildByName()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
}

