/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XObjectFactoryTest                                            *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObjectFactoryTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeustest/System/XObjectFactoryTest.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TXObjectFactoryTest );

/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TXObjectFactoryTest::setUp()
{  
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TXObjectFactoryTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
}

/***************************************************************************/
/*! Tests the constructors
*/
/***************************************************************************/
void TXObjectFactoryTest::test_createObjectFromStream()
{
  TString strObjTree1 = L"<?xml version='1.0' encoding='ISO-8859-1'?>\
<RootObject ID=\"1\" ClassName=\"TXRootObject\">\
  <Attribute Name=\"Label\" Value=\"Hello world\"/>\
  <Attribute Name=\"Text\">Hallo Welt!</Attribute>\
  <SubObject1 ID=\"1\" ClassName=\"TXObject\"/>\
  <SubObject1 ID=\"2\" ClassName=\"TXObject\"/>\
  <SubObject2 ID=\"1\" ClassName=\"TXObject\"/>\
</RootObject>";

  TString strObjTree2 = L"<?xml version='1.0' encoding='ISO-8859-1'?>\
<RootObject ID=\"1\" ClassName=\"TZObject\">\
  <Attribute Name=\"Label\" Value=\"Hello world\"/>\
  <Attribute Name=\"Text\">Hallo Welt!</Attribute>\
  <SubObject1 ID=\"1\" ClassName=\"TXObject\"/>\
  <SubObject1 ID=\"2\" ClassName=\"TXObject\"/>\
  <SubObject2 ID=\"1\" ClassName=\"TXObject\"/>\
</RootObject>";
  
  IXObject* pObject = NULL;
  CPPUNIT_ASSERT(TXObjectFactory::getInstance().createObjectFromStream(&strObjTree1, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject != NULL);
  CPPUNIT_ASSERT(!pObject->isFrozen());
  CPPUNIT_ASSERT(pObject->getChildCount() == 0);
  
  CPPUNIT_ASSERT(TXObjectFactory::getInstance().createObjectFromStream(&strObjTree1, &pObject, false) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject != NULL);
  CPPUNIT_ASSERT(!pObject->isFrozen());
  CPPUNIT_ASSERT(pObject->getChildCount() == 3);
  
  CPPUNIT_ASSERT(TXObjectFactory::getInstance().createObjectFromStream(&strObjTree1, &pObject, false, false) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject != NULL);
  CPPUNIT_ASSERT(pObject->isFrozen());
  CPPUNIT_ASSERT(pObject->getChildCount() == 3);
  
  //Invalid Root node
  pObject = NULL;
  CPPUNIT_ASSERT(TXObjectFactory::getInstance().createObjectFromStream(&strObjTree2, &pObject, true) == RET_UNREGISTERED_CLASS);
  CPPUNIT_ASSERT(pObject == NULL);
}
 
