/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XObjectTest                                                   *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObjectTest.cpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeustest/System/XObjectTest.h>
#include <zeusbase/System/XObjectFactory.h>
#include <stdio.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TXObjectTest );


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TXObjectTest::setUp()
{  
  m_pObject = NULL;
  
  TString strXObjTree1 = L"<?xml version='1.0' encoding='ISO-8859-1'?>\
<RootObject ID=\"1\" ClassName=\"TXRootObject\" AttributeSchema=\"2\">\
  <Attribute Name=\"Label\" Value=\"Begruessung\"/>\
  <Attribute Name=\"Text\" Value=\"Hallo Welt!\"/>\
  <SubObject1 ID=\"1\" ClassName=\"TXObject\">\
    <SubSubObject1 ID=\"1\" ClassName=\"TXObject\"/>\
    <SubSubObject1 ID=\"2\" ClassName=\"TXObject\"/>\
  </SubObject1>\
  <SubObject1 ID=\"2\" ClassName=\"TXObject\">\
    <SubSubObject2 ID=\"1\" ClassName=\"TXObject\"/>\
    <SubSubObject3 ID=\"2\" ClassName=\"TXObject\"/>\
  </SubObject1>\
  <SubObject2 ID=\"1\" ClassName=\"TXObject\"/>\
</RootObject>";

  //The test of the factory: see TXObjectFactoryTest
  TXObjectFactory::getInstance().createObjectFromStream(&strXObjTree1, &m_pObject, false, false); 
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TXObjectTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
  
  if (m_pObject != NULL)
  {
    m_pObject->release();
  }
}

/***************************************************************************/
/*! Tests the name of a XObject
*/
/***************************************************************************/
void TXObjectTest::test_getName()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  
  TString strName;
  m_pObject->getName(&strName);
  CPPUNIT_ASSERT(strName == L"RootObject");
  
  //Name is not changable
  TString strObjName = L"Hallo";
  ((TXObject*)m_pObject)->setObjName(strObjName);
  m_pObject->getName(&strName);
  CPPUNIT_ASSERT(strName == L"RootObject");
}
 
/***************************************************************************/
/*! Tests the id of the root object
*/
/***************************************************************************/
void TXObjectTest::test_getID()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  CPPUNIT_ASSERT(m_pObject->getID() == 1);
}
 

/***************************************************************************/
/*! Tests the child count of the root object
*/
/***************************************************************************/
void TXObjectTest::test_getChildCount()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  CPPUNIT_ASSERT(m_pObject->getChildCount() == 3);
}


/***************************************************************************/
/*! Tests getting children
*/
/***************************************************************************/
void TXObjectTest::test_getChild()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  
  //Has 3 Children
  IXObject* pChild1 = NULL;
  IXObject* pChild2 = NULL;
  IXObject* pChild3 = NULL;
  IXObject* pChild4 = NULL; //Error child
  IXObject* pChild1_1 = NULL;
  TString strName;
  
  //Check 1st Childobject
  CPPUNIT_ASSERT(m_pObject->getChild(0, &pChild1) == RET_NOERROR);
  m_lstObjRef.push(pChild1);
  
  pChild1->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject1");  
  CPPUNIT_ASSERT(pChild1->getID() == 1);
  
  
  //Get child1 of child1
  CPPUNIT_ASSERT(pChild1->getChildCount() == 2);
  CPPUNIT_ASSERT(pChild1->getChild(0, &pChild1_1) == RET_NOERROR);
  m_lstObjRef.push(pChild1_1);
  pChild1_1->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubSubObject1");
  CPPUNIT_ASSERT(pChild1_1->getID() == 1);
  
  CPPUNIT_ASSERT(m_pObject->getChild(1, &pChild2) == RET_NOERROR);
  m_lstObjRef.push(pChild2);
  pChild2->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject1");
  CPPUNIT_ASSERT(pChild2->getID() == 2);
  
  CPPUNIT_ASSERT(m_pObject->getChild(2, &pChild3) == RET_NOERROR);
  m_lstObjRef.push(pChild3);
  pChild3->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject2");
  CPPUNIT_ASSERT(pChild3->getID() == 1);
  
  CPPUNIT_ASSERT(m_pObject->getChild(3, &pChild4) == RET_REQUEST_FAILED);
}


/***************************************************************************/
/*! Tests getting children by name
*/
/***************************************************************************/
void TXObjectTest::test_getChildByName()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  
  //Has 3 Children
  IXObject* pChild1 = NULL;
  IXObject* pChild2 = NULL;
  TString strGetName;
  TString strName;
  
  //Check 1st Childobject
  strGetName = L"SubObject1";
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strGetName, &pChild1) == RET_NOERROR);
  m_lstObjRef.push(pChild1);
  
  pChild1->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject1");  
  CPPUNIT_ASSERT(pChild1->getID() == 1);  

  strGetName = L"SubObject1[0]";
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strGetName, &pChild1) == RET_NOERROR);
  m_lstObjRef.push(pChild1);
  
  pChild1->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject1");  
  CPPUNIT_ASSERT(pChild1->getID() == 1);  
  
  strGetName = L"SubObject1[1]";
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strGetName, &pChild1) == RET_NOERROR);
  m_lstObjRef.push(pChild1);
  
  pChild1->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject1");  
  CPPUNIT_ASSERT(pChild1->getID() == 2);  
  
  //Check 2nd Childobject
  strGetName = L"SubObject2";
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strGetName, &pChild2) == RET_NOERROR);
  m_lstObjRef.push(pChild2);
  
  pChild2->getName(&strName);
  CPPUNIT_ASSERT(strName == L"SubObject2");  
}


/***************************************************************************/
/*! Tests getting children by name
*/
/***************************************************************************/
void TXObjectTest::test_getParent()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
    
  long lChildCount = m_pObject->getChildCount();
  for(int i = 0; i < lChildCount; i++)
  {
    IXObject* pChild1 = NULL;
    IXObject* pParent = NULL;
    
    CPPUNIT_ASSERT(m_pObject->getChild(0, &pChild1) == RET_NOERROR);
    m_lstObjRef.push(pChild1);
    CPPUNIT_ASSERT(pChild1->getParent(&pParent) == RET_NOERROR);
    m_lstObjRef.push(pParent);
    CPPUNIT_ASSERT(pParent == m_pObject);    
  }
}


/***************************************************************************/
/*! Tests deleting a child by Index
*/
/***************************************************************************/
void TXObjectTest::test_deleteChild()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  CPPUNIT_ASSERT(m_pObject->getChildCount() == 3);
  
  //Store the 2nd reference
  IXObject* pChild2 = NULL;
  CPPUNIT_ASSERT(m_pObject->getChild(1, &pChild2) == RET_NOERROR);
  m_lstObjRef.push(pChild2);
  
  CPPUNIT_ASSERT(m_pObject->deleteChild(0) == RET_NOERROR);
  CPPUNIT_ASSERT(m_pObject->getChildCount() == 2);  
  
  //Checkk if the first has been removed
  IXObject* pCheckChild2 = NULL;
  CPPUNIT_ASSERT(m_pObject->getChild(0, &pCheckChild2) == RET_NOERROR);
  m_lstObjRef.push(pCheckChild2);
  CPPUNIT_ASSERT(pCheckChild2 == pChild2);  
}


/***************************************************************************/
/*! Tests getting an object by name and path
*/
/***************************************************************************/
void TXObjectTest::test_getObject()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  TString strPath;
  IXObject* pObject = NULL;
  
  //All variants to get the root (or same not as the calling)
  strPath = L"/"; //Root
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(m_pObject == pObject);  
  
  strPath = L"."; //this node
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(m_pObject == pObject);

  strPath = L""; //same as [.] this node
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(m_pObject == pObject);

  
  //get the child with the name "SubObject1"
  strPath = L"/SubObject1"; 
  pObject = NULL;
  IXObject* pCheckObject = NULL;
  TString strCheckName = L"SubObject1";
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strCheckName, &pCheckObject)== RET_NOERROR);
  m_lstObjRef.push(pCheckObject);  
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pCheckObject == pObject);
  
  //get the child with the name "SubObject1"
  strPath = L"./SubObject1"; 
  pObject = NULL;
  pCheckObject = NULL;
  strCheckName = L"SubObject1";
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strCheckName, &pCheckObject)== RET_NOERROR);
  m_lstObjRef.push(pCheckObject);  
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pCheckObject == pObject);
  
  //get the child with the name "SubSubObject1"
  strPath = L"./SubObject1[1]/SubSubObject3"; 
  pObject = NULL;
  pCheckObject = NULL;
  strCheckName = L"SubObject1[1]";
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strCheckName, &pCheckObject)== RET_NOERROR);
  m_lstObjRef.push(pCheckObject);  
  strCheckName = L"SubSubObject3";
  CPPUNIT_ASSERT(pCheckObject->getChildByName(&strCheckName, &pCheckObject)== RET_NOERROR);
  m_lstObjRef.push(pCheckObject);  
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pCheckObject == pObject);
  
  //Use the ./SubObject1[1]/SubSubObject3 Object for further checks
  strPath = L"/";
  IXObject* pSubSubObject3 = pCheckObject;
  pObject = NULL;
  pCheckObject = NULL;
  strCheckName = L"";
  CPPUNIT_ASSERT(pSubSubObject3->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(m_pObject == pObject);
  
  strPath = L"..";
  pObject = NULL;
  pCheckObject = NULL;
  strCheckName = L"SubObject1[1]";
  CPPUNIT_ASSERT(pSubSubObject3->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strCheckName, &pCheckObject) == RET_NOERROR);
  m_lstObjRef.push(pCheckObject);
  CPPUNIT_ASSERT(pCheckObject == pObject);
  
  strPath = L"../SubSubObject2";
  pObject = NULL;
  pCheckObject = NULL;
  strCheckName = L"SubObject1[1]";
  CPPUNIT_ASSERT(pSubSubObject3->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(m_pObject->getChildByName(&strCheckName, &pCheckObject) == RET_NOERROR);
  m_lstObjRef.push(pCheckObject);
  strCheckName = L"SubSubObject2";
  CPPUNIT_ASSERT(pCheckObject->getChildByName(&strCheckName, &pCheckObject) == RET_NOERROR);
  m_lstObjRef.push(pCheckObject);
  CPPUNIT_ASSERT(pCheckObject == pObject);

  strPath = L"../..";
  pObject = NULL;
  pCheckObject = NULL;
  CPPUNIT_ASSERT(pSubSubObject3->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(m_pObject == pObject);
}

/***************************************************************************/
/*! Tests getting the root object
*/
/***************************************************************************/
void TXObjectTest::test_getRootObject()
{
  CPPUNIT_ASSERT(m_pObject != NULL);
  TString strPath;
  IXObject* pObject = NULL;
  IXObject* pRootObject = NULL;
  
  CPPUNIT_ASSERT(m_pObject->getRootObject(&pRootObject) == RET_NOERROR);
  m_lstObjRef.push(pRootObject);
  CPPUNIT_ASSERT(pRootObject == m_pObject);

  //get the child with the name "SubSubObject1"
  strPath = L"./SubObject2"; 
  pObject = NULL;
  pRootObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject->getRootObject(&pRootObject) == RET_NOERROR);
  m_lstObjRef.push(pRootObject);
  CPPUNIT_ASSERT(pRootObject == m_pObject);
  
  //get the child with the name "SubSubObject1"
  strPath = L"./SubObject1[1]/SubSubObject3"; 
  pObject = NULL;
  pRootObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strPath, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject->getRootObject(&pRootObject) == RET_NOERROR);
  m_lstObjRef.push(pRootObject);
  CPPUNIT_ASSERT(pRootObject == m_pObject);

}

/***************************************************************************/
/*! Tests the freeze functionality
*/
/***************************************************************************/
void TXObjectTest::test_freeze()
{
  //-----------------------------------------------------------------------
  CPPUNIT_ASSERT(m_pObject != NULL);
  CPPUNIT_ASSERT(m_pObject->isFrozen());

  TString strName = L"./SubObject1/SubSubObject1[1]";
  IXObject* pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject->isFrozen());
   
  strName = L"./SubObject1/SubSubObject1[0]";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject->isFrozen());
  //-----------------------------------------------------------------------
  
  //-----------------------------------------------------------------------
  // Unfreeze the root module -> all children and ancestors must be unfrozen
  m_pObject->unfreeze();
  strName = L"./SubObject1/SubSubObject1[1]";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(!pObject->isFrozen());
   
  strName = L"./SubObject1/SubSubObject1[0]";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(!pObject->isFrozen());
  
  //-----------------------------------------------------------------------
  //Freeze the service node -> all children and ancestors has to be frozen
  strName = L"./SubObject1[1]";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  pObject->freeze();
  CPPUNIT_ASSERT(pObject->isFrozen());
  
  strName = L"./SubObject1[1]/SubSubObject2";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject->isFrozen());
   
  strName = L"./SubObject1[1]/SubSubObject3";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(pObject->isFrozen());
  
  strName = L"./SubObject1[0]";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(!pObject->isFrozen());
  
  strName = L"./SubObject1/SubSubObject1[1]";
  pObject = NULL;
  CPPUNIT_ASSERT(m_pObject->getObject(&strName, &pObject) == RET_NOERROR);
  m_lstObjRef.push(pObject);
  CPPUNIT_ASSERT(!pObject->isFrozen());
  //-----------------------------------------------------------------------

}

