/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus
 * Module  : ZVariantTest
 * Package : Test
 * Author  : Benjamin Hadorn
 * Datum   : $Date: 2005/11/10 10:50:22 $
 * Ablage  : $File$
 * System  : Cell Computing Model
 ***************************************************************************
 * Aenderungen :
 *   $Log: ZVariantTest.cpp,v $
 *   Revision 1.1  2005/11/10 10:50:22  bhadorn
 *   created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/System/ZVariantTest.h>
#include <zeustest/System/SerialTestObject.h>
 
USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TZVariantTest );


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TZVariantTest::setUp()
{}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TZVariantTest::tearDown()
{
  //Release all instances
  while(!m_lstObjRef.isEmpty())
  {
    m_lstObjRef.pop()->release();
  }
}

/***************************************************************************/
/*! Tests the byte methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_byte()
{
  TZVariant* vByte = new TZVariant("", (char)132);
  m_lstObjRef.push(vByte);
  char cTestVar = 0;
  
  CPPUNIT_ASSERT(vByte->getType() == varByte);
  CPPUNIT_ASSERT(vByte->getByte(&cTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(cTestVar == (char)132);
  
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(vByte->serialize(&aStream) == RET_NOERROR);
  
  //Create new variant out of the stream
  ISerializable* pSerialObject = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pSerialObject) == RET_NOERROR);
  m_lstObjRef.push(pSerialObject);
  
  TZVariant* vByte2 = (TZVariant*)pSerialObject;
  CPPUNIT_ASSERT(vByte2->getType() == varByte);
  CPPUNIT_ASSERT(vByte2->getByte(&cTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(cTestVar == (char)132);
  
}
 
/***************************************************************************/
/*! Tests the long methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_long()
{
  TZVariant* vLong = new TZVariant("", (long)132);
  m_lstObjRef.push(vLong);
  long lTestVar = 0;
  
  CPPUNIT_ASSERT(vLong->getType() == varLong);
  CPPUNIT_ASSERT(vLong->getLong(&lTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(lTestVar == 132);

  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(vLong->serialize(&aStream) == RET_NOERROR);
  
  //Create new variant out of the stream
  ISerializable* pSerialObject = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pSerialObject) == RET_NOERROR);
  m_lstObjRef.push(pSerialObject);
  
  TZVariant* vLong2 = (TZVariant*)pSerialObject;
  CPPUNIT_ASSERT(vLong2->getType() == varLong);
  CPPUNIT_ASSERT(vLong2->getLong(&lTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(lTestVar == 132);
}

/***************************************************************************/
/*! Tests the double methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_double()
{
  TZVariant* vDouble = new TZVariant("", 132.991);
  m_lstObjRef.push(vDouble);
  double dTestVar = 0;
  
  CPPUNIT_ASSERT(vDouble->getType() == varDouble);
  CPPUNIT_ASSERT(vDouble->getDouble(&dTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(dTestVar == 132.991);

  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(vDouble->serialize(&aStream) == RET_NOERROR);
  
  //Create new variant out of the stream
  ISerializable* pSerialObject = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pSerialObject) == RET_NOERROR);
  m_lstObjRef.push(pSerialObject);

  TZVariant* vDouble2 = (TZVariant*)pSerialObject;
  CPPUNIT_ASSERT(vDouble2->getType() == varDouble);
  CPPUNIT_ASSERT(vDouble2->getDouble(&dTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(dTestVar == 132.991);
}

/***************************************************************************/
/*! Tests the string methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_string()
{
  TZVariant* vString = new TZVariant("", L"This is the test");
  m_lstObjRef.push(vString);
  TString strTestVar;
  CPPUNIT_ASSERT(vString->getType() == varString);
  CPPUNIT_ASSERT(vString->getString(&strTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(strTestVar == L"This is the test");  

  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(vString->serialize(&aStream) == RET_NOERROR);
  
  //Create new variant out of the stream
  ISerializable* pSerialObject = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pSerialObject) == RET_NOERROR);
  m_lstObjRef.push(pSerialObject);

  TZVariant* vString2 = (TZVariant*)pSerialObject;
  CPPUNIT_ASSERT(vString2->getType() == varString);
  CPPUNIT_ASSERT(vString2->getString(&strTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(strTestVar == L"This is the test");  
}


/***************************************************************************/
/*! Tests the bytearray methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_bytearray()
{
  TByteArray aData;
  aData.add(1);
  aData.add(3);
  aData.add(54);
  TZVariant* vArray = new TZVariant("", aData);
  m_lstObjRef.push(vArray);
  
  //printf("Type=%d\n\n", (int)vArray.getType());
  
  TByteArray aTestVar;
  CPPUNIT_ASSERT(vArray->getType() == varByteArray);
  CPPUNIT_ASSERT(vArray->getByteArray(&aTestVar) == RET_NOERROR);
  CPPUNIT_ASSERT(aTestVar == aData);  
  
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(vArray->serialize(&aStream) == RET_NOERROR);
  
  //Create new variant out of the stream
  ISerializable* pSerialObject = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pSerialObject) == RET_NOERROR);
  m_lstObjRef.push(pSerialObject);

  TByteArray aTestVar2;
  TZVariant* vArray2 = (TZVariant*)pSerialObject;
  CPPUNIT_ASSERT(vArray2->getType() == varByteArray);
  CPPUNIT_ASSERT(vArray2->getByteArray(&aTestVar2) == RET_NOERROR);
  CPPUNIT_ASSERT(aTestVar2 == aData);  
}

/***************************************************************************/
/*! Tests the object methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_object()
{
  TSerialTestObject* pObject = new TSerialTestObject();
  m_lstObjRef.push(pObject);
  pObject->setDoubleValue(-102.2);
  
  TZVariant* vObject = new TZVariant("", pObject);
  m_lstObjRef.push(vObject);
  ISerializable* pTestObject = NULL;
  
  CPPUNIT_ASSERT(vObject->getType() == varObject);
  CPPUNIT_ASSERT(vObject->getObject(&pTestObject) == RET_NOERROR);
  m_lstObjRef.push(pTestObject);
  
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(vObject->serialize(&aStream) == RET_NOERROR);
  
  //Create new variant out of the stream
  ISerializable* pSerialObject = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pSerialObject) == RET_NOERROR);
  m_lstObjRef.push(pSerialObject);

  ISerializable* pTestObject2 = NULL;
  TZVariant* vObject2 = (TZVariant*)pSerialObject;
  CPPUNIT_ASSERT(vObject2->getType() == varObject);
  CPPUNIT_ASSERT(vObject2->getObject(&pTestObject2) == RET_NOERROR);
  m_lstObjRef.push(pTestObject2);
  
  TSerialTestObject* pTestObject3 = (TSerialTestObject*)pTestObject2;
  CPPUNIT_ASSERT(pTestObject3->getDoubleValue() == -102.2);  
}


/***************************************************************************/
/*! Creates a list of objects
    \param list : Return value of the list
*/
/***************************************************************************/
void TZVariantTest::createObjectList(TSingleLinkedList<ISerializable*>& list)
{
  TSerialTestObject* pObject1 = new TSerialTestObject();
  m_lstObjRef.push(pObject1);
  pObject1->setLongValue(2);

  TSerialTestObject* pObject2 = new TSerialTestObject();
  m_lstObjRef.push(pObject2);
  pObject2->setLongValue(4);

  TSerialTestObject* pObject3 = new TSerialTestObject();
  m_lstObjRef.push(pObject3);
  pObject3->setLongValue(6);

  TSerialTestObject* pObject4 = new TSerialTestObject();
  m_lstObjRef.push(pObject4);
  pObject4->setLongValue(8);
  
  list.add(pObject1);
  list.add(pObject2);
  list.add(pObject3);
  list.add(pObject4);
}

/***************************************************************************/
/*! Tests the objectlist methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_objectlist()
{
  
  TSingleLinkedList<ISerializable*> lstObjects;
  this->createObjectList(lstObjects);
  TZVariant* vObjectList = new TZVariant("", &lstObjects);
  m_lstObjRef.push(vObjectList);
  TSingleLinkedList<ISerializable*> lstTestObjects;
  
  CPPUNIT_ASSERT(vObjectList->getType() == varObjectList);
  CPPUNIT_ASSERT(vObjectList->getObjectList(&lstTestObjects) == RET_NOERROR);
  IListIterator<ISerializable*>* pIt = lstTestObjects.getIterator();
  while(pIt->hasNextItem())
  {
    m_lstObjRef.push(pIt->getNextItem());
  }
  lstTestObjects.releaseIterator(pIt);

  CPPUNIT_ASSERT(lstTestObjects.getCount() == 4);
  
  TSerialTestObject* pTestObj = (TSerialTestObject*)lstTestObjects[2];
  CPPUNIT_ASSERT(pTestObj->getLongValue() == 6);
    
  //Stream message
  TByteArray aStream;
  CPPUNIT_ASSERT(vObjectList->serialize(&aStream) == RET_NOERROR);
  
  //Create new variant out of the stream
  ISerializable* pSerialObject = NULL;
  CPPUNIT_ASSERT(TZObjectFactory::getInstance().createObjectFromStream(&aStream, (ISerializable**)&pSerialObject) == RET_NOERROR);
  m_lstObjRef.push(pSerialObject);

  TSingleLinkedList<ISerializable*> lstTestObjects2;
  TZVariant* vObjectList2 = (TZVariant*)pSerialObject;
  CPPUNIT_ASSERT(vObjectList2->getType() == varObjectList);
  CPPUNIT_ASSERT(vObjectList2->getObjectList(&lstTestObjects2) == RET_NOERROR);
  IListIterator<ISerializable*>* pIt2 = lstTestObjects2.getIterator();
  while(pIt2->hasNextItem())
  {
    m_lstObjRef.push(pIt2->getNextItem());
  }
  lstTestObjects2.releaseIterator(pIt2);
  CPPUNIT_ASSERT(lstTestObjects2.getCount() == 4);
  
  TSerialTestObject* pTestObj2 = (TSerialTestObject*)lstTestObjects2[2];
  CPPUNIT_ASSERT(pTestObj2->getLongValue() == 6);
}

/***************************************************************************/
/*! Tests the assign methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_assign()
{
  TZVariant* vByte1 = new TZVariant((char)12);
  TZVariant* vByte2 = new TZVariant(vByte1);
  m_lstObjRef.push(vByte1);
  m_lstObjRef.push(vByte2);
  CPPUNIT_ASSERT(vByte1->equals(vByte2));

  TZVariant* vLong1 = new TZVariant((long)12234);
  TZVariant* vLong2 = new TZVariant(vLong1);
  m_lstObjRef.push(vLong1);
  m_lstObjRef.push(vLong2);
  CPPUNIT_ASSERT(vLong1->equals(vLong2));

  TZVariant* vDouble1 = new TZVariant(12234.23);
  TZVariant* vDouble2 = new TZVariant(vDouble1);
  m_lstObjRef.push(vDouble1);
  m_lstObjRef.push(vDouble2);
  CPPUNIT_ASSERT(vDouble1->equals(vDouble2));

  TZVariant* vString1 = new TZVariant("This is a test");
  TZVariant* vString2 = new TZVariant(vString1);
  m_lstObjRef.push(vString1);
  m_lstObjRef.push(vString2);
  CPPUNIT_ASSERT(vString1->equals(vString2));

  TByteArray aData1("HALLO",5);
  TZVariant* vArray1 = new TZVariant(aData1);
  TZVariant* vArray2 = new TZVariant(vArray1);
  m_lstObjRef.push(vArray1);
  m_lstObjRef.push(vArray2);
  CPPUNIT_ASSERT(vArray1->equals(vArray2));

  TSerialTestObject* pObject1 = new TSerialTestObject();
  m_lstObjRef.push(pObject1);
  TZVariant* vObject1 = new TZVariant(pObject1);
  TZVariant* vObject2 = new TZVariant(vObject1);
  m_lstObjRef.push(vObject1);
  m_lstObjRef.push(vObject2);
  CPPUNIT_ASSERT(vObject1->equals(vObject2));

  TSingleLinkedList<ISerializable*> lstObjects1;
  this->createObjectList(lstObjects1);
  TZVariant* vObjectList1 = new TZVariant("", &lstObjects1);
  TZVariant* vObjectList2 = new TZVariant(vObjectList1);
  m_lstObjRef.push(vObjectList1);
  m_lstObjRef.push(vObjectList2);
  CPPUNIT_ASSERT(vObjectList1->equals(vObjectList2));
}

/***************************************************************************/
/*! Tests the equal methods of the variant
*/
/***************************************************************************/
void TZVariantTest::test_equals()
{
  TZVariant* vByte1= new TZVariant((char)12);
  TZVariant* vByte2= new TZVariant((char)12);
  TZVariant* vByte3= new TZVariant((char)11);
  m_lstObjRef.push(vByte1);
  m_lstObjRef.push(vByte2);
  m_lstObjRef.push(vByte3);
  CPPUNIT_ASSERT(vByte1->equals(vByte2));
  CPPUNIT_ASSERT(!vByte1->equals(vByte3));
  
  TZVariant* vLong1= new TZVariant((long)12234);
  TZVariant* vLong2= new TZVariant((long)12234);
  TZVariant* vLong3= new TZVariant((long)11222);
  m_lstObjRef.push(vLong1);
  m_lstObjRef.push(vLong2);
  m_lstObjRef.push(vLong3);
  CPPUNIT_ASSERT(vLong1->equals(vLong2));
  CPPUNIT_ASSERT(!vLong1->equals(vLong3));

  TZVariant* vDouble1= new TZVariant(12234.23);
  TZVariant* vDouble2= new TZVariant(12234.23);
  TZVariant* vDouble3= new TZVariant(12234.25);
  m_lstObjRef.push(vDouble1);
  m_lstObjRef.push(vDouble2);
  m_lstObjRef.push(vDouble3);
  CPPUNIT_ASSERT(vDouble1->equals(vDouble2));
  CPPUNIT_ASSERT(!vDouble1->equals(vDouble3));

  TZVariant* vString1= new TZVariant("This is a test");
  TZVariant* vString2= new TZVariant("This is a test");
  TZVariant* vString3= new TZVariant("This is not a test");
  m_lstObjRef.push(vString1);
  m_lstObjRef.push(vString2);
  m_lstObjRef.push(vString3);
  CPPUNIT_ASSERT(vString1->equals(vString2));
  CPPUNIT_ASSERT(!vString1->equals(vString3));

  TByteArray aData1("HALLO",5);
  TByteArray aData2("HALLO",5);
  TByteArray aData3("HALLo",5);
  TZVariant* vArray1= new TZVariant(aData1);
  TZVariant* vArray2= new TZVariant(aData2);
  TZVariant* vArray3= new TZVariant(aData3);
  m_lstObjRef.push(vArray1);
  m_lstObjRef.push(vArray2);
  m_lstObjRef.push(vArray3);
  CPPUNIT_ASSERT(vArray1->equals(vArray2));
  CPPUNIT_ASSERT(!vArray1->equals(vArray3));
  
  TSerialTestObject* pObject1 = new TSerialTestObject();
  m_lstObjRef.push(pObject1);
  TSerialTestObject* pObject3 = new TSerialTestObject();
  m_lstObjRef.push(pObject3);
  TZVariant* vObject1= new TZVariant(pObject1);
  TZVariant* vObject2= new TZVariant(pObject1);
  TZVariant* vObject3= new TZVariant(pObject3);
  m_lstObjRef.push(vObject1);
  m_lstObjRef.push(vObject2);
  m_lstObjRef.push(vObject3);
  CPPUNIT_ASSERT(vObject1->equals(vObject2));
  CPPUNIT_ASSERT(!vObject1->equals(vObject3));
  
  TSingleLinkedList<ISerializable*> lstObjects1;
  TSingleLinkedList<ISerializable*> lstObjects3;
  this->createObjectList(lstObjects1);
  this->createObjectList(lstObjects3);
  TZVariant* vObjectList1= new TZVariant("", &lstObjects1);
  TZVariant* vObjectList2= new TZVariant("", &lstObjects1);
  TZVariant* vObjectList3= new TZVariant("", &lstObjects3);
  m_lstObjRef.push(vObjectList1);
  m_lstObjRef.push(vObjectList2);
  m_lstObjRef.push(vObjectList3);
  CPPUNIT_ASSERT(vObjectList1->equals(vObjectList2));
  CPPUNIT_ASSERT(!vObjectList1->equals(vObjectList3));
}

