/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ZeusXMLOutputterHook                                          *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:23:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZeusXMLOutputterHook.hpp,v $
 *   Revision 1.1  2005/10/28 12:23:47  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/   

#ifndef ZeusXMLOutputterHookHPP
#define ZeusXMLOutputterHookHPP
 
#include <zeusbase/System/String.h>
#include <cppunit/XmlOutputterHook.h>
#include <cppunit/tools/XmlDocument.h>  
#include <cppunit/tools/XmlElement.h>  
#include <cppunit/tools/StringTools.h>

BEGIN_NAMESPACE_Zeus
 
/***************************************************************************/ 
/*! This class adds information to the output XML file
*/
/***************************************************************************/ 
class TZeusXMLOutputterHook : public CppUnit::XmlOutputterHook  
{ 
  public: 
    /***********************************************************************/
    /*! Creates a XML Hook for the testing framework cppunit
    */
    /***********************************************************************/
    TZeusXMLOutputterHook(const TString& project_name, const TString& author) 
    { 
      m_strProjectName = project_name; 
      m_strAuthor = author; 
    } 
    
    /***********************************************************************/
    /*! Destroys the XML Hook 
    */
    /***********************************************************************/
    virtual ~TZeusXMLOutputterHook() { }
    
    /***********************************************************************/
    /*! Call back method when the file writer starts to write the xml file
    */
    /***********************************************************************/
    void beginDocument(CppUnit::XmlDocument* document) 
    { 
      if (document != NULL)
      {
        // dump current time  
        std::string szDate = CppUnit::StringTools::toString( (int)time(0) ); 
        CppUnit::XmlElement* metaEl = new CppUnit::XmlElement("SuiteInfo", ""); 
        metaEl->addElement( new CppUnit::XmlElement("Author", m_strAuthor.c_str()) ); 
        metaEl->addElement( new CppUnit::XmlElement("Project", m_strProjectName.c_str()) ); 
        metaEl->addElement( new CppUnit::XmlElement("Date", szDate ) ); 
        document->rootElement().addElement(metaEl); 
      }
    }
    
  private: 
    ///Name of the project 
    TString m_strProjectName; 
    ///Name or e-mail of the author
    TString m_strAuthor; 
};
 
END_NAMESPACE_Zeus

#endif
