
#include <cppunit/BriefTestProgressListener.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/TestResult.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/TestRunner.h>
#include <cppunit/ui/text/TestRunner.h>
#include <zeustest/ZeusXMLOutputterHook.hpp>
#include <zeustest/System/SerialTestObject.h>
#include <zeustest/Remote/RemoteTestObject.h>
#include <zeustest/System/ThreadTestObject.h>

#include <zeusbase/System/StdLogger.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/System/Application.h>
#include <zeusbase/System/XRootObject.h>
#include <zeusbase/System/XLoaderObject.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/SingletonManager.h>
#include <zeusbase/System/ZVariant.h>
#include <zeusbase/Remote/RemoteMethodCall.h>
#include <zeusbase/Remote/RemoteReference.h>
#include <zeusbase/Remote/Naming.h>
#include <zeusbase/MOM/Module.h>
#include <zeusbase/MOM/SystemManager.h>
#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/Messaging/BinaryMessage.h>
#include <zeusbase/Messaging/SystemMessage.h>
#include <zeusbase/Messaging/ObjectMessage.h>
#include <zeusbase/CellModel/CommPipe_Stub.h>
#include <zeusbase/CellModel/LocalPipeRegistry.h>
#include <fstream>

USING_NAMESPACE_Zeus


/***********************************************************************/
/*! main entry point of test framework.
*/
/***********************************************************************/
int main( int /*argc*/, char* /*argv[]*/ )
{
  //Setup standard logger
  TString strSingName;
/*  ILogger* pStartLogger = new TStdLogger();
  TLoggerManager::getInstance().setLogger(pStartLogger);
  strSingName = L"ILogger";
  TSingletonManager::getInstance().addSingleton(&strSingName, pStartLogger);
  pStartLogger->release();*/
  
  strSingName = L"IZObjectFactory";
  TSingletonManager::getInstance().addSingleton(&strSingName, (IZObjectFactory*)&TZObjectFactory::getInstance());
  
  ///////////////////////////////////////////////////
  XOBJECTFACTORY_REGISTER_ROOT(TXObjectFactory);
  XOBJECTFACTORY_REGISTER_ROOT(TXMLMessage);
  XOBJECTFACTORY_REGISTER_SUB(TXObjectFactory);
  XOBJECTFACTORY_REGISTER_SUB(TXLoaderObject);
  XOBJECTFACTORY_REGISTER_SUB(TModule);
  XOBJECTFACTORY_REGISTER_ROOT(TSystemManager);
  XOBJECTFACTORY_REGISTER_SUB(TLocalPipeRegistry);
  
  OBJECTFACTORY_REGISTER_CLASS(TBinaryMessage);
  OBJECTFACTORY_REGISTER_CLASS(TXMLMessage);
  OBJECTFACTORY_REGISTER_CLASS(TSystemMessage);
  OBJECTFACTORY_REGISTER_CLASS(TObjectMessage);
  OBJECTFACTORY_REGISTER_CLASS(TZVariant);
  OBJECTFACTORY_REGISTER_CLASS(TRemoteMethodCall);
  OBJECTFACTORY_REGISTER_CLASS(TRemoteReference);
  OBJECTFACTORY_REGISTER_CLASS(TNaming_Stub);
  OBJECTFACTORY_REGISTER_CLASS(TCommPipe_Stub);
  
  OBJECTFACTORY_REGISTER_CLASS(TRemoteTestObject_Stub);
  OBJECTFACTORY_REGISTER_CLASS(TSerialTestObject);
  OBJECTFACTORY_REGISTER_CLASS(TSerialTestSubObject);

  TSingletonManager::getInstance().registerCommonSingletons();
  ///////////////////////////////////////////////////
  
  
/*  TXRootObject* pObject = new TXRootObject(NULL);
  pObject->getXMLStream();
  pObject->release();*/
  
  ////////////////////////////
  //Loads the xml_service
  TString strLibName = "XML_Service"; 
  #ifdef LINUX
    TString strLibPath = "./modules/XML_Service.so";
  #else
    TString strLibPath = ".\\modules\\XML_Service.dll";
  #endif
  TLibraryManager::getInstance().addCodeModule(strLibName, strLibPath);  
  //////////////////////////// 

  // Create the event manager and test controller
  CPPUNIT_NS::TestResult controller;

  CPPUNIT_NS::TestResultCollector result;
  controller.addListener( &result );        

  CPPUNIT_NS::BriefTestProgressListener progress;
  controller.addListener( &progress );      
  
  CppUnit::TextUi::TestRunner runner;
  std::ofstream outputFile("testResults.xml");
 
  CppUnit::XmlOutputter* outputter = new CppUnit::XmlOutputter( &runner.result(),
                                                               outputFile );    
  TZeusXMLOutputterHook Hook("SystemClasses", "bhadorn@swissinfo.org");
  outputter->addHook(&Hook);
  runner.setOutputter(outputter);
  runner.addTest( CPPUNIT_NS::TestFactoryRegistry::getRegistry().makeTest() );
  runner.run();
  
  CPPUNIT_NS::CompilerOutputter stdoutputter( &runner.result(), std::cerr );
  stdoutputter.write();

  outputFile.close(); 
  
  
/*  TApplication Application;
  TThreadTestObject* pObj = new TThreadTestObject();
  pObj->start();
  Application.start();
  
  pObj->kill();
  
  pObj->release();*/
  ////////////////////////////
  // Frees all libraries
  TLibraryManager::getInstance().removeAll();
  ////////////////////////////
  
   
  return 0;
}
 
