/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FuzzyLogic                                                    *
 * Package : Math                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FuzzyLogic.h,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FuzzyLogicHPP 
#define FuzzyLogicHPP

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/SingleLinkedList.hpp>
#include <zeusbase/Math/FuzzySet.h>

BEGIN_NAMESPACE_Zeus 

enum TFuzzyInferenceMethod {fimMamdani, fimLarson};

/***************************************************************************/
/*! The fuzzy logic is used to process the inferences of a fuzzy logic
    model
*/
/***************************************************************************/
class TFuzzyLogic : public TZObject
{
  public:
   TFuzzyLogic(TFuzzyInferenceMethod method = fimMamdani);
   virtual ~TFuzzyLogic();

   void reset();
   
   void addAndCondition(double signal1, const TFuzzySet& set1, double signal2, const TFuzzySet& set2, TFuzzySet output_membership);
   TFuzzySet getConclusionSet();
   
  protected:
    ///Method of the conclusion
    TFuzzyInferenceMethod m_Method;
    ///Result Fuzzy set
    TSingleLinkedList<TFuzzySet> m_lstResults;
  
  private:  
  
};

END_NAMESPACE_Zeus 

#endif
